import datetime


from tests.data.fields_metadata import (
    geolayer_attributes_only_fields_metadata,
    geolayer_data_fields_metadata_complete,
    geolayer_data_fields_metadata_extract,
)

from tests.data.features import (
    feature_attributes_only,
    feature_attributes_only_without_none_field,
    feature_attributes_only_boolean_false,
    feature_structure_alpha_1,
    feature_structure_alpha_2,
    feature_structure_alpha_3,
    feature_dpt_data_only_a,
    feature_dpt_data_only_b,
    feature_dpt_data_only_c,
    feature_dpt_data_only_d,
    feature_dpt_geometry_only_a,
    feature_dpt_geometry_only_b,
    feature_dpt_geometry_only_c,
    feature_dpt_geometry_only_d,
    feature_dpt_data_and_geometry_a,
    feature_dpt_data_and_geometry_b,
    feature_dpt_data_and_geometry_c,
    feature_dpt_data_and_geometry_d,
    feature_dpt_data_and_geometry_a_serialized,
    feature_dpt_data_and_geometry_b_serialized,
    feature_dpt_data_and_geometry_c_serialized,
    feature_dpt_data_and_geometry_d_serialized,
    feature_attributes_bytes_only_a,
    feature_attributes_bytes_only_b,
    feature_attributes_bytes_only_c,
    feature_geometry_only_point,
    feature_geometry_only_point_empty,
    feature_geometry_only_linestring,
    feature_geometry_only_linestring_empty,
    feature_geometry_only_polygon,
    feature_geometry_only_polygon_empty,
    feature_geometry_only_multipoint,
    feature_geometry_only_multipoint_empty,
    feature_geometry_only_multilinestring,
    feature_geometry_only_multilinestring_empty,
    feature_geometry_only_multipolygon,
    feature_geometry_only_multipolygon_empty,
    feature_geometry_only_collection,
    feature_geometry_only_collection_empty,
    feature_geometry_only_collection_with_empty_geometries,
)

from tests.data.geometries import (
    POINT_WKB_BIG_ENDIAN,
    GEOMETRYCOLLECTION_WKB_BIG_ENDIAN,
    POINT_WKB_HEX_BIG_ENDIAN,
    GEOMETRYCOLLECTION_WKB_HEX_BIG_ENDIAN,
)


date_time_value = datetime.datetime(
    year=2020, month=3, day=31, hour=11, minute=22, second=10, microsecond=999
)
date_value = date_time_value.date()
time_value = date_time_value.time()

geolayer_attributes_only = {
    "metadata": {
        "name": "attributes_only",
        "fields": geolayer_attributes_only_fields_metadata,
    },
    "features": {0: feature_attributes_only},
}
geolayer_attributes_only_without_none_value = {
    "metadata": {
        "name": "attributes_only",
        "fields": geolayer_attributes_only_fields_metadata,
    },
    "features": {0: feature_attributes_only_without_none_field},
}
geolayer_attributes_only_boolean_false = {
    "metadata": {
        "name": "attributes_only_boolean_false",
        "fields": geolayer_attributes_only_fields_metadata,
    },
    "features": {0: feature_attributes_only_boolean_false},
}

geolayer_attributes_to_force_only = {
    "metadata": {"name": "geolayer_attributes_to_force_only"},
    "features": {
        0: feature_structure_alpha_1,
        1: feature_structure_alpha_2,
        2: feature_structure_alpha_3,
    },
}

geolayer_attributes_to_force_only_forced = {
    "metadata": {
        "name": "geolayer_attributes_to_force_only",
        "fields": {
            "field_integer": {"type": "Integer", "index": 0},
            "field_integer_list": {"type": "IntegerList", "index": 1},
            "field_real": {"type": "Real", "width": 6, "precision": 2, "index": 2},
            "field_real_list": {
                "type": "RealList",
                "width": 19,
                "precision": 4,
                "index": 3,
            },
            "field_string": {"type": "String", "width": 26, "index": 4},
            "field_date": {"type": "Date", "index": 5},
            "field_time": {"type": "String", "width": 15, "index": 6},
            "field_binary": {"type": "Binary", "index": 7},
            "field_boolean": {"type": "Boolean", "index": 8},
            "field_string_list": {"type": "StringList", "width": 16, "index": 9},
            "field_datetime": {"type": "DateTime", "index": 10},
        },
    },
    "features": {
        0: {
            "attributes": {
                "field_integer": 586,
                "field_integer_list": [5879, 8557],
                "field_real": 8789.98,
                "field_real_list": [89798.3654, 8757.0],
                "field_string": None,
                "field_date": datetime.date(2020, 3, 31),
                "field_time": "18:41:04",
                "field_binary": b"\x00\x00\x00\x00\x04\x00\x00\x00\x02\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01?\xf0\x00\x00\x00\x00\x00\x00?\xf0\x00\x00\x00\x00\x00\x00",
                "field_boolean": True,
            }
        },
        1: {
            "attributes": {
                "field_integer": 1466,
                "field_integer_list": [987, 2345],
                "field_real": 8789.0,
                "field_real_list": [2.0, 5.0],
                "field_string": "salut",
                "field_string_list": ["hi", "mister"],
                "field_time": "11:22:10.000999",
                "field_datetime": datetime.datetime(2020, 3, 31, 11, 22, 10, 999),
                "field_binary": b"\x00\x00\x00\x00\x01@\xef\xf3k\n=p\xa4@\xbd\xe6\x8b\x02\x0cI\xba",
                "field_boolean": False,
            }
        },
        2: {
            "attributes": {
                "field_integer": 149,
                "field_integer_list": [987, 2345],
                "field_real": 8789.0,
                "field_real_list": [2.0, 5.0],
                "field_string": "2020-03-31 11:22:10.000999",
                "field_string_list": ["hi", "mister"],
                "field_time": "11:22:10.000999",
                "field_datetime": datetime.datetime(2020, 3, 31, 11, 22, 10, 999),
                "field_binary": b"\x00\x00\x00\x00\x01@\xef\xf3k\n=p\xa4@\xbd\xe6\x8b\x02\x0cI\xba",
                "field_boolean": True,
            }
        },
    },
}


geolayer_geometry_only_all_geometries_type = {
    "metadata": {
        "name": "all_geometry_type_only",
        "geometry_ref": {
            "type": {
                "Point",
                "LineString",
                "Polygon",
                "MultiPoint",
                "MultiLineString",
                "MultiPolygon",
                "GeometryCollection",
            },
            "crs": 4326,
        },
    },
    "features": {
        0: feature_geometry_only_point,
        1: feature_geometry_only_point_empty,
        2: feature_geometry_only_linestring,
        3: feature_geometry_only_linestring_empty,
        4: feature_geometry_only_polygon,
        5: feature_geometry_only_polygon_empty,
        6: feature_geometry_only_multipoint,
        7: feature_geometry_only_multipoint_empty,
        8: feature_geometry_only_multilinestring,
        9: feature_geometry_only_multilinestring_empty,
        10: feature_geometry_only_multipolygon,
        11: feature_geometry_only_multipolygon_empty,
        12: feature_geometry_only_collection,
        13: feature_geometry_only_collection_empty,
        14: feature_geometry_only_collection_with_empty_geometries,
    },
}

geolayer_attributes_to_force_only_2 = {
    "metadata": {"name": "geolayer_attributes_to_force_only"},
    "features": {
        0: {"attributes": {"test": "['stop', 'covid']"}},
        1: {"attributes": {"test": ["stop", "aids"]}},
        2: {"attributes": {"test": [42]}},
    },
}

geolayer_fr_dept_data_and_geometry = {
    "metadata": {
        "name": "FRANCE_DPT_GENERALIZE_LAMB93_ROUND_DATA_AND_GEOMETRY",
        "fields": {
            "CODE_DEPT": {"type": "String", "width": 2, "index": 0},
            "NOM_DEPT": {"type": "String", "width": 23, "index": 1},
        },
        "geometry_ref": {"type": {"MultiPolygon", "Polygon"}, "crs": 2154},
    },
    "features": {
        0: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    438084.30000010587,
                    6248690.800002651,
                    554722.8000000587,
                    6333950.700002438,
                ),
                "coordinates": [
                    [
                        [505760.0, 6248691.0],
                        [509767.0, 6249274.0],
                        [514960.0, 6255730.0],
                        [523428.0, 6259977.0],
                        [531230.0, 6258583.0],
                        [536034.0, 6253601.0],
                        [539594.0, 6265818.0],
                        [554723.0, 6278961.0],
                        [550482.0, 6283435.0],
                        [545681.0, 6284750.0],
                        [535303.0, 6300840.0],
                        [531445.0, 6300603.0],
                        [531142.0, 6309545.0],
                        [528381.0, 6316132.0],
                        [520187.0, 6318466.0],
                        [528536.0, 6329188.0],
                        [521740.0, 6328834.0],
                        [519117.0, 6332164.0],
                        [512956.0, 6327891.0],
                        [507637.0, 6333951.0],
                        [502833.0, 6331375.0],
                        [496470.0, 6331745.0],
                        [495008.0, 6328849.0],
                        [483787.0, 6325253.0],
                        [474692.0, 6327970.0],
                        [465465.0, 6324803.0],
                        [465239.0, 6317398.0],
                        [461632.0, 6315722.0],
                        [453303.0, 6321029.0],
                        [444807.0, 6320573.0],
                        [440195.0, 6315550.0],
                        [443572.0, 6313646.0],
                        [442698.0, 6298315.0],
                        [439499.0, 6295895.0],
                        [438084.0, 6281592.0],
                        [443497.0, 6282643.0],
                        [449863.0, 6280833.0],
                        [456497.0, 6283051.0],
                        [457450.0, 6278664.0],
                        [461941.0, 6273297.0],
                        [466465.0, 6272970.0],
                        [468215.0, 6263513.0],
                        [471536.0, 6256527.0],
                        [479309.0, 6257879.0],
                        [496988.0, 6250664.0],
                        [505760.0, 6248691.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "32", "NOM_DEPT": "GERS"},
        },
        1: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    449163.9000000953,
                    6324802.500002467,
                    547193.100000056,
                    6410676.300002238,
                ),
                "coordinates": [
                    [
                        [547193.0, 6388334.0],
                        [539761.0, 6395885.0],
                        [530971.0, 6390971.0],
                        [526858.0, 6394383.0],
                        [527849.0, 6401566.0],
                        [514373.0, 6400391.0],
                        [512334.0, 6403639.0],
                        [495031.0, 6397493.0],
                        [491196.0, 6399384.0],
                        [489091.0, 6407815.0],
                        [486169.0, 6410676.0],
                        [474628.0, 6407547.0],
                        [472909.0, 6402270.0],
                        [476701.0, 6399758.0],
                        [474218.0, 6394851.0],
                        [464993.0, 6388179.0],
                        [460343.0, 6383021.0],
                        [460366.0, 6373402.0],
                        [462536.0, 6370133.0],
                        [454032.0, 6364649.0],
                        [457916.0, 6359953.0],
                        [455164.0, 6354504.0],
                        [449164.0, 6352477.0],
                        [449759.0, 6344226.0],
                        [459734.0, 6343570.0],
                        [462757.0, 6341305.0],
                        [470769.0, 6340294.0],
                        [467874.0, 6336236.0],
                        [464468.0, 6326808.0],
                        [465465.0, 6324803.0],
                        [474692.0, 6327970.0],
                        [483787.0, 6325253.0],
                        [495008.0, 6328849.0],
                        [496470.0, 6331745.0],
                        [502833.0, 6331375.0],
                        [507637.0, 6333951.0],
                        [512956.0, 6327891.0],
                        [519117.0, 6332164.0],
                        [520115.0, 6336256.0],
                        [529462.0, 6338671.0],
                        [533815.0, 6350996.0],
                        [536387.0, 6355171.0],
                        [530427.0, 6360537.0],
                        [531660.0, 6365290.0],
                        [544593.0, 6364542.0],
                        [545774.0, 6366277.0],
                        [541673.0, 6377661.0],
                        [542159.0, 6383881.0],
                        [547193.0, 6388334.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "47", "NOM_DEPT": "LOT-ET-GARONNE"},
        },
        2: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    837307.6999999549,
                    6403479.700002258,
                    965006.099999906,
                    6534231.700001889,
                ),
                "coordinates": [
                    [
                        [841110.0, 6468164.0],
                        [847361.0, 6468209.0],
                        [855768.0, 6473591.0],
                        [860939.0, 6470947.0],
                        [866406.0, 6463235.0],
                        [870728.0, 6463360.0],
                        [871909.0, 6454823.0],
                        [871457.0, 6445038.0],
                        [888053.0, 6440277.0],
                        [892767.0, 6444271.0],
                        [895595.0, 6439026.0],
                        [895949.0, 6427870.0],
                        [894862.0, 6413405.0],
                        [907889.0, 6409384.0],
                        [918546.0, 6403480.0],
                        [921863.0, 6404787.0],
                        [925560.0, 6409806.0],
                        [930672.0, 6410433.0],
                        [933177.0, 6416064.0],
                        [939410.0, 6420031.0],
                        [941569.0, 6417648.0],
                        [947667.0, 6423241.0],
                        [956742.0, 6422339.0],
                        [960775.0, 6424797.0],
                        [965006.0, 6422921.0],
                        [964872.0, 6432532.0],
                        [961377.0, 6439340.0],
                        [952342.0, 6439924.0],
                        [953909.0, 6450690.0],
                        [956276.0, 6452799.0],
                        [950505.0, 6456663.0],
                        [945165.0, 6465401.0],
                        [945461.0, 6470305.0],
                        [949441.0, 6473761.0],
                        [948557.0, 6482085.0],
                        [943964.0, 6486985.0],
                        [938381.0, 6486658.0],
                        [931624.0, 6492445.0],
                        [925224.0, 6482705.0],
                        [917457.0, 6486189.0],
                        [913763.0, 6489603.0],
                        [904449.0, 6504924.0],
                        [897998.0, 6515860.0],
                        [888123.0, 6525941.0],
                        [888996.0, 6528631.0],
                        [882529.0, 6534232.0],
                        [879159.0, 6530984.0],
                        [876089.0, 6523611.0],
                        [870291.0, 6521483.0],
                        [863140.0, 6525920.0],
                        [862850.0, 6517700.0],
                        [865784.0, 6514252.0],
                        [858527.0, 6506228.0],
                        [848198.0, 6502534.0],
                        [841056.0, 6498593.0],
                        [845846.0, 6493277.0],
                        [837308.0, 6485557.0],
                        [837457.0, 6475559.0],
                        [841110.0, 6468164.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "38", "NOM_DEPT": "ISERE"},
        },
        3: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    597332.4000000085,
                    6993758.30000057,
                    712382.9999999988,
                    7101576.600000292,
                ),
                "coordinates": [
                    [
                        [634422.0, 7101577.0],
                        [623701.0, 7099214.0],
                        [610465.0, 7094261.0],
                        [605814.0, 7089013.0],
                        [599885.0, 7086576.0],
                        [601399.0, 7078135.0],
                        [598139.0, 7067917.0],
                        [599563.0, 7049801.0],
                        [597332.0, 7035063.0],
                        [603193.0, 7029103.0],
                        [605232.0, 7026713.0],
                        [612989.0, 7030025.0],
                        [622831.0, 7024302.0],
                        [624801.0, 7021563.0],
                        [634147.0, 7017146.0],
                        [635372.0, 7012312.0],
                        [648121.0, 7014552.0],
                        [654991.0, 7012592.0],
                        [660837.0, 7014835.0],
                        [663948.0, 7010834.0],
                        [656343.0, 7006536.0],
                        [658067.0, 7000416.0],
                        [662547.0, 7004433.0],
                        [667892.0, 7004913.0],
                        [669642.0, 7001764.0],
                        [689814.0, 6997681.0],
                        [691147.0, 6993758.0],
                        [706470.0, 6995042.0],
                        [706855.0, 7002888.0],
                        [711164.0, 7009522.0],
                        [712383.0, 7015916.0],
                        [699507.0, 7031355.0],
                        [702680.0, 7038239.0],
                        [701478.0, 7042709.0],
                        [697493.0, 7046164.0],
                        [692820.0, 7044918.0],
                        [687641.0, 7047582.0],
                        [686184.0, 7056263.0],
                        [688698.0, 7062196.0],
                        [683642.0, 7063001.0],
                        [679720.0, 7059405.0],
                        [673807.0, 7057943.0],
                        [664277.0, 7059477.0],
                        [658064.0, 7064147.0],
                        [653390.0, 7071778.0],
                        [644454.0, 7079622.0],
                        [642453.0, 7087769.0],
                        [636514.0, 7100652.0],
                        [634422.0, 7101577.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "62", "NOM_DEPT": "PAS-DE-CALAIS"},
        },
        4: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    774574.399999991,
                    6905214.700000814,
                    873032.1999999789,
                    7008458.800000535,
                ),
                "coordinates": [
                    [
                        [776081.0, 6923412.0],
                        [782994.0, 6923346.0],
                        [790665.0, 6921022.0],
                        [795010.0, 6915033.0],
                        [803525.0, 6909934.0],
                        [814643.0, 6911926.0],
                        [818364.0, 6905215.0],
                        [822754.0, 6907655.0],
                        [833062.0, 6905548.0],
                        [837898.0, 6908256.0],
                        [842092.0, 6905887.0],
                        [849280.0, 6910207.0],
                        [847303.0, 6917040.0],
                        [853548.0, 6926776.0],
                        [849442.0, 6935263.0],
                        [853210.0, 6945817.0],
                        [856532.0, 6942619.0],
                        [867310.0, 6942452.0],
                        [873032.0, 6949057.0],
                        [869864.0, 6950495.0],
                        [863792.0, 6957578.0],
                        [856370.0, 6957018.0],
                        [850915.0, 6964755.0],
                        [848735.0, 6964497.0],
                        [834935.0, 6970707.0],
                        [833403.0, 6975569.0],
                        [835414.0, 6979112.0],
                        [832902.0, 6984653.0],
                        [828614.0, 6985882.0],
                        [833821.0, 7001508.0],
                        [834466.0, 7007583.0],
                        [830491.0, 7008459.0],
                        [826108.0, 7005649.0],
                        [820627.0, 6997569.0],
                        [821199.0, 6989850.0],
                        [814347.0, 6988595.0],
                        [808498.0, 6984157.0],
                        [803958.0, 6983050.0],
                        [795770.0, 6985807.0],
                        [788558.0, 6985051.0],
                        [789845.0, 6973793.0],
                        [787172.0, 6965431.0],
                        [790134.0, 6962730.0],
                        [781387.0, 6953785.0],
                        [779463.0, 6948255.0],
                        [774574.0, 6946610.0],
                        [777906.0, 6941851.0],
                        [775403.0, 6934852.0],
                        [776081.0, 6923412.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "08", "NOM_DEPT": "ARDENNES"},
        },
        5: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    728997.8999999948,
                    6759096.300001229,
                    837934.5999999736,
                    6845890.600000978,
                ),
                "coordinates": [
                    [
                        [730707.0, 6810078.0],
                        [736893.0, 6807760.0],
                        [745765.0, 6796957.0],
                        [746161.0, 6791892.0],
                        [749678.0, 6782304.0],
                        [755033.0, 6785741.0],
                        [759917.0, 6778325.0],
                        [763413.0, 6765164.0],
                        [767353.0, 6760249.0],
                        [783028.0, 6759096.0],
                        [789480.0, 6763963.0],
                        [796594.0, 6759156.0],
                        [797692.0, 6763119.0],
                        [808147.0, 6762855.0],
                        [816397.0, 6764598.0],
                        [817962.0, 6771080.0],
                        [827043.0, 6770243.0],
                        [825768.0, 6776095.0],
                        [828770.0, 6781171.0],
                        [835407.0, 6779651.0],
                        [837598.0, 6783989.0],
                        [837935.0, 6797546.0],
                        [834073.0, 6806764.0],
                        [827022.0, 6811887.0],
                        [822210.0, 6820200.0],
                        [823302.0, 6827043.0],
                        [819107.0, 6829393.0],
                        [814106.0, 6826118.0],
                        [803526.0, 6829924.0],
                        [796995.0, 6835851.0],
                        [797551.0, 6845278.0],
                        [785824.0, 6845891.0],
                        [779497.0, 6845091.0],
                        [773768.0, 6840880.0],
                        [769985.0, 6834113.0],
                        [760761.0, 6824301.0],
                        [753955.0, 6826609.0],
                        [746802.0, 6826973.0],
                        [740952.0, 6835710.0],
                        [737148.0, 6833964.0],
                        [731262.0, 6826009.0],
                        [728998.0, 6813854.0],
                        [730707.0, 6810078.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "10", "NOM_DEPT": "AUBE"},
        },
        6: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    992637.5999998839,
                    6272481.700002606,
                    1077560.2999998543,
                    6370071.400002356,
                ),
                "coordinates": [
                    [
                        [1009696.0, 6370071.0],
                        [1002665.0, 6364730.0],
                        [997268.0, 6357103.0],
                        [994698.0, 6348019.0],
                        [1000668.0, 6338218.0],
                        [1000089.0, 6334256.0],
                        [1007601.0, 6328762.0],
                        [1009068.0, 6324850.0],
                        [1014122.0, 6322114.0],
                        [1012050.0, 6317690.0],
                        [1007566.0, 6320724.0],
                        [1000978.0, 6315262.0],
                        [997097.0, 6315251.0],
                        [992638.0, 6305621.0],
                        [1002326.0, 6300425.0],
                        [1003290.0, 6292496.0],
                        [1006552.0, 6288390.0],
                        [1015523.0, 6285590.0],
                        [1014132.0, 6274778.0],
                        [1018256.0, 6272482.0],
                        [1020971.0, 6279894.0],
                        [1026533.0, 6279746.0],
                        [1033109.0, 6287175.0],
                        [1036126.0, 6293135.0],
                        [1041921.0, 6296903.0],
                        [1048217.0, 6297561.0],
                        [1064350.0, 6309506.0],
                        [1061225.0, 6317546.0],
                        [1066133.0, 6321494.0],
                        [1066468.0, 6326676.0],
                        [1072800.0, 6330433.0],
                        [1073742.0, 6336784.0],
                        [1077560.0, 6342698.0],
                        [1073403.0, 6350462.0],
                        [1060282.0, 6348576.0],
                        [1054180.0, 6344806.0],
                        [1042408.0, 6347558.0],
                        [1034654.0, 6353189.0],
                        [1025044.0, 6356596.0],
                        [1020000.0, 6356725.0],
                        [1018585.0, 6361599.0],
                        [1009696.0, 6370071.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "06", "NOM_DEPT": "ALPES-MARITIMES"},
        },
        7: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    753778.7999999836,
                    6460771.900002093,
                    837457.1999999543,
                    6575508.300001766,
                ),
                "coordinates": [
                    [
                        [753779.0, 6537018.0],
                        [758503.0, 6532097.0],
                        [754369.0, 6520637.0],
                        [760734.0, 6514163.0],
                        [764154.0, 6503988.0],
                        [770440.0, 6500348.0],
                        [774454.0, 6495613.0],
                        [776221.0, 6483674.0],
                        [770335.0, 6479388.0],
                        [770263.0, 6473476.0],
                        [776677.0, 6475557.0],
                        [780259.0, 6472021.0],
                        [792166.0, 6477709.0],
                        [802398.0, 6475469.0],
                        [807029.0, 6471772.0],
                        [806856.0, 6463705.0],
                        [811227.0, 6465007.0],
                        [816370.0, 6460772.0],
                        [825694.0, 6462649.0],
                        [826509.0, 6469123.0],
                        [831517.0, 6473257.0],
                        [837457.0, 6475559.0],
                        [837308.0, 6485557.0],
                        [834511.0, 6489782.0],
                        [829797.0, 6488843.0],
                        [825934.0, 6498617.0],
                        [818524.0, 6498224.0],
                        [809870.0, 6504594.0],
                        [806255.0, 6511913.0],
                        [808147.0, 6518212.0],
                        [807296.0, 6530310.0],
                        [802643.0, 6534285.0],
                        [800220.0, 6540120.0],
                        [801039.0, 6547667.0],
                        [798995.0, 6551156.0],
                        [802035.0, 6559678.0],
                        [810991.0, 6564128.0],
                        [806993.0, 6569822.0],
                        [798969.0, 6562707.0],
                        [797273.0, 6565101.0],
                        [787355.0, 6564848.0],
                        [785138.0, 6567101.0],
                        [776287.0, 6563809.0],
                        [774989.0, 6567447.0],
                        [768620.0, 6568680.0],
                        [769267.0, 6575508.0],
                        [762003.0, 6573092.0],
                        [760949.0, 6561975.0],
                        [763504.0, 6554575.0],
                        [762385.0, 6549859.0],
                        [763887.0, 6544360.0],
                        [761084.0, 6541876.0],
                        [754888.0, 6541809.0],
                        [753779.0, 6537018.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "42", "NOM_DEPT": "LOIRE"},
        },
        8: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    491362.4000000901,
                    6181124.600002802,
                    621363.1000000321,
                    6314318.000002484,
                ),
                "coordinates": [
                    [
                        [505760.0, 6248691.0],
                        [506455.0, 6246240.0],
                        [501020.0, 6240519.0],
                        [500952.0, 6237523.0],
                        [491741.0, 6229153.0],
                        [498958.0, 6224762.0],
                        [501199.0, 6216194.0],
                        [506394.0, 6214104.0],
                        [501753.0, 6199863.0],
                        [493715.0, 6200925.0],
                        [492032.0, 6195420.0],
                        [491362.0, 6184305.0],
                        [493119.0, 6181125.0],
                        [509918.0, 6181609.0],
                        [507071.0, 6189939.0],
                        [508439.0, 6196068.0],
                        [512551.0, 6198492.0],
                        [524704.0, 6194181.0],
                        [522873.0, 6204718.0],
                        [526458.0, 6208824.0],
                        [535012.0, 6210452.0],
                        [537386.0, 6214080.0],
                        [535770.0, 6220735.0],
                        [540236.0, 6224348.0],
                        [541984.0, 6228895.0],
                        [547328.0, 6230443.0],
                        [554118.0, 6225953.0],
                        [555809.0, 6233248.0],
                        [562910.0, 6233771.0],
                        [567825.0, 6236309.0],
                        [561407.0, 6244405.0],
                        [565565.0, 6247682.0],
                        [572065.0, 6240985.0],
                        [575081.0, 6235814.0],
                        [582028.0, 6242918.0],
                        [589272.0, 6240503.0],
                        [593467.0, 6242482.0],
                        [598408.0, 6250205.0],
                        [603733.0, 6249863.0],
                        [603060.0, 6255463.0],
                        [607574.0, 6261192.0],
                        [611672.0, 6256714.0],
                        [621363.0, 6260238.0],
                        [620519.0, 6263967.0],
                        [609628.0, 6269734.0],
                        [605052.0, 6276459.0],
                        [594074.0, 6282175.0],
                        [597601.0, 6285124.0],
                        [595737.0, 6291653.0],
                        [592076.0, 6294561.0],
                        [590928.0, 6301074.0],
                        [586632.0, 6302842.0],
                        [583878.0, 6308763.0],
                        [583992.0, 6314318.0],
                        [575242.0, 6309509.0],
                        [567948.0, 6310839.0],
                        [568130.0, 6303820.0],
                        [556606.0, 6298479.0],
                        [550735.0, 6304341.0],
                        [535303.0, 6300840.0],
                        [545681.0, 6284750.0],
                        [550482.0, 6283435.0],
                        [554723.0, 6278961.0],
                        [539594.0, 6265818.0],
                        [536034.0, 6253601.0],
                        [531230.0, 6258583.0],
                        [523428.0, 6259977.0],
                        [514960.0, 6255730.0],
                        [509767.0, 6249274.0],
                        [505760.0, 6248691.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "31", "NOM_DEPT": "HAUTE-GARONNE"},
        },
        9: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    748048.8999999872,
                    6562706.600001801,
                    887343.0999999508,
                    6673494.900001479,
                ),
                "coordinates": [
                    [
                        [806993.0, 6569822.0],
                        [808209.0, 6578315.0],
                        [815826.0, 6575242.0],
                        [824653.0, 6575157.0],
                        [830305.0, 6579430.0],
                        [833486.0, 6565745.0],
                        [837322.0, 6565642.0],
                        [839484.0, 6574938.0],
                        [845050.0, 6590960.0],
                        [845229.0, 6595067.0],
                        [848718.0, 6603737.0],
                        [857422.0, 6600366.0],
                        [868758.0, 6603264.0],
                        [869949.0, 6598870.0],
                        [877344.0, 6596683.0],
                        [885325.0, 6599852.0],
                        [880824.0, 6605207.0],
                        [886706.0, 6618161.0],
                        [882575.0, 6628163.0],
                        [880659.0, 6638936.0],
                        [887343.0, 6642371.0],
                        [876345.0, 6647462.0],
                        [872158.0, 6650880.0],
                        [871408.0, 6655745.0],
                        [860161.0, 6652754.0],
                        [855755.0, 6655507.0],
                        [831500.0, 6648015.0],
                        [823470.0, 6650227.0],
                        [813434.0, 6659755.0],
                        [806754.0, 6662035.0],
                        [806631.0, 6665548.0],
                        [795506.0, 6668991.0],
                        [791663.0, 6673495.0],
                        [789554.0, 6672918.0],
                        [784602.0, 6669828.0],
                        [780471.0, 6669516.0],
                        [778810.0, 6664936.0],
                        [781374.0, 6662136.0],
                        [779000.0, 6654397.0],
                        [778932.0, 6646959.0],
                        [783308.0, 6642195.0],
                        [780490.0, 6638662.0],
                        [781064.0, 6632682.0],
                        [775685.0, 6631211.0],
                        [770503.0, 6626941.0],
                        [760920.0, 6622686.0],
                        [756196.0, 6628301.0],
                        [748049.0, 6627896.0],
                        [748734.0, 6623209.0],
                        [753291.0, 6618069.0],
                        [756931.0, 6607772.0],
                        [761463.0, 6602522.0],
                        [763922.0, 6603795.0],
                        [768312.0, 6598303.0],
                        [773476.0, 6599308.0],
                        [776586.0, 6596649.0],
                        [775079.0, 6589273.0],
                        [775913.0, 6580395.0],
                        [769267.0, 6575508.0],
                        [768620.0, 6568680.0],
                        [774989.0, 6567447.0],
                        [776287.0, 6563809.0],
                        [785138.0, 6567101.0],
                        [787355.0, 6564848.0],
                        [797273.0, 6565101.0],
                        [798969.0, 6562707.0],
                        [806993.0, 6569822.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "71", "NOM_DEPT": "SAONE-ET-LOIRE"},
        },
        10: {
            "attributes": {"CODE_DEPT": "53", "NOM_DEPT": "MAYENNE"},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [399495.0, 6830885.0],
                        [398130.0, 6822559.0],
                        [400321.0, 6810723.0],
                        [395852.0, 6803336.0],
                        [398626.0, 6784333.0],
                        [400465.0, 6781914.0],
                        [400197.0, 6773697.0],
                        [394099.0, 6773357.0],
                        [390140.0, 6770978.0],
                        [386941.0, 6760260.0],
                        [382932.0, 6754022.0],
                        [389872.0, 6749698.0],
                        [393110.0, 6750366.0],
                        [402067.0, 6747685.0],
                        [404251.0, 6751414.0],
                        [412442.0, 6746090.0],
                        [419671.0, 6744167.0],
                        [429458.0, 6743442.0],
                        [440863.0, 6746201.0],
                        [446732.0, 6745443.0],
                        [446459.0, 6750432.0],
                        [442128.0, 6753611.0],
                        [448124.0, 6758669.0],
                        [447308.0, 6764356.0],
                        [455060.0, 6767070.0],
                        [451057.0, 6776681.0],
                        [459373.0, 6778102.0],
                        [460615.0, 6783387.0],
                        [458409.0, 6789055.0],
                        [466280.0, 6794064.0],
                        [465298.0, 6799724.0],
                        [467628.0, 6811401.0],
                        [473893.0, 6813452.0],
                        [474394.0, 6821359.0],
                        [467262.0, 6822174.0],
                        [466087.0, 6830999.0],
                        [463434.0, 6833996.0],
                        [457920.0, 6827997.0],
                        [451256.0, 6826715.0],
                        [446687.0, 6829012.0],
                        [441174.0, 6828584.0],
                        [437568.0, 6825109.0],
                        [429868.0, 6822252.0],
                        [422197.0, 6821752.0],
                        [414934.0, 6829326.0],
                        [407934.0, 6831360.0],
                        [404267.0, 6828490.0],
                        [399495.0, 6830885.0],
                    ]
                ],
            },
        },
        11: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    434480.3000000837,
                    6459163.500002107,
                    540666.4000000486,
                    6562343.900001808,
                ),
                "coordinates": [
                    [
                        [459784.0, 6545825.0],
                        [457303.0, 6539406.0],
                        [458386.0, 6534878.0],
                        [455132.0, 6526680.0],
                        [449713.0, 6524677.0],
                        [445866.0, 6528260.0],
                        [434480.0, 6521530.0],
                        [434607.0, 6512022.0],
                        [436951.0, 6506668.0],
                        [442913.0, 6501478.0],
                        [446521.0, 6493231.0],
                        [444696.0, 6489309.0],
                        [445861.0, 6484044.0],
                        [443202.0, 6478722.0],
                        [447267.0, 6474282.0],
                        [456356.0, 6470307.0],
                        [455660.0, 6465791.0],
                        [461242.0, 6465553.0],
                        [464807.0, 6459164.0],
                        [468806.0, 6462928.0],
                        [474561.0, 6460744.0],
                        [485798.0, 6470195.0],
                        [484697.0, 6477489.0],
                        [489646.0, 6488189.0],
                        [493466.0, 6488572.0],
                        [504657.0, 6496986.0],
                        [505177.0, 6505055.0],
                        [513233.0, 6510903.0],
                        [515635.0, 6515544.0],
                        [522412.0, 6520738.0],
                        [522669.0, 6525452.0],
                        [527825.0, 6527023.0],
                        [534383.0, 6537804.0],
                        [540666.0, 6541823.0],
                        [539507.0, 6547743.0],
                        [531290.0, 6552122.0],
                        [531964.0, 6561069.0],
                        [526089.0, 6562344.0],
                        [521367.0, 6557890.0],
                        [509938.0, 6556936.0],
                        [507509.0, 6562167.0],
                        [499249.0, 6554736.0],
                        [489751.0, 6554866.0],
                        [483517.0, 6558927.0],
                        [473907.0, 6559092.0],
                        [469589.0, 6554692.0],
                        [465641.0, 6555184.0],
                        [464645.0, 6548283.0],
                        [459784.0, 6545825.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "16", "NOM_DEPT": "CHARENTE"},
        },
        12: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    343011.6000000426,
                    6826524.800001056,
                    423497.90000004787,
                    6969661.300000665,
                ),
                "coordinates": [
                    [
                        [417723.0, 6857101.0],
                        [408651.0, 6861319.0],
                        [399642.0, 6861271.0],
                        [396867.0, 6868913.0],
                        [401493.0, 6871059.0],
                        [411444.0, 6881401.0],
                        [417375.0, 6892264.0],
                        [416434.0, 6899162.0],
                        [413253.0, 6901117.0],
                        [415797.0, 6907383.0],
                        [406933.0, 6907874.0],
                        [400663.0, 6914651.0],
                        [399047.0, 6920143.0],
                        [400759.0, 6925144.0],
                        [397607.0, 6926545.0],
                        [396597.0, 6932989.0],
                        [388320.0, 6946238.0],
                        [388687.0, 6950091.0],
                        [394381.0, 6955463.0],
                        [390587.0, 6963240.0],
                        [381046.0, 6965005.0],
                        [372051.0, 6959988.0],
                        [362230.0, 6961356.0],
                        [343652.0, 6969661.0],
                        [343012.0, 6963991.0],
                        [349037.0, 6960923.0],
                        [349892.0, 6952123.0],
                        [346450.0, 6948841.0],
                        [350114.0, 6938864.0],
                        [350193.0, 6931900.0],
                        [357636.0, 6924163.0],
                        [364780.0, 6906370.0],
                        [363427.0, 6896430.0],
                        [366480.0, 6887586.0],
                        [365701.0, 6878731.0],
                        [362959.0, 6869214.0],
                        [363872.0, 6859932.0],
                        [368562.0, 6852485.0],
                        [376547.0, 6847928.0],
                        [372261.0, 6845150.0],
                        [363309.0, 6845999.0],
                        [366767.0, 6839135.0],
                        [368413.0, 6830443.0],
                        [376536.0, 6826525.0],
                        [384742.0, 6834496.0],
                        [389634.0, 6835155.0],
                        [399495.0, 6830885.0],
                        [404267.0, 6828490.0],
                        [407934.0, 6831360.0],
                        [414934.0, 6829326.0],
                        [421619.0, 6835816.0],
                        [423498.0, 6841841.0],
                        [420581.0, 6852284.0],
                        [417723.0, 6857101.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "50", "NOM_DEPT": "MANCHE"},
        },
        13: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    587215.6000000172,
                    6818024.90000106,
                    643162.6000000093,
                    6887345.000000862,
                ),
                "coordinates": [
                    [
                        [590236.0, 6872273.0],
                        [594217.0, 6866696.0],
                        [596554.0, 6858123.0],
                        [595472.0, 6845570.0],
                        [597086.0, 6841311.0],
                        [604791.0, 6831646.0],
                        [608912.0, 6830618.0],
                        [611402.0, 6819097.0],
                        [620311.0, 6818025.0],
                        [623768.0, 6826369.0],
                        [627504.0, 6829048.0],
                        [627800.0, 6838438.0],
                        [632440.0, 6843248.0],
                        [634691.0, 6849194.0],
                        [643163.0, 6853168.0],
                        [637702.0, 6857774.0],
                        [637864.0, 6864035.0],
                        [641404.0, 6867928.0],
                        [641836.0, 6872490.0],
                        [633062.0, 6879807.0],
                        [628641.0, 6878089.0],
                        [622312.0, 6880731.0],
                        [614076.0, 6886919.0],
                        [606678.0, 6883543.0],
                        [603102.0, 6887292.0],
                        [598361.0, 6887345.0],
                        [591984.0, 6886421.0],
                        [587216.0, 6881812.0],
                        [590236.0, 6872273.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "78", "NOM_DEPT": "YVELINES"},
        },
        14: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    904021.0999999507,
                    6611179.900001672,
                    1006570.4999999271,
                    6724946.800001346,
                ),
                "coordinates": [
                    [
                        [986155.0, 6724947.0],
                        [984341.0, 6721925.0],
                        [977033.0, 6724531.0],
                        [974258.0, 6721322.0],
                        [969354.0, 6721610.0],
                        [967469.0, 6716290.0],
                        [955890.0, 6718838.0],
                        [950918.0, 6717032.0],
                        [945847.0, 6708437.0],
                        [932449.0, 6699390.0],
                        [925719.0, 6696195.0],
                        [922029.0, 6697905.0],
                        [904021.0, 6688452.0],
                        [905832.0, 6683506.0],
                        [912920.0, 6678099.0],
                        [913407.0, 6674258.0],
                        [909981.0, 6669146.0],
                        [913780.0, 6661601.0],
                        [925599.0, 6657600.0],
                        [925393.0, 6653995.0],
                        [928880.0, 6645788.0],
                        [940225.0, 6640408.0],
                        [944367.0, 6635241.0],
                        [942748.0, 6631718.0],
                        [934547.0, 6625608.0],
                        [937312.0, 6621303.0],
                        [933274.0, 6616467.0],
                        [940343.0, 6611180.0],
                        [938167.0, 6613167.0],
                        [950995.0, 6626461.0],
                        [962319.0, 6634781.0],
                        [961533.0, 6640379.0],
                        [963711.0, 6649123.0],
                        [961099.0, 6653277.0],
                        [967437.0, 6658254.0],
                        [976505.0, 6662212.0],
                        [981857.0, 6667617.0],
                        [983508.0, 6674248.0],
                        [992174.0, 6680929.0],
                        [993793.0, 6685149.0],
                        [999048.0, 6690006.0],
                        [1001635.0, 6695987.0],
                        [1006570.0, 6701646.0],
                        [1002944.0, 6704539.0],
                        [995578.0, 6702427.0],
                        [996902.0, 6711079.0],
                        [994087.0, 6717743.0],
                        [993651.0, 6723596.0],
                        [986155.0, 6724947.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "25", "NOM_DEPT": "DOUBS"},
        },
        15: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    838676.7999999775,
                    6816262.700001069,
                    906432.9999999686,
                    6949056.900000696,
                ),
                "coordinates": [
                    [
                        [904250.0, 6825381.0],
                        [904895.0, 6830283.0],
                        [900379.0, 6832469.0],
                        [903561.0, 6837915.0],
                        [902927.0, 6847494.0],
                        [899528.0, 6852809.0],
                        [904125.0, 6862116.0],
                        [901302.0, 6871865.0],
                        [905066.0, 6875501.0],
                        [906433.0, 6884047.0],
                        [905859.0, 6891045.0],
                        [901739.0, 6893315.0],
                        [900172.0, 6898682.0],
                        [901547.0, 6903348.0],
                        [898091.0, 6905220.0],
                        [898652.0, 6915853.0],
                        [895087.0, 6927281.0],
                        [891117.0, 6929633.0],
                        [883576.0, 6926040.0],
                        [878534.0, 6930925.0],
                        [879041.0, 6935896.0],
                        [875512.0, 6946935.0],
                        [873032.0, 6949057.0],
                        [867310.0, 6942452.0],
                        [856532.0, 6942619.0],
                        [853210.0, 6945817.0],
                        [849442.0, 6935263.0],
                        [853548.0, 6926776.0],
                        [847303.0, 6917040.0],
                        [849280.0, 6910207.0],
                        [842092.0, 6905887.0],
                        [841416.0, 6900107.0],
                        [845429.0, 6892400.0],
                        [846791.0, 6883540.0],
                        [848915.0, 6882413.0],
                        [848655.0, 6874905.0],
                        [841874.0, 6870868.0],
                        [838677.0, 6859119.0],
                        [846329.0, 6850924.0],
                        [846365.0, 6844539.0],
                        [847795.0, 6838972.0],
                        [852692.0, 6834838.0],
                        [856191.0, 6834907.0],
                        [863008.0, 6828766.0],
                        [873603.0, 6822775.0],
                        [882722.0, 6816263.0],
                        [893443.0, 6818701.0],
                        [904250.0, 6825381.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "55", "NOM_DEPT": "MEUSE"},
        },
        16: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    361697.30000012426,
                    6349285.200002406,
                    486169.1000000766,
                    6505244.800001987,
                ),
                "coordinates": [
                    [
                        [486169.0, 6410676.0],
                        [482141.0, 6421330.0],
                        [476390.0, 6418176.0],
                        [469504.0, 6418931.0],
                        [461839.0, 6422164.0],
                        [466012.0, 6428425.0],
                        [464018.0, 6432038.0],
                        [469719.0, 6445475.0],
                        [467756.0, 6450339.0],
                        [460931.0, 6449393.0],
                        [457280.0, 6451647.0],
                        [452009.0, 6448234.0],
                        [444096.0, 6451365.0],
                        [437492.0, 6457585.0],
                        [433418.0, 6457450.0],
                        [431214.0, 6470295.0],
                        [421098.0, 6472770.0],
                        [418478.0, 6475494.0],
                        [406100.0, 6476140.0],
                        [404682.0, 6481482.0],
                        [399796.0, 6489042.0],
                        [386164.0, 6505245.0],
                        [380995.0, 6504145.0],
                        [375328.0, 6494206.0],
                        [373905.0, 6475097.0],
                        [366594.0, 6427323.0],
                        [365668.0, 6410104.0],
                        [370987.0, 6417061.0],
                        [380337.0, 6407555.0],
                        [382533.0, 6402380.0],
                        [376517.0, 6401861.0],
                        [367865.0, 6404553.0],
                        [364521.0, 6395279.0],
                        [361697.0, 6391399.0],
                        [361774.0, 6383396.0],
                        [373686.0, 6386677.0],
                        [375548.0, 6389853.0],
                        [383562.0, 6383958.0],
                        [381182.0, 6379048.0],
                        [388231.0, 6379327.0],
                        [393973.0, 6376334.0],
                        [403228.0, 6378937.0],
                        [411314.0, 6378206.0],
                        [411174.0, 6373203.0],
                        [418928.0, 6369127.0],
                        [419635.0, 6366255.0],
                        [426389.0, 6364119.0],
                        [430077.0, 6359927.0],
                        [429214.0, 6351421.0],
                        [438523.0, 6349285.0],
                        [442465.0, 6350467.0],
                        [445091.0, 6357381.0],
                        [449164.0, 6352477.0],
                        [455164.0, 6354504.0],
                        [457916.0, 6359953.0],
                        [454032.0, 6364649.0],
                        [462536.0, 6370133.0],
                        [460366.0, 6373402.0],
                        [460343.0, 6383021.0],
                        [464993.0, 6388179.0],
                        [474218.0, 6394851.0],
                        [476701.0, 6399758.0],
                        [472909.0, 6402270.0],
                        [474628.0, 6407547.0],
                        [486169.0, 6410676.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "33", "NOM_DEPT": "GIRONDE"},
        },
        17: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    396867.2000000489,
                    6857101.400000965,
                    512505.0000000287,
                    6928957.800000754,
                ),
                "coordinates": [
                    [
                        [503908.0, 6928958.0],
                        [498291.0, 6928808.0],
                        [490213.0, 6925480.0],
                        [480923.0, 6917694.0],
                        [474554.0, 6915313.0],
                        [465316.0, 6913926.0],
                        [451821.0, 6920480.0],
                        [444571.0, 6921849.0],
                        [429556.0, 6922759.0],
                        [420281.0, 6924935.0],
                        [414059.0, 6928862.0],
                        [403953.0, 6928646.0],
                        [400759.0, 6925144.0],
                        [399047.0, 6920143.0],
                        [400663.0, 6914651.0],
                        [406933.0, 6907874.0],
                        [415797.0, 6907383.0],
                        [413253.0, 6901117.0],
                        [416434.0, 6899162.0],
                        [417375.0, 6892264.0],
                        [411444.0, 6881401.0],
                        [401493.0, 6871059.0],
                        [396867.0, 6868913.0],
                        [399642.0, 6861271.0],
                        [408651.0, 6861319.0],
                        [417723.0, 6857101.0],
                        [427244.0, 6861949.0],
                        [446005.0, 6869054.0],
                        [449827.0, 6868751.0],
                        [454384.0, 6863198.0],
                        [457511.0, 6867107.0],
                        [469104.0, 6863978.0],
                        [475607.0, 6865852.0],
                        [484257.0, 6870962.0],
                        [489675.0, 6876299.0],
                        [491678.0, 6873174.0],
                        [500558.0, 6877307.0],
                        [504730.0, 6874732.0],
                        [510545.0, 6875400.0],
                        [512505.0, 6881741.0],
                        [508338.0, 6886469.0],
                        [512079.0, 6895331.0],
                        [509950.0, 6897618.0],
                        [509502.0, 6910309.0],
                        [505278.0, 6914121.0],
                        [503908.0, 6928958.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "14", "NOM_DEPT": "CALVADOS"},
        },
        18: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    878315.099999967,
                    6752777.700001265,
                    1011125.3999999412,
                    6831653.50000104,
                ),
                "coordinates": [
                    [
                        [915394.0, 6762341.0],
                        [919858.0, 6768473.0],
                        [924039.0, 6766004.0],
                        [929025.0, 6772821.0],
                        [935284.0, 6772098.0],
                        [936416.0, 6766072.0],
                        [941726.0, 6764118.0],
                        [951150.0, 6767749.0],
                        [956126.0, 6765947.0],
                        [959909.0, 6759629.0],
                        [964569.0, 6761737.0],
                        [968808.0, 6766577.0],
                        [972278.0, 6762245.0],
                        [982963.0, 6756697.0],
                        [986052.0, 6752778.0],
                        [987691.0, 6753958.0],
                        [993060.0, 6757264.0],
                        [991355.0, 6761730.0],
                        [993983.0, 6773739.0],
                        [998205.0, 6776671.0],
                        [1001795.0, 6789636.0],
                        [1006708.0, 6801985.0],
                        [1011125.0, 6809400.0],
                        [1005338.0, 6811635.0],
                        [1003109.0, 6819572.0],
                        [1004385.0, 6831654.0],
                        [994239.0, 6827289.0],
                        [987769.0, 6820181.0],
                        [982282.0, 6817771.0],
                        [973656.0, 6818945.0],
                        [964951.0, 6823820.0],
                        [963830.0, 6819154.0],
                        [935203.0, 6815546.0],
                        [930831.0, 6810526.0],
                        [919362.0, 6809623.0],
                        [910921.0, 6826770.0],
                        [904250.0, 6825381.0],
                        [893443.0, 6818701.0],
                        [882722.0, 6816263.0],
                        [878315.0, 6811920.0],
                        [883843.0, 6809100.0],
                        [896020.0, 6796835.0],
                        [901295.0, 6794520.0],
                        [899609.0, 6786135.0],
                        [895837.0, 6779219.0],
                        [900794.0, 6778635.0],
                        [906998.0, 6773750.0],
                        [908017.0, 6765028.0],
                        [912655.0, 6766332.0],
                        [915394.0, 6762341.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "88", "NOM_DEPT": "VOSGES"},
        },
        19: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    607114.3000000231,
                    6591411.800001717,
                    706033.1999999984,
                    6725640.400001325,
                ),
                "coordinates": [
                    [
                        [642856.0, 6724786.0],
                        [634456.0, 6720842.0],
                        [639501.0, 6716775.0],
                        [642622.0, 6708275.0],
                        [632231.0, 6698879.0],
                        [634922.0, 6687596.0],
                        [624165.0, 6685496.0],
                        [619951.0, 6688294.0],
                        [617060.0, 6680060.0],
                        [612147.0, 6680536.0],
                        [612309.0, 6675948.0],
                        [607114.0, 6670785.0],
                        [615265.0, 6667454.0],
                        [621646.0, 6669974.0],
                        [627833.0, 6667741.0],
                        [627981.0, 6660408.0],
                        [631131.0, 6657674.0],
                        [629301.0, 6648616.0],
                        [633457.0, 6642556.0],
                        [627496.0, 6637025.0],
                        [632500.0, 6630848.0],
                        [628673.0, 6627353.0],
                        [634340.0, 6622075.0],
                        [637665.0, 6611745.0],
                        [635550.0, 6606712.0],
                        [638969.0, 6598575.0],
                        [634893.0, 6595681.0],
                        [636090.0, 6591904.0],
                        [644784.0, 6591412.0],
                        [645179.0, 6595086.0],
                        [650317.0, 6601560.0],
                        [660370.0, 6603761.0],
                        [664479.0, 6602292.0],
                        [670500.0, 6605989.0],
                        [669145.0, 6615318.0],
                        [677474.0, 6626628.0],
                        [682775.0, 6624336.0],
                        [686837.0, 6626145.0],
                        [696943.0, 6633749.0],
                        [702446.0, 6632753.0],
                        [705186.0, 6641947.0],
                        [703828.0, 6645606.0],
                        [706033.0, 6650549.0],
                        [705753.0, 6657660.0],
                        [701762.0, 6662486.0],
                        [702129.0, 6669867.0],
                        [698614.0, 6680189.0],
                        [698745.0, 6684384.0],
                        [690449.0, 6694237.0],
                        [694642.0, 6704915.0],
                        [690565.0, 6713347.0],
                        [685430.0, 6710473.0],
                        [682544.0, 6713710.0],
                        [678869.0, 6710238.0],
                        [674282.0, 6709782.0],
                        [669909.0, 6717111.0],
                        [661757.0, 6718781.0],
                        [657729.0, 6723417.0],
                        [652875.0, 6720759.0],
                        [646396.0, 6725640.0],
                        [642856.0, 6724786.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "18", "NOM_DEPT": "CHER"},
        },
        20: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    768276.2999999755,
                    6352552.800002385,
                    848816.399999948,
                    6475559.3000020515,
                ),
                "coordinates": [
                    [
                        [779411.0, 6373918.0],
                        [785084.0, 6367917.0],
                        [783861.0, 6358147.0],
                        [791158.0, 6357807.0],
                        [800494.0, 6352553.0],
                        [802764.0, 6358131.0],
                        [806518.0, 6360956.0],
                        [819866.0, 6361270.0],
                        [824346.0, 6357163.0],
                        [831641.0, 6353746.0],
                        [831614.0, 6360351.0],
                        [832713.0, 6371581.0],
                        [835151.0, 6374038.0],
                        [834538.0, 6384016.0],
                        [835764.0, 6388411.0],
                        [841041.0, 6396655.0],
                        [839715.0, 6410394.0],
                        [844069.0, 6414766.0],
                        [848816.0, 6428161.0],
                        [844782.0, 6435636.0],
                        [842390.0, 6453321.0],
                        [841110.0, 6468164.0],
                        [837457.0, 6475559.0],
                        [831517.0, 6473257.0],
                        [826509.0, 6469123.0],
                        [825694.0, 6462649.0],
                        [816370.0, 6460772.0],
                        [813436.0, 6450097.0],
                        [808187.0, 6450395.0],
                        [808165.0, 6446004.0],
                        [801638.0, 6433950.0],
                        [796332.0, 6429870.0],
                        [791307.0, 6420019.0],
                        [782106.0, 6419872.0],
                        [781273.0, 6415520.0],
                        [774103.0, 6411136.0],
                        [768276.0, 6405343.0],
                        [771844.0, 6388592.0],
                        [775042.0, 6386528.0],
                        [779411.0, 6373918.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "07", "NOM_DEPT": "ARDECHE"},
        },
        21: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [776081.0, 6923412.0],
                        [775403.0, 6934852.0],
                        [777906.0, 6941851.0],
                        [774574.0, 6946610.0],
                        [779463.0, 6948255.0],
                        [781387.0, 6953785.0],
                        [790134.0, 6962730.0],
                        [787172.0, 6965431.0],
                        [789845.0, 6973793.0],
                        [788558.0, 6985051.0],
                        [781905.0, 6987282.0],
                        [778060.0, 6986162.0],
                        [770359.0, 6988977.0],
                        [766237.0, 6992385.0],
                        [753505.0, 6995276.0],
                        [751253.0, 6997000.0],
                        [744014.0, 6992056.0],
                        [739058.0, 6995179.0],
                        [735248.0, 6991264.0],
                        [725313.0, 6993104.0],
                        [720100.0, 6990781.0],
                        [716534.0, 6992565.0],
                        [712391.0, 6990404.0],
                        [713833.0, 6986563.0],
                        [708480.0, 6979518.0],
                        [705667.0, 6969289.0],
                        [708546.0, 6956332.0],
                        [707064.0, 6950845.0],
                        [709520.0, 6938240.0],
                        [706939.0, 6934901.0],
                        [711648.0, 6928031.0],
                        [706805.0, 6926038.0],
                        [706929.0, 6919738.0],
                        [698137.0, 6911415.0],
                        [701957.0, 6908433.0],
                        [704672.0, 6899225.0],
                        [710189.0, 6894765.0],
                        [705248.0, 6890863.0],
                        [712067.0, 6888882.0],
                        [712559.0, 6879371.0],
                        [722321.0, 6872132.0],
                        [724211.0, 6867685.0],
                        [729581.0, 6862815.0],
                        [735603.0, 6861428.0],
                        [738742.0, 6868146.0],
                        [744067.0, 6871735.0],
                        [747254.0, 6882494.0],
                        [743801.0, 6891376.0],
                        [745398.0, 6894771.0],
                        [751361.0, 6898188.0],
                        [747051.0, 6913033.0],
                        [761575.0, 6918670.0],
                        [767112.0, 6923360.0],
                        [775242.0, 6918312.0],
                        [776081.0, 6923412.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "02", "NOM_DEPT": "AISNE"},
        },
        22: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    314107.40000015876,
                    6194685.200002778,
                    457776.50000009924,
                    6283529.400002572,
                ),
                "coordinates": [
                    [
                        [334283.0, 6280551.0],
                        [327832.0, 6270658.0],
                        [324268.0, 6267623.0],
                        [314107.0, 6264613.0],
                        [316186.0, 6255555.0],
                        [321493.0, 6257288.0],
                        [325696.0, 6250142.0],
                        [329513.0, 6253905.0],
                        [334359.0, 6254169.0],
                        [344050.0, 6249226.0],
                        [343580.0, 6242296.0],
                        [340751.0, 6235590.0],
                        [335750.0, 6230517.0],
                        [338499.0, 6226346.0],
                        [344947.0, 6224142.0],
                        [346289.0, 6231519.0],
                        [352412.0, 6225527.0],
                        [355170.0, 6226484.0],
                        [373097.0, 6218255.0],
                        [377794.0, 6214129.0],
                        [388910.0, 6213290.0],
                        [393755.0, 6214784.0],
                        [395170.0, 6206882.0],
                        [409841.0, 6194695.0],
                        [412796.0, 6198151.0],
                        [418372.0, 6194685.0],
                        [422245.0, 6194862.0],
                        [428954.0, 6200122.0],
                        [428150.0, 6207547.0],
                        [431947.0, 6209367.0],
                        [431385.0, 6215423.0],
                        [437107.0, 6220297.0],
                        [440142.0, 6228799.0],
                        [443868.0, 6230545.0],
                        [446410.0, 6236240.0],
                        [454979.0, 6245934.0],
                        [452746.0, 6249402.0],
                        [457777.0, 6262698.0],
                        [451477.0, 6264635.0],
                        [454138.0, 6272903.0],
                        [449863.0, 6280833.0],
                        [443497.0, 6282643.0],
                        [438084.0, 6281592.0],
                        [433057.0, 6279053.0],
                        [424830.0, 6280287.0],
                        [421554.0, 6283529.0],
                        [412501.0, 6278002.0],
                        [403447.0, 6281078.0],
                        [399714.0, 6278404.0],
                        [394475.0, 6281218.0],
                        [385124.0, 6280151.0],
                        [377063.0, 6275535.0],
                        [367753.0, 6276986.0],
                        [362964.0, 6280741.0],
                        [352961.0, 6276001.0],
                        [342731.0, 6276517.0],
                        [338238.0, 6281381.0],
                        [334283.0, 6280551.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "64", "NOM_DEPT": "PYRENEES-ATLANTIQUES"},
        },
        23: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    518605.20000003977,
                    6677557.600001468,
                    642856.0000000128,
                    6782681.400001166,
                ),
                "coordinates": [
                    [
                        [642856.0, 6724786.0],
                        [640167.0, 6731164.0],
                        [620646.0, 6731114.0],
                        [613256.0, 6732409.0],
                        [609792.0, 6727136.0],
                        [605429.0, 6729771.0],
                        [602374.0, 6738361.0],
                        [591237.0, 6742041.0],
                        [590280.0, 6747700.0],
                        [594464.0, 6753836.0],
                        [589886.0, 6759775.0],
                        [589601.0, 6765689.0],
                        [573758.0, 6762848.0],
                        [566408.0, 6764638.0],
                        [559999.0, 6772636.0],
                        [559439.0, 6777275.0],
                        [551727.0, 6777731.0],
                        [551017.0, 6782681.0],
                        [539332.0, 6780273.0],
                        [535875.0, 6773086.0],
                        [539183.0, 6762306.0],
                        [532634.0, 6757689.0],
                        [533549.0, 6751156.0],
                        [528757.0, 6745774.0],
                        [518605.0, 6737439.0],
                        [521086.0, 6735350.0],
                        [530153.0, 6735215.0],
                        [539386.0, 6732046.0],
                        [543743.0, 6727862.0],
                        [552256.0, 6724799.0],
                        [555366.0, 6719856.0],
                        [552945.0, 6716389.0],
                        [559271.0, 6706788.0],
                        [557303.0, 6702930.0],
                        [558309.0, 6696538.0],
                        [557051.0, 6690388.0],
                        [561058.0, 6687382.0],
                        [565759.0, 6689706.0],
                        [569696.0, 6685244.0],
                        [573297.0, 6677558.0],
                        [582631.0, 6683116.0],
                        [593597.0, 6687211.0],
                        [602149.0, 6685865.0],
                        [612147.0, 6680536.0],
                        [617060.0, 6680060.0],
                        [619951.0, 6688294.0],
                        [624165.0, 6685496.0],
                        [634922.0, 6687596.0],
                        [632231.0, 6698879.0],
                        [642622.0, 6708275.0],
                        [639501.0, 6716775.0],
                        [634456.0, 6720842.0],
                        [642856.0, 6724786.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "41", "NOM_DEPT": "LOIR-ET-CHER"},
        },
        24: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    909643.199999973,
                    6833381.500001036,
                    1038623.699999949,
                    6939384.60000073,
                ),
                "coordinates": [
                    [
                        [1001023.0, 6834020.0],
                        [1007603.0, 6833382.0],
                        [1013753.0, 6840510.0],
                        [1015426.0, 6846104.0],
                        [1012123.0, 6851875.0],
                        [1016765.0, 6860562.0],
                        [1004387.0, 6868517.0],
                        [1002983.0, 6863593.0],
                        [997609.0, 6864713.0],
                        [996607.0, 6871102.0],
                        [990508.0, 6874464.0],
                        [991999.0, 6880708.0],
                        [996400.0, 6888804.0],
                        [1000174.0, 6890502.0],
                        [1005257.0, 6886565.0],
                        [1014136.0, 6883372.0],
                        [1016790.0, 6880171.0],
                        [1025429.0, 6883147.0],
                        [1032318.0, 6879884.0],
                        [1035096.0, 6882767.0],
                        [1038624.0, 6893752.0],
                        [1031356.0, 6897552.0],
                        [1023578.0, 6906994.0],
                        [1018182.0, 6905695.0],
                        [1013674.0, 6899390.0],
                        [1009903.0, 6900358.0],
                        [996736.0, 6898172.0],
                        [992282.0, 6906134.0],
                        [986611.0, 6909639.0],
                        [979390.0, 6908077.0],
                        [979652.0, 6901424.0],
                        [972590.0, 6902443.0],
                        [968604.0, 6909024.0],
                        [966935.0, 6915200.0],
                        [959086.0, 6922273.0],
                        [960438.0, 6926520.0],
                        [957718.0, 6930675.0],
                        [948608.0, 6936180.0],
                        [945901.0, 6934734.0],
                        [938917.0, 6936180.0],
                        [935871.0, 6939385.0],
                        [928691.0, 6938466.0],
                        [926408.0, 6934964.0],
                        [916749.0, 6932316.0],
                        [915342.0, 6936492.0],
                        [909643.0, 6936903.0],
                        [913390.0, 6932141.0],
                        [912688.0, 6923499.0],
                        [920628.0, 6907653.0],
                        [917676.0, 6905219.0],
                        [920671.0, 6900380.0],
                        [914011.0, 6893894.0],
                        [914442.0, 6889764.0],
                        [922708.0, 6883240.0],
                        [922735.0, 6879575.0],
                        [928151.0, 6878713.0],
                        [929417.0, 6875078.0],
                        [939656.0, 6875317.0],
                        [945024.0, 6864515.0],
                        [946451.0, 6858744.0],
                        [961206.0, 6856322.0],
                        [964692.0, 6852206.0],
                        [969100.0, 6851558.0],
                        [971773.0, 6847764.0],
                        [976380.0, 6847597.0],
                        [983189.0, 6843093.0],
                        [993487.0, 6842033.0],
                        [1001023.0, 6834020.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "57", "NOM_DEPT": "MOSELLE"},
        },
        25: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    464624.4000000593,
                    6554736.200001831,
                    562772.8000000388,
                    6677880.4000014765,
                ),
                "coordinates": [
                    [
                        [483517.0, 6558927.0],
                        [489751.0, 6554866.0],
                        [499249.0, 6554736.0],
                        [507509.0, 6562167.0],
                        [509938.0, 6556936.0],
                        [521367.0, 6557890.0],
                        [526089.0, 6562344.0],
                        [531964.0, 6561069.0],
                        [530127.0, 6570272.0],
                        [538896.0, 6578287.0],
                        [546474.0, 6577615.0],
                        [548271.0, 6584464.0],
                        [556218.0, 6586207.0],
                        [559931.0, 6588729.0],
                        [562773.0, 6594026.0],
                        [558128.0, 6596023.0],
                        [557842.0, 6602380.0],
                        [547958.0, 6609437.0],
                        [542426.0, 6611026.0],
                        [538906.0, 6616112.0],
                        [541431.0, 6623495.0],
                        [537221.0, 6629765.0],
                        [533123.0, 6635020.0],
                        [531658.0, 6641492.0],
                        [525272.0, 6647330.0],
                        [524608.0, 6655240.0],
                        [515158.0, 6658631.0],
                        [517389.0, 6653506.0],
                        [510389.0, 6653999.0],
                        [505157.0, 6650870.0],
                        [494833.0, 6656131.0],
                        [494947.0, 6665028.0],
                        [485608.0, 6667284.0],
                        [482922.0, 6672972.0],
                        [478404.0, 6673399.0],
                        [476861.0, 6677880.0],
                        [471947.0, 6677374.0],
                        [469141.0, 6670338.0],
                        [464624.0, 6667352.0],
                        [465537.0, 6662685.0],
                        [471018.0, 6649618.0],
                        [469757.0, 6639127.0],
                        [470952.0, 6628365.0],
                        [467588.0, 6622657.0],
                        [472374.0, 6614809.0],
                        [467690.0, 6607234.0],
                        [468300.0, 6594579.0],
                        [472081.0, 6590232.0],
                        [472141.0, 6584899.0],
                        [475179.0, 6582497.0],
                        [479925.0, 6587264.0],
                        [482897.0, 6584792.0],
                        [478920.0, 6578170.0],
                        [477460.0, 6571967.0],
                        [480502.0, 6565882.0],
                        [485043.0, 6563684.0],
                        [483517.0, 6558927.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "86", "NOM_DEPT": "VIENNE"},
        },
        26: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    460931.20000008226,
                    6388334.100002295,
                    577773.9000000425,
                    6515544.000001942,
                ),
                "coordinates": [
                    [
                        [515635.0, 6515544.0],
                        [513233.0, 6510903.0],
                        [505177.0, 6505055.0],
                        [504657.0, 6496986.0],
                        [493466.0, 6488572.0],
                        [489646.0, 6488189.0],
                        [484697.0, 6477489.0],
                        [485798.0, 6470195.0],
                        [474561.0, 6460744.0],
                        [468806.0, 6462928.0],
                        [464807.0, 6459164.0],
                        [460931.0, 6449393.0],
                        [467756.0, 6450339.0],
                        [469719.0, 6445475.0],
                        [464018.0, 6432038.0],
                        [466012.0, 6428425.0],
                        [461839.0, 6422164.0],
                        [469504.0, 6418931.0],
                        [476390.0, 6418176.0],
                        [482141.0, 6421330.0],
                        [486169.0, 6410676.0],
                        [489091.0, 6407815.0],
                        [491196.0, 6399384.0],
                        [495031.0, 6397493.0],
                        [512334.0, 6403639.0],
                        [514373.0, 6400391.0],
                        [527849.0, 6401566.0],
                        [526858.0, 6394383.0],
                        [530971.0, 6390971.0],
                        [539761.0, 6395885.0],
                        [547193.0, 6388334.0],
                        [553597.0, 6395079.0],
                        [553113.0, 6398528.0],
                        [559319.0, 6399932.0],
                        [564402.0, 6403210.0],
                        [570668.0, 6413830.0],
                        [570522.0, 6417080.0],
                        [576964.0, 6421036.0],
                        [574976.0, 6434607.0],
                        [577774.0, 6436765.0],
                        [573095.0, 6446142.0],
                        [566113.0, 6450433.0],
                        [564865.0, 6461779.0],
                        [560988.0, 6465186.0],
                        [568750.0, 6477311.0],
                        [563844.0, 6479160.0],
                        [563452.0, 6484264.0],
                        [553144.0, 6489340.0],
                        [546900.0, 6501981.0],
                        [539039.0, 6503445.0],
                        [529342.0, 6499739.0],
                        [524801.0, 6504583.0],
                        [526849.0, 6509996.0],
                        [515635.0, 6515544.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "24", "NOM_DEPT": "DORDOGNE"},
        },
        27: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    871407.6999999562,
                    6576892.300001767,
                    944366.999999935,
                    6692352.700001431,
                ),
                "coordinates": [
                    [
                        [877344.0, 6596683.0],
                        [877221.0, 6593352.0],
                        [882664.0, 6589574.0],
                        [882486.0, 6586333.0],
                        [887558.0, 6582363.0],
                        [890444.0, 6576892.0],
                        [895736.0, 6577631.0],
                        [903737.0, 6585593.0],
                        [908880.0, 6582354.0],
                        [909869.0, 6577050.0],
                        [919460.0, 6577537.0],
                        [923919.0, 6580132.0],
                        [929573.0, 6589336.0],
                        [935280.0, 6595262.0],
                        [935835.0, 6600587.0],
                        [941800.0, 6609883.0],
                        [940343.0, 6611180.0],
                        [933274.0, 6616467.0],
                        [937312.0, 6621303.0],
                        [934547.0, 6625608.0],
                        [942748.0, 6631718.0],
                        [944367.0, 6635241.0],
                        [940225.0, 6640408.0],
                        [928880.0, 6645788.0],
                        [925393.0, 6653995.0],
                        [925599.0, 6657600.0],
                        [913780.0, 6661601.0],
                        [909981.0, 6669146.0],
                        [913407.0, 6674258.0],
                        [912920.0, 6678099.0],
                        [905832.0, 6683506.0],
                        [904021.0, 6688452.0],
                        [896736.0, 6687711.0],
                        [890265.0, 6692353.0],
                        [888032.0, 6690633.0],
                        [887407.0, 6682465.0],
                        [884880.0, 6674426.0],
                        [881002.0, 6667371.0],
                        [876191.0, 6666228.0],
                        [875449.0, 6660099.0],
                        [871408.0, 6655745.0],
                        [872158.0, 6650880.0],
                        [876345.0, 6647462.0],
                        [887343.0, 6642371.0],
                        [880659.0, 6638936.0],
                        [882575.0, 6628163.0],
                        [886706.0, 6618161.0],
                        [880824.0, 6605207.0],
                        [885325.0, 6599852.0],
                        [877344.0, 6596683.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "39", "NOM_DEPT": "JURA"},
        },
        28: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    519117.00000006985,
                    6298479.3000025265,
                    619220.100000031,
                    6366276.5000023525,
                ),
                "coordinates": [
                    [
                        [610874.0, 6360746.0],
                        [596271.0, 6358053.0],
                        [592401.0, 6355117.0],
                        [585843.0, 6356520.0],
                        [581893.0, 6352690.0],
                        [570617.0, 6348643.0],
                        [562737.0, 6350024.0],
                        [549131.0, 6360516.0],
                        [545774.0, 6366277.0],
                        [544593.0, 6364542.0],
                        [531660.0, 6365290.0],
                        [530427.0, 6360537.0],
                        [536387.0, 6355171.0],
                        [533815.0, 6350996.0],
                        [529462.0, 6338671.0],
                        [520115.0, 6336256.0],
                        [519117.0, 6332164.0],
                        [521740.0, 6328834.0],
                        [528536.0, 6329188.0],
                        [520187.0, 6318466.0],
                        [528381.0, 6316132.0],
                        [531142.0, 6309545.0],
                        [531445.0, 6300603.0],
                        [535303.0, 6300840.0],
                        [550735.0, 6304341.0],
                        [556606.0, 6298479.0],
                        [568130.0, 6303820.0],
                        [567948.0, 6310839.0],
                        [575242.0, 6309509.0],
                        [583992.0, 6314318.0],
                        [583816.0, 6319290.0],
                        [589156.0, 6320414.0],
                        [595091.0, 6325907.0],
                        [592665.0, 6336179.0],
                        [599594.0, 6336049.0],
                        [602120.0, 6333848.0],
                        [607335.0, 6338978.0],
                        [613603.0, 6341028.0],
                        [619220.0, 6339455.0],
                        [618185.0, 6342854.0],
                        [612268.0, 6345645.0],
                        [617826.0, 6353582.0],
                        [612319.0, 6353954.0],
                        [610874.0, 6360746.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "82", "NOM_DEPT": "TARN-ET-GARONNE"},
        },
        29: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    374866.6000000779,
                    6660135.3000015365,
                    491930.9000000467,
                    6754022.100001265,
                ),
                "coordinates": [
                    [
                        [404251.0, 6660135.0],
                        [413368.0, 6662854.0],
                        [417889.0, 6660618.0],
                        [424993.0, 6661094.0],
                        [429262.0, 6663802.0],
                        [429925.0, 6668493.0],
                        [434893.0, 6670550.0],
                        [442005.0, 6668948.0],
                        [454238.0, 6672319.0],
                        [458546.0, 6672434.0],
                        [464624.0, 6667352.0],
                        [469141.0, 6670338.0],
                        [471947.0, 6677374.0],
                        [476861.0, 6677880.0],
                        [479254.0, 6691029.0],
                        [487846.0, 6706491.0],
                        [490772.0, 6714971.0],
                        [489539.0, 6719725.0],
                        [491931.0, 6726763.0],
                        [485937.0, 6723990.0],
                        [478533.0, 6727039.0],
                        [474490.0, 6731769.0],
                        [469484.0, 6733109.0],
                        [464631.0, 6730466.0],
                        [460339.0, 6732350.0],
                        [457617.0, 6739093.0],
                        [450112.0, 6740918.0],
                        [446732.0, 6745443.0],
                        [440863.0, 6746201.0],
                        [429458.0, 6743442.0],
                        [419671.0, 6744167.0],
                        [412442.0, 6746090.0],
                        [404251.0, 6751414.0],
                        [402067.0, 6747685.0],
                        [393110.0, 6750366.0],
                        [389872.0, 6749698.0],
                        [382932.0, 6754022.0],
                        [382163.0, 6750361.0],
                        [381255.0, 6745617.0],
                        [386987.0, 6740963.0],
                        [389301.0, 6732485.0],
                        [398982.0, 6728603.0],
                        [395905.0, 6725901.0],
                        [386462.0, 6727467.0],
                        [387234.0, 6720426.0],
                        [401401.0, 6717774.0],
                        [402393.0, 6705897.0],
                        [396890.0, 6703794.0],
                        [385609.0, 6704471.0],
                        [375470.0, 6701871.0],
                        [374867.0, 6698071.0],
                        [378391.0, 6691949.0],
                        [382659.0, 6691801.0],
                        [384839.0, 6683588.0],
                        [379159.0, 6679552.0],
                        [379298.0, 6676338.0],
                        [387784.0, 6670618.0],
                        [385082.0, 6667087.0],
                        [392012.0, 6664073.0],
                        [401283.0, 6663962.0],
                        [404251.0, 6660135.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "49", "NOM_DEPT": "MAINE-ET-LOIRE"},
        },
        30: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    798995.1999999703,
                    6485556.600002024,
                    865784.4999999475,
                    6579430.300001755,
                ),
                "coordinates": [
                    [
                        [863140.0, 6525920.0],
                        [849404.0, 6524528.0],
                        [844536.0, 6535588.0],
                        [833933.0, 6540287.0],
                        [835850.0, 6543054.0],
                        [834501.0, 6551198.0],
                        [835220.0, 6556428.0],
                        [838741.0, 6561678.0],
                        [837322.0, 6565642.0],
                        [833486.0, 6565745.0],
                        [830305.0, 6579430.0],
                        [824653.0, 6575157.0],
                        [815826.0, 6575242.0],
                        [808209.0, 6578315.0],
                        [806993.0, 6569822.0],
                        [810991.0, 6564128.0],
                        [802035.0, 6559678.0],
                        [798995.0, 6551156.0],
                        [801039.0, 6547667.0],
                        [800220.0, 6540120.0],
                        [802643.0, 6534285.0],
                        [807296.0, 6530310.0],
                        [808147.0, 6518212.0],
                        [806255.0, 6511913.0],
                        [809870.0, 6504594.0],
                        [818524.0, 6498224.0],
                        [825934.0, 6498617.0],
                        [829797.0, 6488843.0],
                        [834511.0, 6489782.0],
                        [837308.0, 6485557.0],
                        [845846.0, 6493277.0],
                        [841056.0, 6498593.0],
                        [848198.0, 6502534.0],
                        [858527.0, 6506228.0],
                        [865784.0, 6514252.0],
                        [862850.0, 6517700.0],
                        [863140.0, 6525920.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "69", "NOM_DEPT": "RHONE"},
        },
        31: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    503908.00000002584,
                    6843337.50000099,
                    610930.7000000123,
                    6933754.900000739,
                ),
                "coordinates": [
                    [
                        [606642.0, 6924066.0],
                        [602657.0, 6922993.0],
                        [596650.0, 6927678.0],
                        [584841.0, 6929723.0],
                        [579971.0, 6928884.0],
                        [574422.0, 6917678.0],
                        [566281.0, 6917384.0],
                        [564288.0, 6913853.0],
                        [558013.0, 6912674.0],
                        [555352.0, 6907664.0],
                        [548914.0, 6917432.0],
                        [548813.0, 6922655.0],
                        [537794.0, 6926754.0],
                        [530512.0, 6925008.0],
                        [524567.0, 6928708.0],
                        [520591.0, 6933755.0],
                        [506978.0, 6930086.0],
                        [503908.0, 6928958.0],
                        [505278.0, 6914121.0],
                        [509502.0, 6910309.0],
                        [509950.0, 6897618.0],
                        [512079.0, 6895331.0],
                        [508338.0, 6886469.0],
                        [512505.0, 6881741.0],
                        [510545.0, 6875400.0],
                        [508448.0, 6870995.0],
                        [513552.0, 6867418.0],
                        [520512.0, 6866728.0],
                        [524600.0, 6861683.0],
                        [533488.0, 6856087.0],
                        [536389.0, 6850906.0],
                        [533024.0, 6847407.0],
                        [539112.0, 6843338.0],
                        [543803.0, 6848251.0],
                        [555241.0, 6849633.0],
                        [566604.0, 6854037.0],
                        [576482.0, 6852521.0],
                        [580772.0, 6855857.0],
                        [579784.0, 6860594.0],
                        [587701.0, 6866890.0],
                        [590236.0, 6872273.0],
                        [587216.0, 6881812.0],
                        [591984.0, 6886421.0],
                        [598361.0, 6887345.0],
                        [601892.0, 6893098.0],
                        [603501.0, 6902160.0],
                        [605624.0, 6904387.0],
                        [606128.0, 6907968.0],
                        [610931.0, 6911811.0],
                        [609871.0, 6919444.0],
                        [606642.0, 6924066.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "27", "NOM_DEPT": "EURE"},
        },
        32: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    607709.4000000342,
                    6288228.600002549,
                    736139.9999999859,
                    6426924.700002184,
                ),
                "coordinates": [
                    [
                        [619220.0, 6339455.0],
                        [628552.0, 6344336.0],
                        [632260.0, 6344664.0],
                        [651092.0, 6332910.0],
                        [652742.0, 6328715.0],
                        [656723.0, 6328192.0],
                        [661724.0, 6320369.0],
                        [659810.0, 6316970.0],
                        [664259.0, 6313698.0],
                        [664946.0, 6304799.0],
                        [674324.0, 6293934.0],
                        [679276.0, 6292258.0],
                        [684993.0, 6295873.0],
                        [693643.0, 6293013.0],
                        [694796.0, 6288406.0],
                        [698819.0, 6289943.0],
                        [704898.0, 6288229.0],
                        [704815.0, 6303628.0],
                        [716344.0, 6301672.0],
                        [722102.0, 6311077.0],
                        [728786.0, 6312824.0],
                        [736140.0, 6324746.0],
                        [726344.0, 6331169.0],
                        [726879.0, 6340020.0],
                        [729879.0, 6341376.0],
                        [728598.0, 6344387.0],
                        [718362.0, 6347498.0],
                        [709994.0, 6353946.0],
                        [712305.0, 6356606.0],
                        [709537.0, 6364934.0],
                        [710800.0, 6372878.0],
                        [705449.0, 6378272.0],
                        [706639.0, 6384441.0],
                        [698547.0, 6393953.0],
                        [693924.0, 6403282.0],
                        [694820.0, 6408897.0],
                        [690477.0, 6411547.0],
                        [688126.0, 6419157.0],
                        [684359.0, 6419403.0],
                        [679353.0, 6426925.0],
                        [676829.0, 6423158.0],
                        [668604.0, 6416079.0],
                        [668313.0, 6410487.0],
                        [665404.0, 6408608.0],
                        [664849.0, 6402561.0],
                        [655192.0, 6393469.0],
                        [646639.0, 6396959.0],
                        [637228.0, 6394175.0],
                        [637122.0, 6391031.0],
                        [632813.0, 6386229.0],
                        [625170.0, 6387538.0],
                        [615061.0, 6379167.0],
                        [607709.0, 6375883.0],
                        [610874.0, 6360746.0],
                        [612319.0, 6353954.0],
                        [617826.0, 6353582.0],
                        [612268.0, 6345645.0],
                        [618185.0, 6342854.0],
                        [619220.0, 6339455.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "12", "NOM_DEPT": "AVEYRON"},
        },
        33: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    574729.8000000368,
                    6509220.700001954,
                    668605.1000000095,
                    6592907.500001715,
                ),
                "coordinates": [
                    [
                        [636090.0, 6591904.0],
                        [630016.0, 6590287.0],
                        [622684.0, 6592821.0],
                        [609355.0, 6592908.0],
                        [602234.0, 6588456.0],
                        [595863.0, 6588324.0],
                        [586530.0, 6592902.0],
                        [578129.0, 6584256.0],
                        [580236.0, 6579624.0],
                        [574730.0, 6569689.0],
                        [580656.0, 6565605.0],
                        [587795.0, 6549437.0],
                        [586335.0, 6540289.0],
                        [592200.0, 6538029.0],
                        [591561.0, 6529605.0],
                        [597279.0, 6527024.0],
                        [604752.0, 6527760.0],
                        [613074.0, 6522626.0],
                        [612524.0, 6515422.0],
                        [614305.0, 6511565.0],
                        [623634.0, 6517796.0],
                        [631053.0, 6514297.0],
                        [635791.0, 6515072.0],
                        [637278.0, 6511373.0],
                        [648450.0, 6509221.0],
                        [649366.0, 6512420.0],
                        [656101.0, 6511235.0],
                        [660507.0, 6515469.0],
                        [656022.0, 6519074.0],
                        [652489.0, 6525488.0],
                        [656753.0, 6529414.0],
                        [660605.0, 6529501.0],
                        [668605.0, 6543382.0],
                        [665172.0, 6557241.0],
                        [666451.0, 6560450.0],
                        [662729.0, 6571178.0],
                        [659880.0, 6575828.0],
                        [653217.0, 6581299.0],
                        [647921.0, 6581264.0],
                        [644784.0, 6591412.0],
                        [636090.0, 6591904.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "23", "NOM_DEPT": "CREUSE"},
        },
        34: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    589164.5000000226,
                    6709782.20000137,
                    709584.899999998,
                    6802433.8000011025,
                ),
                "coordinates": [
                    [
                        [695265.0, 6784783.0],
                        [685053.0, 6785343.0],
                        [678166.0, 6780535.0],
                        [665644.0, 6782354.0],
                        [663338.0, 6793247.0],
                        [657236.0, 6795707.0],
                        [655721.0, 6802434.0],
                        [653275.0, 6801114.0],
                        [639308.0, 6801806.0],
                        [625387.0, 6798948.0],
                        [623220.0, 6795517.0],
                        [615539.0, 6776549.0],
                        [606511.0, 6774717.0],
                        [597558.0, 6774986.0],
                        [595216.0, 6770968.0],
                        [589165.0, 6770859.0],
                        [589601.0, 6765689.0],
                        [589886.0, 6759775.0],
                        [594464.0, 6753836.0],
                        [590280.0, 6747700.0],
                        [591237.0, 6742041.0],
                        [602374.0, 6738361.0],
                        [605429.0, 6729771.0],
                        [609792.0, 6727136.0],
                        [613256.0, 6732409.0],
                        [620646.0, 6731114.0],
                        [640167.0, 6731164.0],
                        [642856.0, 6724786.0],
                        [646396.0, 6725640.0],
                        [652875.0, 6720759.0],
                        [657729.0, 6723417.0],
                        [661757.0, 6718781.0],
                        [669909.0, 6717111.0],
                        [674282.0, 6709782.0],
                        [678869.0, 6710238.0],
                        [682544.0, 6713710.0],
                        [685430.0, 6710473.0],
                        [690565.0, 6713347.0],
                        [693556.0, 6718403.0],
                        [698236.0, 6718784.0],
                        [695341.0, 6722058.0],
                        [694231.0, 6731133.0],
                        [689391.0, 6734632.0],
                        [689262.0, 6740069.0],
                        [695135.0, 6740319.0],
                        [701782.0, 6742907.0],
                        [700769.0, 6756036.0],
                        [703743.0, 6756785.0],
                        [709585.0, 6763406.0],
                        [706580.0, 6772621.0],
                        [703759.0, 6774663.0],
                        [702192.0, 6781427.0],
                        [695265.0, 6784783.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "45", "NOM_DEPT": "LOIRET"},
        },
        35: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    878352.0999999599,
                    6687710.700001444,
                    986155.499999935,
                    6772821.300001199,
                ),
                "coordinates": [
                    [
                        [986052.0, 6752778.0],
                        [982963.0, 6756697.0],
                        [972278.0, 6762245.0],
                        [968808.0, 6766577.0],
                        [964569.0, 6761737.0],
                        [959909.0, 6759629.0],
                        [956126.0, 6765947.0],
                        [951150.0, 6767749.0],
                        [941726.0, 6764118.0],
                        [936416.0, 6766072.0],
                        [935284.0, 6772098.0],
                        [929025.0, 6772821.0],
                        [924039.0, 6766004.0],
                        [919858.0, 6768473.0],
                        [915394.0, 6762341.0],
                        [910900.0, 6755744.0],
                        [906433.0, 6754603.0],
                        [901288.0, 6749897.0],
                        [900414.0, 6745482.0],
                        [902985.0, 6741752.0],
                        [901778.0, 6735043.0],
                        [894824.0, 6733348.0],
                        [886220.0, 6734507.0],
                        [880542.0, 6733086.0],
                        [878352.0, 6725366.0],
                        [882147.0, 6728546.0],
                        [886108.0, 6726069.0],
                        [887785.0, 6718944.0],
                        [884206.0, 6713509.0],
                        [880595.0, 6713705.0],
                        [884919.0, 6701062.0],
                        [890265.0, 6692353.0],
                        [896736.0, 6687711.0],
                        [904021.0, 6688452.0],
                        [922029.0, 6697905.0],
                        [925719.0, 6696195.0],
                        [932449.0, 6699390.0],
                        [945847.0, 6708437.0],
                        [950918.0, 6717032.0],
                        [955890.0, 6718838.0],
                        [967469.0, 6716290.0],
                        [969354.0, 6721610.0],
                        [974258.0, 6721322.0],
                        [977033.0, 6724531.0],
                        [984341.0, 6721925.0],
                        [986155.0, 6724947.0],
                        [983826.0, 6731782.0],
                        [981470.0, 6745308.0],
                        [986052.0, 6752778.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "70", "NOM_DEPT": "HAUTE-SAONE"},
        },
        36: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    652488.5000000148,
                    6465307.100002078,
                    776221.1999999749,
                    6572856.000001771,
                ),
                "coordinates": [
                    [
                        [753779.0, 6537018.0],
                        [745523.0, 6547324.0],
                        [735860.0, 6546571.0],
                        [732386.0, 6552810.0],
                        [728730.0, 6550656.0],
                        [712873.0, 6551781.0],
                        [697819.0, 6557998.0],
                        [693029.0, 6567076.0],
                        [695157.0, 6571452.0],
                        [688886.0, 6572856.0],
                        [685369.0, 6566281.0],
                        [679424.0, 6569309.0],
                        [672252.0, 6557694.0],
                        [666451.0, 6560450.0],
                        [665172.0, 6557241.0],
                        [668605.0, 6543382.0],
                        [660605.0, 6529501.0],
                        [656753.0, 6529414.0],
                        [652489.0, 6525488.0],
                        [656022.0, 6519074.0],
                        [660507.0, 6515469.0],
                        [663288.0, 6509261.0],
                        [662164.0, 6504560.0],
                        [658322.0, 6500286.0],
                        [662264.0, 6494997.0],
                        [661594.0, 6486680.0],
                        [672750.0, 6482640.0],
                        [677684.0, 6475835.0],
                        [688449.0, 6477003.0],
                        [693859.0, 6474045.0],
                        [695974.0, 6467752.0],
                        [701310.0, 6465307.0],
                        [708104.0, 6472781.0],
                        [713160.0, 6472555.0],
                        [722742.0, 6478087.0],
                        [735734.0, 6477978.0],
                        [739202.0, 6481030.0],
                        [749018.0, 6471190.0],
                        [752222.0, 6475420.0],
                        [760473.0, 6472496.0],
                        [763248.0, 6477235.0],
                        [770263.0, 6473476.0],
                        [770335.0, 6479388.0],
                        [776221.0, 6483674.0],
                        [774454.0, 6495613.0],
                        [770440.0, 6500348.0],
                        [764154.0, 6503988.0],
                        [760734.0, 6514163.0],
                        [754369.0, 6520637.0],
                        [758503.0, 6532097.0],
                        [753779.0, 6537018.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "63", "NOM_DEPT": "PUY-DE-DOME"},
        },
        37: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    583816.2000000454,
                    6255615.400002628,
                    694795.5000000021,
                    6344664.200002406,
                ),
                "coordinates": [
                    [
                        [583992.0, 6314318.0],
                        [583878.0, 6308763.0],
                        [586632.0, 6302842.0],
                        [590928.0, 6301074.0],
                        [592076.0, 6294561.0],
                        [595737.0, 6291653.0],
                        [597601.0, 6285124.0],
                        [594074.0, 6282175.0],
                        [605052.0, 6276459.0],
                        [609628.0, 6269734.0],
                        [620519.0, 6263967.0],
                        [621363.0, 6260238.0],
                        [624846.0, 6255615.0],
                        [637154.0, 6259493.0],
                        [640013.0, 6262596.0],
                        [651680.0, 6257911.0],
                        [654081.0, 6259728.0],
                        [664821.0, 6258302.0],
                        [668090.0, 6259189.0],
                        [672444.0, 6264623.0],
                        [672394.0, 6269695.0],
                        [668935.0, 6274250.0],
                        [671212.0, 6283521.0],
                        [676690.0, 6282993.0],
                        [680178.0, 6279485.0],
                        [693221.0, 6284557.0],
                        [694796.0, 6288406.0],
                        [693643.0, 6293013.0],
                        [684993.0, 6295873.0],
                        [679276.0, 6292258.0],
                        [674324.0, 6293934.0],
                        [664946.0, 6304799.0],
                        [664259.0, 6313698.0],
                        [659810.0, 6316970.0],
                        [661724.0, 6320369.0],
                        [656723.0, 6328192.0],
                        [652742.0, 6328715.0],
                        [651092.0, 6332910.0],
                        [632260.0, 6344664.0],
                        [628552.0, 6344336.0],
                        [619220.0, 6339455.0],
                        [613603.0, 6341028.0],
                        [607335.0, 6338978.0],
                        [602120.0, 6333848.0],
                        [599594.0, 6336049.0],
                        [592665.0, 6336179.0],
                        [595091.0, 6325907.0],
                        [589156.0, 6320414.0],
                        [583816.0, 6319290.0],
                        [583992.0, 6314318.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "81", "NOM_DEPT": "TARN"},
        },
        38: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    487453.2000000271,
                    6907663.900000811,
                    612313.8000000101,
                    6997484.700000563,
                ),
                "coordinates": [
                    [
                        [612314.0, 6962826.0],
                        [608347.0, 6970745.0],
                        [605014.0, 6980736.0],
                        [589321.0, 6994349.0],
                        [583880.0, 6997485.0],
                        [570354.0, 6986979.0],
                        [561015.0, 6982476.0],
                        [553619.0, 6982021.0],
                        [539569.0, 6977082.0],
                        [525341.0, 6975010.0],
                        [506777.0, 6963564.0],
                        [498821.0, 6960824.0],
                        [494437.0, 6953650.0],
                        [487453.0, 6938754.0],
                        [491281.0, 6933115.0],
                        [506978.0, 6930086.0],
                        [520591.0, 6933755.0],
                        [524567.0, 6928708.0],
                        [530512.0, 6925008.0],
                        [537794.0, 6926754.0],
                        [548813.0, 6922655.0],
                        [548914.0, 6917432.0],
                        [555352.0, 6907664.0],
                        [558013.0, 6912674.0],
                        [564288.0, 6913853.0],
                        [566281.0, 6917384.0],
                        [574422.0, 6917678.0],
                        [579971.0, 6928884.0],
                        [584841.0, 6929723.0],
                        [596650.0, 6927678.0],
                        [602657.0, 6922993.0],
                        [606642.0, 6924066.0],
                        [610545.0, 6930338.0],
                        [607205.0, 6935073.0],
                        [609123.0, 6937659.0],
                        [606321.0, 6945902.0],
                        [607933.0, 6953359.0],
                        [605401.0, 6955866.0],
                        [612314.0, 6962826.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "76", "NOM_DEPT": "SEINE-MARITIME"},
        },
        39: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    822210.4999999779,
                    6725365.600001331,
                    915393.7999999551,
                    6844875.900000985,
                ),
                "coordinates": [
                    [
                        [823302.0, 6827043.0],
                        [822210.0, 6820200.0],
                        [827022.0, 6811887.0],
                        [834073.0, 6806764.0],
                        [837935.0, 6797546.0],
                        [837598.0, 6783989.0],
                        [835407.0, 6779651.0],
                        [828770.0, 6781171.0],
                        [825768.0, 6776095.0],
                        [827043.0, 6770243.0],
                        [833398.0, 6769005.0],
                        [833316.0, 6764154.0],
                        [838226.0, 6763382.0],
                        [846087.0, 6753627.0],
                        [848388.0, 6746302.0],
                        [846031.0, 6742315.0],
                        [848455.0, 6733791.0],
                        [858300.0, 6729814.0],
                        [863148.0, 6730266.0],
                        [868188.0, 6726357.0],
                        [878352.0, 6725366.0],
                        [880542.0, 6733086.0],
                        [886220.0, 6734507.0],
                        [894824.0, 6733348.0],
                        [901778.0, 6735043.0],
                        [902985.0, 6741752.0],
                        [900414.0, 6745482.0],
                        [901288.0, 6749897.0],
                        [906433.0, 6754603.0],
                        [910900.0, 6755744.0],
                        [915394.0, 6762341.0],
                        [912655.0, 6766332.0],
                        [908017.0, 6765028.0],
                        [906998.0, 6773750.0],
                        [900794.0, 6778635.0],
                        [895837.0, 6779219.0],
                        [899609.0, 6786135.0],
                        [901295.0, 6794520.0],
                        [896020.0, 6796835.0],
                        [883843.0, 6809100.0],
                        [878315.0, 6811920.0],
                        [882722.0, 6816263.0],
                        [873603.0, 6822775.0],
                        [863008.0, 6828766.0],
                        [856191.0, 6834907.0],
                        [852692.0, 6834838.0],
                        [847795.0, 6838972.0],
                        [846365.0, 6844539.0],
                        [840675.0, 6844876.0],
                        [835676.0, 6840415.0],
                        [836281.0, 6837534.0],
                        [830469.0, 6833515.0],
                        [832829.0, 6827018.0],
                        [823302.0, 6827043.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "52", "NOM_DEPT": "HAUTE-MARNE"},
        },
        40: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    726344.2999999899,
                    6263118.800002612,
                    847769.4999999425,
                    6373918.100002329,
                ),
                "coordinates": [
                    [
                        [831641.0, 6353746.0],
                        [824346.0, 6357163.0],
                        [819866.0, 6361270.0],
                        [806518.0, 6360956.0],
                        [802764.0, 6358131.0],
                        [800494.0, 6352553.0],
                        [791158.0, 6357807.0],
                        [783861.0, 6358147.0],
                        [785084.0, 6367917.0],
                        [779411.0, 6373918.0],
                        [775211.0, 6367175.0],
                        [770571.0, 6365836.0],
                        [775941.0, 6359276.0],
                        [773621.0, 6356674.0],
                        [775968.0, 6350322.0],
                        [776174.0, 6341769.0],
                        [768987.0, 6337162.0],
                        [763780.0, 6336809.0],
                        [754256.0, 6342872.0],
                        [750995.0, 6342037.0],
                        [750700.0, 6336169.0],
                        [735189.0, 6336775.0],
                        [729879.0, 6341376.0],
                        [726879.0, 6340020.0],
                        [726344.0, 6331169.0],
                        [736140.0, 6324746.0],
                        [728786.0, 6312824.0],
                        [734096.0, 6312602.0],
                        [742673.0, 6306434.0],
                        [746263.0, 6307559.0],
                        [750088.0, 6313306.0],
                        [758340.0, 6319159.0],
                        [763184.0, 6318967.0],
                        [766255.0, 6313712.0],
                        [766790.0, 6307842.0],
                        [773726.0, 6310049.0],
                        [780634.0, 6301895.0],
                        [792244.0, 6293431.0],
                        [796379.0, 6284647.0],
                        [792922.0, 6276955.0],
                        [788998.0, 6273432.0],
                        [792112.0, 6270978.0],
                        [792074.0, 6265425.0],
                        [799607.0, 6263119.0],
                        [800175.0, 6267290.0],
                        [813903.0, 6274579.0],
                        [815212.0, 6281222.0],
                        [819908.0, 6290041.0],
                        [828463.0, 6288899.0],
                        [833168.0, 6302217.0],
                        [831988.0, 6308980.0],
                        [839658.0, 6315433.0],
                        [847769.0, 6322554.0],
                        [843231.0, 6331183.0],
                        [836689.0, 6335330.0],
                        [837656.0, 6344658.0],
                        [831641.0, 6353746.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "30", "NOM_DEPT": "GARD"},
        },
        41: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    990508.299999954,
                    6789869.300001165,
                    1081448.299999941,
                    6895577.800000864,
                ),
                "coordinates": [
                    [
                        [1011125.0, 6809400.0],
                        [1016850.0, 6809125.0],
                        [1022638.0, 6802981.0],
                        [1032428.0, 6798669.0],
                        [1032222.0, 6793924.0],
                        [1036067.0, 6790258.0],
                        [1040408.0, 6789869.0],
                        [1046450.0, 6801750.0],
                        [1048508.0, 6811325.0],
                        [1051141.0, 6812923.0],
                        [1050303.0, 6822049.0],
                        [1052109.0, 6831679.0],
                        [1054677.0, 6834498.0],
                        [1054069.0, 6843233.0],
                        [1056359.0, 6848835.0],
                        [1064869.0, 6858273.0],
                        [1065247.0, 6862338.0],
                        [1074008.0, 6868775.0],
                        [1077114.0, 6879387.0],
                        [1081448.0, 6887616.0],
                        [1072364.0, 6888601.0],
                        [1060439.0, 6895482.0],
                        [1055798.0, 6892558.0],
                        [1050607.0, 6895578.0],
                        [1045763.0, 6893084.0],
                        [1038624.0, 6893752.0],
                        [1035096.0, 6882767.0],
                        [1032318.0, 6879884.0],
                        [1025429.0, 6883147.0],
                        [1016790.0, 6880171.0],
                        [1014136.0, 6883372.0],
                        [1005257.0, 6886565.0],
                        [1000174.0, 6890502.0],
                        [996400.0, 6888804.0],
                        [991999.0, 6880708.0],
                        [990508.0, 6874464.0],
                        [996607.0, 6871102.0],
                        [997609.0, 6864713.0],
                        [1002983.0, 6863593.0],
                        [1004387.0, 6868517.0],
                        [1016765.0, 6860562.0],
                        [1012123.0, 6851875.0],
                        [1015426.0, 6846104.0],
                        [1013753.0, 6840510.0],
                        [1007603.0, 6833382.0],
                        [1001023.0, 6834020.0],
                        [1004385.0, 6831654.0],
                        [1003109.0, 6819572.0],
                        [1005338.0, 6811635.0],
                        [1011125.0, 6809400.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "67", "NOM_DEPT": "BAS-RHIN"},
        },
        42: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    593467.3000000443,
                    6174169.900002814,
                    719561.0999999918,
                    6262595.80000261,
                ),
                "coordinates": [
                    [
                        [719561.0, 6234874.0],
                        [716336.0, 6238907.0],
                        [711963.0, 6238657.0],
                        [700385.0, 6243263.0],
                        [700353.0, 6246775.0],
                        [686427.0, 6247043.0],
                        [681495.0, 6239871.0],
                        [667653.0, 6244475.0],
                        [662679.0, 6249784.0],
                        [664821.0, 6258302.0],
                        [654081.0, 6259728.0],
                        [651680.0, 6257911.0],
                        [640013.0, 6262596.0],
                        [637154.0, 6259493.0],
                        [624846.0, 6255615.0],
                        [621363.0, 6260238.0],
                        [611672.0, 6256714.0],
                        [607574.0, 6261192.0],
                        [603060.0, 6255463.0],
                        [603733.0, 6249863.0],
                        [598408.0, 6250205.0],
                        [593467.0, 6242482.0],
                        [596256.0, 6236985.0],
                        [595072.0, 6233132.0],
                        [604330.0, 6228108.0],
                        [614442.0, 6225039.0],
                        [613663.0, 6217624.0],
                        [617210.0, 6214637.0],
                        [617137.0, 6197372.0],
                        [608402.0, 6195450.0],
                        [610538.0, 6185966.0],
                        [614361.0, 6182470.0],
                        [622733.0, 6184536.0],
                        [632373.0, 6176261.0],
                        [631545.0, 6174170.0],
                        [644388.0, 6178927.0],
                        [647326.0, 6181831.0],
                        [644930.0, 6193342.0],
                        [655189.0, 6193206.0],
                        [659825.0, 6194610.0],
                        [675246.0, 6192375.0],
                        [678721.0, 6193157.0],
                        [682753.0, 6199641.0],
                        [688989.0, 6202113.0],
                        [693222.0, 6198384.0],
                        [703560.0, 6193187.0],
                        [704952.0, 6201963.0],
                        [703499.0, 6206754.0],
                        [709550.0, 6222514.0],
                        [719561.0, 6234874.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "11", "NOM_DEPT": "AUDE"},
        },
        43: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    655720.9000000084,
                    6780535.000001167,
                    740951.9999999929,
                    6890862.900000851,
                ),
                "coordinates": [
                    [
                        [695265.0, 6784783.0],
                        [700383.0, 6789702.0],
                        [703237.0, 6796855.0],
                        [703668.0, 6806647.0],
                        [712422.0, 6807964.0],
                        [727042.0, 6808060.0],
                        [730707.0, 6810078.0],
                        [728998.0, 6813854.0],
                        [731262.0, 6826009.0],
                        [737148.0, 6833964.0],
                        [740952.0, 6835710.0],
                        [733915.0, 6839303.0],
                        [735102.0, 6844463.0],
                        [732040.0, 6850473.0],
                        [735603.0, 6861428.0],
                        [729581.0, 6862815.0],
                        [724211.0, 6867685.0],
                        [722321.0, 6872132.0],
                        [712559.0, 6879371.0],
                        [712067.0, 6888882.0],
                        [705248.0, 6890863.0],
                        [700622.0, 6887964.0],
                        [680633.0, 6884542.0],
                        [673222.0, 6889902.0],
                        [670084.0, 6886723.0],
                        [667303.0, 6878971.0],
                        [670882.0, 6870004.0],
                        [667928.0, 6865519.0],
                        [670056.0, 6856451.0],
                        [671600.0, 6851283.0],
                        [668472.0, 6843626.0],
                        [665649.0, 6839786.0],
                        [666363.0, 6833836.0],
                        [663045.0, 6825117.0],
                        [663371.0, 6815578.0],
                        [665496.0, 6812431.0],
                        [657768.0, 6807009.0],
                        [655721.0, 6802434.0],
                        [657236.0, 6795707.0],
                        [663338.0, 6793247.0],
                        [665644.0, 6782354.0],
                        [678166.0, 6780535.0],
                        [685053.0, 6785343.0],
                        [695265.0, 6784783.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "77", "NOM_DEPT": "SEINE-ET-MARNE"},
        },
        44: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    583879.5000000115,
                    6942514.300000707,
                    713832.6999999986,
                    7030025.100000476,
                ),
                "coordinates": [
                    [
                        [706470.0, 6995042.0],
                        [691147.0, 6993758.0],
                        [689814.0, 6997681.0],
                        [669642.0, 7001764.0],
                        [667892.0, 7004913.0],
                        [662547.0, 7004433.0],
                        [658067.0, 7000416.0],
                        [656343.0, 7006536.0],
                        [663948.0, 7010834.0],
                        [660837.0, 7014835.0],
                        [654991.0, 7012592.0],
                        [648121.0, 7014552.0],
                        [635372.0, 7012312.0],
                        [634147.0, 7017146.0],
                        [624801.0, 7021563.0],
                        [622831.0, 7024302.0],
                        [612989.0, 7030025.0],
                        [605232.0, 7026713.0],
                        [603193.0, 7029103.0],
                        [596733.0, 7029014.0],
                        [595668.0, 7021497.0],
                        [599348.0, 7018454.0],
                        [591189.0, 7008466.0],
                        [589294.0, 7002423.0],
                        [583880.0, 6997485.0],
                        [589321.0, 6994349.0],
                        [605014.0, 6980736.0],
                        [608347.0, 6970745.0],
                        [612314.0, 6962826.0],
                        [620154.0, 6956108.0],
                        [623025.0, 6958366.0],
                        [636565.0, 6954687.0],
                        [645659.0, 6956103.0],
                        [654712.0, 6950974.0],
                        [664208.0, 6949037.0],
                        [669006.0, 6944292.0],
                        [676176.0, 6942514.0],
                        [677528.0, 6947426.0],
                        [685504.0, 6946842.0],
                        [685089.0, 6950779.0],
                        [693916.0, 6955962.0],
                        [708546.0, 6956332.0],
                        [705667.0, 6969289.0],
                        [708480.0, 6979518.0],
                        [713833.0, 6986563.0],
                        [712391.0, 6990404.0],
                        [706470.0, 6995042.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "80", "NOM_DEPT": "SOMME"},
        },
        45: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    708104.1999999973,
                    6405343.300002245,
                    816369.7999999607,
                    6481030.400002034,
                ),
                "coordinates": [
                    [
                        [708104.0, 6472781.0],
                        [709251.0, 6464988.0],
                        [717445.0, 6463649.0],
                        [722648.0, 6446835.0],
                        [725628.0, 6444038.0],
                        [723408.0, 6437470.0],
                        [727352.0, 6435060.0],
                        [728488.0, 6430309.0],
                        [732563.0, 6427378.0],
                        [732994.0, 6422485.0],
                        [737614.0, 6413026.0],
                        [746579.0, 6414955.0],
                        [746937.0, 6419760.0],
                        [752090.0, 6419191.0],
                        [752669.0, 6414624.0],
                        [758530.0, 6415778.0],
                        [768276.0, 6405343.0],
                        [774103.0, 6411136.0],
                        [781273.0, 6415520.0],
                        [782106.0, 6419872.0],
                        [791307.0, 6420019.0],
                        [796332.0, 6429870.0],
                        [801638.0, 6433950.0],
                        [808165.0, 6446004.0],
                        [808187.0, 6450395.0],
                        [813436.0, 6450097.0],
                        [816370.0, 6460772.0],
                        [811227.0, 6465007.0],
                        [806856.0, 6463705.0],
                        [807029.0, 6471772.0],
                        [802398.0, 6475469.0],
                        [792166.0, 6477709.0],
                        [780259.0, 6472021.0],
                        [776677.0, 6475557.0],
                        [770263.0, 6473476.0],
                        [763248.0, 6477235.0],
                        [760473.0, 6472496.0],
                        [752222.0, 6475420.0],
                        [749018.0, 6471190.0],
                        [739202.0, 6481030.0],
                        [735734.0, 6477978.0],
                        [722742.0, 6478087.0],
                        [713160.0, 6472555.0],
                        [708104.0, 6472781.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "43", "NOM_DEPT": "HAUTE-LOIRE"},
        },
        46: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    732040.0999999946,
                    6824301.40000104,
                    848914.8999999773,
                    6923412.200000763,
                ),
                "coordinates": [
                    [
                        [776081.0, 6923412.0],
                        [775242.0, 6918312.0],
                        [767112.0, 6923360.0],
                        [761575.0, 6918670.0],
                        [747051.0, 6913033.0],
                        [751361.0, 6898188.0],
                        [745398.0, 6894771.0],
                        [743801.0, 6891376.0],
                        [747254.0, 6882494.0],
                        [744067.0, 6871735.0],
                        [738742.0, 6868146.0],
                        [735603.0, 6861428.0],
                        [732040.0, 6850473.0],
                        [735102.0, 6844463.0],
                        [733915.0, 6839303.0],
                        [740952.0, 6835710.0],
                        [746802.0, 6826973.0],
                        [753955.0, 6826609.0],
                        [760761.0, 6824301.0],
                        [769985.0, 6834113.0],
                        [773768.0, 6840880.0],
                        [779497.0, 6845091.0],
                        [785824.0, 6845891.0],
                        [797551.0, 6845278.0],
                        [796995.0, 6835851.0],
                        [803526.0, 6829924.0],
                        [814106.0, 6826118.0],
                        [819107.0, 6829393.0],
                        [823302.0, 6827043.0],
                        [832829.0, 6827018.0],
                        [830469.0, 6833515.0],
                        [836281.0, 6837534.0],
                        [835676.0, 6840415.0],
                        [840675.0, 6844876.0],
                        [846365.0, 6844539.0],
                        [846329.0, 6850924.0],
                        [838677.0, 6859119.0],
                        [841874.0, 6870868.0],
                        [848655.0, 6874905.0],
                        [848915.0, 6882413.0],
                        [846791.0, 6883540.0],
                        [845429.0, 6892400.0],
                        [841416.0, 6900107.0],
                        [842092.0, 6905887.0],
                        [837898.0, 6908256.0],
                        [833062.0, 6905548.0],
                        [822754.0, 6907655.0],
                        [818364.0, 6905215.0],
                        [814643.0, 6911926.0],
                        [803525.0, 6909934.0],
                        [795010.0, 6915033.0],
                        [790665.0, 6921022.0],
                        [782994.0, 6923346.0],
                        [776081.0, 6923412.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "51", "NOM_DEPT": "MARNE"},
        },
        47: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    428150.0000001159,
                    6178840.6000028085,
                    506455.20000008016,
                    6283050.700002572,
                ),
                "coordinates": [
                    [
                        [428954.0, 6200122.0],
                        [442286.0, 6190571.0],
                        [445343.0, 6185147.0],
                        [453012.0, 6180794.0],
                        [468501.0, 6186020.0],
                        [475317.0, 6183479.0],
                        [477861.0, 6178841.0],
                        [483521.0, 6184043.0],
                        [489111.0, 6180226.0],
                        [493119.0, 6181125.0],
                        [491362.0, 6184305.0],
                        [492032.0, 6195420.0],
                        [493715.0, 6200925.0],
                        [501753.0, 6199863.0],
                        [506394.0, 6214104.0],
                        [501199.0, 6216194.0],
                        [498958.0, 6224762.0],
                        [491741.0, 6229153.0],
                        [500952.0, 6237523.0],
                        [501020.0, 6240519.0],
                        [506455.0, 6246240.0],
                        [505760.0, 6248691.0],
                        [496988.0, 6250664.0],
                        [479309.0, 6257879.0],
                        [471536.0, 6256527.0],
                        [468215.0, 6263513.0],
                        [466465.0, 6272970.0],
                        [461941.0, 6273297.0],
                        [457450.0, 6278664.0],
                        [456497.0, 6283051.0],
                        [449863.0, 6280833.0],
                        [454138.0, 6272903.0],
                        [451477.0, 6264635.0],
                        [457777.0, 6262698.0],
                        [452746.0, 6249402.0],
                        [454979.0, 6245934.0],
                        [446410.0, 6236240.0],
                        [443868.0, 6230545.0],
                        [440142.0, 6228799.0],
                        [437107.0, 6220297.0],
                        [431385.0, 6215423.0],
                        [431947.0, 6209367.0],
                        [428150.0, 6207547.0],
                        [428954.0, 6200122.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "65", "NOM_DEPT": "HAUTES-PYRENEES"},
        },
        48: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    541672.6000000585,
                    6348642.700002397,
                    637121.800000023,
                    6439627.500002151,
                ),
                "coordinates": [
                    [
                        [547193.0, 6388334.0],
                        [542159.0, 6383881.0],
                        [541673.0, 6377661.0],
                        [545774.0, 6366277.0],
                        [549131.0, 6360516.0],
                        [562737.0, 6350024.0],
                        [570617.0, 6348643.0],
                        [581893.0, 6352690.0],
                        [585843.0, 6356520.0],
                        [592401.0, 6355117.0],
                        [596271.0, 6358053.0],
                        [610874.0, 6360746.0],
                        [607709.0, 6375883.0],
                        [615061.0, 6379167.0],
                        [625170.0, 6387538.0],
                        [632813.0, 6386229.0],
                        [637122.0, 6391031.0],
                        [634129.0, 6393565.0],
                        [633003.0, 6406356.0],
                        [634482.0, 6410441.0],
                        [628463.0, 6419614.0],
                        [626129.0, 6431248.0],
                        [613933.0, 6431618.0],
                        [607209.0, 6426078.0],
                        [601475.0, 6429196.0],
                        [593751.0, 6437105.0],
                        [584718.0, 6439628.0],
                        [577774.0, 6436765.0],
                        [574976.0, 6434607.0],
                        [576964.0, 6421036.0],
                        [570522.0, 6417080.0],
                        [570668.0, 6413830.0],
                        [564402.0, 6403210.0],
                        [559319.0, 6399932.0],
                        [553113.0, 6398528.0],
                        [553597.0, 6395079.0],
                        [547193.0, 6388334.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "46", "NOM_DEPT": "LOT"},
        },
        49: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    899977.8999999232,
                    6293054.100002545,
                    1014122.199999877,
                    6403290.200002269,
                ),
                "coordinates": [
                    [
                        [992638.0, 6305621.0],
                        [997097.0, 6315251.0],
                        [1000978.0, 6315262.0],
                        [1007566.0, 6320724.0],
                        [1012050.0, 6317690.0],
                        [1014122.0, 6322114.0],
                        [1009068.0, 6324850.0],
                        [1007601.0, 6328762.0],
                        [1000089.0, 6334256.0],
                        [1000668.0, 6338218.0],
                        [994698.0, 6348019.0],
                        [997268.0, 6357103.0],
                        [1002665.0, 6364730.0],
                        [1009696.0, 6370071.0],
                        [1008626.0, 6382943.0],
                        [1006125.0, 6388590.0],
                        [1012175.0, 6394096.0],
                        [1012913.0, 6402904.0],
                        [1010207.0, 6403290.0],
                        [1004568.0, 6397351.0],
                        [999315.0, 6394909.0],
                        [992592.0, 6389297.0],
                        [988952.0, 6378656.0],
                        [983401.0, 6378221.0],
                        [971476.0, 6380340.0],
                        [967103.0, 6386021.0],
                        [957182.0, 6379131.0],
                        [959705.0, 6373500.0],
                        [957118.0, 6369997.0],
                        [950982.0, 6378672.0],
                        [944316.0, 6377139.0],
                        [935206.0, 6370624.0],
                        [932421.0, 6356788.0],
                        [925339.0, 6357350.0],
                        [930231.0, 6349978.0],
                        [920148.0, 6349530.0],
                        [913873.0, 6347227.0],
                        [911050.0, 6344481.0],
                        [905908.0, 6346585.0],
                        [903520.0, 6340766.0],
                        [899978.0, 6338355.0],
                        [900470.0, 6332559.0],
                        [903879.0, 6328807.0],
                        [901642.0, 6319541.0],
                        [909386.0, 6316209.0],
                        [907367.0, 6306820.0],
                        [913516.0, 6306489.0],
                        [922160.0, 6296144.0],
                        [923991.0, 6299127.0],
                        [933734.0, 6296912.0],
                        [944763.0, 6293054.0],
                        [956283.0, 6301072.0],
                        [958836.0, 6305328.0],
                        [970413.0, 6298787.0],
                        [974848.0, 6301646.0],
                        [974774.0, 6305117.0],
                        [983574.0, 6307147.0],
                        [992638.0, 6305621.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "04", "NOM_DEPT": "ALPES-DE-HAUTE-PROVENCE"},
        },
        50: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    442127.900000055,
                    6722018.100001343,
                    542578.400000031,
                    6823412.100001052,
                ),
                "coordinates": [
                    [
                        [521086.0, 6735350.0],
                        [518605.0, 6737439.0],
                        [528757.0, 6745774.0],
                        [533549.0, 6751156.0],
                        [532634.0, 6757689.0],
                        [539183.0, 6762306.0],
                        [535875.0, 6773086.0],
                        [539332.0, 6780273.0],
                        [542578.0, 6786711.0],
                        [536374.0, 6790515.0],
                        [531913.0, 6790043.0],
                        [528051.0, 6796772.0],
                        [517130.0, 6797243.0],
                        [513744.0, 6803794.0],
                        [506061.0, 6806938.0],
                        [505117.0, 6820071.0],
                        [500193.0, 6823412.0],
                        [486746.0, 6818925.0],
                        [482552.0, 6813147.0],
                        [477803.0, 6815005.0],
                        [473893.0, 6813452.0],
                        [467628.0, 6811401.0],
                        [465298.0, 6799724.0],
                        [466280.0, 6794064.0],
                        [458409.0, 6789055.0],
                        [460615.0, 6783387.0],
                        [459373.0, 6778102.0],
                        [451057.0, 6776681.0],
                        [455060.0, 6767070.0],
                        [447308.0, 6764356.0],
                        [448124.0, 6758669.0],
                        [442128.0, 6753611.0],
                        [446459.0, 6750432.0],
                        [446732.0, 6745443.0],
                        [450112.0, 6740918.0],
                        [457617.0, 6739093.0],
                        [460339.0, 6732350.0],
                        [464631.0, 6730466.0],
                        [469484.0, 6733109.0],
                        [474490.0, 6731769.0],
                        [478533.0, 6727039.0],
                        [485937.0, 6723990.0],
                        [491931.0, 6726763.0],
                        [502971.0, 6722018.0],
                        [509057.0, 6729569.0],
                        [515591.0, 6731294.0],
                        [521086.0, 6735350.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "72", "NOM_DEPT": "SARTHE"},
        },
        51: {
            "geometry": {
                "type": "MultiPolygon",
                "coordinates": [
                    [
                        [
                            [229520.0, 6710085.0],
                            [240383.0, 6704696.0],
                            [240163.0, 6708285.0],
                            [235835.0, 6713741.0],
                            [229006.0, 6716339.0],
                            [229520.0, 6710085.0],
                        ]
                    ],
                    [
                        [
                            [212687.0, 6770001.0],
                            [211559.0, 6762660.0],
                            [216528.0, 6752538.0],
                            [224759.0, 6753321.0],
                            [234540.0, 6747533.0],
                            [234220.0, 6745025.0],
                            [240082.0, 6736634.0],
                            [251238.0, 6736509.0],
                            [260889.0, 6740570.0],
                            [266327.0, 6740184.0],
                            [271833.0, 6736526.0],
                            [269308.0, 6731426.0],
                            [263161.0, 6733044.0],
                            [258643.0, 6731063.0],
                            [263631.0, 6725691.0],
                            [274085.0, 6728424.0],
                            [283689.0, 6728526.0],
                            [290663.0, 6724762.0],
                            [288997.0, 6719519.0],
                            [300067.0, 6720583.0],
                            [315995.0, 6726944.0],
                            [317487.0, 6738014.0],
                            [315562.0, 6748257.0],
                            [318803.0, 6749864.0],
                            [322365.0, 6758359.0],
                            [317839.0, 6765781.0],
                            [320890.0, 6769878.0],
                            [317014.0, 6776765.0],
                            [309516.0, 6779524.0],
                            [306739.0, 6782705.0],
                            [310549.0, 6787938.0],
                            [306966.0, 6794650.0],
                            [300637.0, 6793781.0],
                            [298204.0, 6799128.0],
                            [291749.0, 6798375.0],
                            [285132.0, 6789099.0],
                            [280665.0, 6785525.0],
                            [277875.0, 6787107.0],
                            [279762.0, 6794931.0],
                            [270150.0, 6795768.0],
                            [252150.0, 6805897.0],
                            [246074.0, 6807146.0],
                            [242780.0, 6802066.0],
                            [233945.0, 6800703.0],
                            [229439.0, 6804532.0],
                            [212675.0, 6807535.0],
                            [202658.0, 6804638.0],
                            [202166.0, 6798131.0],
                            [205220.0, 6785800.0],
                            [211608.0, 6786141.0],
                            [216607.0, 6782805.0],
                            [223352.0, 6780867.0],
                            [221788.0, 6771592.0],
                            [212687.0, 6770001.0],
                        ]
                    ],
                ],
            },
            "attributes": {"CODE_DEPT": "56", "NOM_DEPT": "MORBIHAN"},
        },
        52: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    534094.1000000313,
                    6762847.800001221,
                    625387.4000000142,
                    6872272.800000905,
                ),
                "coordinates": [
                    [
                        [539112.0, 6843338.0],
                        [541484.0, 6833537.0],
                        [549859.0, 6826656.0],
                        [547706.0, 6812952.0],
                        [543224.0, 6808364.0],
                        [535921.0, 6806758.0],
                        [534094.0, 6803079.0],
                        [538862.0, 6792254.0],
                        [536374.0, 6790515.0],
                        [542578.0, 6786711.0],
                        [539332.0, 6780273.0],
                        [551017.0, 6782681.0],
                        [551727.0, 6777731.0],
                        [559439.0, 6777275.0],
                        [559999.0, 6772636.0],
                        [566408.0, 6764638.0],
                        [573758.0, 6762848.0],
                        [589601.0, 6765689.0],
                        [589165.0, 6770859.0],
                        [595216.0, 6770968.0],
                        [597558.0, 6774986.0],
                        [606511.0, 6774717.0],
                        [615539.0, 6776549.0],
                        [623220.0, 6795517.0],
                        [625387.0, 6798948.0],
                        [624251.0, 6811501.0],
                        [620852.0, 6812073.0],
                        [620311.0, 6818025.0],
                        [611402.0, 6819097.0],
                        [608912.0, 6830618.0],
                        [604791.0, 6831646.0],
                        [597086.0, 6841311.0],
                        [595472.0, 6845570.0],
                        [596554.0, 6858123.0],
                        [594217.0, 6866696.0],
                        [590236.0, 6872273.0],
                        [587701.0, 6866890.0],
                        [579784.0, 6860594.0],
                        [580772.0, 6855857.0],
                        [576482.0, 6852521.0],
                        [566604.0, 6854037.0],
                        [555241.0, 6849633.0],
                        [543803.0, 6848251.0],
                        [539112.0, 6843338.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "28", "NOM_DEPT": "EURE-ET-LOIR"},
        },
        53: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    1159341.799999797,
                    6050135.700003077,
                    1233465.899999756,
                    6158598.900002873,
                ),
                "coordinates": [
                    [
                        [1232225.0, 6105798.0],
                        [1226359.0, 6102880.0],
                        [1217754.0, 6104291.0],
                        [1215715.0, 6123336.0],
                        [1210743.0, 6122769.0],
                        [1206639.0, 6131576.0],
                        [1202519.0, 6133762.0],
                        [1199675.0, 6142105.0],
                        [1187662.0, 6146432.0],
                        [1180116.0, 6152978.0],
                        [1164950.0, 6158109.0],
                        [1159342.0, 6158599.0],
                        [1169805.0, 6146197.0],
                        [1163901.0, 6144874.0],
                        [1160274.0, 6141810.0],
                        [1162433.0, 6134554.0],
                        [1168722.0, 6128275.0],
                        [1172897.0, 6127710.0],
                        [1175910.0, 6121801.0],
                        [1169135.0, 6117789.0],
                        [1170381.0, 6114875.0],
                        [1164715.0, 6111213.0],
                        [1168686.0, 6106658.0],
                        [1175324.0, 6107066.0],
                        [1177911.0, 6109856.0],
                        [1182246.0, 6105577.0],
                        [1180256.0, 6096485.0],
                        [1173562.0, 6088828.0],
                        [1176478.0, 6086334.0],
                        [1193164.0, 6083876.0],
                        [1190373.0, 6079505.0],
                        [1183548.0, 6076443.0],
                        [1183755.0, 6068439.0],
                        [1188281.0, 6067524.0],
                        [1196826.0, 6061810.0],
                        [1209242.0, 6057430.0],
                        [1210610.0, 6054235.0],
                        [1221376.0, 6050136.0],
                        [1221646.0, 6056536.0],
                        [1224377.0, 6061299.0],
                        [1225618.0, 6068574.0],
                        [1229809.0, 6072152.0],
                        [1228158.0, 6078400.0],
                        [1232811.0, 6083356.0],
                        [1233466.0, 6095728.0],
                        [1232225.0, 6105798.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "2A", "NOM_DEPT": "CORSE-DU-SUD"},
        },
        54: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    878534.0999999767,
                    6809622.900001093,
                    1004384.8999999457,
                    6942685.000000716,
                ),
                "coordinates": [
                    [
                        [879041.0, 6935896.0],
                        [878534.0, 6930925.0],
                        [883576.0, 6926040.0],
                        [891117.0, 6929633.0],
                        [895087.0, 6927281.0],
                        [898652.0, 6915853.0],
                        [898091.0, 6905220.0],
                        [901547.0, 6903348.0],
                        [900172.0, 6898682.0],
                        [901739.0, 6893315.0],
                        [905859.0, 6891045.0],
                        [906433.0, 6884047.0],
                        [905066.0, 6875501.0],
                        [901302.0, 6871865.0],
                        [904125.0, 6862116.0],
                        [899528.0, 6852809.0],
                        [902927.0, 6847494.0],
                        [903561.0, 6837915.0],
                        [900379.0, 6832469.0],
                        [904895.0, 6830283.0],
                        [904250.0, 6825381.0],
                        [910921.0, 6826770.0],
                        [919362.0, 6809623.0],
                        [930831.0, 6810526.0],
                        [935203.0, 6815546.0],
                        [963830.0, 6819154.0],
                        [964951.0, 6823820.0],
                        [973656.0, 6818945.0],
                        [982282.0, 6817771.0],
                        [987769.0, 6820181.0],
                        [994239.0, 6827289.0],
                        [1004385.0, 6831654.0],
                        [1001023.0, 6834020.0],
                        [993487.0, 6842033.0],
                        [983189.0, 6843093.0],
                        [976380.0, 6847597.0],
                        [971773.0, 6847764.0],
                        [969100.0, 6851558.0],
                        [964692.0, 6852206.0],
                        [961206.0, 6856322.0],
                        [946451.0, 6858744.0],
                        [945024.0, 6864515.0],
                        [939656.0, 6875317.0],
                        [929417.0, 6875078.0],
                        [928151.0, 6878713.0],
                        [922735.0, 6879575.0],
                        [922708.0, 6883240.0],
                        [914442.0, 6889764.0],
                        [914011.0, 6893894.0],
                        [920671.0, 6900380.0],
                        [917676.0, 6905219.0],
                        [920628.0, 6907653.0],
                        [912688.0, 6923499.0],
                        [913390.0, 6932141.0],
                        [909643.0, 6936903.0],
                        [902225.0, 6942685.0],
                        [896580.0, 6941163.0],
                        [892512.0, 6942531.0],
                        [879041.0, 6935896.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "54", "NOM_DEPT": "MEURTHE-ET-MOSELLE"},
        },
        55: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    414934.20000005106,
                    6790042.700001144,
                    549859.0000000268,
                    6877306.500000899,
                ),
                "coordinates": [
                    [
                        [510545.0, 6875400.0],
                        [504730.0, 6874732.0],
                        [500558.0, 6877307.0],
                        [491678.0, 6873174.0],
                        [489675.0, 6876299.0],
                        [484257.0, 6870962.0],
                        [475607.0, 6865852.0],
                        [469104.0, 6863978.0],
                        [457511.0, 6867107.0],
                        [454384.0, 6863198.0],
                        [449827.0, 6868751.0],
                        [446005.0, 6869054.0],
                        [427244.0, 6861949.0],
                        [417723.0, 6857101.0],
                        [420581.0, 6852284.0],
                        [423498.0, 6841841.0],
                        [421619.0, 6835816.0],
                        [414934.0, 6829326.0],
                        [422197.0, 6821752.0],
                        [429868.0, 6822252.0],
                        [437568.0, 6825109.0],
                        [441174.0, 6828584.0],
                        [446687.0, 6829012.0],
                        [451256.0, 6826715.0],
                        [457920.0, 6827997.0],
                        [463434.0, 6833996.0],
                        [466087.0, 6830999.0],
                        [467262.0, 6822174.0],
                        [474394.0, 6821359.0],
                        [473893.0, 6813452.0],
                        [477803.0, 6815005.0],
                        [482552.0, 6813147.0],
                        [486746.0, 6818925.0],
                        [500193.0, 6823412.0],
                        [505117.0, 6820071.0],
                        [506061.0, 6806938.0],
                        [513744.0, 6803794.0],
                        [517130.0, 6797243.0],
                        [528051.0, 6796772.0],
                        [531913.0, 6790043.0],
                        [536374.0, 6790515.0],
                        [538862.0, 6792254.0],
                        [534094.0, 6803079.0],
                        [535921.0, 6806758.0],
                        [543224.0, 6808364.0],
                        [547706.0, 6812952.0],
                        [549859.0, 6826656.0],
                        [541484.0, 6833537.0],
                        [539112.0, 6843338.0],
                        [533024.0, 6847407.0],
                        [536389.0, 6850906.0],
                        [533488.0, 6856087.0],
                        [524600.0, 6861683.0],
                        [520512.0, 6866728.0],
                        [513552.0, 6867418.0],
                        [508448.0, 6870995.0],
                        [510545.0, 6875400.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "61", "NOM_DEPT": "ORNE"},
        },
        56: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    833932.999999959,
                    6504924.000001975,
                    943609.5999999304,
                    6603737.100001686,
                ),
                "coordinates": [
                    [
                        [877344.0, 6596683.0],
                        [869949.0, 6598870.0],
                        [868758.0, 6603264.0],
                        [857422.0, 6600366.0],
                        [848718.0, 6603737.0],
                        [845229.0, 6595067.0],
                        [845050.0, 6590960.0],
                        [839484.0, 6574938.0],
                        [837322.0, 6565642.0],
                        [838741.0, 6561678.0],
                        [835220.0, 6556428.0],
                        [834501.0, 6551198.0],
                        [835850.0, 6543054.0],
                        [833933.0, 6540287.0],
                        [844536.0, 6535588.0],
                        [849404.0, 6524528.0],
                        [863140.0, 6525920.0],
                        [870291.0, 6521483.0],
                        [876089.0, 6523611.0],
                        [879159.0, 6530984.0],
                        [882529.0, 6534232.0],
                        [888996.0, 6528631.0],
                        [888123.0, 6525941.0],
                        [897998.0, 6515860.0],
                        [904449.0, 6504924.0],
                        [909413.0, 6509006.0],
                        [910338.0, 6515974.0],
                        [915868.0, 6518199.0],
                        [916332.0, 6528631.0],
                        [919315.0, 6541572.0],
                        [917441.0, 6547210.0],
                        [916992.0, 6555432.0],
                        [918731.0, 6560168.0],
                        [928176.0, 6563414.0],
                        [933670.0, 6575392.0],
                        [940629.0, 6577110.0],
                        [938817.0, 6580823.0],
                        [943610.0, 6590224.0],
                        [935280.0, 6595262.0],
                        [929573.0, 6589336.0],
                        [923919.0, 6580132.0],
                        [919460.0, 6577537.0],
                        [909869.0, 6577050.0],
                        [908880.0, 6582354.0],
                        [903737.0, 6585593.0],
                        [895736.0, 6577631.0],
                        [890444.0, 6576892.0],
                        [887558.0, 6582363.0],
                        [882486.0, 6586333.0],
                        [882664.0, 6589574.0],
                        [877221.0, 6593352.0],
                        [877344.0, 6596683.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "01", "NOM_DEPT": "AIN"},
        },
        57: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    522873.2000000755,
                    6164532.000002834,
                    632373.2000000293,
                    6247681.500002648,
                ),
                "coordinates": [
                    [
                        [524704.0, 6194181.0],
                        [530246.0, 6189977.0],
                        [542336.0, 6189236.0],
                        [549247.0, 6180805.0],
                        [554894.0, 6182516.0],
                        [562658.0, 6181937.0],
                        [567864.0, 6177510.0],
                        [571419.0, 6168649.0],
                        [580902.0, 6173997.0],
                        [585134.0, 6170835.0],
                        [592180.0, 6170254.0],
                        [600206.0, 6164532.0],
                        [607482.0, 6165290.0],
                        [615529.0, 6169131.0],
                        [617954.0, 6174066.0],
                        [631545.0, 6174170.0],
                        [632373.0, 6176261.0],
                        [622733.0, 6184536.0],
                        [614361.0, 6182470.0],
                        [610538.0, 6185966.0],
                        [608402.0, 6195450.0],
                        [617137.0, 6197372.0],
                        [617210.0, 6214637.0],
                        [613663.0, 6217624.0],
                        [614442.0, 6225039.0],
                        [604330.0, 6228108.0],
                        [595072.0, 6233132.0],
                        [596256.0, 6236985.0],
                        [593467.0, 6242482.0],
                        [589272.0, 6240503.0],
                        [582028.0, 6242918.0],
                        [575081.0, 6235814.0],
                        [572065.0, 6240985.0],
                        [565565.0, 6247682.0],
                        [561407.0, 6244405.0],
                        [567825.0, 6236309.0],
                        [562910.0, 6233771.0],
                        [555809.0, 6233248.0],
                        [554118.0, 6225953.0],
                        [547328.0, 6230443.0],
                        [541984.0, 6228895.0],
                        [540236.0, 6224348.0],
                        [535770.0, 6220735.0],
                        [537386.0, 6214080.0],
                        [535012.0, 6210452.0],
                        [526458.0, 6208824.0],
                        [522873.0, 6204718.0],
                        [524704.0, 6194181.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "09", "NOM_DEPT": "ARIEGE"},
        },
        58: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    560988.4000000468,
                    6426078.100002187,
                    663287.6000000116,
                    6517796.40000193,
                ),
                "coordinates": [
                    [
                        [563452.0, 6484264.0],
                        [563844.0, 6479160.0],
                        [568750.0, 6477311.0],
                        [560988.0, 6465186.0],
                        [564865.0, 6461779.0],
                        [566113.0, 6450433.0],
                        [573095.0, 6446142.0],
                        [577774.0, 6436765.0],
                        [584718.0, 6439628.0],
                        [593751.0, 6437105.0],
                        [601475.0, 6429196.0],
                        [607209.0, 6426078.0],
                        [613933.0, 6431618.0],
                        [626129.0, 6431248.0],
                        [631663.0, 6432186.0],
                        [628772.0, 6440050.0],
                        [634831.0, 6442812.0],
                        [638218.0, 6451518.0],
                        [636846.0, 6458269.0],
                        [642886.0, 6465907.0],
                        [649118.0, 6470007.0],
                        [651369.0, 6479623.0],
                        [656321.0, 6476249.0],
                        [659915.0, 6480065.0],
                        [661594.0, 6486680.0],
                        [662264.0, 6494997.0],
                        [658322.0, 6500286.0],
                        [662164.0, 6504560.0],
                        [663288.0, 6509261.0],
                        [660507.0, 6515469.0],
                        [656101.0, 6511235.0],
                        [649366.0, 6512420.0],
                        [648450.0, 6509221.0],
                        [637278.0, 6511373.0],
                        [635791.0, 6515072.0],
                        [631053.0, 6514297.0],
                        [623634.0, 6517796.0],
                        [614305.0, 6511565.0],
                        [612121.0, 6507862.0],
                        [605337.0, 6509707.0],
                        [593766.0, 6498788.0],
                        [587429.0, 6495459.0],
                        [581471.0, 6497412.0],
                        [576059.0, 6493313.0],
                        [571080.0, 6486618.0],
                        [566212.0, 6489282.0],
                        [563452.0, 6484264.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "19", "NOM_DEPT": "CORREZE"},
        },
        59: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    987690.6999999384,
                    6711033.600001389,
                    1044907.7999999225,
                    6809400.400001105,
                ),
                "coordinates": [
                    [
                        [987691.0, 6753958.0],
                        [989189.0, 6749828.0],
                        [1000830.0, 6745745.0],
                        [1003510.0, 6737902.0],
                        [1000706.0, 6731961.0],
                        [1006958.0, 6729298.0],
                        [1010786.0, 6719537.0],
                        [1014326.0, 6713511.0],
                        [1019985.0, 6711034.0],
                        [1024064.0, 6712992.0],
                        [1031730.0, 6713160.0],
                        [1039126.0, 6720266.0],
                        [1038581.0, 6725392.0],
                        [1044459.0, 6729657.0],
                        [1044908.0, 6732411.0],
                        [1039238.0, 6738821.0],
                        [1038337.0, 6743276.0],
                        [1040672.0, 6747502.0],
                        [1039113.0, 6752662.0],
                        [1040418.0, 6763108.0],
                        [1044697.0, 6773634.0],
                        [1040564.0, 6779861.0],
                        [1040408.0, 6789869.0],
                        [1036067.0, 6790258.0],
                        [1032222.0, 6793924.0],
                        [1032428.0, 6798669.0],
                        [1022638.0, 6802981.0],
                        [1016850.0, 6809125.0],
                        [1011125.0, 6809400.0],
                        [1006708.0, 6801985.0],
                        [1001795.0, 6789636.0],
                        [998205.0, 6776671.0],
                        [993983.0, 6773739.0],
                        [991355.0, 6761730.0],
                        [993060.0, 6757264.0],
                        [987691.0, 6753958.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "68", "NOM_DEPT": "HAUT-RHIN"},
        },
        60: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    476861.20000005513,
                    6629764.700001611,
                    576079.4000000309,
                    6735350.200001305,
                ),
                "coordinates": [
                    [
                        [573297.0, 6677558.0],
                        [569696.0, 6685244.0],
                        [565759.0, 6689706.0],
                        [561058.0, 6687382.0],
                        [557051.0, 6690388.0],
                        [558309.0, 6696538.0],
                        [557303.0, 6702930.0],
                        [559271.0, 6706788.0],
                        [552945.0, 6716389.0],
                        [555366.0, 6719856.0],
                        [552256.0, 6724799.0],
                        [543743.0, 6727862.0],
                        [539386.0, 6732046.0],
                        [530153.0, 6735215.0],
                        [521086.0, 6735350.0],
                        [515591.0, 6731294.0],
                        [509057.0, 6729569.0],
                        [502971.0, 6722018.0],
                        [491931.0, 6726763.0],
                        [489539.0, 6719725.0],
                        [490772.0, 6714971.0],
                        [487846.0, 6706491.0],
                        [479254.0, 6691029.0],
                        [476861.0, 6677880.0],
                        [478404.0, 6673399.0],
                        [482922.0, 6672972.0],
                        [485608.0, 6667284.0],
                        [494947.0, 6665028.0],
                        [494833.0, 6656131.0],
                        [505157.0, 6650870.0],
                        [510389.0, 6653999.0],
                        [517389.0, 6653506.0],
                        [515158.0, 6658631.0],
                        [524608.0, 6655240.0],
                        [525272.0, 6647330.0],
                        [531658.0, 6641492.0],
                        [533123.0, 6635020.0],
                        [537221.0, 6629765.0],
                        [546519.0, 6631001.0],
                        [551779.0, 6655211.0],
                        [553982.0, 6659068.0],
                        [560408.0, 6661557.0],
                        [568878.0, 6661167.0],
                        [571950.0, 6667671.0],
                        [576079.0, 6670370.0],
                        [573297.0, 6677558.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "37", "NOM_DEPT": "INDRE-ET-LOIRE"},
        },
        61: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    634421.5000000035,
                    6986161.900000591,
                    788005.7999999913,
                    7110430.10000027,
                ),
                "coordinates": [
                    [
                        [634422.0, 7101577.0],
                        [636514.0, 7100652.0],
                        [642453.0, 7087769.0],
                        [644454.0, 7079622.0],
                        [653390.0, 7071778.0],
                        [658064.0, 7064147.0],
                        [664277.0, 7059477.0],
                        [673807.0, 7057943.0],
                        [679720.0, 7059405.0],
                        [683642.0, 7063001.0],
                        [688698.0, 7062196.0],
                        [686184.0, 7056263.0],
                        [687641.0, 7047582.0],
                        [692820.0, 7044918.0],
                        [697493.0, 7046164.0],
                        [701478.0, 7042709.0],
                        [702680.0, 7038239.0],
                        [699507.0, 7031355.0],
                        [712383.0, 7015916.0],
                        [711164.0, 7009522.0],
                        [706855.0, 7002888.0],
                        [706470.0, 6995042.0],
                        [712391.0, 6990404.0],
                        [716534.0, 6992565.0],
                        [720100.0, 6990781.0],
                        [725313.0, 6993104.0],
                        [735248.0, 6991264.0],
                        [739058.0, 6995179.0],
                        [744014.0, 6992056.0],
                        [751253.0, 6997000.0],
                        [753505.0, 6995276.0],
                        [766237.0, 6992385.0],
                        [770359.0, 6988977.0],
                        [778060.0, 6986162.0],
                        [781905.0, 6987282.0],
                        [783321.0, 6995122.0],
                        [788006.0, 6997129.0],
                        [785882.0, 7004302.0],
                        [780654.0, 7004663.0],
                        [782919.0, 7012216.0],
                        [787237.0, 7018342.0],
                        [781127.0, 7020183.0],
                        [773958.0, 7027731.0],
                        [768979.0, 7028494.0],
                        [763439.0, 7026158.0],
                        [760772.0, 7028536.0],
                        [753265.0, 7028386.0],
                        [749524.0, 7024442.0],
                        [746894.0, 7030603.0],
                        [747241.0, 7039722.0],
                        [743565.0, 7044070.0],
                        [733656.0, 7048580.0],
                        [726572.0, 7043892.0],
                        [720348.0, 7047849.0],
                        [717610.0, 7060148.0],
                        [718286.0, 7067127.0],
                        [713527.0, 7069761.0],
                        [710740.0, 7076212.0],
                        [704337.0, 7075998.0],
                        [695621.0, 7071968.0],
                        [692807.0, 7066387.0],
                        [686789.0, 7068929.0],
                        [680050.0, 7079722.0],
                        [674224.0, 7079651.0],
                        [671748.0, 7084186.0],
                        [674131.0, 7094505.0],
                        [670178.0, 7102078.0],
                        [667909.0, 7110430.0],
                        [659576.0, 7106814.0],
                        [654175.0, 7107325.0],
                        [637372.0, 7101244.0],
                        [634422.0, 7101577.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "59", "NOM_DEPT": "NORD"},
        },
        62: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    981470.1999999387,
                    6711078.800001388,
                    1010785.5999999286,
                    6753957.700001263,
                ),
                "coordinates": [
                    [
                        [1010786.0, 6719537.0],
                        [1006958.0, 6729298.0],
                        [1000706.0, 6731961.0],
                        [1003510.0, 6737902.0],
                        [1000830.0, 6745745.0],
                        [989189.0, 6749828.0],
                        [987691.0, 6753958.0],
                        [986052.0, 6752778.0],
                        [981470.0, 6745308.0],
                        [983826.0, 6731782.0],
                        [986155.0, 6724947.0],
                        [993651.0, 6723596.0],
                        [994087.0, 6717743.0],
                        [996902.0, 6711079.0],
                        [1001194.0, 6713400.0],
                        [999772.0, 6717889.0],
                        [1007028.0, 6717766.0],
                        [1010786.0, 6719537.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "90", "NOM_DEPT": "TERRITOIRE DE BELFORT"},
        },
        63: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    282805.7000000984,
                    6649967.800001572,
                    402393.3000000701,
                    6756672.500001258,
                ),
                "coordinates": [
                    [
                        [382163.0, 6750361.0],
                        [374291.0, 6753124.0],
                        [371672.0, 6756673.0],
                        [356172.0, 6751743.0],
                        [351165.0, 6744974.0],
                        [342268.0, 6743932.0],
                        [335469.0, 6745297.0],
                        [327477.0, 6743723.0],
                        [317487.0, 6738014.0],
                        [315995.0, 6726944.0],
                        [300067.0, 6720583.0],
                        [288997.0, 6719519.0],
                        [282806.0, 6712695.0],
                        [283678.0, 6703214.0],
                        [288393.0, 6698809.0],
                        [292251.0, 6700736.0],
                        [299211.0, 6695249.0],
                        [308160.0, 6699576.0],
                        [309828.0, 6691500.0],
                        [308860.0, 6686791.0],
                        [304084.0, 6683183.0],
                        [316935.0, 6678247.0],
                        [322009.0, 6670666.0],
                        [324590.0, 6666698.0],
                        [332678.0, 6659222.0],
                        [338731.0, 6658583.0],
                        [343129.0, 6653969.0],
                        [353671.0, 6649968.0],
                        [357453.0, 6652400.0],
                        [354064.0, 6663454.0],
                        [354394.0, 6667534.0],
                        [359840.0, 6669393.0],
                        [361112.0, 6656907.0],
                        [367588.0, 6659427.0],
                        [367645.0, 6668175.0],
                        [374065.0, 6668782.0],
                        [376108.0, 6673611.0],
                        [385082.0, 6667087.0],
                        [387784.0, 6670618.0],
                        [379298.0, 6676338.0],
                        [379159.0, 6679552.0],
                        [384839.0, 6683588.0],
                        [382659.0, 6691801.0],
                        [378391.0, 6691949.0],
                        [374867.0, 6698071.0],
                        [375470.0, 6701871.0],
                        [385609.0, 6704471.0],
                        [396890.0, 6703794.0],
                        [402393.0, 6705897.0],
                        [401401.0, 6717774.0],
                        [387234.0, 6720426.0],
                        [386462.0, 6727467.0],
                        [395905.0, 6725901.0],
                        [398982.0, 6728603.0],
                        [389301.0, 6732485.0],
                        [386987.0, 6740963.0],
                        [381255.0, 6745617.0],
                        [382163.0, 6750361.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "44", "NOM_DEPT": "LOIRE-ATLANTIQUE"},
        },
        64: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    689262.3000000024,
                    6692939.9000014225,
                    799826.7999999784,
                    6810077.50000108,
                ),
                "coordinates": [
                    [
                        [698236.0, 6718784.0],
                        [701928.0, 6717598.0],
                        [707666.0, 6720847.0],
                        [709181.0, 6715638.0],
                        [715400.0, 6713642.0],
                        [717731.0, 6709895.0],
                        [723118.0, 6710529.0],
                        [726085.0, 6708066.0],
                        [729904.0, 6712131.0],
                        [736733.0, 6710509.0],
                        [738638.0, 6714270.0],
                        [743712.0, 6710994.0],
                        [743701.0, 6707332.0],
                        [751168.0, 6705240.0],
                        [753904.0, 6700965.0],
                        [759265.0, 6699206.0],
                        [773240.0, 6698003.0],
                        [773484.0, 6692940.0],
                        [783517.0, 6693799.0],
                        [780393.0, 6701310.0],
                        [784360.0, 6705398.0],
                        [783952.0, 6713331.0],
                        [788376.0, 6717645.0],
                        [791050.0, 6725986.0],
                        [795163.0, 6731989.0],
                        [794312.0, 6737086.0],
                        [799827.0, 6740411.0],
                        [799053.0, 6750471.0],
                        [794318.0, 6750026.0],
                        [796594.0, 6759156.0],
                        [789480.0, 6763963.0],
                        [783028.0, 6759096.0],
                        [767353.0, 6760249.0],
                        [763413.0, 6765164.0],
                        [759917.0, 6778325.0],
                        [755033.0, 6785741.0],
                        [749678.0, 6782304.0],
                        [746161.0, 6791892.0],
                        [745765.0, 6796957.0],
                        [736893.0, 6807760.0],
                        [730707.0, 6810078.0],
                        [727042.0, 6808060.0],
                        [712422.0, 6807964.0],
                        [703668.0, 6806647.0],
                        [703237.0, 6796855.0],
                        [700383.0, 6789702.0],
                        [695265.0, 6784783.0],
                        [702192.0, 6781427.0],
                        [703759.0, 6774663.0],
                        [706580.0, 6772621.0],
                        [709585.0, 6763406.0],
                        [703743.0, 6756785.0],
                        [700769.0, 6756036.0],
                        [701782.0, 6742907.0],
                        [695135.0, 6740319.0],
                        [689262.0, 6740069.0],
                        [689391.0, 6734632.0],
                        [694231.0, 6731133.0],
                        [695341.0, 6722058.0],
                        [698236.0, 6718784.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "89", "NOM_DEPT": "YONNE"},
        },
        65: {
            "geometry": {
                "type": "MultiPolygon",
                "bbox": (
                    306739.2000000799,
                    6738013.600001321,
                    400464.5000000602,
                    6855953.00000098,
                ),
                "coordinates": [
                    [
                        [
                            [330825.0, 6841252.0],
                            [328251.0, 6849310.0],
                            [322500.0, 6846053.0],
                            [325749.0, 6841751.0],
                            [330825.0, 6841252.0],
                        ]
                    ],
                    [
                        [
                            [399495.0, 6830885.0],
                            [389634.0, 6835155.0],
                            [384742.0, 6834496.0],
                            [376536.0, 6826525.0],
                            [368413.0, 6830443.0],
                            [366767.0, 6839135.0],
                            [363309.0, 6845999.0],
                            [348952.0, 6844300.0],
                            [343084.0, 6846084.0],
                            [341341.0, 6849649.0],
                            [342875.0, 6855953.0],
                            [334395.0, 6854417.0],
                            [329772.0, 6850320.0],
                            [334969.0, 6837949.0],
                            [337450.0, 6835570.0],
                            [332923.0, 6813386.0],
                            [328273.0, 6809495.0],
                            [325505.0, 6811765.0],
                            [320973.0, 6806998.0],
                            [315234.0, 6806646.0],
                            [311879.0, 6798427.0],
                            [306966.0, 6794650.0],
                            [310549.0, 6787938.0],
                            [306739.0, 6782705.0],
                            [309516.0, 6779524.0],
                            [317014.0, 6776765.0],
                            [320890.0, 6769878.0],
                            [317839.0, 6765781.0],
                            [322365.0, 6758359.0],
                            [318803.0, 6749864.0],
                            [315562.0, 6748257.0],
                            [317487.0, 6738014.0],
                            [327477.0, 6743723.0],
                            [335469.0, 6745297.0],
                            [342268.0, 6743932.0],
                            [351165.0, 6744974.0],
                            [356172.0, 6751743.0],
                            [371672.0, 6756673.0],
                            [374291.0, 6753124.0],
                            [382163.0, 6750361.0],
                            [382932.0, 6754022.0],
                            [386941.0, 6760260.0],
                            [390140.0, 6770978.0],
                            [394099.0, 6773357.0],
                            [400197.0, 6773697.0],
                            [400465.0, 6781914.0],
                            [398626.0, 6784333.0],
                            [395852.0, 6803336.0],
                            [400321.0, 6810723.0],
                            [398130.0, 6822559.0],
                            [399495.0, 6830885.0],
                        ]
                    ],
                ],
            },
            "attributes": {"CODE_DEPT": "35", "NOM_DEPT": "ILLE-ET-VILAINE"},
        },
        66: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    334282.70000014844,
                    6275534.700002595,
                    470768.600000088,
                    6389853.100002306,
                ),
                "coordinates": [
                    [
                        [334283.0, 6280551.0],
                        [338238.0, 6281381.0],
                        [342731.0, 6276517.0],
                        [352961.0, 6276001.0],
                        [362964.0, 6280741.0],
                        [367753.0, 6276986.0],
                        [377063.0, 6275535.0],
                        [385124.0, 6280151.0],
                        [394475.0, 6281218.0],
                        [399714.0, 6278404.0],
                        [403447.0, 6281078.0],
                        [412501.0, 6278002.0],
                        [421554.0, 6283529.0],
                        [424830.0, 6280287.0],
                        [433057.0, 6279053.0],
                        [438084.0, 6281592.0],
                        [439499.0, 6295895.0],
                        [442698.0, 6298315.0],
                        [443572.0, 6313646.0],
                        [440195.0, 6315550.0],
                        [444807.0, 6320573.0],
                        [453303.0, 6321029.0],
                        [461632.0, 6315722.0],
                        [465239.0, 6317398.0],
                        [465465.0, 6324803.0],
                        [464468.0, 6326808.0],
                        [467874.0, 6336236.0],
                        [470769.0, 6340294.0],
                        [462757.0, 6341305.0],
                        [459734.0, 6343570.0],
                        [449759.0, 6344226.0],
                        [449164.0, 6352477.0],
                        [445091.0, 6357381.0],
                        [442465.0, 6350467.0],
                        [438523.0, 6349285.0],
                        [429214.0, 6351421.0],
                        [430077.0, 6359927.0],
                        [426389.0, 6364119.0],
                        [419635.0, 6366255.0],
                        [418928.0, 6369127.0],
                        [411174.0, 6373203.0],
                        [411314.0, 6378206.0],
                        [403228.0, 6378937.0],
                        [393973.0, 6376334.0],
                        [388231.0, 6379327.0],
                        [381182.0, 6379048.0],
                        [383562.0, 6383958.0],
                        [375548.0, 6389853.0],
                        [373686.0, 6386677.0],
                        [361774.0, 6383396.0],
                        [355175.0, 6347840.0],
                        [348748.0, 6322487.0],
                        [342546.0, 6300215.0],
                        [341177.0, 6292691.0],
                        [334283.0, 6280551.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "40", "NOM_DEPT": "LANDES"},
        },
        67: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    124277.10000010568,
                    6761104.000001276,
                    223352.10000009876,
                    6867490.700000975,
                ),
                "coordinates": [
                    [
                        [210124.0, 6860562.0],
                        [210319.0, 6864435.0],
                        [197918.0, 6867491.0],
                        [195168.0, 6863087.0],
                        [188759.0, 6861699.0],
                        [187283.0, 6866983.0],
                        [171676.0, 6866983.0],
                        [165007.0, 6863478.0],
                        [159513.0, 6866951.0],
                        [150836.0, 6862294.0],
                        [143559.0, 6862620.0],
                        [137993.0, 6858007.0],
                        [130421.0, 6856362.0],
                        [126410.0, 6849458.0],
                        [124277.0, 6841317.0],
                        [126191.0, 6836271.0],
                        [137035.0, 6831319.0],
                        [151215.0, 6836632.0],
                        [149516.0, 6831882.0],
                        [159712.0, 6829829.0],
                        [158800.0, 6824711.0],
                        [152925.0, 6822802.0],
                        [150196.0, 6824902.0],
                        [139593.0, 6825317.0],
                        [139209.0, 6819312.0],
                        [147794.0, 6818873.0],
                        [158225.0, 6812964.0],
                        [159803.0, 6806301.0],
                        [156741.0, 6801688.0],
                        [151961.0, 6804475.0],
                        [143250.0, 6802653.0],
                        [129762.0, 6802135.0],
                        [125302.0, 6798484.0],
                        [144750.0, 6790824.0],
                        [150552.0, 6779953.0],
                        [149596.0, 6769606.0],
                        [162294.0, 6768615.0],
                        [164959.0, 6774152.0],
                        [171525.0, 6775493.0],
                        [174013.0, 6772980.0],
                        [183941.0, 6774809.0],
                        [184280.0, 6770949.0],
                        [191411.0, 6764941.0],
                        [196633.0, 6766198.0],
                        [200244.0, 6762976.0],
                        [211251.0, 6761104.0],
                        [212687.0, 6770001.0],
                        [221788.0, 6771592.0],
                        [223352.0, 6780867.0],
                        [216607.0, 6782805.0],
                        [211608.0, 6786141.0],
                        [205220.0, 6785800.0],
                        [202166.0, 6798131.0],
                        [202658.0, 6804638.0],
                        [212675.0, 6807535.0],
                        [216996.0, 6817941.0],
                        [214652.0, 6824074.0],
                        [215204.0, 6828740.0],
                        [213034.0, 6834627.0],
                        [212655.0, 6842190.0],
                        [216761.0, 6846809.0],
                        [210382.0, 6855067.0],
                        [210124.0, 6860562.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "29", "NOM_DEPT": "FINISTERE"},
        },
        68: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    916991.9999999349,
                    6514709.9000019515,
                    1013274.7999999039,
                    6596473.600001719,
                ),
                "coordinates": [
                    [
                        [995359.0, 6526975.0],
                        [996263.0, 6533312.0],
                        [1005620.0, 6535079.0],
                        [1009539.0, 6537663.0],
                        [1013275.0, 6543845.0],
                        [1010939.0, 6550631.0],
                        [1005414.0, 6557665.0],
                        [1000587.0, 6561265.0],
                        [1000913.0, 6565514.0],
                        [994423.0, 6565990.0],
                        [993170.0, 6574229.0],
                        [997429.0, 6583180.0],
                        [990846.0, 6588853.0],
                        [991547.0, 6595136.0],
                        [985884.0, 6596474.0],
                        [972312.0, 6594423.0],
                        [960663.0, 6587846.0],
                        [955674.0, 6591075.0],
                        [952150.0, 6588701.0],
                        [949250.0, 6580383.0],
                        [953939.0, 6574878.0],
                        [949136.0, 6572391.0],
                        [944956.0, 6566962.0],
                        [928176.0, 6563414.0],
                        [918731.0, 6560168.0],
                        [916992.0, 6555432.0],
                        [917441.0, 6547210.0],
                        [919315.0, 6541572.0],
                        [921730.0, 6540986.0],
                        [922988.0, 6530552.0],
                        [926213.0, 6526897.0],
                        [936560.0, 6520059.0],
                        [946066.0, 6522280.0],
                        [948121.0, 6516633.0],
                        [952091.0, 6514710.0],
                        [959117.0, 6516128.0],
                        [962862.0, 6525460.0],
                        [965836.0, 6528299.0],
                        [969073.0, 6537583.0],
                        [971940.0, 6540371.0],
                        [975690.0, 6531451.0],
                        [979688.0, 6528135.0],
                        [984284.0, 6528734.0],
                        [988618.0, 6520489.0],
                        [995359.0, 6526975.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "74", "NOM_DEPT": "HAUTE-SAVOIE"},
        },
        69: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    605400.9000000112,
                    6884542.30000087,
                    711647.6999999984,
                    6962826.200000654,
                ),
                "coordinates": [
                    [
                        [605624.0, 6904387.0],
                        [608283.0, 6898554.0],
                        [612180.0, 6899061.0],
                        [618689.0, 6896449.0],
                        [626847.0, 6897876.0],
                        [633020.0, 6901508.0],
                        [645465.0, 6895048.0],
                        [649761.0, 6898738.0],
                        [652314.0, 6895981.0],
                        [659205.0, 6894147.0],
                        [670084.0, 6886723.0],
                        [673222.0, 6889902.0],
                        [680633.0, 6884542.0],
                        [700622.0, 6887964.0],
                        [705248.0, 6890863.0],
                        [710189.0, 6894765.0],
                        [704672.0, 6899225.0],
                        [701957.0, 6908433.0],
                        [698137.0, 6911415.0],
                        [706929.0, 6919738.0],
                        [706805.0, 6926038.0],
                        [711648.0, 6928031.0],
                        [706939.0, 6934901.0],
                        [709520.0, 6938240.0],
                        [707064.0, 6950845.0],
                        [708546.0, 6956332.0],
                        [693916.0, 6955962.0],
                        [685089.0, 6950779.0],
                        [685504.0, 6946842.0],
                        [677528.0, 6947426.0],
                        [676176.0, 6942514.0],
                        [669006.0, 6944292.0],
                        [664208.0, 6949037.0],
                        [654712.0, 6950974.0],
                        [645659.0, 6956103.0],
                        [636565.0, 6954687.0],
                        [623025.0, 6958366.0],
                        [620154.0, 6956108.0],
                        [612314.0, 6962826.0],
                        [605401.0, 6955866.0],
                        [607933.0, 6953359.0],
                        [606321.0, 6945902.0],
                        [609123.0, 6937659.0],
                        [607205.0, 6935073.0],
                        [610545.0, 6930338.0],
                        [606642.0, 6924066.0],
                        [609871.0, 6919444.0],
                        [610931.0, 6911811.0],
                        [606128.0, 6907968.0],
                        [605624.0, 6904387.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "60", "NOM_DEPT": "OISE"},
        },
        70: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [598361.0, 6887345.0],
                        [603102.0, 6887292.0],
                        [606678.0, 6883543.0],
                        [614076.0, 6886919.0],
                        [622312.0, 6880731.0],
                        [628641.0, 6878089.0],
                        [633062.0, 6879807.0],
                        [641836.0, 6872490.0],
                        [641404.0, 6867928.0],
                        [648071.0, 6872567.0],
                        [653619.0, 6875101.0],
                        [660416.0, 6872923.0],
                        [667303.0, 6878971.0],
                        [670084.0, 6886723.0],
                        [659205.0, 6894147.0],
                        [652314.0, 6895981.0],
                        [649761.0, 6898738.0],
                        [645465.0, 6895048.0],
                        [633020.0, 6901508.0],
                        [626847.0, 6897876.0],
                        [618689.0, 6896449.0],
                        [612180.0, 6899061.0],
                        [608283.0, 6898554.0],
                        [605624.0, 6904387.0],
                        [603501.0, 6902160.0],
                        [601892.0, 6893098.0],
                        [598361.0, 6887345.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "95", "NOM_DEPT": "VAL-D'OISE"},
        },
        71: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    690449.1000000023,
                    6617690.400001641,
                    793165.4999999772,
                    6720847.300001339,
                ),
                "coordinates": [
                    [
                        [783517.0, 6693799.0],
                        [773484.0, 6692940.0],
                        [773240.0, 6698003.0],
                        [759265.0, 6699206.0],
                        [753904.0, 6700965.0],
                        [751168.0, 6705240.0],
                        [743701.0, 6707332.0],
                        [743712.0, 6710994.0],
                        [738638.0, 6714270.0],
                        [736733.0, 6710509.0],
                        [729904.0, 6712131.0],
                        [726085.0, 6708066.0],
                        [723118.0, 6710529.0],
                        [717731.0, 6709895.0],
                        [715400.0, 6713642.0],
                        [709181.0, 6715638.0],
                        [707666.0, 6720847.0],
                        [701928.0, 6717598.0],
                        [698236.0, 6718784.0],
                        [693556.0, 6718403.0],
                        [690565.0, 6713347.0],
                        [694642.0, 6704915.0],
                        [690449.0, 6694237.0],
                        [698745.0, 6684384.0],
                        [698614.0, 6680189.0],
                        [702129.0, 6669867.0],
                        [701762.0, 6662486.0],
                        [705753.0, 6657660.0],
                        [706033.0, 6650549.0],
                        [703828.0, 6645606.0],
                        [705186.0, 6641947.0],
                        [702446.0, 6632753.0],
                        [703745.0, 6628663.0],
                        [716475.0, 6620334.0],
                        [720608.0, 6624106.0],
                        [726499.0, 6620539.0],
                        [733164.0, 6623625.0],
                        [737251.0, 6617690.0],
                        [742042.0, 6624120.0],
                        [748049.0, 6627896.0],
                        [756196.0, 6628301.0],
                        [760920.0, 6622686.0],
                        [770503.0, 6626941.0],
                        [775685.0, 6631211.0],
                        [781064.0, 6632682.0],
                        [780490.0, 6638662.0],
                        [783308.0, 6642195.0],
                        [778932.0, 6646959.0],
                        [779000.0, 6654397.0],
                        [781374.0, 6662136.0],
                        [778810.0, 6664936.0],
                        [780471.0, 6669516.0],
                        [784602.0, 6669828.0],
                        [789554.0, 6672918.0],
                        [791663.0, 6673495.0],
                        [793165.0, 6678355.0],
                        [789098.0, 6683996.0],
                        [785899.0, 6682517.0],
                        [783517.0, 6693799.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "58", "NOM_DEPT": "NIEVRE"},
        },
        72: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    780392.7999999812,
                    6648014.700001555,
                    890264.8999999544,
                    6771079.900001195,
                ),
                "coordinates": [
                    [
                        [871408.0, 6655745.0],
                        [875449.0, 6660099.0],
                        [876191.0, 6666228.0],
                        [881002.0, 6667371.0],
                        [884880.0, 6674426.0],
                        [887407.0, 6682465.0],
                        [888032.0, 6690633.0],
                        [890265.0, 6692353.0],
                        [884919.0, 6701062.0],
                        [880595.0, 6713705.0],
                        [884206.0, 6713509.0],
                        [887785.0, 6718944.0],
                        [886108.0, 6726069.0],
                        [882147.0, 6728546.0],
                        [878352.0, 6725366.0],
                        [868188.0, 6726357.0],
                        [863148.0, 6730266.0],
                        [858300.0, 6729814.0],
                        [848455.0, 6733791.0],
                        [846031.0, 6742315.0],
                        [848388.0, 6746302.0],
                        [846087.0, 6753627.0],
                        [838226.0, 6763382.0],
                        [833316.0, 6764154.0],
                        [833398.0, 6769005.0],
                        [827043.0, 6770243.0],
                        [817962.0, 6771080.0],
                        [816397.0, 6764598.0],
                        [808147.0, 6762855.0],
                        [797692.0, 6763119.0],
                        [796594.0, 6759156.0],
                        [794318.0, 6750026.0],
                        [799053.0, 6750471.0],
                        [799827.0, 6740411.0],
                        [794312.0, 6737086.0],
                        [795163.0, 6731989.0],
                        [791050.0, 6725986.0],
                        [788376.0, 6717645.0],
                        [783952.0, 6713331.0],
                        [784360.0, 6705398.0],
                        [780393.0, 6701310.0],
                        [783517.0, 6693799.0],
                        [785899.0, 6682517.0],
                        [789098.0, 6683996.0],
                        [793165.0, 6678355.0],
                        [791663.0, 6673495.0],
                        [795506.0, 6668991.0],
                        [806631.0, 6665548.0],
                        [806754.0, 6662035.0],
                        [813434.0, 6659755.0],
                        [823470.0, 6650227.0],
                        [831500.0, 6648015.0],
                        [855755.0, 6655507.0],
                        [860161.0, 6652754.0],
                        [871408.0, 6655745.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "21", "NOM_DEPT": "COTE-D'OR"},
        },
        73: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    620310.8000000145,
                    6798947.800001114,
                    668471.5000000054,
                    6853168.000000958,
                ),
                "coordinates": [
                    [
                        [643163.0, 6853168.0],
                        [634691.0, 6849194.0],
                        [632440.0, 6843248.0],
                        [627800.0, 6838438.0],
                        [627504.0, 6829048.0],
                        [623768.0, 6826369.0],
                        [620311.0, 6818025.0],
                        [620852.0, 6812073.0],
                        [624251.0, 6811501.0],
                        [625387.0, 6798948.0],
                        [639308.0, 6801806.0],
                        [653275.0, 6801114.0],
                        [655721.0, 6802434.0],
                        [657768.0, 6807009.0],
                        [665496.0, 6812431.0],
                        [663371.0, 6815578.0],
                        [663045.0, 6825117.0],
                        [666363.0, 6833836.0],
                        [665649.0, 6839786.0],
                        [668472.0, 6843626.0],
                        [663488.0, 6848458.0],
                        [653706.0, 6846856.0],
                        [650056.0, 6850064.0],
                        [643163.0, 6853168.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "91", "NOM_DEPT": "ESSONNE"},
        },
        74: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    404250.7000000758,
                    6545825.20000186,
                    485042.7000000638,
                    6672433.600001494,
                ),
                "coordinates": [
                    [
                        [483517.0, 6558927.0],
                        [485043.0, 6563684.0],
                        [480502.0, 6565882.0],
                        [477460.0, 6571967.0],
                        [478920.0, 6578170.0],
                        [482897.0, 6584792.0],
                        [479925.0, 6587264.0],
                        [475179.0, 6582497.0],
                        [472141.0, 6584899.0],
                        [472081.0, 6590232.0],
                        [468300.0, 6594579.0],
                        [467690.0, 6607234.0],
                        [472374.0, 6614809.0],
                        [467588.0, 6622657.0],
                        [470952.0, 6628365.0],
                        [469757.0, 6639127.0],
                        [471018.0, 6649618.0],
                        [465537.0, 6662685.0],
                        [464624.0, 6667352.0],
                        [458546.0, 6672434.0],
                        [454238.0, 6672319.0],
                        [442005.0, 6668948.0],
                        [434893.0, 6670550.0],
                        [429925.0, 6668493.0],
                        [429262.0, 6663802.0],
                        [424993.0, 6661094.0],
                        [417889.0, 6660618.0],
                        [413368.0, 6662854.0],
                        [404251.0, 6660135.0],
                        [408792.0, 6655192.0],
                        [411920.0, 6644974.0],
                        [417892.0, 6640888.0],
                        [415662.0, 6636435.0],
                        [420740.0, 6628764.0],
                        [421344.0, 6621760.0],
                        [423543.0, 6619698.0],
                        [424018.0, 6609992.0],
                        [422851.0, 6601758.0],
                        [420470.0, 6597142.0],
                        [428213.0, 6593481.0],
                        [426574.0, 6590996.0],
                        [411453.0, 6585116.0],
                        [411065.0, 6578554.0],
                        [415606.0, 6575464.0],
                        [420640.0, 6566176.0],
                        [426299.0, 6566639.0],
                        [429269.0, 6562335.0],
                        [437311.0, 6559539.0],
                        [446458.0, 6558600.0],
                        [459784.0, 6545825.0],
                        [464645.0, 6548283.0],
                        [465641.0, 6555184.0],
                        [469589.0, 6554692.0],
                        [473907.0, 6559092.0],
                        [483517.0, 6558927.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "79", "NOM_DEPT": "DEUX-SEVRES"},
        },
        75: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    210123.50000008388,
                    6785524.900001191,
                    337450.2000000646,
                    6881072.500000926,
                ),
                "coordinates": [
                    [
                        [306966.0, 6794650.0],
                        [311879.0, 6798427.0],
                        [315234.0, 6806646.0],
                        [320973.0, 6806998.0],
                        [325505.0, 6811765.0],
                        [328273.0, 6809495.0],
                        [332923.0, 6813386.0],
                        [337450.0, 6835570.0],
                        [334969.0, 6837949.0],
                        [330825.0, 6841252.0],
                        [325749.0, 6841751.0],
                        [322500.0, 6846053.0],
                        [319331.0, 6843417.0],
                        [315059.0, 6845536.0],
                        [309156.0, 6855461.0],
                        [296913.0, 6849790.0],
                        [284640.0, 6841250.0],
                        [280881.0, 6840740.0],
                        [271227.0, 6848399.0],
                        [271155.0, 6854763.0],
                        [262910.0, 6863924.0],
                        [263482.0, 6867351.0],
                        [257346.0, 6869777.0],
                        [251930.0, 6880323.0],
                        [243322.0, 6881073.0],
                        [228138.0, 6874463.0],
                        [222446.0, 6879145.0],
                        [216642.0, 6872989.0],
                        [215941.0, 6861279.0],
                        [210124.0, 6860562.0],
                        [210382.0, 6855067.0],
                        [216761.0, 6846809.0],
                        [212655.0, 6842190.0],
                        [213034.0, 6834627.0],
                        [215204.0, 6828740.0],
                        [214652.0, 6824074.0],
                        [216996.0, 6817941.0],
                        [212675.0, 6807535.0],
                        [229439.0, 6804532.0],
                        [233945.0, 6800703.0],
                        [242780.0, 6802066.0],
                        [246074.0, 6807146.0],
                        [252150.0, 6805897.0],
                        [270150.0, 6795768.0],
                        [279762.0, 6794931.0],
                        [277875.0, 6787107.0],
                        [280665.0, 6785525.0],
                        [285132.0, 6789099.0],
                        [291749.0, 6798375.0],
                        [298204.0, 6799128.0],
                        [300637.0, 6793781.0],
                        [306966.0, 6794650.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "22", "NOM_DEPT": "COTES-D'ARMOR"},
        },
        76: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    644783.7000000156,
                    6537017.500001875,
                    776585.8999999785,
                    6633748.500001593,
                ),
                "coordinates": [
                    [
                        [702446.0, 6632753.0],
                        [696943.0, 6633749.0],
                        [686837.0, 6626145.0],
                        [682775.0, 6624336.0],
                        [677474.0, 6626628.0],
                        [669145.0, 6615318.0],
                        [670500.0, 6605989.0],
                        [664479.0, 6602292.0],
                        [660370.0, 6603761.0],
                        [650317.0, 6601560.0],
                        [645179.0, 6595086.0],
                        [644784.0, 6591412.0],
                        [647921.0, 6581264.0],
                        [653217.0, 6581299.0],
                        [659880.0, 6575828.0],
                        [662729.0, 6571178.0],
                        [666451.0, 6560450.0],
                        [672252.0, 6557694.0],
                        [679424.0, 6569309.0],
                        [685369.0, 6566281.0],
                        [688886.0, 6572856.0],
                        [695157.0, 6571452.0],
                        [693029.0, 6567076.0],
                        [697819.0, 6557998.0],
                        [712873.0, 6551781.0],
                        [728730.0, 6550656.0],
                        [732386.0, 6552810.0],
                        [735860.0, 6546571.0],
                        [745523.0, 6547324.0],
                        [753779.0, 6537018.0],
                        [754888.0, 6541809.0],
                        [761084.0, 6541876.0],
                        [763887.0, 6544360.0],
                        [762385.0, 6549859.0],
                        [763504.0, 6554575.0],
                        [760949.0, 6561975.0],
                        [762003.0, 6573092.0],
                        [769267.0, 6575508.0],
                        [775913.0, 6580395.0],
                        [775079.0, 6589273.0],
                        [776586.0, 6596649.0],
                        [773476.0, 6599308.0],
                        [768312.0, 6598303.0],
                        [763922.0, 6603795.0],
                        [761463.0, 6602522.0],
                        [756931.0, 6607772.0],
                        [753291.0, 6618069.0],
                        [748734.0, 6623209.0],
                        [748049.0, 6627896.0],
                        [742042.0, 6624120.0],
                        [737251.0, 6617690.0],
                        [733164.0, 6623625.0],
                        [726499.0, 6620539.0],
                        [720608.0, 6624106.0],
                        [716475.0, 6620334.0],
                        [703745.0, 6628663.0],
                        [702446.0, 6632753.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "03", "NOM_DEPT": "ALLIER"},
        },
        77: {
            "geometry": {
                "type": "MultiPolygon",
                "bbox": (
                    350493.90000010264,
                    6448234.400002138,
                    464807.40000008,
                    6591808.800001736,
                ),
                "coordinates": [
                    [
                        [
                            [365341.0, 6575902.0],
                            [359213.0, 6576771.0],
                            [352445.0, 6583031.0],
                            [350494.0, 6577212.0],
                            [356155.0, 6576636.0],
                            [364082.0, 6571051.0],
                            [365341.0, 6575902.0],
                        ]
                    ],
                    [
                        [
                            [373352.0, 6542832.0],
                            [371342.0, 6552252.0],
                            [366346.0, 6552808.0],
                            [362447.0, 6556836.0],
                            [360591.0, 6548506.0],
                            [369260.0, 6539881.0],
                            [371124.0, 6531592.0],
                            [374437.0, 6534213.0],
                            [375330.0, 6540510.0],
                            [373352.0, 6542832.0],
                        ]
                    ],
                    [
                        [
                            [464807.0, 6459164.0],
                            [461242.0, 6465553.0],
                            [455660.0, 6465791.0],
                            [456356.0, 6470307.0],
                            [447267.0, 6474282.0],
                            [443202.0, 6478722.0],
                            [445861.0, 6484044.0],
                            [444696.0, 6489309.0],
                            [446521.0, 6493231.0],
                            [442913.0, 6501478.0],
                            [436951.0, 6506668.0],
                            [434607.0, 6512022.0],
                            [434480.0, 6521530.0],
                            [445866.0, 6528260.0],
                            [449713.0, 6524677.0],
                            [455132.0, 6526680.0],
                            [458386.0, 6534878.0],
                            [457303.0, 6539406.0],
                            [459784.0, 6545825.0],
                            [446458.0, 6558600.0],
                            [437311.0, 6559539.0],
                            [429269.0, 6562335.0],
                            [426299.0, 6566639.0],
                            [420640.0, 6566176.0],
                            [415606.0, 6575464.0],
                            [411065.0, 6578554.0],
                            [411453.0, 6585116.0],
                            [404775.0, 6589452.0],
                            [397324.0, 6586760.0],
                            [391500.0, 6591809.0],
                            [382360.0, 6587238.0],
                            [383479.0, 6581738.0],
                            [376439.0, 6576760.0],
                            [374243.0, 6571617.0],
                            [381511.0, 6566597.0],
                            [386552.0, 6553761.0],
                            [382858.0, 6551653.0],
                            [385303.0, 6547012.0],
                            [378912.0, 6532887.0],
                            [370510.0, 6529071.0],
                            [370474.0, 6520657.0],
                            [379564.0, 6513624.0],
                            [387626.0, 6509256.0],
                            [386164.0, 6505245.0],
                            [399796.0, 6489042.0],
                            [404682.0, 6481482.0],
                            [406100.0, 6476140.0],
                            [418478.0, 6475494.0],
                            [421098.0, 6472770.0],
                            [431214.0, 6470295.0],
                            [433418.0, 6457450.0],
                            [437492.0, 6457585.0],
                            [444096.0, 6451365.0],
                            [452009.0, 6448234.0],
                            [457280.0, 6451647.0],
                            [460931.0, 6449393.0],
                            [464807.0, 6459164.0],
                        ]
                    ],
                ],
            },
            "attributes": {"CODE_DEPT": "17", "NOM_DEPT": "CHARENTE-MARITIME"},
        },
        78: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    626128.5000000258,
                    6391030.700002284,
                    728488.39999999,
                    6486680.300002018,
                ),
                "coordinates": [
                    [
                        [728488.0, 6430309.0],
                        [727352.0, 6435060.0],
                        [723408.0, 6437470.0],
                        [725628.0, 6444038.0],
                        [722648.0, 6446835.0],
                        [717445.0, 6463649.0],
                        [709251.0, 6464988.0],
                        [708104.0, 6472781.0],
                        [701310.0, 6465307.0],
                        [695974.0, 6467752.0],
                        [693859.0, 6474045.0],
                        [688449.0, 6477003.0],
                        [677684.0, 6475835.0],
                        [672750.0, 6482640.0],
                        [661594.0, 6486680.0],
                        [659915.0, 6480065.0],
                        [656321.0, 6476249.0],
                        [651369.0, 6479623.0],
                        [649118.0, 6470007.0],
                        [642886.0, 6465907.0],
                        [636846.0, 6458269.0],
                        [638218.0, 6451518.0],
                        [634831.0, 6442812.0],
                        [628772.0, 6440050.0],
                        [631663.0, 6432186.0],
                        [626129.0, 6431248.0],
                        [628463.0, 6419614.0],
                        [634482.0, 6410441.0],
                        [633003.0, 6406356.0],
                        [634129.0, 6393565.0],
                        [637122.0, 6391031.0],
                        [637228.0, 6394175.0],
                        [646639.0, 6396959.0],
                        [655192.0, 6393469.0],
                        [664849.0, 6402561.0],
                        [665404.0, 6408608.0],
                        [668313.0, 6410487.0],
                        [668604.0, 6416079.0],
                        [676829.0, 6423158.0],
                        [679353.0, 6426925.0],
                        [684359.0, 6419403.0],
                        [688126.0, 6419157.0],
                        [690477.0, 6411547.0],
                        [694820.0, 6408897.0],
                        [693924.0, 6403282.0],
                        [698547.0, 6393953.0],
                        [704864.0, 6413115.0],
                        [707862.0, 6414941.0],
                        [708330.0, 6420849.0],
                        [714401.0, 6418301.0],
                        [719705.0, 6424147.0],
                        [728488.0, 6430309.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "15", "NOM_DEPT": "CANTAL"},
        },
        79: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    662679.3000000154,
                    6234873.500002678,
                    796379.4999999614,
                    6319158.60000247,
                ),
                "coordinates": [
                    [
                        [664821.0, 6258302.0],
                        [662679.0, 6249784.0],
                        [667653.0, 6244475.0],
                        [681495.0, 6239871.0],
                        [686427.0, 6247043.0],
                        [700353.0, 6246775.0],
                        [700385.0, 6243263.0],
                        [711963.0, 6238657.0],
                        [716336.0, 6238907.0],
                        [719561.0, 6234874.0],
                        [727933.0, 6241307.0],
                        [734945.0, 6243544.0],
                        [741504.0, 6241707.0],
                        [748880.0, 6250885.0],
                        [753675.0, 6255020.0],
                        [764523.0, 6260540.0],
                        [773373.0, 6269062.0],
                        [781738.0, 6273121.0],
                        [788998.0, 6273432.0],
                        [792922.0, 6276955.0],
                        [796379.0, 6284647.0],
                        [792244.0, 6293431.0],
                        [780634.0, 6301895.0],
                        [773726.0, 6310049.0],
                        [766790.0, 6307842.0],
                        [766255.0, 6313712.0],
                        [763184.0, 6318967.0],
                        [758340.0, 6319159.0],
                        [750088.0, 6313306.0],
                        [746263.0, 6307559.0],
                        [742673.0, 6306434.0],
                        [734096.0, 6312602.0],
                        [728786.0, 6312824.0],
                        [722102.0, 6311077.0],
                        [716344.0, 6301672.0],
                        [704815.0, 6303628.0],
                        [704898.0, 6288229.0],
                        [698819.0, 6289943.0],
                        [694796.0, 6288406.0],
                        [693221.0, 6284557.0],
                        [680178.0, 6279485.0],
                        [676690.0, 6282993.0],
                        [671212.0, 6283521.0],
                        [668935.0, 6274250.0],
                        [672394.0, 6269695.0],
                        [672444.0, 6264623.0],
                        [668090.0, 6259189.0],
                        [664821.0, 6258302.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "34", "NOM_DEPT": "HERAULT"},
        },
        80: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    831613.6999999505,
                    6338354.700002426,
                    921862.99999992,
                    6473590.500002059,
                ),
                "coordinates": [
                    [
                        [921863.0, 6404787.0],
                        [918546.0, 6403480.0],
                        [907889.0, 6409384.0],
                        [894862.0, 6413405.0],
                        [895949.0, 6427870.0],
                        [895595.0, 6439026.0],
                        [892767.0, 6444271.0],
                        [888053.0, 6440277.0],
                        [871457.0, 6445038.0],
                        [871909.0, 6454823.0],
                        [870728.0, 6463360.0],
                        [866406.0, 6463235.0],
                        [860939.0, 6470947.0],
                        [855768.0, 6473591.0],
                        [847361.0, 6468209.0],
                        [841110.0, 6468164.0],
                        [842390.0, 6453321.0],
                        [844782.0, 6435636.0],
                        [848816.0, 6428161.0],
                        [844069.0, 6414766.0],
                        [839715.0, 6410394.0],
                        [841041.0, 6396655.0],
                        [835764.0, 6388411.0],
                        [834538.0, 6384016.0],
                        [835151.0, 6374038.0],
                        [832713.0, 6371581.0],
                        [831614.0, 6360351.0],
                        [840525.0, 6360052.0],
                        [843837.0, 6356928.0],
                        [844895.0, 6349809.0],
                        [864358.0, 6358713.0],
                        [871547.0, 6356009.0],
                        [872115.0, 6350341.0],
                        [890550.0, 6347548.0],
                        [890601.0, 6342463.0],
                        [894810.0, 6342254.0],
                        [899978.0, 6338355.0],
                        [903520.0, 6340766.0],
                        [905908.0, 6346585.0],
                        [911050.0, 6344481.0],
                        [913873.0, 6347227.0],
                        [914428.0, 6355637.0],
                        [911254.0, 6355549.0],
                        [909816.0, 6362305.0],
                        [898744.0, 6362932.0],
                        [892505.0, 6372493.0],
                        [899815.0, 6380077.0],
                        [907085.0, 6377469.0],
                        [909000.0, 6384874.0],
                        [906293.0, 6386082.0],
                        [910174.0, 6394503.0],
                        [921214.0, 6398819.0],
                        [921863.0, 6404787.0],
                    ],
                    [
                        [850837.0, 6364673.0],
                        [852761.0, 6369494.0],
                        [856829.0, 6372215.0],
                        [860545.0, 6368019.0],
                        [860544.0, 6360615.0],
                        [857850.0, 6357360.0],
                        [850729.0, 6357929.0],
                        [850837.0, 6364673.0],
                    ],
                ],
            },
            "attributes": {"CODE_DEPT": "26", "NOM_DEPT": "DROME"},
        },
        81: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    595443.1000000458,
                    6137190.400002889,
                    714274.8999999937,
                    6202112.800002751,
                ),
                "coordinates": [
                    [
                        [600206.0, 6164532.0],
                        [597209.0, 6163462.0],
                        [595443.0, 6155896.0],
                        [604042.0, 6154257.0],
                        [614139.0, 6147692.0],
                        [616327.0, 6140765.0],
                        [624388.0, 6140739.0],
                        [630462.0, 6147428.0],
                        [638840.0, 6149005.0],
                        [653442.0, 6143379.0],
                        [657322.0, 6137875.0],
                        [662029.0, 6137190.0],
                        [665163.0, 6139863.0],
                        [672907.0, 6137962.0],
                        [673207.0, 6145058.0],
                        [679658.0, 6147303.0],
                        [683451.0, 6146537.0],
                        [686936.0, 6150974.0],
                        [703357.0, 6152588.0],
                        [707042.0, 6147292.0],
                        [714275.0, 6148346.0],
                        [710994.0, 6157024.0],
                        [704080.0, 6161154.0],
                        [702907.0, 6175403.0],
                        [703560.0, 6193187.0],
                        [693222.0, 6198384.0],
                        [688989.0, 6202113.0],
                        [682753.0, 6199641.0],
                        [678721.0, 6193157.0],
                        [675246.0, 6192375.0],
                        [659825.0, 6194610.0],
                        [655189.0, 6193206.0],
                        [644930.0, 6193342.0],
                        [647326.0, 6181831.0],
                        [644388.0, 6178927.0],
                        [631545.0, 6174170.0],
                        [617954.0, 6174066.0],
                        [615529.0, 6169131.0],
                        [607482.0, 6165290.0],
                        [600206.0, 6164532.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "66", "NOM_DEPT": "PYRENEES-ORIENTALES"},
        },
        82: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    904448.5999999343,
                    6445704.300002147,
                    1027166.6999998918,
                    6541571.800001871,
                ),
                "coordinates": [
                    [
                        [956276.0, 6452799.0],
                        [964422.0, 6450656.0],
                        [967061.0, 6446025.0],
                        [974349.0, 6445704.0],
                        [973771.0, 6449910.0],
                        [985382.0, 6452122.0],
                        [989157.0, 6455721.0],
                        [995952.0, 6458223.0],
                        [1002558.0, 6454919.0],
                        [1011443.0, 6464355.0],
                        [1024403.0, 6470318.0],
                        [1021887.0, 6478062.0],
                        [1027167.0, 6487344.0],
                        [1021485.0, 6490016.0],
                        [1020210.0, 6493869.0],
                        [1012252.0, 6497350.0],
                        [1011518.0, 6512378.0],
                        [1006337.0, 6512917.0],
                        [997830.0, 6518696.0],
                        [995359.0, 6526975.0],
                        [988618.0, 6520489.0],
                        [984284.0, 6528734.0],
                        [979688.0, 6528135.0],
                        [975690.0, 6531451.0],
                        [971940.0, 6540371.0],
                        [969073.0, 6537583.0],
                        [965836.0, 6528299.0],
                        [962862.0, 6525460.0],
                        [959117.0, 6516128.0],
                        [952091.0, 6514710.0],
                        [948121.0, 6516633.0],
                        [946066.0, 6522280.0],
                        [936560.0, 6520059.0],
                        [926213.0, 6526897.0],
                        [922988.0, 6530552.0],
                        [921730.0, 6540986.0],
                        [919315.0, 6541572.0],
                        [916332.0, 6528631.0],
                        [915868.0, 6518199.0],
                        [910338.0, 6515974.0],
                        [909413.0, 6509006.0],
                        [904449.0, 6504924.0],
                        [913763.0, 6489603.0],
                        [917457.0, 6486189.0],
                        [925224.0, 6482705.0],
                        [931624.0, 6492445.0],
                        [938381.0, 6486658.0],
                        [943964.0, 6486985.0],
                        [948557.0, 6482085.0],
                        [949441.0, 6473761.0],
                        [945461.0, 6470305.0],
                        [945165.0, 6465401.0],
                        [950505.0, 6456663.0],
                        [956276.0, 6452799.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "73", "NOM_DEPT": "SAVOIE"},
        },
        83: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    892505.1999999284,
                    6347226.500002407,
                    1022959.6999998831,
                    6452798.600002125,
                ),
                "coordinates": [
                    [
                        [921863.0, 6404787.0],
                        [921214.0, 6398819.0],
                        [910174.0, 6394503.0],
                        [906293.0, 6386082.0],
                        [909000.0, 6384874.0],
                        [907085.0, 6377469.0],
                        [899815.0, 6380077.0],
                        [892505.0, 6372493.0],
                        [898744.0, 6362932.0],
                        [909816.0, 6362305.0],
                        [911254.0, 6355549.0],
                        [914428.0, 6355637.0],
                        [913873.0, 6347227.0],
                        [920148.0, 6349530.0],
                        [930231.0, 6349978.0],
                        [925339.0, 6357350.0],
                        [932421.0, 6356788.0],
                        [935206.0, 6370624.0],
                        [944316.0, 6377139.0],
                        [950982.0, 6378672.0],
                        [957118.0, 6369997.0],
                        [959705.0, 6373500.0],
                        [957182.0, 6379131.0],
                        [967103.0, 6386021.0],
                        [971476.0, 6380340.0],
                        [983401.0, 6378221.0],
                        [988952.0, 6378656.0],
                        [992592.0, 6389297.0],
                        [999315.0, 6394909.0],
                        [1004568.0, 6397351.0],
                        [1010207.0, 6403290.0],
                        [1012913.0, 6402904.0],
                        [1015781.0, 6406977.0],
                        [1022960.0, 6406318.0],
                        [1016213.0, 6418043.0],
                        [1016501.0, 6423604.0],
                        [1010607.0, 6425833.0],
                        [1004832.0, 6424533.0],
                        [995855.0, 6430156.0],
                        [996598.0, 6437292.0],
                        [994575.0, 6442256.0],
                        [989294.0, 6442329.0],
                        [985382.0, 6452122.0],
                        [973771.0, 6449910.0],
                        [974349.0, 6445704.0],
                        [967061.0, 6446025.0],
                        [964422.0, 6450656.0],
                        [956276.0, 6452799.0],
                        [953909.0, 6450690.0],
                        [952342.0, 6439924.0],
                        [961377.0, 6439340.0],
                        [964872.0, 6432532.0],
                        [965006.0, 6422921.0],
                        [960775.0, 6424797.0],
                        [956742.0, 6422339.0],
                        [947667.0, 6423241.0],
                        [941569.0, 6417648.0],
                        [939410.0, 6420031.0],
                        [933177.0, 6416064.0],
                        [930672.0, 6410433.0],
                        [925560.0, 6409806.0],
                        [921863.0, 6404787.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "05", "NOM_DEPT": "HAUTES-ALPES"},
        },
        84: {
            "geometry": {
                "type": "MultiPolygon",
                "bbox": (
                    831613.6999999505,
                    6287714.5000025565,
                    922160.3999999114,
                    6372214.700002337,
                ),
                "coordinates": [
                    [
                        [
                            [850837.0, 6364673.0],
                            [850729.0, 6357929.0],
                            [857850.0, 6357360.0],
                            [860544.0, 6360615.0],
                            [860545.0, 6368019.0],
                            [856829.0, 6372215.0],
                            [852761.0, 6369494.0],
                            [850837.0, 6364673.0],
                        ]
                    ],
                    [
                        [
                            [839658.0, 6315433.0],
                            [848847.0, 6314233.0],
                            [858049.0, 6310039.0],
                            [863255.0, 6305296.0],
                            [865136.0, 6300908.0],
                            [876282.0, 6295340.0],
                            [879627.0, 6296831.0],
                            [886796.0, 6295631.0],
                            [896706.0, 6289935.0],
                            [904218.0, 6287715.0],
                            [910307.0, 6287877.0],
                            [921881.0, 6295602.0],
                            [922160.0, 6296144.0],
                            [913516.0, 6306489.0],
                            [907367.0, 6306820.0],
                            [909386.0, 6316209.0],
                            [901642.0, 6319541.0],
                            [903879.0, 6328807.0],
                            [900470.0, 6332559.0],
                            [899978.0, 6338355.0],
                            [894810.0, 6342254.0],
                            [890601.0, 6342463.0],
                            [890550.0, 6347548.0],
                            [872115.0, 6350341.0],
                            [871547.0, 6356009.0],
                            [864358.0, 6358713.0],
                            [844895.0, 6349809.0],
                            [843837.0, 6356928.0],
                            [840525.0, 6360052.0],
                            [831614.0, 6360351.0],
                            [831641.0, 6353746.0],
                            [837656.0, 6344658.0],
                            [836689.0, 6335330.0],
                            [843231.0, 6331183.0],
                            [847769.0, 6322554.0],
                            [839658.0, 6315433.0],
                        ]
                    ],
                ],
            },
            "attributes": {"CODE_DEPT": "84", "NOM_DEPT": "VAUCLUSE"},
        },
        85: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    537221.0000000435,
                    6584255.600001739,
                    638968.6000000171,
                    6687210.900001439,
                ),
                "coordinates": [
                    [
                        [636090.0, 6591904.0],
                        [634893.0, 6595681.0],
                        [638969.0, 6598575.0],
                        [635550.0, 6606712.0],
                        [637665.0, 6611745.0],
                        [634340.0, 6622075.0],
                        [628673.0, 6627353.0],
                        [632500.0, 6630848.0],
                        [627496.0, 6637025.0],
                        [633457.0, 6642556.0],
                        [629301.0, 6648616.0],
                        [631131.0, 6657674.0],
                        [627981.0, 6660408.0],
                        [627833.0, 6667741.0],
                        [621646.0, 6669974.0],
                        [615265.0, 6667454.0],
                        [607114.0, 6670785.0],
                        [612309.0, 6675948.0],
                        [612147.0, 6680536.0],
                        [602149.0, 6685865.0],
                        [593597.0, 6687211.0],
                        [582631.0, 6683116.0],
                        [573297.0, 6677558.0],
                        [576079.0, 6670370.0],
                        [571950.0, 6667671.0],
                        [568878.0, 6661167.0],
                        [560408.0, 6661557.0],
                        [553982.0, 6659068.0],
                        [551779.0, 6655211.0],
                        [546519.0, 6631001.0],
                        [537221.0, 6629765.0],
                        [541431.0, 6623495.0],
                        [538906.0, 6616112.0],
                        [542426.0, 6611026.0],
                        [547958.0, 6609437.0],
                        [557842.0, 6602380.0],
                        [558128.0, 6596023.0],
                        [562773.0, 6594026.0],
                        [559931.0, 6588729.0],
                        [569564.0, 6587075.0],
                        [573702.0, 6590223.0],
                        [578129.0, 6584256.0],
                        [586530.0, 6592902.0],
                        [595863.0, 6588324.0],
                        [602234.0, 6588456.0],
                        [609355.0, 6592908.0],
                        [622684.0, 6592821.0],
                        [630016.0, 6590287.0],
                        [636090.0, 6591904.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "36", "NOM_DEPT": "INDRE"},
        },
        86: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    1159341.799999797,
                    6102880.200002988,
                    1242212.6999997562,
                    6233783.10000272,
                ),
                "coordinates": [
                    [
                        [1232225.0, 6105798.0],
                        [1232233.0, 6116272.0],
                        [1242213.0, 6134033.0],
                        [1241272.0, 6154177.0],
                        [1238247.0, 6164499.0],
                        [1238682.0, 6169952.0],
                        [1236764.0, 6182904.0],
                        [1228823.0, 6195229.0],
                        [1228376.0, 6197858.0],
                        [1231051.0, 6211355.0],
                        [1228846.0, 6218876.0],
                        [1227050.0, 6231255.0],
                        [1223421.0, 6233783.0],
                        [1216996.0, 6231337.0],
                        [1218749.0, 6222752.0],
                        [1215986.0, 6213431.0],
                        [1218915.0, 6209217.0],
                        [1219580.0, 6202912.0],
                        [1215962.0, 6195537.0],
                        [1212393.0, 6200184.0],
                        [1205077.0, 6201621.0],
                        [1198634.0, 6198671.0],
                        [1193909.0, 6190462.0],
                        [1183050.0, 6187935.0],
                        [1176939.0, 6184145.0],
                        [1176781.0, 6180877.0],
                        [1165934.0, 6173994.0],
                        [1165800.0, 6162888.0],
                        [1161943.0, 6162734.0],
                        [1159342.0, 6158599.0],
                        [1164950.0, 6158109.0],
                        [1180116.0, 6152978.0],
                        [1187662.0, 6146432.0],
                        [1199675.0, 6142105.0],
                        [1202519.0, 6133762.0],
                        [1206639.0, 6131576.0],
                        [1210743.0, 6122769.0],
                        [1215715.0, 6123336.0],
                        [1217754.0, 6104291.0],
                        [1226359.0, 6102880.0],
                        [1232225.0, 6105798.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "2B", "NOM_DEPT": "HAUTE-CORSE"},
        },
        87: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    515634.90000005835,
                    6484264.10000203,
                    614305.1000000271,
                    6590222.6000017235,
                ),
                "coordinates": [
                    [
                        [563452.0, 6484264.0],
                        [566212.0, 6489282.0],
                        [571080.0, 6486618.0],
                        [576059.0, 6493313.0],
                        [581471.0, 6497412.0],
                        [587429.0, 6495459.0],
                        [593766.0, 6498788.0],
                        [605337.0, 6509707.0],
                        [612121.0, 6507862.0],
                        [614305.0, 6511565.0],
                        [612524.0, 6515422.0],
                        [613074.0, 6522626.0],
                        [604752.0, 6527760.0],
                        [597279.0, 6527024.0],
                        [591561.0, 6529605.0],
                        [592200.0, 6538029.0],
                        [586335.0, 6540289.0],
                        [587795.0, 6549437.0],
                        [580656.0, 6565605.0],
                        [574730.0, 6569689.0],
                        [580236.0, 6579624.0],
                        [578129.0, 6584256.0],
                        [573702.0, 6590223.0],
                        [569564.0, 6587075.0],
                        [559931.0, 6588729.0],
                        [556218.0, 6586207.0],
                        [548271.0, 6584464.0],
                        [546474.0, 6577615.0],
                        [538896.0, 6578287.0],
                        [530127.0, 6570272.0],
                        [531964.0, 6561069.0],
                        [531290.0, 6552122.0],
                        [539507.0, 6547743.0],
                        [540666.0, 6541823.0],
                        [534383.0, 6537804.0],
                        [527825.0, 6527023.0],
                        [522669.0, 6525452.0],
                        [522412.0, 6520738.0],
                        [515635.0, 6515544.0],
                        [526849.0, 6509996.0],
                        [524801.0, 6504583.0],
                        [529342.0, 6499739.0],
                        [539039.0, 6503445.0],
                        [546900.0, 6501981.0],
                        [553144.0, 6489340.0],
                        [563452.0, 6484264.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "87", "NOM_DEPT": "HAUTE-VIENNE"},
        },
        88: {
            "geometry": {
                "type": "MultiPolygon",
                "bbox": (
                    297360.00000010285,
                    6582707.500001765,
                    428213.4000000774,
                    6673611.100001499,
                ),
                "coordinates": [
                    [
                        [
                            [297360.0, 6667914.0],
                            [299985.0, 6664525.0],
                            [303823.0, 6669495.0],
                            [298725.0, 6671705.0],
                            [297360.0, 6667914.0],
                        ]
                    ],
                    [
                        [
                            [322009.0, 6670666.0],
                            [318317.0, 6668787.0],
                            [315194.0, 6662103.0],
                            [307861.0, 6656132.0],
                            [308108.0, 6649020.0],
                            [322518.0, 6633220.0],
                            [328569.0, 6623483.0],
                            [331126.0, 6610513.0],
                            [333817.0, 6610276.0],
                            [344797.0, 6600884.0],
                            [354269.0, 6598459.0],
                            [356701.0, 6592204.0],
                            [364357.0, 6592471.0],
                            [376108.0, 6582708.0],
                            [376887.0, 6588202.0],
                            [382360.0, 6587238.0],
                            [391500.0, 6591809.0],
                            [397324.0, 6586760.0],
                            [404775.0, 6589452.0],
                            [411453.0, 6585116.0],
                            [426574.0, 6590996.0],
                            [428213.0, 6593481.0],
                            [420470.0, 6597142.0],
                            [422851.0, 6601758.0],
                            [424018.0, 6609992.0],
                            [423543.0, 6619698.0],
                            [421344.0, 6621760.0],
                            [420740.0, 6628764.0],
                            [415662.0, 6636435.0],
                            [417892.0, 6640888.0],
                            [411920.0, 6644974.0],
                            [408792.0, 6655192.0],
                            [404251.0, 6660135.0],
                            [401283.0, 6663962.0],
                            [392012.0, 6664073.0],
                            [385082.0, 6667087.0],
                            [376108.0, 6673611.0],
                            [374065.0, 6668782.0],
                            [367645.0, 6668175.0],
                            [367588.0, 6659427.0],
                            [361112.0, 6656907.0],
                            [359840.0, 6669393.0],
                            [354394.0, 6667534.0],
                            [354064.0, 6663454.0],
                            [357453.0, 6652400.0],
                            [353671.0, 6649968.0],
                            [343129.0, 6653969.0],
                            [338731.0, 6658583.0],
                            [332678.0, 6659222.0],
                            [324590.0, 6666698.0],
                            [322009.0, 6670666.0],
                        ]
                    ],
                ],
            },
            "attributes": {"CODE_DEPT": "85", "NOM_DEPT": "VENDEE"},
        },
        89: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    916540.6999999128,
                    6220583.000002719,
                    1018255.5999998702,
                    6307147.400002514,
                ),
                "coordinates": [
                    [
                        [917346.0, 6234793.0],
                        [919352.0, 6230892.0],
                        [929076.0, 6227424.0],
                        [928534.0, 6223947.0],
                        [932969.0, 6220583.0],
                        [939545.0, 6227616.0],
                        [947016.0, 6224540.0],
                        [954550.0, 6225038.0],
                        [959301.0, 6228480.0],
                        [965861.0, 6230140.0],
                        [974387.0, 6227034.0],
                        [975697.0, 6233209.0],
                        [984286.0, 6234176.0],
                        [989472.0, 6238682.0],
                        [995722.0, 6237173.0],
                        [997536.0, 6239863.0],
                        [998568.0, 6249133.0],
                        [997443.0, 6253468.0],
                        [1002933.0, 6264260.0],
                        [1012590.0, 6265313.0],
                        [1018256.0, 6272482.0],
                        [1014132.0, 6274778.0],
                        [1015523.0, 6285590.0],
                        [1006552.0, 6288390.0],
                        [1003290.0, 6292496.0],
                        [1002326.0, 6300425.0],
                        [992638.0, 6305621.0],
                        [983574.0, 6307147.0],
                        [974774.0, 6305117.0],
                        [974848.0, 6301646.0],
                        [970413.0, 6298787.0],
                        [958836.0, 6305328.0],
                        [956283.0, 6301072.0],
                        [944763.0, 6293054.0],
                        [933734.0, 6296912.0],
                        [923991.0, 6299127.0],
                        [922160.0, 6296144.0],
                        [921881.0, 6295602.0],
                        [926823.0, 6291811.0],
                        [925792.0, 6288611.0],
                        [917893.0, 6286245.0],
                        [916541.0, 6283087.0],
                        [920209.0, 6276225.0],
                        [918385.0, 6268519.0],
                        [922861.0, 6263349.0],
                        [917403.0, 6259253.0],
                        [924245.0, 6244827.0],
                        [918110.0, 6241136.0],
                        [917346.0, 6234793.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "83", "NOM_DEPT": "VAR"},
        },
        90: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    650056.2000000083,
                    6843626.300000984,
                    671600.3000000048,
                    6862430.900000933,
                ),
                "coordinates": [
                    [
                        [670056.0, 6856451.0],
                        [662997.0, 6862431.0],
                        [657169.0, 6861180.0],
                        [650942.0, 6857646.0],
                        [650056.0, 6850064.0],
                        [653706.0, 6846856.0],
                        [663488.0, 6848458.0],
                        [668472.0, 6843626.0],
                        [671600.0, 6851283.0],
                        [670056.0, 6856451.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "94", "NOM_DEPT": "VAL-DE-MARNE"},
        },
        91: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    637702.2000000101,
                    6850064.200000968,
                    650942.200000008,
                    6872566.900000905,
                ),
                "coordinates": [
                    [
                        [650942.0, 6857646.0],
                        [645325.0, 6859675.0],
                        [643644.0, 6863501.0],
                        [650140.0, 6866932.0],
                        [648071.0, 6872567.0],
                        [641404.0, 6867928.0],
                        [637864.0, 6864035.0],
                        [637702.0, 6857774.0],
                        [643163.0, 6853168.0],
                        [650056.0, 6850064.0],
                        [650942.0, 6857646.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "92", "NOM_DEPT": "HAUTS-DE-SEINE"},
        },
        92: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    698546.9000000006,
                    6336169.1000024285,
                    779410.8999999706,
                    6430308.600002175,
                ),
                "coordinates": [
                    [
                        [698547.0, 6393953.0],
                        [706639.0, 6384441.0],
                        [705449.0, 6378272.0],
                        [710800.0, 6372878.0],
                        [709537.0, 6364934.0],
                        [712305.0, 6356606.0],
                        [709994.0, 6353946.0],
                        [718362.0, 6347498.0],
                        [728598.0, 6344387.0],
                        [729879.0, 6341376.0],
                        [735189.0, 6336775.0],
                        [750700.0, 6336169.0],
                        [750995.0, 6342037.0],
                        [754256.0, 6342872.0],
                        [763780.0, 6336809.0],
                        [768987.0, 6337162.0],
                        [776174.0, 6341769.0],
                        [775968.0, 6350322.0],
                        [773621.0, 6356674.0],
                        [775941.0, 6359276.0],
                        [770571.0, 6365836.0],
                        [775211.0, 6367175.0],
                        [779411.0, 6373918.0],
                        [775042.0, 6386528.0],
                        [771844.0, 6388592.0],
                        [768276.0, 6405343.0],
                        [758530.0, 6415778.0],
                        [752669.0, 6414624.0],
                        [752090.0, 6419191.0],
                        [746937.0, 6419760.0],
                        [746579.0, 6414955.0],
                        [737614.0, 6413026.0],
                        [732994.0, 6422485.0],
                        [732563.0, 6427378.0],
                        [728488.0, 6430309.0],
                        [719705.0, 6424147.0],
                        [714401.0, 6418301.0],
                        [708330.0, 6420849.0],
                        [707862.0, 6414941.0],
                        [704864.0, 6413115.0],
                        [698547.0, 6393953.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "48", "NOM_DEPT": "LOZERE"},
        },
        93: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    799606.7999999593,
                    6232742.000002689,
                    926823.2999999092,
                    6315433.300002484,
                ),
                "coordinates": [
                    [
                        [799607.0, 6263119.0],
                        [825918.0, 6262030.0],
                        [828657.0, 6259471.0],
                        [826719.0, 6253810.0],
                        [834804.0, 6251115.0],
                        [842728.0, 6251679.0],
                        [850515.0, 6249931.0],
                        [851283.0, 6257962.0],
                        [859395.0, 6260541.0],
                        [865618.0, 6249711.0],
                        [881792.0, 6250642.0],
                        [887411.0, 6254047.0],
                        [891677.0, 6249845.0],
                        [890480.0, 6245406.0],
                        [892188.0, 6237089.0],
                        [906247.0, 6238345.0],
                        [911635.0, 6232742.0],
                        [917346.0, 6234793.0],
                        [918110.0, 6241136.0],
                        [924245.0, 6244827.0],
                        [917403.0, 6259253.0],
                        [922861.0, 6263349.0],
                        [918385.0, 6268519.0],
                        [920209.0, 6276225.0],
                        [916541.0, 6283087.0],
                        [917893.0, 6286245.0],
                        [925792.0, 6288611.0],
                        [926823.0, 6291811.0],
                        [921881.0, 6295602.0],
                        [910307.0, 6287877.0],
                        [904218.0, 6287715.0],
                        [896706.0, 6289935.0],
                        [886796.0, 6295631.0],
                        [879627.0, 6296831.0],
                        [876282.0, 6295340.0],
                        [865136.0, 6300908.0],
                        [863255.0, 6305296.0],
                        [858049.0, 6310039.0],
                        [848847.0, 6314233.0],
                        [839658.0, 6315433.0],
                        [831988.0, 6308980.0],
                        [833168.0, 6302217.0],
                        [828463.0, 6288899.0],
                        [819908.0, 6290041.0],
                        [815212.0, 6281222.0],
                        [813903.0, 6274579.0],
                        [800175.0, 6267290.0],
                        [799607.0, 6263119.0],
                    ],
                    [
                        [869889.0, 6258065.0],
                        [866899.0, 6258272.0],
                        [866104.0, 6264876.0],
                        [862374.0, 6265459.0],
                        [862969.0, 6272152.0],
                        [870516.0, 6271879.0],
                        [874819.0, 6259107.0],
                        [869889.0, 6258065.0],
                    ],
                ],
            },
            "attributes": {"CODE_DEPT": "13", "NOM_DEPT": "BOUCHES-DU-RHONE"},
        },
        94: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    643644.0000000091,
                    6857645.800000944,
                    657169.1000000068,
                    6867065.000000917,
                ),
                "coordinates": [
                    [
                        [650942.0, 6857646.0],
                        [657169.0, 6861180.0],
                        [654926.0, 6867065.0],
                        [650140.0, 6866932.0],
                        [643644.0, 6863501.0],
                        [645325.0, 6859675.0],
                        [650942.0, 6857646.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "75", "NOM_DEPT": "PARIS"},
        },
        95: {
            "geometry": {
                "type": "Polygon",
                "bbox": (
                    648070.7000000081,
                    6856451.100000947,
                    670882.1000000045,
                    6878970.900000885,
                ),
                "coordinates": [
                    [
                        [670056.0, 6856451.0],
                        [667928.0, 6865519.0],
                        [670882.0, 6870004.0],
                        [667303.0, 6878971.0],
                        [660416.0, 6872923.0],
                        [653619.0, 6875101.0],
                        [648071.0, 6872567.0],
                        [650140.0, 6866932.0],
                        [654926.0, 6867065.0],
                        [657169.0, 6861180.0],
                        [662997.0, 6862431.0],
                        [670056.0, 6856451.0],
                    ]
                ],
            },
            "attributes": {"CODE_DEPT": "93", "NOM_DEPT": "SEINE-SAINT-DENIS"},
        },
    },
}

geolayer_fr_dept_geometry_only = {
    "metadata": {
        "name": "FRANCE_DPT_GENERALIZE_LAMB93_ROUND_GEOMETRY_ONLY",
        "geometry_ref": {
            "type": {"MultiPolygon", "Polygon"},
            "extent": (124277.0, 6050136.0, 1242213.0, 7110430.0),
            "crs": 2154,
        },
    },
    "features": {
        0: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [505760.0, 6248691.0],
                        [509767.0, 6249274.0],
                        [514960.0, 6255730.0],
                        [523428.0, 6259977.0],
                        [531230.0, 6258583.0],
                        [536034.0, 6253601.0],
                        [539594.0, 6265818.0],
                        [554723.0, 6278961.0],
                        [550482.0, 6283435.0],
                        [545681.0, 6284750.0],
                        [535303.0, 6300840.0],
                        [531445.0, 6300603.0],
                        [531142.0, 6309545.0],
                        [528381.0, 6316132.0],
                        [520187.0, 6318466.0],
                        [528536.0, 6329188.0],
                        [521740.0, 6328834.0],
                        [519117.0, 6332164.0],
                        [512956.0, 6327891.0],
                        [507637.0, 6333951.0],
                        [502833.0, 6331375.0],
                        [496470.0, 6331745.0],
                        [495008.0, 6328849.0],
                        [483787.0, 6325253.0],
                        [474692.0, 6327970.0],
                        [465465.0, 6324803.0],
                        [465239.0, 6317398.0],
                        [461632.0, 6315722.0],
                        [453303.0, 6321029.0],
                        [444807.0, 6320573.0],
                        [440195.0, 6315550.0],
                        [443572.0, 6313646.0],
                        [442698.0, 6298315.0],
                        [439499.0, 6295895.0],
                        [438084.0, 6281592.0],
                        [443497.0, 6282643.0],
                        [449863.0, 6280833.0],
                        [456497.0, 6283051.0],
                        [457450.0, 6278664.0],
                        [461941.0, 6273297.0],
                        [466465.0, 6272970.0],
                        [468215.0, 6263513.0],
                        [471536.0, 6256527.0],
                        [479309.0, 6257879.0],
                        [496988.0, 6250664.0],
                        [505760.0, 6248691.0],
                    ]
                ],
                "bbox": (438084.0, 6248691.0, 554723.0, 6333951.0),
            }
        },
        1: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [547193.0, 6388334.0],
                        [539761.0, 6395885.0],
                        [530971.0, 6390971.0],
                        [526858.0, 6394383.0],
                        [527849.0, 6401566.0],
                        [514373.0, 6400391.0],
                        [512334.0, 6403639.0],
                        [495031.0, 6397493.0],
                        [491196.0, 6399384.0],
                        [489091.0, 6407815.0],
                        [486169.0, 6410676.0],
                        [474628.0, 6407547.0],
                        [472909.0, 6402270.0],
                        [476701.0, 6399758.0],
                        [474218.0, 6394851.0],
                        [464993.0, 6388179.0],
                        [460343.0, 6383021.0],
                        [460366.0, 6373402.0],
                        [462536.0, 6370133.0],
                        [454032.0, 6364649.0],
                        [457916.0, 6359953.0],
                        [455164.0, 6354504.0],
                        [449164.0, 6352477.0],
                        [449759.0, 6344226.0],
                        [459734.0, 6343570.0],
                        [462757.0, 6341305.0],
                        [470769.0, 6340294.0],
                        [467874.0, 6336236.0],
                        [464468.0, 6326808.0],
                        [465465.0, 6324803.0],
                        [474692.0, 6327970.0],
                        [483787.0, 6325253.0],
                        [495008.0, 6328849.0],
                        [496470.0, 6331745.0],
                        [502833.0, 6331375.0],
                        [507637.0, 6333951.0],
                        [512956.0, 6327891.0],
                        [519117.0, 6332164.0],
                        [520115.0, 6336256.0],
                        [529462.0, 6338671.0],
                        [533815.0, 6350996.0],
                        [536387.0, 6355171.0],
                        [530427.0, 6360537.0],
                        [531660.0, 6365290.0],
                        [544593.0, 6364542.0],
                        [545774.0, 6366277.0],
                        [541673.0, 6377661.0],
                        [542159.0, 6383881.0],
                        [547193.0, 6388334.0],
                    ]
                ],
                "bbox": (449164.0, 6324803.0, 547193.0, 6410676.0),
            }
        },
        2: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [841110.0, 6468164.0],
                        [847361.0, 6468209.0],
                        [855768.0, 6473591.0],
                        [860939.0, 6470947.0],
                        [866406.0, 6463235.0],
                        [870728.0, 6463360.0],
                        [871909.0, 6454823.0],
                        [871457.0, 6445038.0],
                        [888053.0, 6440277.0],
                        [892767.0, 6444271.0],
                        [895595.0, 6439026.0],
                        [895949.0, 6427870.0],
                        [894862.0, 6413405.0],
                        [907889.0, 6409384.0],
                        [918546.0, 6403480.0],
                        [921863.0, 6404787.0],
                        [925560.0, 6409806.0],
                        [930672.0, 6410433.0],
                        [933177.0, 6416064.0],
                        [939410.0, 6420031.0],
                        [941569.0, 6417648.0],
                        [947667.0, 6423241.0],
                        [956742.0, 6422339.0],
                        [960775.0, 6424797.0],
                        [965006.0, 6422921.0],
                        [964872.0, 6432532.0],
                        [961377.0, 6439340.0],
                        [952342.0, 6439924.0],
                        [953909.0, 6450690.0],
                        [956276.0, 6452799.0],
                        [950505.0, 6456663.0],
                        [945165.0, 6465401.0],
                        [945461.0, 6470305.0],
                        [949441.0, 6473761.0],
                        [948557.0, 6482085.0],
                        [943964.0, 6486985.0],
                        [938381.0, 6486658.0],
                        [931624.0, 6492445.0],
                        [925224.0, 6482705.0],
                        [917457.0, 6486189.0],
                        [913763.0, 6489603.0],
                        [904449.0, 6504924.0],
                        [897998.0, 6515860.0],
                        [888123.0, 6525941.0],
                        [888996.0, 6528631.0],
                        [882529.0, 6534232.0],
                        [879159.0, 6530984.0],
                        [876089.0, 6523611.0],
                        [870291.0, 6521483.0],
                        [863140.0, 6525920.0],
                        [862850.0, 6517700.0],
                        [865784.0, 6514252.0],
                        [858527.0, 6506228.0],
                        [848198.0, 6502534.0],
                        [841056.0, 6498593.0],
                        [845846.0, 6493277.0],
                        [837308.0, 6485557.0],
                        [837457.0, 6475559.0],
                        [841110.0, 6468164.0],
                    ]
                ],
                "bbox": (837308.0, 6403480.0, 965006.0, 6534232.0),
            }
        },
        3: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [634422.0, 7101577.0],
                        [623701.0, 7099214.0],
                        [610465.0, 7094261.0],
                        [605814.0, 7089013.0],
                        [599885.0, 7086576.0],
                        [601399.0, 7078135.0],
                        [598139.0, 7067917.0],
                        [599563.0, 7049801.0],
                        [597332.0, 7035063.0],
                        [603193.0, 7029103.0],
                        [605232.0, 7026713.0],
                        [612989.0, 7030025.0],
                        [622831.0, 7024302.0],
                        [624801.0, 7021563.0],
                        [634147.0, 7017146.0],
                        [635372.0, 7012312.0],
                        [648121.0, 7014552.0],
                        [654991.0, 7012592.0],
                        [660837.0, 7014835.0],
                        [663948.0, 7010834.0],
                        [656343.0, 7006536.0],
                        [658067.0, 7000416.0],
                        [662547.0, 7004433.0],
                        [667892.0, 7004913.0],
                        [669642.0, 7001764.0],
                        [689814.0, 6997681.0],
                        [691147.0, 6993758.0],
                        [706470.0, 6995042.0],
                        [706855.0, 7002888.0],
                        [711164.0, 7009522.0],
                        [712383.0, 7015916.0],
                        [699507.0, 7031355.0],
                        [702680.0, 7038239.0],
                        [701478.0, 7042709.0],
                        [697493.0, 7046164.0],
                        [692820.0, 7044918.0],
                        [687641.0, 7047582.0],
                        [686184.0, 7056263.0],
                        [688698.0, 7062196.0],
                        [683642.0, 7063001.0],
                        [679720.0, 7059405.0],
                        [673807.0, 7057943.0],
                        [664277.0, 7059477.0],
                        [658064.0, 7064147.0],
                        [653390.0, 7071778.0],
                        [644454.0, 7079622.0],
                        [642453.0, 7087769.0],
                        [636514.0, 7100652.0],
                        [634422.0, 7101577.0],
                    ]
                ],
                "bbox": (597332.0, 6993758.0, 712383.0, 7101577.0),
            }
        },
        4: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [776081.0, 6923412.0],
                        [782994.0, 6923346.0],
                        [790665.0, 6921022.0],
                        [795010.0, 6915033.0],
                        [803525.0, 6909934.0],
                        [814643.0, 6911926.0],
                        [818364.0, 6905215.0],
                        [822754.0, 6907655.0],
                        [833062.0, 6905548.0],
                        [837898.0, 6908256.0],
                        [842092.0, 6905887.0],
                        [849280.0, 6910207.0],
                        [847303.0, 6917040.0],
                        [853548.0, 6926776.0],
                        [849442.0, 6935263.0],
                        [853210.0, 6945817.0],
                        [856532.0, 6942619.0],
                        [867310.0, 6942452.0],
                        [873032.0, 6949057.0],
                        [869864.0, 6950495.0],
                        [863792.0, 6957578.0],
                        [856370.0, 6957018.0],
                        [850915.0, 6964755.0],
                        [848735.0, 6964497.0],
                        [834935.0, 6970707.0],
                        [833403.0, 6975569.0],
                        [835414.0, 6979112.0],
                        [832902.0, 6984653.0],
                        [828614.0, 6985882.0],
                        [833821.0, 7001508.0],
                        [834466.0, 7007583.0],
                        [830491.0, 7008459.0],
                        [826108.0, 7005649.0],
                        [820627.0, 6997569.0],
                        [821199.0, 6989850.0],
                        [814347.0, 6988595.0],
                        [808498.0, 6984157.0],
                        [803958.0, 6983050.0],
                        [795770.0, 6985807.0],
                        [788558.0, 6985051.0],
                        [789845.0, 6973793.0],
                        [787172.0, 6965431.0],
                        [790134.0, 6962730.0],
                        [781387.0, 6953785.0],
                        [779463.0, 6948255.0],
                        [774574.0, 6946610.0],
                        [777906.0, 6941851.0],
                        [775403.0, 6934852.0],
                        [776081.0, 6923412.0],
                    ]
                ],
                "bbox": (774574.0, 6905215.0, 873032.0, 7008459.0),
            }
        },
        5: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [730707.0, 6810078.0],
                        [736893.0, 6807760.0],
                        [745765.0, 6796957.0],
                        [746161.0, 6791892.0],
                        [749678.0, 6782304.0],
                        [755033.0, 6785741.0],
                        [759917.0, 6778325.0],
                        [763413.0, 6765164.0],
                        [767353.0, 6760249.0],
                        [783028.0, 6759096.0],
                        [789480.0, 6763963.0],
                        [796594.0, 6759156.0],
                        [797692.0, 6763119.0],
                        [808147.0, 6762855.0],
                        [816397.0, 6764598.0],
                        [817962.0, 6771080.0],
                        [827043.0, 6770243.0],
                        [825768.0, 6776095.0],
                        [828770.0, 6781171.0],
                        [835407.0, 6779651.0],
                        [837598.0, 6783989.0],
                        [837935.0, 6797546.0],
                        [834073.0, 6806764.0],
                        [827022.0, 6811887.0],
                        [822210.0, 6820200.0],
                        [823302.0, 6827043.0],
                        [819107.0, 6829393.0],
                        [814106.0, 6826118.0],
                        [803526.0, 6829924.0],
                        [796995.0, 6835851.0],
                        [797551.0, 6845278.0],
                        [785824.0, 6845891.0],
                        [779497.0, 6845091.0],
                        [773768.0, 6840880.0],
                        [769985.0, 6834113.0],
                        [760761.0, 6824301.0],
                        [753955.0, 6826609.0],
                        [746802.0, 6826973.0],
                        [740952.0, 6835710.0],
                        [737148.0, 6833964.0],
                        [731262.0, 6826009.0],
                        [728998.0, 6813854.0],
                        [730707.0, 6810078.0],
                    ]
                ],
                "bbox": (728998.0, 6759096.0, 837935.0, 6845891.0),
            }
        },
        6: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [1009696.0, 6370071.0],
                        [1002665.0, 6364730.0],
                        [997268.0, 6357103.0],
                        [994698.0, 6348019.0],
                        [1000668.0, 6338218.0],
                        [1000089.0, 6334256.0],
                        [1007601.0, 6328762.0],
                        [1009068.0, 6324850.0],
                        [1014122.0, 6322114.0],
                        [1012050.0, 6317690.0],
                        [1007566.0, 6320724.0],
                        [1000978.0, 6315262.0],
                        [997097.0, 6315251.0],
                        [992638.0, 6305621.0],
                        [1002326.0, 6300425.0],
                        [1003290.0, 6292496.0],
                        [1006552.0, 6288390.0],
                        [1015523.0, 6285590.0],
                        [1014132.0, 6274778.0],
                        [1018256.0, 6272482.0],
                        [1020971.0, 6279894.0],
                        [1026533.0, 6279746.0],
                        [1033109.0, 6287175.0],
                        [1036126.0, 6293135.0],
                        [1041921.0, 6296903.0],
                        [1048217.0, 6297561.0],
                        [1064350.0, 6309506.0],
                        [1061225.0, 6317546.0],
                        [1066133.0, 6321494.0],
                        [1066468.0, 6326676.0],
                        [1072800.0, 6330433.0],
                        [1073742.0, 6336784.0],
                        [1077560.0, 6342698.0],
                        [1073403.0, 6350462.0],
                        [1060282.0, 6348576.0],
                        [1054180.0, 6344806.0],
                        [1042408.0, 6347558.0],
                        [1034654.0, 6353189.0],
                        [1025044.0, 6356596.0],
                        [1020000.0, 6356725.0],
                        [1018585.0, 6361599.0],
                        [1009696.0, 6370071.0],
                    ]
                ],
                "bbox": (992638.0, 6272482.0, 1077560.0, 6370071.0),
            }
        },
        7: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [753779.0, 6537018.0],
                        [758503.0, 6532097.0],
                        [754369.0, 6520637.0],
                        [760734.0, 6514163.0],
                        [764154.0, 6503988.0],
                        [770440.0, 6500348.0],
                        [774454.0, 6495613.0],
                        [776221.0, 6483674.0],
                        [770335.0, 6479388.0],
                        [770263.0, 6473476.0],
                        [776677.0, 6475557.0],
                        [780259.0, 6472021.0],
                        [792166.0, 6477709.0],
                        [802398.0, 6475469.0],
                        [807029.0, 6471772.0],
                        [806856.0, 6463705.0],
                        [811227.0, 6465007.0],
                        [816370.0, 6460772.0],
                        [825694.0, 6462649.0],
                        [826509.0, 6469123.0],
                        [831517.0, 6473257.0],
                        [837457.0, 6475559.0],
                        [837308.0, 6485557.0],
                        [834511.0, 6489782.0],
                        [829797.0, 6488843.0],
                        [825934.0, 6498617.0],
                        [818524.0, 6498224.0],
                        [809870.0, 6504594.0],
                        [806255.0, 6511913.0],
                        [808147.0, 6518212.0],
                        [807296.0, 6530310.0],
                        [802643.0, 6534285.0],
                        [800220.0, 6540120.0],
                        [801039.0, 6547667.0],
                        [798995.0, 6551156.0],
                        [802035.0, 6559678.0],
                        [810991.0, 6564128.0],
                        [806993.0, 6569822.0],
                        [798969.0, 6562707.0],
                        [797273.0, 6565101.0],
                        [787355.0, 6564848.0],
                        [785138.0, 6567101.0],
                        [776287.0, 6563809.0],
                        [774989.0, 6567447.0],
                        [768620.0, 6568680.0],
                        [769267.0, 6575508.0],
                        [762003.0, 6573092.0],
                        [760949.0, 6561975.0],
                        [763504.0, 6554575.0],
                        [762385.0, 6549859.0],
                        [763887.0, 6544360.0],
                        [761084.0, 6541876.0],
                        [754888.0, 6541809.0],
                        [753779.0, 6537018.0],
                    ]
                ],
                "bbox": (753779.0, 6460772.0, 837457.0, 6575508.0),
            }
        },
        8: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [505760.0, 6248691.0],
                        [506455.0, 6246240.0],
                        [501020.0, 6240519.0],
                        [500952.0, 6237523.0],
                        [491741.0, 6229153.0],
                        [498958.0, 6224762.0],
                        [501199.0, 6216194.0],
                        [506394.0, 6214104.0],
                        [501753.0, 6199863.0],
                        [493715.0, 6200925.0],
                        [492032.0, 6195420.0],
                        [491362.0, 6184305.0],
                        [493119.0, 6181125.0],
                        [509918.0, 6181609.0],
                        [507071.0, 6189939.0],
                        [508439.0, 6196068.0],
                        [512551.0, 6198492.0],
                        [524704.0, 6194181.0],
                        [522873.0, 6204718.0],
                        [526458.0, 6208824.0],
                        [535012.0, 6210452.0],
                        [537386.0, 6214080.0],
                        [535770.0, 6220735.0],
                        [540236.0, 6224348.0],
                        [541984.0, 6228895.0],
                        [547328.0, 6230443.0],
                        [554118.0, 6225953.0],
                        [555809.0, 6233248.0],
                        [562910.0, 6233771.0],
                        [567825.0, 6236309.0],
                        [561407.0, 6244405.0],
                        [565565.0, 6247682.0],
                        [572065.0, 6240985.0],
                        [575081.0, 6235814.0],
                        [582028.0, 6242918.0],
                        [589272.0, 6240503.0],
                        [593467.0, 6242482.0],
                        [598408.0, 6250205.0],
                        [603733.0, 6249863.0],
                        [603060.0, 6255463.0],
                        [607574.0, 6261192.0],
                        [611672.0, 6256714.0],
                        [621363.0, 6260238.0],
                        [620519.0, 6263967.0],
                        [609628.0, 6269734.0],
                        [605052.0, 6276459.0],
                        [594074.0, 6282175.0],
                        [597601.0, 6285124.0],
                        [595737.0, 6291653.0],
                        [592076.0, 6294561.0],
                        [590928.0, 6301074.0],
                        [586632.0, 6302842.0],
                        [583878.0, 6308763.0],
                        [583992.0, 6314318.0],
                        [575242.0, 6309509.0],
                        [567948.0, 6310839.0],
                        [568130.0, 6303820.0],
                        [556606.0, 6298479.0],
                        [550735.0, 6304341.0],
                        [535303.0, 6300840.0],
                        [545681.0, 6284750.0],
                        [550482.0, 6283435.0],
                        [554723.0, 6278961.0],
                        [539594.0, 6265818.0],
                        [536034.0, 6253601.0],
                        [531230.0, 6258583.0],
                        [523428.0, 6259977.0],
                        [514960.0, 6255730.0],
                        [509767.0, 6249274.0],
                        [505760.0, 6248691.0],
                    ]
                ],
                "bbox": (491362.0, 6181125.0, 621363.0, 6314318.0),
            }
        },
        9: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [806993.0, 6569822.0],
                        [808209.0, 6578315.0],
                        [815826.0, 6575242.0],
                        [824653.0, 6575157.0],
                        [830305.0, 6579430.0],
                        [833486.0, 6565745.0],
                        [837322.0, 6565642.0],
                        [839484.0, 6574938.0],
                        [845050.0, 6590960.0],
                        [845229.0, 6595067.0],
                        [848718.0, 6603737.0],
                        [857422.0, 6600366.0],
                        [868758.0, 6603264.0],
                        [869949.0, 6598870.0],
                        [877344.0, 6596683.0],
                        [885325.0, 6599852.0],
                        [880824.0, 6605207.0],
                        [886706.0, 6618161.0],
                        [882575.0, 6628163.0],
                        [880659.0, 6638936.0],
                        [887343.0, 6642371.0],
                        [876345.0, 6647462.0],
                        [872158.0, 6650880.0],
                        [871408.0, 6655745.0],
                        [860161.0, 6652754.0],
                        [855755.0, 6655507.0],
                        [831500.0, 6648015.0],
                        [823470.0, 6650227.0],
                        [813434.0, 6659755.0],
                        [806754.0, 6662035.0],
                        [806631.0, 6665548.0],
                        [795506.0, 6668991.0],
                        [791663.0, 6673495.0],
                        [789554.0, 6672918.0],
                        [784602.0, 6669828.0],
                        [780471.0, 6669516.0],
                        [778810.0, 6664936.0],
                        [781374.0, 6662136.0],
                        [779000.0, 6654397.0],
                        [778932.0, 6646959.0],
                        [783308.0, 6642195.0],
                        [780490.0, 6638662.0],
                        [781064.0, 6632682.0],
                        [775685.0, 6631211.0],
                        [770503.0, 6626941.0],
                        [760920.0, 6622686.0],
                        [756196.0, 6628301.0],
                        [748049.0, 6627896.0],
                        [748734.0, 6623209.0],
                        [753291.0, 6618069.0],
                        [756931.0, 6607772.0],
                        [761463.0, 6602522.0],
                        [763922.0, 6603795.0],
                        [768312.0, 6598303.0],
                        [773476.0, 6599308.0],
                        [776586.0, 6596649.0],
                        [775079.0, 6589273.0],
                        [775913.0, 6580395.0],
                        [769267.0, 6575508.0],
                        [768620.0, 6568680.0],
                        [774989.0, 6567447.0],
                        [776287.0, 6563809.0],
                        [785138.0, 6567101.0],
                        [787355.0, 6564848.0],
                        [797273.0, 6565101.0],
                        [798969.0, 6562707.0],
                        [806993.0, 6569822.0],
                    ]
                ],
                "bbox": (748049.0, 6562707.0, 887343.0, 6673495.0),
            }
        },
        10: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [399495.0, 6830885.0],
                        [398130.0, 6822559.0],
                        [400321.0, 6810723.0],
                        [395852.0, 6803336.0],
                        [398626.0, 6784333.0],
                        [400465.0, 6781914.0],
                        [400197.0, 6773697.0],
                        [394099.0, 6773357.0],
                        [390140.0, 6770978.0],
                        [386941.0, 6760260.0],
                        [382932.0, 6754022.0],
                        [389872.0, 6749698.0],
                        [393110.0, 6750366.0],
                        [402067.0, 6747685.0],
                        [404251.0, 6751414.0],
                        [412442.0, 6746090.0],
                        [419671.0, 6744167.0],
                        [429458.0, 6743442.0],
                        [440863.0, 6746201.0],
                        [446732.0, 6745443.0],
                        [446459.0, 6750432.0],
                        [442128.0, 6753611.0],
                        [448124.0, 6758669.0],
                        [447308.0, 6764356.0],
                        [455060.0, 6767070.0],
                        [451057.0, 6776681.0],
                        [459373.0, 6778102.0],
                        [460615.0, 6783387.0],
                        [458409.0, 6789055.0],
                        [466280.0, 6794064.0],
                        [465298.0, 6799724.0],
                        [467628.0, 6811401.0],
                        [473893.0, 6813452.0],
                        [474394.0, 6821359.0],
                        [467262.0, 6822174.0],
                        [466087.0, 6830999.0],
                        [463434.0, 6833996.0],
                        [457920.0, 6827997.0],
                        [451256.0, 6826715.0],
                        [446687.0, 6829012.0],
                        [441174.0, 6828584.0],
                        [437568.0, 6825109.0],
                        [429868.0, 6822252.0],
                        [422197.0, 6821752.0],
                        [414934.0, 6829326.0],
                        [407934.0, 6831360.0],
                        [404267.0, 6828490.0],
                        [399495.0, 6830885.0],
                    ]
                ],
                "bbox": (382932.0, 6743442.0, 474394.0, 6833996.0),
            }
        },
        11: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [459784.0, 6545825.0],
                        [457303.0, 6539406.0],
                        [458386.0, 6534878.0],
                        [455132.0, 6526680.0],
                        [449713.0, 6524677.0],
                        [445866.0, 6528260.0],
                        [434480.0, 6521530.0],
                        [434607.0, 6512022.0],
                        [436951.0, 6506668.0],
                        [442913.0, 6501478.0],
                        [446521.0, 6493231.0],
                        [444696.0, 6489309.0],
                        [445861.0, 6484044.0],
                        [443202.0, 6478722.0],
                        [447267.0, 6474282.0],
                        [456356.0, 6470307.0],
                        [455660.0, 6465791.0],
                        [461242.0, 6465553.0],
                        [464807.0, 6459164.0],
                        [468806.0, 6462928.0],
                        [474561.0, 6460744.0],
                        [485798.0, 6470195.0],
                        [484697.0, 6477489.0],
                        [489646.0, 6488189.0],
                        [493466.0, 6488572.0],
                        [504657.0, 6496986.0],
                        [505177.0, 6505055.0],
                        [513233.0, 6510903.0],
                        [515635.0, 6515544.0],
                        [522412.0, 6520738.0],
                        [522669.0, 6525452.0],
                        [527825.0, 6527023.0],
                        [534383.0, 6537804.0],
                        [540666.0, 6541823.0],
                        [539507.0, 6547743.0],
                        [531290.0, 6552122.0],
                        [531964.0, 6561069.0],
                        [526089.0, 6562344.0],
                        [521367.0, 6557890.0],
                        [509938.0, 6556936.0],
                        [507509.0, 6562167.0],
                        [499249.0, 6554736.0],
                        [489751.0, 6554866.0],
                        [483517.0, 6558927.0],
                        [473907.0, 6559092.0],
                        [469589.0, 6554692.0],
                        [465641.0, 6555184.0],
                        [464645.0, 6548283.0],
                        [459784.0, 6545825.0],
                    ]
                ],
                "bbox": (434480.0, 6459164.0, 540666.0, 6562344.0),
            }
        },
        12: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [417723.0, 6857101.0],
                        [408651.0, 6861319.0],
                        [399642.0, 6861271.0],
                        [396867.0, 6868913.0],
                        [401493.0, 6871059.0],
                        [411444.0, 6881401.0],
                        [417375.0, 6892264.0],
                        [416434.0, 6899162.0],
                        [413253.0, 6901117.0],
                        [415797.0, 6907383.0],
                        [406933.0, 6907874.0],
                        [400663.0, 6914651.0],
                        [399047.0, 6920143.0],
                        [400759.0, 6925144.0],
                        [397607.0, 6926545.0],
                        [396597.0, 6932989.0],
                        [388320.0, 6946238.0],
                        [388687.0, 6950091.0],
                        [394381.0, 6955463.0],
                        [390587.0, 6963240.0],
                        [381046.0, 6965005.0],
                        [372051.0, 6959988.0],
                        [362230.0, 6961356.0],
                        [343652.0, 6969661.0],
                        [343012.0, 6963991.0],
                        [349037.0, 6960923.0],
                        [349892.0, 6952123.0],
                        [346450.0, 6948841.0],
                        [350114.0, 6938864.0],
                        [350193.0, 6931900.0],
                        [357636.0, 6924163.0],
                        [364780.0, 6906370.0],
                        [363427.0, 6896430.0],
                        [366480.0, 6887586.0],
                        [365701.0, 6878731.0],
                        [362959.0, 6869214.0],
                        [363872.0, 6859932.0],
                        [368562.0, 6852485.0],
                        [376547.0, 6847928.0],
                        [372261.0, 6845150.0],
                        [363309.0, 6845999.0],
                        [366767.0, 6839135.0],
                        [368413.0, 6830443.0],
                        [376536.0, 6826525.0],
                        [384742.0, 6834496.0],
                        [389634.0, 6835155.0],
                        [399495.0, 6830885.0],
                        [404267.0, 6828490.0],
                        [407934.0, 6831360.0],
                        [414934.0, 6829326.0],
                        [421619.0, 6835816.0],
                        [423498.0, 6841841.0],
                        [420581.0, 6852284.0],
                        [417723.0, 6857101.0],
                    ]
                ],
                "bbox": (343012.0, 6826525.0, 423498.0, 6969661.0),
            }
        },
        13: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [590236.0, 6872273.0],
                        [594217.0, 6866696.0],
                        [596554.0, 6858123.0],
                        [595472.0, 6845570.0],
                        [597086.0, 6841311.0],
                        [604791.0, 6831646.0],
                        [608912.0, 6830618.0],
                        [611402.0, 6819097.0],
                        [620311.0, 6818025.0],
                        [623768.0, 6826369.0],
                        [627504.0, 6829048.0],
                        [627800.0, 6838438.0],
                        [632440.0, 6843248.0],
                        [634691.0, 6849194.0],
                        [643163.0, 6853168.0],
                        [637702.0, 6857774.0],
                        [637864.0, 6864035.0],
                        [641404.0, 6867928.0],
                        [641836.0, 6872490.0],
                        [633062.0, 6879807.0],
                        [628641.0, 6878089.0],
                        [622312.0, 6880731.0],
                        [614076.0, 6886919.0],
                        [606678.0, 6883543.0],
                        [603102.0, 6887292.0],
                        [598361.0, 6887345.0],
                        [591984.0, 6886421.0],
                        [587216.0, 6881812.0],
                        [590236.0, 6872273.0],
                    ]
                ],
                "bbox": (587216.0, 6818025.0, 643163.0, 6887345.0),
            }
        },
        14: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [986155.0, 6724947.0],
                        [984341.0, 6721925.0],
                        [977033.0, 6724531.0],
                        [974258.0, 6721322.0],
                        [969354.0, 6721610.0],
                        [967469.0, 6716290.0],
                        [955890.0, 6718838.0],
                        [950918.0, 6717032.0],
                        [945847.0, 6708437.0],
                        [932449.0, 6699390.0],
                        [925719.0, 6696195.0],
                        [922029.0, 6697905.0],
                        [904021.0, 6688452.0],
                        [905832.0, 6683506.0],
                        [912920.0, 6678099.0],
                        [913407.0, 6674258.0],
                        [909981.0, 6669146.0],
                        [913780.0, 6661601.0],
                        [925599.0, 6657600.0],
                        [925393.0, 6653995.0],
                        [928880.0, 6645788.0],
                        [940225.0, 6640408.0],
                        [944367.0, 6635241.0],
                        [942748.0, 6631718.0],
                        [934547.0, 6625608.0],
                        [937312.0, 6621303.0],
                        [933274.0, 6616467.0],
                        [940343.0, 6611180.0],
                        [938167.0, 6613167.0],
                        [950995.0, 6626461.0],
                        [962319.0, 6634781.0],
                        [961533.0, 6640379.0],
                        [963711.0, 6649123.0],
                        [961099.0, 6653277.0],
                        [967437.0, 6658254.0],
                        [976505.0, 6662212.0],
                        [981857.0, 6667617.0],
                        [983508.0, 6674248.0],
                        [992174.0, 6680929.0],
                        [993793.0, 6685149.0],
                        [999048.0, 6690006.0],
                        [1001635.0, 6695987.0],
                        [1006570.0, 6701646.0],
                        [1002944.0, 6704539.0],
                        [995578.0, 6702427.0],
                        [996902.0, 6711079.0],
                        [994087.0, 6717743.0],
                        [993651.0, 6723596.0],
                        [986155.0, 6724947.0],
                    ]
                ],
                "bbox": (904021.0, 6611180.0, 1006570.0, 6724947.0),
            }
        },
        15: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [904250.0, 6825381.0],
                        [904895.0, 6830283.0],
                        [900379.0, 6832469.0],
                        [903561.0, 6837915.0],
                        [902927.0, 6847494.0],
                        [899528.0, 6852809.0],
                        [904125.0, 6862116.0],
                        [901302.0, 6871865.0],
                        [905066.0, 6875501.0],
                        [906433.0, 6884047.0],
                        [905859.0, 6891045.0],
                        [901739.0, 6893315.0],
                        [900172.0, 6898682.0],
                        [901547.0, 6903348.0],
                        [898091.0, 6905220.0],
                        [898652.0, 6915853.0],
                        [895087.0, 6927281.0],
                        [891117.0, 6929633.0],
                        [883576.0, 6926040.0],
                        [878534.0, 6930925.0],
                        [879041.0, 6935896.0],
                        [875512.0, 6946935.0],
                        [873032.0, 6949057.0],
                        [867310.0, 6942452.0],
                        [856532.0, 6942619.0],
                        [853210.0, 6945817.0],
                        [849442.0, 6935263.0],
                        [853548.0, 6926776.0],
                        [847303.0, 6917040.0],
                        [849280.0, 6910207.0],
                        [842092.0, 6905887.0],
                        [841416.0, 6900107.0],
                        [845429.0, 6892400.0],
                        [846791.0, 6883540.0],
                        [848915.0, 6882413.0],
                        [848655.0, 6874905.0],
                        [841874.0, 6870868.0],
                        [838677.0, 6859119.0],
                        [846329.0, 6850924.0],
                        [846365.0, 6844539.0],
                        [847795.0, 6838972.0],
                        [852692.0, 6834838.0],
                        [856191.0, 6834907.0],
                        [863008.0, 6828766.0],
                        [873603.0, 6822775.0],
                        [882722.0, 6816263.0],
                        [893443.0, 6818701.0],
                        [904250.0, 6825381.0],
                    ]
                ],
                "bbox": (838677.0, 6816263.0, 906433.0, 6949057.0),
            }
        },
        16: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [486169.0, 6410676.0],
                        [482141.0, 6421330.0],
                        [476390.0, 6418176.0],
                        [469504.0, 6418931.0],
                        [461839.0, 6422164.0],
                        [466012.0, 6428425.0],
                        [464018.0, 6432038.0],
                        [469719.0, 6445475.0],
                        [467756.0, 6450339.0],
                        [460931.0, 6449393.0],
                        [457280.0, 6451647.0],
                        [452009.0, 6448234.0],
                        [444096.0, 6451365.0],
                        [437492.0, 6457585.0],
                        [433418.0, 6457450.0],
                        [431214.0, 6470295.0],
                        [421098.0, 6472770.0],
                        [418478.0, 6475494.0],
                        [406100.0, 6476140.0],
                        [404682.0, 6481482.0],
                        [399796.0, 6489042.0],
                        [386164.0, 6505245.0],
                        [380995.0, 6504145.0],
                        [375328.0, 6494206.0],
                        [373905.0, 6475097.0],
                        [366594.0, 6427323.0],
                        [365668.0, 6410104.0],
                        [370987.0, 6417061.0],
                        [380337.0, 6407555.0],
                        [382533.0, 6402380.0],
                        [376517.0, 6401861.0],
                        [367865.0, 6404553.0],
                        [364521.0, 6395279.0],
                        [361697.0, 6391399.0],
                        [361774.0, 6383396.0],
                        [373686.0, 6386677.0],
                        [375548.0, 6389853.0],
                        [383562.0, 6383958.0],
                        [381182.0, 6379048.0],
                        [388231.0, 6379327.0],
                        [393973.0, 6376334.0],
                        [403228.0, 6378937.0],
                        [411314.0, 6378206.0],
                        [411174.0, 6373203.0],
                        [418928.0, 6369127.0],
                        [419635.0, 6366255.0],
                        [426389.0, 6364119.0],
                        [430077.0, 6359927.0],
                        [429214.0, 6351421.0],
                        [438523.0, 6349285.0],
                        [442465.0, 6350467.0],
                        [445091.0, 6357381.0],
                        [449164.0, 6352477.0],
                        [455164.0, 6354504.0],
                        [457916.0, 6359953.0],
                        [454032.0, 6364649.0],
                        [462536.0, 6370133.0],
                        [460366.0, 6373402.0],
                        [460343.0, 6383021.0],
                        [464993.0, 6388179.0],
                        [474218.0, 6394851.0],
                        [476701.0, 6399758.0],
                        [472909.0, 6402270.0],
                        [474628.0, 6407547.0],
                        [486169.0, 6410676.0],
                    ]
                ],
                "bbox": (361697.0, 6349285.0, 486169.0, 6505245.0),
            }
        },
        17: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [503908.0, 6928958.0],
                        [498291.0, 6928808.0],
                        [490213.0, 6925480.0],
                        [480923.0, 6917694.0],
                        [474554.0, 6915313.0],
                        [465316.0, 6913926.0],
                        [451821.0, 6920480.0],
                        [444571.0, 6921849.0],
                        [429556.0, 6922759.0],
                        [420281.0, 6924935.0],
                        [414059.0, 6928862.0],
                        [403953.0, 6928646.0],
                        [400759.0, 6925144.0],
                        [399047.0, 6920143.0],
                        [400663.0, 6914651.0],
                        [406933.0, 6907874.0],
                        [415797.0, 6907383.0],
                        [413253.0, 6901117.0],
                        [416434.0, 6899162.0],
                        [417375.0, 6892264.0],
                        [411444.0, 6881401.0],
                        [401493.0, 6871059.0],
                        [396867.0, 6868913.0],
                        [399642.0, 6861271.0],
                        [408651.0, 6861319.0],
                        [417723.0, 6857101.0],
                        [427244.0, 6861949.0],
                        [446005.0, 6869054.0],
                        [449827.0, 6868751.0],
                        [454384.0, 6863198.0],
                        [457511.0, 6867107.0],
                        [469104.0, 6863978.0],
                        [475607.0, 6865852.0],
                        [484257.0, 6870962.0],
                        [489675.0, 6876299.0],
                        [491678.0, 6873174.0],
                        [500558.0, 6877307.0],
                        [504730.0, 6874732.0],
                        [510545.0, 6875400.0],
                        [512505.0, 6881741.0],
                        [508338.0, 6886469.0],
                        [512079.0, 6895331.0],
                        [509950.0, 6897618.0],
                        [509502.0, 6910309.0],
                        [505278.0, 6914121.0],
                        [503908.0, 6928958.0],
                    ]
                ],
                "bbox": (396867.0, 6857101.0, 512505.0, 6928958.0),
            }
        },
        18: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [915394.0, 6762341.0],
                        [919858.0, 6768473.0],
                        [924039.0, 6766004.0],
                        [929025.0, 6772821.0],
                        [935284.0, 6772098.0],
                        [936416.0, 6766072.0],
                        [941726.0, 6764118.0],
                        [951150.0, 6767749.0],
                        [956126.0, 6765947.0],
                        [959909.0, 6759629.0],
                        [964569.0, 6761737.0],
                        [968808.0, 6766577.0],
                        [972278.0, 6762245.0],
                        [982963.0, 6756697.0],
                        [986052.0, 6752778.0],
                        [987691.0, 6753958.0],
                        [993060.0, 6757264.0],
                        [991355.0, 6761730.0],
                        [993983.0, 6773739.0],
                        [998205.0, 6776671.0],
                        [1001795.0, 6789636.0],
                        [1006708.0, 6801985.0],
                        [1011125.0, 6809400.0],
                        [1005338.0, 6811635.0],
                        [1003109.0, 6819572.0],
                        [1004385.0, 6831654.0],
                        [994239.0, 6827289.0],
                        [987769.0, 6820181.0],
                        [982282.0, 6817771.0],
                        [973656.0, 6818945.0],
                        [964951.0, 6823820.0],
                        [963830.0, 6819154.0],
                        [935203.0, 6815546.0],
                        [930831.0, 6810526.0],
                        [919362.0, 6809623.0],
                        [910921.0, 6826770.0],
                        [904250.0, 6825381.0],
                        [893443.0, 6818701.0],
                        [882722.0, 6816263.0],
                        [878315.0, 6811920.0],
                        [883843.0, 6809100.0],
                        [896020.0, 6796835.0],
                        [901295.0, 6794520.0],
                        [899609.0, 6786135.0],
                        [895837.0, 6779219.0],
                        [900794.0, 6778635.0],
                        [906998.0, 6773750.0],
                        [908017.0, 6765028.0],
                        [912655.0, 6766332.0],
                        [915394.0, 6762341.0],
                    ]
                ],
                "bbox": (878315.0, 6752778.0, 1011125.0, 6831654.0),
            }
        },
        19: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [642856.0, 6724786.0],
                        [634456.0, 6720842.0],
                        [639501.0, 6716775.0],
                        [642622.0, 6708275.0],
                        [632231.0, 6698879.0],
                        [634922.0, 6687596.0],
                        [624165.0, 6685496.0],
                        [619951.0, 6688294.0],
                        [617060.0, 6680060.0],
                        [612147.0, 6680536.0],
                        [612309.0, 6675948.0],
                        [607114.0, 6670785.0],
                        [615265.0, 6667454.0],
                        [621646.0, 6669974.0],
                        [627833.0, 6667741.0],
                        [627981.0, 6660408.0],
                        [631131.0, 6657674.0],
                        [629301.0, 6648616.0],
                        [633457.0, 6642556.0],
                        [627496.0, 6637025.0],
                        [632500.0, 6630848.0],
                        [628673.0, 6627353.0],
                        [634340.0, 6622075.0],
                        [637665.0, 6611745.0],
                        [635550.0, 6606712.0],
                        [638969.0, 6598575.0],
                        [634893.0, 6595681.0],
                        [636090.0, 6591904.0],
                        [644784.0, 6591412.0],
                        [645179.0, 6595086.0],
                        [650317.0, 6601560.0],
                        [660370.0, 6603761.0],
                        [664479.0, 6602292.0],
                        [670500.0, 6605989.0],
                        [669145.0, 6615318.0],
                        [677474.0, 6626628.0],
                        [682775.0, 6624336.0],
                        [686837.0, 6626145.0],
                        [696943.0, 6633749.0],
                        [702446.0, 6632753.0],
                        [705186.0, 6641947.0],
                        [703828.0, 6645606.0],
                        [706033.0, 6650549.0],
                        [705753.0, 6657660.0],
                        [701762.0, 6662486.0],
                        [702129.0, 6669867.0],
                        [698614.0, 6680189.0],
                        [698745.0, 6684384.0],
                        [690449.0, 6694237.0],
                        [694642.0, 6704915.0],
                        [690565.0, 6713347.0],
                        [685430.0, 6710473.0],
                        [682544.0, 6713710.0],
                        [678869.0, 6710238.0],
                        [674282.0, 6709782.0],
                        [669909.0, 6717111.0],
                        [661757.0, 6718781.0],
                        [657729.0, 6723417.0],
                        [652875.0, 6720759.0],
                        [646396.0, 6725640.0],
                        [642856.0, 6724786.0],
                    ]
                ],
                "bbox": (607114.0, 6591412.0, 706033.0, 6725640.0),
            }
        },
        20: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [779411.0, 6373918.0],
                        [785084.0, 6367917.0],
                        [783861.0, 6358147.0],
                        [791158.0, 6357807.0],
                        [800494.0, 6352553.0],
                        [802764.0, 6358131.0],
                        [806518.0, 6360956.0],
                        [819866.0, 6361270.0],
                        [824346.0, 6357163.0],
                        [831641.0, 6353746.0],
                        [831614.0, 6360351.0],
                        [832713.0, 6371581.0],
                        [835151.0, 6374038.0],
                        [834538.0, 6384016.0],
                        [835764.0, 6388411.0],
                        [841041.0, 6396655.0],
                        [839715.0, 6410394.0],
                        [844069.0, 6414766.0],
                        [848816.0, 6428161.0],
                        [844782.0, 6435636.0],
                        [842390.0, 6453321.0],
                        [841110.0, 6468164.0],
                        [837457.0, 6475559.0],
                        [831517.0, 6473257.0],
                        [826509.0, 6469123.0],
                        [825694.0, 6462649.0],
                        [816370.0, 6460772.0],
                        [813436.0, 6450097.0],
                        [808187.0, 6450395.0],
                        [808165.0, 6446004.0],
                        [801638.0, 6433950.0],
                        [796332.0, 6429870.0],
                        [791307.0, 6420019.0],
                        [782106.0, 6419872.0],
                        [781273.0, 6415520.0],
                        [774103.0, 6411136.0],
                        [768276.0, 6405343.0],
                        [771844.0, 6388592.0],
                        [775042.0, 6386528.0],
                        [779411.0, 6373918.0],
                    ]
                ],
                "bbox": (768276.0, 6352553.0, 848816.0, 6475559.0),
            }
        },
        21: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [776081.0, 6923412.0],
                        [775403.0, 6934852.0],
                        [777906.0, 6941851.0],
                        [774574.0, 6946610.0],
                        [779463.0, 6948255.0],
                        [781387.0, 6953785.0],
                        [790134.0, 6962730.0],
                        [787172.0, 6965431.0],
                        [789845.0, 6973793.0],
                        [788558.0, 6985051.0],
                        [781905.0, 6987282.0],
                        [778060.0, 6986162.0],
                        [770359.0, 6988977.0],
                        [766237.0, 6992385.0],
                        [753505.0, 6995276.0],
                        [751253.0, 6997000.0],
                        [744014.0, 6992056.0],
                        [739058.0, 6995179.0],
                        [735248.0, 6991264.0],
                        [725313.0, 6993104.0],
                        [720100.0, 6990781.0],
                        [716534.0, 6992565.0],
                        [712391.0, 6990404.0],
                        [713833.0, 6986563.0],
                        [708480.0, 6979518.0],
                        [705667.0, 6969289.0],
                        [708546.0, 6956332.0],
                        [707064.0, 6950845.0],
                        [709520.0, 6938240.0],
                        [706939.0, 6934901.0],
                        [711648.0, 6928031.0],
                        [706805.0, 6926038.0],
                        [706929.0, 6919738.0],
                        [698137.0, 6911415.0],
                        [701957.0, 6908433.0],
                        [704672.0, 6899225.0],
                        [710189.0, 6894765.0],
                        [705248.0, 6890863.0],
                        [712067.0, 6888882.0],
                        [712559.0, 6879371.0],
                        [722321.0, 6872132.0],
                        [724211.0, 6867685.0],
                        [729581.0, 6862815.0],
                        [735603.0, 6861428.0],
                        [738742.0, 6868146.0],
                        [744067.0, 6871735.0],
                        [747254.0, 6882494.0],
                        [743801.0, 6891376.0],
                        [745398.0, 6894771.0],
                        [751361.0, 6898188.0],
                        [747051.0, 6913033.0],
                        [761575.0, 6918670.0],
                        [767112.0, 6923360.0],
                        [775242.0, 6918312.0],
                        [776081.0, 6923412.0],
                    ]
                ],
                "bbox": (698137.0, 6861428.0, 790134.0, 6997000.0),
            }
        },
        22: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [334283.0, 6280551.0],
                        [327832.0, 6270658.0],
                        [324268.0, 6267623.0],
                        [314107.0, 6264613.0],
                        [316186.0, 6255555.0],
                        [321493.0, 6257288.0],
                        [325696.0, 6250142.0],
                        [329513.0, 6253905.0],
                        [334359.0, 6254169.0],
                        [344050.0, 6249226.0],
                        [343580.0, 6242296.0],
                        [340751.0, 6235590.0],
                        [335750.0, 6230517.0],
                        [338499.0, 6226346.0],
                        [344947.0, 6224142.0],
                        [346289.0, 6231519.0],
                        [352412.0, 6225527.0],
                        [355170.0, 6226484.0],
                        [373097.0, 6218255.0],
                        [377794.0, 6214129.0],
                        [388910.0, 6213290.0],
                        [393755.0, 6214784.0],
                        [395170.0, 6206882.0],
                        [409841.0, 6194695.0],
                        [412796.0, 6198151.0],
                        [418372.0, 6194685.0],
                        [422245.0, 6194862.0],
                        [428954.0, 6200122.0],
                        [428150.0, 6207547.0],
                        [431947.0, 6209367.0],
                        [431385.0, 6215423.0],
                        [437107.0, 6220297.0],
                        [440142.0, 6228799.0],
                        [443868.0, 6230545.0],
                        [446410.0, 6236240.0],
                        [454979.0, 6245934.0],
                        [452746.0, 6249402.0],
                        [457777.0, 6262698.0],
                        [451477.0, 6264635.0],
                        [454138.0, 6272903.0],
                        [449863.0, 6280833.0],
                        [443497.0, 6282643.0],
                        [438084.0, 6281592.0],
                        [433057.0, 6279053.0],
                        [424830.0, 6280287.0],
                        [421554.0, 6283529.0],
                        [412501.0, 6278002.0],
                        [403447.0, 6281078.0],
                        [399714.0, 6278404.0],
                        [394475.0, 6281218.0],
                        [385124.0, 6280151.0],
                        [377063.0, 6275535.0],
                        [367753.0, 6276986.0],
                        [362964.0, 6280741.0],
                        [352961.0, 6276001.0],
                        [342731.0, 6276517.0],
                        [338238.0, 6281381.0],
                        [334283.0, 6280551.0],
                    ]
                ],
                "bbox": (314107.0, 6194685.0, 457777.0, 6283529.0),
            }
        },
        23: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [642856.0, 6724786.0],
                        [640167.0, 6731164.0],
                        [620646.0, 6731114.0],
                        [613256.0, 6732409.0],
                        [609792.0, 6727136.0],
                        [605429.0, 6729771.0],
                        [602374.0, 6738361.0],
                        [591237.0, 6742041.0],
                        [590280.0, 6747700.0],
                        [594464.0, 6753836.0],
                        [589886.0, 6759775.0],
                        [589601.0, 6765689.0],
                        [573758.0, 6762848.0],
                        [566408.0, 6764638.0],
                        [559999.0, 6772636.0],
                        [559439.0, 6777275.0],
                        [551727.0, 6777731.0],
                        [551017.0, 6782681.0],
                        [539332.0, 6780273.0],
                        [535875.0, 6773086.0],
                        [539183.0, 6762306.0],
                        [532634.0, 6757689.0],
                        [533549.0, 6751156.0],
                        [528757.0, 6745774.0],
                        [518605.0, 6737439.0],
                        [521086.0, 6735350.0],
                        [530153.0, 6735215.0],
                        [539386.0, 6732046.0],
                        [543743.0, 6727862.0],
                        [552256.0, 6724799.0],
                        [555366.0, 6719856.0],
                        [552945.0, 6716389.0],
                        [559271.0, 6706788.0],
                        [557303.0, 6702930.0],
                        [558309.0, 6696538.0],
                        [557051.0, 6690388.0],
                        [561058.0, 6687382.0],
                        [565759.0, 6689706.0],
                        [569696.0, 6685244.0],
                        [573297.0, 6677558.0],
                        [582631.0, 6683116.0],
                        [593597.0, 6687211.0],
                        [602149.0, 6685865.0],
                        [612147.0, 6680536.0],
                        [617060.0, 6680060.0],
                        [619951.0, 6688294.0],
                        [624165.0, 6685496.0],
                        [634922.0, 6687596.0],
                        [632231.0, 6698879.0],
                        [642622.0, 6708275.0],
                        [639501.0, 6716775.0],
                        [634456.0, 6720842.0],
                        [642856.0, 6724786.0],
                    ]
                ],
                "bbox": (518605.0, 6677558.0, 642856.0, 6782681.0),
            }
        },
        24: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [1001023.0, 6834020.0],
                        [1007603.0, 6833382.0],
                        [1013753.0, 6840510.0],
                        [1015426.0, 6846104.0],
                        [1012123.0, 6851875.0],
                        [1016765.0, 6860562.0],
                        [1004387.0, 6868517.0],
                        [1002983.0, 6863593.0],
                        [997609.0, 6864713.0],
                        [996607.0, 6871102.0],
                        [990508.0, 6874464.0],
                        [991999.0, 6880708.0],
                        [996400.0, 6888804.0],
                        [1000174.0, 6890502.0],
                        [1005257.0, 6886565.0],
                        [1014136.0, 6883372.0],
                        [1016790.0, 6880171.0],
                        [1025429.0, 6883147.0],
                        [1032318.0, 6879884.0],
                        [1035096.0, 6882767.0],
                        [1038624.0, 6893752.0],
                        [1031356.0, 6897552.0],
                        [1023578.0, 6906994.0],
                        [1018182.0, 6905695.0],
                        [1013674.0, 6899390.0],
                        [1009903.0, 6900358.0],
                        [996736.0, 6898172.0],
                        [992282.0, 6906134.0],
                        [986611.0, 6909639.0],
                        [979390.0, 6908077.0],
                        [979652.0, 6901424.0],
                        [972590.0, 6902443.0],
                        [968604.0, 6909024.0],
                        [966935.0, 6915200.0],
                        [959086.0, 6922273.0],
                        [960438.0, 6926520.0],
                        [957718.0, 6930675.0],
                        [948608.0, 6936180.0],
                        [945901.0, 6934734.0],
                        [938917.0, 6936180.0],
                        [935871.0, 6939385.0],
                        [928691.0, 6938466.0],
                        [926408.0, 6934964.0],
                        [916749.0, 6932316.0],
                        [915342.0, 6936492.0],
                        [909643.0, 6936903.0],
                        [913390.0, 6932141.0],
                        [912688.0, 6923499.0],
                        [920628.0, 6907653.0],
                        [917676.0, 6905219.0],
                        [920671.0, 6900380.0],
                        [914011.0, 6893894.0],
                        [914442.0, 6889764.0],
                        [922708.0, 6883240.0],
                        [922735.0, 6879575.0],
                        [928151.0, 6878713.0],
                        [929417.0, 6875078.0],
                        [939656.0, 6875317.0],
                        [945024.0, 6864515.0],
                        [946451.0, 6858744.0],
                        [961206.0, 6856322.0],
                        [964692.0, 6852206.0],
                        [969100.0, 6851558.0],
                        [971773.0, 6847764.0],
                        [976380.0, 6847597.0],
                        [983189.0, 6843093.0],
                        [993487.0, 6842033.0],
                        [1001023.0, 6834020.0],
                    ]
                ],
                "bbox": (909643.0, 6833382.0, 1038624.0, 6939385.0),
            }
        },
        25: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [483517.0, 6558927.0],
                        [489751.0, 6554866.0],
                        [499249.0, 6554736.0],
                        [507509.0, 6562167.0],
                        [509938.0, 6556936.0],
                        [521367.0, 6557890.0],
                        [526089.0, 6562344.0],
                        [531964.0, 6561069.0],
                        [530127.0, 6570272.0],
                        [538896.0, 6578287.0],
                        [546474.0, 6577615.0],
                        [548271.0, 6584464.0],
                        [556218.0, 6586207.0],
                        [559931.0, 6588729.0],
                        [562773.0, 6594026.0],
                        [558128.0, 6596023.0],
                        [557842.0, 6602380.0],
                        [547958.0, 6609437.0],
                        [542426.0, 6611026.0],
                        [538906.0, 6616112.0],
                        [541431.0, 6623495.0],
                        [537221.0, 6629765.0],
                        [533123.0, 6635020.0],
                        [531658.0, 6641492.0],
                        [525272.0, 6647330.0],
                        [524608.0, 6655240.0],
                        [515158.0, 6658631.0],
                        [517389.0, 6653506.0],
                        [510389.0, 6653999.0],
                        [505157.0, 6650870.0],
                        [494833.0, 6656131.0],
                        [494947.0, 6665028.0],
                        [485608.0, 6667284.0],
                        [482922.0, 6672972.0],
                        [478404.0, 6673399.0],
                        [476861.0, 6677880.0],
                        [471947.0, 6677374.0],
                        [469141.0, 6670338.0],
                        [464624.0, 6667352.0],
                        [465537.0, 6662685.0],
                        [471018.0, 6649618.0],
                        [469757.0, 6639127.0],
                        [470952.0, 6628365.0],
                        [467588.0, 6622657.0],
                        [472374.0, 6614809.0],
                        [467690.0, 6607234.0],
                        [468300.0, 6594579.0],
                        [472081.0, 6590232.0],
                        [472141.0, 6584899.0],
                        [475179.0, 6582497.0],
                        [479925.0, 6587264.0],
                        [482897.0, 6584792.0],
                        [478920.0, 6578170.0],
                        [477460.0, 6571967.0],
                        [480502.0, 6565882.0],
                        [485043.0, 6563684.0],
                        [483517.0, 6558927.0],
                    ]
                ],
                "bbox": (464624.0, 6554736.0, 562773.0, 6677880.0),
            }
        },
        26: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [515635.0, 6515544.0],
                        [513233.0, 6510903.0],
                        [505177.0, 6505055.0],
                        [504657.0, 6496986.0],
                        [493466.0, 6488572.0],
                        [489646.0, 6488189.0],
                        [484697.0, 6477489.0],
                        [485798.0, 6470195.0],
                        [474561.0, 6460744.0],
                        [468806.0, 6462928.0],
                        [464807.0, 6459164.0],
                        [460931.0, 6449393.0],
                        [467756.0, 6450339.0],
                        [469719.0, 6445475.0],
                        [464018.0, 6432038.0],
                        [466012.0, 6428425.0],
                        [461839.0, 6422164.0],
                        [469504.0, 6418931.0],
                        [476390.0, 6418176.0],
                        [482141.0, 6421330.0],
                        [486169.0, 6410676.0],
                        [489091.0, 6407815.0],
                        [491196.0, 6399384.0],
                        [495031.0, 6397493.0],
                        [512334.0, 6403639.0],
                        [514373.0, 6400391.0],
                        [527849.0, 6401566.0],
                        [526858.0, 6394383.0],
                        [530971.0, 6390971.0],
                        [539761.0, 6395885.0],
                        [547193.0, 6388334.0],
                        [553597.0, 6395079.0],
                        [553113.0, 6398528.0],
                        [559319.0, 6399932.0],
                        [564402.0, 6403210.0],
                        [570668.0, 6413830.0],
                        [570522.0, 6417080.0],
                        [576964.0, 6421036.0],
                        [574976.0, 6434607.0],
                        [577774.0, 6436765.0],
                        [573095.0, 6446142.0],
                        [566113.0, 6450433.0],
                        [564865.0, 6461779.0],
                        [560988.0, 6465186.0],
                        [568750.0, 6477311.0],
                        [563844.0, 6479160.0],
                        [563452.0, 6484264.0],
                        [553144.0, 6489340.0],
                        [546900.0, 6501981.0],
                        [539039.0, 6503445.0],
                        [529342.0, 6499739.0],
                        [524801.0, 6504583.0],
                        [526849.0, 6509996.0],
                        [515635.0, 6515544.0],
                    ]
                ],
                "bbox": (460931.0, 6388334.0, 577774.0, 6515544.0),
            }
        },
        27: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [877344.0, 6596683.0],
                        [877221.0, 6593352.0],
                        [882664.0, 6589574.0],
                        [882486.0, 6586333.0],
                        [887558.0, 6582363.0],
                        [890444.0, 6576892.0],
                        [895736.0, 6577631.0],
                        [903737.0, 6585593.0],
                        [908880.0, 6582354.0],
                        [909869.0, 6577050.0],
                        [919460.0, 6577537.0],
                        [923919.0, 6580132.0],
                        [929573.0, 6589336.0],
                        [935280.0, 6595262.0],
                        [935835.0, 6600587.0],
                        [941800.0, 6609883.0],
                        [940343.0, 6611180.0],
                        [933274.0, 6616467.0],
                        [937312.0, 6621303.0],
                        [934547.0, 6625608.0],
                        [942748.0, 6631718.0],
                        [944367.0, 6635241.0],
                        [940225.0, 6640408.0],
                        [928880.0, 6645788.0],
                        [925393.0, 6653995.0],
                        [925599.0, 6657600.0],
                        [913780.0, 6661601.0],
                        [909981.0, 6669146.0],
                        [913407.0, 6674258.0],
                        [912920.0, 6678099.0],
                        [905832.0, 6683506.0],
                        [904021.0, 6688452.0],
                        [896736.0, 6687711.0],
                        [890265.0, 6692353.0],
                        [888032.0, 6690633.0],
                        [887407.0, 6682465.0],
                        [884880.0, 6674426.0],
                        [881002.0, 6667371.0],
                        [876191.0, 6666228.0],
                        [875449.0, 6660099.0],
                        [871408.0, 6655745.0],
                        [872158.0, 6650880.0],
                        [876345.0, 6647462.0],
                        [887343.0, 6642371.0],
                        [880659.0, 6638936.0],
                        [882575.0, 6628163.0],
                        [886706.0, 6618161.0],
                        [880824.0, 6605207.0],
                        [885325.0, 6599852.0],
                        [877344.0, 6596683.0],
                    ]
                ],
                "bbox": (871408.0, 6576892.0, 944367.0, 6692353.0),
            }
        },
        28: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [610874.0, 6360746.0],
                        [596271.0, 6358053.0],
                        [592401.0, 6355117.0],
                        [585843.0, 6356520.0],
                        [581893.0, 6352690.0],
                        [570617.0, 6348643.0],
                        [562737.0, 6350024.0],
                        [549131.0, 6360516.0],
                        [545774.0, 6366277.0],
                        [544593.0, 6364542.0],
                        [531660.0, 6365290.0],
                        [530427.0, 6360537.0],
                        [536387.0, 6355171.0],
                        [533815.0, 6350996.0],
                        [529462.0, 6338671.0],
                        [520115.0, 6336256.0],
                        [519117.0, 6332164.0],
                        [521740.0, 6328834.0],
                        [528536.0, 6329188.0],
                        [520187.0, 6318466.0],
                        [528381.0, 6316132.0],
                        [531142.0, 6309545.0],
                        [531445.0, 6300603.0],
                        [535303.0, 6300840.0],
                        [550735.0, 6304341.0],
                        [556606.0, 6298479.0],
                        [568130.0, 6303820.0],
                        [567948.0, 6310839.0],
                        [575242.0, 6309509.0],
                        [583992.0, 6314318.0],
                        [583816.0, 6319290.0],
                        [589156.0, 6320414.0],
                        [595091.0, 6325907.0],
                        [592665.0, 6336179.0],
                        [599594.0, 6336049.0],
                        [602120.0, 6333848.0],
                        [607335.0, 6338978.0],
                        [613603.0, 6341028.0],
                        [619220.0, 6339455.0],
                        [618185.0, 6342854.0],
                        [612268.0, 6345645.0],
                        [617826.0, 6353582.0],
                        [612319.0, 6353954.0],
                        [610874.0, 6360746.0],
                    ]
                ],
                "bbox": (519117.0, 6298479.0, 619220.0, 6366277.0),
            }
        },
        29: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [404251.0, 6660135.0],
                        [413368.0, 6662854.0],
                        [417889.0, 6660618.0],
                        [424993.0, 6661094.0],
                        [429262.0, 6663802.0],
                        [429925.0, 6668493.0],
                        [434893.0, 6670550.0],
                        [442005.0, 6668948.0],
                        [454238.0, 6672319.0],
                        [458546.0, 6672434.0],
                        [464624.0, 6667352.0],
                        [469141.0, 6670338.0],
                        [471947.0, 6677374.0],
                        [476861.0, 6677880.0],
                        [479254.0, 6691029.0],
                        [487846.0, 6706491.0],
                        [490772.0, 6714971.0],
                        [489539.0, 6719725.0],
                        [491931.0, 6726763.0],
                        [485937.0, 6723990.0],
                        [478533.0, 6727039.0],
                        [474490.0, 6731769.0],
                        [469484.0, 6733109.0],
                        [464631.0, 6730466.0],
                        [460339.0, 6732350.0],
                        [457617.0, 6739093.0],
                        [450112.0, 6740918.0],
                        [446732.0, 6745443.0],
                        [440863.0, 6746201.0],
                        [429458.0, 6743442.0],
                        [419671.0, 6744167.0],
                        [412442.0, 6746090.0],
                        [404251.0, 6751414.0],
                        [402067.0, 6747685.0],
                        [393110.0, 6750366.0],
                        [389872.0, 6749698.0],
                        [382932.0, 6754022.0],
                        [382163.0, 6750361.0],
                        [381255.0, 6745617.0],
                        [386987.0, 6740963.0],
                        [389301.0, 6732485.0],
                        [398982.0, 6728603.0],
                        [395905.0, 6725901.0],
                        [386462.0, 6727467.0],
                        [387234.0, 6720426.0],
                        [401401.0, 6717774.0],
                        [402393.0, 6705897.0],
                        [396890.0, 6703794.0],
                        [385609.0, 6704471.0],
                        [375470.0, 6701871.0],
                        [374867.0, 6698071.0],
                        [378391.0, 6691949.0],
                        [382659.0, 6691801.0],
                        [384839.0, 6683588.0],
                        [379159.0, 6679552.0],
                        [379298.0, 6676338.0],
                        [387784.0, 6670618.0],
                        [385082.0, 6667087.0],
                        [392012.0, 6664073.0],
                        [401283.0, 6663962.0],
                        [404251.0, 6660135.0],
                    ]
                ],
                "bbox": (374867.0, 6660135.0, 491931.0, 6754022.0),
            }
        },
        30: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [863140.0, 6525920.0],
                        [849404.0, 6524528.0],
                        [844536.0, 6535588.0],
                        [833933.0, 6540287.0],
                        [835850.0, 6543054.0],
                        [834501.0, 6551198.0],
                        [835220.0, 6556428.0],
                        [838741.0, 6561678.0],
                        [837322.0, 6565642.0],
                        [833486.0, 6565745.0],
                        [830305.0, 6579430.0],
                        [824653.0, 6575157.0],
                        [815826.0, 6575242.0],
                        [808209.0, 6578315.0],
                        [806993.0, 6569822.0],
                        [810991.0, 6564128.0],
                        [802035.0, 6559678.0],
                        [798995.0, 6551156.0],
                        [801039.0, 6547667.0],
                        [800220.0, 6540120.0],
                        [802643.0, 6534285.0],
                        [807296.0, 6530310.0],
                        [808147.0, 6518212.0],
                        [806255.0, 6511913.0],
                        [809870.0, 6504594.0],
                        [818524.0, 6498224.0],
                        [825934.0, 6498617.0],
                        [829797.0, 6488843.0],
                        [834511.0, 6489782.0],
                        [837308.0, 6485557.0],
                        [845846.0, 6493277.0],
                        [841056.0, 6498593.0],
                        [848198.0, 6502534.0],
                        [858527.0, 6506228.0],
                        [865784.0, 6514252.0],
                        [862850.0, 6517700.0],
                        [863140.0, 6525920.0],
                    ]
                ],
                "bbox": (798995.0, 6485557.0, 865784.0, 6579430.0),
            }
        },
        31: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [606642.0, 6924066.0],
                        [602657.0, 6922993.0],
                        [596650.0, 6927678.0],
                        [584841.0, 6929723.0],
                        [579971.0, 6928884.0],
                        [574422.0, 6917678.0],
                        [566281.0, 6917384.0],
                        [564288.0, 6913853.0],
                        [558013.0, 6912674.0],
                        [555352.0, 6907664.0],
                        [548914.0, 6917432.0],
                        [548813.0, 6922655.0],
                        [537794.0, 6926754.0],
                        [530512.0, 6925008.0],
                        [524567.0, 6928708.0],
                        [520591.0, 6933755.0],
                        [506978.0, 6930086.0],
                        [503908.0, 6928958.0],
                        [505278.0, 6914121.0],
                        [509502.0, 6910309.0],
                        [509950.0, 6897618.0],
                        [512079.0, 6895331.0],
                        [508338.0, 6886469.0],
                        [512505.0, 6881741.0],
                        [510545.0, 6875400.0],
                        [508448.0, 6870995.0],
                        [513552.0, 6867418.0],
                        [520512.0, 6866728.0],
                        [524600.0, 6861683.0],
                        [533488.0, 6856087.0],
                        [536389.0, 6850906.0],
                        [533024.0, 6847407.0],
                        [539112.0, 6843338.0],
                        [543803.0, 6848251.0],
                        [555241.0, 6849633.0],
                        [566604.0, 6854037.0],
                        [576482.0, 6852521.0],
                        [580772.0, 6855857.0],
                        [579784.0, 6860594.0],
                        [587701.0, 6866890.0],
                        [590236.0, 6872273.0],
                        [587216.0, 6881812.0],
                        [591984.0, 6886421.0],
                        [598361.0, 6887345.0],
                        [601892.0, 6893098.0],
                        [603501.0, 6902160.0],
                        [605624.0, 6904387.0],
                        [606128.0, 6907968.0],
                        [610931.0, 6911811.0],
                        [609871.0, 6919444.0],
                        [606642.0, 6924066.0],
                    ]
                ],
                "bbox": (503908.0, 6843338.0, 610931.0, 6933755.0),
            }
        },
        32: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [619220.0, 6339455.0],
                        [628552.0, 6344336.0],
                        [632260.0, 6344664.0],
                        [651092.0, 6332910.0],
                        [652742.0, 6328715.0],
                        [656723.0, 6328192.0],
                        [661724.0, 6320369.0],
                        [659810.0, 6316970.0],
                        [664259.0, 6313698.0],
                        [664946.0, 6304799.0],
                        [674324.0, 6293934.0],
                        [679276.0, 6292258.0],
                        [684993.0, 6295873.0],
                        [693643.0, 6293013.0],
                        [694796.0, 6288406.0],
                        [698819.0, 6289943.0],
                        [704898.0, 6288229.0],
                        [704815.0, 6303628.0],
                        [716344.0, 6301672.0],
                        [722102.0, 6311077.0],
                        [728786.0, 6312824.0],
                        [736140.0, 6324746.0],
                        [726344.0, 6331169.0],
                        [726879.0, 6340020.0],
                        [729879.0, 6341376.0],
                        [728598.0, 6344387.0],
                        [718362.0, 6347498.0],
                        [709994.0, 6353946.0],
                        [712305.0, 6356606.0],
                        [709537.0, 6364934.0],
                        [710800.0, 6372878.0],
                        [705449.0, 6378272.0],
                        [706639.0, 6384441.0],
                        [698547.0, 6393953.0],
                        [693924.0, 6403282.0],
                        [694820.0, 6408897.0],
                        [690477.0, 6411547.0],
                        [688126.0, 6419157.0],
                        [684359.0, 6419403.0],
                        [679353.0, 6426925.0],
                        [676829.0, 6423158.0],
                        [668604.0, 6416079.0],
                        [668313.0, 6410487.0],
                        [665404.0, 6408608.0],
                        [664849.0, 6402561.0],
                        [655192.0, 6393469.0],
                        [646639.0, 6396959.0],
                        [637228.0, 6394175.0],
                        [637122.0, 6391031.0],
                        [632813.0, 6386229.0],
                        [625170.0, 6387538.0],
                        [615061.0, 6379167.0],
                        [607709.0, 6375883.0],
                        [610874.0, 6360746.0],
                        [612319.0, 6353954.0],
                        [617826.0, 6353582.0],
                        [612268.0, 6345645.0],
                        [618185.0, 6342854.0],
                        [619220.0, 6339455.0],
                    ]
                ],
                "bbox": (607709.0, 6288229.0, 736140.0, 6426925.0),
            }
        },
        33: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [636090.0, 6591904.0],
                        [630016.0, 6590287.0],
                        [622684.0, 6592821.0],
                        [609355.0, 6592908.0],
                        [602234.0, 6588456.0],
                        [595863.0, 6588324.0],
                        [586530.0, 6592902.0],
                        [578129.0, 6584256.0],
                        [580236.0, 6579624.0],
                        [574730.0, 6569689.0],
                        [580656.0, 6565605.0],
                        [587795.0, 6549437.0],
                        [586335.0, 6540289.0],
                        [592200.0, 6538029.0],
                        [591561.0, 6529605.0],
                        [597279.0, 6527024.0],
                        [604752.0, 6527760.0],
                        [613074.0, 6522626.0],
                        [612524.0, 6515422.0],
                        [614305.0, 6511565.0],
                        [623634.0, 6517796.0],
                        [631053.0, 6514297.0],
                        [635791.0, 6515072.0],
                        [637278.0, 6511373.0],
                        [648450.0, 6509221.0],
                        [649366.0, 6512420.0],
                        [656101.0, 6511235.0],
                        [660507.0, 6515469.0],
                        [656022.0, 6519074.0],
                        [652489.0, 6525488.0],
                        [656753.0, 6529414.0],
                        [660605.0, 6529501.0],
                        [668605.0, 6543382.0],
                        [665172.0, 6557241.0],
                        [666451.0, 6560450.0],
                        [662729.0, 6571178.0],
                        [659880.0, 6575828.0],
                        [653217.0, 6581299.0],
                        [647921.0, 6581264.0],
                        [644784.0, 6591412.0],
                        [636090.0, 6591904.0],
                    ]
                ],
                "bbox": (574730.0, 6509221.0, 668605.0, 6592908.0),
            }
        },
        34: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [695265.0, 6784783.0],
                        [685053.0, 6785343.0],
                        [678166.0, 6780535.0],
                        [665644.0, 6782354.0],
                        [663338.0, 6793247.0],
                        [657236.0, 6795707.0],
                        [655721.0, 6802434.0],
                        [653275.0, 6801114.0],
                        [639308.0, 6801806.0],
                        [625387.0, 6798948.0],
                        [623220.0, 6795517.0],
                        [615539.0, 6776549.0],
                        [606511.0, 6774717.0],
                        [597558.0, 6774986.0],
                        [595216.0, 6770968.0],
                        [589165.0, 6770859.0],
                        [589601.0, 6765689.0],
                        [589886.0, 6759775.0],
                        [594464.0, 6753836.0],
                        [590280.0, 6747700.0],
                        [591237.0, 6742041.0],
                        [602374.0, 6738361.0],
                        [605429.0, 6729771.0],
                        [609792.0, 6727136.0],
                        [613256.0, 6732409.0],
                        [620646.0, 6731114.0],
                        [640167.0, 6731164.0],
                        [642856.0, 6724786.0],
                        [646396.0, 6725640.0],
                        [652875.0, 6720759.0],
                        [657729.0, 6723417.0],
                        [661757.0, 6718781.0],
                        [669909.0, 6717111.0],
                        [674282.0, 6709782.0],
                        [678869.0, 6710238.0],
                        [682544.0, 6713710.0],
                        [685430.0, 6710473.0],
                        [690565.0, 6713347.0],
                        [693556.0, 6718403.0],
                        [698236.0, 6718784.0],
                        [695341.0, 6722058.0],
                        [694231.0, 6731133.0],
                        [689391.0, 6734632.0],
                        [689262.0, 6740069.0],
                        [695135.0, 6740319.0],
                        [701782.0, 6742907.0],
                        [700769.0, 6756036.0],
                        [703743.0, 6756785.0],
                        [709585.0, 6763406.0],
                        [706580.0, 6772621.0],
                        [703759.0, 6774663.0],
                        [702192.0, 6781427.0],
                        [695265.0, 6784783.0],
                    ]
                ],
                "bbox": (589165.0, 6709782.0, 709585.0, 6802434.0),
            }
        },
        35: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [986052.0, 6752778.0],
                        [982963.0, 6756697.0],
                        [972278.0, 6762245.0],
                        [968808.0, 6766577.0],
                        [964569.0, 6761737.0],
                        [959909.0, 6759629.0],
                        [956126.0, 6765947.0],
                        [951150.0, 6767749.0],
                        [941726.0, 6764118.0],
                        [936416.0, 6766072.0],
                        [935284.0, 6772098.0],
                        [929025.0, 6772821.0],
                        [924039.0, 6766004.0],
                        [919858.0, 6768473.0],
                        [915394.0, 6762341.0],
                        [910900.0, 6755744.0],
                        [906433.0, 6754603.0],
                        [901288.0, 6749897.0],
                        [900414.0, 6745482.0],
                        [902985.0, 6741752.0],
                        [901778.0, 6735043.0],
                        [894824.0, 6733348.0],
                        [886220.0, 6734507.0],
                        [880542.0, 6733086.0],
                        [878352.0, 6725366.0],
                        [882147.0, 6728546.0],
                        [886108.0, 6726069.0],
                        [887785.0, 6718944.0],
                        [884206.0, 6713509.0],
                        [880595.0, 6713705.0],
                        [884919.0, 6701062.0],
                        [890265.0, 6692353.0],
                        [896736.0, 6687711.0],
                        [904021.0, 6688452.0],
                        [922029.0, 6697905.0],
                        [925719.0, 6696195.0],
                        [932449.0, 6699390.0],
                        [945847.0, 6708437.0],
                        [950918.0, 6717032.0],
                        [955890.0, 6718838.0],
                        [967469.0, 6716290.0],
                        [969354.0, 6721610.0],
                        [974258.0, 6721322.0],
                        [977033.0, 6724531.0],
                        [984341.0, 6721925.0],
                        [986155.0, 6724947.0],
                        [983826.0, 6731782.0],
                        [981470.0, 6745308.0],
                        [986052.0, 6752778.0],
                    ]
                ],
                "bbox": (878352.0, 6687711.0, 986155.0, 6772821.0),
            }
        },
        36: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [753779.0, 6537018.0],
                        [745523.0, 6547324.0],
                        [735860.0, 6546571.0],
                        [732386.0, 6552810.0],
                        [728730.0, 6550656.0],
                        [712873.0, 6551781.0],
                        [697819.0, 6557998.0],
                        [693029.0, 6567076.0],
                        [695157.0, 6571452.0],
                        [688886.0, 6572856.0],
                        [685369.0, 6566281.0],
                        [679424.0, 6569309.0],
                        [672252.0, 6557694.0],
                        [666451.0, 6560450.0],
                        [665172.0, 6557241.0],
                        [668605.0, 6543382.0],
                        [660605.0, 6529501.0],
                        [656753.0, 6529414.0],
                        [652489.0, 6525488.0],
                        [656022.0, 6519074.0],
                        [660507.0, 6515469.0],
                        [663288.0, 6509261.0],
                        [662164.0, 6504560.0],
                        [658322.0, 6500286.0],
                        [662264.0, 6494997.0],
                        [661594.0, 6486680.0],
                        [672750.0, 6482640.0],
                        [677684.0, 6475835.0],
                        [688449.0, 6477003.0],
                        [693859.0, 6474045.0],
                        [695974.0, 6467752.0],
                        [701310.0, 6465307.0],
                        [708104.0, 6472781.0],
                        [713160.0, 6472555.0],
                        [722742.0, 6478087.0],
                        [735734.0, 6477978.0],
                        [739202.0, 6481030.0],
                        [749018.0, 6471190.0],
                        [752222.0, 6475420.0],
                        [760473.0, 6472496.0],
                        [763248.0, 6477235.0],
                        [770263.0, 6473476.0],
                        [770335.0, 6479388.0],
                        [776221.0, 6483674.0],
                        [774454.0, 6495613.0],
                        [770440.0, 6500348.0],
                        [764154.0, 6503988.0],
                        [760734.0, 6514163.0],
                        [754369.0, 6520637.0],
                        [758503.0, 6532097.0],
                        [753779.0, 6537018.0],
                    ]
                ],
                "bbox": (652489.0, 6465307.0, 776221.0, 6572856.0),
            }
        },
        37: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [583992.0, 6314318.0],
                        [583878.0, 6308763.0],
                        [586632.0, 6302842.0],
                        [590928.0, 6301074.0],
                        [592076.0, 6294561.0],
                        [595737.0, 6291653.0],
                        [597601.0, 6285124.0],
                        [594074.0, 6282175.0],
                        [605052.0, 6276459.0],
                        [609628.0, 6269734.0],
                        [620519.0, 6263967.0],
                        [621363.0, 6260238.0],
                        [624846.0, 6255615.0],
                        [637154.0, 6259493.0],
                        [640013.0, 6262596.0],
                        [651680.0, 6257911.0],
                        [654081.0, 6259728.0],
                        [664821.0, 6258302.0],
                        [668090.0, 6259189.0],
                        [672444.0, 6264623.0],
                        [672394.0, 6269695.0],
                        [668935.0, 6274250.0],
                        [671212.0, 6283521.0],
                        [676690.0, 6282993.0],
                        [680178.0, 6279485.0],
                        [693221.0, 6284557.0],
                        [694796.0, 6288406.0],
                        [693643.0, 6293013.0],
                        [684993.0, 6295873.0],
                        [679276.0, 6292258.0],
                        [674324.0, 6293934.0],
                        [664946.0, 6304799.0],
                        [664259.0, 6313698.0],
                        [659810.0, 6316970.0],
                        [661724.0, 6320369.0],
                        [656723.0, 6328192.0],
                        [652742.0, 6328715.0],
                        [651092.0, 6332910.0],
                        [632260.0, 6344664.0],
                        [628552.0, 6344336.0],
                        [619220.0, 6339455.0],
                        [613603.0, 6341028.0],
                        [607335.0, 6338978.0],
                        [602120.0, 6333848.0],
                        [599594.0, 6336049.0],
                        [592665.0, 6336179.0],
                        [595091.0, 6325907.0],
                        [589156.0, 6320414.0],
                        [583816.0, 6319290.0],
                        [583992.0, 6314318.0],
                    ]
                ],
                "bbox": (583816.0, 6255615.0, 694796.0, 6344664.0),
            }
        },
        38: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [612314.0, 6962826.0],
                        [608347.0, 6970745.0],
                        [605014.0, 6980736.0],
                        [589321.0, 6994349.0],
                        [583880.0, 6997485.0],
                        [570354.0, 6986979.0],
                        [561015.0, 6982476.0],
                        [553619.0, 6982021.0],
                        [539569.0, 6977082.0],
                        [525341.0, 6975010.0],
                        [506777.0, 6963564.0],
                        [498821.0, 6960824.0],
                        [494437.0, 6953650.0],
                        [487453.0, 6938754.0],
                        [491281.0, 6933115.0],
                        [506978.0, 6930086.0],
                        [520591.0, 6933755.0],
                        [524567.0, 6928708.0],
                        [530512.0, 6925008.0],
                        [537794.0, 6926754.0],
                        [548813.0, 6922655.0],
                        [548914.0, 6917432.0],
                        [555352.0, 6907664.0],
                        [558013.0, 6912674.0],
                        [564288.0, 6913853.0],
                        [566281.0, 6917384.0],
                        [574422.0, 6917678.0],
                        [579971.0, 6928884.0],
                        [584841.0, 6929723.0],
                        [596650.0, 6927678.0],
                        [602657.0, 6922993.0],
                        [606642.0, 6924066.0],
                        [610545.0, 6930338.0],
                        [607205.0, 6935073.0],
                        [609123.0, 6937659.0],
                        [606321.0, 6945902.0],
                        [607933.0, 6953359.0],
                        [605401.0, 6955866.0],
                        [612314.0, 6962826.0],
                    ]
                ],
                "bbox": (487453.0, 6907664.0, 612314.0, 6997485.0),
            }
        },
        39: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [823302.0, 6827043.0],
                        [822210.0, 6820200.0],
                        [827022.0, 6811887.0],
                        [834073.0, 6806764.0],
                        [837935.0, 6797546.0],
                        [837598.0, 6783989.0],
                        [835407.0, 6779651.0],
                        [828770.0, 6781171.0],
                        [825768.0, 6776095.0],
                        [827043.0, 6770243.0],
                        [833398.0, 6769005.0],
                        [833316.0, 6764154.0],
                        [838226.0, 6763382.0],
                        [846087.0, 6753627.0],
                        [848388.0, 6746302.0],
                        [846031.0, 6742315.0],
                        [848455.0, 6733791.0],
                        [858300.0, 6729814.0],
                        [863148.0, 6730266.0],
                        [868188.0, 6726357.0],
                        [878352.0, 6725366.0],
                        [880542.0, 6733086.0],
                        [886220.0, 6734507.0],
                        [894824.0, 6733348.0],
                        [901778.0, 6735043.0],
                        [902985.0, 6741752.0],
                        [900414.0, 6745482.0],
                        [901288.0, 6749897.0],
                        [906433.0, 6754603.0],
                        [910900.0, 6755744.0],
                        [915394.0, 6762341.0],
                        [912655.0, 6766332.0],
                        [908017.0, 6765028.0],
                        [906998.0, 6773750.0],
                        [900794.0, 6778635.0],
                        [895837.0, 6779219.0],
                        [899609.0, 6786135.0],
                        [901295.0, 6794520.0],
                        [896020.0, 6796835.0],
                        [883843.0, 6809100.0],
                        [878315.0, 6811920.0],
                        [882722.0, 6816263.0],
                        [873603.0, 6822775.0],
                        [863008.0, 6828766.0],
                        [856191.0, 6834907.0],
                        [852692.0, 6834838.0],
                        [847795.0, 6838972.0],
                        [846365.0, 6844539.0],
                        [840675.0, 6844876.0],
                        [835676.0, 6840415.0],
                        [836281.0, 6837534.0],
                        [830469.0, 6833515.0],
                        [832829.0, 6827018.0],
                        [823302.0, 6827043.0],
                    ]
                ],
                "bbox": (822210.0, 6725366.0, 915394.0, 6844876.0),
            }
        },
        40: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [831641.0, 6353746.0],
                        [824346.0, 6357163.0],
                        [819866.0, 6361270.0],
                        [806518.0, 6360956.0],
                        [802764.0, 6358131.0],
                        [800494.0, 6352553.0],
                        [791158.0, 6357807.0],
                        [783861.0, 6358147.0],
                        [785084.0, 6367917.0],
                        [779411.0, 6373918.0],
                        [775211.0, 6367175.0],
                        [770571.0, 6365836.0],
                        [775941.0, 6359276.0],
                        [773621.0, 6356674.0],
                        [775968.0, 6350322.0],
                        [776174.0, 6341769.0],
                        [768987.0, 6337162.0],
                        [763780.0, 6336809.0],
                        [754256.0, 6342872.0],
                        [750995.0, 6342037.0],
                        [750700.0, 6336169.0],
                        [735189.0, 6336775.0],
                        [729879.0, 6341376.0],
                        [726879.0, 6340020.0],
                        [726344.0, 6331169.0],
                        [736140.0, 6324746.0],
                        [728786.0, 6312824.0],
                        [734096.0, 6312602.0],
                        [742673.0, 6306434.0],
                        [746263.0, 6307559.0],
                        [750088.0, 6313306.0],
                        [758340.0, 6319159.0],
                        [763184.0, 6318967.0],
                        [766255.0, 6313712.0],
                        [766790.0, 6307842.0],
                        [773726.0, 6310049.0],
                        [780634.0, 6301895.0],
                        [792244.0, 6293431.0],
                        [796379.0, 6284647.0],
                        [792922.0, 6276955.0],
                        [788998.0, 6273432.0],
                        [792112.0, 6270978.0],
                        [792074.0, 6265425.0],
                        [799607.0, 6263119.0],
                        [800175.0, 6267290.0],
                        [813903.0, 6274579.0],
                        [815212.0, 6281222.0],
                        [819908.0, 6290041.0],
                        [828463.0, 6288899.0],
                        [833168.0, 6302217.0],
                        [831988.0, 6308980.0],
                        [839658.0, 6315433.0],
                        [847769.0, 6322554.0],
                        [843231.0, 6331183.0],
                        [836689.0, 6335330.0],
                        [837656.0, 6344658.0],
                        [831641.0, 6353746.0],
                    ]
                ],
                "bbox": (726344.0, 6263119.0, 847769.0, 6373918.0),
            }
        },
        41: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [1011125.0, 6809400.0],
                        [1016850.0, 6809125.0],
                        [1022638.0, 6802981.0],
                        [1032428.0, 6798669.0],
                        [1032222.0, 6793924.0],
                        [1036067.0, 6790258.0],
                        [1040408.0, 6789869.0],
                        [1046450.0, 6801750.0],
                        [1048508.0, 6811325.0],
                        [1051141.0, 6812923.0],
                        [1050303.0, 6822049.0],
                        [1052109.0, 6831679.0],
                        [1054677.0, 6834498.0],
                        [1054069.0, 6843233.0],
                        [1056359.0, 6848835.0],
                        [1064869.0, 6858273.0],
                        [1065247.0, 6862338.0],
                        [1074008.0, 6868775.0],
                        [1077114.0, 6879387.0],
                        [1081448.0, 6887616.0],
                        [1072364.0, 6888601.0],
                        [1060439.0, 6895482.0],
                        [1055798.0, 6892558.0],
                        [1050607.0, 6895578.0],
                        [1045763.0, 6893084.0],
                        [1038624.0, 6893752.0],
                        [1035096.0, 6882767.0],
                        [1032318.0, 6879884.0],
                        [1025429.0, 6883147.0],
                        [1016790.0, 6880171.0],
                        [1014136.0, 6883372.0],
                        [1005257.0, 6886565.0],
                        [1000174.0, 6890502.0],
                        [996400.0, 6888804.0],
                        [991999.0, 6880708.0],
                        [990508.0, 6874464.0],
                        [996607.0, 6871102.0],
                        [997609.0, 6864713.0],
                        [1002983.0, 6863593.0],
                        [1004387.0, 6868517.0],
                        [1016765.0, 6860562.0],
                        [1012123.0, 6851875.0],
                        [1015426.0, 6846104.0],
                        [1013753.0, 6840510.0],
                        [1007603.0, 6833382.0],
                        [1001023.0, 6834020.0],
                        [1004385.0, 6831654.0],
                        [1003109.0, 6819572.0],
                        [1005338.0, 6811635.0],
                        [1011125.0, 6809400.0],
                    ]
                ],
                "bbox": (990508.0, 6789869.0, 1081448.0, 6895578.0),
            }
        },
        42: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [719561.0, 6234874.0],
                        [716336.0, 6238907.0],
                        [711963.0, 6238657.0],
                        [700385.0, 6243263.0],
                        [700353.0, 6246775.0],
                        [686427.0, 6247043.0],
                        [681495.0, 6239871.0],
                        [667653.0, 6244475.0],
                        [662679.0, 6249784.0],
                        [664821.0, 6258302.0],
                        [654081.0, 6259728.0],
                        [651680.0, 6257911.0],
                        [640013.0, 6262596.0],
                        [637154.0, 6259493.0],
                        [624846.0, 6255615.0],
                        [621363.0, 6260238.0],
                        [611672.0, 6256714.0],
                        [607574.0, 6261192.0],
                        [603060.0, 6255463.0],
                        [603733.0, 6249863.0],
                        [598408.0, 6250205.0],
                        [593467.0, 6242482.0],
                        [596256.0, 6236985.0],
                        [595072.0, 6233132.0],
                        [604330.0, 6228108.0],
                        [614442.0, 6225039.0],
                        [613663.0, 6217624.0],
                        [617210.0, 6214637.0],
                        [617137.0, 6197372.0],
                        [608402.0, 6195450.0],
                        [610538.0, 6185966.0],
                        [614361.0, 6182470.0],
                        [622733.0, 6184536.0],
                        [632373.0, 6176261.0],
                        [631545.0, 6174170.0],
                        [644388.0, 6178927.0],
                        [647326.0, 6181831.0],
                        [644930.0, 6193342.0],
                        [655189.0, 6193206.0],
                        [659825.0, 6194610.0],
                        [675246.0, 6192375.0],
                        [678721.0, 6193157.0],
                        [682753.0, 6199641.0],
                        [688989.0, 6202113.0],
                        [693222.0, 6198384.0],
                        [703560.0, 6193187.0],
                        [704952.0, 6201963.0],
                        [703499.0, 6206754.0],
                        [709550.0, 6222514.0],
                        [719561.0, 6234874.0],
                    ]
                ],
                "bbox": (593467.0, 6174170.0, 719561.0, 6262596.0),
            }
        },
        43: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [695265.0, 6784783.0],
                        [700383.0, 6789702.0],
                        [703237.0, 6796855.0],
                        [703668.0, 6806647.0],
                        [712422.0, 6807964.0],
                        [727042.0, 6808060.0],
                        [730707.0, 6810078.0],
                        [728998.0, 6813854.0],
                        [731262.0, 6826009.0],
                        [737148.0, 6833964.0],
                        [740952.0, 6835710.0],
                        [733915.0, 6839303.0],
                        [735102.0, 6844463.0],
                        [732040.0, 6850473.0],
                        [735603.0, 6861428.0],
                        [729581.0, 6862815.0],
                        [724211.0, 6867685.0],
                        [722321.0, 6872132.0],
                        [712559.0, 6879371.0],
                        [712067.0, 6888882.0],
                        [705248.0, 6890863.0],
                        [700622.0, 6887964.0],
                        [680633.0, 6884542.0],
                        [673222.0, 6889902.0],
                        [670084.0, 6886723.0],
                        [667303.0, 6878971.0],
                        [670882.0, 6870004.0],
                        [667928.0, 6865519.0],
                        [670056.0, 6856451.0],
                        [671600.0, 6851283.0],
                        [668472.0, 6843626.0],
                        [665649.0, 6839786.0],
                        [666363.0, 6833836.0],
                        [663045.0, 6825117.0],
                        [663371.0, 6815578.0],
                        [665496.0, 6812431.0],
                        [657768.0, 6807009.0],
                        [655721.0, 6802434.0],
                        [657236.0, 6795707.0],
                        [663338.0, 6793247.0],
                        [665644.0, 6782354.0],
                        [678166.0, 6780535.0],
                        [685053.0, 6785343.0],
                        [695265.0, 6784783.0],
                    ]
                ],
                "bbox": (655721.0, 6780535.0, 740952.0, 6890863.0),
            }
        },
        44: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [706470.0, 6995042.0],
                        [691147.0, 6993758.0],
                        [689814.0, 6997681.0],
                        [669642.0, 7001764.0],
                        [667892.0, 7004913.0],
                        [662547.0, 7004433.0],
                        [658067.0, 7000416.0],
                        [656343.0, 7006536.0],
                        [663948.0, 7010834.0],
                        [660837.0, 7014835.0],
                        [654991.0, 7012592.0],
                        [648121.0, 7014552.0],
                        [635372.0, 7012312.0],
                        [634147.0, 7017146.0],
                        [624801.0, 7021563.0],
                        [622831.0, 7024302.0],
                        [612989.0, 7030025.0],
                        [605232.0, 7026713.0],
                        [603193.0, 7029103.0],
                        [596733.0, 7029014.0],
                        [595668.0, 7021497.0],
                        [599348.0, 7018454.0],
                        [591189.0, 7008466.0],
                        [589294.0, 7002423.0],
                        [583880.0, 6997485.0],
                        [589321.0, 6994349.0],
                        [605014.0, 6980736.0],
                        [608347.0, 6970745.0],
                        [612314.0, 6962826.0],
                        [620154.0, 6956108.0],
                        [623025.0, 6958366.0],
                        [636565.0, 6954687.0],
                        [645659.0, 6956103.0],
                        [654712.0, 6950974.0],
                        [664208.0, 6949037.0],
                        [669006.0, 6944292.0],
                        [676176.0, 6942514.0],
                        [677528.0, 6947426.0],
                        [685504.0, 6946842.0],
                        [685089.0, 6950779.0],
                        [693916.0, 6955962.0],
                        [708546.0, 6956332.0],
                        [705667.0, 6969289.0],
                        [708480.0, 6979518.0],
                        [713833.0, 6986563.0],
                        [712391.0, 6990404.0],
                        [706470.0, 6995042.0],
                    ]
                ],
                "bbox": (583880.0, 6942514.0, 713833.0, 7030025.0),
            }
        },
        45: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [708104.0, 6472781.0],
                        [709251.0, 6464988.0],
                        [717445.0, 6463649.0],
                        [722648.0, 6446835.0],
                        [725628.0, 6444038.0],
                        [723408.0, 6437470.0],
                        [727352.0, 6435060.0],
                        [728488.0, 6430309.0],
                        [732563.0, 6427378.0],
                        [732994.0, 6422485.0],
                        [737614.0, 6413026.0],
                        [746579.0, 6414955.0],
                        [746937.0, 6419760.0],
                        [752090.0, 6419191.0],
                        [752669.0, 6414624.0],
                        [758530.0, 6415778.0],
                        [768276.0, 6405343.0],
                        [774103.0, 6411136.0],
                        [781273.0, 6415520.0],
                        [782106.0, 6419872.0],
                        [791307.0, 6420019.0],
                        [796332.0, 6429870.0],
                        [801638.0, 6433950.0],
                        [808165.0, 6446004.0],
                        [808187.0, 6450395.0],
                        [813436.0, 6450097.0],
                        [816370.0, 6460772.0],
                        [811227.0, 6465007.0],
                        [806856.0, 6463705.0],
                        [807029.0, 6471772.0],
                        [802398.0, 6475469.0],
                        [792166.0, 6477709.0],
                        [780259.0, 6472021.0],
                        [776677.0, 6475557.0],
                        [770263.0, 6473476.0],
                        [763248.0, 6477235.0],
                        [760473.0, 6472496.0],
                        [752222.0, 6475420.0],
                        [749018.0, 6471190.0],
                        [739202.0, 6481030.0],
                        [735734.0, 6477978.0],
                        [722742.0, 6478087.0],
                        [713160.0, 6472555.0],
                        [708104.0, 6472781.0],
                    ]
                ],
                "bbox": (708104.0, 6405343.0, 816370.0, 6481030.0),
            }
        },
        46: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [776081.0, 6923412.0],
                        [775242.0, 6918312.0],
                        [767112.0, 6923360.0],
                        [761575.0, 6918670.0],
                        [747051.0, 6913033.0],
                        [751361.0, 6898188.0],
                        [745398.0, 6894771.0],
                        [743801.0, 6891376.0],
                        [747254.0, 6882494.0],
                        [744067.0, 6871735.0],
                        [738742.0, 6868146.0],
                        [735603.0, 6861428.0],
                        [732040.0, 6850473.0],
                        [735102.0, 6844463.0],
                        [733915.0, 6839303.0],
                        [740952.0, 6835710.0],
                        [746802.0, 6826973.0],
                        [753955.0, 6826609.0],
                        [760761.0, 6824301.0],
                        [769985.0, 6834113.0],
                        [773768.0, 6840880.0],
                        [779497.0, 6845091.0],
                        [785824.0, 6845891.0],
                        [797551.0, 6845278.0],
                        [796995.0, 6835851.0],
                        [803526.0, 6829924.0],
                        [814106.0, 6826118.0],
                        [819107.0, 6829393.0],
                        [823302.0, 6827043.0],
                        [832829.0, 6827018.0],
                        [830469.0, 6833515.0],
                        [836281.0, 6837534.0],
                        [835676.0, 6840415.0],
                        [840675.0, 6844876.0],
                        [846365.0, 6844539.0],
                        [846329.0, 6850924.0],
                        [838677.0, 6859119.0],
                        [841874.0, 6870868.0],
                        [848655.0, 6874905.0],
                        [848915.0, 6882413.0],
                        [846791.0, 6883540.0],
                        [845429.0, 6892400.0],
                        [841416.0, 6900107.0],
                        [842092.0, 6905887.0],
                        [837898.0, 6908256.0],
                        [833062.0, 6905548.0],
                        [822754.0, 6907655.0],
                        [818364.0, 6905215.0],
                        [814643.0, 6911926.0],
                        [803525.0, 6909934.0],
                        [795010.0, 6915033.0],
                        [790665.0, 6921022.0],
                        [782994.0, 6923346.0],
                        [776081.0, 6923412.0],
                    ]
                ],
                "bbox": (732040.0, 6824301.0, 848915.0, 6923412.0),
            }
        },
        47: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [428954.0, 6200122.0],
                        [442286.0, 6190571.0],
                        [445343.0, 6185147.0],
                        [453012.0, 6180794.0],
                        [468501.0, 6186020.0],
                        [475317.0, 6183479.0],
                        [477861.0, 6178841.0],
                        [483521.0, 6184043.0],
                        [489111.0, 6180226.0],
                        [493119.0, 6181125.0],
                        [491362.0, 6184305.0],
                        [492032.0, 6195420.0],
                        [493715.0, 6200925.0],
                        [501753.0, 6199863.0],
                        [506394.0, 6214104.0],
                        [501199.0, 6216194.0],
                        [498958.0, 6224762.0],
                        [491741.0, 6229153.0],
                        [500952.0, 6237523.0],
                        [501020.0, 6240519.0],
                        [506455.0, 6246240.0],
                        [505760.0, 6248691.0],
                        [496988.0, 6250664.0],
                        [479309.0, 6257879.0],
                        [471536.0, 6256527.0],
                        [468215.0, 6263513.0],
                        [466465.0, 6272970.0],
                        [461941.0, 6273297.0],
                        [457450.0, 6278664.0],
                        [456497.0, 6283051.0],
                        [449863.0, 6280833.0],
                        [454138.0, 6272903.0],
                        [451477.0, 6264635.0],
                        [457777.0, 6262698.0],
                        [452746.0, 6249402.0],
                        [454979.0, 6245934.0],
                        [446410.0, 6236240.0],
                        [443868.0, 6230545.0],
                        [440142.0, 6228799.0],
                        [437107.0, 6220297.0],
                        [431385.0, 6215423.0],
                        [431947.0, 6209367.0],
                        [428150.0, 6207547.0],
                        [428954.0, 6200122.0],
                    ]
                ],
                "bbox": (428150.0, 6178841.0, 506455.0, 6283051.0),
            }
        },
        48: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [547193.0, 6388334.0],
                        [542159.0, 6383881.0],
                        [541673.0, 6377661.0],
                        [545774.0, 6366277.0],
                        [549131.0, 6360516.0],
                        [562737.0, 6350024.0],
                        [570617.0, 6348643.0],
                        [581893.0, 6352690.0],
                        [585843.0, 6356520.0],
                        [592401.0, 6355117.0],
                        [596271.0, 6358053.0],
                        [610874.0, 6360746.0],
                        [607709.0, 6375883.0],
                        [615061.0, 6379167.0],
                        [625170.0, 6387538.0],
                        [632813.0, 6386229.0],
                        [637122.0, 6391031.0],
                        [634129.0, 6393565.0],
                        [633003.0, 6406356.0],
                        [634482.0, 6410441.0],
                        [628463.0, 6419614.0],
                        [626129.0, 6431248.0],
                        [613933.0, 6431618.0],
                        [607209.0, 6426078.0],
                        [601475.0, 6429196.0],
                        [593751.0, 6437105.0],
                        [584718.0, 6439628.0],
                        [577774.0, 6436765.0],
                        [574976.0, 6434607.0],
                        [576964.0, 6421036.0],
                        [570522.0, 6417080.0],
                        [570668.0, 6413830.0],
                        [564402.0, 6403210.0],
                        [559319.0, 6399932.0],
                        [553113.0, 6398528.0],
                        [553597.0, 6395079.0],
                        [547193.0, 6388334.0],
                    ]
                ],
                "bbox": (541673.0, 6348643.0, 637122.0, 6439628.0),
            }
        },
        49: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [992638.0, 6305621.0],
                        [997097.0, 6315251.0],
                        [1000978.0, 6315262.0],
                        [1007566.0, 6320724.0],
                        [1012050.0, 6317690.0],
                        [1014122.0, 6322114.0],
                        [1009068.0, 6324850.0],
                        [1007601.0, 6328762.0],
                        [1000089.0, 6334256.0],
                        [1000668.0, 6338218.0],
                        [994698.0, 6348019.0],
                        [997268.0, 6357103.0],
                        [1002665.0, 6364730.0],
                        [1009696.0, 6370071.0],
                        [1008626.0, 6382943.0],
                        [1006125.0, 6388590.0],
                        [1012175.0, 6394096.0],
                        [1012913.0, 6402904.0],
                        [1010207.0, 6403290.0],
                        [1004568.0, 6397351.0],
                        [999315.0, 6394909.0],
                        [992592.0, 6389297.0],
                        [988952.0, 6378656.0],
                        [983401.0, 6378221.0],
                        [971476.0, 6380340.0],
                        [967103.0, 6386021.0],
                        [957182.0, 6379131.0],
                        [959705.0, 6373500.0],
                        [957118.0, 6369997.0],
                        [950982.0, 6378672.0],
                        [944316.0, 6377139.0],
                        [935206.0, 6370624.0],
                        [932421.0, 6356788.0],
                        [925339.0, 6357350.0],
                        [930231.0, 6349978.0],
                        [920148.0, 6349530.0],
                        [913873.0, 6347227.0],
                        [911050.0, 6344481.0],
                        [905908.0, 6346585.0],
                        [903520.0, 6340766.0],
                        [899978.0, 6338355.0],
                        [900470.0, 6332559.0],
                        [903879.0, 6328807.0],
                        [901642.0, 6319541.0],
                        [909386.0, 6316209.0],
                        [907367.0, 6306820.0],
                        [913516.0, 6306489.0],
                        [922160.0, 6296144.0],
                        [923991.0, 6299127.0],
                        [933734.0, 6296912.0],
                        [944763.0, 6293054.0],
                        [956283.0, 6301072.0],
                        [958836.0, 6305328.0],
                        [970413.0, 6298787.0],
                        [974848.0, 6301646.0],
                        [974774.0, 6305117.0],
                        [983574.0, 6307147.0],
                        [992638.0, 6305621.0],
                    ]
                ],
                "bbox": (899978.0, 6293054.0, 1014122.0, 6403290.0),
            }
        },
        50: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [521086.0, 6735350.0],
                        [518605.0, 6737439.0],
                        [528757.0, 6745774.0],
                        [533549.0, 6751156.0],
                        [532634.0, 6757689.0],
                        [539183.0, 6762306.0],
                        [535875.0, 6773086.0],
                        [539332.0, 6780273.0],
                        [542578.0, 6786711.0],
                        [536374.0, 6790515.0],
                        [531913.0, 6790043.0],
                        [528051.0, 6796772.0],
                        [517130.0, 6797243.0],
                        [513744.0, 6803794.0],
                        [506061.0, 6806938.0],
                        [505117.0, 6820071.0],
                        [500193.0, 6823412.0],
                        [486746.0, 6818925.0],
                        [482552.0, 6813147.0],
                        [477803.0, 6815005.0],
                        [473893.0, 6813452.0],
                        [467628.0, 6811401.0],
                        [465298.0, 6799724.0],
                        [466280.0, 6794064.0],
                        [458409.0, 6789055.0],
                        [460615.0, 6783387.0],
                        [459373.0, 6778102.0],
                        [451057.0, 6776681.0],
                        [455060.0, 6767070.0],
                        [447308.0, 6764356.0],
                        [448124.0, 6758669.0],
                        [442128.0, 6753611.0],
                        [446459.0, 6750432.0],
                        [446732.0, 6745443.0],
                        [450112.0, 6740918.0],
                        [457617.0, 6739093.0],
                        [460339.0, 6732350.0],
                        [464631.0, 6730466.0],
                        [469484.0, 6733109.0],
                        [474490.0, 6731769.0],
                        [478533.0, 6727039.0],
                        [485937.0, 6723990.0],
                        [491931.0, 6726763.0],
                        [502971.0, 6722018.0],
                        [509057.0, 6729569.0],
                        [515591.0, 6731294.0],
                        [521086.0, 6735350.0],
                    ]
                ],
                "bbox": (442128.0, 6722018.0, 542578.0, 6823412.0),
            }
        },
        51: {
            "geometry": {
                "type": "MultiPolygon",
                "coordinates": [
                    [
                        [
                            [229520.0, 6710085.0],
                            [240383.0, 6704696.0],
                            [240163.0, 6708285.0],
                            [235835.0, 6713741.0],
                            [229006.0, 6716339.0],
                            [229520.0, 6710085.0],
                        ]
                    ],
                    [
                        [
                            [212687.0, 6770001.0],
                            [211559.0, 6762660.0],
                            [216528.0, 6752538.0],
                            [224759.0, 6753321.0],
                            [234540.0, 6747533.0],
                            [234220.0, 6745025.0],
                            [240082.0, 6736634.0],
                            [251238.0, 6736509.0],
                            [260889.0, 6740570.0],
                            [266327.0, 6740184.0],
                            [271833.0, 6736526.0],
                            [269308.0, 6731426.0],
                            [263161.0, 6733044.0],
                            [258643.0, 6731063.0],
                            [263631.0, 6725691.0],
                            [274085.0, 6728424.0],
                            [283689.0, 6728526.0],
                            [290663.0, 6724762.0],
                            [288997.0, 6719519.0],
                            [300067.0, 6720583.0],
                            [315995.0, 6726944.0],
                            [317487.0, 6738014.0],
                            [315562.0, 6748257.0],
                            [318803.0, 6749864.0],
                            [322365.0, 6758359.0],
                            [317839.0, 6765781.0],
                            [320890.0, 6769878.0],
                            [317014.0, 6776765.0],
                            [309516.0, 6779524.0],
                            [306739.0, 6782705.0],
                            [310549.0, 6787938.0],
                            [306966.0, 6794650.0],
                            [300637.0, 6793781.0],
                            [298204.0, 6799128.0],
                            [291749.0, 6798375.0],
                            [285132.0, 6789099.0],
                            [280665.0, 6785525.0],
                            [277875.0, 6787107.0],
                            [279762.0, 6794931.0],
                            [270150.0, 6795768.0],
                            [252150.0, 6805897.0],
                            [246074.0, 6807146.0],
                            [242780.0, 6802066.0],
                            [233945.0, 6800703.0],
                            [229439.0, 6804532.0],
                            [212675.0, 6807535.0],
                            [202658.0, 6804638.0],
                            [202166.0, 6798131.0],
                            [205220.0, 6785800.0],
                            [211608.0, 6786141.0],
                            [216607.0, 6782805.0],
                            [223352.0, 6780867.0],
                            [221788.0, 6771592.0],
                            [212687.0, 6770001.0],
                        ]
                    ],
                ],
                "bbox": (202166.0, 6704696.0, 322365.0, 6807535.0),
            }
        },
        52: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [539112.0, 6843338.0],
                        [541484.0, 6833537.0],
                        [549859.0, 6826656.0],
                        [547706.0, 6812952.0],
                        [543224.0, 6808364.0],
                        [535921.0, 6806758.0],
                        [534094.0, 6803079.0],
                        [538862.0, 6792254.0],
                        [536374.0, 6790515.0],
                        [542578.0, 6786711.0],
                        [539332.0, 6780273.0],
                        [551017.0, 6782681.0],
                        [551727.0, 6777731.0],
                        [559439.0, 6777275.0],
                        [559999.0, 6772636.0],
                        [566408.0, 6764638.0],
                        [573758.0, 6762848.0],
                        [589601.0, 6765689.0],
                        [589165.0, 6770859.0],
                        [595216.0, 6770968.0],
                        [597558.0, 6774986.0],
                        [606511.0, 6774717.0],
                        [615539.0, 6776549.0],
                        [623220.0, 6795517.0],
                        [625387.0, 6798948.0],
                        [624251.0, 6811501.0],
                        [620852.0, 6812073.0],
                        [620311.0, 6818025.0],
                        [611402.0, 6819097.0],
                        [608912.0, 6830618.0],
                        [604791.0, 6831646.0],
                        [597086.0, 6841311.0],
                        [595472.0, 6845570.0],
                        [596554.0, 6858123.0],
                        [594217.0, 6866696.0],
                        [590236.0, 6872273.0],
                        [587701.0, 6866890.0],
                        [579784.0, 6860594.0],
                        [580772.0, 6855857.0],
                        [576482.0, 6852521.0],
                        [566604.0, 6854037.0],
                        [555241.0, 6849633.0],
                        [543803.0, 6848251.0],
                        [539112.0, 6843338.0],
                    ]
                ],
                "bbox": (534094.0, 6762848.0, 625387.0, 6872273.0),
            }
        },
        53: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [1232225.0, 6105798.0],
                        [1226359.0, 6102880.0],
                        [1217754.0, 6104291.0],
                        [1215715.0, 6123336.0],
                        [1210743.0, 6122769.0],
                        [1206639.0, 6131576.0],
                        [1202519.0, 6133762.0],
                        [1199675.0, 6142105.0],
                        [1187662.0, 6146432.0],
                        [1180116.0, 6152978.0],
                        [1164950.0, 6158109.0],
                        [1159342.0, 6158599.0],
                        [1169805.0, 6146197.0],
                        [1163901.0, 6144874.0],
                        [1160274.0, 6141810.0],
                        [1162433.0, 6134554.0],
                        [1168722.0, 6128275.0],
                        [1172897.0, 6127710.0],
                        [1175910.0, 6121801.0],
                        [1169135.0, 6117789.0],
                        [1170381.0, 6114875.0],
                        [1164715.0, 6111213.0],
                        [1168686.0, 6106658.0],
                        [1175324.0, 6107066.0],
                        [1177911.0, 6109856.0],
                        [1182246.0, 6105577.0],
                        [1180256.0, 6096485.0],
                        [1173562.0, 6088828.0],
                        [1176478.0, 6086334.0],
                        [1193164.0, 6083876.0],
                        [1190373.0, 6079505.0],
                        [1183548.0, 6076443.0],
                        [1183755.0, 6068439.0],
                        [1188281.0, 6067524.0],
                        [1196826.0, 6061810.0],
                        [1209242.0, 6057430.0],
                        [1210610.0, 6054235.0],
                        [1221376.0, 6050136.0],
                        [1221646.0, 6056536.0],
                        [1224377.0, 6061299.0],
                        [1225618.0, 6068574.0],
                        [1229809.0, 6072152.0],
                        [1228158.0, 6078400.0],
                        [1232811.0, 6083356.0],
                        [1233466.0, 6095728.0],
                        [1232225.0, 6105798.0],
                    ]
                ],
                "bbox": (1159342.0, 6050136.0, 1233466.0, 6158599.0),
            }
        },
        54: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [879041.0, 6935896.0],
                        [878534.0, 6930925.0],
                        [883576.0, 6926040.0],
                        [891117.0, 6929633.0],
                        [895087.0, 6927281.0],
                        [898652.0, 6915853.0],
                        [898091.0, 6905220.0],
                        [901547.0, 6903348.0],
                        [900172.0, 6898682.0],
                        [901739.0, 6893315.0],
                        [905859.0, 6891045.0],
                        [906433.0, 6884047.0],
                        [905066.0, 6875501.0],
                        [901302.0, 6871865.0],
                        [904125.0, 6862116.0],
                        [899528.0, 6852809.0],
                        [902927.0, 6847494.0],
                        [903561.0, 6837915.0],
                        [900379.0, 6832469.0],
                        [904895.0, 6830283.0],
                        [904250.0, 6825381.0],
                        [910921.0, 6826770.0],
                        [919362.0, 6809623.0],
                        [930831.0, 6810526.0],
                        [935203.0, 6815546.0],
                        [963830.0, 6819154.0],
                        [964951.0, 6823820.0],
                        [973656.0, 6818945.0],
                        [982282.0, 6817771.0],
                        [987769.0, 6820181.0],
                        [994239.0, 6827289.0],
                        [1004385.0, 6831654.0],
                        [1001023.0, 6834020.0],
                        [993487.0, 6842033.0],
                        [983189.0, 6843093.0],
                        [976380.0, 6847597.0],
                        [971773.0, 6847764.0],
                        [969100.0, 6851558.0],
                        [964692.0, 6852206.0],
                        [961206.0, 6856322.0],
                        [946451.0, 6858744.0],
                        [945024.0, 6864515.0],
                        [939656.0, 6875317.0],
                        [929417.0, 6875078.0],
                        [928151.0, 6878713.0],
                        [922735.0, 6879575.0],
                        [922708.0, 6883240.0],
                        [914442.0, 6889764.0],
                        [914011.0, 6893894.0],
                        [920671.0, 6900380.0],
                        [917676.0, 6905219.0],
                        [920628.0, 6907653.0],
                        [912688.0, 6923499.0],
                        [913390.0, 6932141.0],
                        [909643.0, 6936903.0],
                        [902225.0, 6942685.0],
                        [896580.0, 6941163.0],
                        [892512.0, 6942531.0],
                        [879041.0, 6935896.0],
                    ]
                ],
                "bbox": (878534.0, 6809623.0, 1004385.0, 6942685.0),
            }
        },
        55: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [510545.0, 6875400.0],
                        [504730.0, 6874732.0],
                        [500558.0, 6877307.0],
                        [491678.0, 6873174.0],
                        [489675.0, 6876299.0],
                        [484257.0, 6870962.0],
                        [475607.0, 6865852.0],
                        [469104.0, 6863978.0],
                        [457511.0, 6867107.0],
                        [454384.0, 6863198.0],
                        [449827.0, 6868751.0],
                        [446005.0, 6869054.0],
                        [427244.0, 6861949.0],
                        [417723.0, 6857101.0],
                        [420581.0, 6852284.0],
                        [423498.0, 6841841.0],
                        [421619.0, 6835816.0],
                        [414934.0, 6829326.0],
                        [422197.0, 6821752.0],
                        [429868.0, 6822252.0],
                        [437568.0, 6825109.0],
                        [441174.0, 6828584.0],
                        [446687.0, 6829012.0],
                        [451256.0, 6826715.0],
                        [457920.0, 6827997.0],
                        [463434.0, 6833996.0],
                        [466087.0, 6830999.0],
                        [467262.0, 6822174.0],
                        [474394.0, 6821359.0],
                        [473893.0, 6813452.0],
                        [477803.0, 6815005.0],
                        [482552.0, 6813147.0],
                        [486746.0, 6818925.0],
                        [500193.0, 6823412.0],
                        [505117.0, 6820071.0],
                        [506061.0, 6806938.0],
                        [513744.0, 6803794.0],
                        [517130.0, 6797243.0],
                        [528051.0, 6796772.0],
                        [531913.0, 6790043.0],
                        [536374.0, 6790515.0],
                        [538862.0, 6792254.0],
                        [534094.0, 6803079.0],
                        [535921.0, 6806758.0],
                        [543224.0, 6808364.0],
                        [547706.0, 6812952.0],
                        [549859.0, 6826656.0],
                        [541484.0, 6833537.0],
                        [539112.0, 6843338.0],
                        [533024.0, 6847407.0],
                        [536389.0, 6850906.0],
                        [533488.0, 6856087.0],
                        [524600.0, 6861683.0],
                        [520512.0, 6866728.0],
                        [513552.0, 6867418.0],
                        [508448.0, 6870995.0],
                        [510545.0, 6875400.0],
                    ]
                ],
                "bbox": (414934.0, 6790043.0, 549859.0, 6877307.0),
            }
        },
        56: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [877344.0, 6596683.0],
                        [869949.0, 6598870.0],
                        [868758.0, 6603264.0],
                        [857422.0, 6600366.0],
                        [848718.0, 6603737.0],
                        [845229.0, 6595067.0],
                        [845050.0, 6590960.0],
                        [839484.0, 6574938.0],
                        [837322.0, 6565642.0],
                        [838741.0, 6561678.0],
                        [835220.0, 6556428.0],
                        [834501.0, 6551198.0],
                        [835850.0, 6543054.0],
                        [833933.0, 6540287.0],
                        [844536.0, 6535588.0],
                        [849404.0, 6524528.0],
                        [863140.0, 6525920.0],
                        [870291.0, 6521483.0],
                        [876089.0, 6523611.0],
                        [879159.0, 6530984.0],
                        [882529.0, 6534232.0],
                        [888996.0, 6528631.0],
                        [888123.0, 6525941.0],
                        [897998.0, 6515860.0],
                        [904449.0, 6504924.0],
                        [909413.0, 6509006.0],
                        [910338.0, 6515974.0],
                        [915868.0, 6518199.0],
                        [916332.0, 6528631.0],
                        [919315.0, 6541572.0],
                        [917441.0, 6547210.0],
                        [916992.0, 6555432.0],
                        [918731.0, 6560168.0],
                        [928176.0, 6563414.0],
                        [933670.0, 6575392.0],
                        [940629.0, 6577110.0],
                        [938817.0, 6580823.0],
                        [943610.0, 6590224.0],
                        [935280.0, 6595262.0],
                        [929573.0, 6589336.0],
                        [923919.0, 6580132.0],
                        [919460.0, 6577537.0],
                        [909869.0, 6577050.0],
                        [908880.0, 6582354.0],
                        [903737.0, 6585593.0],
                        [895736.0, 6577631.0],
                        [890444.0, 6576892.0],
                        [887558.0, 6582363.0],
                        [882486.0, 6586333.0],
                        [882664.0, 6589574.0],
                        [877221.0, 6593352.0],
                        [877344.0, 6596683.0],
                    ]
                ],
                "bbox": (833933.0, 6504924.0, 943610.0, 6603737.0),
            }
        },
        57: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [524704.0, 6194181.0],
                        [530246.0, 6189977.0],
                        [542336.0, 6189236.0],
                        [549247.0, 6180805.0],
                        [554894.0, 6182516.0],
                        [562658.0, 6181937.0],
                        [567864.0, 6177510.0],
                        [571419.0, 6168649.0],
                        [580902.0, 6173997.0],
                        [585134.0, 6170835.0],
                        [592180.0, 6170254.0],
                        [600206.0, 6164532.0],
                        [607482.0, 6165290.0],
                        [615529.0, 6169131.0],
                        [617954.0, 6174066.0],
                        [631545.0, 6174170.0],
                        [632373.0, 6176261.0],
                        [622733.0, 6184536.0],
                        [614361.0, 6182470.0],
                        [610538.0, 6185966.0],
                        [608402.0, 6195450.0],
                        [617137.0, 6197372.0],
                        [617210.0, 6214637.0],
                        [613663.0, 6217624.0],
                        [614442.0, 6225039.0],
                        [604330.0, 6228108.0],
                        [595072.0, 6233132.0],
                        [596256.0, 6236985.0],
                        [593467.0, 6242482.0],
                        [589272.0, 6240503.0],
                        [582028.0, 6242918.0],
                        [575081.0, 6235814.0],
                        [572065.0, 6240985.0],
                        [565565.0, 6247682.0],
                        [561407.0, 6244405.0],
                        [567825.0, 6236309.0],
                        [562910.0, 6233771.0],
                        [555809.0, 6233248.0],
                        [554118.0, 6225953.0],
                        [547328.0, 6230443.0],
                        [541984.0, 6228895.0],
                        [540236.0, 6224348.0],
                        [535770.0, 6220735.0],
                        [537386.0, 6214080.0],
                        [535012.0, 6210452.0],
                        [526458.0, 6208824.0],
                        [522873.0, 6204718.0],
                        [524704.0, 6194181.0],
                    ]
                ],
                "bbox": (522873.0, 6164532.0, 632373.0, 6247682.0),
            }
        },
        58: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [563452.0, 6484264.0],
                        [563844.0, 6479160.0],
                        [568750.0, 6477311.0],
                        [560988.0, 6465186.0],
                        [564865.0, 6461779.0],
                        [566113.0, 6450433.0],
                        [573095.0, 6446142.0],
                        [577774.0, 6436765.0],
                        [584718.0, 6439628.0],
                        [593751.0, 6437105.0],
                        [601475.0, 6429196.0],
                        [607209.0, 6426078.0],
                        [613933.0, 6431618.0],
                        [626129.0, 6431248.0],
                        [631663.0, 6432186.0],
                        [628772.0, 6440050.0],
                        [634831.0, 6442812.0],
                        [638218.0, 6451518.0],
                        [636846.0, 6458269.0],
                        [642886.0, 6465907.0],
                        [649118.0, 6470007.0],
                        [651369.0, 6479623.0],
                        [656321.0, 6476249.0],
                        [659915.0, 6480065.0],
                        [661594.0, 6486680.0],
                        [662264.0, 6494997.0],
                        [658322.0, 6500286.0],
                        [662164.0, 6504560.0],
                        [663288.0, 6509261.0],
                        [660507.0, 6515469.0],
                        [656101.0, 6511235.0],
                        [649366.0, 6512420.0],
                        [648450.0, 6509221.0],
                        [637278.0, 6511373.0],
                        [635791.0, 6515072.0],
                        [631053.0, 6514297.0],
                        [623634.0, 6517796.0],
                        [614305.0, 6511565.0],
                        [612121.0, 6507862.0],
                        [605337.0, 6509707.0],
                        [593766.0, 6498788.0],
                        [587429.0, 6495459.0],
                        [581471.0, 6497412.0],
                        [576059.0, 6493313.0],
                        [571080.0, 6486618.0],
                        [566212.0, 6489282.0],
                        [563452.0, 6484264.0],
                    ]
                ],
                "bbox": (560988.0, 6426078.0, 663288.0, 6517796.0),
            }
        },
        59: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [987691.0, 6753958.0],
                        [989189.0, 6749828.0],
                        [1000830.0, 6745745.0],
                        [1003510.0, 6737902.0],
                        [1000706.0, 6731961.0],
                        [1006958.0, 6729298.0],
                        [1010786.0, 6719537.0],
                        [1014326.0, 6713511.0],
                        [1019985.0, 6711034.0],
                        [1024064.0, 6712992.0],
                        [1031730.0, 6713160.0],
                        [1039126.0, 6720266.0],
                        [1038581.0, 6725392.0],
                        [1044459.0, 6729657.0],
                        [1044908.0, 6732411.0],
                        [1039238.0, 6738821.0],
                        [1038337.0, 6743276.0],
                        [1040672.0, 6747502.0],
                        [1039113.0, 6752662.0],
                        [1040418.0, 6763108.0],
                        [1044697.0, 6773634.0],
                        [1040564.0, 6779861.0],
                        [1040408.0, 6789869.0],
                        [1036067.0, 6790258.0],
                        [1032222.0, 6793924.0],
                        [1032428.0, 6798669.0],
                        [1022638.0, 6802981.0],
                        [1016850.0, 6809125.0],
                        [1011125.0, 6809400.0],
                        [1006708.0, 6801985.0],
                        [1001795.0, 6789636.0],
                        [998205.0, 6776671.0],
                        [993983.0, 6773739.0],
                        [991355.0, 6761730.0],
                        [993060.0, 6757264.0],
                        [987691.0, 6753958.0],
                    ]
                ],
                "bbox": (987691.0, 6711034.0, 1044908.0, 6809400.0),
            }
        },
        60: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [573297.0, 6677558.0],
                        [569696.0, 6685244.0],
                        [565759.0, 6689706.0],
                        [561058.0, 6687382.0],
                        [557051.0, 6690388.0],
                        [558309.0, 6696538.0],
                        [557303.0, 6702930.0],
                        [559271.0, 6706788.0],
                        [552945.0, 6716389.0],
                        [555366.0, 6719856.0],
                        [552256.0, 6724799.0],
                        [543743.0, 6727862.0],
                        [539386.0, 6732046.0],
                        [530153.0, 6735215.0],
                        [521086.0, 6735350.0],
                        [515591.0, 6731294.0],
                        [509057.0, 6729569.0],
                        [502971.0, 6722018.0],
                        [491931.0, 6726763.0],
                        [489539.0, 6719725.0],
                        [490772.0, 6714971.0],
                        [487846.0, 6706491.0],
                        [479254.0, 6691029.0],
                        [476861.0, 6677880.0],
                        [478404.0, 6673399.0],
                        [482922.0, 6672972.0],
                        [485608.0, 6667284.0],
                        [494947.0, 6665028.0],
                        [494833.0, 6656131.0],
                        [505157.0, 6650870.0],
                        [510389.0, 6653999.0],
                        [517389.0, 6653506.0],
                        [515158.0, 6658631.0],
                        [524608.0, 6655240.0],
                        [525272.0, 6647330.0],
                        [531658.0, 6641492.0],
                        [533123.0, 6635020.0],
                        [537221.0, 6629765.0],
                        [546519.0, 6631001.0],
                        [551779.0, 6655211.0],
                        [553982.0, 6659068.0],
                        [560408.0, 6661557.0],
                        [568878.0, 6661167.0],
                        [571950.0, 6667671.0],
                        [576079.0, 6670370.0],
                        [573297.0, 6677558.0],
                    ]
                ],
                "bbox": (476861.0, 6629765.0, 576079.0, 6735350.0),
            }
        },
        61: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [634422.0, 7101577.0],
                        [636514.0, 7100652.0],
                        [642453.0, 7087769.0],
                        [644454.0, 7079622.0],
                        [653390.0, 7071778.0],
                        [658064.0, 7064147.0],
                        [664277.0, 7059477.0],
                        [673807.0, 7057943.0],
                        [679720.0, 7059405.0],
                        [683642.0, 7063001.0],
                        [688698.0, 7062196.0],
                        [686184.0, 7056263.0],
                        [687641.0, 7047582.0],
                        [692820.0, 7044918.0],
                        [697493.0, 7046164.0],
                        [701478.0, 7042709.0],
                        [702680.0, 7038239.0],
                        [699507.0, 7031355.0],
                        [712383.0, 7015916.0],
                        [711164.0, 7009522.0],
                        [706855.0, 7002888.0],
                        [706470.0, 6995042.0],
                        [712391.0, 6990404.0],
                        [716534.0, 6992565.0],
                        [720100.0, 6990781.0],
                        [725313.0, 6993104.0],
                        [735248.0, 6991264.0],
                        [739058.0, 6995179.0],
                        [744014.0, 6992056.0],
                        [751253.0, 6997000.0],
                        [753505.0, 6995276.0],
                        [766237.0, 6992385.0],
                        [770359.0, 6988977.0],
                        [778060.0, 6986162.0],
                        [781905.0, 6987282.0],
                        [783321.0, 6995122.0],
                        [788006.0, 6997129.0],
                        [785882.0, 7004302.0],
                        [780654.0, 7004663.0],
                        [782919.0, 7012216.0],
                        [787237.0, 7018342.0],
                        [781127.0, 7020183.0],
                        [773958.0, 7027731.0],
                        [768979.0, 7028494.0],
                        [763439.0, 7026158.0],
                        [760772.0, 7028536.0],
                        [753265.0, 7028386.0],
                        [749524.0, 7024442.0],
                        [746894.0, 7030603.0],
                        [747241.0, 7039722.0],
                        [743565.0, 7044070.0],
                        [733656.0, 7048580.0],
                        [726572.0, 7043892.0],
                        [720348.0, 7047849.0],
                        [717610.0, 7060148.0],
                        [718286.0, 7067127.0],
                        [713527.0, 7069761.0],
                        [710740.0, 7076212.0],
                        [704337.0, 7075998.0],
                        [695621.0, 7071968.0],
                        [692807.0, 7066387.0],
                        [686789.0, 7068929.0],
                        [680050.0, 7079722.0],
                        [674224.0, 7079651.0],
                        [671748.0, 7084186.0],
                        [674131.0, 7094505.0],
                        [670178.0, 7102078.0],
                        [667909.0, 7110430.0],
                        [659576.0, 7106814.0],
                        [654175.0, 7107325.0],
                        [637372.0, 7101244.0],
                        [634422.0, 7101577.0],
                    ]
                ],
                "bbox": (634422.0, 6986162.0, 788006.0, 7110430.0),
            }
        },
        62: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [1010786.0, 6719537.0],
                        [1006958.0, 6729298.0],
                        [1000706.0, 6731961.0],
                        [1003510.0, 6737902.0],
                        [1000830.0, 6745745.0],
                        [989189.0, 6749828.0],
                        [987691.0, 6753958.0],
                        [986052.0, 6752778.0],
                        [981470.0, 6745308.0],
                        [983826.0, 6731782.0],
                        [986155.0, 6724947.0],
                        [993651.0, 6723596.0],
                        [994087.0, 6717743.0],
                        [996902.0, 6711079.0],
                        [1001194.0, 6713400.0],
                        [999772.0, 6717889.0],
                        [1007028.0, 6717766.0],
                        [1010786.0, 6719537.0],
                    ]
                ],
                "bbox": (981470.0, 6711079.0, 1010786.0, 6753958.0),
            }
        },
        63: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [382163.0, 6750361.0],
                        [374291.0, 6753124.0],
                        [371672.0, 6756673.0],
                        [356172.0, 6751743.0],
                        [351165.0, 6744974.0],
                        [342268.0, 6743932.0],
                        [335469.0, 6745297.0],
                        [327477.0, 6743723.0],
                        [317487.0, 6738014.0],
                        [315995.0, 6726944.0],
                        [300067.0, 6720583.0],
                        [288997.0, 6719519.0],
                        [282806.0, 6712695.0],
                        [283678.0, 6703214.0],
                        [288393.0, 6698809.0],
                        [292251.0, 6700736.0],
                        [299211.0, 6695249.0],
                        [308160.0, 6699576.0],
                        [309828.0, 6691500.0],
                        [308860.0, 6686791.0],
                        [304084.0, 6683183.0],
                        [316935.0, 6678247.0],
                        [322009.0, 6670666.0],
                        [324590.0, 6666698.0],
                        [332678.0, 6659222.0],
                        [338731.0, 6658583.0],
                        [343129.0, 6653969.0],
                        [353671.0, 6649968.0],
                        [357453.0, 6652400.0],
                        [354064.0, 6663454.0],
                        [354394.0, 6667534.0],
                        [359840.0, 6669393.0],
                        [361112.0, 6656907.0],
                        [367588.0, 6659427.0],
                        [367645.0, 6668175.0],
                        [374065.0, 6668782.0],
                        [376108.0, 6673611.0],
                        [385082.0, 6667087.0],
                        [387784.0, 6670618.0],
                        [379298.0, 6676338.0],
                        [379159.0, 6679552.0],
                        [384839.0, 6683588.0],
                        [382659.0, 6691801.0],
                        [378391.0, 6691949.0],
                        [374867.0, 6698071.0],
                        [375470.0, 6701871.0],
                        [385609.0, 6704471.0],
                        [396890.0, 6703794.0],
                        [402393.0, 6705897.0],
                        [401401.0, 6717774.0],
                        [387234.0, 6720426.0],
                        [386462.0, 6727467.0],
                        [395905.0, 6725901.0],
                        [398982.0, 6728603.0],
                        [389301.0, 6732485.0],
                        [386987.0, 6740963.0],
                        [381255.0, 6745617.0],
                        [382163.0, 6750361.0],
                    ]
                ],
                "bbox": (282806.0, 6649968.0, 402393.0, 6756673.0),
            }
        },
        64: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [698236.0, 6718784.0],
                        [701928.0, 6717598.0],
                        [707666.0, 6720847.0],
                        [709181.0, 6715638.0],
                        [715400.0, 6713642.0],
                        [717731.0, 6709895.0],
                        [723118.0, 6710529.0],
                        [726085.0, 6708066.0],
                        [729904.0, 6712131.0],
                        [736733.0, 6710509.0],
                        [738638.0, 6714270.0],
                        [743712.0, 6710994.0],
                        [743701.0, 6707332.0],
                        [751168.0, 6705240.0],
                        [753904.0, 6700965.0],
                        [759265.0, 6699206.0],
                        [773240.0, 6698003.0],
                        [773484.0, 6692940.0],
                        [783517.0, 6693799.0],
                        [780393.0, 6701310.0],
                        [784360.0, 6705398.0],
                        [783952.0, 6713331.0],
                        [788376.0, 6717645.0],
                        [791050.0, 6725986.0],
                        [795163.0, 6731989.0],
                        [794312.0, 6737086.0],
                        [799827.0, 6740411.0],
                        [799053.0, 6750471.0],
                        [794318.0, 6750026.0],
                        [796594.0, 6759156.0],
                        [789480.0, 6763963.0],
                        [783028.0, 6759096.0],
                        [767353.0, 6760249.0],
                        [763413.0, 6765164.0],
                        [759917.0, 6778325.0],
                        [755033.0, 6785741.0],
                        [749678.0, 6782304.0],
                        [746161.0, 6791892.0],
                        [745765.0, 6796957.0],
                        [736893.0, 6807760.0],
                        [730707.0, 6810078.0],
                        [727042.0, 6808060.0],
                        [712422.0, 6807964.0],
                        [703668.0, 6806647.0],
                        [703237.0, 6796855.0],
                        [700383.0, 6789702.0],
                        [695265.0, 6784783.0],
                        [702192.0, 6781427.0],
                        [703759.0, 6774663.0],
                        [706580.0, 6772621.0],
                        [709585.0, 6763406.0],
                        [703743.0, 6756785.0],
                        [700769.0, 6756036.0],
                        [701782.0, 6742907.0],
                        [695135.0, 6740319.0],
                        [689262.0, 6740069.0],
                        [689391.0, 6734632.0],
                        [694231.0, 6731133.0],
                        [695341.0, 6722058.0],
                        [698236.0, 6718784.0],
                    ]
                ],
                "bbox": (689262.0, 6692940.0, 799827.0, 6810078.0),
            }
        },
        65: {
            "geometry": {
                "type": "MultiPolygon",
                "coordinates": [
                    [
                        [
                            [330825.0, 6841252.0],
                            [328251.0, 6849310.0],
                            [322500.0, 6846053.0],
                            [325749.0, 6841751.0],
                            [330825.0, 6841252.0],
                        ]
                    ],
                    [
                        [
                            [399495.0, 6830885.0],
                            [389634.0, 6835155.0],
                            [384742.0, 6834496.0],
                            [376536.0, 6826525.0],
                            [368413.0, 6830443.0],
                            [366767.0, 6839135.0],
                            [363309.0, 6845999.0],
                            [348952.0, 6844300.0],
                            [343084.0, 6846084.0],
                            [341341.0, 6849649.0],
                            [342875.0, 6855953.0],
                            [334395.0, 6854417.0],
                            [329772.0, 6850320.0],
                            [334969.0, 6837949.0],
                            [337450.0, 6835570.0],
                            [332923.0, 6813386.0],
                            [328273.0, 6809495.0],
                            [325505.0, 6811765.0],
                            [320973.0, 6806998.0],
                            [315234.0, 6806646.0],
                            [311879.0, 6798427.0],
                            [306966.0, 6794650.0],
                            [310549.0, 6787938.0],
                            [306739.0, 6782705.0],
                            [309516.0, 6779524.0],
                            [317014.0, 6776765.0],
                            [320890.0, 6769878.0],
                            [317839.0, 6765781.0],
                            [322365.0, 6758359.0],
                            [318803.0, 6749864.0],
                            [315562.0, 6748257.0],
                            [317487.0, 6738014.0],
                            [327477.0, 6743723.0],
                            [335469.0, 6745297.0],
                            [342268.0, 6743932.0],
                            [351165.0, 6744974.0],
                            [356172.0, 6751743.0],
                            [371672.0, 6756673.0],
                            [374291.0, 6753124.0],
                            [382163.0, 6750361.0],
                            [382932.0, 6754022.0],
                            [386941.0, 6760260.0],
                            [390140.0, 6770978.0],
                            [394099.0, 6773357.0],
                            [400197.0, 6773697.0],
                            [400465.0, 6781914.0],
                            [398626.0, 6784333.0],
                            [395852.0, 6803336.0],
                            [400321.0, 6810723.0],
                            [398130.0, 6822559.0],
                            [399495.0, 6830885.0],
                        ]
                    ],
                ],
                "bbox": (306739.0, 6738014.0, 400465.0, 6855953.0),
            }
        },
        66: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [334283.0, 6280551.0],
                        [338238.0, 6281381.0],
                        [342731.0, 6276517.0],
                        [352961.0, 6276001.0],
                        [362964.0, 6280741.0],
                        [367753.0, 6276986.0],
                        [377063.0, 6275535.0],
                        [385124.0, 6280151.0],
                        [394475.0, 6281218.0],
                        [399714.0, 6278404.0],
                        [403447.0, 6281078.0],
                        [412501.0, 6278002.0],
                        [421554.0, 6283529.0],
                        [424830.0, 6280287.0],
                        [433057.0, 6279053.0],
                        [438084.0, 6281592.0],
                        [439499.0, 6295895.0],
                        [442698.0, 6298315.0],
                        [443572.0, 6313646.0],
                        [440195.0, 6315550.0],
                        [444807.0, 6320573.0],
                        [453303.0, 6321029.0],
                        [461632.0, 6315722.0],
                        [465239.0, 6317398.0],
                        [465465.0, 6324803.0],
                        [464468.0, 6326808.0],
                        [467874.0, 6336236.0],
                        [470769.0, 6340294.0],
                        [462757.0, 6341305.0],
                        [459734.0, 6343570.0],
                        [449759.0, 6344226.0],
                        [449164.0, 6352477.0],
                        [445091.0, 6357381.0],
                        [442465.0, 6350467.0],
                        [438523.0, 6349285.0],
                        [429214.0, 6351421.0],
                        [430077.0, 6359927.0],
                        [426389.0, 6364119.0],
                        [419635.0, 6366255.0],
                        [418928.0, 6369127.0],
                        [411174.0, 6373203.0],
                        [411314.0, 6378206.0],
                        [403228.0, 6378937.0],
                        [393973.0, 6376334.0],
                        [388231.0, 6379327.0],
                        [381182.0, 6379048.0],
                        [383562.0, 6383958.0],
                        [375548.0, 6389853.0],
                        [373686.0, 6386677.0],
                        [361774.0, 6383396.0],
                        [355175.0, 6347840.0],
                        [348748.0, 6322487.0],
                        [342546.0, 6300215.0],
                        [341177.0, 6292691.0],
                        [334283.0, 6280551.0],
                    ]
                ],
                "bbox": (334283.0, 6275535.0, 470769.0, 6389853.0),
            }
        },
        67: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [210124.0, 6860562.0],
                        [210319.0, 6864435.0],
                        [197918.0, 6867491.0],
                        [195168.0, 6863087.0],
                        [188759.0, 6861699.0],
                        [187283.0, 6866983.0],
                        [171676.0, 6866983.0],
                        [165007.0, 6863478.0],
                        [159513.0, 6866951.0],
                        [150836.0, 6862294.0],
                        [143559.0, 6862620.0],
                        [137993.0, 6858007.0],
                        [130421.0, 6856362.0],
                        [126410.0, 6849458.0],
                        [124277.0, 6841317.0],
                        [126191.0, 6836271.0],
                        [137035.0, 6831319.0],
                        [151215.0, 6836632.0],
                        [149516.0, 6831882.0],
                        [159712.0, 6829829.0],
                        [158800.0, 6824711.0],
                        [152925.0, 6822802.0],
                        [150196.0, 6824902.0],
                        [139593.0, 6825317.0],
                        [139209.0, 6819312.0],
                        [147794.0, 6818873.0],
                        [158225.0, 6812964.0],
                        [159803.0, 6806301.0],
                        [156741.0, 6801688.0],
                        [151961.0, 6804475.0],
                        [143250.0, 6802653.0],
                        [129762.0, 6802135.0],
                        [125302.0, 6798484.0],
                        [144750.0, 6790824.0],
                        [150552.0, 6779953.0],
                        [149596.0, 6769606.0],
                        [162294.0, 6768615.0],
                        [164959.0, 6774152.0],
                        [171525.0, 6775493.0],
                        [174013.0, 6772980.0],
                        [183941.0, 6774809.0],
                        [184280.0, 6770949.0],
                        [191411.0, 6764941.0],
                        [196633.0, 6766198.0],
                        [200244.0, 6762976.0],
                        [211251.0, 6761104.0],
                        [212687.0, 6770001.0],
                        [221788.0, 6771592.0],
                        [223352.0, 6780867.0],
                        [216607.0, 6782805.0],
                        [211608.0, 6786141.0],
                        [205220.0, 6785800.0],
                        [202166.0, 6798131.0],
                        [202658.0, 6804638.0],
                        [212675.0, 6807535.0],
                        [216996.0, 6817941.0],
                        [214652.0, 6824074.0],
                        [215204.0, 6828740.0],
                        [213034.0, 6834627.0],
                        [212655.0, 6842190.0],
                        [216761.0, 6846809.0],
                        [210382.0, 6855067.0],
                        [210124.0, 6860562.0],
                    ]
                ],
                "bbox": (124277.0, 6761104.0, 223352.0, 6867491.0),
            }
        },
        68: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [995359.0, 6526975.0],
                        [996263.0, 6533312.0],
                        [1005620.0, 6535079.0],
                        [1009539.0, 6537663.0],
                        [1013275.0, 6543845.0],
                        [1010939.0, 6550631.0],
                        [1005414.0, 6557665.0],
                        [1000587.0, 6561265.0],
                        [1000913.0, 6565514.0],
                        [994423.0, 6565990.0],
                        [993170.0, 6574229.0],
                        [997429.0, 6583180.0],
                        [990846.0, 6588853.0],
                        [991547.0, 6595136.0],
                        [985884.0, 6596474.0],
                        [972312.0, 6594423.0],
                        [960663.0, 6587846.0],
                        [955674.0, 6591075.0],
                        [952150.0, 6588701.0],
                        [949250.0, 6580383.0],
                        [953939.0, 6574878.0],
                        [949136.0, 6572391.0],
                        [944956.0, 6566962.0],
                        [928176.0, 6563414.0],
                        [918731.0, 6560168.0],
                        [916992.0, 6555432.0],
                        [917441.0, 6547210.0],
                        [919315.0, 6541572.0],
                        [921730.0, 6540986.0],
                        [922988.0, 6530552.0],
                        [926213.0, 6526897.0],
                        [936560.0, 6520059.0],
                        [946066.0, 6522280.0],
                        [948121.0, 6516633.0],
                        [952091.0, 6514710.0],
                        [959117.0, 6516128.0],
                        [962862.0, 6525460.0],
                        [965836.0, 6528299.0],
                        [969073.0, 6537583.0],
                        [971940.0, 6540371.0],
                        [975690.0, 6531451.0],
                        [979688.0, 6528135.0],
                        [984284.0, 6528734.0],
                        [988618.0, 6520489.0],
                        [995359.0, 6526975.0],
                    ]
                ],
                "bbox": (916992.0, 6514710.0, 1013275.0, 6596474.0),
            }
        },
        69: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [605624.0, 6904387.0],
                        [608283.0, 6898554.0],
                        [612180.0, 6899061.0],
                        [618689.0, 6896449.0],
                        [626847.0, 6897876.0],
                        [633020.0, 6901508.0],
                        [645465.0, 6895048.0],
                        [649761.0, 6898738.0],
                        [652314.0, 6895981.0],
                        [659205.0, 6894147.0],
                        [670084.0, 6886723.0],
                        [673222.0, 6889902.0],
                        [680633.0, 6884542.0],
                        [700622.0, 6887964.0],
                        [705248.0, 6890863.0],
                        [710189.0, 6894765.0],
                        [704672.0, 6899225.0],
                        [701957.0, 6908433.0],
                        [698137.0, 6911415.0],
                        [706929.0, 6919738.0],
                        [706805.0, 6926038.0],
                        [711648.0, 6928031.0],
                        [706939.0, 6934901.0],
                        [709520.0, 6938240.0],
                        [707064.0, 6950845.0],
                        [708546.0, 6956332.0],
                        [693916.0, 6955962.0],
                        [685089.0, 6950779.0],
                        [685504.0, 6946842.0],
                        [677528.0, 6947426.0],
                        [676176.0, 6942514.0],
                        [669006.0, 6944292.0],
                        [664208.0, 6949037.0],
                        [654712.0, 6950974.0],
                        [645659.0, 6956103.0],
                        [636565.0, 6954687.0],
                        [623025.0, 6958366.0],
                        [620154.0, 6956108.0],
                        [612314.0, 6962826.0],
                        [605401.0, 6955866.0],
                        [607933.0, 6953359.0],
                        [606321.0, 6945902.0],
                        [609123.0, 6937659.0],
                        [607205.0, 6935073.0],
                        [610545.0, 6930338.0],
                        [606642.0, 6924066.0],
                        [609871.0, 6919444.0],
                        [610931.0, 6911811.0],
                        [606128.0, 6907968.0],
                        [605624.0, 6904387.0],
                    ]
                ],
                "bbox": (605401.0, 6884542.0, 711648.0, 6962826.0),
            }
        },
        70: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [598361.0, 6887345.0],
                        [603102.0, 6887292.0],
                        [606678.0, 6883543.0],
                        [614076.0, 6886919.0],
                        [622312.0, 6880731.0],
                        [628641.0, 6878089.0],
                        [633062.0, 6879807.0],
                        [641836.0, 6872490.0],
                        [641404.0, 6867928.0],
                        [648071.0, 6872567.0],
                        [653619.0, 6875101.0],
                        [660416.0, 6872923.0],
                        [667303.0, 6878971.0],
                        [670084.0, 6886723.0],
                        [659205.0, 6894147.0],
                        [652314.0, 6895981.0],
                        [649761.0, 6898738.0],
                        [645465.0, 6895048.0],
                        [633020.0, 6901508.0],
                        [626847.0, 6897876.0],
                        [618689.0, 6896449.0],
                        [612180.0, 6899061.0],
                        [608283.0, 6898554.0],
                        [605624.0, 6904387.0],
                        [603501.0, 6902160.0],
                        [601892.0, 6893098.0],
                        [598361.0, 6887345.0],
                    ]
                ],
                "bbox": (598361.0, 6867928.0, 670084.0, 6904387.0),
            }
        },
        71: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [783517.0, 6693799.0],
                        [773484.0, 6692940.0],
                        [773240.0, 6698003.0],
                        [759265.0, 6699206.0],
                        [753904.0, 6700965.0],
                        [751168.0, 6705240.0],
                        [743701.0, 6707332.0],
                        [743712.0, 6710994.0],
                        [738638.0, 6714270.0],
                        [736733.0, 6710509.0],
                        [729904.0, 6712131.0],
                        [726085.0, 6708066.0],
                        [723118.0, 6710529.0],
                        [717731.0, 6709895.0],
                        [715400.0, 6713642.0],
                        [709181.0, 6715638.0],
                        [707666.0, 6720847.0],
                        [701928.0, 6717598.0],
                        [698236.0, 6718784.0],
                        [693556.0, 6718403.0],
                        [690565.0, 6713347.0],
                        [694642.0, 6704915.0],
                        [690449.0, 6694237.0],
                        [698745.0, 6684384.0],
                        [698614.0, 6680189.0],
                        [702129.0, 6669867.0],
                        [701762.0, 6662486.0],
                        [705753.0, 6657660.0],
                        [706033.0, 6650549.0],
                        [703828.0, 6645606.0],
                        [705186.0, 6641947.0],
                        [702446.0, 6632753.0],
                        [703745.0, 6628663.0],
                        [716475.0, 6620334.0],
                        [720608.0, 6624106.0],
                        [726499.0, 6620539.0],
                        [733164.0, 6623625.0],
                        [737251.0, 6617690.0],
                        [742042.0, 6624120.0],
                        [748049.0, 6627896.0],
                        [756196.0, 6628301.0],
                        [760920.0, 6622686.0],
                        [770503.0, 6626941.0],
                        [775685.0, 6631211.0],
                        [781064.0, 6632682.0],
                        [780490.0, 6638662.0],
                        [783308.0, 6642195.0],
                        [778932.0, 6646959.0],
                        [779000.0, 6654397.0],
                        [781374.0, 6662136.0],
                        [778810.0, 6664936.0],
                        [780471.0, 6669516.0],
                        [784602.0, 6669828.0],
                        [789554.0, 6672918.0],
                        [791663.0, 6673495.0],
                        [793165.0, 6678355.0],
                        [789098.0, 6683996.0],
                        [785899.0, 6682517.0],
                        [783517.0, 6693799.0],
                    ]
                ],
                "bbox": (690449.0, 6617690.0, 793165.0, 6720847.0),
            }
        },
        72: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [871408.0, 6655745.0],
                        [875449.0, 6660099.0],
                        [876191.0, 6666228.0],
                        [881002.0, 6667371.0],
                        [884880.0, 6674426.0],
                        [887407.0, 6682465.0],
                        [888032.0, 6690633.0],
                        [890265.0, 6692353.0],
                        [884919.0, 6701062.0],
                        [880595.0, 6713705.0],
                        [884206.0, 6713509.0],
                        [887785.0, 6718944.0],
                        [886108.0, 6726069.0],
                        [882147.0, 6728546.0],
                        [878352.0, 6725366.0],
                        [868188.0, 6726357.0],
                        [863148.0, 6730266.0],
                        [858300.0, 6729814.0],
                        [848455.0, 6733791.0],
                        [846031.0, 6742315.0],
                        [848388.0, 6746302.0],
                        [846087.0, 6753627.0],
                        [838226.0, 6763382.0],
                        [833316.0, 6764154.0],
                        [833398.0, 6769005.0],
                        [827043.0, 6770243.0],
                        [817962.0, 6771080.0],
                        [816397.0, 6764598.0],
                        [808147.0, 6762855.0],
                        [797692.0, 6763119.0],
                        [796594.0, 6759156.0],
                        [794318.0, 6750026.0],
                        [799053.0, 6750471.0],
                        [799827.0, 6740411.0],
                        [794312.0, 6737086.0],
                        [795163.0, 6731989.0],
                        [791050.0, 6725986.0],
                        [788376.0, 6717645.0],
                        [783952.0, 6713331.0],
                        [784360.0, 6705398.0],
                        [780393.0, 6701310.0],
                        [783517.0, 6693799.0],
                        [785899.0, 6682517.0],
                        [789098.0, 6683996.0],
                        [793165.0, 6678355.0],
                        [791663.0, 6673495.0],
                        [795506.0, 6668991.0],
                        [806631.0, 6665548.0],
                        [806754.0, 6662035.0],
                        [813434.0, 6659755.0],
                        [823470.0, 6650227.0],
                        [831500.0, 6648015.0],
                        [855755.0, 6655507.0],
                        [860161.0, 6652754.0],
                        [871408.0, 6655745.0],
                    ]
                ],
                "bbox": (780393.0, 6648015.0, 890265.0, 6771080.0),
            }
        },
        73: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [643163.0, 6853168.0],
                        [634691.0, 6849194.0],
                        [632440.0, 6843248.0],
                        [627800.0, 6838438.0],
                        [627504.0, 6829048.0],
                        [623768.0, 6826369.0],
                        [620311.0, 6818025.0],
                        [620852.0, 6812073.0],
                        [624251.0, 6811501.0],
                        [625387.0, 6798948.0],
                        [639308.0, 6801806.0],
                        [653275.0, 6801114.0],
                        [655721.0, 6802434.0],
                        [657768.0, 6807009.0],
                        [665496.0, 6812431.0],
                        [663371.0, 6815578.0],
                        [663045.0, 6825117.0],
                        [666363.0, 6833836.0],
                        [665649.0, 6839786.0],
                        [668472.0, 6843626.0],
                        [663488.0, 6848458.0],
                        [653706.0, 6846856.0],
                        [650056.0, 6850064.0],
                        [643163.0, 6853168.0],
                    ]
                ],
                "bbox": (620311.0, 6798948.0, 668472.0, 6853168.0),
            }
        },
        74: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [483517.0, 6558927.0],
                        [485043.0, 6563684.0],
                        [480502.0, 6565882.0],
                        [477460.0, 6571967.0],
                        [478920.0, 6578170.0],
                        [482897.0, 6584792.0],
                        [479925.0, 6587264.0],
                        [475179.0, 6582497.0],
                        [472141.0, 6584899.0],
                        [472081.0, 6590232.0],
                        [468300.0, 6594579.0],
                        [467690.0, 6607234.0],
                        [472374.0, 6614809.0],
                        [467588.0, 6622657.0],
                        [470952.0, 6628365.0],
                        [469757.0, 6639127.0],
                        [471018.0, 6649618.0],
                        [465537.0, 6662685.0],
                        [464624.0, 6667352.0],
                        [458546.0, 6672434.0],
                        [454238.0, 6672319.0],
                        [442005.0, 6668948.0],
                        [434893.0, 6670550.0],
                        [429925.0, 6668493.0],
                        [429262.0, 6663802.0],
                        [424993.0, 6661094.0],
                        [417889.0, 6660618.0],
                        [413368.0, 6662854.0],
                        [404251.0, 6660135.0],
                        [408792.0, 6655192.0],
                        [411920.0, 6644974.0],
                        [417892.0, 6640888.0],
                        [415662.0, 6636435.0],
                        [420740.0, 6628764.0],
                        [421344.0, 6621760.0],
                        [423543.0, 6619698.0],
                        [424018.0, 6609992.0],
                        [422851.0, 6601758.0],
                        [420470.0, 6597142.0],
                        [428213.0, 6593481.0],
                        [426574.0, 6590996.0],
                        [411453.0, 6585116.0],
                        [411065.0, 6578554.0],
                        [415606.0, 6575464.0],
                        [420640.0, 6566176.0],
                        [426299.0, 6566639.0],
                        [429269.0, 6562335.0],
                        [437311.0, 6559539.0],
                        [446458.0, 6558600.0],
                        [459784.0, 6545825.0],
                        [464645.0, 6548283.0],
                        [465641.0, 6555184.0],
                        [469589.0, 6554692.0],
                        [473907.0, 6559092.0],
                        [483517.0, 6558927.0],
                    ]
                ],
                "bbox": (404251.0, 6545825.0, 485043.0, 6672434.0),
            }
        },
        75: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [306966.0, 6794650.0],
                        [311879.0, 6798427.0],
                        [315234.0, 6806646.0],
                        [320973.0, 6806998.0],
                        [325505.0, 6811765.0],
                        [328273.0, 6809495.0],
                        [332923.0, 6813386.0],
                        [337450.0, 6835570.0],
                        [334969.0, 6837949.0],
                        [330825.0, 6841252.0],
                        [325749.0, 6841751.0],
                        [322500.0, 6846053.0],
                        [319331.0, 6843417.0],
                        [315059.0, 6845536.0],
                        [309156.0, 6855461.0],
                        [296913.0, 6849790.0],
                        [284640.0, 6841250.0],
                        [280881.0, 6840740.0],
                        [271227.0, 6848399.0],
                        [271155.0, 6854763.0],
                        [262910.0, 6863924.0],
                        [263482.0, 6867351.0],
                        [257346.0, 6869777.0],
                        [251930.0, 6880323.0],
                        [243322.0, 6881073.0],
                        [228138.0, 6874463.0],
                        [222446.0, 6879145.0],
                        [216642.0, 6872989.0],
                        [215941.0, 6861279.0],
                        [210124.0, 6860562.0],
                        [210382.0, 6855067.0],
                        [216761.0, 6846809.0],
                        [212655.0, 6842190.0],
                        [213034.0, 6834627.0],
                        [215204.0, 6828740.0],
                        [214652.0, 6824074.0],
                        [216996.0, 6817941.0],
                        [212675.0, 6807535.0],
                        [229439.0, 6804532.0],
                        [233945.0, 6800703.0],
                        [242780.0, 6802066.0],
                        [246074.0, 6807146.0],
                        [252150.0, 6805897.0],
                        [270150.0, 6795768.0],
                        [279762.0, 6794931.0],
                        [277875.0, 6787107.0],
                        [280665.0, 6785525.0],
                        [285132.0, 6789099.0],
                        [291749.0, 6798375.0],
                        [298204.0, 6799128.0],
                        [300637.0, 6793781.0],
                        [306966.0, 6794650.0],
                    ]
                ],
                "bbox": (210124.0, 6785525.0, 337450.0, 6881073.0),
            }
        },
        76: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [702446.0, 6632753.0],
                        [696943.0, 6633749.0],
                        [686837.0, 6626145.0],
                        [682775.0, 6624336.0],
                        [677474.0, 6626628.0],
                        [669145.0, 6615318.0],
                        [670500.0, 6605989.0],
                        [664479.0, 6602292.0],
                        [660370.0, 6603761.0],
                        [650317.0, 6601560.0],
                        [645179.0, 6595086.0],
                        [644784.0, 6591412.0],
                        [647921.0, 6581264.0],
                        [653217.0, 6581299.0],
                        [659880.0, 6575828.0],
                        [662729.0, 6571178.0],
                        [666451.0, 6560450.0],
                        [672252.0, 6557694.0],
                        [679424.0, 6569309.0],
                        [685369.0, 6566281.0],
                        [688886.0, 6572856.0],
                        [695157.0, 6571452.0],
                        [693029.0, 6567076.0],
                        [697819.0, 6557998.0],
                        [712873.0, 6551781.0],
                        [728730.0, 6550656.0],
                        [732386.0, 6552810.0],
                        [735860.0, 6546571.0],
                        [745523.0, 6547324.0],
                        [753779.0, 6537018.0],
                        [754888.0, 6541809.0],
                        [761084.0, 6541876.0],
                        [763887.0, 6544360.0],
                        [762385.0, 6549859.0],
                        [763504.0, 6554575.0],
                        [760949.0, 6561975.0],
                        [762003.0, 6573092.0],
                        [769267.0, 6575508.0],
                        [775913.0, 6580395.0],
                        [775079.0, 6589273.0],
                        [776586.0, 6596649.0],
                        [773476.0, 6599308.0],
                        [768312.0, 6598303.0],
                        [763922.0, 6603795.0],
                        [761463.0, 6602522.0],
                        [756931.0, 6607772.0],
                        [753291.0, 6618069.0],
                        [748734.0, 6623209.0],
                        [748049.0, 6627896.0],
                        [742042.0, 6624120.0],
                        [737251.0, 6617690.0],
                        [733164.0, 6623625.0],
                        [726499.0, 6620539.0],
                        [720608.0, 6624106.0],
                        [716475.0, 6620334.0],
                        [703745.0, 6628663.0],
                        [702446.0, 6632753.0],
                    ]
                ],
                "bbox": (644784.0, 6537018.0, 776586.0, 6633749.0),
            }
        },
        77: {
            "geometry": {
                "type": "MultiPolygon",
                "coordinates": [
                    [
                        [
                            [365341.0, 6575902.0],
                            [359213.0, 6576771.0],
                            [352445.0, 6583031.0],
                            [350494.0, 6577212.0],
                            [356155.0, 6576636.0],
                            [364082.0, 6571051.0],
                            [365341.0, 6575902.0],
                        ]
                    ],
                    [
                        [
                            [373352.0, 6542832.0],
                            [371342.0, 6552252.0],
                            [366346.0, 6552808.0],
                            [362447.0, 6556836.0],
                            [360591.0, 6548506.0],
                            [369260.0, 6539881.0],
                            [371124.0, 6531592.0],
                            [374437.0, 6534213.0],
                            [375330.0, 6540510.0],
                            [373352.0, 6542832.0],
                        ]
                    ],
                    [
                        [
                            [464807.0, 6459164.0],
                            [461242.0, 6465553.0],
                            [455660.0, 6465791.0],
                            [456356.0, 6470307.0],
                            [447267.0, 6474282.0],
                            [443202.0, 6478722.0],
                            [445861.0, 6484044.0],
                            [444696.0, 6489309.0],
                            [446521.0, 6493231.0],
                            [442913.0, 6501478.0],
                            [436951.0, 6506668.0],
                            [434607.0, 6512022.0],
                            [434480.0, 6521530.0],
                            [445866.0, 6528260.0],
                            [449713.0, 6524677.0],
                            [455132.0, 6526680.0],
                            [458386.0, 6534878.0],
                            [457303.0, 6539406.0],
                            [459784.0, 6545825.0],
                            [446458.0, 6558600.0],
                            [437311.0, 6559539.0],
                            [429269.0, 6562335.0],
                            [426299.0, 6566639.0],
                            [420640.0, 6566176.0],
                            [415606.0, 6575464.0],
                            [411065.0, 6578554.0],
                            [411453.0, 6585116.0],
                            [404775.0, 6589452.0],
                            [397324.0, 6586760.0],
                            [391500.0, 6591809.0],
                            [382360.0, 6587238.0],
                            [383479.0, 6581738.0],
                            [376439.0, 6576760.0],
                            [374243.0, 6571617.0],
                            [381511.0, 6566597.0],
                            [386552.0, 6553761.0],
                            [382858.0, 6551653.0],
                            [385303.0, 6547012.0],
                            [378912.0, 6532887.0],
                            [370510.0, 6529071.0],
                            [370474.0, 6520657.0],
                            [379564.0, 6513624.0],
                            [387626.0, 6509256.0],
                            [386164.0, 6505245.0],
                            [399796.0, 6489042.0],
                            [404682.0, 6481482.0],
                            [406100.0, 6476140.0],
                            [418478.0, 6475494.0],
                            [421098.0, 6472770.0],
                            [431214.0, 6470295.0],
                            [433418.0, 6457450.0],
                            [437492.0, 6457585.0],
                            [444096.0, 6451365.0],
                            [452009.0, 6448234.0],
                            [457280.0, 6451647.0],
                            [460931.0, 6449393.0],
                            [464807.0, 6459164.0],
                        ]
                    ],
                ],
                "bbox": (350494.0, 6448234.0, 464807.0, 6591809.0),
            }
        },
        78: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [728488.0, 6430309.0],
                        [727352.0, 6435060.0],
                        [723408.0, 6437470.0],
                        [725628.0, 6444038.0],
                        [722648.0, 6446835.0],
                        [717445.0, 6463649.0],
                        [709251.0, 6464988.0],
                        [708104.0, 6472781.0],
                        [701310.0, 6465307.0],
                        [695974.0, 6467752.0],
                        [693859.0, 6474045.0],
                        [688449.0, 6477003.0],
                        [677684.0, 6475835.0],
                        [672750.0, 6482640.0],
                        [661594.0, 6486680.0],
                        [659915.0, 6480065.0],
                        [656321.0, 6476249.0],
                        [651369.0, 6479623.0],
                        [649118.0, 6470007.0],
                        [642886.0, 6465907.0],
                        [636846.0, 6458269.0],
                        [638218.0, 6451518.0],
                        [634831.0, 6442812.0],
                        [628772.0, 6440050.0],
                        [631663.0, 6432186.0],
                        [626129.0, 6431248.0],
                        [628463.0, 6419614.0],
                        [634482.0, 6410441.0],
                        [633003.0, 6406356.0],
                        [634129.0, 6393565.0],
                        [637122.0, 6391031.0],
                        [637228.0, 6394175.0],
                        [646639.0, 6396959.0],
                        [655192.0, 6393469.0],
                        [664849.0, 6402561.0],
                        [665404.0, 6408608.0],
                        [668313.0, 6410487.0],
                        [668604.0, 6416079.0],
                        [676829.0, 6423158.0],
                        [679353.0, 6426925.0],
                        [684359.0, 6419403.0],
                        [688126.0, 6419157.0],
                        [690477.0, 6411547.0],
                        [694820.0, 6408897.0],
                        [693924.0, 6403282.0],
                        [698547.0, 6393953.0],
                        [704864.0, 6413115.0],
                        [707862.0, 6414941.0],
                        [708330.0, 6420849.0],
                        [714401.0, 6418301.0],
                        [719705.0, 6424147.0],
                        [728488.0, 6430309.0],
                    ]
                ],
                "bbox": (626129.0, 6391031.0, 728488.0, 6486680.0),
            }
        },
        79: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [664821.0, 6258302.0],
                        [662679.0, 6249784.0],
                        [667653.0, 6244475.0],
                        [681495.0, 6239871.0],
                        [686427.0, 6247043.0],
                        [700353.0, 6246775.0],
                        [700385.0, 6243263.0],
                        [711963.0, 6238657.0],
                        [716336.0, 6238907.0],
                        [719561.0, 6234874.0],
                        [727933.0, 6241307.0],
                        [734945.0, 6243544.0],
                        [741504.0, 6241707.0],
                        [748880.0, 6250885.0],
                        [753675.0, 6255020.0],
                        [764523.0, 6260540.0],
                        [773373.0, 6269062.0],
                        [781738.0, 6273121.0],
                        [788998.0, 6273432.0],
                        [792922.0, 6276955.0],
                        [796379.0, 6284647.0],
                        [792244.0, 6293431.0],
                        [780634.0, 6301895.0],
                        [773726.0, 6310049.0],
                        [766790.0, 6307842.0],
                        [766255.0, 6313712.0],
                        [763184.0, 6318967.0],
                        [758340.0, 6319159.0],
                        [750088.0, 6313306.0],
                        [746263.0, 6307559.0],
                        [742673.0, 6306434.0],
                        [734096.0, 6312602.0],
                        [728786.0, 6312824.0],
                        [722102.0, 6311077.0],
                        [716344.0, 6301672.0],
                        [704815.0, 6303628.0],
                        [704898.0, 6288229.0],
                        [698819.0, 6289943.0],
                        [694796.0, 6288406.0],
                        [693221.0, 6284557.0],
                        [680178.0, 6279485.0],
                        [676690.0, 6282993.0],
                        [671212.0, 6283521.0],
                        [668935.0, 6274250.0],
                        [672394.0, 6269695.0],
                        [672444.0, 6264623.0],
                        [668090.0, 6259189.0],
                        [664821.0, 6258302.0],
                    ]
                ],
                "bbox": (662679.0, 6234874.0, 796379.0, 6319159.0),
            }
        },
        80: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [921863.0, 6404787.0],
                        [918546.0, 6403480.0],
                        [907889.0, 6409384.0],
                        [894862.0, 6413405.0],
                        [895949.0, 6427870.0],
                        [895595.0, 6439026.0],
                        [892767.0, 6444271.0],
                        [888053.0, 6440277.0],
                        [871457.0, 6445038.0],
                        [871909.0, 6454823.0],
                        [870728.0, 6463360.0],
                        [866406.0, 6463235.0],
                        [860939.0, 6470947.0],
                        [855768.0, 6473591.0],
                        [847361.0, 6468209.0],
                        [841110.0, 6468164.0],
                        [842390.0, 6453321.0],
                        [844782.0, 6435636.0],
                        [848816.0, 6428161.0],
                        [844069.0, 6414766.0],
                        [839715.0, 6410394.0],
                        [841041.0, 6396655.0],
                        [835764.0, 6388411.0],
                        [834538.0, 6384016.0],
                        [835151.0, 6374038.0],
                        [832713.0, 6371581.0],
                        [831614.0, 6360351.0],
                        [840525.0, 6360052.0],
                        [843837.0, 6356928.0],
                        [844895.0, 6349809.0],
                        [864358.0, 6358713.0],
                        [871547.0, 6356009.0],
                        [872115.0, 6350341.0],
                        [890550.0, 6347548.0],
                        [890601.0, 6342463.0],
                        [894810.0, 6342254.0],
                        [899978.0, 6338355.0],
                        [903520.0, 6340766.0],
                        [905908.0, 6346585.0],
                        [911050.0, 6344481.0],
                        [913873.0, 6347227.0],
                        [914428.0, 6355637.0],
                        [911254.0, 6355549.0],
                        [909816.0, 6362305.0],
                        [898744.0, 6362932.0],
                        [892505.0, 6372493.0],
                        [899815.0, 6380077.0],
                        [907085.0, 6377469.0],
                        [909000.0, 6384874.0],
                        [906293.0, 6386082.0],
                        [910174.0, 6394503.0],
                        [921214.0, 6398819.0],
                        [921863.0, 6404787.0],
                    ],
                    [
                        [850837.0, 6364673.0],
                        [852761.0, 6369494.0],
                        [856829.0, 6372215.0],
                        [860545.0, 6368019.0],
                        [860544.0, 6360615.0],
                        [857850.0, 6357360.0],
                        [850729.0, 6357929.0],
                        [850837.0, 6364673.0],
                    ],
                ],
                "bbox": (831614.0, 6338355.0, 921863.0, 6473591.0),
            }
        },
        81: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [600206.0, 6164532.0],
                        [597209.0, 6163462.0],
                        [595443.0, 6155896.0],
                        [604042.0, 6154257.0],
                        [614139.0, 6147692.0],
                        [616327.0, 6140765.0],
                        [624388.0, 6140739.0],
                        [630462.0, 6147428.0],
                        [638840.0, 6149005.0],
                        [653442.0, 6143379.0],
                        [657322.0, 6137875.0],
                        [662029.0, 6137190.0],
                        [665163.0, 6139863.0],
                        [672907.0, 6137962.0],
                        [673207.0, 6145058.0],
                        [679658.0, 6147303.0],
                        [683451.0, 6146537.0],
                        [686936.0, 6150974.0],
                        [703357.0, 6152588.0],
                        [707042.0, 6147292.0],
                        [714275.0, 6148346.0],
                        [710994.0, 6157024.0],
                        [704080.0, 6161154.0],
                        [702907.0, 6175403.0],
                        [703560.0, 6193187.0],
                        [693222.0, 6198384.0],
                        [688989.0, 6202113.0],
                        [682753.0, 6199641.0],
                        [678721.0, 6193157.0],
                        [675246.0, 6192375.0],
                        [659825.0, 6194610.0],
                        [655189.0, 6193206.0],
                        [644930.0, 6193342.0],
                        [647326.0, 6181831.0],
                        [644388.0, 6178927.0],
                        [631545.0, 6174170.0],
                        [617954.0, 6174066.0],
                        [615529.0, 6169131.0],
                        [607482.0, 6165290.0],
                        [600206.0, 6164532.0],
                    ]
                ],
                "bbox": (595443.0, 6137190.0, 714275.0, 6202113.0),
            }
        },
        82: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [956276.0, 6452799.0],
                        [964422.0, 6450656.0],
                        [967061.0, 6446025.0],
                        [974349.0, 6445704.0],
                        [973771.0, 6449910.0],
                        [985382.0, 6452122.0],
                        [989157.0, 6455721.0],
                        [995952.0, 6458223.0],
                        [1002558.0, 6454919.0],
                        [1011443.0, 6464355.0],
                        [1024403.0, 6470318.0],
                        [1021887.0, 6478062.0],
                        [1027167.0, 6487344.0],
                        [1021485.0, 6490016.0],
                        [1020210.0, 6493869.0],
                        [1012252.0, 6497350.0],
                        [1011518.0, 6512378.0],
                        [1006337.0, 6512917.0],
                        [997830.0, 6518696.0],
                        [995359.0, 6526975.0],
                        [988618.0, 6520489.0],
                        [984284.0, 6528734.0],
                        [979688.0, 6528135.0],
                        [975690.0, 6531451.0],
                        [971940.0, 6540371.0],
                        [969073.0, 6537583.0],
                        [965836.0, 6528299.0],
                        [962862.0, 6525460.0],
                        [959117.0, 6516128.0],
                        [952091.0, 6514710.0],
                        [948121.0, 6516633.0],
                        [946066.0, 6522280.0],
                        [936560.0, 6520059.0],
                        [926213.0, 6526897.0],
                        [922988.0, 6530552.0],
                        [921730.0, 6540986.0],
                        [919315.0, 6541572.0],
                        [916332.0, 6528631.0],
                        [915868.0, 6518199.0],
                        [910338.0, 6515974.0],
                        [909413.0, 6509006.0],
                        [904449.0, 6504924.0],
                        [913763.0, 6489603.0],
                        [917457.0, 6486189.0],
                        [925224.0, 6482705.0],
                        [931624.0, 6492445.0],
                        [938381.0, 6486658.0],
                        [943964.0, 6486985.0],
                        [948557.0, 6482085.0],
                        [949441.0, 6473761.0],
                        [945461.0, 6470305.0],
                        [945165.0, 6465401.0],
                        [950505.0, 6456663.0],
                        [956276.0, 6452799.0],
                    ]
                ],
                "bbox": (904449.0, 6445704.0, 1027167.0, 6541572.0),
            }
        },
        83: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [921863.0, 6404787.0],
                        [921214.0, 6398819.0],
                        [910174.0, 6394503.0],
                        [906293.0, 6386082.0],
                        [909000.0, 6384874.0],
                        [907085.0, 6377469.0],
                        [899815.0, 6380077.0],
                        [892505.0, 6372493.0],
                        [898744.0, 6362932.0],
                        [909816.0, 6362305.0],
                        [911254.0, 6355549.0],
                        [914428.0, 6355637.0],
                        [913873.0, 6347227.0],
                        [920148.0, 6349530.0],
                        [930231.0, 6349978.0],
                        [925339.0, 6357350.0],
                        [932421.0, 6356788.0],
                        [935206.0, 6370624.0],
                        [944316.0, 6377139.0],
                        [950982.0, 6378672.0],
                        [957118.0, 6369997.0],
                        [959705.0, 6373500.0],
                        [957182.0, 6379131.0],
                        [967103.0, 6386021.0],
                        [971476.0, 6380340.0],
                        [983401.0, 6378221.0],
                        [988952.0, 6378656.0],
                        [992592.0, 6389297.0],
                        [999315.0, 6394909.0],
                        [1004568.0, 6397351.0],
                        [1010207.0, 6403290.0],
                        [1012913.0, 6402904.0],
                        [1015781.0, 6406977.0],
                        [1022960.0, 6406318.0],
                        [1016213.0, 6418043.0],
                        [1016501.0, 6423604.0],
                        [1010607.0, 6425833.0],
                        [1004832.0, 6424533.0],
                        [995855.0, 6430156.0],
                        [996598.0, 6437292.0],
                        [994575.0, 6442256.0],
                        [989294.0, 6442329.0],
                        [985382.0, 6452122.0],
                        [973771.0, 6449910.0],
                        [974349.0, 6445704.0],
                        [967061.0, 6446025.0],
                        [964422.0, 6450656.0],
                        [956276.0, 6452799.0],
                        [953909.0, 6450690.0],
                        [952342.0, 6439924.0],
                        [961377.0, 6439340.0],
                        [964872.0, 6432532.0],
                        [965006.0, 6422921.0],
                        [960775.0, 6424797.0],
                        [956742.0, 6422339.0],
                        [947667.0, 6423241.0],
                        [941569.0, 6417648.0],
                        [939410.0, 6420031.0],
                        [933177.0, 6416064.0],
                        [930672.0, 6410433.0],
                        [925560.0, 6409806.0],
                        [921863.0, 6404787.0],
                    ]
                ],
                "bbox": (892505.0, 6347227.0, 1022960.0, 6452799.0),
            }
        },
        84: {
            "geometry": {
                "type": "MultiPolygon",
                "coordinates": [
                    [
                        [
                            [850837.0, 6364673.0],
                            [850729.0, 6357929.0],
                            [857850.0, 6357360.0],
                            [860544.0, 6360615.0],
                            [860545.0, 6368019.0],
                            [856829.0, 6372215.0],
                            [852761.0, 6369494.0],
                            [850837.0, 6364673.0],
                        ]
                    ],
                    [
                        [
                            [839658.0, 6315433.0],
                            [848847.0, 6314233.0],
                            [858049.0, 6310039.0],
                            [863255.0, 6305296.0],
                            [865136.0, 6300908.0],
                            [876282.0, 6295340.0],
                            [879627.0, 6296831.0],
                            [886796.0, 6295631.0],
                            [896706.0, 6289935.0],
                            [904218.0, 6287715.0],
                            [910307.0, 6287877.0],
                            [921881.0, 6295602.0],
                            [922160.0, 6296144.0],
                            [913516.0, 6306489.0],
                            [907367.0, 6306820.0],
                            [909386.0, 6316209.0],
                            [901642.0, 6319541.0],
                            [903879.0, 6328807.0],
                            [900470.0, 6332559.0],
                            [899978.0, 6338355.0],
                            [894810.0, 6342254.0],
                            [890601.0, 6342463.0],
                            [890550.0, 6347548.0],
                            [872115.0, 6350341.0],
                            [871547.0, 6356009.0],
                            [864358.0, 6358713.0],
                            [844895.0, 6349809.0],
                            [843837.0, 6356928.0],
                            [840525.0, 6360052.0],
                            [831614.0, 6360351.0],
                            [831641.0, 6353746.0],
                            [837656.0, 6344658.0],
                            [836689.0, 6335330.0],
                            [843231.0, 6331183.0],
                            [847769.0, 6322554.0],
                            [839658.0, 6315433.0],
                        ]
                    ],
                ],
                "bbox": (831614.0, 6287715.0, 922160.0, 6372215.0),
            }
        },
        85: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [636090.0, 6591904.0],
                        [634893.0, 6595681.0],
                        [638969.0, 6598575.0],
                        [635550.0, 6606712.0],
                        [637665.0, 6611745.0],
                        [634340.0, 6622075.0],
                        [628673.0, 6627353.0],
                        [632500.0, 6630848.0],
                        [627496.0, 6637025.0],
                        [633457.0, 6642556.0],
                        [629301.0, 6648616.0],
                        [631131.0, 6657674.0],
                        [627981.0, 6660408.0],
                        [627833.0, 6667741.0],
                        [621646.0, 6669974.0],
                        [615265.0, 6667454.0],
                        [607114.0, 6670785.0],
                        [612309.0, 6675948.0],
                        [612147.0, 6680536.0],
                        [602149.0, 6685865.0],
                        [593597.0, 6687211.0],
                        [582631.0, 6683116.0],
                        [573297.0, 6677558.0],
                        [576079.0, 6670370.0],
                        [571950.0, 6667671.0],
                        [568878.0, 6661167.0],
                        [560408.0, 6661557.0],
                        [553982.0, 6659068.0],
                        [551779.0, 6655211.0],
                        [546519.0, 6631001.0],
                        [537221.0, 6629765.0],
                        [541431.0, 6623495.0],
                        [538906.0, 6616112.0],
                        [542426.0, 6611026.0],
                        [547958.0, 6609437.0],
                        [557842.0, 6602380.0],
                        [558128.0, 6596023.0],
                        [562773.0, 6594026.0],
                        [559931.0, 6588729.0],
                        [569564.0, 6587075.0],
                        [573702.0, 6590223.0],
                        [578129.0, 6584256.0],
                        [586530.0, 6592902.0],
                        [595863.0, 6588324.0],
                        [602234.0, 6588456.0],
                        [609355.0, 6592908.0],
                        [622684.0, 6592821.0],
                        [630016.0, 6590287.0],
                        [636090.0, 6591904.0],
                    ]
                ],
                "bbox": (537221.0, 6584256.0, 638969.0, 6687211.0),
            }
        },
        86: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [1232225.0, 6105798.0],
                        [1232233.0, 6116272.0],
                        [1242213.0, 6134033.0],
                        [1241272.0, 6154177.0],
                        [1238247.0, 6164499.0],
                        [1238682.0, 6169952.0],
                        [1236764.0, 6182904.0],
                        [1228823.0, 6195229.0],
                        [1228376.0, 6197858.0],
                        [1231051.0, 6211355.0],
                        [1228846.0, 6218876.0],
                        [1227050.0, 6231255.0],
                        [1223421.0, 6233783.0],
                        [1216996.0, 6231337.0],
                        [1218749.0, 6222752.0],
                        [1215986.0, 6213431.0],
                        [1218915.0, 6209217.0],
                        [1219580.0, 6202912.0],
                        [1215962.0, 6195537.0],
                        [1212393.0, 6200184.0],
                        [1205077.0, 6201621.0],
                        [1198634.0, 6198671.0],
                        [1193909.0, 6190462.0],
                        [1183050.0, 6187935.0],
                        [1176939.0, 6184145.0],
                        [1176781.0, 6180877.0],
                        [1165934.0, 6173994.0],
                        [1165800.0, 6162888.0],
                        [1161943.0, 6162734.0],
                        [1159342.0, 6158599.0],
                        [1164950.0, 6158109.0],
                        [1180116.0, 6152978.0],
                        [1187662.0, 6146432.0],
                        [1199675.0, 6142105.0],
                        [1202519.0, 6133762.0],
                        [1206639.0, 6131576.0],
                        [1210743.0, 6122769.0],
                        [1215715.0, 6123336.0],
                        [1217754.0, 6104291.0],
                        [1226359.0, 6102880.0],
                        [1232225.0, 6105798.0],
                    ]
                ],
                "bbox": (1159342.0, 6102880.0, 1242213.0, 6233783.0),
            }
        },
        87: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [563452.0, 6484264.0],
                        [566212.0, 6489282.0],
                        [571080.0, 6486618.0],
                        [576059.0, 6493313.0],
                        [581471.0, 6497412.0],
                        [587429.0, 6495459.0],
                        [593766.0, 6498788.0],
                        [605337.0, 6509707.0],
                        [612121.0, 6507862.0],
                        [614305.0, 6511565.0],
                        [612524.0, 6515422.0],
                        [613074.0, 6522626.0],
                        [604752.0, 6527760.0],
                        [597279.0, 6527024.0],
                        [591561.0, 6529605.0],
                        [592200.0, 6538029.0],
                        [586335.0, 6540289.0],
                        [587795.0, 6549437.0],
                        [580656.0, 6565605.0],
                        [574730.0, 6569689.0],
                        [580236.0, 6579624.0],
                        [578129.0, 6584256.0],
                        [573702.0, 6590223.0],
                        [569564.0, 6587075.0],
                        [559931.0, 6588729.0],
                        [556218.0, 6586207.0],
                        [548271.0, 6584464.0],
                        [546474.0, 6577615.0],
                        [538896.0, 6578287.0],
                        [530127.0, 6570272.0],
                        [531964.0, 6561069.0],
                        [531290.0, 6552122.0],
                        [539507.0, 6547743.0],
                        [540666.0, 6541823.0],
                        [534383.0, 6537804.0],
                        [527825.0, 6527023.0],
                        [522669.0, 6525452.0],
                        [522412.0, 6520738.0],
                        [515635.0, 6515544.0],
                        [526849.0, 6509996.0],
                        [524801.0, 6504583.0],
                        [529342.0, 6499739.0],
                        [539039.0, 6503445.0],
                        [546900.0, 6501981.0],
                        [553144.0, 6489340.0],
                        [563452.0, 6484264.0],
                    ]
                ],
                "bbox": (515635.0, 6484264.0, 614305.0, 6590223.0),
            }
        },
        88: {
            "geometry": {
                "type": "MultiPolygon",
                "coordinates": [
                    [
                        [
                            [297360.0, 6667914.0],
                            [299985.0, 6664525.0],
                            [303823.0, 6669495.0],
                            [298725.0, 6671705.0],
                            [297360.0, 6667914.0],
                        ]
                    ],
                    [
                        [
                            [322009.0, 6670666.0],
                            [318317.0, 6668787.0],
                            [315194.0, 6662103.0],
                            [307861.0, 6656132.0],
                            [308108.0, 6649020.0],
                            [322518.0, 6633220.0],
                            [328569.0, 6623483.0],
                            [331126.0, 6610513.0],
                            [333817.0, 6610276.0],
                            [344797.0, 6600884.0],
                            [354269.0, 6598459.0],
                            [356701.0, 6592204.0],
                            [364357.0, 6592471.0],
                            [376108.0, 6582708.0],
                            [376887.0, 6588202.0],
                            [382360.0, 6587238.0],
                            [391500.0, 6591809.0],
                            [397324.0, 6586760.0],
                            [404775.0, 6589452.0],
                            [411453.0, 6585116.0],
                            [426574.0, 6590996.0],
                            [428213.0, 6593481.0],
                            [420470.0, 6597142.0],
                            [422851.0, 6601758.0],
                            [424018.0, 6609992.0],
                            [423543.0, 6619698.0],
                            [421344.0, 6621760.0],
                            [420740.0, 6628764.0],
                            [415662.0, 6636435.0],
                            [417892.0, 6640888.0],
                            [411920.0, 6644974.0],
                            [408792.0, 6655192.0],
                            [404251.0, 6660135.0],
                            [401283.0, 6663962.0],
                            [392012.0, 6664073.0],
                            [385082.0, 6667087.0],
                            [376108.0, 6673611.0],
                            [374065.0, 6668782.0],
                            [367645.0, 6668175.0],
                            [367588.0, 6659427.0],
                            [361112.0, 6656907.0],
                            [359840.0, 6669393.0],
                            [354394.0, 6667534.0],
                            [354064.0, 6663454.0],
                            [357453.0, 6652400.0],
                            [353671.0, 6649968.0],
                            [343129.0, 6653969.0],
                            [338731.0, 6658583.0],
                            [332678.0, 6659222.0],
                            [324590.0, 6666698.0],
                            [322009.0, 6670666.0],
                        ]
                    ],
                ],
                "bbox": (297360.0, 6582708.0, 428213.0, 6673611.0),
            }
        },
        89: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [917346.0, 6234793.0],
                        [919352.0, 6230892.0],
                        [929076.0, 6227424.0],
                        [928534.0, 6223947.0],
                        [932969.0, 6220583.0],
                        [939545.0, 6227616.0],
                        [947016.0, 6224540.0],
                        [954550.0, 6225038.0],
                        [959301.0, 6228480.0],
                        [965861.0, 6230140.0],
                        [974387.0, 6227034.0],
                        [975697.0, 6233209.0],
                        [984286.0, 6234176.0],
                        [989472.0, 6238682.0],
                        [995722.0, 6237173.0],
                        [997536.0, 6239863.0],
                        [998568.0, 6249133.0],
                        [997443.0, 6253468.0],
                        [1002933.0, 6264260.0],
                        [1012590.0, 6265313.0],
                        [1018256.0, 6272482.0],
                        [1014132.0, 6274778.0],
                        [1015523.0, 6285590.0],
                        [1006552.0, 6288390.0],
                        [1003290.0, 6292496.0],
                        [1002326.0, 6300425.0],
                        [992638.0, 6305621.0],
                        [983574.0, 6307147.0],
                        [974774.0, 6305117.0],
                        [974848.0, 6301646.0],
                        [970413.0, 6298787.0],
                        [958836.0, 6305328.0],
                        [956283.0, 6301072.0],
                        [944763.0, 6293054.0],
                        [933734.0, 6296912.0],
                        [923991.0, 6299127.0],
                        [922160.0, 6296144.0],
                        [921881.0, 6295602.0],
                        [926823.0, 6291811.0],
                        [925792.0, 6288611.0],
                        [917893.0, 6286245.0],
                        [916541.0, 6283087.0],
                        [920209.0, 6276225.0],
                        [918385.0, 6268519.0],
                        [922861.0, 6263349.0],
                        [917403.0, 6259253.0],
                        [924245.0, 6244827.0],
                        [918110.0, 6241136.0],
                        [917346.0, 6234793.0],
                    ]
                ],
                "bbox": (916541.0, 6220583.0, 1018256.0, 6307147.0),
            }
        },
        90: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [670056.0, 6856451.0],
                        [662997.0, 6862431.0],
                        [657169.0, 6861180.0],
                        [650942.0, 6857646.0],
                        [650056.0, 6850064.0],
                        [653706.0, 6846856.0],
                        [663488.0, 6848458.0],
                        [668472.0, 6843626.0],
                        [671600.0, 6851283.0],
                        [670056.0, 6856451.0],
                    ]
                ],
                "bbox": (650056.0, 6843626.0, 671600.0, 6862431.0),
            }
        },
        91: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [650942.0, 6857646.0],
                        [645325.0, 6859675.0],
                        [643644.0, 6863501.0],
                        [650140.0, 6866932.0],
                        [648071.0, 6872567.0],
                        [641404.0, 6867928.0],
                        [637864.0, 6864035.0],
                        [637702.0, 6857774.0],
                        [643163.0, 6853168.0],
                        [650056.0, 6850064.0],
                        [650942.0, 6857646.0],
                    ]
                ],
                "bbox": (637702.0, 6850064.0, 650942.0, 6872567.0),
            }
        },
        92: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [698547.0, 6393953.0],
                        [706639.0, 6384441.0],
                        [705449.0, 6378272.0],
                        [710800.0, 6372878.0],
                        [709537.0, 6364934.0],
                        [712305.0, 6356606.0],
                        [709994.0, 6353946.0],
                        [718362.0, 6347498.0],
                        [728598.0, 6344387.0],
                        [729879.0, 6341376.0],
                        [735189.0, 6336775.0],
                        [750700.0, 6336169.0],
                        [750995.0, 6342037.0],
                        [754256.0, 6342872.0],
                        [763780.0, 6336809.0],
                        [768987.0, 6337162.0],
                        [776174.0, 6341769.0],
                        [775968.0, 6350322.0],
                        [773621.0, 6356674.0],
                        [775941.0, 6359276.0],
                        [770571.0, 6365836.0],
                        [775211.0, 6367175.0],
                        [779411.0, 6373918.0],
                        [775042.0, 6386528.0],
                        [771844.0, 6388592.0],
                        [768276.0, 6405343.0],
                        [758530.0, 6415778.0],
                        [752669.0, 6414624.0],
                        [752090.0, 6419191.0],
                        [746937.0, 6419760.0],
                        [746579.0, 6414955.0],
                        [737614.0, 6413026.0],
                        [732994.0, 6422485.0],
                        [732563.0, 6427378.0],
                        [728488.0, 6430309.0],
                        [719705.0, 6424147.0],
                        [714401.0, 6418301.0],
                        [708330.0, 6420849.0],
                        [707862.0, 6414941.0],
                        [704864.0, 6413115.0],
                        [698547.0, 6393953.0],
                    ]
                ],
                "bbox": (698547.0, 6336169.0, 779411.0, 6430309.0),
            }
        },
        93: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [799607.0, 6263119.0],
                        [825918.0, 6262030.0],
                        [828657.0, 6259471.0],
                        [826719.0, 6253810.0],
                        [834804.0, 6251115.0],
                        [842728.0, 6251679.0],
                        [850515.0, 6249931.0],
                        [851283.0, 6257962.0],
                        [859395.0, 6260541.0],
                        [865618.0, 6249711.0],
                        [881792.0, 6250642.0],
                        [887411.0, 6254047.0],
                        [891677.0, 6249845.0],
                        [890480.0, 6245406.0],
                        [892188.0, 6237089.0],
                        [906247.0, 6238345.0],
                        [911635.0, 6232742.0],
                        [917346.0, 6234793.0],
                        [918110.0, 6241136.0],
                        [924245.0, 6244827.0],
                        [917403.0, 6259253.0],
                        [922861.0, 6263349.0],
                        [918385.0, 6268519.0],
                        [920209.0, 6276225.0],
                        [916541.0, 6283087.0],
                        [917893.0, 6286245.0],
                        [925792.0, 6288611.0],
                        [926823.0, 6291811.0],
                        [921881.0, 6295602.0],
                        [910307.0, 6287877.0],
                        [904218.0, 6287715.0],
                        [896706.0, 6289935.0],
                        [886796.0, 6295631.0],
                        [879627.0, 6296831.0],
                        [876282.0, 6295340.0],
                        [865136.0, 6300908.0],
                        [863255.0, 6305296.0],
                        [858049.0, 6310039.0],
                        [848847.0, 6314233.0],
                        [839658.0, 6315433.0],
                        [831988.0, 6308980.0],
                        [833168.0, 6302217.0],
                        [828463.0, 6288899.0],
                        [819908.0, 6290041.0],
                        [815212.0, 6281222.0],
                        [813903.0, 6274579.0],
                        [800175.0, 6267290.0],
                        [799607.0, 6263119.0],
                    ],
                    [
                        [869889.0, 6258065.0],
                        [866899.0, 6258272.0],
                        [866104.0, 6264876.0],
                        [862374.0, 6265459.0],
                        [862969.0, 6272152.0],
                        [870516.0, 6271879.0],
                        [874819.0, 6259107.0],
                        [869889.0, 6258065.0],
                    ],
                ],
                "bbox": (799607.0, 6232742.0, 926823.0, 6315433.0),
            }
        },
        94: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [650942.0, 6857646.0],
                        [657169.0, 6861180.0],
                        [654926.0, 6867065.0],
                        [650140.0, 6866932.0],
                        [643644.0, 6863501.0],
                        [645325.0, 6859675.0],
                        [650942.0, 6857646.0],
                    ]
                ],
                "bbox": (643644.0, 6857646.0, 657169.0, 6867065.0),
            }
        },
        95: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [670056.0, 6856451.0],
                        [667928.0, 6865519.0],
                        [670882.0, 6870004.0],
                        [667303.0, 6878971.0],
                        [660416.0, 6872923.0],
                        [653619.0, 6875101.0],
                        [648071.0, 6872567.0],
                        [650140.0, 6866932.0],
                        [654926.0, 6867065.0],
                        [657169.0, 6861180.0],
                        [662997.0, 6862431.0],
                        [670056.0, 6856451.0],
                    ]
                ],
                "bbox": (648071.0, 6856451.0, 670882.0, 6878971.0),
            }
        },
    },
}

geolayer_fr_dept_data_only = {
    "metadata": {
        "fields": geolayer_data_fields_metadata_complete,
        "name": "FRANCE_DPT_GENERALIZE_LAMB93_ROUND_DATA_ONLY",
    },
    "features": {
        0: {"attributes": {"CODE_DEPT": "32", "NOM_DEPT": "GERS"}},
        1: {"attributes": {"CODE_DEPT": "47", "NOM_DEPT": "LOT-ET-GARONNE"}},
        2: {"attributes": {"CODE_DEPT": "38", "NOM_DEPT": "ISERE"}},
        3: {"attributes": {"CODE_DEPT": "62", "NOM_DEPT": "PAS-DE-CALAIS"}},
        4: {"attributes": {"CODE_DEPT": "08", "NOM_DEPT": "ARDENNES"}},
        5: {"attributes": {"CODE_DEPT": "10", "NOM_DEPT": "AUBE"}},
        6: {"attributes": {"CODE_DEPT": "06", "NOM_DEPT": "ALPES-MARITIMES"}},
        7: {"attributes": {"CODE_DEPT": "42", "NOM_DEPT": "LOIRE"}},
        8: {"attributes": {"CODE_DEPT": "31", "NOM_DEPT": "HAUTE-GARONNE"}},
        9: {"attributes": {"CODE_DEPT": "71", "NOM_DEPT": "SAONE-ET-LOIRE"}},
        10: feature_dpt_data_only_a,
        11: {"attributes": {"CODE_DEPT": "16", "NOM_DEPT": "CHARENTE"}},
        12: {"attributes": {"CODE_DEPT": "50", "NOM_DEPT": "MANCHE"}},
        13: {"attributes": {"CODE_DEPT": "78", "NOM_DEPT": "YVELINES"}},
        14: {"attributes": {"CODE_DEPT": "25", "NOM_DEPT": "DOUBS"}},
        15: {"attributes": {"CODE_DEPT": "55", "NOM_DEPT": "MEUSE"}},
        16: {"attributes": {"CODE_DEPT": "33", "NOM_DEPT": "GIRONDE"}},
        17: {"attributes": {"CODE_DEPT": "14", "NOM_DEPT": "CALVADOS"}},
        18: {"attributes": {"CODE_DEPT": "88", "NOM_DEPT": "VOSGES"}},
        19: {"attributes": {"CODE_DEPT": "18", "NOM_DEPT": "CHER"}},
        20: {"attributes": {"CODE_DEPT": "07", "NOM_DEPT": "ARDECHE"}},
        21: feature_dpt_data_only_b,
        22: {"attributes": {"CODE_DEPT": "64", "NOM_DEPT": "PYRENEES-ATLANTIQUES"}},
        23: {"attributes": {"CODE_DEPT": "41", "NOM_DEPT": "LOIR-ET-CHER"}},
        24: {"attributes": {"CODE_DEPT": "57", "NOM_DEPT": "MOSELLE"}},
        25: {"attributes": {"CODE_DEPT": "86", "NOM_DEPT": "VIENNE"}},
        26: {"attributes": {"CODE_DEPT": "24", "NOM_DEPT": "DORDOGNE"}},
        27: {"attributes": {"CODE_DEPT": "39", "NOM_DEPT": "JURA"}},
        28: {"attributes": {"CODE_DEPT": "82", "NOM_DEPT": "TARN-ET-GARONNE"}},
        29: {"attributes": {"CODE_DEPT": "49", "NOM_DEPT": "MAINE-ET-LOIRE"}},
        30: {"attributes": {"CODE_DEPT": "69", "NOM_DEPT": "RHONE"}},
        31: {"attributes": {"CODE_DEPT": "27", "NOM_DEPT": "EURE"}},
        32: {"attributes": {"CODE_DEPT": "12", "NOM_DEPT": "AVEYRON"}},
        33: {"attributes": {"CODE_DEPT": "23", "NOM_DEPT": "CREUSE"}},
        34: {"attributes": {"CODE_DEPT": "45", "NOM_DEPT": "LOIRET"}},
        35: {"attributes": {"CODE_DEPT": "70", "NOM_DEPT": "HAUTE-SAONE"}},
        36: {"attributes": {"CODE_DEPT": "63", "NOM_DEPT": "PUY-DE-DOME"}},
        37: {"attributes": {"CODE_DEPT": "81", "NOM_DEPT": "TARN"}},
        38: {"attributes": {"CODE_DEPT": "76", "NOM_DEPT": "SEINE-MARITIME"}},
        39: {"attributes": {"CODE_DEPT": "52", "NOM_DEPT": "HAUTE-MARNE"}},
        40: {"attributes": {"CODE_DEPT": "30", "NOM_DEPT": "GARD"}},
        41: {"attributes": {"CODE_DEPT": "67", "NOM_DEPT": "BAS-RHIN"}},
        42: {"attributes": {"CODE_DEPT": "11", "NOM_DEPT": "AUDE"}},
        43: {"attributes": {"CODE_DEPT": "77", "NOM_DEPT": "SEINE-ET-MARNE"}},
        44: {"attributes": {"CODE_DEPT": "80", "NOM_DEPT": "SOMME"}},
        45: {"attributes": {"CODE_DEPT": "43", "NOM_DEPT": "HAUTE-LOIRE"}},
        46: {"attributes": {"CODE_DEPT": "51", "NOM_DEPT": "MARNE"}},
        47: {"attributes": {"CODE_DEPT": "65", "NOM_DEPT": "HAUTES-PYRENEES"}},
        48: {"attributes": {"CODE_DEPT": "46", "NOM_DEPT": "LOT"}},
        49: {"attributes": {"CODE_DEPT": "04", "NOM_DEPT": "ALPES-DE-HAUTE-PROVENCE"}},
        50: {"attributes": {"CODE_DEPT": "72", "NOM_DEPT": "SARTHE"}},
        51: feature_dpt_data_only_d,
        52: {"attributes": {"CODE_DEPT": "28", "NOM_DEPT": "EURE-ET-LOIR"}},
        53: {"attributes": {"CODE_DEPT": "2A", "NOM_DEPT": "CORSE-DU-SUD"}},
        54: {"attributes": {"CODE_DEPT": "54", "NOM_DEPT": "MEURTHE-ET-MOSELLE"}},
        55: {"attributes": {"CODE_DEPT": "61", "NOM_DEPT": "ORNE"}},
        56: {"attributes": {"CODE_DEPT": "01", "NOM_DEPT": "AIN"}},
        57: {"attributes": {"CODE_DEPT": "09", "NOM_DEPT": "ARIEGE"}},
        58: {"attributes": {"CODE_DEPT": "19", "NOM_DEPT": "CORREZE"}},
        59: {"attributes": {"CODE_DEPT": "68", "NOM_DEPT": "HAUT-RHIN"}},
        60: {"attributes": {"CODE_DEPT": "37", "NOM_DEPT": "INDRE-ET-LOIRE"}},
        61: {"attributes": {"CODE_DEPT": "59", "NOM_DEPT": "NORD"}},
        62: {"attributes": {"CODE_DEPT": "90", "NOM_DEPT": "TERRITOIRE DE BELFORT"}},
        63: {"attributes": {"CODE_DEPT": "44", "NOM_DEPT": "LOIRE-ATLANTIQUE"}},
        64: {"attributes": {"CODE_DEPT": "89", "NOM_DEPT": "YONNE"}},
        65: {"attributes": {"CODE_DEPT": "35", "NOM_DEPT": "ILLE-ET-VILAINE"}},
        66: {"attributes": {"CODE_DEPT": "40", "NOM_DEPT": "LANDES"}},
        67: {"attributes": {"CODE_DEPT": "29", "NOM_DEPT": "FINISTERE"}},
        68: {"attributes": {"CODE_DEPT": "74", "NOM_DEPT": "HAUTE-SAVOIE"}},
        69: {"attributes": {"CODE_DEPT": "60", "NOM_DEPT": "OISE"}},
        70: feature_dpt_data_only_c,
        71: {"attributes": {"CODE_DEPT": "58", "NOM_DEPT": "NIEVRE"}},
        72: {"attributes": {"CODE_DEPT": "21", "NOM_DEPT": "COTE-D'OR"}},
        73: {"attributes": {"CODE_DEPT": "91", "NOM_DEPT": "ESSONNE"}},
        74: {"attributes": {"CODE_DEPT": "79", "NOM_DEPT": "DEUX-SEVRES"}},
        75: {"attributes": {"CODE_DEPT": "22", "NOM_DEPT": "COTES-D'ARMOR"}},
        76: {"attributes": {"CODE_DEPT": "03", "NOM_DEPT": "ALLIER"}},
        77: {"attributes": {"CODE_DEPT": "17", "NOM_DEPT": "CHARENTE-MARITIME"}},
        78: {"attributes": {"CODE_DEPT": "15", "NOM_DEPT": "CANTAL"}},
        79: {"attributes": {"CODE_DEPT": "34", "NOM_DEPT": "HERAULT"}},
        80: {"attributes": {"CODE_DEPT": "26", "NOM_DEPT": "DROME"}},
        81: {"attributes": {"CODE_DEPT": "66", "NOM_DEPT": "PYRENEES-ORIENTALES"}},
        82: {"attributes": {"CODE_DEPT": "73", "NOM_DEPT": "SAVOIE"}},
        83: {"attributes": {"CODE_DEPT": "05", "NOM_DEPT": "HAUTES-ALPES"}},
        84: {"attributes": {"CODE_DEPT": "84", "NOM_DEPT": "VAUCLUSE"}},
        85: {"attributes": {"CODE_DEPT": "36", "NOM_DEPT": "INDRE"}},
        86: {"attributes": {"CODE_DEPT": "2B", "NOM_DEPT": "HAUTE-CORSE"}},
        87: {"attributes": {"CODE_DEPT": "87", "NOM_DEPT": "HAUTE-VIENNE"}},
        88: {"attributes": {"CODE_DEPT": "85", "NOM_DEPT": "VENDEE"}},
        89: {"attributes": {"CODE_DEPT": "83", "NOM_DEPT": "VAR"}},
        90: {"attributes": {"CODE_DEPT": "94", "NOM_DEPT": "VAL-DE-MARNE"}},
        91: {"attributes": {"CODE_DEPT": "92", "NOM_DEPT": "HAUTS-DE-SEINE"}},
        92: {"attributes": {"CODE_DEPT": "48", "NOM_DEPT": "LOZERE"}},
        93: {"attributes": {"CODE_DEPT": "13", "NOM_DEPT": "BOUCHES-DU-RHONE"}},
        94: {"attributes": {"CODE_DEPT": "75", "NOM_DEPT": "PARIS"}},
        95: {"attributes": {"CODE_DEPT": "93", "NOM_DEPT": "SEINE-SAINT-DENIS"}},
    },
}
geolayer_fr_dept_population = {
    "metadata": {
        "name": "dept_population",
        "fields": {
            "CODE_DEPT": {"type": "String", "width": 2, "index": 0},
            "INSEE_REG": {"type": "String", "width": 2, "index": 1},
            "POPULATION": {"type": "Integer", "index": 2},
            "AREA": {"type": "Real", "width": 7, "precision": 2, "index": 3},
            "DENSITY": {"type": "Real", "width": 7, "precision": 2, "index": 4},
        },
    },
    "features": {
        0: {
            "attributes": {
                "CODE_DEPT": "32",
                "INSEE_REG": "76",
                "POPULATION": 191091,
                "AREA": 6304.33,
                "DENSITY": 30.31,
            }
        },
        1: {
            "attributes": {
                "CODE_DEPT": "47",
                "INSEE_REG": "75",
                "POPULATION": 332842,
                "AREA": 5382.87,
                "DENSITY": 61.83,
            }
        },
        2: {
            "attributes": {
                "CODE_DEPT": "38",
                "INSEE_REG": "84",
                "POPULATION": 1258722,
                "AREA": 7868.79,
                "DENSITY": 159.96,
            }
        },
        3: {
            "attributes": {
                "CODE_DEPT": "62",
                "INSEE_REG": "32",
                "POPULATION": 1468018,
                "AREA": 6714.14,
                "DENSITY": 218.65,
            }
        },
        4: {
            "attributes": {
                "CODE_DEPT": "08",
                "INSEE_REG": "44",
                "POPULATION": 273579,
                "AREA": 5253.13,
                "DENSITY": 52.08,
            }
        },
        5: {
            "attributes": {
                "CODE_DEPT": "10",
                "INSEE_REG": "44",
                "POPULATION": 310020,
                "AREA": 6021.83,
                "DENSITY": 51.48,
            }
        },
        6: {
            "attributes": {
                "CODE_DEPT": "42",
                "INSEE_REG": "84",
                "POPULATION": 762941,
                "AREA": 4795.85,
                "DENSITY": 159.08,
            }
        },
        7: {
            "attributes": {
                "CODE_DEPT": "06",
                "INSEE_REG": "93",
                "POPULATION": 1083310,
                "AREA": 4291.62,
                "DENSITY": 252.42,
            }
        },
        8: {
            "attributes": {
                "CODE_DEPT": "31",
                "INSEE_REG": "76",
                "POPULATION": 1362672,
                "AREA": 6364.82,
                "DENSITY": 214.09,
            }
        },
        9: {
            "attributes": {
                "CODE_DEPT": "71",
                "INSEE_REG": "27",
                "POPULATION": 553595,
                "AREA": 8598.33,
                "DENSITY": 64.38,
            }
        },
        10: {
            "attributes": {
                "CODE_DEPT": "53",
                "INSEE_REG": "52",
                "POPULATION": 307445,
                "AREA": 5208.37,
                "DENSITY": 59.03,
            }
        },
        11: {
            "attributes": {
                "CODE_DEPT": "78",
                "INSEE_REG": "11",
                "POPULATION": 1438266,
                "AREA": 2305.64,
                "DENSITY": 623.8,
            }
        },
        12: {
            "attributes": {
                "CODE_DEPT": "50",
                "INSEE_REG": "28",
                "POPULATION": 496883,
                "AREA": 6015.07,
                "DENSITY": 82.61,
            }
        },
        13: {
            "attributes": {
                "CODE_DEPT": "16",
                "INSEE_REG": "75",
                "POPULATION": 352335,
                "AREA": 5963.54,
                "DENSITY": 59.08,
            }
        },
        14: {
            "attributes": {
                "CODE_DEPT": "25",
                "INSEE_REG": "27",
                "POPULATION": 539067,
                "AREA": 5248.31,
                "DENSITY": 102.71,
            }
        },
        15: {
            "attributes": {
                "CODE_DEPT": "55",
                "INSEE_REG": "44",
                "POPULATION": 187187,
                "AREA": 6233.18,
                "DENSITY": 30.03,
            }
        },
        16: {
            "attributes": {
                "CODE_DEPT": "33",
                "INSEE_REG": "75",
                "POPULATION": 1583384,
                "AREA": 10068.74,
                "DENSITY": 157.26,
            }
        },
        17: {
            "attributes": {
                "CODE_DEPT": "14",
                "INSEE_REG": "28",
                "POPULATION": 694002,
                "AREA": 5588.48,
                "DENSITY": 124.18,
            }
        },
        18: {
            "attributes": {
                "CODE_DEPT": "88",
                "INSEE_REG": "44",
                "POPULATION": 367673,
                "AREA": 5891.56,
                "DENSITY": 62.41,
            }
        },
        19: {
            "attributes": {
                "CODE_DEPT": "18",
                "INSEE_REG": "24",
                "POPULATION": 304256,
                "AREA": 7292.67,
                "DENSITY": 41.72,
            }
        },
        20: {
            "attributes": {
                "CODE_DEPT": "07",
                "INSEE_REG": "84",
                "POPULATION": 325712,
                "AREA": 5562.05,
                "DENSITY": 58.56,
            }
        },
        21: {
            "attributes": {
                "CODE_DEPT": "02",
                "INSEE_REG": "32",
                "POPULATION": 534490,
                "AREA": 7418.97,
                "DENSITY": 72.04,
            }
        },
        22: {
            "attributes": {
                "CODE_DEPT": "64",
                "INSEE_REG": "75",
                "POPULATION": 677309,
                "AREA": 7691.6,
                "DENSITY": 88.06,
            }
        },
        23: {
            "attributes": {
                "CODE_DEPT": "41",
                "INSEE_REG": "24",
                "POPULATION": 331915,
                "AREA": 6412.3,
                "DENSITY": 51.76,
            }
        },
        24: {
            "attributes": {
                "CODE_DEPT": "57",
                "INSEE_REG": "44",
                "POPULATION": 1043522,
                "AREA": 6252.63,
                "DENSITY": 166.89,
            }
        },
        25: {
            "attributes": {
                "CODE_DEPT": "86",
                "INSEE_REG": "75",
                "POPULATION": 436876,
                "AREA": 7025.24,
                "DENSITY": 62.19,
            }
        },
        26: {
            "attributes": {
                "CODE_DEPT": "24",
                "INSEE_REG": "75",
                "POPULATION": 413606,
                "AREA": 9209.9,
                "DENSITY": 44.91,
            }
        },
        27: {
            "attributes": {
                "CODE_DEPT": "39",
                "INSEE_REG": "27",
                "POPULATION": 260188,
                "AREA": 5040.63,
                "DENSITY": 51.62,
            }
        },
        28: {
            "attributes": {
                "CODE_DEPT": "82",
                "INSEE_REG": "76",
                "POPULATION": 258349,
                "AREA": 3731.0,
                "DENSITY": 69.24,
            }
        },
        29: {
            "attributes": {
                "CODE_DEPT": "49",
                "INSEE_REG": "52",
                "POPULATION": 813493,
                "AREA": 7161.34,
                "DENSITY": 113.6,
            }
        },
        30: {
            "attributes": {
                "CODE_DEPT": "69",
                "INSEE_REG": "84",
                "POPULATION": 1843319,
                "AREA": 3253.11,
                "DENSITY": 566.63,
            }
        },
        31: {
            "attributes": {
                "CODE_DEPT": "12",
                "INSEE_REG": "76",
                "POPULATION": 279206,
                "AREA": 8770.69,
                "DENSITY": 31.83,
            }
        },
        32: {
            "attributes": {
                "CODE_DEPT": "23",
                "INSEE_REG": "75",
                "POPULATION": 118638,
                "AREA": 5589.16,
                "DENSITY": 21.23,
            }
        },
        33: {
            "attributes": {
                "CODE_DEPT": "45",
                "INSEE_REG": "24",
                "POPULATION": 678008,
                "AREA": 6804.01,
                "DENSITY": 99.65,
            }
        },
        34: {
            "attributes": {
                "CODE_DEPT": "70",
                "INSEE_REG": "27",
                "POPULATION": 236659,
                "AREA": 5382.37,
                "DENSITY": 43.97,
            }
        },
        35: {
            "attributes": {
                "CODE_DEPT": "63",
                "INSEE_REG": "84",
                "POPULATION": 653742,
                "AREA": 8003.1,
                "DENSITY": 81.69,
            }
        },
        36: {
            "attributes": {
                "CODE_DEPT": "81",
                "INSEE_REG": "76",
                "POPULATION": 387890,
                "AREA": 5785.79,
                "DENSITY": 67.04,
            }
        },
        37: {
            "attributes": {
                "CODE_DEPT": "27",
                "INSEE_REG": "28",
                "POPULATION": 601843,
                "AREA": 6035.85,
                "DENSITY": 99.71,
            }
        },
        38: {
            "attributes": {
                "CODE_DEPT": "76",
                "INSEE_REG": "28",
                "POPULATION": 1254378,
                "AREA": 6318.26,
                "DENSITY": 198.53,
            }
        },
        39: {
            "attributes": {
                "CODE_DEPT": "52",
                "INSEE_REG": "44",
                "POPULATION": 175640,
                "AREA": 6249.91,
                "DENSITY": 28.1,
            }
        },
        40: {
            "attributes": {
                "CODE_DEPT": "30",
                "INSEE_REG": "76",
                "POPULATION": 744178,
                "AREA": 5874.71,
                "DENSITY": 126.67,
            }
        },
        41: {
            "attributes": {
                "CODE_DEPT": "67",
                "INSEE_REG": "44",
                "POPULATION": 1125559,
                "AREA": 4796.37,
                "DENSITY": 234.67,
            }
        },
        42: {
            "attributes": {
                "CODE_DEPT": "11",
                "INSEE_REG": "76",
                "POPULATION": 370260,
                "AREA": 6351.35,
                "DENSITY": 58.3,
            }
        },
        43: {
            "attributes": {
                "CODE_DEPT": "77",
                "INSEE_REG": "11",
                "POPULATION": 1403997,
                "AREA": 5924.64,
                "DENSITY": 236.98,
            }
        },
        44: {
            "attributes": {
                "CODE_DEPT": "43",
                "INSEE_REG": "84",
                "POPULATION": 227283,
                "AREA": 4996.58,
                "DENSITY": 45.49,
            }
        },
        45: {
            "attributes": {
                "CODE_DEPT": "51",
                "INSEE_REG": "44",
                "POPULATION": 568895,
                "AREA": 8195.78,
                "DENSITY": 69.41,
            }
        },
        46: {
            "attributes": {
                "CODE_DEPT": "80",
                "INSEE_REG": "32",
                "POPULATION": 572443,
                "AREA": 6206.58,
                "DENSITY": 92.23,
            }
        },
        47: {
            "attributes": {
                "CODE_DEPT": "46",
                "INSEE_REG": "76",
                "POPULATION": 173828,
                "AREA": 5221.64,
                "DENSITY": 33.29,
            }
        },
        48: {
            "attributes": {
                "CODE_DEPT": "65",
                "INSEE_REG": "76",
                "POPULATION": 228530,
                "AREA": 4527.89,
                "DENSITY": 50.47,
            }
        },
        49: {
            "attributes": {
                "CODE_DEPT": "04",
                "INSEE_REG": "93",
                "POPULATION": 163915,
                "AREA": 6993.79,
                "DENSITY": 23.44,
            }
        },
        50: {
            "attributes": {
                "CODE_DEPT": "72",
                "INSEE_REG": "52",
                "POPULATION": 566506,
                "AREA": 6236.75,
                "DENSITY": 90.83,
            }
        },
        51: {
            "attributes": {
                "CODE_DEPT": "56",
                "INSEE_REG": "53",
                "POPULATION": 750863,
                "AREA": 6864.07,
                "DENSITY": 109.39,
            }
        },
        52: {
            "attributes": {
                "CODE_DEPT": "2A",
                "INSEE_REG": "94",
                "POPULATION": 157249,
                "AREA": 4028.53,
                "DENSITY": 39.03,
            }
        },
        53: {
            "attributes": {
                "CODE_DEPT": "28",
                "INSEE_REG": "24",
                "POPULATION": 433233,
                "AREA": 5927.23,
                "DENSITY": 73.09,
            }
        },
        54: {
            "attributes": {
                "CODE_DEPT": "54",
                "INSEE_REG": "44",
                "POPULATION": 733481,
                "AREA": 5283.29,
                "DENSITY": 138.83,
            }
        },
        55: {
            "attributes": {
                "CODE_DEPT": "01",
                "INSEE_REG": "84",
                "POPULATION": 643350,
                "AREA": 5773.77,
                "DENSITY": 111.43,
            }
        },
        56: {
            "attributes": {
                "CODE_DEPT": "19",
                "INSEE_REG": "75",
                "POPULATION": 241464,
                "AREA": 5888.93,
                "DENSITY": 41.0,
            }
        },
        57: {
            "attributes": {
                "CODE_DEPT": "09",
                "INSEE_REG": "76",
                "POPULATION": 153153,
                "AREA": 4921.75,
                "DENSITY": 31.12,
            }
        },
        58: {
            "attributes": {
                "CODE_DEPT": "68",
                "INSEE_REG": "44",
                "POPULATION": 764030,
                "AREA": 3526.37,
                "DENSITY": 216.66,
            }
        },
        59: {
            "attributes": {
                "CODE_DEPT": "59",
                "INSEE_REG": "32",
                "POPULATION": 2604361,
                "AREA": 5774.99,
                "DENSITY": 450.97,
            }
        },
        60: {
            "attributes": {
                "CODE_DEPT": "90",
                "INSEE_REG": "27",
                "POPULATION": 142622,
                "AREA": 609.64,
                "DENSITY": 233.94,
            }
        },
        61: {
            "attributes": {
                "CODE_DEPT": "44",
                "INSEE_REG": "52",
                "POPULATION": 1394909,
                "AREA": 6992.78,
                "DENSITY": 199.48,
            }
        },
        62: {
            "attributes": {
                "CODE_DEPT": "89",
                "INSEE_REG": "27",
                "POPULATION": 338291,
                "AREA": 7450.97,
                "DENSITY": 45.4,
            }
        },
        63: {
            "attributes": {
                "CODE_DEPT": "35",
                "INSEE_REG": "53",
                "POPULATION": 1060199,
                "AREA": 6830.2,
                "DENSITY": 155.22,
            }
        },
        64: {
            "attributes": {
                "CODE_DEPT": "40",
                "INSEE_REG": "75",
                "POPULATION": 407444,
                "AREA": 9353.03,
                "DENSITY": 43.56,
            }
        },
        65: {
            "attributes": {
                "CODE_DEPT": "29",
                "INSEE_REG": "53",
                "POPULATION": 909028,
                "AREA": 6756.76,
                "DENSITY": 134.54,
            }
        },
        66: {
            "attributes": {
                "CODE_DEPT": "74",
                "INSEE_REG": "84",
                "POPULATION": 807360,
                "AREA": 4596.53,
                "DENSITY": 175.65,
            }
        },
        67: {
            "attributes": {
                "CODE_DEPT": "60",
                "INSEE_REG": "32",
                "POPULATION": 824503,
                "AREA": 5893.6,
                "DENSITY": 139.9,
            }
        },
        68: {
            "attributes": {
                "CODE_DEPT": "95",
                "INSEE_REG": "11",
                "POPULATION": 1228618,
                "AREA": 1254.18,
                "DENSITY": 979.62,
            }
        },
        69: {
            "attributes": {
                "CODE_DEPT": "58",
                "INSEE_REG": "27",
                "POPULATION": 207182,
                "AREA": 6862.87,
                "DENSITY": 30.19,
            }
        },
        70: {
            "attributes": {
                "CODE_DEPT": "61",
                "INSEE_REG": "28",
                "POPULATION": 283372,
                "AREA": 6142.73,
                "DENSITY": 46.13,
            }
        },
        71: {
            "attributes": {
                "CODE_DEPT": "91",
                "INSEE_REG": "11",
                "POPULATION": 1296130,
                "AREA": 1818.35,
                "DENSITY": 712.81,
            }
        },
        72: {
            "attributes": {
                "CODE_DEPT": "21",
                "INSEE_REG": "27",
                "POPULATION": 532871,
                "AREA": 8787.51,
                "DENSITY": 60.64,
            }
        },
        73: {
            "attributes": {
                "CODE_DEPT": "22",
                "INSEE_REG": "53",
                "POPULATION": 598814,
                "AREA": 6963.26,
                "DENSITY": 86.0,
            }
        },
        74: {
            "attributes": {
                "CODE_DEPT": "03",
                "INSEE_REG": "84",
                "POPULATION": 337988,
                "AREA": 7365.26,
                "DENSITY": 45.89,
            }
        },
        75: {
            "attributes": {
                "CODE_DEPT": "17",
                "INSEE_REG": "75",
                "POPULATION": 644303,
                "AREA": 6913.03,
                "DENSITY": 93.2,
            }
        },
        76: {
            "attributes": {
                "CODE_DEPT": "15",
                "INSEE_REG": "84",
                "POPULATION": 145143,
                "AREA": 5767.47,
                "DENSITY": 25.17,
            }
        },
        77: {
            "attributes": {
                "CODE_DEPT": "34",
                "INSEE_REG": "76",
                "POPULATION": 1144892,
                "AREA": 6231.05,
                "DENSITY": 183.74,
            }
        },
        78: {
            "attributes": {
                "CODE_DEPT": "26",
                "INSEE_REG": "84",
                "POPULATION": 511553,
                "AREA": 6553.53,
                "DENSITY": 78.06,
            }
        },
        79: {
            "attributes": {
                "CODE_DEPT": "66",
                "INSEE_REG": "76",
                "POPULATION": 474452,
                "AREA": 4147.76,
                "DENSITY": 114.39,
            }
        },
        80: {
            "attributes": {
                "CODE_DEPT": "73",
                "INSEE_REG": "84",
                "POPULATION": 431174,
                "AREA": 6260.4,
                "DENSITY": 68.87,
            }
        },
        81: {
            "attributes": {
                "CODE_DEPT": "37",
                "INSEE_REG": "24",
                "POPULATION": 606511,
                "AREA": 6147.6,
                "DENSITY": 98.66,
            }
        },
        82: {
            "attributes": {
                "CODE_DEPT": "05",
                "INSEE_REG": "93",
                "POPULATION": 141284,
                "AREA": 5685.31,
                "DENSITY": 24.85,
            }
        },
        83: {
            "attributes": {
                "CODE_DEPT": "79",
                "INSEE_REG": "75",
                "POPULATION": 374351,
                "AREA": 6029.06,
                "DENSITY": 62.09,
            }
        },
        84: {
            "attributes": {
                "CODE_DEPT": "84",
                "INSEE_REG": "93",
                "POPULATION": 559479,
                "AREA": 3577.19,
                "DENSITY": 156.4,
            }
        },
        85: {
            "attributes": {
                "CODE_DEPT": "36",
                "INSEE_REG": "24",
                "POPULATION": 222232,
                "AREA": 6887.38,
                "DENSITY": 32.27,
            }
        },
        86: {
            "attributes": {
                "CODE_DEPT": "2B",
                "INSEE_REG": "94",
                "POPULATION": 177689,
                "AREA": 4719.71,
                "DENSITY": 37.65,
            }
        },
        87: {
            "attributes": {
                "CODE_DEPT": "87",
                "INSEE_REG": "75",
                "POPULATION": 374426,
                "AREA": 5549.31,
                "DENSITY": 67.47,
            }
        },
        88: {
            "attributes": {
                "CODE_DEPT": "85",
                "INSEE_REG": "52",
                "POPULATION": 675247,
                "AREA": 6758.23,
                "DENSITY": 99.91,
            }
        },
        89: {
            "attributes": {
                "CODE_DEPT": "83",
                "INSEE_REG": "93",
                "POPULATION": 1058740,
                "AREA": 6002.84,
                "DENSITY": 176.37,
            }
        },
        90: {
            "attributes": {
                "CODE_DEPT": "94",
                "INSEE_REG": "11",
                "POPULATION": 1387926,
                "AREA": 244.7,
                "DENSITY": 5671.95,
            }
        },
        91: {
            "attributes": {
                "CODE_DEPT": "92",
                "INSEE_REG": "11",
                "POPULATION": 1609306,
                "AREA": 175.63,
                "DENSITY": 9163.05,
            }
        },
        92: {
            "attributes": {
                "CODE_DEPT": "48",
                "INSEE_REG": "76",
                "POPULATION": 76601,
                "AREA": 5172.02,
                "DENSITY": 14.81,
            }
        },
        93: {
            "attributes": {
                "CODE_DEPT": "13",
                "INSEE_REG": "93",
                "POPULATION": 2024162,
                "AREA": 5082.57,
                "DENSITY": 398.26,
            }
        },
        94: {
            "attributes": {
                "CODE_DEPT": "93",
                "INSEE_REG": "11",
                "POPULATION": 1623111,
                "AREA": 236.96,
                "DENSITY": 6849.73,
            }
        },
        95: {
            "attributes": {
                "CODE_DEPT": "75",
                "INSEE_REG": "11",
                "POPULATION": 2187526,
                "AREA": 105.44,
                "DENSITY": 20746.64,
            }
        },
    },
}


geolayer_fields_with_bytes_values = {
    "metadata": {
        "name": "bytes_values",
        "fields": {
            "field_bytes": {"type": "Binary", "index": 0},
            "field_hexa_a": {"type": "String", "width": 1116, "index": 1},
            "field_hexa_b": {"type": "String", "width": 1116, "index": 2},
            "field_bytes_hexa_a": {"type": "String", "width": 1647, "index": 3},
            "field_bytes_hexa_b": {"type": "String", "width": 1116, "index": 4},
        },
    },
    "features": {
        0: feature_attributes_bytes_only_a,
        1: {
            "attributes": {
                "field_bytes": None,
                "field_hexa_a": POINT_WKB_HEX_BIG_ENDIAN,
                "field_hexa_b": "hello world",
                "field_bytes_hexa_a": None,
                "field_bytes_hexa_b": POINT_WKB_BIG_ENDIAN.hex(),
            }
        },
        2: {
            "attributes": {
                "field_bytes": GEOMETRYCOLLECTION_WKB_BIG_ENDIAN,
                "field_hexa_a": GEOMETRYCOLLECTION_WKB_HEX_BIG_ENDIAN,
                "field_hexa_b": GEOMETRYCOLLECTION_WKB_HEX_BIG_ENDIAN,
                "field_bytes_hexa_a": GEOMETRYCOLLECTION_WKB_BIG_ENDIAN.hex(),
                "field_bytes_hexa_b": GEOMETRYCOLLECTION_WKB_HEX_BIG_ENDIAN,
            }
        },
    },
}

geolayer_fields_with_bytes_values_forced = {
    "metadata": {
        "name": "bytes_values",
        "fields": {
            "field_bytes": {"type": "Binary", "index": 0},
            "field_hexa_a": {"type": "Binary", "index": 1},
            "field_hexa_b": {"type": "String", "width": 1116, "index": 2},
            "field_bytes_hexa_a": {"type": "Binary", "index": 3},
            "field_bytes_hexa_b": {"type": "String", "width": 1116, "index": 4},
        },
    },
    "features": {
        0: {
            "attributes": {
                "field_bytes": POINT_WKB_BIG_ENDIAN,
                "field_hexa_a": None,
                "field_hexa_b": POINT_WKB_HEX_BIG_ENDIAN,
                "field_bytes_hexa_a": bytes.fromhex(POINT_WKB_HEX_BIG_ENDIAN),
                "field_bytes_hexa_b": "hello world",
            }
        },
        1: {
            "attributes": {
                "field_bytes": None,
                "field_hexa_a": bytes.fromhex(POINT_WKB_HEX_BIG_ENDIAN),
                "field_hexa_b": "hello world",
                "field_bytes_hexa_a": None,
                "field_bytes_hexa_b": POINT_WKB_BIG_ENDIAN.hex(),
            }
        },
        2: {
            "attributes": {
                "field_bytes": GEOMETRYCOLLECTION_WKB_BIG_ENDIAN,
                "field_hexa_a": bytes.fromhex(GEOMETRYCOLLECTION_WKB_HEX_BIG_ENDIAN),
                "field_hexa_b": GEOMETRYCOLLECTION_WKB_HEX_BIG_ENDIAN,
                "field_bytes_hexa_a": GEOMETRYCOLLECTION_WKB_BIG_ENDIAN,
                "field_bytes_hexa_b": GEOMETRYCOLLECTION_WKB_HEX_BIG_ENDIAN,
            }
        },
    },
}


feature_list_data_and_geometry_geolayer = {
    "metadata": {
        "name": "data_and_geometries",
        "fields": geolayer_data_fields_metadata_extract,
        "geometry_ref": {"type": {"Polygon", "MultiPolygon"}, "crs": 2154},
    },
    "features": {
        0: feature_dpt_data_and_geometry_a,
        1: feature_dpt_data_and_geometry_b,
        2: feature_dpt_data_and_geometry_c,
        3: feature_dpt_data_and_geometry_d,
    },
}

feature_list_data_and_geometry_serialized_geolayer = {
    "metadata": {
        "name": "data_and_geometries",
        "fields": geolayer_data_fields_metadata_extract,
        "geometry_ref": {"type": {"Polygon", "MultiPolygon"}, "crs": 2154},
        "feature_serialize": True,
    },
    "features": {
        0: feature_dpt_data_and_geometry_a_serialized,
        1: feature_dpt_data_and_geometry_b_serialized,
        2: feature_dpt_data_and_geometry_c_serialized,
        3: feature_dpt_data_and_geometry_d_serialized,
    },
}

geolayer_for_index = {
    "metadata": {"geometry_ref": {"type": "Polygon"}, "name": "geolayer_for_index"},
    "features": {
        0: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    ((3.78, 10.0), (-14.0, 1.52), (3.78, 6.5), (8.0, 3.0), (3.78, 10.0))
                ],
            }
        },
        1: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [((3.0, 3.0), (8.0, -2.0), (-3.0, -3.0), (3.0, 3.0))],
            }
        },
        2: {
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    ((-13.28, -5.0), (-9.0, -2.2), (-1.0, -8.0), (-13.28, -5.0))
                ],
            }
        },
    },
}

geolayer_for_index_width = {
    "metadata": {
        "geometry_ref": {"type": "LineString"},
        "name": "geolayer_for_index_width",
    },
    "features": {
        0: {
            "geometry": {
                "type": "LineString",
                "coordinates": [
                    [167952.16853944218, 5388680.860679348],
                    [167972.2060477877, 5388818.984447511],
                ],
            }
        }
    },
}

geolayer_for_index_height = {
    "metadata": {
        "geometry_ref": {"type": "LineString"},
        "name": "geolayer_for_index_height",
    },
    "features": {
        0: {
            "geometry": {
                "type": "LineString",
                "coordinates": [
                    [168420.82359568248, 5386574.910679045],
                    [168474.70014136125, 5386578.5641577],
                ],
            }
        }
    },
}

geolayer_for_index_width_height = {
    "metadata": {
        "geometry_ref": {"type": "LineString"},
        "name": "geolayer_for_index_width_height",
    },
    "features": {
        0: {
            "geometry": {
                "type": "LineString",
                "coordinates": [
                    [168148.09084323974, 5388941.762858973],
                    [169325.85105583287, 5387529.909342526],
                ],
            }
        }
    },
}

geolayer_for_index_border_width_height = {
    "metadata": {
        "geometry_ref": {"type": "LineString"},
        "name": "geolayer_for_index_border_width_height",
    },
    "features": {
        0: {"geometry": {"type": "LineString", "coordinates": [[0, 0], [-30, -30]]}}
    },
}

geolayer_grid_3_3 = {
    "metadata": {
        "name": "grid_3_3",
        "geometry_ref": {"type": "Polygon"},
        "fields": {"g_id": {"type": "String", "width": 254, "index": 0}},
    },
    "features": {
        0: {
            "attributes": {"g_id": (0, 0)},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [[0.0, 0.0], [0.0, 3.0], [3.0, 3.0], [3.0, 0.0], [0.0, 0.0]]
                ],
            },
        },
        1: {
            "attributes": {"g_id": (0, 1)},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [[0.0, 3.0], [0.0, 6.0], [3.0, 6.0], [3.0, 3.0], [0.0, 3.0]]
                ],
            },
        },
        2: {
            "attributes": {"g_id": (0, 2)},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [[0.0, 6.0], [0.0, 9.0], [3.0, 9.0], [3.0, 6.0], [0.0, 6.0]]
                ],
            },
        },
        3: {
            "attributes": {"g_id": (1, 0)},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [[3.0, 0.0], [3.0, 3.0], [6.0, 3.0], [6.0, 0.0], [3.0, 0.0]]
                ],
            },
        },
        4: {
            "attributes": {"g_id": (1, 1)},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [[3.0, 3.0], [3.0, 6.0], [6.0, 6.0], [6.0, 3.0], [3.0, 3.0]]
                ],
            },
        },
        5: {
            "attributes": {"g_id": (1, 2)},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [[3.0, 6.0], [3.0, 9.0], [6.0, 9.0], [6.0, 6.0], [3.0, 6.0]]
                ],
            },
        },
        6: {
            "attributes": {"g_id": (2, 0)},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [[6.0, 0.0], [6.0, 3.0], [9.0, 3.0], [9.0, 0.0], [6.0, 0.0]]
                ],
            },
        },
        7: {
            "attributes": {"g_id": (2, 1)},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [[6.0, 3.0], [6.0, 6.0], [9.0, 6.0], [9.0, 3.0], [6.0, 3.0]]
                ],
            },
        },
        8: {
            "attributes": {"g_id": (2, 2)},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [[6.0, 6.0], [6.0, 9.0], [9.0, 9.0], [9.0, 6.0], [6.0, 6.0]]
                ],
            },
        },
    },
}

geolayer_for_index_grid_geolayer = {
    "features": {
        0: {
            "attributes": {"g_id": (-1, 0), "i_feat_list": [0, 1]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [[-30.0, 0.0], [-30.0, 30.0], [0.0, 30.0], [0.0, 0.0], [-30.0, 0.0]]
                ],
            },
        },
        1: {
            "attributes": {"g_id": (0, 0), "i_feat_list": [0, 1]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [[0.0, 0.0], [0.0, 30.0], [30.0, 30.0], [30.0, 0.0], [0.0, 0.0]]
                ],
            },
        },
        2: {
            "attributes": {"g_id": (-1, -1), "i_feat_list": [1, 2]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [-30.0, -30.0],
                        [-30.0, 0.0],
                        [0.0, 0.0],
                        [0.0, -30.0],
                        [-30.0, -30.0],
                    ]
                ],
            },
        },
        3: {
            "attributes": {"g_id": (0, -1), "i_feat_list": [1]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [[0.0, -30.0], [0.0, 0.0], [30.0, 0.0], [30.0, -30.0], [0.0, -30.0]]
                ],
            },
        },
    },
    "metadata": {
        "name": "grid_index",
        "geometry_ref": {"type": "Polygon"},
        "fields": {
            "g_id": {"type": "String", "width": 254, "index": 0},
            "i_feat_list": {"type": "IntegerList", "index": 1},
        },
    },
}
geolayer_for_index_width_grid_geolayer = {
    "features": {
        0: {
            "attributes": {"g_id": (5598, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [167940.0, 5388660.0],
                        [167940.0, 5388690.0],
                        [167970.0, 5388690.0],
                        [167970.0, 5388660.0],
                        [167940.0, 5388660.0],
                    ]
                ],
            },
        },
        1: {
            "attributes": {"g_id": (5598, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [167940.0, 5388690.0],
                        [167940.0, 5388720.0],
                        [167970.0, 5388720.0],
                        [167970.0, 5388690.0],
                        [167940.0, 5388690.0],
                    ]
                ],
            },
        },
        2: {
            "attributes": {"g_id": (5598, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [167940.0, 5388720.0],
                        [167940.0, 5388750.0],
                        [167970.0, 5388750.0],
                        [167970.0, 5388720.0],
                        [167940.0, 5388720.0],
                    ]
                ],
            },
        },
        3: {
            "attributes": {"g_id": (5598, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [167940.0, 5388750.0],
                        [167940.0, 5388780.0],
                        [167970.0, 5388780.0],
                        [167970.0, 5388750.0],
                        [167940.0, 5388750.0],
                    ]
                ],
            },
        },
        4: {
            "attributes": {"g_id": (5598, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [167940.0, 5388780.0],
                        [167940.0, 5388810.0],
                        [167970.0, 5388810.0],
                        [167970.0, 5388780.0],
                        [167940.0, 5388780.0],
                    ]
                ],
            },
        },
        5: {
            "attributes": {"g_id": (5598, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [167940.0, 5388810.0],
                        [167940.0, 5388840.0],
                        [167970.0, 5388840.0],
                        [167970.0, 5388810.0],
                        [167940.0, 5388810.0],
                    ]
                ],
            },
        },
        6: {
            "attributes": {"g_id": (5599, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [167970.0, 5388660.0],
                        [167970.0, 5388690.0],
                        [168000.0, 5388690.0],
                        [168000.0, 5388660.0],
                        [167970.0, 5388660.0],
                    ]
                ],
            },
        },
        7: {
            "attributes": {"g_id": (5599, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [167970.0, 5388690.0],
                        [167970.0, 5388720.0],
                        [168000.0, 5388720.0],
                        [168000.0, 5388690.0],
                        [167970.0, 5388690.0],
                    ]
                ],
            },
        },
        8: {
            "attributes": {"g_id": (5599, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [167970.0, 5388720.0],
                        [167970.0, 5388750.0],
                        [168000.0, 5388750.0],
                        [168000.0, 5388720.0],
                        [167970.0, 5388720.0],
                    ]
                ],
            },
        },
        9: {
            "attributes": {"g_id": (5599, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [167970.0, 5388750.0],
                        [167970.0, 5388780.0],
                        [168000.0, 5388780.0],
                        [168000.0, 5388750.0],
                        [167970.0, 5388750.0],
                    ]
                ],
            },
        },
        10: {
            "attributes": {"g_id": (5599, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [167970.0, 5388780.0],
                        [167970.0, 5388810.0],
                        [168000.0, 5388810.0],
                        [168000.0, 5388780.0],
                        [167970.0, 5388780.0],
                    ]
                ],
            },
        },
        11: {
            "attributes": {"g_id": (5599, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [167970.0, 5388810.0],
                        [167970.0, 5388840.0],
                        [168000.0, 5388840.0],
                        [168000.0, 5388810.0],
                        [167970.0, 5388810.0],
                    ]
                ],
            },
        },
    },
    "metadata": {
        "name": "grid_index",
        "geometry_ref": {"type": "Polygon"},
        "fields": {
            "g_id": {"type": "String", "width": 254, "index": 0},
            "i_feat_list": {"type": "IntegerList", "index": 1},
        },
    },
}
geolayer_for_index_height_grid_geolayer = {
    "features": {
        0: {
            "attributes": {"g_id": (5614, 179552), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5386560.0],
                        [168420.0, 5386590.0],
                        [168450.0, 5386590.0],
                        [168450.0, 5386560.0],
                        [168420.0, 5386560.0],
                    ]
                ],
            },
        },
        1: {
            "attributes": {"g_id": (5615, 179552), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5386560.0],
                        [168450.0, 5386590.0],
                        [168480.0, 5386590.0],
                        [168480.0, 5386560.0],
                        [168450.0, 5386560.0],
                    ]
                ],
            },
        },
    },
    "metadata": {
        "name": "grid_index",
        "geometry_ref": {"type": "Polygon"},
        "fields": {
            "g_id": {"type": "String", "width": 254, "index": 0},
            "i_feat_list": {"type": "IntegerList", "index": 1},
        },
    },
}

geolayer_for_index_width_height_grid_geolayer = {
    "features": {
        0: {
            "attributes": {"g_id": (5604, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387520.0],
                        [168120.0, 5387550.0],
                        [168150.0, 5387550.0],
                        [168150.0, 5387520.0],
                        [168120.0, 5387520.0],
                    ]
                ],
            },
        },
        1: {
            "attributes": {"g_id": (5604, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387550.0],
                        [168120.0, 5387580.0],
                        [168150.0, 5387580.0],
                        [168150.0, 5387550.0],
                        [168120.0, 5387550.0],
                    ]
                ],
            },
        },
        2: {
            "attributes": {"g_id": (5604, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387580.0],
                        [168120.0, 5387610.0],
                        [168150.0, 5387610.0],
                        [168150.0, 5387580.0],
                        [168120.0, 5387580.0],
                    ]
                ],
            },
        },
        3: {
            "attributes": {"g_id": (5604, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387610.0],
                        [168120.0, 5387640.0],
                        [168150.0, 5387640.0],
                        [168150.0, 5387610.0],
                        [168120.0, 5387610.0],
                    ]
                ],
            },
        },
        4: {
            "attributes": {"g_id": (5604, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387640.0],
                        [168120.0, 5387670.0],
                        [168150.0, 5387670.0],
                        [168150.0, 5387640.0],
                        [168120.0, 5387640.0],
                    ]
                ],
            },
        },
        5: {
            "attributes": {"g_id": (5604, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387670.0],
                        [168120.0, 5387700.0],
                        [168150.0, 5387700.0],
                        [168150.0, 5387670.0],
                        [168120.0, 5387670.0],
                    ]
                ],
            },
        },
        6: {
            "attributes": {"g_id": (5604, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387700.0],
                        [168120.0, 5387730.0],
                        [168150.0, 5387730.0],
                        [168150.0, 5387700.0],
                        [168120.0, 5387700.0],
                    ]
                ],
            },
        },
        7: {
            "attributes": {"g_id": (5604, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387730.0],
                        [168120.0, 5387760.0],
                        [168150.0, 5387760.0],
                        [168150.0, 5387730.0],
                        [168120.0, 5387730.0],
                    ]
                ],
            },
        },
        8: {
            "attributes": {"g_id": (5604, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387760.0],
                        [168120.0, 5387790.0],
                        [168150.0, 5387790.0],
                        [168150.0, 5387760.0],
                        [168120.0, 5387760.0],
                    ]
                ],
            },
        },
        9: {
            "attributes": {"g_id": (5604, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387790.0],
                        [168120.0, 5387820.0],
                        [168150.0, 5387820.0],
                        [168150.0, 5387790.0],
                        [168120.0, 5387790.0],
                    ]
                ],
            },
        },
        10: {
            "attributes": {"g_id": (5604, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387820.0],
                        [168120.0, 5387850.0],
                        [168150.0, 5387850.0],
                        [168150.0, 5387820.0],
                        [168120.0, 5387820.0],
                    ]
                ],
            },
        },
        11: {
            "attributes": {"g_id": (5604, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387850.0],
                        [168120.0, 5387880.0],
                        [168150.0, 5387880.0],
                        [168150.0, 5387850.0],
                        [168120.0, 5387850.0],
                    ]
                ],
            },
        },
        12: {
            "attributes": {"g_id": (5604, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387880.0],
                        [168120.0, 5387910.0],
                        [168150.0, 5387910.0],
                        [168150.0, 5387880.0],
                        [168120.0, 5387880.0],
                    ]
                ],
            },
        },
        13: {
            "attributes": {"g_id": (5604, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387910.0],
                        [168120.0, 5387940.0],
                        [168150.0, 5387940.0],
                        [168150.0, 5387910.0],
                        [168120.0, 5387910.0],
                    ]
                ],
            },
        },
        14: {
            "attributes": {"g_id": (5604, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387940.0],
                        [168120.0, 5387970.0],
                        [168150.0, 5387970.0],
                        [168150.0, 5387940.0],
                        [168120.0, 5387940.0],
                    ]
                ],
            },
        },
        15: {
            "attributes": {"g_id": (5604, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5387970.0],
                        [168120.0, 5388000.0],
                        [168150.0, 5388000.0],
                        [168150.0, 5387970.0],
                        [168120.0, 5387970.0],
                    ]
                ],
            },
        },
        16: {
            "attributes": {"g_id": (5604, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388000.0],
                        [168120.0, 5388030.0],
                        [168150.0, 5388030.0],
                        [168150.0, 5388000.0],
                        [168120.0, 5388000.0],
                    ]
                ],
            },
        },
        17: {
            "attributes": {"g_id": (5604, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388030.0],
                        [168120.0, 5388060.0],
                        [168150.0, 5388060.0],
                        [168150.0, 5388030.0],
                        [168120.0, 5388030.0],
                    ]
                ],
            },
        },
        18: {
            "attributes": {"g_id": (5604, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388060.0],
                        [168120.0, 5388090.0],
                        [168150.0, 5388090.0],
                        [168150.0, 5388060.0],
                        [168120.0, 5388060.0],
                    ]
                ],
            },
        },
        19: {
            "attributes": {"g_id": (5604, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388090.0],
                        [168120.0, 5388120.0],
                        [168150.0, 5388120.0],
                        [168150.0, 5388090.0],
                        [168120.0, 5388090.0],
                    ]
                ],
            },
        },
        20: {
            "attributes": {"g_id": (5604, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388120.0],
                        [168120.0, 5388150.0],
                        [168150.0, 5388150.0],
                        [168150.0, 5388120.0],
                        [168120.0, 5388120.0],
                    ]
                ],
            },
        },
        21: {
            "attributes": {"g_id": (5604, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388150.0],
                        [168120.0, 5388180.0],
                        [168150.0, 5388180.0],
                        [168150.0, 5388150.0],
                        [168120.0, 5388150.0],
                    ]
                ],
            },
        },
        22: {
            "attributes": {"g_id": (5604, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388180.0],
                        [168120.0, 5388210.0],
                        [168150.0, 5388210.0],
                        [168150.0, 5388180.0],
                        [168120.0, 5388180.0],
                    ]
                ],
            },
        },
        23: {
            "attributes": {"g_id": (5604, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388210.0],
                        [168120.0, 5388240.0],
                        [168150.0, 5388240.0],
                        [168150.0, 5388210.0],
                        [168120.0, 5388210.0],
                    ]
                ],
            },
        },
        24: {
            "attributes": {"g_id": (5604, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388240.0],
                        [168120.0, 5388270.0],
                        [168150.0, 5388270.0],
                        [168150.0, 5388240.0],
                        [168120.0, 5388240.0],
                    ]
                ],
            },
        },
        25: {
            "attributes": {"g_id": (5604, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388270.0],
                        [168120.0, 5388300.0],
                        [168150.0, 5388300.0],
                        [168150.0, 5388270.0],
                        [168120.0, 5388270.0],
                    ]
                ],
            },
        },
        26: {
            "attributes": {"g_id": (5604, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388300.0],
                        [168120.0, 5388330.0],
                        [168150.0, 5388330.0],
                        [168150.0, 5388300.0],
                        [168120.0, 5388300.0],
                    ]
                ],
            },
        },
        27: {
            "attributes": {"g_id": (5604, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388330.0],
                        [168120.0, 5388360.0],
                        [168150.0, 5388360.0],
                        [168150.0, 5388330.0],
                        [168120.0, 5388330.0],
                    ]
                ],
            },
        },
        28: {
            "attributes": {"g_id": (5604, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388360.0],
                        [168120.0, 5388390.0],
                        [168150.0, 5388390.0],
                        [168150.0, 5388360.0],
                        [168120.0, 5388360.0],
                    ]
                ],
            },
        },
        29: {
            "attributes": {"g_id": (5604, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388390.0],
                        [168120.0, 5388420.0],
                        [168150.0, 5388420.0],
                        [168150.0, 5388390.0],
                        [168120.0, 5388390.0],
                    ]
                ],
            },
        },
        30: {
            "attributes": {"g_id": (5604, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388420.0],
                        [168120.0, 5388450.0],
                        [168150.0, 5388450.0],
                        [168150.0, 5388420.0],
                        [168120.0, 5388420.0],
                    ]
                ],
            },
        },
        31: {
            "attributes": {"g_id": (5604, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388450.0],
                        [168120.0, 5388480.0],
                        [168150.0, 5388480.0],
                        [168150.0, 5388450.0],
                        [168120.0, 5388450.0],
                    ]
                ],
            },
        },
        32: {
            "attributes": {"g_id": (5604, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388480.0],
                        [168120.0, 5388510.0],
                        [168150.0, 5388510.0],
                        [168150.0, 5388480.0],
                        [168120.0, 5388480.0],
                    ]
                ],
            },
        },
        33: {
            "attributes": {"g_id": (5604, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388510.0],
                        [168120.0, 5388540.0],
                        [168150.0, 5388540.0],
                        [168150.0, 5388510.0],
                        [168120.0, 5388510.0],
                    ]
                ],
            },
        },
        34: {
            "attributes": {"g_id": (5604, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388540.0],
                        [168120.0, 5388570.0],
                        [168150.0, 5388570.0],
                        [168150.0, 5388540.0],
                        [168120.0, 5388540.0],
                    ]
                ],
            },
        },
        35: {
            "attributes": {"g_id": (5604, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388570.0],
                        [168120.0, 5388600.0],
                        [168150.0, 5388600.0],
                        [168150.0, 5388570.0],
                        [168120.0, 5388570.0],
                    ]
                ],
            },
        },
        36: {
            "attributes": {"g_id": (5604, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388600.0],
                        [168120.0, 5388630.0],
                        [168150.0, 5388630.0],
                        [168150.0, 5388600.0],
                        [168120.0, 5388600.0],
                    ]
                ],
            },
        },
        37: {
            "attributes": {"g_id": (5604, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388630.0],
                        [168120.0, 5388660.0],
                        [168150.0, 5388660.0],
                        [168150.0, 5388630.0],
                        [168120.0, 5388630.0],
                    ]
                ],
            },
        },
        38: {
            "attributes": {"g_id": (5604, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388660.0],
                        [168120.0, 5388690.0],
                        [168150.0, 5388690.0],
                        [168150.0, 5388660.0],
                        [168120.0, 5388660.0],
                    ]
                ],
            },
        },
        39: {
            "attributes": {"g_id": (5604, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388690.0],
                        [168120.0, 5388720.0],
                        [168150.0, 5388720.0],
                        [168150.0, 5388690.0],
                        [168120.0, 5388690.0],
                    ]
                ],
            },
        },
        40: {
            "attributes": {"g_id": (5604, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388720.0],
                        [168120.0, 5388750.0],
                        [168150.0, 5388750.0],
                        [168150.0, 5388720.0],
                        [168120.0, 5388720.0],
                    ]
                ],
            },
        },
        41: {
            "attributes": {"g_id": (5604, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388750.0],
                        [168120.0, 5388780.0],
                        [168150.0, 5388780.0],
                        [168150.0, 5388750.0],
                        [168120.0, 5388750.0],
                    ]
                ],
            },
        },
        42: {
            "attributes": {"g_id": (5604, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388780.0],
                        [168120.0, 5388810.0],
                        [168150.0, 5388810.0],
                        [168150.0, 5388780.0],
                        [168120.0, 5388780.0],
                    ]
                ],
            },
        },
        43: {
            "attributes": {"g_id": (5604, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388810.0],
                        [168120.0, 5388840.0],
                        [168150.0, 5388840.0],
                        [168150.0, 5388810.0],
                        [168120.0, 5388810.0],
                    ]
                ],
            },
        },
        44: {
            "attributes": {"g_id": (5604, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388840.0],
                        [168120.0, 5388870.0],
                        [168150.0, 5388870.0],
                        [168150.0, 5388840.0],
                        [168120.0, 5388840.0],
                    ]
                ],
            },
        },
        45: {
            "attributes": {"g_id": (5604, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388870.0],
                        [168120.0, 5388900.0],
                        [168150.0, 5388900.0],
                        [168150.0, 5388870.0],
                        [168120.0, 5388870.0],
                    ]
                ],
            },
        },
        46: {
            "attributes": {"g_id": (5604, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388900.0],
                        [168120.0, 5388930.0],
                        [168150.0, 5388930.0],
                        [168150.0, 5388900.0],
                        [168120.0, 5388900.0],
                    ]
                ],
            },
        },
        47: {
            "attributes": {"g_id": (5604, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168120.0, 5388930.0],
                        [168120.0, 5388960.0],
                        [168150.0, 5388960.0],
                        [168150.0, 5388930.0],
                        [168120.0, 5388930.0],
                    ]
                ],
            },
        },
        48: {
            "attributes": {"g_id": (5605, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387520.0],
                        [168150.0, 5387550.0],
                        [168180.0, 5387550.0],
                        [168180.0, 5387520.0],
                        [168150.0, 5387520.0],
                    ]
                ],
            },
        },
        49: {
            "attributes": {"g_id": (5605, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387550.0],
                        [168150.0, 5387580.0],
                        [168180.0, 5387580.0],
                        [168180.0, 5387550.0],
                        [168150.0, 5387550.0],
                    ]
                ],
            },
        },
        50: {
            "attributes": {"g_id": (5605, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387580.0],
                        [168150.0, 5387610.0],
                        [168180.0, 5387610.0],
                        [168180.0, 5387580.0],
                        [168150.0, 5387580.0],
                    ]
                ],
            },
        },
        51: {
            "attributes": {"g_id": (5605, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387610.0],
                        [168150.0, 5387640.0],
                        [168180.0, 5387640.0],
                        [168180.0, 5387610.0],
                        [168150.0, 5387610.0],
                    ]
                ],
            },
        },
        52: {
            "attributes": {"g_id": (5605, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387640.0],
                        [168150.0, 5387670.0],
                        [168180.0, 5387670.0],
                        [168180.0, 5387640.0],
                        [168150.0, 5387640.0],
                    ]
                ],
            },
        },
        53: {
            "attributes": {"g_id": (5605, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387670.0],
                        [168150.0, 5387700.0],
                        [168180.0, 5387700.0],
                        [168180.0, 5387670.0],
                        [168150.0, 5387670.0],
                    ]
                ],
            },
        },
        54: {
            "attributes": {"g_id": (5605, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387700.0],
                        [168150.0, 5387730.0],
                        [168180.0, 5387730.0],
                        [168180.0, 5387700.0],
                        [168150.0, 5387700.0],
                    ]
                ],
            },
        },
        55: {
            "attributes": {"g_id": (5605, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387730.0],
                        [168150.0, 5387760.0],
                        [168180.0, 5387760.0],
                        [168180.0, 5387730.0],
                        [168150.0, 5387730.0],
                    ]
                ],
            },
        },
        56: {
            "attributes": {"g_id": (5605, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387760.0],
                        [168150.0, 5387790.0],
                        [168180.0, 5387790.0],
                        [168180.0, 5387760.0],
                        [168150.0, 5387760.0],
                    ]
                ],
            },
        },
        57: {
            "attributes": {"g_id": (5605, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387790.0],
                        [168150.0, 5387820.0],
                        [168180.0, 5387820.0],
                        [168180.0, 5387790.0],
                        [168150.0, 5387790.0],
                    ]
                ],
            },
        },
        58: {
            "attributes": {"g_id": (5605, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387820.0],
                        [168150.0, 5387850.0],
                        [168180.0, 5387850.0],
                        [168180.0, 5387820.0],
                        [168150.0, 5387820.0],
                    ]
                ],
            },
        },
        59: {
            "attributes": {"g_id": (5605, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387850.0],
                        [168150.0, 5387880.0],
                        [168180.0, 5387880.0],
                        [168180.0, 5387850.0],
                        [168150.0, 5387850.0],
                    ]
                ],
            },
        },
        60: {
            "attributes": {"g_id": (5605, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387880.0],
                        [168150.0, 5387910.0],
                        [168180.0, 5387910.0],
                        [168180.0, 5387880.0],
                        [168150.0, 5387880.0],
                    ]
                ],
            },
        },
        61: {
            "attributes": {"g_id": (5605, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387910.0],
                        [168150.0, 5387940.0],
                        [168180.0, 5387940.0],
                        [168180.0, 5387910.0],
                        [168150.0, 5387910.0],
                    ]
                ],
            },
        },
        62: {
            "attributes": {"g_id": (5605, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387940.0],
                        [168150.0, 5387970.0],
                        [168180.0, 5387970.0],
                        [168180.0, 5387940.0],
                        [168150.0, 5387940.0],
                    ]
                ],
            },
        },
        63: {
            "attributes": {"g_id": (5605, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5387970.0],
                        [168150.0, 5388000.0],
                        [168180.0, 5388000.0],
                        [168180.0, 5387970.0],
                        [168150.0, 5387970.0],
                    ]
                ],
            },
        },
        64: {
            "attributes": {"g_id": (5605, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388000.0],
                        [168150.0, 5388030.0],
                        [168180.0, 5388030.0],
                        [168180.0, 5388000.0],
                        [168150.0, 5388000.0],
                    ]
                ],
            },
        },
        65: {
            "attributes": {"g_id": (5605, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388030.0],
                        [168150.0, 5388060.0],
                        [168180.0, 5388060.0],
                        [168180.0, 5388030.0],
                        [168150.0, 5388030.0],
                    ]
                ],
            },
        },
        66: {
            "attributes": {"g_id": (5605, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388060.0],
                        [168150.0, 5388090.0],
                        [168180.0, 5388090.0],
                        [168180.0, 5388060.0],
                        [168150.0, 5388060.0],
                    ]
                ],
            },
        },
        67: {
            "attributes": {"g_id": (5605, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388090.0],
                        [168150.0, 5388120.0],
                        [168180.0, 5388120.0],
                        [168180.0, 5388090.0],
                        [168150.0, 5388090.0],
                    ]
                ],
            },
        },
        68: {
            "attributes": {"g_id": (5605, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388120.0],
                        [168150.0, 5388150.0],
                        [168180.0, 5388150.0],
                        [168180.0, 5388120.0],
                        [168150.0, 5388120.0],
                    ]
                ],
            },
        },
        69: {
            "attributes": {"g_id": (5605, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388150.0],
                        [168150.0, 5388180.0],
                        [168180.0, 5388180.0],
                        [168180.0, 5388150.0],
                        [168150.0, 5388150.0],
                    ]
                ],
            },
        },
        70: {
            "attributes": {"g_id": (5605, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388180.0],
                        [168150.0, 5388210.0],
                        [168180.0, 5388210.0],
                        [168180.0, 5388180.0],
                        [168150.0, 5388180.0],
                    ]
                ],
            },
        },
        71: {
            "attributes": {"g_id": (5605, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388210.0],
                        [168150.0, 5388240.0],
                        [168180.0, 5388240.0],
                        [168180.0, 5388210.0],
                        [168150.0, 5388210.0],
                    ]
                ],
            },
        },
        72: {
            "attributes": {"g_id": (5605, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388240.0],
                        [168150.0, 5388270.0],
                        [168180.0, 5388270.0],
                        [168180.0, 5388240.0],
                        [168150.0, 5388240.0],
                    ]
                ],
            },
        },
        73: {
            "attributes": {"g_id": (5605, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388270.0],
                        [168150.0, 5388300.0],
                        [168180.0, 5388300.0],
                        [168180.0, 5388270.0],
                        [168150.0, 5388270.0],
                    ]
                ],
            },
        },
        74: {
            "attributes": {"g_id": (5605, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388300.0],
                        [168150.0, 5388330.0],
                        [168180.0, 5388330.0],
                        [168180.0, 5388300.0],
                        [168150.0, 5388300.0],
                    ]
                ],
            },
        },
        75: {
            "attributes": {"g_id": (5605, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388330.0],
                        [168150.0, 5388360.0],
                        [168180.0, 5388360.0],
                        [168180.0, 5388330.0],
                        [168150.0, 5388330.0],
                    ]
                ],
            },
        },
        76: {
            "attributes": {"g_id": (5605, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388360.0],
                        [168150.0, 5388390.0],
                        [168180.0, 5388390.0],
                        [168180.0, 5388360.0],
                        [168150.0, 5388360.0],
                    ]
                ],
            },
        },
        77: {
            "attributes": {"g_id": (5605, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388390.0],
                        [168150.0, 5388420.0],
                        [168180.0, 5388420.0],
                        [168180.0, 5388390.0],
                        [168150.0, 5388390.0],
                    ]
                ],
            },
        },
        78: {
            "attributes": {"g_id": (5605, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388420.0],
                        [168150.0, 5388450.0],
                        [168180.0, 5388450.0],
                        [168180.0, 5388420.0],
                        [168150.0, 5388420.0],
                    ]
                ],
            },
        },
        79: {
            "attributes": {"g_id": (5605, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388450.0],
                        [168150.0, 5388480.0],
                        [168180.0, 5388480.0],
                        [168180.0, 5388450.0],
                        [168150.0, 5388450.0],
                    ]
                ],
            },
        },
        80: {
            "attributes": {"g_id": (5605, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388480.0],
                        [168150.0, 5388510.0],
                        [168180.0, 5388510.0],
                        [168180.0, 5388480.0],
                        [168150.0, 5388480.0],
                    ]
                ],
            },
        },
        81: {
            "attributes": {"g_id": (5605, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388510.0],
                        [168150.0, 5388540.0],
                        [168180.0, 5388540.0],
                        [168180.0, 5388510.0],
                        [168150.0, 5388510.0],
                    ]
                ],
            },
        },
        82: {
            "attributes": {"g_id": (5605, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388540.0],
                        [168150.0, 5388570.0],
                        [168180.0, 5388570.0],
                        [168180.0, 5388540.0],
                        [168150.0, 5388540.0],
                    ]
                ],
            },
        },
        83: {
            "attributes": {"g_id": (5605, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388570.0],
                        [168150.0, 5388600.0],
                        [168180.0, 5388600.0],
                        [168180.0, 5388570.0],
                        [168150.0, 5388570.0],
                    ]
                ],
            },
        },
        84: {
            "attributes": {"g_id": (5605, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388600.0],
                        [168150.0, 5388630.0],
                        [168180.0, 5388630.0],
                        [168180.0, 5388600.0],
                        [168150.0, 5388600.0],
                    ]
                ],
            },
        },
        85: {
            "attributes": {"g_id": (5605, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388630.0],
                        [168150.0, 5388660.0],
                        [168180.0, 5388660.0],
                        [168180.0, 5388630.0],
                        [168150.0, 5388630.0],
                    ]
                ],
            },
        },
        86: {
            "attributes": {"g_id": (5605, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388660.0],
                        [168150.0, 5388690.0],
                        [168180.0, 5388690.0],
                        [168180.0, 5388660.0],
                        [168150.0, 5388660.0],
                    ]
                ],
            },
        },
        87: {
            "attributes": {"g_id": (5605, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388690.0],
                        [168150.0, 5388720.0],
                        [168180.0, 5388720.0],
                        [168180.0, 5388690.0],
                        [168150.0, 5388690.0],
                    ]
                ],
            },
        },
        88: {
            "attributes": {"g_id": (5605, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388720.0],
                        [168150.0, 5388750.0],
                        [168180.0, 5388750.0],
                        [168180.0, 5388720.0],
                        [168150.0, 5388720.0],
                    ]
                ],
            },
        },
        89: {
            "attributes": {"g_id": (5605, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388750.0],
                        [168150.0, 5388780.0],
                        [168180.0, 5388780.0],
                        [168180.0, 5388750.0],
                        [168150.0, 5388750.0],
                    ]
                ],
            },
        },
        90: {
            "attributes": {"g_id": (5605, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388780.0],
                        [168150.0, 5388810.0],
                        [168180.0, 5388810.0],
                        [168180.0, 5388780.0],
                        [168150.0, 5388780.0],
                    ]
                ],
            },
        },
        91: {
            "attributes": {"g_id": (5605, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388810.0],
                        [168150.0, 5388840.0],
                        [168180.0, 5388840.0],
                        [168180.0, 5388810.0],
                        [168150.0, 5388810.0],
                    ]
                ],
            },
        },
        92: {
            "attributes": {"g_id": (5605, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388840.0],
                        [168150.0, 5388870.0],
                        [168180.0, 5388870.0],
                        [168180.0, 5388840.0],
                        [168150.0, 5388840.0],
                    ]
                ],
            },
        },
        93: {
            "attributes": {"g_id": (5605, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388870.0],
                        [168150.0, 5388900.0],
                        [168180.0, 5388900.0],
                        [168180.0, 5388870.0],
                        [168150.0, 5388870.0],
                    ]
                ],
            },
        },
        94: {
            "attributes": {"g_id": (5605, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388900.0],
                        [168150.0, 5388930.0],
                        [168180.0, 5388930.0],
                        [168180.0, 5388900.0],
                        [168150.0, 5388900.0],
                    ]
                ],
            },
        },
        95: {
            "attributes": {"g_id": (5605, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168150.0, 5388930.0],
                        [168150.0, 5388960.0],
                        [168180.0, 5388960.0],
                        [168180.0, 5388930.0],
                        [168150.0, 5388930.0],
                    ]
                ],
            },
        },
        96: {
            "attributes": {"g_id": (5606, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387520.0],
                        [168180.0, 5387550.0],
                        [168210.0, 5387550.0],
                        [168210.0, 5387520.0],
                        [168180.0, 5387520.0],
                    ]
                ],
            },
        },
        97: {
            "attributes": {"g_id": (5606, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387550.0],
                        [168180.0, 5387580.0],
                        [168210.0, 5387580.0],
                        [168210.0, 5387550.0],
                        [168180.0, 5387550.0],
                    ]
                ],
            },
        },
        98: {
            "attributes": {"g_id": (5606, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387580.0],
                        [168180.0, 5387610.0],
                        [168210.0, 5387610.0],
                        [168210.0, 5387580.0],
                        [168180.0, 5387580.0],
                    ]
                ],
            },
        },
        99: {
            "attributes": {"g_id": (5606, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387610.0],
                        [168180.0, 5387640.0],
                        [168210.0, 5387640.0],
                        [168210.0, 5387610.0],
                        [168180.0, 5387610.0],
                    ]
                ],
            },
        },
        100: {
            "attributes": {"g_id": (5606, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387640.0],
                        [168180.0, 5387670.0],
                        [168210.0, 5387670.0],
                        [168210.0, 5387640.0],
                        [168180.0, 5387640.0],
                    ]
                ],
            },
        },
        101: {
            "attributes": {"g_id": (5606, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387670.0],
                        [168180.0, 5387700.0],
                        [168210.0, 5387700.0],
                        [168210.0, 5387670.0],
                        [168180.0, 5387670.0],
                    ]
                ],
            },
        },
        102: {
            "attributes": {"g_id": (5606, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387700.0],
                        [168180.0, 5387730.0],
                        [168210.0, 5387730.0],
                        [168210.0, 5387700.0],
                        [168180.0, 5387700.0],
                    ]
                ],
            },
        },
        103: {
            "attributes": {"g_id": (5606, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387730.0],
                        [168180.0, 5387760.0],
                        [168210.0, 5387760.0],
                        [168210.0, 5387730.0],
                        [168180.0, 5387730.0],
                    ]
                ],
            },
        },
        104: {
            "attributes": {"g_id": (5606, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387760.0],
                        [168180.0, 5387790.0],
                        [168210.0, 5387790.0],
                        [168210.0, 5387760.0],
                        [168180.0, 5387760.0],
                    ]
                ],
            },
        },
        105: {
            "attributes": {"g_id": (5606, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387790.0],
                        [168180.0, 5387820.0],
                        [168210.0, 5387820.0],
                        [168210.0, 5387790.0],
                        [168180.0, 5387790.0],
                    ]
                ],
            },
        },
        106: {
            "attributes": {"g_id": (5606, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387820.0],
                        [168180.0, 5387850.0],
                        [168210.0, 5387850.0],
                        [168210.0, 5387820.0],
                        [168180.0, 5387820.0],
                    ]
                ],
            },
        },
        107: {
            "attributes": {"g_id": (5606, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387850.0],
                        [168180.0, 5387880.0],
                        [168210.0, 5387880.0],
                        [168210.0, 5387850.0],
                        [168180.0, 5387850.0],
                    ]
                ],
            },
        },
        108: {
            "attributes": {"g_id": (5606, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387880.0],
                        [168180.0, 5387910.0],
                        [168210.0, 5387910.0],
                        [168210.0, 5387880.0],
                        [168180.0, 5387880.0],
                    ]
                ],
            },
        },
        109: {
            "attributes": {"g_id": (5606, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387910.0],
                        [168180.0, 5387940.0],
                        [168210.0, 5387940.0],
                        [168210.0, 5387910.0],
                        [168180.0, 5387910.0],
                    ]
                ],
            },
        },
        110: {
            "attributes": {"g_id": (5606, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387940.0],
                        [168180.0, 5387970.0],
                        [168210.0, 5387970.0],
                        [168210.0, 5387940.0],
                        [168180.0, 5387940.0],
                    ]
                ],
            },
        },
        111: {
            "attributes": {"g_id": (5606, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5387970.0],
                        [168180.0, 5388000.0],
                        [168210.0, 5388000.0],
                        [168210.0, 5387970.0],
                        [168180.0, 5387970.0],
                    ]
                ],
            },
        },
        112: {
            "attributes": {"g_id": (5606, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388000.0],
                        [168180.0, 5388030.0],
                        [168210.0, 5388030.0],
                        [168210.0, 5388000.0],
                        [168180.0, 5388000.0],
                    ]
                ],
            },
        },
        113: {
            "attributes": {"g_id": (5606, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388030.0],
                        [168180.0, 5388060.0],
                        [168210.0, 5388060.0],
                        [168210.0, 5388030.0],
                        [168180.0, 5388030.0],
                    ]
                ],
            },
        },
        114: {
            "attributes": {"g_id": (5606, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388060.0],
                        [168180.0, 5388090.0],
                        [168210.0, 5388090.0],
                        [168210.0, 5388060.0],
                        [168180.0, 5388060.0],
                    ]
                ],
            },
        },
        115: {
            "attributes": {"g_id": (5606, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388090.0],
                        [168180.0, 5388120.0],
                        [168210.0, 5388120.0],
                        [168210.0, 5388090.0],
                        [168180.0, 5388090.0],
                    ]
                ],
            },
        },
        116: {
            "attributes": {"g_id": (5606, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388120.0],
                        [168180.0, 5388150.0],
                        [168210.0, 5388150.0],
                        [168210.0, 5388120.0],
                        [168180.0, 5388120.0],
                    ]
                ],
            },
        },
        117: {
            "attributes": {"g_id": (5606, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388150.0],
                        [168180.0, 5388180.0],
                        [168210.0, 5388180.0],
                        [168210.0, 5388150.0],
                        [168180.0, 5388150.0],
                    ]
                ],
            },
        },
        118: {
            "attributes": {"g_id": (5606, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388180.0],
                        [168180.0, 5388210.0],
                        [168210.0, 5388210.0],
                        [168210.0, 5388180.0],
                        [168180.0, 5388180.0],
                    ]
                ],
            },
        },
        119: {
            "attributes": {"g_id": (5606, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388210.0],
                        [168180.0, 5388240.0],
                        [168210.0, 5388240.0],
                        [168210.0, 5388210.0],
                        [168180.0, 5388210.0],
                    ]
                ],
            },
        },
        120: {
            "attributes": {"g_id": (5606, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388240.0],
                        [168180.0, 5388270.0],
                        [168210.0, 5388270.0],
                        [168210.0, 5388240.0],
                        [168180.0, 5388240.0],
                    ]
                ],
            },
        },
        121: {
            "attributes": {"g_id": (5606, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388270.0],
                        [168180.0, 5388300.0],
                        [168210.0, 5388300.0],
                        [168210.0, 5388270.0],
                        [168180.0, 5388270.0],
                    ]
                ],
            },
        },
        122: {
            "attributes": {"g_id": (5606, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388300.0],
                        [168180.0, 5388330.0],
                        [168210.0, 5388330.0],
                        [168210.0, 5388300.0],
                        [168180.0, 5388300.0],
                    ]
                ],
            },
        },
        123: {
            "attributes": {"g_id": (5606, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388330.0],
                        [168180.0, 5388360.0],
                        [168210.0, 5388360.0],
                        [168210.0, 5388330.0],
                        [168180.0, 5388330.0],
                    ]
                ],
            },
        },
        124: {
            "attributes": {"g_id": (5606, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388360.0],
                        [168180.0, 5388390.0],
                        [168210.0, 5388390.0],
                        [168210.0, 5388360.0],
                        [168180.0, 5388360.0],
                    ]
                ],
            },
        },
        125: {
            "attributes": {"g_id": (5606, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388390.0],
                        [168180.0, 5388420.0],
                        [168210.0, 5388420.0],
                        [168210.0, 5388390.0],
                        [168180.0, 5388390.0],
                    ]
                ],
            },
        },
        126: {
            "attributes": {"g_id": (5606, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388420.0],
                        [168180.0, 5388450.0],
                        [168210.0, 5388450.0],
                        [168210.0, 5388420.0],
                        [168180.0, 5388420.0],
                    ]
                ],
            },
        },
        127: {
            "attributes": {"g_id": (5606, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388450.0],
                        [168180.0, 5388480.0],
                        [168210.0, 5388480.0],
                        [168210.0, 5388450.0],
                        [168180.0, 5388450.0],
                    ]
                ],
            },
        },
        128: {
            "attributes": {"g_id": (5606, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388480.0],
                        [168180.0, 5388510.0],
                        [168210.0, 5388510.0],
                        [168210.0, 5388480.0],
                        [168180.0, 5388480.0],
                    ]
                ],
            },
        },
        129: {
            "attributes": {"g_id": (5606, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388510.0],
                        [168180.0, 5388540.0],
                        [168210.0, 5388540.0],
                        [168210.0, 5388510.0],
                        [168180.0, 5388510.0],
                    ]
                ],
            },
        },
        130: {
            "attributes": {"g_id": (5606, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388540.0],
                        [168180.0, 5388570.0],
                        [168210.0, 5388570.0],
                        [168210.0, 5388540.0],
                        [168180.0, 5388540.0],
                    ]
                ],
            },
        },
        131: {
            "attributes": {"g_id": (5606, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388570.0],
                        [168180.0, 5388600.0],
                        [168210.0, 5388600.0],
                        [168210.0, 5388570.0],
                        [168180.0, 5388570.0],
                    ]
                ],
            },
        },
        132: {
            "attributes": {"g_id": (5606, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388600.0],
                        [168180.0, 5388630.0],
                        [168210.0, 5388630.0],
                        [168210.0, 5388600.0],
                        [168180.0, 5388600.0],
                    ]
                ],
            },
        },
        133: {
            "attributes": {"g_id": (5606, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388630.0],
                        [168180.0, 5388660.0],
                        [168210.0, 5388660.0],
                        [168210.0, 5388630.0],
                        [168180.0, 5388630.0],
                    ]
                ],
            },
        },
        134: {
            "attributes": {"g_id": (5606, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388660.0],
                        [168180.0, 5388690.0],
                        [168210.0, 5388690.0],
                        [168210.0, 5388660.0],
                        [168180.0, 5388660.0],
                    ]
                ],
            },
        },
        135: {
            "attributes": {"g_id": (5606, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388690.0],
                        [168180.0, 5388720.0],
                        [168210.0, 5388720.0],
                        [168210.0, 5388690.0],
                        [168180.0, 5388690.0],
                    ]
                ],
            },
        },
        136: {
            "attributes": {"g_id": (5606, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388720.0],
                        [168180.0, 5388750.0],
                        [168210.0, 5388750.0],
                        [168210.0, 5388720.0],
                        [168180.0, 5388720.0],
                    ]
                ],
            },
        },
        137: {
            "attributes": {"g_id": (5606, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388750.0],
                        [168180.0, 5388780.0],
                        [168210.0, 5388780.0],
                        [168210.0, 5388750.0],
                        [168180.0, 5388750.0],
                    ]
                ],
            },
        },
        138: {
            "attributes": {"g_id": (5606, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388780.0],
                        [168180.0, 5388810.0],
                        [168210.0, 5388810.0],
                        [168210.0, 5388780.0],
                        [168180.0, 5388780.0],
                    ]
                ],
            },
        },
        139: {
            "attributes": {"g_id": (5606, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388810.0],
                        [168180.0, 5388840.0],
                        [168210.0, 5388840.0],
                        [168210.0, 5388810.0],
                        [168180.0, 5388810.0],
                    ]
                ],
            },
        },
        140: {
            "attributes": {"g_id": (5606, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388840.0],
                        [168180.0, 5388870.0],
                        [168210.0, 5388870.0],
                        [168210.0, 5388840.0],
                        [168180.0, 5388840.0],
                    ]
                ],
            },
        },
        141: {
            "attributes": {"g_id": (5606, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388870.0],
                        [168180.0, 5388900.0],
                        [168210.0, 5388900.0],
                        [168210.0, 5388870.0],
                        [168180.0, 5388870.0],
                    ]
                ],
            },
        },
        142: {
            "attributes": {"g_id": (5606, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388900.0],
                        [168180.0, 5388930.0],
                        [168210.0, 5388930.0],
                        [168210.0, 5388900.0],
                        [168180.0, 5388900.0],
                    ]
                ],
            },
        },
        143: {
            "attributes": {"g_id": (5606, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168180.0, 5388930.0],
                        [168180.0, 5388960.0],
                        [168210.0, 5388960.0],
                        [168210.0, 5388930.0],
                        [168180.0, 5388930.0],
                    ]
                ],
            },
        },
        144: {
            "attributes": {"g_id": (5607, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387520.0],
                        [168210.0, 5387550.0],
                        [168240.0, 5387550.0],
                        [168240.0, 5387520.0],
                        [168210.0, 5387520.0],
                    ]
                ],
            },
        },
        145: {
            "attributes": {"g_id": (5607, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387550.0],
                        [168210.0, 5387580.0],
                        [168240.0, 5387580.0],
                        [168240.0, 5387550.0],
                        [168210.0, 5387550.0],
                    ]
                ],
            },
        },
        146: {
            "attributes": {"g_id": (5607, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387580.0],
                        [168210.0, 5387610.0],
                        [168240.0, 5387610.0],
                        [168240.0, 5387580.0],
                        [168210.0, 5387580.0],
                    ]
                ],
            },
        },
        147: {
            "attributes": {"g_id": (5607, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387610.0],
                        [168210.0, 5387640.0],
                        [168240.0, 5387640.0],
                        [168240.0, 5387610.0],
                        [168210.0, 5387610.0],
                    ]
                ],
            },
        },
        148: {
            "attributes": {"g_id": (5607, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387640.0],
                        [168210.0, 5387670.0],
                        [168240.0, 5387670.0],
                        [168240.0, 5387640.0],
                        [168210.0, 5387640.0],
                    ]
                ],
            },
        },
        149: {
            "attributes": {"g_id": (5607, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387670.0],
                        [168210.0, 5387700.0],
                        [168240.0, 5387700.0],
                        [168240.0, 5387670.0],
                        [168210.0, 5387670.0],
                    ]
                ],
            },
        },
        150: {
            "attributes": {"g_id": (5607, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387700.0],
                        [168210.0, 5387730.0],
                        [168240.0, 5387730.0],
                        [168240.0, 5387700.0],
                        [168210.0, 5387700.0],
                    ]
                ],
            },
        },
        151: {
            "attributes": {"g_id": (5607, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387730.0],
                        [168210.0, 5387760.0],
                        [168240.0, 5387760.0],
                        [168240.0, 5387730.0],
                        [168210.0, 5387730.0],
                    ]
                ],
            },
        },
        152: {
            "attributes": {"g_id": (5607, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387760.0],
                        [168210.0, 5387790.0],
                        [168240.0, 5387790.0],
                        [168240.0, 5387760.0],
                        [168210.0, 5387760.0],
                    ]
                ],
            },
        },
        153: {
            "attributes": {"g_id": (5607, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387790.0],
                        [168210.0, 5387820.0],
                        [168240.0, 5387820.0],
                        [168240.0, 5387790.0],
                        [168210.0, 5387790.0],
                    ]
                ],
            },
        },
        154: {
            "attributes": {"g_id": (5607, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387820.0],
                        [168210.0, 5387850.0],
                        [168240.0, 5387850.0],
                        [168240.0, 5387820.0],
                        [168210.0, 5387820.0],
                    ]
                ],
            },
        },
        155: {
            "attributes": {"g_id": (5607, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387850.0],
                        [168210.0, 5387880.0],
                        [168240.0, 5387880.0],
                        [168240.0, 5387850.0],
                        [168210.0, 5387850.0],
                    ]
                ],
            },
        },
        156: {
            "attributes": {"g_id": (5607, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387880.0],
                        [168210.0, 5387910.0],
                        [168240.0, 5387910.0],
                        [168240.0, 5387880.0],
                        [168210.0, 5387880.0],
                    ]
                ],
            },
        },
        157: {
            "attributes": {"g_id": (5607, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387910.0],
                        [168210.0, 5387940.0],
                        [168240.0, 5387940.0],
                        [168240.0, 5387910.0],
                        [168210.0, 5387910.0],
                    ]
                ],
            },
        },
        158: {
            "attributes": {"g_id": (5607, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387940.0],
                        [168210.0, 5387970.0],
                        [168240.0, 5387970.0],
                        [168240.0, 5387940.0],
                        [168210.0, 5387940.0],
                    ]
                ],
            },
        },
        159: {
            "attributes": {"g_id": (5607, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5387970.0],
                        [168210.0, 5388000.0],
                        [168240.0, 5388000.0],
                        [168240.0, 5387970.0],
                        [168210.0, 5387970.0],
                    ]
                ],
            },
        },
        160: {
            "attributes": {"g_id": (5607, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388000.0],
                        [168210.0, 5388030.0],
                        [168240.0, 5388030.0],
                        [168240.0, 5388000.0],
                        [168210.0, 5388000.0],
                    ]
                ],
            },
        },
        161: {
            "attributes": {"g_id": (5607, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388030.0],
                        [168210.0, 5388060.0],
                        [168240.0, 5388060.0],
                        [168240.0, 5388030.0],
                        [168210.0, 5388030.0],
                    ]
                ],
            },
        },
        162: {
            "attributes": {"g_id": (5607, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388060.0],
                        [168210.0, 5388090.0],
                        [168240.0, 5388090.0],
                        [168240.0, 5388060.0],
                        [168210.0, 5388060.0],
                    ]
                ],
            },
        },
        163: {
            "attributes": {"g_id": (5607, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388090.0],
                        [168210.0, 5388120.0],
                        [168240.0, 5388120.0],
                        [168240.0, 5388090.0],
                        [168210.0, 5388090.0],
                    ]
                ],
            },
        },
        164: {
            "attributes": {"g_id": (5607, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388120.0],
                        [168210.0, 5388150.0],
                        [168240.0, 5388150.0],
                        [168240.0, 5388120.0],
                        [168210.0, 5388120.0],
                    ]
                ],
            },
        },
        165: {
            "attributes": {"g_id": (5607, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388150.0],
                        [168210.0, 5388180.0],
                        [168240.0, 5388180.0],
                        [168240.0, 5388150.0],
                        [168210.0, 5388150.0],
                    ]
                ],
            },
        },
        166: {
            "attributes": {"g_id": (5607, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388180.0],
                        [168210.0, 5388210.0],
                        [168240.0, 5388210.0],
                        [168240.0, 5388180.0],
                        [168210.0, 5388180.0],
                    ]
                ],
            },
        },
        167: {
            "attributes": {"g_id": (5607, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388210.0],
                        [168210.0, 5388240.0],
                        [168240.0, 5388240.0],
                        [168240.0, 5388210.0],
                        [168210.0, 5388210.0],
                    ]
                ],
            },
        },
        168: {
            "attributes": {"g_id": (5607, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388240.0],
                        [168210.0, 5388270.0],
                        [168240.0, 5388270.0],
                        [168240.0, 5388240.0],
                        [168210.0, 5388240.0],
                    ]
                ],
            },
        },
        169: {
            "attributes": {"g_id": (5607, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388270.0],
                        [168210.0, 5388300.0],
                        [168240.0, 5388300.0],
                        [168240.0, 5388270.0],
                        [168210.0, 5388270.0],
                    ]
                ],
            },
        },
        170: {
            "attributes": {"g_id": (5607, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388300.0],
                        [168210.0, 5388330.0],
                        [168240.0, 5388330.0],
                        [168240.0, 5388300.0],
                        [168210.0, 5388300.0],
                    ]
                ],
            },
        },
        171: {
            "attributes": {"g_id": (5607, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388330.0],
                        [168210.0, 5388360.0],
                        [168240.0, 5388360.0],
                        [168240.0, 5388330.0],
                        [168210.0, 5388330.0],
                    ]
                ],
            },
        },
        172: {
            "attributes": {"g_id": (5607, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388360.0],
                        [168210.0, 5388390.0],
                        [168240.0, 5388390.0],
                        [168240.0, 5388360.0],
                        [168210.0, 5388360.0],
                    ]
                ],
            },
        },
        173: {
            "attributes": {"g_id": (5607, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388390.0],
                        [168210.0, 5388420.0],
                        [168240.0, 5388420.0],
                        [168240.0, 5388390.0],
                        [168210.0, 5388390.0],
                    ]
                ],
            },
        },
        174: {
            "attributes": {"g_id": (5607, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388420.0],
                        [168210.0, 5388450.0],
                        [168240.0, 5388450.0],
                        [168240.0, 5388420.0],
                        [168210.0, 5388420.0],
                    ]
                ],
            },
        },
        175: {
            "attributes": {"g_id": (5607, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388450.0],
                        [168210.0, 5388480.0],
                        [168240.0, 5388480.0],
                        [168240.0, 5388450.0],
                        [168210.0, 5388450.0],
                    ]
                ],
            },
        },
        176: {
            "attributes": {"g_id": (5607, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388480.0],
                        [168210.0, 5388510.0],
                        [168240.0, 5388510.0],
                        [168240.0, 5388480.0],
                        [168210.0, 5388480.0],
                    ]
                ],
            },
        },
        177: {
            "attributes": {"g_id": (5607, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388510.0],
                        [168210.0, 5388540.0],
                        [168240.0, 5388540.0],
                        [168240.0, 5388510.0],
                        [168210.0, 5388510.0],
                    ]
                ],
            },
        },
        178: {
            "attributes": {"g_id": (5607, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388540.0],
                        [168210.0, 5388570.0],
                        [168240.0, 5388570.0],
                        [168240.0, 5388540.0],
                        [168210.0, 5388540.0],
                    ]
                ],
            },
        },
        179: {
            "attributes": {"g_id": (5607, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388570.0],
                        [168210.0, 5388600.0],
                        [168240.0, 5388600.0],
                        [168240.0, 5388570.0],
                        [168210.0, 5388570.0],
                    ]
                ],
            },
        },
        180: {
            "attributes": {"g_id": (5607, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388600.0],
                        [168210.0, 5388630.0],
                        [168240.0, 5388630.0],
                        [168240.0, 5388600.0],
                        [168210.0, 5388600.0],
                    ]
                ],
            },
        },
        181: {
            "attributes": {"g_id": (5607, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388630.0],
                        [168210.0, 5388660.0],
                        [168240.0, 5388660.0],
                        [168240.0, 5388630.0],
                        [168210.0, 5388630.0],
                    ]
                ],
            },
        },
        182: {
            "attributes": {"g_id": (5607, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388660.0],
                        [168210.0, 5388690.0],
                        [168240.0, 5388690.0],
                        [168240.0, 5388660.0],
                        [168210.0, 5388660.0],
                    ]
                ],
            },
        },
        183: {
            "attributes": {"g_id": (5607, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388690.0],
                        [168210.0, 5388720.0],
                        [168240.0, 5388720.0],
                        [168240.0, 5388690.0],
                        [168210.0, 5388690.0],
                    ]
                ],
            },
        },
        184: {
            "attributes": {"g_id": (5607, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388720.0],
                        [168210.0, 5388750.0],
                        [168240.0, 5388750.0],
                        [168240.0, 5388720.0],
                        [168210.0, 5388720.0],
                    ]
                ],
            },
        },
        185: {
            "attributes": {"g_id": (5607, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388750.0],
                        [168210.0, 5388780.0],
                        [168240.0, 5388780.0],
                        [168240.0, 5388750.0],
                        [168210.0, 5388750.0],
                    ]
                ],
            },
        },
        186: {
            "attributes": {"g_id": (5607, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388780.0],
                        [168210.0, 5388810.0],
                        [168240.0, 5388810.0],
                        [168240.0, 5388780.0],
                        [168210.0, 5388780.0],
                    ]
                ],
            },
        },
        187: {
            "attributes": {"g_id": (5607, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388810.0],
                        [168210.0, 5388840.0],
                        [168240.0, 5388840.0],
                        [168240.0, 5388810.0],
                        [168210.0, 5388810.0],
                    ]
                ],
            },
        },
        188: {
            "attributes": {"g_id": (5607, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388840.0],
                        [168210.0, 5388870.0],
                        [168240.0, 5388870.0],
                        [168240.0, 5388840.0],
                        [168210.0, 5388840.0],
                    ]
                ],
            },
        },
        189: {
            "attributes": {"g_id": (5607, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388870.0],
                        [168210.0, 5388900.0],
                        [168240.0, 5388900.0],
                        [168240.0, 5388870.0],
                        [168210.0, 5388870.0],
                    ]
                ],
            },
        },
        190: {
            "attributes": {"g_id": (5607, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388900.0],
                        [168210.0, 5388930.0],
                        [168240.0, 5388930.0],
                        [168240.0, 5388900.0],
                        [168210.0, 5388900.0],
                    ]
                ],
            },
        },
        191: {
            "attributes": {"g_id": (5607, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168210.0, 5388930.0],
                        [168210.0, 5388960.0],
                        [168240.0, 5388960.0],
                        [168240.0, 5388930.0],
                        [168210.0, 5388930.0],
                    ]
                ],
            },
        },
        192: {
            "attributes": {"g_id": (5608, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387520.0],
                        [168240.0, 5387550.0],
                        [168270.0, 5387550.0],
                        [168270.0, 5387520.0],
                        [168240.0, 5387520.0],
                    ]
                ],
            },
        },
        193: {
            "attributes": {"g_id": (5608, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387550.0],
                        [168240.0, 5387580.0],
                        [168270.0, 5387580.0],
                        [168270.0, 5387550.0],
                        [168240.0, 5387550.0],
                    ]
                ],
            },
        },
        194: {
            "attributes": {"g_id": (5608, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387580.0],
                        [168240.0, 5387610.0],
                        [168270.0, 5387610.0],
                        [168270.0, 5387580.0],
                        [168240.0, 5387580.0],
                    ]
                ],
            },
        },
        195: {
            "attributes": {"g_id": (5608, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387610.0],
                        [168240.0, 5387640.0],
                        [168270.0, 5387640.0],
                        [168270.0, 5387610.0],
                        [168240.0, 5387610.0],
                    ]
                ],
            },
        },
        196: {
            "attributes": {"g_id": (5608, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387640.0],
                        [168240.0, 5387670.0],
                        [168270.0, 5387670.0],
                        [168270.0, 5387640.0],
                        [168240.0, 5387640.0],
                    ]
                ],
            },
        },
        197: {
            "attributes": {"g_id": (5608, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387670.0],
                        [168240.0, 5387700.0],
                        [168270.0, 5387700.0],
                        [168270.0, 5387670.0],
                        [168240.0, 5387670.0],
                    ]
                ],
            },
        },
        198: {
            "attributes": {"g_id": (5608, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387700.0],
                        [168240.0, 5387730.0],
                        [168270.0, 5387730.0],
                        [168270.0, 5387700.0],
                        [168240.0, 5387700.0],
                    ]
                ],
            },
        },
        199: {
            "attributes": {"g_id": (5608, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387730.0],
                        [168240.0, 5387760.0],
                        [168270.0, 5387760.0],
                        [168270.0, 5387730.0],
                        [168240.0, 5387730.0],
                    ]
                ],
            },
        },
        200: {
            "attributes": {"g_id": (5608, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387760.0],
                        [168240.0, 5387790.0],
                        [168270.0, 5387790.0],
                        [168270.0, 5387760.0],
                        [168240.0, 5387760.0],
                    ]
                ],
            },
        },
        201: {
            "attributes": {"g_id": (5608, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387790.0],
                        [168240.0, 5387820.0],
                        [168270.0, 5387820.0],
                        [168270.0, 5387790.0],
                        [168240.0, 5387790.0],
                    ]
                ],
            },
        },
        202: {
            "attributes": {"g_id": (5608, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387820.0],
                        [168240.0, 5387850.0],
                        [168270.0, 5387850.0],
                        [168270.0, 5387820.0],
                        [168240.0, 5387820.0],
                    ]
                ],
            },
        },
        203: {
            "attributes": {"g_id": (5608, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387850.0],
                        [168240.0, 5387880.0],
                        [168270.0, 5387880.0],
                        [168270.0, 5387850.0],
                        [168240.0, 5387850.0],
                    ]
                ],
            },
        },
        204: {
            "attributes": {"g_id": (5608, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387880.0],
                        [168240.0, 5387910.0],
                        [168270.0, 5387910.0],
                        [168270.0, 5387880.0],
                        [168240.0, 5387880.0],
                    ]
                ],
            },
        },
        205: {
            "attributes": {"g_id": (5608, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387910.0],
                        [168240.0, 5387940.0],
                        [168270.0, 5387940.0],
                        [168270.0, 5387910.0],
                        [168240.0, 5387910.0],
                    ]
                ],
            },
        },
        206: {
            "attributes": {"g_id": (5608, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387940.0],
                        [168240.0, 5387970.0],
                        [168270.0, 5387970.0],
                        [168270.0, 5387940.0],
                        [168240.0, 5387940.0],
                    ]
                ],
            },
        },
        207: {
            "attributes": {"g_id": (5608, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5387970.0],
                        [168240.0, 5388000.0],
                        [168270.0, 5388000.0],
                        [168270.0, 5387970.0],
                        [168240.0, 5387970.0],
                    ]
                ],
            },
        },
        208: {
            "attributes": {"g_id": (5608, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388000.0],
                        [168240.0, 5388030.0],
                        [168270.0, 5388030.0],
                        [168270.0, 5388000.0],
                        [168240.0, 5388000.0],
                    ]
                ],
            },
        },
        209: {
            "attributes": {"g_id": (5608, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388030.0],
                        [168240.0, 5388060.0],
                        [168270.0, 5388060.0],
                        [168270.0, 5388030.0],
                        [168240.0, 5388030.0],
                    ]
                ],
            },
        },
        210: {
            "attributes": {"g_id": (5608, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388060.0],
                        [168240.0, 5388090.0],
                        [168270.0, 5388090.0],
                        [168270.0, 5388060.0],
                        [168240.0, 5388060.0],
                    ]
                ],
            },
        },
        211: {
            "attributes": {"g_id": (5608, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388090.0],
                        [168240.0, 5388120.0],
                        [168270.0, 5388120.0],
                        [168270.0, 5388090.0],
                        [168240.0, 5388090.0],
                    ]
                ],
            },
        },
        212: {
            "attributes": {"g_id": (5608, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388120.0],
                        [168240.0, 5388150.0],
                        [168270.0, 5388150.0],
                        [168270.0, 5388120.0],
                        [168240.0, 5388120.0],
                    ]
                ],
            },
        },
        213: {
            "attributes": {"g_id": (5608, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388150.0],
                        [168240.0, 5388180.0],
                        [168270.0, 5388180.0],
                        [168270.0, 5388150.0],
                        [168240.0, 5388150.0],
                    ]
                ],
            },
        },
        214: {
            "attributes": {"g_id": (5608, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388180.0],
                        [168240.0, 5388210.0],
                        [168270.0, 5388210.0],
                        [168270.0, 5388180.0],
                        [168240.0, 5388180.0],
                    ]
                ],
            },
        },
        215: {
            "attributes": {"g_id": (5608, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388210.0],
                        [168240.0, 5388240.0],
                        [168270.0, 5388240.0],
                        [168270.0, 5388210.0],
                        [168240.0, 5388210.0],
                    ]
                ],
            },
        },
        216: {
            "attributes": {"g_id": (5608, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388240.0],
                        [168240.0, 5388270.0],
                        [168270.0, 5388270.0],
                        [168270.0, 5388240.0],
                        [168240.0, 5388240.0],
                    ]
                ],
            },
        },
        217: {
            "attributes": {"g_id": (5608, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388270.0],
                        [168240.0, 5388300.0],
                        [168270.0, 5388300.0],
                        [168270.0, 5388270.0],
                        [168240.0, 5388270.0],
                    ]
                ],
            },
        },
        218: {
            "attributes": {"g_id": (5608, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388300.0],
                        [168240.0, 5388330.0],
                        [168270.0, 5388330.0],
                        [168270.0, 5388300.0],
                        [168240.0, 5388300.0],
                    ]
                ],
            },
        },
        219: {
            "attributes": {"g_id": (5608, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388330.0],
                        [168240.0, 5388360.0],
                        [168270.0, 5388360.0],
                        [168270.0, 5388330.0],
                        [168240.0, 5388330.0],
                    ]
                ],
            },
        },
        220: {
            "attributes": {"g_id": (5608, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388360.0],
                        [168240.0, 5388390.0],
                        [168270.0, 5388390.0],
                        [168270.0, 5388360.0],
                        [168240.0, 5388360.0],
                    ]
                ],
            },
        },
        221: {
            "attributes": {"g_id": (5608, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388390.0],
                        [168240.0, 5388420.0],
                        [168270.0, 5388420.0],
                        [168270.0, 5388390.0],
                        [168240.0, 5388390.0],
                    ]
                ],
            },
        },
        222: {
            "attributes": {"g_id": (5608, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388420.0],
                        [168240.0, 5388450.0],
                        [168270.0, 5388450.0],
                        [168270.0, 5388420.0],
                        [168240.0, 5388420.0],
                    ]
                ],
            },
        },
        223: {
            "attributes": {"g_id": (5608, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388450.0],
                        [168240.0, 5388480.0],
                        [168270.0, 5388480.0],
                        [168270.0, 5388450.0],
                        [168240.0, 5388450.0],
                    ]
                ],
            },
        },
        224: {
            "attributes": {"g_id": (5608, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388480.0],
                        [168240.0, 5388510.0],
                        [168270.0, 5388510.0],
                        [168270.0, 5388480.0],
                        [168240.0, 5388480.0],
                    ]
                ],
            },
        },
        225: {
            "attributes": {"g_id": (5608, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388510.0],
                        [168240.0, 5388540.0],
                        [168270.0, 5388540.0],
                        [168270.0, 5388510.0],
                        [168240.0, 5388510.0],
                    ]
                ],
            },
        },
        226: {
            "attributes": {"g_id": (5608, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388540.0],
                        [168240.0, 5388570.0],
                        [168270.0, 5388570.0],
                        [168270.0, 5388540.0],
                        [168240.0, 5388540.0],
                    ]
                ],
            },
        },
        227: {
            "attributes": {"g_id": (5608, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388570.0],
                        [168240.0, 5388600.0],
                        [168270.0, 5388600.0],
                        [168270.0, 5388570.0],
                        [168240.0, 5388570.0],
                    ]
                ],
            },
        },
        228: {
            "attributes": {"g_id": (5608, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388600.0],
                        [168240.0, 5388630.0],
                        [168270.0, 5388630.0],
                        [168270.0, 5388600.0],
                        [168240.0, 5388600.0],
                    ]
                ],
            },
        },
        229: {
            "attributes": {"g_id": (5608, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388630.0],
                        [168240.0, 5388660.0],
                        [168270.0, 5388660.0],
                        [168270.0, 5388630.0],
                        [168240.0, 5388630.0],
                    ]
                ],
            },
        },
        230: {
            "attributes": {"g_id": (5608, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388660.0],
                        [168240.0, 5388690.0],
                        [168270.0, 5388690.0],
                        [168270.0, 5388660.0],
                        [168240.0, 5388660.0],
                    ]
                ],
            },
        },
        231: {
            "attributes": {"g_id": (5608, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388690.0],
                        [168240.0, 5388720.0],
                        [168270.0, 5388720.0],
                        [168270.0, 5388690.0],
                        [168240.0, 5388690.0],
                    ]
                ],
            },
        },
        232: {
            "attributes": {"g_id": (5608, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388720.0],
                        [168240.0, 5388750.0],
                        [168270.0, 5388750.0],
                        [168270.0, 5388720.0],
                        [168240.0, 5388720.0],
                    ]
                ],
            },
        },
        233: {
            "attributes": {"g_id": (5608, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388750.0],
                        [168240.0, 5388780.0],
                        [168270.0, 5388780.0],
                        [168270.0, 5388750.0],
                        [168240.0, 5388750.0],
                    ]
                ],
            },
        },
        234: {
            "attributes": {"g_id": (5608, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388780.0],
                        [168240.0, 5388810.0],
                        [168270.0, 5388810.0],
                        [168270.0, 5388780.0],
                        [168240.0, 5388780.0],
                    ]
                ],
            },
        },
        235: {
            "attributes": {"g_id": (5608, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388810.0],
                        [168240.0, 5388840.0],
                        [168270.0, 5388840.0],
                        [168270.0, 5388810.0],
                        [168240.0, 5388810.0],
                    ]
                ],
            },
        },
        236: {
            "attributes": {"g_id": (5608, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388840.0],
                        [168240.0, 5388870.0],
                        [168270.0, 5388870.0],
                        [168270.0, 5388840.0],
                        [168240.0, 5388840.0],
                    ]
                ],
            },
        },
        237: {
            "attributes": {"g_id": (5608, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388870.0],
                        [168240.0, 5388900.0],
                        [168270.0, 5388900.0],
                        [168270.0, 5388870.0],
                        [168240.0, 5388870.0],
                    ]
                ],
            },
        },
        238: {
            "attributes": {"g_id": (5608, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388900.0],
                        [168240.0, 5388930.0],
                        [168270.0, 5388930.0],
                        [168270.0, 5388900.0],
                        [168240.0, 5388900.0],
                    ]
                ],
            },
        },
        239: {
            "attributes": {"g_id": (5608, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168240.0, 5388930.0],
                        [168240.0, 5388960.0],
                        [168270.0, 5388960.0],
                        [168270.0, 5388930.0],
                        [168240.0, 5388930.0],
                    ]
                ],
            },
        },
        240: {
            "attributes": {"g_id": (5609, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387520.0],
                        [168270.0, 5387550.0],
                        [168300.0, 5387550.0],
                        [168300.0, 5387520.0],
                        [168270.0, 5387520.0],
                    ]
                ],
            },
        },
        241: {
            "attributes": {"g_id": (5609, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387550.0],
                        [168270.0, 5387580.0],
                        [168300.0, 5387580.0],
                        [168300.0, 5387550.0],
                        [168270.0, 5387550.0],
                    ]
                ],
            },
        },
        242: {
            "attributes": {"g_id": (5609, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387580.0],
                        [168270.0, 5387610.0],
                        [168300.0, 5387610.0],
                        [168300.0, 5387580.0],
                        [168270.0, 5387580.0],
                    ]
                ],
            },
        },
        243: {
            "attributes": {"g_id": (5609, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387610.0],
                        [168270.0, 5387640.0],
                        [168300.0, 5387640.0],
                        [168300.0, 5387610.0],
                        [168270.0, 5387610.0],
                    ]
                ],
            },
        },
        244: {
            "attributes": {"g_id": (5609, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387640.0],
                        [168270.0, 5387670.0],
                        [168300.0, 5387670.0],
                        [168300.0, 5387640.0],
                        [168270.0, 5387640.0],
                    ]
                ],
            },
        },
        245: {
            "attributes": {"g_id": (5609, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387670.0],
                        [168270.0, 5387700.0],
                        [168300.0, 5387700.0],
                        [168300.0, 5387670.0],
                        [168270.0, 5387670.0],
                    ]
                ],
            },
        },
        246: {
            "attributes": {"g_id": (5609, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387700.0],
                        [168270.0, 5387730.0],
                        [168300.0, 5387730.0],
                        [168300.0, 5387700.0],
                        [168270.0, 5387700.0],
                    ]
                ],
            },
        },
        247: {
            "attributes": {"g_id": (5609, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387730.0],
                        [168270.0, 5387760.0],
                        [168300.0, 5387760.0],
                        [168300.0, 5387730.0],
                        [168270.0, 5387730.0],
                    ]
                ],
            },
        },
        248: {
            "attributes": {"g_id": (5609, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387760.0],
                        [168270.0, 5387790.0],
                        [168300.0, 5387790.0],
                        [168300.0, 5387760.0],
                        [168270.0, 5387760.0],
                    ]
                ],
            },
        },
        249: {
            "attributes": {"g_id": (5609, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387790.0],
                        [168270.0, 5387820.0],
                        [168300.0, 5387820.0],
                        [168300.0, 5387790.0],
                        [168270.0, 5387790.0],
                    ]
                ],
            },
        },
        250: {
            "attributes": {"g_id": (5609, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387820.0],
                        [168270.0, 5387850.0],
                        [168300.0, 5387850.0],
                        [168300.0, 5387820.0],
                        [168270.0, 5387820.0],
                    ]
                ],
            },
        },
        251: {
            "attributes": {"g_id": (5609, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387850.0],
                        [168270.0, 5387880.0],
                        [168300.0, 5387880.0],
                        [168300.0, 5387850.0],
                        [168270.0, 5387850.0],
                    ]
                ],
            },
        },
        252: {
            "attributes": {"g_id": (5609, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387880.0],
                        [168270.0, 5387910.0],
                        [168300.0, 5387910.0],
                        [168300.0, 5387880.0],
                        [168270.0, 5387880.0],
                    ]
                ],
            },
        },
        253: {
            "attributes": {"g_id": (5609, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387910.0],
                        [168270.0, 5387940.0],
                        [168300.0, 5387940.0],
                        [168300.0, 5387910.0],
                        [168270.0, 5387910.0],
                    ]
                ],
            },
        },
        254: {
            "attributes": {"g_id": (5609, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387940.0],
                        [168270.0, 5387970.0],
                        [168300.0, 5387970.0],
                        [168300.0, 5387940.0],
                        [168270.0, 5387940.0],
                    ]
                ],
            },
        },
        255: {
            "attributes": {"g_id": (5609, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5387970.0],
                        [168270.0, 5388000.0],
                        [168300.0, 5388000.0],
                        [168300.0, 5387970.0],
                        [168270.0, 5387970.0],
                    ]
                ],
            },
        },
        256: {
            "attributes": {"g_id": (5609, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388000.0],
                        [168270.0, 5388030.0],
                        [168300.0, 5388030.0],
                        [168300.0, 5388000.0],
                        [168270.0, 5388000.0],
                    ]
                ],
            },
        },
        257: {
            "attributes": {"g_id": (5609, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388030.0],
                        [168270.0, 5388060.0],
                        [168300.0, 5388060.0],
                        [168300.0, 5388030.0],
                        [168270.0, 5388030.0],
                    ]
                ],
            },
        },
        258: {
            "attributes": {"g_id": (5609, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388060.0],
                        [168270.0, 5388090.0],
                        [168300.0, 5388090.0],
                        [168300.0, 5388060.0],
                        [168270.0, 5388060.0],
                    ]
                ],
            },
        },
        259: {
            "attributes": {"g_id": (5609, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388090.0],
                        [168270.0, 5388120.0],
                        [168300.0, 5388120.0],
                        [168300.0, 5388090.0],
                        [168270.0, 5388090.0],
                    ]
                ],
            },
        },
        260: {
            "attributes": {"g_id": (5609, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388120.0],
                        [168270.0, 5388150.0],
                        [168300.0, 5388150.0],
                        [168300.0, 5388120.0],
                        [168270.0, 5388120.0],
                    ]
                ],
            },
        },
        261: {
            "attributes": {"g_id": (5609, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388150.0],
                        [168270.0, 5388180.0],
                        [168300.0, 5388180.0],
                        [168300.0, 5388150.0],
                        [168270.0, 5388150.0],
                    ]
                ],
            },
        },
        262: {
            "attributes": {"g_id": (5609, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388180.0],
                        [168270.0, 5388210.0],
                        [168300.0, 5388210.0],
                        [168300.0, 5388180.0],
                        [168270.0, 5388180.0],
                    ]
                ],
            },
        },
        263: {
            "attributes": {"g_id": (5609, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388210.0],
                        [168270.0, 5388240.0],
                        [168300.0, 5388240.0],
                        [168300.0, 5388210.0],
                        [168270.0, 5388210.0],
                    ]
                ],
            },
        },
        264: {
            "attributes": {"g_id": (5609, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388240.0],
                        [168270.0, 5388270.0],
                        [168300.0, 5388270.0],
                        [168300.0, 5388240.0],
                        [168270.0, 5388240.0],
                    ]
                ],
            },
        },
        265: {
            "attributes": {"g_id": (5609, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388270.0],
                        [168270.0, 5388300.0],
                        [168300.0, 5388300.0],
                        [168300.0, 5388270.0],
                        [168270.0, 5388270.0],
                    ]
                ],
            },
        },
        266: {
            "attributes": {"g_id": (5609, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388300.0],
                        [168270.0, 5388330.0],
                        [168300.0, 5388330.0],
                        [168300.0, 5388300.0],
                        [168270.0, 5388300.0],
                    ]
                ],
            },
        },
        267: {
            "attributes": {"g_id": (5609, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388330.0],
                        [168270.0, 5388360.0],
                        [168300.0, 5388360.0],
                        [168300.0, 5388330.0],
                        [168270.0, 5388330.0],
                    ]
                ],
            },
        },
        268: {
            "attributes": {"g_id": (5609, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388360.0],
                        [168270.0, 5388390.0],
                        [168300.0, 5388390.0],
                        [168300.0, 5388360.0],
                        [168270.0, 5388360.0],
                    ]
                ],
            },
        },
        269: {
            "attributes": {"g_id": (5609, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388390.0],
                        [168270.0, 5388420.0],
                        [168300.0, 5388420.0],
                        [168300.0, 5388390.0],
                        [168270.0, 5388390.0],
                    ]
                ],
            },
        },
        270: {
            "attributes": {"g_id": (5609, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388420.0],
                        [168270.0, 5388450.0],
                        [168300.0, 5388450.0],
                        [168300.0, 5388420.0],
                        [168270.0, 5388420.0],
                    ]
                ],
            },
        },
        271: {
            "attributes": {"g_id": (5609, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388450.0],
                        [168270.0, 5388480.0],
                        [168300.0, 5388480.0],
                        [168300.0, 5388450.0],
                        [168270.0, 5388450.0],
                    ]
                ],
            },
        },
        272: {
            "attributes": {"g_id": (5609, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388480.0],
                        [168270.0, 5388510.0],
                        [168300.0, 5388510.0],
                        [168300.0, 5388480.0],
                        [168270.0, 5388480.0],
                    ]
                ],
            },
        },
        273: {
            "attributes": {"g_id": (5609, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388510.0],
                        [168270.0, 5388540.0],
                        [168300.0, 5388540.0],
                        [168300.0, 5388510.0],
                        [168270.0, 5388510.0],
                    ]
                ],
            },
        },
        274: {
            "attributes": {"g_id": (5609, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388540.0],
                        [168270.0, 5388570.0],
                        [168300.0, 5388570.0],
                        [168300.0, 5388540.0],
                        [168270.0, 5388540.0],
                    ]
                ],
            },
        },
        275: {
            "attributes": {"g_id": (5609, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388570.0],
                        [168270.0, 5388600.0],
                        [168300.0, 5388600.0],
                        [168300.0, 5388570.0],
                        [168270.0, 5388570.0],
                    ]
                ],
            },
        },
        276: {
            "attributes": {"g_id": (5609, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388600.0],
                        [168270.0, 5388630.0],
                        [168300.0, 5388630.0],
                        [168300.0, 5388600.0],
                        [168270.0, 5388600.0],
                    ]
                ],
            },
        },
        277: {
            "attributes": {"g_id": (5609, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388630.0],
                        [168270.0, 5388660.0],
                        [168300.0, 5388660.0],
                        [168300.0, 5388630.0],
                        [168270.0, 5388630.0],
                    ]
                ],
            },
        },
        278: {
            "attributes": {"g_id": (5609, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388660.0],
                        [168270.0, 5388690.0],
                        [168300.0, 5388690.0],
                        [168300.0, 5388660.0],
                        [168270.0, 5388660.0],
                    ]
                ],
            },
        },
        279: {
            "attributes": {"g_id": (5609, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388690.0],
                        [168270.0, 5388720.0],
                        [168300.0, 5388720.0],
                        [168300.0, 5388690.0],
                        [168270.0, 5388690.0],
                    ]
                ],
            },
        },
        280: {
            "attributes": {"g_id": (5609, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388720.0],
                        [168270.0, 5388750.0],
                        [168300.0, 5388750.0],
                        [168300.0, 5388720.0],
                        [168270.0, 5388720.0],
                    ]
                ],
            },
        },
        281: {
            "attributes": {"g_id": (5609, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388750.0],
                        [168270.0, 5388780.0],
                        [168300.0, 5388780.0],
                        [168300.0, 5388750.0],
                        [168270.0, 5388750.0],
                    ]
                ],
            },
        },
        282: {
            "attributes": {"g_id": (5609, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388780.0],
                        [168270.0, 5388810.0],
                        [168300.0, 5388810.0],
                        [168300.0, 5388780.0],
                        [168270.0, 5388780.0],
                    ]
                ],
            },
        },
        283: {
            "attributes": {"g_id": (5609, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388810.0],
                        [168270.0, 5388840.0],
                        [168300.0, 5388840.0],
                        [168300.0, 5388810.0],
                        [168270.0, 5388810.0],
                    ]
                ],
            },
        },
        284: {
            "attributes": {"g_id": (5609, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388840.0],
                        [168270.0, 5388870.0],
                        [168300.0, 5388870.0],
                        [168300.0, 5388840.0],
                        [168270.0, 5388840.0],
                    ]
                ],
            },
        },
        285: {
            "attributes": {"g_id": (5609, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388870.0],
                        [168270.0, 5388900.0],
                        [168300.0, 5388900.0],
                        [168300.0, 5388870.0],
                        [168270.0, 5388870.0],
                    ]
                ],
            },
        },
        286: {
            "attributes": {"g_id": (5609, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388900.0],
                        [168270.0, 5388930.0],
                        [168300.0, 5388930.0],
                        [168300.0, 5388900.0],
                        [168270.0, 5388900.0],
                    ]
                ],
            },
        },
        287: {
            "attributes": {"g_id": (5609, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168270.0, 5388930.0],
                        [168270.0, 5388960.0],
                        [168300.0, 5388960.0],
                        [168300.0, 5388930.0],
                        [168270.0, 5388930.0],
                    ]
                ],
            },
        },
        288: {
            "attributes": {"g_id": (5610, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387520.0],
                        [168300.0, 5387550.0],
                        [168330.0, 5387550.0],
                        [168330.0, 5387520.0],
                        [168300.0, 5387520.0],
                    ]
                ],
            },
        },
        289: {
            "attributes": {"g_id": (5610, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387550.0],
                        [168300.0, 5387580.0],
                        [168330.0, 5387580.0],
                        [168330.0, 5387550.0],
                        [168300.0, 5387550.0],
                    ]
                ],
            },
        },
        290: {
            "attributes": {"g_id": (5610, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387580.0],
                        [168300.0, 5387610.0],
                        [168330.0, 5387610.0],
                        [168330.0, 5387580.0],
                        [168300.0, 5387580.0],
                    ]
                ],
            },
        },
        291: {
            "attributes": {"g_id": (5610, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387610.0],
                        [168300.0, 5387640.0],
                        [168330.0, 5387640.0],
                        [168330.0, 5387610.0],
                        [168300.0, 5387610.0],
                    ]
                ],
            },
        },
        292: {
            "attributes": {"g_id": (5610, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387640.0],
                        [168300.0, 5387670.0],
                        [168330.0, 5387670.0],
                        [168330.0, 5387640.0],
                        [168300.0, 5387640.0],
                    ]
                ],
            },
        },
        293: {
            "attributes": {"g_id": (5610, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387670.0],
                        [168300.0, 5387700.0],
                        [168330.0, 5387700.0],
                        [168330.0, 5387670.0],
                        [168300.0, 5387670.0],
                    ]
                ],
            },
        },
        294: {
            "attributes": {"g_id": (5610, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387700.0],
                        [168300.0, 5387730.0],
                        [168330.0, 5387730.0],
                        [168330.0, 5387700.0],
                        [168300.0, 5387700.0],
                    ]
                ],
            },
        },
        295: {
            "attributes": {"g_id": (5610, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387730.0],
                        [168300.0, 5387760.0],
                        [168330.0, 5387760.0],
                        [168330.0, 5387730.0],
                        [168300.0, 5387730.0],
                    ]
                ],
            },
        },
        296: {
            "attributes": {"g_id": (5610, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387760.0],
                        [168300.0, 5387790.0],
                        [168330.0, 5387790.0],
                        [168330.0, 5387760.0],
                        [168300.0, 5387760.0],
                    ]
                ],
            },
        },
        297: {
            "attributes": {"g_id": (5610, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387790.0],
                        [168300.0, 5387820.0],
                        [168330.0, 5387820.0],
                        [168330.0, 5387790.0],
                        [168300.0, 5387790.0],
                    ]
                ],
            },
        },
        298: {
            "attributes": {"g_id": (5610, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387820.0],
                        [168300.0, 5387850.0],
                        [168330.0, 5387850.0],
                        [168330.0, 5387820.0],
                        [168300.0, 5387820.0],
                    ]
                ],
            },
        },
        299: {
            "attributes": {"g_id": (5610, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387850.0],
                        [168300.0, 5387880.0],
                        [168330.0, 5387880.0],
                        [168330.0, 5387850.0],
                        [168300.0, 5387850.0],
                    ]
                ],
            },
        },
        300: {
            "attributes": {"g_id": (5610, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387880.0],
                        [168300.0, 5387910.0],
                        [168330.0, 5387910.0],
                        [168330.0, 5387880.0],
                        [168300.0, 5387880.0],
                    ]
                ],
            },
        },
        301: {
            "attributes": {"g_id": (5610, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387910.0],
                        [168300.0, 5387940.0],
                        [168330.0, 5387940.0],
                        [168330.0, 5387910.0],
                        [168300.0, 5387910.0],
                    ]
                ],
            },
        },
        302: {
            "attributes": {"g_id": (5610, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387940.0],
                        [168300.0, 5387970.0],
                        [168330.0, 5387970.0],
                        [168330.0, 5387940.0],
                        [168300.0, 5387940.0],
                    ]
                ],
            },
        },
        303: {
            "attributes": {"g_id": (5610, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5387970.0],
                        [168300.0, 5388000.0],
                        [168330.0, 5388000.0],
                        [168330.0, 5387970.0],
                        [168300.0, 5387970.0],
                    ]
                ],
            },
        },
        304: {
            "attributes": {"g_id": (5610, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388000.0],
                        [168300.0, 5388030.0],
                        [168330.0, 5388030.0],
                        [168330.0, 5388000.0],
                        [168300.0, 5388000.0],
                    ]
                ],
            },
        },
        305: {
            "attributes": {"g_id": (5610, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388030.0],
                        [168300.0, 5388060.0],
                        [168330.0, 5388060.0],
                        [168330.0, 5388030.0],
                        [168300.0, 5388030.0],
                    ]
                ],
            },
        },
        306: {
            "attributes": {"g_id": (5610, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388060.0],
                        [168300.0, 5388090.0],
                        [168330.0, 5388090.0],
                        [168330.0, 5388060.0],
                        [168300.0, 5388060.0],
                    ]
                ],
            },
        },
        307: {
            "attributes": {"g_id": (5610, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388090.0],
                        [168300.0, 5388120.0],
                        [168330.0, 5388120.0],
                        [168330.0, 5388090.0],
                        [168300.0, 5388090.0],
                    ]
                ],
            },
        },
        308: {
            "attributes": {"g_id": (5610, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388120.0],
                        [168300.0, 5388150.0],
                        [168330.0, 5388150.0],
                        [168330.0, 5388120.0],
                        [168300.0, 5388120.0],
                    ]
                ],
            },
        },
        309: {
            "attributes": {"g_id": (5610, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388150.0],
                        [168300.0, 5388180.0],
                        [168330.0, 5388180.0],
                        [168330.0, 5388150.0],
                        [168300.0, 5388150.0],
                    ]
                ],
            },
        },
        310: {
            "attributes": {"g_id": (5610, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388180.0],
                        [168300.0, 5388210.0],
                        [168330.0, 5388210.0],
                        [168330.0, 5388180.0],
                        [168300.0, 5388180.0],
                    ]
                ],
            },
        },
        311: {
            "attributes": {"g_id": (5610, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388210.0],
                        [168300.0, 5388240.0],
                        [168330.0, 5388240.0],
                        [168330.0, 5388210.0],
                        [168300.0, 5388210.0],
                    ]
                ],
            },
        },
        312: {
            "attributes": {"g_id": (5610, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388240.0],
                        [168300.0, 5388270.0],
                        [168330.0, 5388270.0],
                        [168330.0, 5388240.0],
                        [168300.0, 5388240.0],
                    ]
                ],
            },
        },
        313: {
            "attributes": {"g_id": (5610, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388270.0],
                        [168300.0, 5388300.0],
                        [168330.0, 5388300.0],
                        [168330.0, 5388270.0],
                        [168300.0, 5388270.0],
                    ]
                ],
            },
        },
        314: {
            "attributes": {"g_id": (5610, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388300.0],
                        [168300.0, 5388330.0],
                        [168330.0, 5388330.0],
                        [168330.0, 5388300.0],
                        [168300.0, 5388300.0],
                    ]
                ],
            },
        },
        315: {
            "attributes": {"g_id": (5610, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388330.0],
                        [168300.0, 5388360.0],
                        [168330.0, 5388360.0],
                        [168330.0, 5388330.0],
                        [168300.0, 5388330.0],
                    ]
                ],
            },
        },
        316: {
            "attributes": {"g_id": (5610, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388360.0],
                        [168300.0, 5388390.0],
                        [168330.0, 5388390.0],
                        [168330.0, 5388360.0],
                        [168300.0, 5388360.0],
                    ]
                ],
            },
        },
        317: {
            "attributes": {"g_id": (5610, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388390.0],
                        [168300.0, 5388420.0],
                        [168330.0, 5388420.0],
                        [168330.0, 5388390.0],
                        [168300.0, 5388390.0],
                    ]
                ],
            },
        },
        318: {
            "attributes": {"g_id": (5610, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388420.0],
                        [168300.0, 5388450.0],
                        [168330.0, 5388450.0],
                        [168330.0, 5388420.0],
                        [168300.0, 5388420.0],
                    ]
                ],
            },
        },
        319: {
            "attributes": {"g_id": (5610, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388450.0],
                        [168300.0, 5388480.0],
                        [168330.0, 5388480.0],
                        [168330.0, 5388450.0],
                        [168300.0, 5388450.0],
                    ]
                ],
            },
        },
        320: {
            "attributes": {"g_id": (5610, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388480.0],
                        [168300.0, 5388510.0],
                        [168330.0, 5388510.0],
                        [168330.0, 5388480.0],
                        [168300.0, 5388480.0],
                    ]
                ],
            },
        },
        321: {
            "attributes": {"g_id": (5610, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388510.0],
                        [168300.0, 5388540.0],
                        [168330.0, 5388540.0],
                        [168330.0, 5388510.0],
                        [168300.0, 5388510.0],
                    ]
                ],
            },
        },
        322: {
            "attributes": {"g_id": (5610, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388540.0],
                        [168300.0, 5388570.0],
                        [168330.0, 5388570.0],
                        [168330.0, 5388540.0],
                        [168300.0, 5388540.0],
                    ]
                ],
            },
        },
        323: {
            "attributes": {"g_id": (5610, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388570.0],
                        [168300.0, 5388600.0],
                        [168330.0, 5388600.0],
                        [168330.0, 5388570.0],
                        [168300.0, 5388570.0],
                    ]
                ],
            },
        },
        324: {
            "attributes": {"g_id": (5610, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388600.0],
                        [168300.0, 5388630.0],
                        [168330.0, 5388630.0],
                        [168330.0, 5388600.0],
                        [168300.0, 5388600.0],
                    ]
                ],
            },
        },
        325: {
            "attributes": {"g_id": (5610, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388630.0],
                        [168300.0, 5388660.0],
                        [168330.0, 5388660.0],
                        [168330.0, 5388630.0],
                        [168300.0, 5388630.0],
                    ]
                ],
            },
        },
        326: {
            "attributes": {"g_id": (5610, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388660.0],
                        [168300.0, 5388690.0],
                        [168330.0, 5388690.0],
                        [168330.0, 5388660.0],
                        [168300.0, 5388660.0],
                    ]
                ],
            },
        },
        327: {
            "attributes": {"g_id": (5610, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388690.0],
                        [168300.0, 5388720.0],
                        [168330.0, 5388720.0],
                        [168330.0, 5388690.0],
                        [168300.0, 5388690.0],
                    ]
                ],
            },
        },
        328: {
            "attributes": {"g_id": (5610, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388720.0],
                        [168300.0, 5388750.0],
                        [168330.0, 5388750.0],
                        [168330.0, 5388720.0],
                        [168300.0, 5388720.0],
                    ]
                ],
            },
        },
        329: {
            "attributes": {"g_id": (5610, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388750.0],
                        [168300.0, 5388780.0],
                        [168330.0, 5388780.0],
                        [168330.0, 5388750.0],
                        [168300.0, 5388750.0],
                    ]
                ],
            },
        },
        330: {
            "attributes": {"g_id": (5610, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388780.0],
                        [168300.0, 5388810.0],
                        [168330.0, 5388810.0],
                        [168330.0, 5388780.0],
                        [168300.0, 5388780.0],
                    ]
                ],
            },
        },
        331: {
            "attributes": {"g_id": (5610, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388810.0],
                        [168300.0, 5388840.0],
                        [168330.0, 5388840.0],
                        [168330.0, 5388810.0],
                        [168300.0, 5388810.0],
                    ]
                ],
            },
        },
        332: {
            "attributes": {"g_id": (5610, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388840.0],
                        [168300.0, 5388870.0],
                        [168330.0, 5388870.0],
                        [168330.0, 5388840.0],
                        [168300.0, 5388840.0],
                    ]
                ],
            },
        },
        333: {
            "attributes": {"g_id": (5610, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388870.0],
                        [168300.0, 5388900.0],
                        [168330.0, 5388900.0],
                        [168330.0, 5388870.0],
                        [168300.0, 5388870.0],
                    ]
                ],
            },
        },
        334: {
            "attributes": {"g_id": (5610, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388900.0],
                        [168300.0, 5388930.0],
                        [168330.0, 5388930.0],
                        [168330.0, 5388900.0],
                        [168300.0, 5388900.0],
                    ]
                ],
            },
        },
        335: {
            "attributes": {"g_id": (5610, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168300.0, 5388930.0],
                        [168300.0, 5388960.0],
                        [168330.0, 5388960.0],
                        [168330.0, 5388930.0],
                        [168300.0, 5388930.0],
                    ]
                ],
            },
        },
        336: {
            "attributes": {"g_id": (5611, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387520.0],
                        [168330.0, 5387550.0],
                        [168360.0, 5387550.0],
                        [168360.0, 5387520.0],
                        [168330.0, 5387520.0],
                    ]
                ],
            },
        },
        337: {
            "attributes": {"g_id": (5611, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387550.0],
                        [168330.0, 5387580.0],
                        [168360.0, 5387580.0],
                        [168360.0, 5387550.0],
                        [168330.0, 5387550.0],
                    ]
                ],
            },
        },
        338: {
            "attributes": {"g_id": (5611, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387580.0],
                        [168330.0, 5387610.0],
                        [168360.0, 5387610.0],
                        [168360.0, 5387580.0],
                        [168330.0, 5387580.0],
                    ]
                ],
            },
        },
        339: {
            "attributes": {"g_id": (5611, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387610.0],
                        [168330.0, 5387640.0],
                        [168360.0, 5387640.0],
                        [168360.0, 5387610.0],
                        [168330.0, 5387610.0],
                    ]
                ],
            },
        },
        340: {
            "attributes": {"g_id": (5611, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387640.0],
                        [168330.0, 5387670.0],
                        [168360.0, 5387670.0],
                        [168360.0, 5387640.0],
                        [168330.0, 5387640.0],
                    ]
                ],
            },
        },
        341: {
            "attributes": {"g_id": (5611, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387670.0],
                        [168330.0, 5387700.0],
                        [168360.0, 5387700.0],
                        [168360.0, 5387670.0],
                        [168330.0, 5387670.0],
                    ]
                ],
            },
        },
        342: {
            "attributes": {"g_id": (5611, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387700.0],
                        [168330.0, 5387730.0],
                        [168360.0, 5387730.0],
                        [168360.0, 5387700.0],
                        [168330.0, 5387700.0],
                    ]
                ],
            },
        },
        343: {
            "attributes": {"g_id": (5611, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387730.0],
                        [168330.0, 5387760.0],
                        [168360.0, 5387760.0],
                        [168360.0, 5387730.0],
                        [168330.0, 5387730.0],
                    ]
                ],
            },
        },
        344: {
            "attributes": {"g_id": (5611, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387760.0],
                        [168330.0, 5387790.0],
                        [168360.0, 5387790.0],
                        [168360.0, 5387760.0],
                        [168330.0, 5387760.0],
                    ]
                ],
            },
        },
        345: {
            "attributes": {"g_id": (5611, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387790.0],
                        [168330.0, 5387820.0],
                        [168360.0, 5387820.0],
                        [168360.0, 5387790.0],
                        [168330.0, 5387790.0],
                    ]
                ],
            },
        },
        346: {
            "attributes": {"g_id": (5611, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387820.0],
                        [168330.0, 5387850.0],
                        [168360.0, 5387850.0],
                        [168360.0, 5387820.0],
                        [168330.0, 5387820.0],
                    ]
                ],
            },
        },
        347: {
            "attributes": {"g_id": (5611, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387850.0],
                        [168330.0, 5387880.0],
                        [168360.0, 5387880.0],
                        [168360.0, 5387850.0],
                        [168330.0, 5387850.0],
                    ]
                ],
            },
        },
        348: {
            "attributes": {"g_id": (5611, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387880.0],
                        [168330.0, 5387910.0],
                        [168360.0, 5387910.0],
                        [168360.0, 5387880.0],
                        [168330.0, 5387880.0],
                    ]
                ],
            },
        },
        349: {
            "attributes": {"g_id": (5611, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387910.0],
                        [168330.0, 5387940.0],
                        [168360.0, 5387940.0],
                        [168360.0, 5387910.0],
                        [168330.0, 5387910.0],
                    ]
                ],
            },
        },
        350: {
            "attributes": {"g_id": (5611, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387940.0],
                        [168330.0, 5387970.0],
                        [168360.0, 5387970.0],
                        [168360.0, 5387940.0],
                        [168330.0, 5387940.0],
                    ]
                ],
            },
        },
        351: {
            "attributes": {"g_id": (5611, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5387970.0],
                        [168330.0, 5388000.0],
                        [168360.0, 5388000.0],
                        [168360.0, 5387970.0],
                        [168330.0, 5387970.0],
                    ]
                ],
            },
        },
        352: {
            "attributes": {"g_id": (5611, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388000.0],
                        [168330.0, 5388030.0],
                        [168360.0, 5388030.0],
                        [168360.0, 5388000.0],
                        [168330.0, 5388000.0],
                    ]
                ],
            },
        },
        353: {
            "attributes": {"g_id": (5611, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388030.0],
                        [168330.0, 5388060.0],
                        [168360.0, 5388060.0],
                        [168360.0, 5388030.0],
                        [168330.0, 5388030.0],
                    ]
                ],
            },
        },
        354: {
            "attributes": {"g_id": (5611, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388060.0],
                        [168330.0, 5388090.0],
                        [168360.0, 5388090.0],
                        [168360.0, 5388060.0],
                        [168330.0, 5388060.0],
                    ]
                ],
            },
        },
        355: {
            "attributes": {"g_id": (5611, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388090.0],
                        [168330.0, 5388120.0],
                        [168360.0, 5388120.0],
                        [168360.0, 5388090.0],
                        [168330.0, 5388090.0],
                    ]
                ],
            },
        },
        356: {
            "attributes": {"g_id": (5611, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388120.0],
                        [168330.0, 5388150.0],
                        [168360.0, 5388150.0],
                        [168360.0, 5388120.0],
                        [168330.0, 5388120.0],
                    ]
                ],
            },
        },
        357: {
            "attributes": {"g_id": (5611, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388150.0],
                        [168330.0, 5388180.0],
                        [168360.0, 5388180.0],
                        [168360.0, 5388150.0],
                        [168330.0, 5388150.0],
                    ]
                ],
            },
        },
        358: {
            "attributes": {"g_id": (5611, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388180.0],
                        [168330.0, 5388210.0],
                        [168360.0, 5388210.0],
                        [168360.0, 5388180.0],
                        [168330.0, 5388180.0],
                    ]
                ],
            },
        },
        359: {
            "attributes": {"g_id": (5611, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388210.0],
                        [168330.0, 5388240.0],
                        [168360.0, 5388240.0],
                        [168360.0, 5388210.0],
                        [168330.0, 5388210.0],
                    ]
                ],
            },
        },
        360: {
            "attributes": {"g_id": (5611, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388240.0],
                        [168330.0, 5388270.0],
                        [168360.0, 5388270.0],
                        [168360.0, 5388240.0],
                        [168330.0, 5388240.0],
                    ]
                ],
            },
        },
        361: {
            "attributes": {"g_id": (5611, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388270.0],
                        [168330.0, 5388300.0],
                        [168360.0, 5388300.0],
                        [168360.0, 5388270.0],
                        [168330.0, 5388270.0],
                    ]
                ],
            },
        },
        362: {
            "attributes": {"g_id": (5611, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388300.0],
                        [168330.0, 5388330.0],
                        [168360.0, 5388330.0],
                        [168360.0, 5388300.0],
                        [168330.0, 5388300.0],
                    ]
                ],
            },
        },
        363: {
            "attributes": {"g_id": (5611, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388330.0],
                        [168330.0, 5388360.0],
                        [168360.0, 5388360.0],
                        [168360.0, 5388330.0],
                        [168330.0, 5388330.0],
                    ]
                ],
            },
        },
        364: {
            "attributes": {"g_id": (5611, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388360.0],
                        [168330.0, 5388390.0],
                        [168360.0, 5388390.0],
                        [168360.0, 5388360.0],
                        [168330.0, 5388360.0],
                    ]
                ],
            },
        },
        365: {
            "attributes": {"g_id": (5611, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388390.0],
                        [168330.0, 5388420.0],
                        [168360.0, 5388420.0],
                        [168360.0, 5388390.0],
                        [168330.0, 5388390.0],
                    ]
                ],
            },
        },
        366: {
            "attributes": {"g_id": (5611, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388420.0],
                        [168330.0, 5388450.0],
                        [168360.0, 5388450.0],
                        [168360.0, 5388420.0],
                        [168330.0, 5388420.0],
                    ]
                ],
            },
        },
        367: {
            "attributes": {"g_id": (5611, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388450.0],
                        [168330.0, 5388480.0],
                        [168360.0, 5388480.0],
                        [168360.0, 5388450.0],
                        [168330.0, 5388450.0],
                    ]
                ],
            },
        },
        368: {
            "attributes": {"g_id": (5611, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388480.0],
                        [168330.0, 5388510.0],
                        [168360.0, 5388510.0],
                        [168360.0, 5388480.0],
                        [168330.0, 5388480.0],
                    ]
                ],
            },
        },
        369: {
            "attributes": {"g_id": (5611, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388510.0],
                        [168330.0, 5388540.0],
                        [168360.0, 5388540.0],
                        [168360.0, 5388510.0],
                        [168330.0, 5388510.0],
                    ]
                ],
            },
        },
        370: {
            "attributes": {"g_id": (5611, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388540.0],
                        [168330.0, 5388570.0],
                        [168360.0, 5388570.0],
                        [168360.0, 5388540.0],
                        [168330.0, 5388540.0],
                    ]
                ],
            },
        },
        371: {
            "attributes": {"g_id": (5611, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388570.0],
                        [168330.0, 5388600.0],
                        [168360.0, 5388600.0],
                        [168360.0, 5388570.0],
                        [168330.0, 5388570.0],
                    ]
                ],
            },
        },
        372: {
            "attributes": {"g_id": (5611, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388600.0],
                        [168330.0, 5388630.0],
                        [168360.0, 5388630.0],
                        [168360.0, 5388600.0],
                        [168330.0, 5388600.0],
                    ]
                ],
            },
        },
        373: {
            "attributes": {"g_id": (5611, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388630.0],
                        [168330.0, 5388660.0],
                        [168360.0, 5388660.0],
                        [168360.0, 5388630.0],
                        [168330.0, 5388630.0],
                    ]
                ],
            },
        },
        374: {
            "attributes": {"g_id": (5611, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388660.0],
                        [168330.0, 5388690.0],
                        [168360.0, 5388690.0],
                        [168360.0, 5388660.0],
                        [168330.0, 5388660.0],
                    ]
                ],
            },
        },
        375: {
            "attributes": {"g_id": (5611, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388690.0],
                        [168330.0, 5388720.0],
                        [168360.0, 5388720.0],
                        [168360.0, 5388690.0],
                        [168330.0, 5388690.0],
                    ]
                ],
            },
        },
        376: {
            "attributes": {"g_id": (5611, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388720.0],
                        [168330.0, 5388750.0],
                        [168360.0, 5388750.0],
                        [168360.0, 5388720.0],
                        [168330.0, 5388720.0],
                    ]
                ],
            },
        },
        377: {
            "attributes": {"g_id": (5611, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388750.0],
                        [168330.0, 5388780.0],
                        [168360.0, 5388780.0],
                        [168360.0, 5388750.0],
                        [168330.0, 5388750.0],
                    ]
                ],
            },
        },
        378: {
            "attributes": {"g_id": (5611, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388780.0],
                        [168330.0, 5388810.0],
                        [168360.0, 5388810.0],
                        [168360.0, 5388780.0],
                        [168330.0, 5388780.0],
                    ]
                ],
            },
        },
        379: {
            "attributes": {"g_id": (5611, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388810.0],
                        [168330.0, 5388840.0],
                        [168360.0, 5388840.0],
                        [168360.0, 5388810.0],
                        [168330.0, 5388810.0],
                    ]
                ],
            },
        },
        380: {
            "attributes": {"g_id": (5611, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388840.0],
                        [168330.0, 5388870.0],
                        [168360.0, 5388870.0],
                        [168360.0, 5388840.0],
                        [168330.0, 5388840.0],
                    ]
                ],
            },
        },
        381: {
            "attributes": {"g_id": (5611, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388870.0],
                        [168330.0, 5388900.0],
                        [168360.0, 5388900.0],
                        [168360.0, 5388870.0],
                        [168330.0, 5388870.0],
                    ]
                ],
            },
        },
        382: {
            "attributes": {"g_id": (5611, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388900.0],
                        [168330.0, 5388930.0],
                        [168360.0, 5388930.0],
                        [168360.0, 5388900.0],
                        [168330.0, 5388900.0],
                    ]
                ],
            },
        },
        383: {
            "attributes": {"g_id": (5611, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168330.0, 5388930.0],
                        [168330.0, 5388960.0],
                        [168360.0, 5388960.0],
                        [168360.0, 5388930.0],
                        [168330.0, 5388930.0],
                    ]
                ],
            },
        },
        384: {
            "attributes": {"g_id": (5612, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387520.0],
                        [168360.0, 5387550.0],
                        [168390.0, 5387550.0],
                        [168390.0, 5387520.0],
                        [168360.0, 5387520.0],
                    ]
                ],
            },
        },
        385: {
            "attributes": {"g_id": (5612, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387550.0],
                        [168360.0, 5387580.0],
                        [168390.0, 5387580.0],
                        [168390.0, 5387550.0],
                        [168360.0, 5387550.0],
                    ]
                ],
            },
        },
        386: {
            "attributes": {"g_id": (5612, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387580.0],
                        [168360.0, 5387610.0],
                        [168390.0, 5387610.0],
                        [168390.0, 5387580.0],
                        [168360.0, 5387580.0],
                    ]
                ],
            },
        },
        387: {
            "attributes": {"g_id": (5612, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387610.0],
                        [168360.0, 5387640.0],
                        [168390.0, 5387640.0],
                        [168390.0, 5387610.0],
                        [168360.0, 5387610.0],
                    ]
                ],
            },
        },
        388: {
            "attributes": {"g_id": (5612, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387640.0],
                        [168360.0, 5387670.0],
                        [168390.0, 5387670.0],
                        [168390.0, 5387640.0],
                        [168360.0, 5387640.0],
                    ]
                ],
            },
        },
        389: {
            "attributes": {"g_id": (5612, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387670.0],
                        [168360.0, 5387700.0],
                        [168390.0, 5387700.0],
                        [168390.0, 5387670.0],
                        [168360.0, 5387670.0],
                    ]
                ],
            },
        },
        390: {
            "attributes": {"g_id": (5612, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387700.0],
                        [168360.0, 5387730.0],
                        [168390.0, 5387730.0],
                        [168390.0, 5387700.0],
                        [168360.0, 5387700.0],
                    ]
                ],
            },
        },
        391: {
            "attributes": {"g_id": (5612, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387730.0],
                        [168360.0, 5387760.0],
                        [168390.0, 5387760.0],
                        [168390.0, 5387730.0],
                        [168360.0, 5387730.0],
                    ]
                ],
            },
        },
        392: {
            "attributes": {"g_id": (5612, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387760.0],
                        [168360.0, 5387790.0],
                        [168390.0, 5387790.0],
                        [168390.0, 5387760.0],
                        [168360.0, 5387760.0],
                    ]
                ],
            },
        },
        393: {
            "attributes": {"g_id": (5612, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387790.0],
                        [168360.0, 5387820.0],
                        [168390.0, 5387820.0],
                        [168390.0, 5387790.0],
                        [168360.0, 5387790.0],
                    ]
                ],
            },
        },
        394: {
            "attributes": {"g_id": (5612, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387820.0],
                        [168360.0, 5387850.0],
                        [168390.0, 5387850.0],
                        [168390.0, 5387820.0],
                        [168360.0, 5387820.0],
                    ]
                ],
            },
        },
        395: {
            "attributes": {"g_id": (5612, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387850.0],
                        [168360.0, 5387880.0],
                        [168390.0, 5387880.0],
                        [168390.0, 5387850.0],
                        [168360.0, 5387850.0],
                    ]
                ],
            },
        },
        396: {
            "attributes": {"g_id": (5612, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387880.0],
                        [168360.0, 5387910.0],
                        [168390.0, 5387910.0],
                        [168390.0, 5387880.0],
                        [168360.0, 5387880.0],
                    ]
                ],
            },
        },
        397: {
            "attributes": {"g_id": (5612, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387910.0],
                        [168360.0, 5387940.0],
                        [168390.0, 5387940.0],
                        [168390.0, 5387910.0],
                        [168360.0, 5387910.0],
                    ]
                ],
            },
        },
        398: {
            "attributes": {"g_id": (5612, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387940.0],
                        [168360.0, 5387970.0],
                        [168390.0, 5387970.0],
                        [168390.0, 5387940.0],
                        [168360.0, 5387940.0],
                    ]
                ],
            },
        },
        399: {
            "attributes": {"g_id": (5612, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5387970.0],
                        [168360.0, 5388000.0],
                        [168390.0, 5388000.0],
                        [168390.0, 5387970.0],
                        [168360.0, 5387970.0],
                    ]
                ],
            },
        },
        400: {
            "attributes": {"g_id": (5612, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388000.0],
                        [168360.0, 5388030.0],
                        [168390.0, 5388030.0],
                        [168390.0, 5388000.0],
                        [168360.0, 5388000.0],
                    ]
                ],
            },
        },
        401: {
            "attributes": {"g_id": (5612, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388030.0],
                        [168360.0, 5388060.0],
                        [168390.0, 5388060.0],
                        [168390.0, 5388030.0],
                        [168360.0, 5388030.0],
                    ]
                ],
            },
        },
        402: {
            "attributes": {"g_id": (5612, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388060.0],
                        [168360.0, 5388090.0],
                        [168390.0, 5388090.0],
                        [168390.0, 5388060.0],
                        [168360.0, 5388060.0],
                    ]
                ],
            },
        },
        403: {
            "attributes": {"g_id": (5612, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388090.0],
                        [168360.0, 5388120.0],
                        [168390.0, 5388120.0],
                        [168390.0, 5388090.0],
                        [168360.0, 5388090.0],
                    ]
                ],
            },
        },
        404: {
            "attributes": {"g_id": (5612, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388120.0],
                        [168360.0, 5388150.0],
                        [168390.0, 5388150.0],
                        [168390.0, 5388120.0],
                        [168360.0, 5388120.0],
                    ]
                ],
            },
        },
        405: {
            "attributes": {"g_id": (5612, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388150.0],
                        [168360.0, 5388180.0],
                        [168390.0, 5388180.0],
                        [168390.0, 5388150.0],
                        [168360.0, 5388150.0],
                    ]
                ],
            },
        },
        406: {
            "attributes": {"g_id": (5612, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388180.0],
                        [168360.0, 5388210.0],
                        [168390.0, 5388210.0],
                        [168390.0, 5388180.0],
                        [168360.0, 5388180.0],
                    ]
                ],
            },
        },
        407: {
            "attributes": {"g_id": (5612, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388210.0],
                        [168360.0, 5388240.0],
                        [168390.0, 5388240.0],
                        [168390.0, 5388210.0],
                        [168360.0, 5388210.0],
                    ]
                ],
            },
        },
        408: {
            "attributes": {"g_id": (5612, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388240.0],
                        [168360.0, 5388270.0],
                        [168390.0, 5388270.0],
                        [168390.0, 5388240.0],
                        [168360.0, 5388240.0],
                    ]
                ],
            },
        },
        409: {
            "attributes": {"g_id": (5612, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388270.0],
                        [168360.0, 5388300.0],
                        [168390.0, 5388300.0],
                        [168390.0, 5388270.0],
                        [168360.0, 5388270.0],
                    ]
                ],
            },
        },
        410: {
            "attributes": {"g_id": (5612, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388300.0],
                        [168360.0, 5388330.0],
                        [168390.0, 5388330.0],
                        [168390.0, 5388300.0],
                        [168360.0, 5388300.0],
                    ]
                ],
            },
        },
        411: {
            "attributes": {"g_id": (5612, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388330.0],
                        [168360.0, 5388360.0],
                        [168390.0, 5388360.0],
                        [168390.0, 5388330.0],
                        [168360.0, 5388330.0],
                    ]
                ],
            },
        },
        412: {
            "attributes": {"g_id": (5612, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388360.0],
                        [168360.0, 5388390.0],
                        [168390.0, 5388390.0],
                        [168390.0, 5388360.0],
                        [168360.0, 5388360.0],
                    ]
                ],
            },
        },
        413: {
            "attributes": {"g_id": (5612, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388390.0],
                        [168360.0, 5388420.0],
                        [168390.0, 5388420.0],
                        [168390.0, 5388390.0],
                        [168360.0, 5388390.0],
                    ]
                ],
            },
        },
        414: {
            "attributes": {"g_id": (5612, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388420.0],
                        [168360.0, 5388450.0],
                        [168390.0, 5388450.0],
                        [168390.0, 5388420.0],
                        [168360.0, 5388420.0],
                    ]
                ],
            },
        },
        415: {
            "attributes": {"g_id": (5612, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388450.0],
                        [168360.0, 5388480.0],
                        [168390.0, 5388480.0],
                        [168390.0, 5388450.0],
                        [168360.0, 5388450.0],
                    ]
                ],
            },
        },
        416: {
            "attributes": {"g_id": (5612, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388480.0],
                        [168360.0, 5388510.0],
                        [168390.0, 5388510.0],
                        [168390.0, 5388480.0],
                        [168360.0, 5388480.0],
                    ]
                ],
            },
        },
        417: {
            "attributes": {"g_id": (5612, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388510.0],
                        [168360.0, 5388540.0],
                        [168390.0, 5388540.0],
                        [168390.0, 5388510.0],
                        [168360.0, 5388510.0],
                    ]
                ],
            },
        },
        418: {
            "attributes": {"g_id": (5612, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388540.0],
                        [168360.0, 5388570.0],
                        [168390.0, 5388570.0],
                        [168390.0, 5388540.0],
                        [168360.0, 5388540.0],
                    ]
                ],
            },
        },
        419: {
            "attributes": {"g_id": (5612, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388570.0],
                        [168360.0, 5388600.0],
                        [168390.0, 5388600.0],
                        [168390.0, 5388570.0],
                        [168360.0, 5388570.0],
                    ]
                ],
            },
        },
        420: {
            "attributes": {"g_id": (5612, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388600.0],
                        [168360.0, 5388630.0],
                        [168390.0, 5388630.0],
                        [168390.0, 5388600.0],
                        [168360.0, 5388600.0],
                    ]
                ],
            },
        },
        421: {
            "attributes": {"g_id": (5612, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388630.0],
                        [168360.0, 5388660.0],
                        [168390.0, 5388660.0],
                        [168390.0, 5388630.0],
                        [168360.0, 5388630.0],
                    ]
                ],
            },
        },
        422: {
            "attributes": {"g_id": (5612, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388660.0],
                        [168360.0, 5388690.0],
                        [168390.0, 5388690.0],
                        [168390.0, 5388660.0],
                        [168360.0, 5388660.0],
                    ]
                ],
            },
        },
        423: {
            "attributes": {"g_id": (5612, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388690.0],
                        [168360.0, 5388720.0],
                        [168390.0, 5388720.0],
                        [168390.0, 5388690.0],
                        [168360.0, 5388690.0],
                    ]
                ],
            },
        },
        424: {
            "attributes": {"g_id": (5612, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388720.0],
                        [168360.0, 5388750.0],
                        [168390.0, 5388750.0],
                        [168390.0, 5388720.0],
                        [168360.0, 5388720.0],
                    ]
                ],
            },
        },
        425: {
            "attributes": {"g_id": (5612, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388750.0],
                        [168360.0, 5388780.0],
                        [168390.0, 5388780.0],
                        [168390.0, 5388750.0],
                        [168360.0, 5388750.0],
                    ]
                ],
            },
        },
        426: {
            "attributes": {"g_id": (5612, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388780.0],
                        [168360.0, 5388810.0],
                        [168390.0, 5388810.0],
                        [168390.0, 5388780.0],
                        [168360.0, 5388780.0],
                    ]
                ],
            },
        },
        427: {
            "attributes": {"g_id": (5612, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388810.0],
                        [168360.0, 5388840.0],
                        [168390.0, 5388840.0],
                        [168390.0, 5388810.0],
                        [168360.0, 5388810.0],
                    ]
                ],
            },
        },
        428: {
            "attributes": {"g_id": (5612, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388840.0],
                        [168360.0, 5388870.0],
                        [168390.0, 5388870.0],
                        [168390.0, 5388840.0],
                        [168360.0, 5388840.0],
                    ]
                ],
            },
        },
        429: {
            "attributes": {"g_id": (5612, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388870.0],
                        [168360.0, 5388900.0],
                        [168390.0, 5388900.0],
                        [168390.0, 5388870.0],
                        [168360.0, 5388870.0],
                    ]
                ],
            },
        },
        430: {
            "attributes": {"g_id": (5612, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388900.0],
                        [168360.0, 5388930.0],
                        [168390.0, 5388930.0],
                        [168390.0, 5388900.0],
                        [168360.0, 5388900.0],
                    ]
                ],
            },
        },
        431: {
            "attributes": {"g_id": (5612, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168360.0, 5388930.0],
                        [168360.0, 5388960.0],
                        [168390.0, 5388960.0],
                        [168390.0, 5388930.0],
                        [168360.0, 5388930.0],
                    ]
                ],
            },
        },
        432: {
            "attributes": {"g_id": (5613, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387520.0],
                        [168390.0, 5387550.0],
                        [168420.0, 5387550.0],
                        [168420.0, 5387520.0],
                        [168390.0, 5387520.0],
                    ]
                ],
            },
        },
        433: {
            "attributes": {"g_id": (5613, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387550.0],
                        [168390.0, 5387580.0],
                        [168420.0, 5387580.0],
                        [168420.0, 5387550.0],
                        [168390.0, 5387550.0],
                    ]
                ],
            },
        },
        434: {
            "attributes": {"g_id": (5613, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387580.0],
                        [168390.0, 5387610.0],
                        [168420.0, 5387610.0],
                        [168420.0, 5387580.0],
                        [168390.0, 5387580.0],
                    ]
                ],
            },
        },
        435: {
            "attributes": {"g_id": (5613, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387610.0],
                        [168390.0, 5387640.0],
                        [168420.0, 5387640.0],
                        [168420.0, 5387610.0],
                        [168390.0, 5387610.0],
                    ]
                ],
            },
        },
        436: {
            "attributes": {"g_id": (5613, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387640.0],
                        [168390.0, 5387670.0],
                        [168420.0, 5387670.0],
                        [168420.0, 5387640.0],
                        [168390.0, 5387640.0],
                    ]
                ],
            },
        },
        437: {
            "attributes": {"g_id": (5613, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387670.0],
                        [168390.0, 5387700.0],
                        [168420.0, 5387700.0],
                        [168420.0, 5387670.0],
                        [168390.0, 5387670.0],
                    ]
                ],
            },
        },
        438: {
            "attributes": {"g_id": (5613, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387700.0],
                        [168390.0, 5387730.0],
                        [168420.0, 5387730.0],
                        [168420.0, 5387700.0],
                        [168390.0, 5387700.0],
                    ]
                ],
            },
        },
        439: {
            "attributes": {"g_id": (5613, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387730.0],
                        [168390.0, 5387760.0],
                        [168420.0, 5387760.0],
                        [168420.0, 5387730.0],
                        [168390.0, 5387730.0],
                    ]
                ],
            },
        },
        440: {
            "attributes": {"g_id": (5613, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387760.0],
                        [168390.0, 5387790.0],
                        [168420.0, 5387790.0],
                        [168420.0, 5387760.0],
                        [168390.0, 5387760.0],
                    ]
                ],
            },
        },
        441: {
            "attributes": {"g_id": (5613, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387790.0],
                        [168390.0, 5387820.0],
                        [168420.0, 5387820.0],
                        [168420.0, 5387790.0],
                        [168390.0, 5387790.0],
                    ]
                ],
            },
        },
        442: {
            "attributes": {"g_id": (5613, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387820.0],
                        [168390.0, 5387850.0],
                        [168420.0, 5387850.0],
                        [168420.0, 5387820.0],
                        [168390.0, 5387820.0],
                    ]
                ],
            },
        },
        443: {
            "attributes": {"g_id": (5613, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387850.0],
                        [168390.0, 5387880.0],
                        [168420.0, 5387880.0],
                        [168420.0, 5387850.0],
                        [168390.0, 5387850.0],
                    ]
                ],
            },
        },
        444: {
            "attributes": {"g_id": (5613, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387880.0],
                        [168390.0, 5387910.0],
                        [168420.0, 5387910.0],
                        [168420.0, 5387880.0],
                        [168390.0, 5387880.0],
                    ]
                ],
            },
        },
        445: {
            "attributes": {"g_id": (5613, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387910.0],
                        [168390.0, 5387940.0],
                        [168420.0, 5387940.0],
                        [168420.0, 5387910.0],
                        [168390.0, 5387910.0],
                    ]
                ],
            },
        },
        446: {
            "attributes": {"g_id": (5613, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387940.0],
                        [168390.0, 5387970.0],
                        [168420.0, 5387970.0],
                        [168420.0, 5387940.0],
                        [168390.0, 5387940.0],
                    ]
                ],
            },
        },
        447: {
            "attributes": {"g_id": (5613, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5387970.0],
                        [168390.0, 5388000.0],
                        [168420.0, 5388000.0],
                        [168420.0, 5387970.0],
                        [168390.0, 5387970.0],
                    ]
                ],
            },
        },
        448: {
            "attributes": {"g_id": (5613, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388000.0],
                        [168390.0, 5388030.0],
                        [168420.0, 5388030.0],
                        [168420.0, 5388000.0],
                        [168390.0, 5388000.0],
                    ]
                ],
            },
        },
        449: {
            "attributes": {"g_id": (5613, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388030.0],
                        [168390.0, 5388060.0],
                        [168420.0, 5388060.0],
                        [168420.0, 5388030.0],
                        [168390.0, 5388030.0],
                    ]
                ],
            },
        },
        450: {
            "attributes": {"g_id": (5613, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388060.0],
                        [168390.0, 5388090.0],
                        [168420.0, 5388090.0],
                        [168420.0, 5388060.0],
                        [168390.0, 5388060.0],
                    ]
                ],
            },
        },
        451: {
            "attributes": {"g_id": (5613, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388090.0],
                        [168390.0, 5388120.0],
                        [168420.0, 5388120.0],
                        [168420.0, 5388090.0],
                        [168390.0, 5388090.0],
                    ]
                ],
            },
        },
        452: {
            "attributes": {"g_id": (5613, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388120.0],
                        [168390.0, 5388150.0],
                        [168420.0, 5388150.0],
                        [168420.0, 5388120.0],
                        [168390.0, 5388120.0],
                    ]
                ],
            },
        },
        453: {
            "attributes": {"g_id": (5613, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388150.0],
                        [168390.0, 5388180.0],
                        [168420.0, 5388180.0],
                        [168420.0, 5388150.0],
                        [168390.0, 5388150.0],
                    ]
                ],
            },
        },
        454: {
            "attributes": {"g_id": (5613, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388180.0],
                        [168390.0, 5388210.0],
                        [168420.0, 5388210.0],
                        [168420.0, 5388180.0],
                        [168390.0, 5388180.0],
                    ]
                ],
            },
        },
        455: {
            "attributes": {"g_id": (5613, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388210.0],
                        [168390.0, 5388240.0],
                        [168420.0, 5388240.0],
                        [168420.0, 5388210.0],
                        [168390.0, 5388210.0],
                    ]
                ],
            },
        },
        456: {
            "attributes": {"g_id": (5613, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388240.0],
                        [168390.0, 5388270.0],
                        [168420.0, 5388270.0],
                        [168420.0, 5388240.0],
                        [168390.0, 5388240.0],
                    ]
                ],
            },
        },
        457: {
            "attributes": {"g_id": (5613, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388270.0],
                        [168390.0, 5388300.0],
                        [168420.0, 5388300.0],
                        [168420.0, 5388270.0],
                        [168390.0, 5388270.0],
                    ]
                ],
            },
        },
        458: {
            "attributes": {"g_id": (5613, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388300.0],
                        [168390.0, 5388330.0],
                        [168420.0, 5388330.0],
                        [168420.0, 5388300.0],
                        [168390.0, 5388300.0],
                    ]
                ],
            },
        },
        459: {
            "attributes": {"g_id": (5613, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388330.0],
                        [168390.0, 5388360.0],
                        [168420.0, 5388360.0],
                        [168420.0, 5388330.0],
                        [168390.0, 5388330.0],
                    ]
                ],
            },
        },
        460: {
            "attributes": {"g_id": (5613, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388360.0],
                        [168390.0, 5388390.0],
                        [168420.0, 5388390.0],
                        [168420.0, 5388360.0],
                        [168390.0, 5388360.0],
                    ]
                ],
            },
        },
        461: {
            "attributes": {"g_id": (5613, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388390.0],
                        [168390.0, 5388420.0],
                        [168420.0, 5388420.0],
                        [168420.0, 5388390.0],
                        [168390.0, 5388390.0],
                    ]
                ],
            },
        },
        462: {
            "attributes": {"g_id": (5613, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388420.0],
                        [168390.0, 5388450.0],
                        [168420.0, 5388450.0],
                        [168420.0, 5388420.0],
                        [168390.0, 5388420.0],
                    ]
                ],
            },
        },
        463: {
            "attributes": {"g_id": (5613, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388450.0],
                        [168390.0, 5388480.0],
                        [168420.0, 5388480.0],
                        [168420.0, 5388450.0],
                        [168390.0, 5388450.0],
                    ]
                ],
            },
        },
        464: {
            "attributes": {"g_id": (5613, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388480.0],
                        [168390.0, 5388510.0],
                        [168420.0, 5388510.0],
                        [168420.0, 5388480.0],
                        [168390.0, 5388480.0],
                    ]
                ],
            },
        },
        465: {
            "attributes": {"g_id": (5613, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388510.0],
                        [168390.0, 5388540.0],
                        [168420.0, 5388540.0],
                        [168420.0, 5388510.0],
                        [168390.0, 5388510.0],
                    ]
                ],
            },
        },
        466: {
            "attributes": {"g_id": (5613, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388540.0],
                        [168390.0, 5388570.0],
                        [168420.0, 5388570.0],
                        [168420.0, 5388540.0],
                        [168390.0, 5388540.0],
                    ]
                ],
            },
        },
        467: {
            "attributes": {"g_id": (5613, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388570.0],
                        [168390.0, 5388600.0],
                        [168420.0, 5388600.0],
                        [168420.0, 5388570.0],
                        [168390.0, 5388570.0],
                    ]
                ],
            },
        },
        468: {
            "attributes": {"g_id": (5613, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388600.0],
                        [168390.0, 5388630.0],
                        [168420.0, 5388630.0],
                        [168420.0, 5388600.0],
                        [168390.0, 5388600.0],
                    ]
                ],
            },
        },
        469: {
            "attributes": {"g_id": (5613, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388630.0],
                        [168390.0, 5388660.0],
                        [168420.0, 5388660.0],
                        [168420.0, 5388630.0],
                        [168390.0, 5388630.0],
                    ]
                ],
            },
        },
        470: {
            "attributes": {"g_id": (5613, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388660.0],
                        [168390.0, 5388690.0],
                        [168420.0, 5388690.0],
                        [168420.0, 5388660.0],
                        [168390.0, 5388660.0],
                    ]
                ],
            },
        },
        471: {
            "attributes": {"g_id": (5613, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388690.0],
                        [168390.0, 5388720.0],
                        [168420.0, 5388720.0],
                        [168420.0, 5388690.0],
                        [168390.0, 5388690.0],
                    ]
                ],
            },
        },
        472: {
            "attributes": {"g_id": (5613, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388720.0],
                        [168390.0, 5388750.0],
                        [168420.0, 5388750.0],
                        [168420.0, 5388720.0],
                        [168390.0, 5388720.0],
                    ]
                ],
            },
        },
        473: {
            "attributes": {"g_id": (5613, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388750.0],
                        [168390.0, 5388780.0],
                        [168420.0, 5388780.0],
                        [168420.0, 5388750.0],
                        [168390.0, 5388750.0],
                    ]
                ],
            },
        },
        474: {
            "attributes": {"g_id": (5613, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388780.0],
                        [168390.0, 5388810.0],
                        [168420.0, 5388810.0],
                        [168420.0, 5388780.0],
                        [168390.0, 5388780.0],
                    ]
                ],
            },
        },
        475: {
            "attributes": {"g_id": (5613, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388810.0],
                        [168390.0, 5388840.0],
                        [168420.0, 5388840.0],
                        [168420.0, 5388810.0],
                        [168390.0, 5388810.0],
                    ]
                ],
            },
        },
        476: {
            "attributes": {"g_id": (5613, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388840.0],
                        [168390.0, 5388870.0],
                        [168420.0, 5388870.0],
                        [168420.0, 5388840.0],
                        [168390.0, 5388840.0],
                    ]
                ],
            },
        },
        477: {
            "attributes": {"g_id": (5613, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388870.0],
                        [168390.0, 5388900.0],
                        [168420.0, 5388900.0],
                        [168420.0, 5388870.0],
                        [168390.0, 5388870.0],
                    ]
                ],
            },
        },
        478: {
            "attributes": {"g_id": (5613, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388900.0],
                        [168390.0, 5388930.0],
                        [168420.0, 5388930.0],
                        [168420.0, 5388900.0],
                        [168390.0, 5388900.0],
                    ]
                ],
            },
        },
        479: {
            "attributes": {"g_id": (5613, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168390.0, 5388930.0],
                        [168390.0, 5388960.0],
                        [168420.0, 5388960.0],
                        [168420.0, 5388930.0],
                        [168390.0, 5388930.0],
                    ]
                ],
            },
        },
        480: {
            "attributes": {"g_id": (5614, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387520.0],
                        [168420.0, 5387550.0],
                        [168450.0, 5387550.0],
                        [168450.0, 5387520.0],
                        [168420.0, 5387520.0],
                    ]
                ],
            },
        },
        481: {
            "attributes": {"g_id": (5614, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387550.0],
                        [168420.0, 5387580.0],
                        [168450.0, 5387580.0],
                        [168450.0, 5387550.0],
                        [168420.0, 5387550.0],
                    ]
                ],
            },
        },
        482: {
            "attributes": {"g_id": (5614, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387580.0],
                        [168420.0, 5387610.0],
                        [168450.0, 5387610.0],
                        [168450.0, 5387580.0],
                        [168420.0, 5387580.0],
                    ]
                ],
            },
        },
        483: {
            "attributes": {"g_id": (5614, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387610.0],
                        [168420.0, 5387640.0],
                        [168450.0, 5387640.0],
                        [168450.0, 5387610.0],
                        [168420.0, 5387610.0],
                    ]
                ],
            },
        },
        484: {
            "attributes": {"g_id": (5614, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387640.0],
                        [168420.0, 5387670.0],
                        [168450.0, 5387670.0],
                        [168450.0, 5387640.0],
                        [168420.0, 5387640.0],
                    ]
                ],
            },
        },
        485: {
            "attributes": {"g_id": (5614, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387670.0],
                        [168420.0, 5387700.0],
                        [168450.0, 5387700.0],
                        [168450.0, 5387670.0],
                        [168420.0, 5387670.0],
                    ]
                ],
            },
        },
        486: {
            "attributes": {"g_id": (5614, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387700.0],
                        [168420.0, 5387730.0],
                        [168450.0, 5387730.0],
                        [168450.0, 5387700.0],
                        [168420.0, 5387700.0],
                    ]
                ],
            },
        },
        487: {
            "attributes": {"g_id": (5614, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387730.0],
                        [168420.0, 5387760.0],
                        [168450.0, 5387760.0],
                        [168450.0, 5387730.0],
                        [168420.0, 5387730.0],
                    ]
                ],
            },
        },
        488: {
            "attributes": {"g_id": (5614, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387760.0],
                        [168420.0, 5387790.0],
                        [168450.0, 5387790.0],
                        [168450.0, 5387760.0],
                        [168420.0, 5387760.0],
                    ]
                ],
            },
        },
        489: {
            "attributes": {"g_id": (5614, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387790.0],
                        [168420.0, 5387820.0],
                        [168450.0, 5387820.0],
                        [168450.0, 5387790.0],
                        [168420.0, 5387790.0],
                    ]
                ],
            },
        },
        490: {
            "attributes": {"g_id": (5614, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387820.0],
                        [168420.0, 5387850.0],
                        [168450.0, 5387850.0],
                        [168450.0, 5387820.0],
                        [168420.0, 5387820.0],
                    ]
                ],
            },
        },
        491: {
            "attributes": {"g_id": (5614, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387850.0],
                        [168420.0, 5387880.0],
                        [168450.0, 5387880.0],
                        [168450.0, 5387850.0],
                        [168420.0, 5387850.0],
                    ]
                ],
            },
        },
        492: {
            "attributes": {"g_id": (5614, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387880.0],
                        [168420.0, 5387910.0],
                        [168450.0, 5387910.0],
                        [168450.0, 5387880.0],
                        [168420.0, 5387880.0],
                    ]
                ],
            },
        },
        493: {
            "attributes": {"g_id": (5614, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387910.0],
                        [168420.0, 5387940.0],
                        [168450.0, 5387940.0],
                        [168450.0, 5387910.0],
                        [168420.0, 5387910.0],
                    ]
                ],
            },
        },
        494: {
            "attributes": {"g_id": (5614, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387940.0],
                        [168420.0, 5387970.0],
                        [168450.0, 5387970.0],
                        [168450.0, 5387940.0],
                        [168420.0, 5387940.0],
                    ]
                ],
            },
        },
        495: {
            "attributes": {"g_id": (5614, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5387970.0],
                        [168420.0, 5388000.0],
                        [168450.0, 5388000.0],
                        [168450.0, 5387970.0],
                        [168420.0, 5387970.0],
                    ]
                ],
            },
        },
        496: {
            "attributes": {"g_id": (5614, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388000.0],
                        [168420.0, 5388030.0],
                        [168450.0, 5388030.0],
                        [168450.0, 5388000.0],
                        [168420.0, 5388000.0],
                    ]
                ],
            },
        },
        497: {
            "attributes": {"g_id": (5614, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388030.0],
                        [168420.0, 5388060.0],
                        [168450.0, 5388060.0],
                        [168450.0, 5388030.0],
                        [168420.0, 5388030.0],
                    ]
                ],
            },
        },
        498: {
            "attributes": {"g_id": (5614, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388060.0],
                        [168420.0, 5388090.0],
                        [168450.0, 5388090.0],
                        [168450.0, 5388060.0],
                        [168420.0, 5388060.0],
                    ]
                ],
            },
        },
        499: {
            "attributes": {"g_id": (5614, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388090.0],
                        [168420.0, 5388120.0],
                        [168450.0, 5388120.0],
                        [168450.0, 5388090.0],
                        [168420.0, 5388090.0],
                    ]
                ],
            },
        },
        500: {
            "attributes": {"g_id": (5614, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388120.0],
                        [168420.0, 5388150.0],
                        [168450.0, 5388150.0],
                        [168450.0, 5388120.0],
                        [168420.0, 5388120.0],
                    ]
                ],
            },
        },
        501: {
            "attributes": {"g_id": (5614, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388150.0],
                        [168420.0, 5388180.0],
                        [168450.0, 5388180.0],
                        [168450.0, 5388150.0],
                        [168420.0, 5388150.0],
                    ]
                ],
            },
        },
        502: {
            "attributes": {"g_id": (5614, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388180.0],
                        [168420.0, 5388210.0],
                        [168450.0, 5388210.0],
                        [168450.0, 5388180.0],
                        [168420.0, 5388180.0],
                    ]
                ],
            },
        },
        503: {
            "attributes": {"g_id": (5614, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388210.0],
                        [168420.0, 5388240.0],
                        [168450.0, 5388240.0],
                        [168450.0, 5388210.0],
                        [168420.0, 5388210.0],
                    ]
                ],
            },
        },
        504: {
            "attributes": {"g_id": (5614, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388240.0],
                        [168420.0, 5388270.0],
                        [168450.0, 5388270.0],
                        [168450.0, 5388240.0],
                        [168420.0, 5388240.0],
                    ]
                ],
            },
        },
        505: {
            "attributes": {"g_id": (5614, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388270.0],
                        [168420.0, 5388300.0],
                        [168450.0, 5388300.0],
                        [168450.0, 5388270.0],
                        [168420.0, 5388270.0],
                    ]
                ],
            },
        },
        506: {
            "attributes": {"g_id": (5614, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388300.0],
                        [168420.0, 5388330.0],
                        [168450.0, 5388330.0],
                        [168450.0, 5388300.0],
                        [168420.0, 5388300.0],
                    ]
                ],
            },
        },
        507: {
            "attributes": {"g_id": (5614, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388330.0],
                        [168420.0, 5388360.0],
                        [168450.0, 5388360.0],
                        [168450.0, 5388330.0],
                        [168420.0, 5388330.0],
                    ]
                ],
            },
        },
        508: {
            "attributes": {"g_id": (5614, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388360.0],
                        [168420.0, 5388390.0],
                        [168450.0, 5388390.0],
                        [168450.0, 5388360.0],
                        [168420.0, 5388360.0],
                    ]
                ],
            },
        },
        509: {
            "attributes": {"g_id": (5614, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388390.0],
                        [168420.0, 5388420.0],
                        [168450.0, 5388420.0],
                        [168450.0, 5388390.0],
                        [168420.0, 5388390.0],
                    ]
                ],
            },
        },
        510: {
            "attributes": {"g_id": (5614, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388420.0],
                        [168420.0, 5388450.0],
                        [168450.0, 5388450.0],
                        [168450.0, 5388420.0],
                        [168420.0, 5388420.0],
                    ]
                ],
            },
        },
        511: {
            "attributes": {"g_id": (5614, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388450.0],
                        [168420.0, 5388480.0],
                        [168450.0, 5388480.0],
                        [168450.0, 5388450.0],
                        [168420.0, 5388450.0],
                    ]
                ],
            },
        },
        512: {
            "attributes": {"g_id": (5614, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388480.0],
                        [168420.0, 5388510.0],
                        [168450.0, 5388510.0],
                        [168450.0, 5388480.0],
                        [168420.0, 5388480.0],
                    ]
                ],
            },
        },
        513: {
            "attributes": {"g_id": (5614, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388510.0],
                        [168420.0, 5388540.0],
                        [168450.0, 5388540.0],
                        [168450.0, 5388510.0],
                        [168420.0, 5388510.0],
                    ]
                ],
            },
        },
        514: {
            "attributes": {"g_id": (5614, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388540.0],
                        [168420.0, 5388570.0],
                        [168450.0, 5388570.0],
                        [168450.0, 5388540.0],
                        [168420.0, 5388540.0],
                    ]
                ],
            },
        },
        515: {
            "attributes": {"g_id": (5614, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388570.0],
                        [168420.0, 5388600.0],
                        [168450.0, 5388600.0],
                        [168450.0, 5388570.0],
                        [168420.0, 5388570.0],
                    ]
                ],
            },
        },
        516: {
            "attributes": {"g_id": (5614, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388600.0],
                        [168420.0, 5388630.0],
                        [168450.0, 5388630.0],
                        [168450.0, 5388600.0],
                        [168420.0, 5388600.0],
                    ]
                ],
            },
        },
        517: {
            "attributes": {"g_id": (5614, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388630.0],
                        [168420.0, 5388660.0],
                        [168450.0, 5388660.0],
                        [168450.0, 5388630.0],
                        [168420.0, 5388630.0],
                    ]
                ],
            },
        },
        518: {
            "attributes": {"g_id": (5614, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388660.0],
                        [168420.0, 5388690.0],
                        [168450.0, 5388690.0],
                        [168450.0, 5388660.0],
                        [168420.0, 5388660.0],
                    ]
                ],
            },
        },
        519: {
            "attributes": {"g_id": (5614, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388690.0],
                        [168420.0, 5388720.0],
                        [168450.0, 5388720.0],
                        [168450.0, 5388690.0],
                        [168420.0, 5388690.0],
                    ]
                ],
            },
        },
        520: {
            "attributes": {"g_id": (5614, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388720.0],
                        [168420.0, 5388750.0],
                        [168450.0, 5388750.0],
                        [168450.0, 5388720.0],
                        [168420.0, 5388720.0],
                    ]
                ],
            },
        },
        521: {
            "attributes": {"g_id": (5614, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388750.0],
                        [168420.0, 5388780.0],
                        [168450.0, 5388780.0],
                        [168450.0, 5388750.0],
                        [168420.0, 5388750.0],
                    ]
                ],
            },
        },
        522: {
            "attributes": {"g_id": (5614, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388780.0],
                        [168420.0, 5388810.0],
                        [168450.0, 5388810.0],
                        [168450.0, 5388780.0],
                        [168420.0, 5388780.0],
                    ]
                ],
            },
        },
        523: {
            "attributes": {"g_id": (5614, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388810.0],
                        [168420.0, 5388840.0],
                        [168450.0, 5388840.0],
                        [168450.0, 5388810.0],
                        [168420.0, 5388810.0],
                    ]
                ],
            },
        },
        524: {
            "attributes": {"g_id": (5614, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388840.0],
                        [168420.0, 5388870.0],
                        [168450.0, 5388870.0],
                        [168450.0, 5388840.0],
                        [168420.0, 5388840.0],
                    ]
                ],
            },
        },
        525: {
            "attributes": {"g_id": (5614, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388870.0],
                        [168420.0, 5388900.0],
                        [168450.0, 5388900.0],
                        [168450.0, 5388870.0],
                        [168420.0, 5388870.0],
                    ]
                ],
            },
        },
        526: {
            "attributes": {"g_id": (5614, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388900.0],
                        [168420.0, 5388930.0],
                        [168450.0, 5388930.0],
                        [168450.0, 5388900.0],
                        [168420.0, 5388900.0],
                    ]
                ],
            },
        },
        527: {
            "attributes": {"g_id": (5614, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168420.0, 5388930.0],
                        [168420.0, 5388960.0],
                        [168450.0, 5388960.0],
                        [168450.0, 5388930.0],
                        [168420.0, 5388930.0],
                    ]
                ],
            },
        },
        528: {
            "attributes": {"g_id": (5615, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387520.0],
                        [168450.0, 5387550.0],
                        [168480.0, 5387550.0],
                        [168480.0, 5387520.0],
                        [168450.0, 5387520.0],
                    ]
                ],
            },
        },
        529: {
            "attributes": {"g_id": (5615, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387550.0],
                        [168450.0, 5387580.0],
                        [168480.0, 5387580.0],
                        [168480.0, 5387550.0],
                        [168450.0, 5387550.0],
                    ]
                ],
            },
        },
        530: {
            "attributes": {"g_id": (5615, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387580.0],
                        [168450.0, 5387610.0],
                        [168480.0, 5387610.0],
                        [168480.0, 5387580.0],
                        [168450.0, 5387580.0],
                    ]
                ],
            },
        },
        531: {
            "attributes": {"g_id": (5615, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387610.0],
                        [168450.0, 5387640.0],
                        [168480.0, 5387640.0],
                        [168480.0, 5387610.0],
                        [168450.0, 5387610.0],
                    ]
                ],
            },
        },
        532: {
            "attributes": {"g_id": (5615, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387640.0],
                        [168450.0, 5387670.0],
                        [168480.0, 5387670.0],
                        [168480.0, 5387640.0],
                        [168450.0, 5387640.0],
                    ]
                ],
            },
        },
        533: {
            "attributes": {"g_id": (5615, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387670.0],
                        [168450.0, 5387700.0],
                        [168480.0, 5387700.0],
                        [168480.0, 5387670.0],
                        [168450.0, 5387670.0],
                    ]
                ],
            },
        },
        534: {
            "attributes": {"g_id": (5615, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387700.0],
                        [168450.0, 5387730.0],
                        [168480.0, 5387730.0],
                        [168480.0, 5387700.0],
                        [168450.0, 5387700.0],
                    ]
                ],
            },
        },
        535: {
            "attributes": {"g_id": (5615, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387730.0],
                        [168450.0, 5387760.0],
                        [168480.0, 5387760.0],
                        [168480.0, 5387730.0],
                        [168450.0, 5387730.0],
                    ]
                ],
            },
        },
        536: {
            "attributes": {"g_id": (5615, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387760.0],
                        [168450.0, 5387790.0],
                        [168480.0, 5387790.0],
                        [168480.0, 5387760.0],
                        [168450.0, 5387760.0],
                    ]
                ],
            },
        },
        537: {
            "attributes": {"g_id": (5615, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387790.0],
                        [168450.0, 5387820.0],
                        [168480.0, 5387820.0],
                        [168480.0, 5387790.0],
                        [168450.0, 5387790.0],
                    ]
                ],
            },
        },
        538: {
            "attributes": {"g_id": (5615, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387820.0],
                        [168450.0, 5387850.0],
                        [168480.0, 5387850.0],
                        [168480.0, 5387820.0],
                        [168450.0, 5387820.0],
                    ]
                ],
            },
        },
        539: {
            "attributes": {"g_id": (5615, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387850.0],
                        [168450.0, 5387880.0],
                        [168480.0, 5387880.0],
                        [168480.0, 5387850.0],
                        [168450.0, 5387850.0],
                    ]
                ],
            },
        },
        540: {
            "attributes": {"g_id": (5615, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387880.0],
                        [168450.0, 5387910.0],
                        [168480.0, 5387910.0],
                        [168480.0, 5387880.0],
                        [168450.0, 5387880.0],
                    ]
                ],
            },
        },
        541: {
            "attributes": {"g_id": (5615, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387910.0],
                        [168450.0, 5387940.0],
                        [168480.0, 5387940.0],
                        [168480.0, 5387910.0],
                        [168450.0, 5387910.0],
                    ]
                ],
            },
        },
        542: {
            "attributes": {"g_id": (5615, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387940.0],
                        [168450.0, 5387970.0],
                        [168480.0, 5387970.0],
                        [168480.0, 5387940.0],
                        [168450.0, 5387940.0],
                    ]
                ],
            },
        },
        543: {
            "attributes": {"g_id": (5615, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5387970.0],
                        [168450.0, 5388000.0],
                        [168480.0, 5388000.0],
                        [168480.0, 5387970.0],
                        [168450.0, 5387970.0],
                    ]
                ],
            },
        },
        544: {
            "attributes": {"g_id": (5615, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388000.0],
                        [168450.0, 5388030.0],
                        [168480.0, 5388030.0],
                        [168480.0, 5388000.0],
                        [168450.0, 5388000.0],
                    ]
                ],
            },
        },
        545: {
            "attributes": {"g_id": (5615, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388030.0],
                        [168450.0, 5388060.0],
                        [168480.0, 5388060.0],
                        [168480.0, 5388030.0],
                        [168450.0, 5388030.0],
                    ]
                ],
            },
        },
        546: {
            "attributes": {"g_id": (5615, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388060.0],
                        [168450.0, 5388090.0],
                        [168480.0, 5388090.0],
                        [168480.0, 5388060.0],
                        [168450.0, 5388060.0],
                    ]
                ],
            },
        },
        547: {
            "attributes": {"g_id": (5615, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388090.0],
                        [168450.0, 5388120.0],
                        [168480.0, 5388120.0],
                        [168480.0, 5388090.0],
                        [168450.0, 5388090.0],
                    ]
                ],
            },
        },
        548: {
            "attributes": {"g_id": (5615, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388120.0],
                        [168450.0, 5388150.0],
                        [168480.0, 5388150.0],
                        [168480.0, 5388120.0],
                        [168450.0, 5388120.0],
                    ]
                ],
            },
        },
        549: {
            "attributes": {"g_id": (5615, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388150.0],
                        [168450.0, 5388180.0],
                        [168480.0, 5388180.0],
                        [168480.0, 5388150.0],
                        [168450.0, 5388150.0],
                    ]
                ],
            },
        },
        550: {
            "attributes": {"g_id": (5615, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388180.0],
                        [168450.0, 5388210.0],
                        [168480.0, 5388210.0],
                        [168480.0, 5388180.0],
                        [168450.0, 5388180.0],
                    ]
                ],
            },
        },
        551: {
            "attributes": {"g_id": (5615, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388210.0],
                        [168450.0, 5388240.0],
                        [168480.0, 5388240.0],
                        [168480.0, 5388210.0],
                        [168450.0, 5388210.0],
                    ]
                ],
            },
        },
        552: {
            "attributes": {"g_id": (5615, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388240.0],
                        [168450.0, 5388270.0],
                        [168480.0, 5388270.0],
                        [168480.0, 5388240.0],
                        [168450.0, 5388240.0],
                    ]
                ],
            },
        },
        553: {
            "attributes": {"g_id": (5615, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388270.0],
                        [168450.0, 5388300.0],
                        [168480.0, 5388300.0],
                        [168480.0, 5388270.0],
                        [168450.0, 5388270.0],
                    ]
                ],
            },
        },
        554: {
            "attributes": {"g_id": (5615, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388300.0],
                        [168450.0, 5388330.0],
                        [168480.0, 5388330.0],
                        [168480.0, 5388300.0],
                        [168450.0, 5388300.0],
                    ]
                ],
            },
        },
        555: {
            "attributes": {"g_id": (5615, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388330.0],
                        [168450.0, 5388360.0],
                        [168480.0, 5388360.0],
                        [168480.0, 5388330.0],
                        [168450.0, 5388330.0],
                    ]
                ],
            },
        },
        556: {
            "attributes": {"g_id": (5615, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388360.0],
                        [168450.0, 5388390.0],
                        [168480.0, 5388390.0],
                        [168480.0, 5388360.0],
                        [168450.0, 5388360.0],
                    ]
                ],
            },
        },
        557: {
            "attributes": {"g_id": (5615, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388390.0],
                        [168450.0, 5388420.0],
                        [168480.0, 5388420.0],
                        [168480.0, 5388390.0],
                        [168450.0, 5388390.0],
                    ]
                ],
            },
        },
        558: {
            "attributes": {"g_id": (5615, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388420.0],
                        [168450.0, 5388450.0],
                        [168480.0, 5388450.0],
                        [168480.0, 5388420.0],
                        [168450.0, 5388420.0],
                    ]
                ],
            },
        },
        559: {
            "attributes": {"g_id": (5615, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388450.0],
                        [168450.0, 5388480.0],
                        [168480.0, 5388480.0],
                        [168480.0, 5388450.0],
                        [168450.0, 5388450.0],
                    ]
                ],
            },
        },
        560: {
            "attributes": {"g_id": (5615, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388480.0],
                        [168450.0, 5388510.0],
                        [168480.0, 5388510.0],
                        [168480.0, 5388480.0],
                        [168450.0, 5388480.0],
                    ]
                ],
            },
        },
        561: {
            "attributes": {"g_id": (5615, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388510.0],
                        [168450.0, 5388540.0],
                        [168480.0, 5388540.0],
                        [168480.0, 5388510.0],
                        [168450.0, 5388510.0],
                    ]
                ],
            },
        },
        562: {
            "attributes": {"g_id": (5615, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388540.0],
                        [168450.0, 5388570.0],
                        [168480.0, 5388570.0],
                        [168480.0, 5388540.0],
                        [168450.0, 5388540.0],
                    ]
                ],
            },
        },
        563: {
            "attributes": {"g_id": (5615, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388570.0],
                        [168450.0, 5388600.0],
                        [168480.0, 5388600.0],
                        [168480.0, 5388570.0],
                        [168450.0, 5388570.0],
                    ]
                ],
            },
        },
        564: {
            "attributes": {"g_id": (5615, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388600.0],
                        [168450.0, 5388630.0],
                        [168480.0, 5388630.0],
                        [168480.0, 5388600.0],
                        [168450.0, 5388600.0],
                    ]
                ],
            },
        },
        565: {
            "attributes": {"g_id": (5615, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388630.0],
                        [168450.0, 5388660.0],
                        [168480.0, 5388660.0],
                        [168480.0, 5388630.0],
                        [168450.0, 5388630.0],
                    ]
                ],
            },
        },
        566: {
            "attributes": {"g_id": (5615, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388660.0],
                        [168450.0, 5388690.0],
                        [168480.0, 5388690.0],
                        [168480.0, 5388660.0],
                        [168450.0, 5388660.0],
                    ]
                ],
            },
        },
        567: {
            "attributes": {"g_id": (5615, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388690.0],
                        [168450.0, 5388720.0],
                        [168480.0, 5388720.0],
                        [168480.0, 5388690.0],
                        [168450.0, 5388690.0],
                    ]
                ],
            },
        },
        568: {
            "attributes": {"g_id": (5615, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388720.0],
                        [168450.0, 5388750.0],
                        [168480.0, 5388750.0],
                        [168480.0, 5388720.0],
                        [168450.0, 5388720.0],
                    ]
                ],
            },
        },
        569: {
            "attributes": {"g_id": (5615, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388750.0],
                        [168450.0, 5388780.0],
                        [168480.0, 5388780.0],
                        [168480.0, 5388750.0],
                        [168450.0, 5388750.0],
                    ]
                ],
            },
        },
        570: {
            "attributes": {"g_id": (5615, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388780.0],
                        [168450.0, 5388810.0],
                        [168480.0, 5388810.0],
                        [168480.0, 5388780.0],
                        [168450.0, 5388780.0],
                    ]
                ],
            },
        },
        571: {
            "attributes": {"g_id": (5615, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388810.0],
                        [168450.0, 5388840.0],
                        [168480.0, 5388840.0],
                        [168480.0, 5388810.0],
                        [168450.0, 5388810.0],
                    ]
                ],
            },
        },
        572: {
            "attributes": {"g_id": (5615, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388840.0],
                        [168450.0, 5388870.0],
                        [168480.0, 5388870.0],
                        [168480.0, 5388840.0],
                        [168450.0, 5388840.0],
                    ]
                ],
            },
        },
        573: {
            "attributes": {"g_id": (5615, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388870.0],
                        [168450.0, 5388900.0],
                        [168480.0, 5388900.0],
                        [168480.0, 5388870.0],
                        [168450.0, 5388870.0],
                    ]
                ],
            },
        },
        574: {
            "attributes": {"g_id": (5615, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388900.0],
                        [168450.0, 5388930.0],
                        [168480.0, 5388930.0],
                        [168480.0, 5388900.0],
                        [168450.0, 5388900.0],
                    ]
                ],
            },
        },
        575: {
            "attributes": {"g_id": (5615, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168450.0, 5388930.0],
                        [168450.0, 5388960.0],
                        [168480.0, 5388960.0],
                        [168480.0, 5388930.0],
                        [168450.0, 5388930.0],
                    ]
                ],
            },
        },
        576: {
            "attributes": {"g_id": (5616, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387520.0],
                        [168480.0, 5387550.0],
                        [168510.0, 5387550.0],
                        [168510.0, 5387520.0],
                        [168480.0, 5387520.0],
                    ]
                ],
            },
        },
        577: {
            "attributes": {"g_id": (5616, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387550.0],
                        [168480.0, 5387580.0],
                        [168510.0, 5387580.0],
                        [168510.0, 5387550.0],
                        [168480.0, 5387550.0],
                    ]
                ],
            },
        },
        578: {
            "attributes": {"g_id": (5616, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387580.0],
                        [168480.0, 5387610.0],
                        [168510.0, 5387610.0],
                        [168510.0, 5387580.0],
                        [168480.0, 5387580.0],
                    ]
                ],
            },
        },
        579: {
            "attributes": {"g_id": (5616, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387610.0],
                        [168480.0, 5387640.0],
                        [168510.0, 5387640.0],
                        [168510.0, 5387610.0],
                        [168480.0, 5387610.0],
                    ]
                ],
            },
        },
        580: {
            "attributes": {"g_id": (5616, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387640.0],
                        [168480.0, 5387670.0],
                        [168510.0, 5387670.0],
                        [168510.0, 5387640.0],
                        [168480.0, 5387640.0],
                    ]
                ],
            },
        },
        581: {
            "attributes": {"g_id": (5616, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387670.0],
                        [168480.0, 5387700.0],
                        [168510.0, 5387700.0],
                        [168510.0, 5387670.0],
                        [168480.0, 5387670.0],
                    ]
                ],
            },
        },
        582: {
            "attributes": {"g_id": (5616, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387700.0],
                        [168480.0, 5387730.0],
                        [168510.0, 5387730.0],
                        [168510.0, 5387700.0],
                        [168480.0, 5387700.0],
                    ]
                ],
            },
        },
        583: {
            "attributes": {"g_id": (5616, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387730.0],
                        [168480.0, 5387760.0],
                        [168510.0, 5387760.0],
                        [168510.0, 5387730.0],
                        [168480.0, 5387730.0],
                    ]
                ],
            },
        },
        584: {
            "attributes": {"g_id": (5616, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387760.0],
                        [168480.0, 5387790.0],
                        [168510.0, 5387790.0],
                        [168510.0, 5387760.0],
                        [168480.0, 5387760.0],
                    ]
                ],
            },
        },
        585: {
            "attributes": {"g_id": (5616, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387790.0],
                        [168480.0, 5387820.0],
                        [168510.0, 5387820.0],
                        [168510.0, 5387790.0],
                        [168480.0, 5387790.0],
                    ]
                ],
            },
        },
        586: {
            "attributes": {"g_id": (5616, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387820.0],
                        [168480.0, 5387850.0],
                        [168510.0, 5387850.0],
                        [168510.0, 5387820.0],
                        [168480.0, 5387820.0],
                    ]
                ],
            },
        },
        587: {
            "attributes": {"g_id": (5616, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387850.0],
                        [168480.0, 5387880.0],
                        [168510.0, 5387880.0],
                        [168510.0, 5387850.0],
                        [168480.0, 5387850.0],
                    ]
                ],
            },
        },
        588: {
            "attributes": {"g_id": (5616, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387880.0],
                        [168480.0, 5387910.0],
                        [168510.0, 5387910.0],
                        [168510.0, 5387880.0],
                        [168480.0, 5387880.0],
                    ]
                ],
            },
        },
        589: {
            "attributes": {"g_id": (5616, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387910.0],
                        [168480.0, 5387940.0],
                        [168510.0, 5387940.0],
                        [168510.0, 5387910.0],
                        [168480.0, 5387910.0],
                    ]
                ],
            },
        },
        590: {
            "attributes": {"g_id": (5616, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387940.0],
                        [168480.0, 5387970.0],
                        [168510.0, 5387970.0],
                        [168510.0, 5387940.0],
                        [168480.0, 5387940.0],
                    ]
                ],
            },
        },
        591: {
            "attributes": {"g_id": (5616, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5387970.0],
                        [168480.0, 5388000.0],
                        [168510.0, 5388000.0],
                        [168510.0, 5387970.0],
                        [168480.0, 5387970.0],
                    ]
                ],
            },
        },
        592: {
            "attributes": {"g_id": (5616, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388000.0],
                        [168480.0, 5388030.0],
                        [168510.0, 5388030.0],
                        [168510.0, 5388000.0],
                        [168480.0, 5388000.0],
                    ]
                ],
            },
        },
        593: {
            "attributes": {"g_id": (5616, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388030.0],
                        [168480.0, 5388060.0],
                        [168510.0, 5388060.0],
                        [168510.0, 5388030.0],
                        [168480.0, 5388030.0],
                    ]
                ],
            },
        },
        594: {
            "attributes": {"g_id": (5616, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388060.0],
                        [168480.0, 5388090.0],
                        [168510.0, 5388090.0],
                        [168510.0, 5388060.0],
                        [168480.0, 5388060.0],
                    ]
                ],
            },
        },
        595: {
            "attributes": {"g_id": (5616, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388090.0],
                        [168480.0, 5388120.0],
                        [168510.0, 5388120.0],
                        [168510.0, 5388090.0],
                        [168480.0, 5388090.0],
                    ]
                ],
            },
        },
        596: {
            "attributes": {"g_id": (5616, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388120.0],
                        [168480.0, 5388150.0],
                        [168510.0, 5388150.0],
                        [168510.0, 5388120.0],
                        [168480.0, 5388120.0],
                    ]
                ],
            },
        },
        597: {
            "attributes": {"g_id": (5616, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388150.0],
                        [168480.0, 5388180.0],
                        [168510.0, 5388180.0],
                        [168510.0, 5388150.0],
                        [168480.0, 5388150.0],
                    ]
                ],
            },
        },
        598: {
            "attributes": {"g_id": (5616, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388180.0],
                        [168480.0, 5388210.0],
                        [168510.0, 5388210.0],
                        [168510.0, 5388180.0],
                        [168480.0, 5388180.0],
                    ]
                ],
            },
        },
        599: {
            "attributes": {"g_id": (5616, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388210.0],
                        [168480.0, 5388240.0],
                        [168510.0, 5388240.0],
                        [168510.0, 5388210.0],
                        [168480.0, 5388210.0],
                    ]
                ],
            },
        },
        600: {
            "attributes": {"g_id": (5616, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388240.0],
                        [168480.0, 5388270.0],
                        [168510.0, 5388270.0],
                        [168510.0, 5388240.0],
                        [168480.0, 5388240.0],
                    ]
                ],
            },
        },
        601: {
            "attributes": {"g_id": (5616, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388270.0],
                        [168480.0, 5388300.0],
                        [168510.0, 5388300.0],
                        [168510.0, 5388270.0],
                        [168480.0, 5388270.0],
                    ]
                ],
            },
        },
        602: {
            "attributes": {"g_id": (5616, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388300.0],
                        [168480.0, 5388330.0],
                        [168510.0, 5388330.0],
                        [168510.0, 5388300.0],
                        [168480.0, 5388300.0],
                    ]
                ],
            },
        },
        603: {
            "attributes": {"g_id": (5616, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388330.0],
                        [168480.0, 5388360.0],
                        [168510.0, 5388360.0],
                        [168510.0, 5388330.0],
                        [168480.0, 5388330.0],
                    ]
                ],
            },
        },
        604: {
            "attributes": {"g_id": (5616, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388360.0],
                        [168480.0, 5388390.0],
                        [168510.0, 5388390.0],
                        [168510.0, 5388360.0],
                        [168480.0, 5388360.0],
                    ]
                ],
            },
        },
        605: {
            "attributes": {"g_id": (5616, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388390.0],
                        [168480.0, 5388420.0],
                        [168510.0, 5388420.0],
                        [168510.0, 5388390.0],
                        [168480.0, 5388390.0],
                    ]
                ],
            },
        },
        606: {
            "attributes": {"g_id": (5616, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388420.0],
                        [168480.0, 5388450.0],
                        [168510.0, 5388450.0],
                        [168510.0, 5388420.0],
                        [168480.0, 5388420.0],
                    ]
                ],
            },
        },
        607: {
            "attributes": {"g_id": (5616, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388450.0],
                        [168480.0, 5388480.0],
                        [168510.0, 5388480.0],
                        [168510.0, 5388450.0],
                        [168480.0, 5388450.0],
                    ]
                ],
            },
        },
        608: {
            "attributes": {"g_id": (5616, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388480.0],
                        [168480.0, 5388510.0],
                        [168510.0, 5388510.0],
                        [168510.0, 5388480.0],
                        [168480.0, 5388480.0],
                    ]
                ],
            },
        },
        609: {
            "attributes": {"g_id": (5616, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388510.0],
                        [168480.0, 5388540.0],
                        [168510.0, 5388540.0],
                        [168510.0, 5388510.0],
                        [168480.0, 5388510.0],
                    ]
                ],
            },
        },
        610: {
            "attributes": {"g_id": (5616, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388540.0],
                        [168480.0, 5388570.0],
                        [168510.0, 5388570.0],
                        [168510.0, 5388540.0],
                        [168480.0, 5388540.0],
                    ]
                ],
            },
        },
        611: {
            "attributes": {"g_id": (5616, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388570.0],
                        [168480.0, 5388600.0],
                        [168510.0, 5388600.0],
                        [168510.0, 5388570.0],
                        [168480.0, 5388570.0],
                    ]
                ],
            },
        },
        612: {
            "attributes": {"g_id": (5616, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388600.0],
                        [168480.0, 5388630.0],
                        [168510.0, 5388630.0],
                        [168510.0, 5388600.0],
                        [168480.0, 5388600.0],
                    ]
                ],
            },
        },
        613: {
            "attributes": {"g_id": (5616, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388630.0],
                        [168480.0, 5388660.0],
                        [168510.0, 5388660.0],
                        [168510.0, 5388630.0],
                        [168480.0, 5388630.0],
                    ]
                ],
            },
        },
        614: {
            "attributes": {"g_id": (5616, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388660.0],
                        [168480.0, 5388690.0],
                        [168510.0, 5388690.0],
                        [168510.0, 5388660.0],
                        [168480.0, 5388660.0],
                    ]
                ],
            },
        },
        615: {
            "attributes": {"g_id": (5616, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388690.0],
                        [168480.0, 5388720.0],
                        [168510.0, 5388720.0],
                        [168510.0, 5388690.0],
                        [168480.0, 5388690.0],
                    ]
                ],
            },
        },
        616: {
            "attributes": {"g_id": (5616, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388720.0],
                        [168480.0, 5388750.0],
                        [168510.0, 5388750.0],
                        [168510.0, 5388720.0],
                        [168480.0, 5388720.0],
                    ]
                ],
            },
        },
        617: {
            "attributes": {"g_id": (5616, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388750.0],
                        [168480.0, 5388780.0],
                        [168510.0, 5388780.0],
                        [168510.0, 5388750.0],
                        [168480.0, 5388750.0],
                    ]
                ],
            },
        },
        618: {
            "attributes": {"g_id": (5616, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388780.0],
                        [168480.0, 5388810.0],
                        [168510.0, 5388810.0],
                        [168510.0, 5388780.0],
                        [168480.0, 5388780.0],
                    ]
                ],
            },
        },
        619: {
            "attributes": {"g_id": (5616, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388810.0],
                        [168480.0, 5388840.0],
                        [168510.0, 5388840.0],
                        [168510.0, 5388810.0],
                        [168480.0, 5388810.0],
                    ]
                ],
            },
        },
        620: {
            "attributes": {"g_id": (5616, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388840.0],
                        [168480.0, 5388870.0],
                        [168510.0, 5388870.0],
                        [168510.0, 5388840.0],
                        [168480.0, 5388840.0],
                    ]
                ],
            },
        },
        621: {
            "attributes": {"g_id": (5616, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388870.0],
                        [168480.0, 5388900.0],
                        [168510.0, 5388900.0],
                        [168510.0, 5388870.0],
                        [168480.0, 5388870.0],
                    ]
                ],
            },
        },
        622: {
            "attributes": {"g_id": (5616, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388900.0],
                        [168480.0, 5388930.0],
                        [168510.0, 5388930.0],
                        [168510.0, 5388900.0],
                        [168480.0, 5388900.0],
                    ]
                ],
            },
        },
        623: {
            "attributes": {"g_id": (5616, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168480.0, 5388930.0],
                        [168480.0, 5388960.0],
                        [168510.0, 5388960.0],
                        [168510.0, 5388930.0],
                        [168480.0, 5388930.0],
                    ]
                ],
            },
        },
        624: {
            "attributes": {"g_id": (5617, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387520.0],
                        [168510.0, 5387550.0],
                        [168540.0, 5387550.0],
                        [168540.0, 5387520.0],
                        [168510.0, 5387520.0],
                    ]
                ],
            },
        },
        625: {
            "attributes": {"g_id": (5617, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387550.0],
                        [168510.0, 5387580.0],
                        [168540.0, 5387580.0],
                        [168540.0, 5387550.0],
                        [168510.0, 5387550.0],
                    ]
                ],
            },
        },
        626: {
            "attributes": {"g_id": (5617, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387580.0],
                        [168510.0, 5387610.0],
                        [168540.0, 5387610.0],
                        [168540.0, 5387580.0],
                        [168510.0, 5387580.0],
                    ]
                ],
            },
        },
        627: {
            "attributes": {"g_id": (5617, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387610.0],
                        [168510.0, 5387640.0],
                        [168540.0, 5387640.0],
                        [168540.0, 5387610.0],
                        [168510.0, 5387610.0],
                    ]
                ],
            },
        },
        628: {
            "attributes": {"g_id": (5617, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387640.0],
                        [168510.0, 5387670.0],
                        [168540.0, 5387670.0],
                        [168540.0, 5387640.0],
                        [168510.0, 5387640.0],
                    ]
                ],
            },
        },
        629: {
            "attributes": {"g_id": (5617, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387670.0],
                        [168510.0, 5387700.0],
                        [168540.0, 5387700.0],
                        [168540.0, 5387670.0],
                        [168510.0, 5387670.0],
                    ]
                ],
            },
        },
        630: {
            "attributes": {"g_id": (5617, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387700.0],
                        [168510.0, 5387730.0],
                        [168540.0, 5387730.0],
                        [168540.0, 5387700.0],
                        [168510.0, 5387700.0],
                    ]
                ],
            },
        },
        631: {
            "attributes": {"g_id": (5617, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387730.0],
                        [168510.0, 5387760.0],
                        [168540.0, 5387760.0],
                        [168540.0, 5387730.0],
                        [168510.0, 5387730.0],
                    ]
                ],
            },
        },
        632: {
            "attributes": {"g_id": (5617, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387760.0],
                        [168510.0, 5387790.0],
                        [168540.0, 5387790.0],
                        [168540.0, 5387760.0],
                        [168510.0, 5387760.0],
                    ]
                ],
            },
        },
        633: {
            "attributes": {"g_id": (5617, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387790.0],
                        [168510.0, 5387820.0],
                        [168540.0, 5387820.0],
                        [168540.0, 5387790.0],
                        [168510.0, 5387790.0],
                    ]
                ],
            },
        },
        634: {
            "attributes": {"g_id": (5617, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387820.0],
                        [168510.0, 5387850.0],
                        [168540.0, 5387850.0],
                        [168540.0, 5387820.0],
                        [168510.0, 5387820.0],
                    ]
                ],
            },
        },
        635: {
            "attributes": {"g_id": (5617, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387850.0],
                        [168510.0, 5387880.0],
                        [168540.0, 5387880.0],
                        [168540.0, 5387850.0],
                        [168510.0, 5387850.0],
                    ]
                ],
            },
        },
        636: {
            "attributes": {"g_id": (5617, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387880.0],
                        [168510.0, 5387910.0],
                        [168540.0, 5387910.0],
                        [168540.0, 5387880.0],
                        [168510.0, 5387880.0],
                    ]
                ],
            },
        },
        637: {
            "attributes": {"g_id": (5617, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387910.0],
                        [168510.0, 5387940.0],
                        [168540.0, 5387940.0],
                        [168540.0, 5387910.0],
                        [168510.0, 5387910.0],
                    ]
                ],
            },
        },
        638: {
            "attributes": {"g_id": (5617, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387940.0],
                        [168510.0, 5387970.0],
                        [168540.0, 5387970.0],
                        [168540.0, 5387940.0],
                        [168510.0, 5387940.0],
                    ]
                ],
            },
        },
        639: {
            "attributes": {"g_id": (5617, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5387970.0],
                        [168510.0, 5388000.0],
                        [168540.0, 5388000.0],
                        [168540.0, 5387970.0],
                        [168510.0, 5387970.0],
                    ]
                ],
            },
        },
        640: {
            "attributes": {"g_id": (5617, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388000.0],
                        [168510.0, 5388030.0],
                        [168540.0, 5388030.0],
                        [168540.0, 5388000.0],
                        [168510.0, 5388000.0],
                    ]
                ],
            },
        },
        641: {
            "attributes": {"g_id": (5617, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388030.0],
                        [168510.0, 5388060.0],
                        [168540.0, 5388060.0],
                        [168540.0, 5388030.0],
                        [168510.0, 5388030.0],
                    ]
                ],
            },
        },
        642: {
            "attributes": {"g_id": (5617, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388060.0],
                        [168510.0, 5388090.0],
                        [168540.0, 5388090.0],
                        [168540.0, 5388060.0],
                        [168510.0, 5388060.0],
                    ]
                ],
            },
        },
        643: {
            "attributes": {"g_id": (5617, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388090.0],
                        [168510.0, 5388120.0],
                        [168540.0, 5388120.0],
                        [168540.0, 5388090.0],
                        [168510.0, 5388090.0],
                    ]
                ],
            },
        },
        644: {
            "attributes": {"g_id": (5617, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388120.0],
                        [168510.0, 5388150.0],
                        [168540.0, 5388150.0],
                        [168540.0, 5388120.0],
                        [168510.0, 5388120.0],
                    ]
                ],
            },
        },
        645: {
            "attributes": {"g_id": (5617, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388150.0],
                        [168510.0, 5388180.0],
                        [168540.0, 5388180.0],
                        [168540.0, 5388150.0],
                        [168510.0, 5388150.0],
                    ]
                ],
            },
        },
        646: {
            "attributes": {"g_id": (5617, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388180.0],
                        [168510.0, 5388210.0],
                        [168540.0, 5388210.0],
                        [168540.0, 5388180.0],
                        [168510.0, 5388180.0],
                    ]
                ],
            },
        },
        647: {
            "attributes": {"g_id": (5617, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388210.0],
                        [168510.0, 5388240.0],
                        [168540.0, 5388240.0],
                        [168540.0, 5388210.0],
                        [168510.0, 5388210.0],
                    ]
                ],
            },
        },
        648: {
            "attributes": {"g_id": (5617, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388240.0],
                        [168510.0, 5388270.0],
                        [168540.0, 5388270.0],
                        [168540.0, 5388240.0],
                        [168510.0, 5388240.0],
                    ]
                ],
            },
        },
        649: {
            "attributes": {"g_id": (5617, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388270.0],
                        [168510.0, 5388300.0],
                        [168540.0, 5388300.0],
                        [168540.0, 5388270.0],
                        [168510.0, 5388270.0],
                    ]
                ],
            },
        },
        650: {
            "attributes": {"g_id": (5617, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388300.0],
                        [168510.0, 5388330.0],
                        [168540.0, 5388330.0],
                        [168540.0, 5388300.0],
                        [168510.0, 5388300.0],
                    ]
                ],
            },
        },
        651: {
            "attributes": {"g_id": (5617, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388330.0],
                        [168510.0, 5388360.0],
                        [168540.0, 5388360.0],
                        [168540.0, 5388330.0],
                        [168510.0, 5388330.0],
                    ]
                ],
            },
        },
        652: {
            "attributes": {"g_id": (5617, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388360.0],
                        [168510.0, 5388390.0],
                        [168540.0, 5388390.0],
                        [168540.0, 5388360.0],
                        [168510.0, 5388360.0],
                    ]
                ],
            },
        },
        653: {
            "attributes": {"g_id": (5617, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388390.0],
                        [168510.0, 5388420.0],
                        [168540.0, 5388420.0],
                        [168540.0, 5388390.0],
                        [168510.0, 5388390.0],
                    ]
                ],
            },
        },
        654: {
            "attributes": {"g_id": (5617, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388420.0],
                        [168510.0, 5388450.0],
                        [168540.0, 5388450.0],
                        [168540.0, 5388420.0],
                        [168510.0, 5388420.0],
                    ]
                ],
            },
        },
        655: {
            "attributes": {"g_id": (5617, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388450.0],
                        [168510.0, 5388480.0],
                        [168540.0, 5388480.0],
                        [168540.0, 5388450.0],
                        [168510.0, 5388450.0],
                    ]
                ],
            },
        },
        656: {
            "attributes": {"g_id": (5617, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388480.0],
                        [168510.0, 5388510.0],
                        [168540.0, 5388510.0],
                        [168540.0, 5388480.0],
                        [168510.0, 5388480.0],
                    ]
                ],
            },
        },
        657: {
            "attributes": {"g_id": (5617, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388510.0],
                        [168510.0, 5388540.0],
                        [168540.0, 5388540.0],
                        [168540.0, 5388510.0],
                        [168510.0, 5388510.0],
                    ]
                ],
            },
        },
        658: {
            "attributes": {"g_id": (5617, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388540.0],
                        [168510.0, 5388570.0],
                        [168540.0, 5388570.0],
                        [168540.0, 5388540.0],
                        [168510.0, 5388540.0],
                    ]
                ],
            },
        },
        659: {
            "attributes": {"g_id": (5617, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388570.0],
                        [168510.0, 5388600.0],
                        [168540.0, 5388600.0],
                        [168540.0, 5388570.0],
                        [168510.0, 5388570.0],
                    ]
                ],
            },
        },
        660: {
            "attributes": {"g_id": (5617, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388600.0],
                        [168510.0, 5388630.0],
                        [168540.0, 5388630.0],
                        [168540.0, 5388600.0],
                        [168510.0, 5388600.0],
                    ]
                ],
            },
        },
        661: {
            "attributes": {"g_id": (5617, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388630.0],
                        [168510.0, 5388660.0],
                        [168540.0, 5388660.0],
                        [168540.0, 5388630.0],
                        [168510.0, 5388630.0],
                    ]
                ],
            },
        },
        662: {
            "attributes": {"g_id": (5617, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388660.0],
                        [168510.0, 5388690.0],
                        [168540.0, 5388690.0],
                        [168540.0, 5388660.0],
                        [168510.0, 5388660.0],
                    ]
                ],
            },
        },
        663: {
            "attributes": {"g_id": (5617, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388690.0],
                        [168510.0, 5388720.0],
                        [168540.0, 5388720.0],
                        [168540.0, 5388690.0],
                        [168510.0, 5388690.0],
                    ]
                ],
            },
        },
        664: {
            "attributes": {"g_id": (5617, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388720.0],
                        [168510.0, 5388750.0],
                        [168540.0, 5388750.0],
                        [168540.0, 5388720.0],
                        [168510.0, 5388720.0],
                    ]
                ],
            },
        },
        665: {
            "attributes": {"g_id": (5617, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388750.0],
                        [168510.0, 5388780.0],
                        [168540.0, 5388780.0],
                        [168540.0, 5388750.0],
                        [168510.0, 5388750.0],
                    ]
                ],
            },
        },
        666: {
            "attributes": {"g_id": (5617, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388780.0],
                        [168510.0, 5388810.0],
                        [168540.0, 5388810.0],
                        [168540.0, 5388780.0],
                        [168510.0, 5388780.0],
                    ]
                ],
            },
        },
        667: {
            "attributes": {"g_id": (5617, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388810.0],
                        [168510.0, 5388840.0],
                        [168540.0, 5388840.0],
                        [168540.0, 5388810.0],
                        [168510.0, 5388810.0],
                    ]
                ],
            },
        },
        668: {
            "attributes": {"g_id": (5617, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388840.0],
                        [168510.0, 5388870.0],
                        [168540.0, 5388870.0],
                        [168540.0, 5388840.0],
                        [168510.0, 5388840.0],
                    ]
                ],
            },
        },
        669: {
            "attributes": {"g_id": (5617, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388870.0],
                        [168510.0, 5388900.0],
                        [168540.0, 5388900.0],
                        [168540.0, 5388870.0],
                        [168510.0, 5388870.0],
                    ]
                ],
            },
        },
        670: {
            "attributes": {"g_id": (5617, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388900.0],
                        [168510.0, 5388930.0],
                        [168540.0, 5388930.0],
                        [168540.0, 5388900.0],
                        [168510.0, 5388900.0],
                    ]
                ],
            },
        },
        671: {
            "attributes": {"g_id": (5617, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168510.0, 5388930.0],
                        [168510.0, 5388960.0],
                        [168540.0, 5388960.0],
                        [168540.0, 5388930.0],
                        [168510.0, 5388930.0],
                    ]
                ],
            },
        },
        672: {
            "attributes": {"g_id": (5618, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387520.0],
                        [168540.0, 5387550.0],
                        [168570.0, 5387550.0],
                        [168570.0, 5387520.0],
                        [168540.0, 5387520.0],
                    ]
                ],
            },
        },
        673: {
            "attributes": {"g_id": (5618, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387550.0],
                        [168540.0, 5387580.0],
                        [168570.0, 5387580.0],
                        [168570.0, 5387550.0],
                        [168540.0, 5387550.0],
                    ]
                ],
            },
        },
        674: {
            "attributes": {"g_id": (5618, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387580.0],
                        [168540.0, 5387610.0],
                        [168570.0, 5387610.0],
                        [168570.0, 5387580.0],
                        [168540.0, 5387580.0],
                    ]
                ],
            },
        },
        675: {
            "attributes": {"g_id": (5618, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387610.0],
                        [168540.0, 5387640.0],
                        [168570.0, 5387640.0],
                        [168570.0, 5387610.0],
                        [168540.0, 5387610.0],
                    ]
                ],
            },
        },
        676: {
            "attributes": {"g_id": (5618, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387640.0],
                        [168540.0, 5387670.0],
                        [168570.0, 5387670.0],
                        [168570.0, 5387640.0],
                        [168540.0, 5387640.0],
                    ]
                ],
            },
        },
        677: {
            "attributes": {"g_id": (5618, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387670.0],
                        [168540.0, 5387700.0],
                        [168570.0, 5387700.0],
                        [168570.0, 5387670.0],
                        [168540.0, 5387670.0],
                    ]
                ],
            },
        },
        678: {
            "attributes": {"g_id": (5618, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387700.0],
                        [168540.0, 5387730.0],
                        [168570.0, 5387730.0],
                        [168570.0, 5387700.0],
                        [168540.0, 5387700.0],
                    ]
                ],
            },
        },
        679: {
            "attributes": {"g_id": (5618, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387730.0],
                        [168540.0, 5387760.0],
                        [168570.0, 5387760.0],
                        [168570.0, 5387730.0],
                        [168540.0, 5387730.0],
                    ]
                ],
            },
        },
        680: {
            "attributes": {"g_id": (5618, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387760.0],
                        [168540.0, 5387790.0],
                        [168570.0, 5387790.0],
                        [168570.0, 5387760.0],
                        [168540.0, 5387760.0],
                    ]
                ],
            },
        },
        681: {
            "attributes": {"g_id": (5618, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387790.0],
                        [168540.0, 5387820.0],
                        [168570.0, 5387820.0],
                        [168570.0, 5387790.0],
                        [168540.0, 5387790.0],
                    ]
                ],
            },
        },
        682: {
            "attributes": {"g_id": (5618, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387820.0],
                        [168540.0, 5387850.0],
                        [168570.0, 5387850.0],
                        [168570.0, 5387820.0],
                        [168540.0, 5387820.0],
                    ]
                ],
            },
        },
        683: {
            "attributes": {"g_id": (5618, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387850.0],
                        [168540.0, 5387880.0],
                        [168570.0, 5387880.0],
                        [168570.0, 5387850.0],
                        [168540.0, 5387850.0],
                    ]
                ],
            },
        },
        684: {
            "attributes": {"g_id": (5618, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387880.0],
                        [168540.0, 5387910.0],
                        [168570.0, 5387910.0],
                        [168570.0, 5387880.0],
                        [168540.0, 5387880.0],
                    ]
                ],
            },
        },
        685: {
            "attributes": {"g_id": (5618, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387910.0],
                        [168540.0, 5387940.0],
                        [168570.0, 5387940.0],
                        [168570.0, 5387910.0],
                        [168540.0, 5387910.0],
                    ]
                ],
            },
        },
        686: {
            "attributes": {"g_id": (5618, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387940.0],
                        [168540.0, 5387970.0],
                        [168570.0, 5387970.0],
                        [168570.0, 5387940.0],
                        [168540.0, 5387940.0],
                    ]
                ],
            },
        },
        687: {
            "attributes": {"g_id": (5618, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5387970.0],
                        [168540.0, 5388000.0],
                        [168570.0, 5388000.0],
                        [168570.0, 5387970.0],
                        [168540.0, 5387970.0],
                    ]
                ],
            },
        },
        688: {
            "attributes": {"g_id": (5618, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388000.0],
                        [168540.0, 5388030.0],
                        [168570.0, 5388030.0],
                        [168570.0, 5388000.0],
                        [168540.0, 5388000.0],
                    ]
                ],
            },
        },
        689: {
            "attributes": {"g_id": (5618, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388030.0],
                        [168540.0, 5388060.0],
                        [168570.0, 5388060.0],
                        [168570.0, 5388030.0],
                        [168540.0, 5388030.0],
                    ]
                ],
            },
        },
        690: {
            "attributes": {"g_id": (5618, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388060.0],
                        [168540.0, 5388090.0],
                        [168570.0, 5388090.0],
                        [168570.0, 5388060.0],
                        [168540.0, 5388060.0],
                    ]
                ],
            },
        },
        691: {
            "attributes": {"g_id": (5618, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388090.0],
                        [168540.0, 5388120.0],
                        [168570.0, 5388120.0],
                        [168570.0, 5388090.0],
                        [168540.0, 5388090.0],
                    ]
                ],
            },
        },
        692: {
            "attributes": {"g_id": (5618, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388120.0],
                        [168540.0, 5388150.0],
                        [168570.0, 5388150.0],
                        [168570.0, 5388120.0],
                        [168540.0, 5388120.0],
                    ]
                ],
            },
        },
        693: {
            "attributes": {"g_id": (5618, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388150.0],
                        [168540.0, 5388180.0],
                        [168570.0, 5388180.0],
                        [168570.0, 5388150.0],
                        [168540.0, 5388150.0],
                    ]
                ],
            },
        },
        694: {
            "attributes": {"g_id": (5618, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388180.0],
                        [168540.0, 5388210.0],
                        [168570.0, 5388210.0],
                        [168570.0, 5388180.0],
                        [168540.0, 5388180.0],
                    ]
                ],
            },
        },
        695: {
            "attributes": {"g_id": (5618, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388210.0],
                        [168540.0, 5388240.0],
                        [168570.0, 5388240.0],
                        [168570.0, 5388210.0],
                        [168540.0, 5388210.0],
                    ]
                ],
            },
        },
        696: {
            "attributes": {"g_id": (5618, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388240.0],
                        [168540.0, 5388270.0],
                        [168570.0, 5388270.0],
                        [168570.0, 5388240.0],
                        [168540.0, 5388240.0],
                    ]
                ],
            },
        },
        697: {
            "attributes": {"g_id": (5618, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388270.0],
                        [168540.0, 5388300.0],
                        [168570.0, 5388300.0],
                        [168570.0, 5388270.0],
                        [168540.0, 5388270.0],
                    ]
                ],
            },
        },
        698: {
            "attributes": {"g_id": (5618, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388300.0],
                        [168540.0, 5388330.0],
                        [168570.0, 5388330.0],
                        [168570.0, 5388300.0],
                        [168540.0, 5388300.0],
                    ]
                ],
            },
        },
        699: {
            "attributes": {"g_id": (5618, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388330.0],
                        [168540.0, 5388360.0],
                        [168570.0, 5388360.0],
                        [168570.0, 5388330.0],
                        [168540.0, 5388330.0],
                    ]
                ],
            },
        },
        700: {
            "attributes": {"g_id": (5618, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388360.0],
                        [168540.0, 5388390.0],
                        [168570.0, 5388390.0],
                        [168570.0, 5388360.0],
                        [168540.0, 5388360.0],
                    ]
                ],
            },
        },
        701: {
            "attributes": {"g_id": (5618, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388390.0],
                        [168540.0, 5388420.0],
                        [168570.0, 5388420.0],
                        [168570.0, 5388390.0],
                        [168540.0, 5388390.0],
                    ]
                ],
            },
        },
        702: {
            "attributes": {"g_id": (5618, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388420.0],
                        [168540.0, 5388450.0],
                        [168570.0, 5388450.0],
                        [168570.0, 5388420.0],
                        [168540.0, 5388420.0],
                    ]
                ],
            },
        },
        703: {
            "attributes": {"g_id": (5618, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388450.0],
                        [168540.0, 5388480.0],
                        [168570.0, 5388480.0],
                        [168570.0, 5388450.0],
                        [168540.0, 5388450.0],
                    ]
                ],
            },
        },
        704: {
            "attributes": {"g_id": (5618, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388480.0],
                        [168540.0, 5388510.0],
                        [168570.0, 5388510.0],
                        [168570.0, 5388480.0],
                        [168540.0, 5388480.0],
                    ]
                ],
            },
        },
        705: {
            "attributes": {"g_id": (5618, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388510.0],
                        [168540.0, 5388540.0],
                        [168570.0, 5388540.0],
                        [168570.0, 5388510.0],
                        [168540.0, 5388510.0],
                    ]
                ],
            },
        },
        706: {
            "attributes": {"g_id": (5618, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388540.0],
                        [168540.0, 5388570.0],
                        [168570.0, 5388570.0],
                        [168570.0, 5388540.0],
                        [168540.0, 5388540.0],
                    ]
                ],
            },
        },
        707: {
            "attributes": {"g_id": (5618, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388570.0],
                        [168540.0, 5388600.0],
                        [168570.0, 5388600.0],
                        [168570.0, 5388570.0],
                        [168540.0, 5388570.0],
                    ]
                ],
            },
        },
        708: {
            "attributes": {"g_id": (5618, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388600.0],
                        [168540.0, 5388630.0],
                        [168570.0, 5388630.0],
                        [168570.0, 5388600.0],
                        [168540.0, 5388600.0],
                    ]
                ],
            },
        },
        709: {
            "attributes": {"g_id": (5618, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388630.0],
                        [168540.0, 5388660.0],
                        [168570.0, 5388660.0],
                        [168570.0, 5388630.0],
                        [168540.0, 5388630.0],
                    ]
                ],
            },
        },
        710: {
            "attributes": {"g_id": (5618, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388660.0],
                        [168540.0, 5388690.0],
                        [168570.0, 5388690.0],
                        [168570.0, 5388660.0],
                        [168540.0, 5388660.0],
                    ]
                ],
            },
        },
        711: {
            "attributes": {"g_id": (5618, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388690.0],
                        [168540.0, 5388720.0],
                        [168570.0, 5388720.0],
                        [168570.0, 5388690.0],
                        [168540.0, 5388690.0],
                    ]
                ],
            },
        },
        712: {
            "attributes": {"g_id": (5618, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388720.0],
                        [168540.0, 5388750.0],
                        [168570.0, 5388750.0],
                        [168570.0, 5388720.0],
                        [168540.0, 5388720.0],
                    ]
                ],
            },
        },
        713: {
            "attributes": {"g_id": (5618, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388750.0],
                        [168540.0, 5388780.0],
                        [168570.0, 5388780.0],
                        [168570.0, 5388750.0],
                        [168540.0, 5388750.0],
                    ]
                ],
            },
        },
        714: {
            "attributes": {"g_id": (5618, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388780.0],
                        [168540.0, 5388810.0],
                        [168570.0, 5388810.0],
                        [168570.0, 5388780.0],
                        [168540.0, 5388780.0],
                    ]
                ],
            },
        },
        715: {
            "attributes": {"g_id": (5618, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388810.0],
                        [168540.0, 5388840.0],
                        [168570.0, 5388840.0],
                        [168570.0, 5388810.0],
                        [168540.0, 5388810.0],
                    ]
                ],
            },
        },
        716: {
            "attributes": {"g_id": (5618, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388840.0],
                        [168540.0, 5388870.0],
                        [168570.0, 5388870.0],
                        [168570.0, 5388840.0],
                        [168540.0, 5388840.0],
                    ]
                ],
            },
        },
        717: {
            "attributes": {"g_id": (5618, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388870.0],
                        [168540.0, 5388900.0],
                        [168570.0, 5388900.0],
                        [168570.0, 5388870.0],
                        [168540.0, 5388870.0],
                    ]
                ],
            },
        },
        718: {
            "attributes": {"g_id": (5618, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388900.0],
                        [168540.0, 5388930.0],
                        [168570.0, 5388930.0],
                        [168570.0, 5388900.0],
                        [168540.0, 5388900.0],
                    ]
                ],
            },
        },
        719: {
            "attributes": {"g_id": (5618, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168540.0, 5388930.0],
                        [168540.0, 5388960.0],
                        [168570.0, 5388960.0],
                        [168570.0, 5388930.0],
                        [168540.0, 5388930.0],
                    ]
                ],
            },
        },
        720: {
            "attributes": {"g_id": (5619, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387520.0],
                        [168570.0, 5387550.0],
                        [168600.0, 5387550.0],
                        [168600.0, 5387520.0],
                        [168570.0, 5387520.0],
                    ]
                ],
            },
        },
        721: {
            "attributes": {"g_id": (5619, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387550.0],
                        [168570.0, 5387580.0],
                        [168600.0, 5387580.0],
                        [168600.0, 5387550.0],
                        [168570.0, 5387550.0],
                    ]
                ],
            },
        },
        722: {
            "attributes": {"g_id": (5619, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387580.0],
                        [168570.0, 5387610.0],
                        [168600.0, 5387610.0],
                        [168600.0, 5387580.0],
                        [168570.0, 5387580.0],
                    ]
                ],
            },
        },
        723: {
            "attributes": {"g_id": (5619, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387610.0],
                        [168570.0, 5387640.0],
                        [168600.0, 5387640.0],
                        [168600.0, 5387610.0],
                        [168570.0, 5387610.0],
                    ]
                ],
            },
        },
        724: {
            "attributes": {"g_id": (5619, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387640.0],
                        [168570.0, 5387670.0],
                        [168600.0, 5387670.0],
                        [168600.0, 5387640.0],
                        [168570.0, 5387640.0],
                    ]
                ],
            },
        },
        725: {
            "attributes": {"g_id": (5619, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387670.0],
                        [168570.0, 5387700.0],
                        [168600.0, 5387700.0],
                        [168600.0, 5387670.0],
                        [168570.0, 5387670.0],
                    ]
                ],
            },
        },
        726: {
            "attributes": {"g_id": (5619, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387700.0],
                        [168570.0, 5387730.0],
                        [168600.0, 5387730.0],
                        [168600.0, 5387700.0],
                        [168570.0, 5387700.0],
                    ]
                ],
            },
        },
        727: {
            "attributes": {"g_id": (5619, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387730.0],
                        [168570.0, 5387760.0],
                        [168600.0, 5387760.0],
                        [168600.0, 5387730.0],
                        [168570.0, 5387730.0],
                    ]
                ],
            },
        },
        728: {
            "attributes": {"g_id": (5619, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387760.0],
                        [168570.0, 5387790.0],
                        [168600.0, 5387790.0],
                        [168600.0, 5387760.0],
                        [168570.0, 5387760.0],
                    ]
                ],
            },
        },
        729: {
            "attributes": {"g_id": (5619, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387790.0],
                        [168570.0, 5387820.0],
                        [168600.0, 5387820.0],
                        [168600.0, 5387790.0],
                        [168570.0, 5387790.0],
                    ]
                ],
            },
        },
        730: {
            "attributes": {"g_id": (5619, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387820.0],
                        [168570.0, 5387850.0],
                        [168600.0, 5387850.0],
                        [168600.0, 5387820.0],
                        [168570.0, 5387820.0],
                    ]
                ],
            },
        },
        731: {
            "attributes": {"g_id": (5619, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387850.0],
                        [168570.0, 5387880.0],
                        [168600.0, 5387880.0],
                        [168600.0, 5387850.0],
                        [168570.0, 5387850.0],
                    ]
                ],
            },
        },
        732: {
            "attributes": {"g_id": (5619, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387880.0],
                        [168570.0, 5387910.0],
                        [168600.0, 5387910.0],
                        [168600.0, 5387880.0],
                        [168570.0, 5387880.0],
                    ]
                ],
            },
        },
        733: {
            "attributes": {"g_id": (5619, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387910.0],
                        [168570.0, 5387940.0],
                        [168600.0, 5387940.0],
                        [168600.0, 5387910.0],
                        [168570.0, 5387910.0],
                    ]
                ],
            },
        },
        734: {
            "attributes": {"g_id": (5619, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387940.0],
                        [168570.0, 5387970.0],
                        [168600.0, 5387970.0],
                        [168600.0, 5387940.0],
                        [168570.0, 5387940.0],
                    ]
                ],
            },
        },
        735: {
            "attributes": {"g_id": (5619, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5387970.0],
                        [168570.0, 5388000.0],
                        [168600.0, 5388000.0],
                        [168600.0, 5387970.0],
                        [168570.0, 5387970.0],
                    ]
                ],
            },
        },
        736: {
            "attributes": {"g_id": (5619, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388000.0],
                        [168570.0, 5388030.0],
                        [168600.0, 5388030.0],
                        [168600.0, 5388000.0],
                        [168570.0, 5388000.0],
                    ]
                ],
            },
        },
        737: {
            "attributes": {"g_id": (5619, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388030.0],
                        [168570.0, 5388060.0],
                        [168600.0, 5388060.0],
                        [168600.0, 5388030.0],
                        [168570.0, 5388030.0],
                    ]
                ],
            },
        },
        738: {
            "attributes": {"g_id": (5619, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388060.0],
                        [168570.0, 5388090.0],
                        [168600.0, 5388090.0],
                        [168600.0, 5388060.0],
                        [168570.0, 5388060.0],
                    ]
                ],
            },
        },
        739: {
            "attributes": {"g_id": (5619, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388090.0],
                        [168570.0, 5388120.0],
                        [168600.0, 5388120.0],
                        [168600.0, 5388090.0],
                        [168570.0, 5388090.0],
                    ]
                ],
            },
        },
        740: {
            "attributes": {"g_id": (5619, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388120.0],
                        [168570.0, 5388150.0],
                        [168600.0, 5388150.0],
                        [168600.0, 5388120.0],
                        [168570.0, 5388120.0],
                    ]
                ],
            },
        },
        741: {
            "attributes": {"g_id": (5619, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388150.0],
                        [168570.0, 5388180.0],
                        [168600.0, 5388180.0],
                        [168600.0, 5388150.0],
                        [168570.0, 5388150.0],
                    ]
                ],
            },
        },
        742: {
            "attributes": {"g_id": (5619, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388180.0],
                        [168570.0, 5388210.0],
                        [168600.0, 5388210.0],
                        [168600.0, 5388180.0],
                        [168570.0, 5388180.0],
                    ]
                ],
            },
        },
        743: {
            "attributes": {"g_id": (5619, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388210.0],
                        [168570.0, 5388240.0],
                        [168600.0, 5388240.0],
                        [168600.0, 5388210.0],
                        [168570.0, 5388210.0],
                    ]
                ],
            },
        },
        744: {
            "attributes": {"g_id": (5619, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388240.0],
                        [168570.0, 5388270.0],
                        [168600.0, 5388270.0],
                        [168600.0, 5388240.0],
                        [168570.0, 5388240.0],
                    ]
                ],
            },
        },
        745: {
            "attributes": {"g_id": (5619, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388270.0],
                        [168570.0, 5388300.0],
                        [168600.0, 5388300.0],
                        [168600.0, 5388270.0],
                        [168570.0, 5388270.0],
                    ]
                ],
            },
        },
        746: {
            "attributes": {"g_id": (5619, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388300.0],
                        [168570.0, 5388330.0],
                        [168600.0, 5388330.0],
                        [168600.0, 5388300.0],
                        [168570.0, 5388300.0],
                    ]
                ],
            },
        },
        747: {
            "attributes": {"g_id": (5619, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388330.0],
                        [168570.0, 5388360.0],
                        [168600.0, 5388360.0],
                        [168600.0, 5388330.0],
                        [168570.0, 5388330.0],
                    ]
                ],
            },
        },
        748: {
            "attributes": {"g_id": (5619, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388360.0],
                        [168570.0, 5388390.0],
                        [168600.0, 5388390.0],
                        [168600.0, 5388360.0],
                        [168570.0, 5388360.0],
                    ]
                ],
            },
        },
        749: {
            "attributes": {"g_id": (5619, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388390.0],
                        [168570.0, 5388420.0],
                        [168600.0, 5388420.0],
                        [168600.0, 5388390.0],
                        [168570.0, 5388390.0],
                    ]
                ],
            },
        },
        750: {
            "attributes": {"g_id": (5619, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388420.0],
                        [168570.0, 5388450.0],
                        [168600.0, 5388450.0],
                        [168600.0, 5388420.0],
                        [168570.0, 5388420.0],
                    ]
                ],
            },
        },
        751: {
            "attributes": {"g_id": (5619, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388450.0],
                        [168570.0, 5388480.0],
                        [168600.0, 5388480.0],
                        [168600.0, 5388450.0],
                        [168570.0, 5388450.0],
                    ]
                ],
            },
        },
        752: {
            "attributes": {"g_id": (5619, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388480.0],
                        [168570.0, 5388510.0],
                        [168600.0, 5388510.0],
                        [168600.0, 5388480.0],
                        [168570.0, 5388480.0],
                    ]
                ],
            },
        },
        753: {
            "attributes": {"g_id": (5619, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388510.0],
                        [168570.0, 5388540.0],
                        [168600.0, 5388540.0],
                        [168600.0, 5388510.0],
                        [168570.0, 5388510.0],
                    ]
                ],
            },
        },
        754: {
            "attributes": {"g_id": (5619, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388540.0],
                        [168570.0, 5388570.0],
                        [168600.0, 5388570.0],
                        [168600.0, 5388540.0],
                        [168570.0, 5388540.0],
                    ]
                ],
            },
        },
        755: {
            "attributes": {"g_id": (5619, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388570.0],
                        [168570.0, 5388600.0],
                        [168600.0, 5388600.0],
                        [168600.0, 5388570.0],
                        [168570.0, 5388570.0],
                    ]
                ],
            },
        },
        756: {
            "attributes": {"g_id": (5619, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388600.0],
                        [168570.0, 5388630.0],
                        [168600.0, 5388630.0],
                        [168600.0, 5388600.0],
                        [168570.0, 5388600.0],
                    ]
                ],
            },
        },
        757: {
            "attributes": {"g_id": (5619, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388630.0],
                        [168570.0, 5388660.0],
                        [168600.0, 5388660.0],
                        [168600.0, 5388630.0],
                        [168570.0, 5388630.0],
                    ]
                ],
            },
        },
        758: {
            "attributes": {"g_id": (5619, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388660.0],
                        [168570.0, 5388690.0],
                        [168600.0, 5388690.0],
                        [168600.0, 5388660.0],
                        [168570.0, 5388660.0],
                    ]
                ],
            },
        },
        759: {
            "attributes": {"g_id": (5619, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388690.0],
                        [168570.0, 5388720.0],
                        [168600.0, 5388720.0],
                        [168600.0, 5388690.0],
                        [168570.0, 5388690.0],
                    ]
                ],
            },
        },
        760: {
            "attributes": {"g_id": (5619, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388720.0],
                        [168570.0, 5388750.0],
                        [168600.0, 5388750.0],
                        [168600.0, 5388720.0],
                        [168570.0, 5388720.0],
                    ]
                ],
            },
        },
        761: {
            "attributes": {"g_id": (5619, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388750.0],
                        [168570.0, 5388780.0],
                        [168600.0, 5388780.0],
                        [168600.0, 5388750.0],
                        [168570.0, 5388750.0],
                    ]
                ],
            },
        },
        762: {
            "attributes": {"g_id": (5619, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388780.0],
                        [168570.0, 5388810.0],
                        [168600.0, 5388810.0],
                        [168600.0, 5388780.0],
                        [168570.0, 5388780.0],
                    ]
                ],
            },
        },
        763: {
            "attributes": {"g_id": (5619, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388810.0],
                        [168570.0, 5388840.0],
                        [168600.0, 5388840.0],
                        [168600.0, 5388810.0],
                        [168570.0, 5388810.0],
                    ]
                ],
            },
        },
        764: {
            "attributes": {"g_id": (5619, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388840.0],
                        [168570.0, 5388870.0],
                        [168600.0, 5388870.0],
                        [168600.0, 5388840.0],
                        [168570.0, 5388840.0],
                    ]
                ],
            },
        },
        765: {
            "attributes": {"g_id": (5619, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388870.0],
                        [168570.0, 5388900.0],
                        [168600.0, 5388900.0],
                        [168600.0, 5388870.0],
                        [168570.0, 5388870.0],
                    ]
                ],
            },
        },
        766: {
            "attributes": {"g_id": (5619, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388900.0],
                        [168570.0, 5388930.0],
                        [168600.0, 5388930.0],
                        [168600.0, 5388900.0],
                        [168570.0, 5388900.0],
                    ]
                ],
            },
        },
        767: {
            "attributes": {"g_id": (5619, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168570.0, 5388930.0],
                        [168570.0, 5388960.0],
                        [168600.0, 5388960.0],
                        [168600.0, 5388930.0],
                        [168570.0, 5388930.0],
                    ]
                ],
            },
        },
        768: {
            "attributes": {"g_id": (5620, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387520.0],
                        [168600.0, 5387550.0],
                        [168630.0, 5387550.0],
                        [168630.0, 5387520.0],
                        [168600.0, 5387520.0],
                    ]
                ],
            },
        },
        769: {
            "attributes": {"g_id": (5620, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387550.0],
                        [168600.0, 5387580.0],
                        [168630.0, 5387580.0],
                        [168630.0, 5387550.0],
                        [168600.0, 5387550.0],
                    ]
                ],
            },
        },
        770: {
            "attributes": {"g_id": (5620, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387580.0],
                        [168600.0, 5387610.0],
                        [168630.0, 5387610.0],
                        [168630.0, 5387580.0],
                        [168600.0, 5387580.0],
                    ]
                ],
            },
        },
        771: {
            "attributes": {"g_id": (5620, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387610.0],
                        [168600.0, 5387640.0],
                        [168630.0, 5387640.0],
                        [168630.0, 5387610.0],
                        [168600.0, 5387610.0],
                    ]
                ],
            },
        },
        772: {
            "attributes": {"g_id": (5620, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387640.0],
                        [168600.0, 5387670.0],
                        [168630.0, 5387670.0],
                        [168630.0, 5387640.0],
                        [168600.0, 5387640.0],
                    ]
                ],
            },
        },
        773: {
            "attributes": {"g_id": (5620, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387670.0],
                        [168600.0, 5387700.0],
                        [168630.0, 5387700.0],
                        [168630.0, 5387670.0],
                        [168600.0, 5387670.0],
                    ]
                ],
            },
        },
        774: {
            "attributes": {"g_id": (5620, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387700.0],
                        [168600.0, 5387730.0],
                        [168630.0, 5387730.0],
                        [168630.0, 5387700.0],
                        [168600.0, 5387700.0],
                    ]
                ],
            },
        },
        775: {
            "attributes": {"g_id": (5620, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387730.0],
                        [168600.0, 5387760.0],
                        [168630.0, 5387760.0],
                        [168630.0, 5387730.0],
                        [168600.0, 5387730.0],
                    ]
                ],
            },
        },
        776: {
            "attributes": {"g_id": (5620, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387760.0],
                        [168600.0, 5387790.0],
                        [168630.0, 5387790.0],
                        [168630.0, 5387760.0],
                        [168600.0, 5387760.0],
                    ]
                ],
            },
        },
        777: {
            "attributes": {"g_id": (5620, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387790.0],
                        [168600.0, 5387820.0],
                        [168630.0, 5387820.0],
                        [168630.0, 5387790.0],
                        [168600.0, 5387790.0],
                    ]
                ],
            },
        },
        778: {
            "attributes": {"g_id": (5620, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387820.0],
                        [168600.0, 5387850.0],
                        [168630.0, 5387850.0],
                        [168630.0, 5387820.0],
                        [168600.0, 5387820.0],
                    ]
                ],
            },
        },
        779: {
            "attributes": {"g_id": (5620, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387850.0],
                        [168600.0, 5387880.0],
                        [168630.0, 5387880.0],
                        [168630.0, 5387850.0],
                        [168600.0, 5387850.0],
                    ]
                ],
            },
        },
        780: {
            "attributes": {"g_id": (5620, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387880.0],
                        [168600.0, 5387910.0],
                        [168630.0, 5387910.0],
                        [168630.0, 5387880.0],
                        [168600.0, 5387880.0],
                    ]
                ],
            },
        },
        781: {
            "attributes": {"g_id": (5620, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387910.0],
                        [168600.0, 5387940.0],
                        [168630.0, 5387940.0],
                        [168630.0, 5387910.0],
                        [168600.0, 5387910.0],
                    ]
                ],
            },
        },
        782: {
            "attributes": {"g_id": (5620, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387940.0],
                        [168600.0, 5387970.0],
                        [168630.0, 5387970.0],
                        [168630.0, 5387940.0],
                        [168600.0, 5387940.0],
                    ]
                ],
            },
        },
        783: {
            "attributes": {"g_id": (5620, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5387970.0],
                        [168600.0, 5388000.0],
                        [168630.0, 5388000.0],
                        [168630.0, 5387970.0],
                        [168600.0, 5387970.0],
                    ]
                ],
            },
        },
        784: {
            "attributes": {"g_id": (5620, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388000.0],
                        [168600.0, 5388030.0],
                        [168630.0, 5388030.0],
                        [168630.0, 5388000.0],
                        [168600.0, 5388000.0],
                    ]
                ],
            },
        },
        785: {
            "attributes": {"g_id": (5620, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388030.0],
                        [168600.0, 5388060.0],
                        [168630.0, 5388060.0],
                        [168630.0, 5388030.0],
                        [168600.0, 5388030.0],
                    ]
                ],
            },
        },
        786: {
            "attributes": {"g_id": (5620, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388060.0],
                        [168600.0, 5388090.0],
                        [168630.0, 5388090.0],
                        [168630.0, 5388060.0],
                        [168600.0, 5388060.0],
                    ]
                ],
            },
        },
        787: {
            "attributes": {"g_id": (5620, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388090.0],
                        [168600.0, 5388120.0],
                        [168630.0, 5388120.0],
                        [168630.0, 5388090.0],
                        [168600.0, 5388090.0],
                    ]
                ],
            },
        },
        788: {
            "attributes": {"g_id": (5620, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388120.0],
                        [168600.0, 5388150.0],
                        [168630.0, 5388150.0],
                        [168630.0, 5388120.0],
                        [168600.0, 5388120.0],
                    ]
                ],
            },
        },
        789: {
            "attributes": {"g_id": (5620, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388150.0],
                        [168600.0, 5388180.0],
                        [168630.0, 5388180.0],
                        [168630.0, 5388150.0],
                        [168600.0, 5388150.0],
                    ]
                ],
            },
        },
        790: {
            "attributes": {"g_id": (5620, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388180.0],
                        [168600.0, 5388210.0],
                        [168630.0, 5388210.0],
                        [168630.0, 5388180.0],
                        [168600.0, 5388180.0],
                    ]
                ],
            },
        },
        791: {
            "attributes": {"g_id": (5620, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388210.0],
                        [168600.0, 5388240.0],
                        [168630.0, 5388240.0],
                        [168630.0, 5388210.0],
                        [168600.0, 5388210.0],
                    ]
                ],
            },
        },
        792: {
            "attributes": {"g_id": (5620, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388240.0],
                        [168600.0, 5388270.0],
                        [168630.0, 5388270.0],
                        [168630.0, 5388240.0],
                        [168600.0, 5388240.0],
                    ]
                ],
            },
        },
        793: {
            "attributes": {"g_id": (5620, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388270.0],
                        [168600.0, 5388300.0],
                        [168630.0, 5388300.0],
                        [168630.0, 5388270.0],
                        [168600.0, 5388270.0],
                    ]
                ],
            },
        },
        794: {
            "attributes": {"g_id": (5620, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388300.0],
                        [168600.0, 5388330.0],
                        [168630.0, 5388330.0],
                        [168630.0, 5388300.0],
                        [168600.0, 5388300.0],
                    ]
                ],
            },
        },
        795: {
            "attributes": {"g_id": (5620, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388330.0],
                        [168600.0, 5388360.0],
                        [168630.0, 5388360.0],
                        [168630.0, 5388330.0],
                        [168600.0, 5388330.0],
                    ]
                ],
            },
        },
        796: {
            "attributes": {"g_id": (5620, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388360.0],
                        [168600.0, 5388390.0],
                        [168630.0, 5388390.0],
                        [168630.0, 5388360.0],
                        [168600.0, 5388360.0],
                    ]
                ],
            },
        },
        797: {
            "attributes": {"g_id": (5620, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388390.0],
                        [168600.0, 5388420.0],
                        [168630.0, 5388420.0],
                        [168630.0, 5388390.0],
                        [168600.0, 5388390.0],
                    ]
                ],
            },
        },
        798: {
            "attributes": {"g_id": (5620, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388420.0],
                        [168600.0, 5388450.0],
                        [168630.0, 5388450.0],
                        [168630.0, 5388420.0],
                        [168600.0, 5388420.0],
                    ]
                ],
            },
        },
        799: {
            "attributes": {"g_id": (5620, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388450.0],
                        [168600.0, 5388480.0],
                        [168630.0, 5388480.0],
                        [168630.0, 5388450.0],
                        [168600.0, 5388450.0],
                    ]
                ],
            },
        },
        800: {
            "attributes": {"g_id": (5620, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388480.0],
                        [168600.0, 5388510.0],
                        [168630.0, 5388510.0],
                        [168630.0, 5388480.0],
                        [168600.0, 5388480.0],
                    ]
                ],
            },
        },
        801: {
            "attributes": {"g_id": (5620, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388510.0],
                        [168600.0, 5388540.0],
                        [168630.0, 5388540.0],
                        [168630.0, 5388510.0],
                        [168600.0, 5388510.0],
                    ]
                ],
            },
        },
        802: {
            "attributes": {"g_id": (5620, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388540.0],
                        [168600.0, 5388570.0],
                        [168630.0, 5388570.0],
                        [168630.0, 5388540.0],
                        [168600.0, 5388540.0],
                    ]
                ],
            },
        },
        803: {
            "attributes": {"g_id": (5620, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388570.0],
                        [168600.0, 5388600.0],
                        [168630.0, 5388600.0],
                        [168630.0, 5388570.0],
                        [168600.0, 5388570.0],
                    ]
                ],
            },
        },
        804: {
            "attributes": {"g_id": (5620, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388600.0],
                        [168600.0, 5388630.0],
                        [168630.0, 5388630.0],
                        [168630.0, 5388600.0],
                        [168600.0, 5388600.0],
                    ]
                ],
            },
        },
        805: {
            "attributes": {"g_id": (5620, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388630.0],
                        [168600.0, 5388660.0],
                        [168630.0, 5388660.0],
                        [168630.0, 5388630.0],
                        [168600.0, 5388630.0],
                    ]
                ],
            },
        },
        806: {
            "attributes": {"g_id": (5620, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388660.0],
                        [168600.0, 5388690.0],
                        [168630.0, 5388690.0],
                        [168630.0, 5388660.0],
                        [168600.0, 5388660.0],
                    ]
                ],
            },
        },
        807: {
            "attributes": {"g_id": (5620, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388690.0],
                        [168600.0, 5388720.0],
                        [168630.0, 5388720.0],
                        [168630.0, 5388690.0],
                        [168600.0, 5388690.0],
                    ]
                ],
            },
        },
        808: {
            "attributes": {"g_id": (5620, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388720.0],
                        [168600.0, 5388750.0],
                        [168630.0, 5388750.0],
                        [168630.0, 5388720.0],
                        [168600.0, 5388720.0],
                    ]
                ],
            },
        },
        809: {
            "attributes": {"g_id": (5620, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388750.0],
                        [168600.0, 5388780.0],
                        [168630.0, 5388780.0],
                        [168630.0, 5388750.0],
                        [168600.0, 5388750.0],
                    ]
                ],
            },
        },
        810: {
            "attributes": {"g_id": (5620, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388780.0],
                        [168600.0, 5388810.0],
                        [168630.0, 5388810.0],
                        [168630.0, 5388780.0],
                        [168600.0, 5388780.0],
                    ]
                ],
            },
        },
        811: {
            "attributes": {"g_id": (5620, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388810.0],
                        [168600.0, 5388840.0],
                        [168630.0, 5388840.0],
                        [168630.0, 5388810.0],
                        [168600.0, 5388810.0],
                    ]
                ],
            },
        },
        812: {
            "attributes": {"g_id": (5620, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388840.0],
                        [168600.0, 5388870.0],
                        [168630.0, 5388870.0],
                        [168630.0, 5388840.0],
                        [168600.0, 5388840.0],
                    ]
                ],
            },
        },
        813: {
            "attributes": {"g_id": (5620, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388870.0],
                        [168600.0, 5388900.0],
                        [168630.0, 5388900.0],
                        [168630.0, 5388870.0],
                        [168600.0, 5388870.0],
                    ]
                ],
            },
        },
        814: {
            "attributes": {"g_id": (5620, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388900.0],
                        [168600.0, 5388930.0],
                        [168630.0, 5388930.0],
                        [168630.0, 5388900.0],
                        [168600.0, 5388900.0],
                    ]
                ],
            },
        },
        815: {
            "attributes": {"g_id": (5620, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168600.0, 5388930.0],
                        [168600.0, 5388960.0],
                        [168630.0, 5388960.0],
                        [168630.0, 5388930.0],
                        [168600.0, 5388930.0],
                    ]
                ],
            },
        },
        816: {
            "attributes": {"g_id": (5621, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387520.0],
                        [168630.0, 5387550.0],
                        [168660.0, 5387550.0],
                        [168660.0, 5387520.0],
                        [168630.0, 5387520.0],
                    ]
                ],
            },
        },
        817: {
            "attributes": {"g_id": (5621, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387550.0],
                        [168630.0, 5387580.0],
                        [168660.0, 5387580.0],
                        [168660.0, 5387550.0],
                        [168630.0, 5387550.0],
                    ]
                ],
            },
        },
        818: {
            "attributes": {"g_id": (5621, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387580.0],
                        [168630.0, 5387610.0],
                        [168660.0, 5387610.0],
                        [168660.0, 5387580.0],
                        [168630.0, 5387580.0],
                    ]
                ],
            },
        },
        819: {
            "attributes": {"g_id": (5621, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387610.0],
                        [168630.0, 5387640.0],
                        [168660.0, 5387640.0],
                        [168660.0, 5387610.0],
                        [168630.0, 5387610.0],
                    ]
                ],
            },
        },
        820: {
            "attributes": {"g_id": (5621, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387640.0],
                        [168630.0, 5387670.0],
                        [168660.0, 5387670.0],
                        [168660.0, 5387640.0],
                        [168630.0, 5387640.0],
                    ]
                ],
            },
        },
        821: {
            "attributes": {"g_id": (5621, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387670.0],
                        [168630.0, 5387700.0],
                        [168660.0, 5387700.0],
                        [168660.0, 5387670.0],
                        [168630.0, 5387670.0],
                    ]
                ],
            },
        },
        822: {
            "attributes": {"g_id": (5621, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387700.0],
                        [168630.0, 5387730.0],
                        [168660.0, 5387730.0],
                        [168660.0, 5387700.0],
                        [168630.0, 5387700.0],
                    ]
                ],
            },
        },
        823: {
            "attributes": {"g_id": (5621, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387730.0],
                        [168630.0, 5387760.0],
                        [168660.0, 5387760.0],
                        [168660.0, 5387730.0],
                        [168630.0, 5387730.0],
                    ]
                ],
            },
        },
        824: {
            "attributes": {"g_id": (5621, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387760.0],
                        [168630.0, 5387790.0],
                        [168660.0, 5387790.0],
                        [168660.0, 5387760.0],
                        [168630.0, 5387760.0],
                    ]
                ],
            },
        },
        825: {
            "attributes": {"g_id": (5621, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387790.0],
                        [168630.0, 5387820.0],
                        [168660.0, 5387820.0],
                        [168660.0, 5387790.0],
                        [168630.0, 5387790.0],
                    ]
                ],
            },
        },
        826: {
            "attributes": {"g_id": (5621, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387820.0],
                        [168630.0, 5387850.0],
                        [168660.0, 5387850.0],
                        [168660.0, 5387820.0],
                        [168630.0, 5387820.0],
                    ]
                ],
            },
        },
        827: {
            "attributes": {"g_id": (5621, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387850.0],
                        [168630.0, 5387880.0],
                        [168660.0, 5387880.0],
                        [168660.0, 5387850.0],
                        [168630.0, 5387850.0],
                    ]
                ],
            },
        },
        828: {
            "attributes": {"g_id": (5621, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387880.0],
                        [168630.0, 5387910.0],
                        [168660.0, 5387910.0],
                        [168660.0, 5387880.0],
                        [168630.0, 5387880.0],
                    ]
                ],
            },
        },
        829: {
            "attributes": {"g_id": (5621, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387910.0],
                        [168630.0, 5387940.0],
                        [168660.0, 5387940.0],
                        [168660.0, 5387910.0],
                        [168630.0, 5387910.0],
                    ]
                ],
            },
        },
        830: {
            "attributes": {"g_id": (5621, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387940.0],
                        [168630.0, 5387970.0],
                        [168660.0, 5387970.0],
                        [168660.0, 5387940.0],
                        [168630.0, 5387940.0],
                    ]
                ],
            },
        },
        831: {
            "attributes": {"g_id": (5621, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5387970.0],
                        [168630.0, 5388000.0],
                        [168660.0, 5388000.0],
                        [168660.0, 5387970.0],
                        [168630.0, 5387970.0],
                    ]
                ],
            },
        },
        832: {
            "attributes": {"g_id": (5621, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388000.0],
                        [168630.0, 5388030.0],
                        [168660.0, 5388030.0],
                        [168660.0, 5388000.0],
                        [168630.0, 5388000.0],
                    ]
                ],
            },
        },
        833: {
            "attributes": {"g_id": (5621, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388030.0],
                        [168630.0, 5388060.0],
                        [168660.0, 5388060.0],
                        [168660.0, 5388030.0],
                        [168630.0, 5388030.0],
                    ]
                ],
            },
        },
        834: {
            "attributes": {"g_id": (5621, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388060.0],
                        [168630.0, 5388090.0],
                        [168660.0, 5388090.0],
                        [168660.0, 5388060.0],
                        [168630.0, 5388060.0],
                    ]
                ],
            },
        },
        835: {
            "attributes": {"g_id": (5621, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388090.0],
                        [168630.0, 5388120.0],
                        [168660.0, 5388120.0],
                        [168660.0, 5388090.0],
                        [168630.0, 5388090.0],
                    ]
                ],
            },
        },
        836: {
            "attributes": {"g_id": (5621, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388120.0],
                        [168630.0, 5388150.0],
                        [168660.0, 5388150.0],
                        [168660.0, 5388120.0],
                        [168630.0, 5388120.0],
                    ]
                ],
            },
        },
        837: {
            "attributes": {"g_id": (5621, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388150.0],
                        [168630.0, 5388180.0],
                        [168660.0, 5388180.0],
                        [168660.0, 5388150.0],
                        [168630.0, 5388150.0],
                    ]
                ],
            },
        },
        838: {
            "attributes": {"g_id": (5621, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388180.0],
                        [168630.0, 5388210.0],
                        [168660.0, 5388210.0],
                        [168660.0, 5388180.0],
                        [168630.0, 5388180.0],
                    ]
                ],
            },
        },
        839: {
            "attributes": {"g_id": (5621, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388210.0],
                        [168630.0, 5388240.0],
                        [168660.0, 5388240.0],
                        [168660.0, 5388210.0],
                        [168630.0, 5388210.0],
                    ]
                ],
            },
        },
        840: {
            "attributes": {"g_id": (5621, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388240.0],
                        [168630.0, 5388270.0],
                        [168660.0, 5388270.0],
                        [168660.0, 5388240.0],
                        [168630.0, 5388240.0],
                    ]
                ],
            },
        },
        841: {
            "attributes": {"g_id": (5621, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388270.0],
                        [168630.0, 5388300.0],
                        [168660.0, 5388300.0],
                        [168660.0, 5388270.0],
                        [168630.0, 5388270.0],
                    ]
                ],
            },
        },
        842: {
            "attributes": {"g_id": (5621, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388300.0],
                        [168630.0, 5388330.0],
                        [168660.0, 5388330.0],
                        [168660.0, 5388300.0],
                        [168630.0, 5388300.0],
                    ]
                ],
            },
        },
        843: {
            "attributes": {"g_id": (5621, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388330.0],
                        [168630.0, 5388360.0],
                        [168660.0, 5388360.0],
                        [168660.0, 5388330.0],
                        [168630.0, 5388330.0],
                    ]
                ],
            },
        },
        844: {
            "attributes": {"g_id": (5621, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388360.0],
                        [168630.0, 5388390.0],
                        [168660.0, 5388390.0],
                        [168660.0, 5388360.0],
                        [168630.0, 5388360.0],
                    ]
                ],
            },
        },
        845: {
            "attributes": {"g_id": (5621, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388390.0],
                        [168630.0, 5388420.0],
                        [168660.0, 5388420.0],
                        [168660.0, 5388390.0],
                        [168630.0, 5388390.0],
                    ]
                ],
            },
        },
        846: {
            "attributes": {"g_id": (5621, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388420.0],
                        [168630.0, 5388450.0],
                        [168660.0, 5388450.0],
                        [168660.0, 5388420.0],
                        [168630.0, 5388420.0],
                    ]
                ],
            },
        },
        847: {
            "attributes": {"g_id": (5621, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388450.0],
                        [168630.0, 5388480.0],
                        [168660.0, 5388480.0],
                        [168660.0, 5388450.0],
                        [168630.0, 5388450.0],
                    ]
                ],
            },
        },
        848: {
            "attributes": {"g_id": (5621, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388480.0],
                        [168630.0, 5388510.0],
                        [168660.0, 5388510.0],
                        [168660.0, 5388480.0],
                        [168630.0, 5388480.0],
                    ]
                ],
            },
        },
        849: {
            "attributes": {"g_id": (5621, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388510.0],
                        [168630.0, 5388540.0],
                        [168660.0, 5388540.0],
                        [168660.0, 5388510.0],
                        [168630.0, 5388510.0],
                    ]
                ],
            },
        },
        850: {
            "attributes": {"g_id": (5621, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388540.0],
                        [168630.0, 5388570.0],
                        [168660.0, 5388570.0],
                        [168660.0, 5388540.0],
                        [168630.0, 5388540.0],
                    ]
                ],
            },
        },
        851: {
            "attributes": {"g_id": (5621, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388570.0],
                        [168630.0, 5388600.0],
                        [168660.0, 5388600.0],
                        [168660.0, 5388570.0],
                        [168630.0, 5388570.0],
                    ]
                ],
            },
        },
        852: {
            "attributes": {"g_id": (5621, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388600.0],
                        [168630.0, 5388630.0],
                        [168660.0, 5388630.0],
                        [168660.0, 5388600.0],
                        [168630.0, 5388600.0],
                    ]
                ],
            },
        },
        853: {
            "attributes": {"g_id": (5621, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388630.0],
                        [168630.0, 5388660.0],
                        [168660.0, 5388660.0],
                        [168660.0, 5388630.0],
                        [168630.0, 5388630.0],
                    ]
                ],
            },
        },
        854: {
            "attributes": {"g_id": (5621, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388660.0],
                        [168630.0, 5388690.0],
                        [168660.0, 5388690.0],
                        [168660.0, 5388660.0],
                        [168630.0, 5388660.0],
                    ]
                ],
            },
        },
        855: {
            "attributes": {"g_id": (5621, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388690.0],
                        [168630.0, 5388720.0],
                        [168660.0, 5388720.0],
                        [168660.0, 5388690.0],
                        [168630.0, 5388690.0],
                    ]
                ],
            },
        },
        856: {
            "attributes": {"g_id": (5621, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388720.0],
                        [168630.0, 5388750.0],
                        [168660.0, 5388750.0],
                        [168660.0, 5388720.0],
                        [168630.0, 5388720.0],
                    ]
                ],
            },
        },
        857: {
            "attributes": {"g_id": (5621, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388750.0],
                        [168630.0, 5388780.0],
                        [168660.0, 5388780.0],
                        [168660.0, 5388750.0],
                        [168630.0, 5388750.0],
                    ]
                ],
            },
        },
        858: {
            "attributes": {"g_id": (5621, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388780.0],
                        [168630.0, 5388810.0],
                        [168660.0, 5388810.0],
                        [168660.0, 5388780.0],
                        [168630.0, 5388780.0],
                    ]
                ],
            },
        },
        859: {
            "attributes": {"g_id": (5621, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388810.0],
                        [168630.0, 5388840.0],
                        [168660.0, 5388840.0],
                        [168660.0, 5388810.0],
                        [168630.0, 5388810.0],
                    ]
                ],
            },
        },
        860: {
            "attributes": {"g_id": (5621, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388840.0],
                        [168630.0, 5388870.0],
                        [168660.0, 5388870.0],
                        [168660.0, 5388840.0],
                        [168630.0, 5388840.0],
                    ]
                ],
            },
        },
        861: {
            "attributes": {"g_id": (5621, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388870.0],
                        [168630.0, 5388900.0],
                        [168660.0, 5388900.0],
                        [168660.0, 5388870.0],
                        [168630.0, 5388870.0],
                    ]
                ],
            },
        },
        862: {
            "attributes": {"g_id": (5621, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388900.0],
                        [168630.0, 5388930.0],
                        [168660.0, 5388930.0],
                        [168660.0, 5388900.0],
                        [168630.0, 5388900.0],
                    ]
                ],
            },
        },
        863: {
            "attributes": {"g_id": (5621, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168630.0, 5388930.0],
                        [168630.0, 5388960.0],
                        [168660.0, 5388960.0],
                        [168660.0, 5388930.0],
                        [168630.0, 5388930.0],
                    ]
                ],
            },
        },
        864: {
            "attributes": {"g_id": (5622, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387520.0],
                        [168660.0, 5387550.0],
                        [168690.0, 5387550.0],
                        [168690.0, 5387520.0],
                        [168660.0, 5387520.0],
                    ]
                ],
            },
        },
        865: {
            "attributes": {"g_id": (5622, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387550.0],
                        [168660.0, 5387580.0],
                        [168690.0, 5387580.0],
                        [168690.0, 5387550.0],
                        [168660.0, 5387550.0],
                    ]
                ],
            },
        },
        866: {
            "attributes": {"g_id": (5622, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387580.0],
                        [168660.0, 5387610.0],
                        [168690.0, 5387610.0],
                        [168690.0, 5387580.0],
                        [168660.0, 5387580.0],
                    ]
                ],
            },
        },
        867: {
            "attributes": {"g_id": (5622, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387610.0],
                        [168660.0, 5387640.0],
                        [168690.0, 5387640.0],
                        [168690.0, 5387610.0],
                        [168660.0, 5387610.0],
                    ]
                ],
            },
        },
        868: {
            "attributes": {"g_id": (5622, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387640.0],
                        [168660.0, 5387670.0],
                        [168690.0, 5387670.0],
                        [168690.0, 5387640.0],
                        [168660.0, 5387640.0],
                    ]
                ],
            },
        },
        869: {
            "attributes": {"g_id": (5622, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387670.0],
                        [168660.0, 5387700.0],
                        [168690.0, 5387700.0],
                        [168690.0, 5387670.0],
                        [168660.0, 5387670.0],
                    ]
                ],
            },
        },
        870: {
            "attributes": {"g_id": (5622, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387700.0],
                        [168660.0, 5387730.0],
                        [168690.0, 5387730.0],
                        [168690.0, 5387700.0],
                        [168660.0, 5387700.0],
                    ]
                ],
            },
        },
        871: {
            "attributes": {"g_id": (5622, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387730.0],
                        [168660.0, 5387760.0],
                        [168690.0, 5387760.0],
                        [168690.0, 5387730.0],
                        [168660.0, 5387730.0],
                    ]
                ],
            },
        },
        872: {
            "attributes": {"g_id": (5622, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387760.0],
                        [168660.0, 5387790.0],
                        [168690.0, 5387790.0],
                        [168690.0, 5387760.0],
                        [168660.0, 5387760.0],
                    ]
                ],
            },
        },
        873: {
            "attributes": {"g_id": (5622, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387790.0],
                        [168660.0, 5387820.0],
                        [168690.0, 5387820.0],
                        [168690.0, 5387790.0],
                        [168660.0, 5387790.0],
                    ]
                ],
            },
        },
        874: {
            "attributes": {"g_id": (5622, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387820.0],
                        [168660.0, 5387850.0],
                        [168690.0, 5387850.0],
                        [168690.0, 5387820.0],
                        [168660.0, 5387820.0],
                    ]
                ],
            },
        },
        875: {
            "attributes": {"g_id": (5622, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387850.0],
                        [168660.0, 5387880.0],
                        [168690.0, 5387880.0],
                        [168690.0, 5387850.0],
                        [168660.0, 5387850.0],
                    ]
                ],
            },
        },
        876: {
            "attributes": {"g_id": (5622, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387880.0],
                        [168660.0, 5387910.0],
                        [168690.0, 5387910.0],
                        [168690.0, 5387880.0],
                        [168660.0, 5387880.0],
                    ]
                ],
            },
        },
        877: {
            "attributes": {"g_id": (5622, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387910.0],
                        [168660.0, 5387940.0],
                        [168690.0, 5387940.0],
                        [168690.0, 5387910.0],
                        [168660.0, 5387910.0],
                    ]
                ],
            },
        },
        878: {
            "attributes": {"g_id": (5622, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387940.0],
                        [168660.0, 5387970.0],
                        [168690.0, 5387970.0],
                        [168690.0, 5387940.0],
                        [168660.0, 5387940.0],
                    ]
                ],
            },
        },
        879: {
            "attributes": {"g_id": (5622, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5387970.0],
                        [168660.0, 5388000.0],
                        [168690.0, 5388000.0],
                        [168690.0, 5387970.0],
                        [168660.0, 5387970.0],
                    ]
                ],
            },
        },
        880: {
            "attributes": {"g_id": (5622, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388000.0],
                        [168660.0, 5388030.0],
                        [168690.0, 5388030.0],
                        [168690.0, 5388000.0],
                        [168660.0, 5388000.0],
                    ]
                ],
            },
        },
        881: {
            "attributes": {"g_id": (5622, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388030.0],
                        [168660.0, 5388060.0],
                        [168690.0, 5388060.0],
                        [168690.0, 5388030.0],
                        [168660.0, 5388030.0],
                    ]
                ],
            },
        },
        882: {
            "attributes": {"g_id": (5622, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388060.0],
                        [168660.0, 5388090.0],
                        [168690.0, 5388090.0],
                        [168690.0, 5388060.0],
                        [168660.0, 5388060.0],
                    ]
                ],
            },
        },
        883: {
            "attributes": {"g_id": (5622, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388090.0],
                        [168660.0, 5388120.0],
                        [168690.0, 5388120.0],
                        [168690.0, 5388090.0],
                        [168660.0, 5388090.0],
                    ]
                ],
            },
        },
        884: {
            "attributes": {"g_id": (5622, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388120.0],
                        [168660.0, 5388150.0],
                        [168690.0, 5388150.0],
                        [168690.0, 5388120.0],
                        [168660.0, 5388120.0],
                    ]
                ],
            },
        },
        885: {
            "attributes": {"g_id": (5622, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388150.0],
                        [168660.0, 5388180.0],
                        [168690.0, 5388180.0],
                        [168690.0, 5388150.0],
                        [168660.0, 5388150.0],
                    ]
                ],
            },
        },
        886: {
            "attributes": {"g_id": (5622, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388180.0],
                        [168660.0, 5388210.0],
                        [168690.0, 5388210.0],
                        [168690.0, 5388180.0],
                        [168660.0, 5388180.0],
                    ]
                ],
            },
        },
        887: {
            "attributes": {"g_id": (5622, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388210.0],
                        [168660.0, 5388240.0],
                        [168690.0, 5388240.0],
                        [168690.0, 5388210.0],
                        [168660.0, 5388210.0],
                    ]
                ],
            },
        },
        888: {
            "attributes": {"g_id": (5622, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388240.0],
                        [168660.0, 5388270.0],
                        [168690.0, 5388270.0],
                        [168690.0, 5388240.0],
                        [168660.0, 5388240.0],
                    ]
                ],
            },
        },
        889: {
            "attributes": {"g_id": (5622, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388270.0],
                        [168660.0, 5388300.0],
                        [168690.0, 5388300.0],
                        [168690.0, 5388270.0],
                        [168660.0, 5388270.0],
                    ]
                ],
            },
        },
        890: {
            "attributes": {"g_id": (5622, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388300.0],
                        [168660.0, 5388330.0],
                        [168690.0, 5388330.0],
                        [168690.0, 5388300.0],
                        [168660.0, 5388300.0],
                    ]
                ],
            },
        },
        891: {
            "attributes": {"g_id": (5622, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388330.0],
                        [168660.0, 5388360.0],
                        [168690.0, 5388360.0],
                        [168690.0, 5388330.0],
                        [168660.0, 5388330.0],
                    ]
                ],
            },
        },
        892: {
            "attributes": {"g_id": (5622, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388360.0],
                        [168660.0, 5388390.0],
                        [168690.0, 5388390.0],
                        [168690.0, 5388360.0],
                        [168660.0, 5388360.0],
                    ]
                ],
            },
        },
        893: {
            "attributes": {"g_id": (5622, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388390.0],
                        [168660.0, 5388420.0],
                        [168690.0, 5388420.0],
                        [168690.0, 5388390.0],
                        [168660.0, 5388390.0],
                    ]
                ],
            },
        },
        894: {
            "attributes": {"g_id": (5622, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388420.0],
                        [168660.0, 5388450.0],
                        [168690.0, 5388450.0],
                        [168690.0, 5388420.0],
                        [168660.0, 5388420.0],
                    ]
                ],
            },
        },
        895: {
            "attributes": {"g_id": (5622, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388450.0],
                        [168660.0, 5388480.0],
                        [168690.0, 5388480.0],
                        [168690.0, 5388450.0],
                        [168660.0, 5388450.0],
                    ]
                ],
            },
        },
        896: {
            "attributes": {"g_id": (5622, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388480.0],
                        [168660.0, 5388510.0],
                        [168690.0, 5388510.0],
                        [168690.0, 5388480.0],
                        [168660.0, 5388480.0],
                    ]
                ],
            },
        },
        897: {
            "attributes": {"g_id": (5622, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388510.0],
                        [168660.0, 5388540.0],
                        [168690.0, 5388540.0],
                        [168690.0, 5388510.0],
                        [168660.0, 5388510.0],
                    ]
                ],
            },
        },
        898: {
            "attributes": {"g_id": (5622, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388540.0],
                        [168660.0, 5388570.0],
                        [168690.0, 5388570.0],
                        [168690.0, 5388540.0],
                        [168660.0, 5388540.0],
                    ]
                ],
            },
        },
        899: {
            "attributes": {"g_id": (5622, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388570.0],
                        [168660.0, 5388600.0],
                        [168690.0, 5388600.0],
                        [168690.0, 5388570.0],
                        [168660.0, 5388570.0],
                    ]
                ],
            },
        },
        900: {
            "attributes": {"g_id": (5622, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388600.0],
                        [168660.0, 5388630.0],
                        [168690.0, 5388630.0],
                        [168690.0, 5388600.0],
                        [168660.0, 5388600.0],
                    ]
                ],
            },
        },
        901: {
            "attributes": {"g_id": (5622, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388630.0],
                        [168660.0, 5388660.0],
                        [168690.0, 5388660.0],
                        [168690.0, 5388630.0],
                        [168660.0, 5388630.0],
                    ]
                ],
            },
        },
        902: {
            "attributes": {"g_id": (5622, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388660.0],
                        [168660.0, 5388690.0],
                        [168690.0, 5388690.0],
                        [168690.0, 5388660.0],
                        [168660.0, 5388660.0],
                    ]
                ],
            },
        },
        903: {
            "attributes": {"g_id": (5622, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388690.0],
                        [168660.0, 5388720.0],
                        [168690.0, 5388720.0],
                        [168690.0, 5388690.0],
                        [168660.0, 5388690.0],
                    ]
                ],
            },
        },
        904: {
            "attributes": {"g_id": (5622, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388720.0],
                        [168660.0, 5388750.0],
                        [168690.0, 5388750.0],
                        [168690.0, 5388720.0],
                        [168660.0, 5388720.0],
                    ]
                ],
            },
        },
        905: {
            "attributes": {"g_id": (5622, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388750.0],
                        [168660.0, 5388780.0],
                        [168690.0, 5388780.0],
                        [168690.0, 5388750.0],
                        [168660.0, 5388750.0],
                    ]
                ],
            },
        },
        906: {
            "attributes": {"g_id": (5622, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388780.0],
                        [168660.0, 5388810.0],
                        [168690.0, 5388810.0],
                        [168690.0, 5388780.0],
                        [168660.0, 5388780.0],
                    ]
                ],
            },
        },
        907: {
            "attributes": {"g_id": (5622, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388810.0],
                        [168660.0, 5388840.0],
                        [168690.0, 5388840.0],
                        [168690.0, 5388810.0],
                        [168660.0, 5388810.0],
                    ]
                ],
            },
        },
        908: {
            "attributes": {"g_id": (5622, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388840.0],
                        [168660.0, 5388870.0],
                        [168690.0, 5388870.0],
                        [168690.0, 5388840.0],
                        [168660.0, 5388840.0],
                    ]
                ],
            },
        },
        909: {
            "attributes": {"g_id": (5622, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388870.0],
                        [168660.0, 5388900.0],
                        [168690.0, 5388900.0],
                        [168690.0, 5388870.0],
                        [168660.0, 5388870.0],
                    ]
                ],
            },
        },
        910: {
            "attributes": {"g_id": (5622, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388900.0],
                        [168660.0, 5388930.0],
                        [168690.0, 5388930.0],
                        [168690.0, 5388900.0],
                        [168660.0, 5388900.0],
                    ]
                ],
            },
        },
        911: {
            "attributes": {"g_id": (5622, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168660.0, 5388930.0],
                        [168660.0, 5388960.0],
                        [168690.0, 5388960.0],
                        [168690.0, 5388930.0],
                        [168660.0, 5388930.0],
                    ]
                ],
            },
        },
        912: {
            "attributes": {"g_id": (5623, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387520.0],
                        [168690.0, 5387550.0],
                        [168720.0, 5387550.0],
                        [168720.0, 5387520.0],
                        [168690.0, 5387520.0],
                    ]
                ],
            },
        },
        913: {
            "attributes": {"g_id": (5623, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387550.0],
                        [168690.0, 5387580.0],
                        [168720.0, 5387580.0],
                        [168720.0, 5387550.0],
                        [168690.0, 5387550.0],
                    ]
                ],
            },
        },
        914: {
            "attributes": {"g_id": (5623, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387580.0],
                        [168690.0, 5387610.0],
                        [168720.0, 5387610.0],
                        [168720.0, 5387580.0],
                        [168690.0, 5387580.0],
                    ]
                ],
            },
        },
        915: {
            "attributes": {"g_id": (5623, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387610.0],
                        [168690.0, 5387640.0],
                        [168720.0, 5387640.0],
                        [168720.0, 5387610.0],
                        [168690.0, 5387610.0],
                    ]
                ],
            },
        },
        916: {
            "attributes": {"g_id": (5623, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387640.0],
                        [168690.0, 5387670.0],
                        [168720.0, 5387670.0],
                        [168720.0, 5387640.0],
                        [168690.0, 5387640.0],
                    ]
                ],
            },
        },
        917: {
            "attributes": {"g_id": (5623, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387670.0],
                        [168690.0, 5387700.0],
                        [168720.0, 5387700.0],
                        [168720.0, 5387670.0],
                        [168690.0, 5387670.0],
                    ]
                ],
            },
        },
        918: {
            "attributes": {"g_id": (5623, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387700.0],
                        [168690.0, 5387730.0],
                        [168720.0, 5387730.0],
                        [168720.0, 5387700.0],
                        [168690.0, 5387700.0],
                    ]
                ],
            },
        },
        919: {
            "attributes": {"g_id": (5623, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387730.0],
                        [168690.0, 5387760.0],
                        [168720.0, 5387760.0],
                        [168720.0, 5387730.0],
                        [168690.0, 5387730.0],
                    ]
                ],
            },
        },
        920: {
            "attributes": {"g_id": (5623, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387760.0],
                        [168690.0, 5387790.0],
                        [168720.0, 5387790.0],
                        [168720.0, 5387760.0],
                        [168690.0, 5387760.0],
                    ]
                ],
            },
        },
        921: {
            "attributes": {"g_id": (5623, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387790.0],
                        [168690.0, 5387820.0],
                        [168720.0, 5387820.0],
                        [168720.0, 5387790.0],
                        [168690.0, 5387790.0],
                    ]
                ],
            },
        },
        922: {
            "attributes": {"g_id": (5623, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387820.0],
                        [168690.0, 5387850.0],
                        [168720.0, 5387850.0],
                        [168720.0, 5387820.0],
                        [168690.0, 5387820.0],
                    ]
                ],
            },
        },
        923: {
            "attributes": {"g_id": (5623, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387850.0],
                        [168690.0, 5387880.0],
                        [168720.0, 5387880.0],
                        [168720.0, 5387850.0],
                        [168690.0, 5387850.0],
                    ]
                ],
            },
        },
        924: {
            "attributes": {"g_id": (5623, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387880.0],
                        [168690.0, 5387910.0],
                        [168720.0, 5387910.0],
                        [168720.0, 5387880.0],
                        [168690.0, 5387880.0],
                    ]
                ],
            },
        },
        925: {
            "attributes": {"g_id": (5623, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387910.0],
                        [168690.0, 5387940.0],
                        [168720.0, 5387940.0],
                        [168720.0, 5387910.0],
                        [168690.0, 5387910.0],
                    ]
                ],
            },
        },
        926: {
            "attributes": {"g_id": (5623, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387940.0],
                        [168690.0, 5387970.0],
                        [168720.0, 5387970.0],
                        [168720.0, 5387940.0],
                        [168690.0, 5387940.0],
                    ]
                ],
            },
        },
        927: {
            "attributes": {"g_id": (5623, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5387970.0],
                        [168690.0, 5388000.0],
                        [168720.0, 5388000.0],
                        [168720.0, 5387970.0],
                        [168690.0, 5387970.0],
                    ]
                ],
            },
        },
        928: {
            "attributes": {"g_id": (5623, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388000.0],
                        [168690.0, 5388030.0],
                        [168720.0, 5388030.0],
                        [168720.0, 5388000.0],
                        [168690.0, 5388000.0],
                    ]
                ],
            },
        },
        929: {
            "attributes": {"g_id": (5623, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388030.0],
                        [168690.0, 5388060.0],
                        [168720.0, 5388060.0],
                        [168720.0, 5388030.0],
                        [168690.0, 5388030.0],
                    ]
                ],
            },
        },
        930: {
            "attributes": {"g_id": (5623, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388060.0],
                        [168690.0, 5388090.0],
                        [168720.0, 5388090.0],
                        [168720.0, 5388060.0],
                        [168690.0, 5388060.0],
                    ]
                ],
            },
        },
        931: {
            "attributes": {"g_id": (5623, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388090.0],
                        [168690.0, 5388120.0],
                        [168720.0, 5388120.0],
                        [168720.0, 5388090.0],
                        [168690.0, 5388090.0],
                    ]
                ],
            },
        },
        932: {
            "attributes": {"g_id": (5623, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388120.0],
                        [168690.0, 5388150.0],
                        [168720.0, 5388150.0],
                        [168720.0, 5388120.0],
                        [168690.0, 5388120.0],
                    ]
                ],
            },
        },
        933: {
            "attributes": {"g_id": (5623, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388150.0],
                        [168690.0, 5388180.0],
                        [168720.0, 5388180.0],
                        [168720.0, 5388150.0],
                        [168690.0, 5388150.0],
                    ]
                ],
            },
        },
        934: {
            "attributes": {"g_id": (5623, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388180.0],
                        [168690.0, 5388210.0],
                        [168720.0, 5388210.0],
                        [168720.0, 5388180.0],
                        [168690.0, 5388180.0],
                    ]
                ],
            },
        },
        935: {
            "attributes": {"g_id": (5623, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388210.0],
                        [168690.0, 5388240.0],
                        [168720.0, 5388240.0],
                        [168720.0, 5388210.0],
                        [168690.0, 5388210.0],
                    ]
                ],
            },
        },
        936: {
            "attributes": {"g_id": (5623, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388240.0],
                        [168690.0, 5388270.0],
                        [168720.0, 5388270.0],
                        [168720.0, 5388240.0],
                        [168690.0, 5388240.0],
                    ]
                ],
            },
        },
        937: {
            "attributes": {"g_id": (5623, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388270.0],
                        [168690.0, 5388300.0],
                        [168720.0, 5388300.0],
                        [168720.0, 5388270.0],
                        [168690.0, 5388270.0],
                    ]
                ],
            },
        },
        938: {
            "attributes": {"g_id": (5623, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388300.0],
                        [168690.0, 5388330.0],
                        [168720.0, 5388330.0],
                        [168720.0, 5388300.0],
                        [168690.0, 5388300.0],
                    ]
                ],
            },
        },
        939: {
            "attributes": {"g_id": (5623, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388330.0],
                        [168690.0, 5388360.0],
                        [168720.0, 5388360.0],
                        [168720.0, 5388330.0],
                        [168690.0, 5388330.0],
                    ]
                ],
            },
        },
        940: {
            "attributes": {"g_id": (5623, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388360.0],
                        [168690.0, 5388390.0],
                        [168720.0, 5388390.0],
                        [168720.0, 5388360.0],
                        [168690.0, 5388360.0],
                    ]
                ],
            },
        },
        941: {
            "attributes": {"g_id": (5623, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388390.0],
                        [168690.0, 5388420.0],
                        [168720.0, 5388420.0],
                        [168720.0, 5388390.0],
                        [168690.0, 5388390.0],
                    ]
                ],
            },
        },
        942: {
            "attributes": {"g_id": (5623, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388420.0],
                        [168690.0, 5388450.0],
                        [168720.0, 5388450.0],
                        [168720.0, 5388420.0],
                        [168690.0, 5388420.0],
                    ]
                ],
            },
        },
        943: {
            "attributes": {"g_id": (5623, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388450.0],
                        [168690.0, 5388480.0],
                        [168720.0, 5388480.0],
                        [168720.0, 5388450.0],
                        [168690.0, 5388450.0],
                    ]
                ],
            },
        },
        944: {
            "attributes": {"g_id": (5623, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388480.0],
                        [168690.0, 5388510.0],
                        [168720.0, 5388510.0],
                        [168720.0, 5388480.0],
                        [168690.0, 5388480.0],
                    ]
                ],
            },
        },
        945: {
            "attributes": {"g_id": (5623, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388510.0],
                        [168690.0, 5388540.0],
                        [168720.0, 5388540.0],
                        [168720.0, 5388510.0],
                        [168690.0, 5388510.0],
                    ]
                ],
            },
        },
        946: {
            "attributes": {"g_id": (5623, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388540.0],
                        [168690.0, 5388570.0],
                        [168720.0, 5388570.0],
                        [168720.0, 5388540.0],
                        [168690.0, 5388540.0],
                    ]
                ],
            },
        },
        947: {
            "attributes": {"g_id": (5623, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388570.0],
                        [168690.0, 5388600.0],
                        [168720.0, 5388600.0],
                        [168720.0, 5388570.0],
                        [168690.0, 5388570.0],
                    ]
                ],
            },
        },
        948: {
            "attributes": {"g_id": (5623, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388600.0],
                        [168690.0, 5388630.0],
                        [168720.0, 5388630.0],
                        [168720.0, 5388600.0],
                        [168690.0, 5388600.0],
                    ]
                ],
            },
        },
        949: {
            "attributes": {"g_id": (5623, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388630.0],
                        [168690.0, 5388660.0],
                        [168720.0, 5388660.0],
                        [168720.0, 5388630.0],
                        [168690.0, 5388630.0],
                    ]
                ],
            },
        },
        950: {
            "attributes": {"g_id": (5623, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388660.0],
                        [168690.0, 5388690.0],
                        [168720.0, 5388690.0],
                        [168720.0, 5388660.0],
                        [168690.0, 5388660.0],
                    ]
                ],
            },
        },
        951: {
            "attributes": {"g_id": (5623, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388690.0],
                        [168690.0, 5388720.0],
                        [168720.0, 5388720.0],
                        [168720.0, 5388690.0],
                        [168690.0, 5388690.0],
                    ]
                ],
            },
        },
        952: {
            "attributes": {"g_id": (5623, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388720.0],
                        [168690.0, 5388750.0],
                        [168720.0, 5388750.0],
                        [168720.0, 5388720.0],
                        [168690.0, 5388720.0],
                    ]
                ],
            },
        },
        953: {
            "attributes": {"g_id": (5623, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388750.0],
                        [168690.0, 5388780.0],
                        [168720.0, 5388780.0],
                        [168720.0, 5388750.0],
                        [168690.0, 5388750.0],
                    ]
                ],
            },
        },
        954: {
            "attributes": {"g_id": (5623, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388780.0],
                        [168690.0, 5388810.0],
                        [168720.0, 5388810.0],
                        [168720.0, 5388780.0],
                        [168690.0, 5388780.0],
                    ]
                ],
            },
        },
        955: {
            "attributes": {"g_id": (5623, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388810.0],
                        [168690.0, 5388840.0],
                        [168720.0, 5388840.0],
                        [168720.0, 5388810.0],
                        [168690.0, 5388810.0],
                    ]
                ],
            },
        },
        956: {
            "attributes": {"g_id": (5623, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388840.0],
                        [168690.0, 5388870.0],
                        [168720.0, 5388870.0],
                        [168720.0, 5388840.0],
                        [168690.0, 5388840.0],
                    ]
                ],
            },
        },
        957: {
            "attributes": {"g_id": (5623, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388870.0],
                        [168690.0, 5388900.0],
                        [168720.0, 5388900.0],
                        [168720.0, 5388870.0],
                        [168690.0, 5388870.0],
                    ]
                ],
            },
        },
        958: {
            "attributes": {"g_id": (5623, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388900.0],
                        [168690.0, 5388930.0],
                        [168720.0, 5388930.0],
                        [168720.0, 5388900.0],
                        [168690.0, 5388900.0],
                    ]
                ],
            },
        },
        959: {
            "attributes": {"g_id": (5623, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168690.0, 5388930.0],
                        [168690.0, 5388960.0],
                        [168720.0, 5388960.0],
                        [168720.0, 5388930.0],
                        [168690.0, 5388930.0],
                    ]
                ],
            },
        },
        960: {
            "attributes": {"g_id": (5624, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387520.0],
                        [168720.0, 5387550.0],
                        [168750.0, 5387550.0],
                        [168750.0, 5387520.0],
                        [168720.0, 5387520.0],
                    ]
                ],
            },
        },
        961: {
            "attributes": {"g_id": (5624, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387550.0],
                        [168720.0, 5387580.0],
                        [168750.0, 5387580.0],
                        [168750.0, 5387550.0],
                        [168720.0, 5387550.0],
                    ]
                ],
            },
        },
        962: {
            "attributes": {"g_id": (5624, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387580.0],
                        [168720.0, 5387610.0],
                        [168750.0, 5387610.0],
                        [168750.0, 5387580.0],
                        [168720.0, 5387580.0],
                    ]
                ],
            },
        },
        963: {
            "attributes": {"g_id": (5624, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387610.0],
                        [168720.0, 5387640.0],
                        [168750.0, 5387640.0],
                        [168750.0, 5387610.0],
                        [168720.0, 5387610.0],
                    ]
                ],
            },
        },
        964: {
            "attributes": {"g_id": (5624, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387640.0],
                        [168720.0, 5387670.0],
                        [168750.0, 5387670.0],
                        [168750.0, 5387640.0],
                        [168720.0, 5387640.0],
                    ]
                ],
            },
        },
        965: {
            "attributes": {"g_id": (5624, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387670.0],
                        [168720.0, 5387700.0],
                        [168750.0, 5387700.0],
                        [168750.0, 5387670.0],
                        [168720.0, 5387670.0],
                    ]
                ],
            },
        },
        966: {
            "attributes": {"g_id": (5624, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387700.0],
                        [168720.0, 5387730.0],
                        [168750.0, 5387730.0],
                        [168750.0, 5387700.0],
                        [168720.0, 5387700.0],
                    ]
                ],
            },
        },
        967: {
            "attributes": {"g_id": (5624, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387730.0],
                        [168720.0, 5387760.0],
                        [168750.0, 5387760.0],
                        [168750.0, 5387730.0],
                        [168720.0, 5387730.0],
                    ]
                ],
            },
        },
        968: {
            "attributes": {"g_id": (5624, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387760.0],
                        [168720.0, 5387790.0],
                        [168750.0, 5387790.0],
                        [168750.0, 5387760.0],
                        [168720.0, 5387760.0],
                    ]
                ],
            },
        },
        969: {
            "attributes": {"g_id": (5624, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387790.0],
                        [168720.0, 5387820.0],
                        [168750.0, 5387820.0],
                        [168750.0, 5387790.0],
                        [168720.0, 5387790.0],
                    ]
                ],
            },
        },
        970: {
            "attributes": {"g_id": (5624, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387820.0],
                        [168720.0, 5387850.0],
                        [168750.0, 5387850.0],
                        [168750.0, 5387820.0],
                        [168720.0, 5387820.0],
                    ]
                ],
            },
        },
        971: {
            "attributes": {"g_id": (5624, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387850.0],
                        [168720.0, 5387880.0],
                        [168750.0, 5387880.0],
                        [168750.0, 5387850.0],
                        [168720.0, 5387850.0],
                    ]
                ],
            },
        },
        972: {
            "attributes": {"g_id": (5624, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387880.0],
                        [168720.0, 5387910.0],
                        [168750.0, 5387910.0],
                        [168750.0, 5387880.0],
                        [168720.0, 5387880.0],
                    ]
                ],
            },
        },
        973: {
            "attributes": {"g_id": (5624, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387910.0],
                        [168720.0, 5387940.0],
                        [168750.0, 5387940.0],
                        [168750.0, 5387910.0],
                        [168720.0, 5387910.0],
                    ]
                ],
            },
        },
        974: {
            "attributes": {"g_id": (5624, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387940.0],
                        [168720.0, 5387970.0],
                        [168750.0, 5387970.0],
                        [168750.0, 5387940.0],
                        [168720.0, 5387940.0],
                    ]
                ],
            },
        },
        975: {
            "attributes": {"g_id": (5624, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5387970.0],
                        [168720.0, 5388000.0],
                        [168750.0, 5388000.0],
                        [168750.0, 5387970.0],
                        [168720.0, 5387970.0],
                    ]
                ],
            },
        },
        976: {
            "attributes": {"g_id": (5624, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388000.0],
                        [168720.0, 5388030.0],
                        [168750.0, 5388030.0],
                        [168750.0, 5388000.0],
                        [168720.0, 5388000.0],
                    ]
                ],
            },
        },
        977: {
            "attributes": {"g_id": (5624, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388030.0],
                        [168720.0, 5388060.0],
                        [168750.0, 5388060.0],
                        [168750.0, 5388030.0],
                        [168720.0, 5388030.0],
                    ]
                ],
            },
        },
        978: {
            "attributes": {"g_id": (5624, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388060.0],
                        [168720.0, 5388090.0],
                        [168750.0, 5388090.0],
                        [168750.0, 5388060.0],
                        [168720.0, 5388060.0],
                    ]
                ],
            },
        },
        979: {
            "attributes": {"g_id": (5624, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388090.0],
                        [168720.0, 5388120.0],
                        [168750.0, 5388120.0],
                        [168750.0, 5388090.0],
                        [168720.0, 5388090.0],
                    ]
                ],
            },
        },
        980: {
            "attributes": {"g_id": (5624, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388120.0],
                        [168720.0, 5388150.0],
                        [168750.0, 5388150.0],
                        [168750.0, 5388120.0],
                        [168720.0, 5388120.0],
                    ]
                ],
            },
        },
        981: {
            "attributes": {"g_id": (5624, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388150.0],
                        [168720.0, 5388180.0],
                        [168750.0, 5388180.0],
                        [168750.0, 5388150.0],
                        [168720.0, 5388150.0],
                    ]
                ],
            },
        },
        982: {
            "attributes": {"g_id": (5624, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388180.0],
                        [168720.0, 5388210.0],
                        [168750.0, 5388210.0],
                        [168750.0, 5388180.0],
                        [168720.0, 5388180.0],
                    ]
                ],
            },
        },
        983: {
            "attributes": {"g_id": (5624, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388210.0],
                        [168720.0, 5388240.0],
                        [168750.0, 5388240.0],
                        [168750.0, 5388210.0],
                        [168720.0, 5388210.0],
                    ]
                ],
            },
        },
        984: {
            "attributes": {"g_id": (5624, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388240.0],
                        [168720.0, 5388270.0],
                        [168750.0, 5388270.0],
                        [168750.0, 5388240.0],
                        [168720.0, 5388240.0],
                    ]
                ],
            },
        },
        985: {
            "attributes": {"g_id": (5624, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388270.0],
                        [168720.0, 5388300.0],
                        [168750.0, 5388300.0],
                        [168750.0, 5388270.0],
                        [168720.0, 5388270.0],
                    ]
                ],
            },
        },
        986: {
            "attributes": {"g_id": (5624, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388300.0],
                        [168720.0, 5388330.0],
                        [168750.0, 5388330.0],
                        [168750.0, 5388300.0],
                        [168720.0, 5388300.0],
                    ]
                ],
            },
        },
        987: {
            "attributes": {"g_id": (5624, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388330.0],
                        [168720.0, 5388360.0],
                        [168750.0, 5388360.0],
                        [168750.0, 5388330.0],
                        [168720.0, 5388330.0],
                    ]
                ],
            },
        },
        988: {
            "attributes": {"g_id": (5624, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388360.0],
                        [168720.0, 5388390.0],
                        [168750.0, 5388390.0],
                        [168750.0, 5388360.0],
                        [168720.0, 5388360.0],
                    ]
                ],
            },
        },
        989: {
            "attributes": {"g_id": (5624, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388390.0],
                        [168720.0, 5388420.0],
                        [168750.0, 5388420.0],
                        [168750.0, 5388390.0],
                        [168720.0, 5388390.0],
                    ]
                ],
            },
        },
        990: {
            "attributes": {"g_id": (5624, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388420.0],
                        [168720.0, 5388450.0],
                        [168750.0, 5388450.0],
                        [168750.0, 5388420.0],
                        [168720.0, 5388420.0],
                    ]
                ],
            },
        },
        991: {
            "attributes": {"g_id": (5624, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388450.0],
                        [168720.0, 5388480.0],
                        [168750.0, 5388480.0],
                        [168750.0, 5388450.0],
                        [168720.0, 5388450.0],
                    ]
                ],
            },
        },
        992: {
            "attributes": {"g_id": (5624, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388480.0],
                        [168720.0, 5388510.0],
                        [168750.0, 5388510.0],
                        [168750.0, 5388480.0],
                        [168720.0, 5388480.0],
                    ]
                ],
            },
        },
        993: {
            "attributes": {"g_id": (5624, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388510.0],
                        [168720.0, 5388540.0],
                        [168750.0, 5388540.0],
                        [168750.0, 5388510.0],
                        [168720.0, 5388510.0],
                    ]
                ],
            },
        },
        994: {
            "attributes": {"g_id": (5624, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388540.0],
                        [168720.0, 5388570.0],
                        [168750.0, 5388570.0],
                        [168750.0, 5388540.0],
                        [168720.0, 5388540.0],
                    ]
                ],
            },
        },
        995: {
            "attributes": {"g_id": (5624, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388570.0],
                        [168720.0, 5388600.0],
                        [168750.0, 5388600.0],
                        [168750.0, 5388570.0],
                        [168720.0, 5388570.0],
                    ]
                ],
            },
        },
        996: {
            "attributes": {"g_id": (5624, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388600.0],
                        [168720.0, 5388630.0],
                        [168750.0, 5388630.0],
                        [168750.0, 5388600.0],
                        [168720.0, 5388600.0],
                    ]
                ],
            },
        },
        997: {
            "attributes": {"g_id": (5624, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388630.0],
                        [168720.0, 5388660.0],
                        [168750.0, 5388660.0],
                        [168750.0, 5388630.0],
                        [168720.0, 5388630.0],
                    ]
                ],
            },
        },
        998: {
            "attributes": {"g_id": (5624, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388660.0],
                        [168720.0, 5388690.0],
                        [168750.0, 5388690.0],
                        [168750.0, 5388660.0],
                        [168720.0, 5388660.0],
                    ]
                ],
            },
        },
        999: {
            "attributes": {"g_id": (5624, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388690.0],
                        [168720.0, 5388720.0],
                        [168750.0, 5388720.0],
                        [168750.0, 5388690.0],
                        [168720.0, 5388690.0],
                    ]
                ],
            },
        },
        1000: {
            "attributes": {"g_id": (5624, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388720.0],
                        [168720.0, 5388750.0],
                        [168750.0, 5388750.0],
                        [168750.0, 5388720.0],
                        [168720.0, 5388720.0],
                    ]
                ],
            },
        },
        1001: {
            "attributes": {"g_id": (5624, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388750.0],
                        [168720.0, 5388780.0],
                        [168750.0, 5388780.0],
                        [168750.0, 5388750.0],
                        [168720.0, 5388750.0],
                    ]
                ],
            },
        },
        1002: {
            "attributes": {"g_id": (5624, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388780.0],
                        [168720.0, 5388810.0],
                        [168750.0, 5388810.0],
                        [168750.0, 5388780.0],
                        [168720.0, 5388780.0],
                    ]
                ],
            },
        },
        1003: {
            "attributes": {"g_id": (5624, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388810.0],
                        [168720.0, 5388840.0],
                        [168750.0, 5388840.0],
                        [168750.0, 5388810.0],
                        [168720.0, 5388810.0],
                    ]
                ],
            },
        },
        1004: {
            "attributes": {"g_id": (5624, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388840.0],
                        [168720.0, 5388870.0],
                        [168750.0, 5388870.0],
                        [168750.0, 5388840.0],
                        [168720.0, 5388840.0],
                    ]
                ],
            },
        },
        1005: {
            "attributes": {"g_id": (5624, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388870.0],
                        [168720.0, 5388900.0],
                        [168750.0, 5388900.0],
                        [168750.0, 5388870.0],
                        [168720.0, 5388870.0],
                    ]
                ],
            },
        },
        1006: {
            "attributes": {"g_id": (5624, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388900.0],
                        [168720.0, 5388930.0],
                        [168750.0, 5388930.0],
                        [168750.0, 5388900.0],
                        [168720.0, 5388900.0],
                    ]
                ],
            },
        },
        1007: {
            "attributes": {"g_id": (5624, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168720.0, 5388930.0],
                        [168720.0, 5388960.0],
                        [168750.0, 5388960.0],
                        [168750.0, 5388930.0],
                        [168720.0, 5388930.0],
                    ]
                ],
            },
        },
        1008: {
            "attributes": {"g_id": (5625, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387520.0],
                        [168750.0, 5387550.0],
                        [168780.0, 5387550.0],
                        [168780.0, 5387520.0],
                        [168750.0, 5387520.0],
                    ]
                ],
            },
        },
        1009: {
            "attributes": {"g_id": (5625, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387550.0],
                        [168750.0, 5387580.0],
                        [168780.0, 5387580.0],
                        [168780.0, 5387550.0],
                        [168750.0, 5387550.0],
                    ]
                ],
            },
        },
        1010: {
            "attributes": {"g_id": (5625, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387580.0],
                        [168750.0, 5387610.0],
                        [168780.0, 5387610.0],
                        [168780.0, 5387580.0],
                        [168750.0, 5387580.0],
                    ]
                ],
            },
        },
        1011: {
            "attributes": {"g_id": (5625, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387610.0],
                        [168750.0, 5387640.0],
                        [168780.0, 5387640.0],
                        [168780.0, 5387610.0],
                        [168750.0, 5387610.0],
                    ]
                ],
            },
        },
        1012: {
            "attributes": {"g_id": (5625, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387640.0],
                        [168750.0, 5387670.0],
                        [168780.0, 5387670.0],
                        [168780.0, 5387640.0],
                        [168750.0, 5387640.0],
                    ]
                ],
            },
        },
        1013: {
            "attributes": {"g_id": (5625, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387670.0],
                        [168750.0, 5387700.0],
                        [168780.0, 5387700.0],
                        [168780.0, 5387670.0],
                        [168750.0, 5387670.0],
                    ]
                ],
            },
        },
        1014: {
            "attributes": {"g_id": (5625, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387700.0],
                        [168750.0, 5387730.0],
                        [168780.0, 5387730.0],
                        [168780.0, 5387700.0],
                        [168750.0, 5387700.0],
                    ]
                ],
            },
        },
        1015: {
            "attributes": {"g_id": (5625, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387730.0],
                        [168750.0, 5387760.0],
                        [168780.0, 5387760.0],
                        [168780.0, 5387730.0],
                        [168750.0, 5387730.0],
                    ]
                ],
            },
        },
        1016: {
            "attributes": {"g_id": (5625, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387760.0],
                        [168750.0, 5387790.0],
                        [168780.0, 5387790.0],
                        [168780.0, 5387760.0],
                        [168750.0, 5387760.0],
                    ]
                ],
            },
        },
        1017: {
            "attributes": {"g_id": (5625, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387790.0],
                        [168750.0, 5387820.0],
                        [168780.0, 5387820.0],
                        [168780.0, 5387790.0],
                        [168750.0, 5387790.0],
                    ]
                ],
            },
        },
        1018: {
            "attributes": {"g_id": (5625, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387820.0],
                        [168750.0, 5387850.0],
                        [168780.0, 5387850.0],
                        [168780.0, 5387820.0],
                        [168750.0, 5387820.0],
                    ]
                ],
            },
        },
        1019: {
            "attributes": {"g_id": (5625, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387850.0],
                        [168750.0, 5387880.0],
                        [168780.0, 5387880.0],
                        [168780.0, 5387850.0],
                        [168750.0, 5387850.0],
                    ]
                ],
            },
        },
        1020: {
            "attributes": {"g_id": (5625, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387880.0],
                        [168750.0, 5387910.0],
                        [168780.0, 5387910.0],
                        [168780.0, 5387880.0],
                        [168750.0, 5387880.0],
                    ]
                ],
            },
        },
        1021: {
            "attributes": {"g_id": (5625, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387910.0],
                        [168750.0, 5387940.0],
                        [168780.0, 5387940.0],
                        [168780.0, 5387910.0],
                        [168750.0, 5387910.0],
                    ]
                ],
            },
        },
        1022: {
            "attributes": {"g_id": (5625, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387940.0],
                        [168750.0, 5387970.0],
                        [168780.0, 5387970.0],
                        [168780.0, 5387940.0],
                        [168750.0, 5387940.0],
                    ]
                ],
            },
        },
        1023: {
            "attributes": {"g_id": (5625, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5387970.0],
                        [168750.0, 5388000.0],
                        [168780.0, 5388000.0],
                        [168780.0, 5387970.0],
                        [168750.0, 5387970.0],
                    ]
                ],
            },
        },
        1024: {
            "attributes": {"g_id": (5625, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388000.0],
                        [168750.0, 5388030.0],
                        [168780.0, 5388030.0],
                        [168780.0, 5388000.0],
                        [168750.0, 5388000.0],
                    ]
                ],
            },
        },
        1025: {
            "attributes": {"g_id": (5625, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388030.0],
                        [168750.0, 5388060.0],
                        [168780.0, 5388060.0],
                        [168780.0, 5388030.0],
                        [168750.0, 5388030.0],
                    ]
                ],
            },
        },
        1026: {
            "attributes": {"g_id": (5625, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388060.0],
                        [168750.0, 5388090.0],
                        [168780.0, 5388090.0],
                        [168780.0, 5388060.0],
                        [168750.0, 5388060.0],
                    ]
                ],
            },
        },
        1027: {
            "attributes": {"g_id": (5625, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388090.0],
                        [168750.0, 5388120.0],
                        [168780.0, 5388120.0],
                        [168780.0, 5388090.0],
                        [168750.0, 5388090.0],
                    ]
                ],
            },
        },
        1028: {
            "attributes": {"g_id": (5625, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388120.0],
                        [168750.0, 5388150.0],
                        [168780.0, 5388150.0],
                        [168780.0, 5388120.0],
                        [168750.0, 5388120.0],
                    ]
                ],
            },
        },
        1029: {
            "attributes": {"g_id": (5625, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388150.0],
                        [168750.0, 5388180.0],
                        [168780.0, 5388180.0],
                        [168780.0, 5388150.0],
                        [168750.0, 5388150.0],
                    ]
                ],
            },
        },
        1030: {
            "attributes": {"g_id": (5625, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388180.0],
                        [168750.0, 5388210.0],
                        [168780.0, 5388210.0],
                        [168780.0, 5388180.0],
                        [168750.0, 5388180.0],
                    ]
                ],
            },
        },
        1031: {
            "attributes": {"g_id": (5625, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388210.0],
                        [168750.0, 5388240.0],
                        [168780.0, 5388240.0],
                        [168780.0, 5388210.0],
                        [168750.0, 5388210.0],
                    ]
                ],
            },
        },
        1032: {
            "attributes": {"g_id": (5625, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388240.0],
                        [168750.0, 5388270.0],
                        [168780.0, 5388270.0],
                        [168780.0, 5388240.0],
                        [168750.0, 5388240.0],
                    ]
                ],
            },
        },
        1033: {
            "attributes": {"g_id": (5625, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388270.0],
                        [168750.0, 5388300.0],
                        [168780.0, 5388300.0],
                        [168780.0, 5388270.0],
                        [168750.0, 5388270.0],
                    ]
                ],
            },
        },
        1034: {
            "attributes": {"g_id": (5625, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388300.0],
                        [168750.0, 5388330.0],
                        [168780.0, 5388330.0],
                        [168780.0, 5388300.0],
                        [168750.0, 5388300.0],
                    ]
                ],
            },
        },
        1035: {
            "attributes": {"g_id": (5625, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388330.0],
                        [168750.0, 5388360.0],
                        [168780.0, 5388360.0],
                        [168780.0, 5388330.0],
                        [168750.0, 5388330.0],
                    ]
                ],
            },
        },
        1036: {
            "attributes": {"g_id": (5625, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388360.0],
                        [168750.0, 5388390.0],
                        [168780.0, 5388390.0],
                        [168780.0, 5388360.0],
                        [168750.0, 5388360.0],
                    ]
                ],
            },
        },
        1037: {
            "attributes": {"g_id": (5625, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388390.0],
                        [168750.0, 5388420.0],
                        [168780.0, 5388420.0],
                        [168780.0, 5388390.0],
                        [168750.0, 5388390.0],
                    ]
                ],
            },
        },
        1038: {
            "attributes": {"g_id": (5625, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388420.0],
                        [168750.0, 5388450.0],
                        [168780.0, 5388450.0],
                        [168780.0, 5388420.0],
                        [168750.0, 5388420.0],
                    ]
                ],
            },
        },
        1039: {
            "attributes": {"g_id": (5625, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388450.0],
                        [168750.0, 5388480.0],
                        [168780.0, 5388480.0],
                        [168780.0, 5388450.0],
                        [168750.0, 5388450.0],
                    ]
                ],
            },
        },
        1040: {
            "attributes": {"g_id": (5625, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388480.0],
                        [168750.0, 5388510.0],
                        [168780.0, 5388510.0],
                        [168780.0, 5388480.0],
                        [168750.0, 5388480.0],
                    ]
                ],
            },
        },
        1041: {
            "attributes": {"g_id": (5625, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388510.0],
                        [168750.0, 5388540.0],
                        [168780.0, 5388540.0],
                        [168780.0, 5388510.0],
                        [168750.0, 5388510.0],
                    ]
                ],
            },
        },
        1042: {
            "attributes": {"g_id": (5625, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388540.0],
                        [168750.0, 5388570.0],
                        [168780.0, 5388570.0],
                        [168780.0, 5388540.0],
                        [168750.0, 5388540.0],
                    ]
                ],
            },
        },
        1043: {
            "attributes": {"g_id": (5625, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388570.0],
                        [168750.0, 5388600.0],
                        [168780.0, 5388600.0],
                        [168780.0, 5388570.0],
                        [168750.0, 5388570.0],
                    ]
                ],
            },
        },
        1044: {
            "attributes": {"g_id": (5625, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388600.0],
                        [168750.0, 5388630.0],
                        [168780.0, 5388630.0],
                        [168780.0, 5388600.0],
                        [168750.0, 5388600.0],
                    ]
                ],
            },
        },
        1045: {
            "attributes": {"g_id": (5625, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388630.0],
                        [168750.0, 5388660.0],
                        [168780.0, 5388660.0],
                        [168780.0, 5388630.0],
                        [168750.0, 5388630.0],
                    ]
                ],
            },
        },
        1046: {
            "attributes": {"g_id": (5625, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388660.0],
                        [168750.0, 5388690.0],
                        [168780.0, 5388690.0],
                        [168780.0, 5388660.0],
                        [168750.0, 5388660.0],
                    ]
                ],
            },
        },
        1047: {
            "attributes": {"g_id": (5625, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388690.0],
                        [168750.0, 5388720.0],
                        [168780.0, 5388720.0],
                        [168780.0, 5388690.0],
                        [168750.0, 5388690.0],
                    ]
                ],
            },
        },
        1048: {
            "attributes": {"g_id": (5625, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388720.0],
                        [168750.0, 5388750.0],
                        [168780.0, 5388750.0],
                        [168780.0, 5388720.0],
                        [168750.0, 5388720.0],
                    ]
                ],
            },
        },
        1049: {
            "attributes": {"g_id": (5625, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388750.0],
                        [168750.0, 5388780.0],
                        [168780.0, 5388780.0],
                        [168780.0, 5388750.0],
                        [168750.0, 5388750.0],
                    ]
                ],
            },
        },
        1050: {
            "attributes": {"g_id": (5625, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388780.0],
                        [168750.0, 5388810.0],
                        [168780.0, 5388810.0],
                        [168780.0, 5388780.0],
                        [168750.0, 5388780.0],
                    ]
                ],
            },
        },
        1051: {
            "attributes": {"g_id": (5625, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388810.0],
                        [168750.0, 5388840.0],
                        [168780.0, 5388840.0],
                        [168780.0, 5388810.0],
                        [168750.0, 5388810.0],
                    ]
                ],
            },
        },
        1052: {
            "attributes": {"g_id": (5625, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388840.0],
                        [168750.0, 5388870.0],
                        [168780.0, 5388870.0],
                        [168780.0, 5388840.0],
                        [168750.0, 5388840.0],
                    ]
                ],
            },
        },
        1053: {
            "attributes": {"g_id": (5625, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388870.0],
                        [168750.0, 5388900.0],
                        [168780.0, 5388900.0],
                        [168780.0, 5388870.0],
                        [168750.0, 5388870.0],
                    ]
                ],
            },
        },
        1054: {
            "attributes": {"g_id": (5625, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388900.0],
                        [168750.0, 5388930.0],
                        [168780.0, 5388930.0],
                        [168780.0, 5388900.0],
                        [168750.0, 5388900.0],
                    ]
                ],
            },
        },
        1055: {
            "attributes": {"g_id": (5625, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168750.0, 5388930.0],
                        [168750.0, 5388960.0],
                        [168780.0, 5388960.0],
                        [168780.0, 5388930.0],
                        [168750.0, 5388930.0],
                    ]
                ],
            },
        },
        1056: {
            "attributes": {"g_id": (5626, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387520.0],
                        [168780.0, 5387550.0],
                        [168810.0, 5387550.0],
                        [168810.0, 5387520.0],
                        [168780.0, 5387520.0],
                    ]
                ],
            },
        },
        1057: {
            "attributes": {"g_id": (5626, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387550.0],
                        [168780.0, 5387580.0],
                        [168810.0, 5387580.0],
                        [168810.0, 5387550.0],
                        [168780.0, 5387550.0],
                    ]
                ],
            },
        },
        1058: {
            "attributes": {"g_id": (5626, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387580.0],
                        [168780.0, 5387610.0],
                        [168810.0, 5387610.0],
                        [168810.0, 5387580.0],
                        [168780.0, 5387580.0],
                    ]
                ],
            },
        },
        1059: {
            "attributes": {"g_id": (5626, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387610.0],
                        [168780.0, 5387640.0],
                        [168810.0, 5387640.0],
                        [168810.0, 5387610.0],
                        [168780.0, 5387610.0],
                    ]
                ],
            },
        },
        1060: {
            "attributes": {"g_id": (5626, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387640.0],
                        [168780.0, 5387670.0],
                        [168810.0, 5387670.0],
                        [168810.0, 5387640.0],
                        [168780.0, 5387640.0],
                    ]
                ],
            },
        },
        1061: {
            "attributes": {"g_id": (5626, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387670.0],
                        [168780.0, 5387700.0],
                        [168810.0, 5387700.0],
                        [168810.0, 5387670.0],
                        [168780.0, 5387670.0],
                    ]
                ],
            },
        },
        1062: {
            "attributes": {"g_id": (5626, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387700.0],
                        [168780.0, 5387730.0],
                        [168810.0, 5387730.0],
                        [168810.0, 5387700.0],
                        [168780.0, 5387700.0],
                    ]
                ],
            },
        },
        1063: {
            "attributes": {"g_id": (5626, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387730.0],
                        [168780.0, 5387760.0],
                        [168810.0, 5387760.0],
                        [168810.0, 5387730.0],
                        [168780.0, 5387730.0],
                    ]
                ],
            },
        },
        1064: {
            "attributes": {"g_id": (5626, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387760.0],
                        [168780.0, 5387790.0],
                        [168810.0, 5387790.0],
                        [168810.0, 5387760.0],
                        [168780.0, 5387760.0],
                    ]
                ],
            },
        },
        1065: {
            "attributes": {"g_id": (5626, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387790.0],
                        [168780.0, 5387820.0],
                        [168810.0, 5387820.0],
                        [168810.0, 5387790.0],
                        [168780.0, 5387790.0],
                    ]
                ],
            },
        },
        1066: {
            "attributes": {"g_id": (5626, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387820.0],
                        [168780.0, 5387850.0],
                        [168810.0, 5387850.0],
                        [168810.0, 5387820.0],
                        [168780.0, 5387820.0],
                    ]
                ],
            },
        },
        1067: {
            "attributes": {"g_id": (5626, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387850.0],
                        [168780.0, 5387880.0],
                        [168810.0, 5387880.0],
                        [168810.0, 5387850.0],
                        [168780.0, 5387850.0],
                    ]
                ],
            },
        },
        1068: {
            "attributes": {"g_id": (5626, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387880.0],
                        [168780.0, 5387910.0],
                        [168810.0, 5387910.0],
                        [168810.0, 5387880.0],
                        [168780.0, 5387880.0],
                    ]
                ],
            },
        },
        1069: {
            "attributes": {"g_id": (5626, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387910.0],
                        [168780.0, 5387940.0],
                        [168810.0, 5387940.0],
                        [168810.0, 5387910.0],
                        [168780.0, 5387910.0],
                    ]
                ],
            },
        },
        1070: {
            "attributes": {"g_id": (5626, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387940.0],
                        [168780.0, 5387970.0],
                        [168810.0, 5387970.0],
                        [168810.0, 5387940.0],
                        [168780.0, 5387940.0],
                    ]
                ],
            },
        },
        1071: {
            "attributes": {"g_id": (5626, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5387970.0],
                        [168780.0, 5388000.0],
                        [168810.0, 5388000.0],
                        [168810.0, 5387970.0],
                        [168780.0, 5387970.0],
                    ]
                ],
            },
        },
        1072: {
            "attributes": {"g_id": (5626, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388000.0],
                        [168780.0, 5388030.0],
                        [168810.0, 5388030.0],
                        [168810.0, 5388000.0],
                        [168780.0, 5388000.0],
                    ]
                ],
            },
        },
        1073: {
            "attributes": {"g_id": (5626, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388030.0],
                        [168780.0, 5388060.0],
                        [168810.0, 5388060.0],
                        [168810.0, 5388030.0],
                        [168780.0, 5388030.0],
                    ]
                ],
            },
        },
        1074: {
            "attributes": {"g_id": (5626, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388060.0],
                        [168780.0, 5388090.0],
                        [168810.0, 5388090.0],
                        [168810.0, 5388060.0],
                        [168780.0, 5388060.0],
                    ]
                ],
            },
        },
        1075: {
            "attributes": {"g_id": (5626, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388090.0],
                        [168780.0, 5388120.0],
                        [168810.0, 5388120.0],
                        [168810.0, 5388090.0],
                        [168780.0, 5388090.0],
                    ]
                ],
            },
        },
        1076: {
            "attributes": {"g_id": (5626, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388120.0],
                        [168780.0, 5388150.0],
                        [168810.0, 5388150.0],
                        [168810.0, 5388120.0],
                        [168780.0, 5388120.0],
                    ]
                ],
            },
        },
        1077: {
            "attributes": {"g_id": (5626, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388150.0],
                        [168780.0, 5388180.0],
                        [168810.0, 5388180.0],
                        [168810.0, 5388150.0],
                        [168780.0, 5388150.0],
                    ]
                ],
            },
        },
        1078: {
            "attributes": {"g_id": (5626, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388180.0],
                        [168780.0, 5388210.0],
                        [168810.0, 5388210.0],
                        [168810.0, 5388180.0],
                        [168780.0, 5388180.0],
                    ]
                ],
            },
        },
        1079: {
            "attributes": {"g_id": (5626, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388210.0],
                        [168780.0, 5388240.0],
                        [168810.0, 5388240.0],
                        [168810.0, 5388210.0],
                        [168780.0, 5388210.0],
                    ]
                ],
            },
        },
        1080: {
            "attributes": {"g_id": (5626, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388240.0],
                        [168780.0, 5388270.0],
                        [168810.0, 5388270.0],
                        [168810.0, 5388240.0],
                        [168780.0, 5388240.0],
                    ]
                ],
            },
        },
        1081: {
            "attributes": {"g_id": (5626, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388270.0],
                        [168780.0, 5388300.0],
                        [168810.0, 5388300.0],
                        [168810.0, 5388270.0],
                        [168780.0, 5388270.0],
                    ]
                ],
            },
        },
        1082: {
            "attributes": {"g_id": (5626, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388300.0],
                        [168780.0, 5388330.0],
                        [168810.0, 5388330.0],
                        [168810.0, 5388300.0],
                        [168780.0, 5388300.0],
                    ]
                ],
            },
        },
        1083: {
            "attributes": {"g_id": (5626, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388330.0],
                        [168780.0, 5388360.0],
                        [168810.0, 5388360.0],
                        [168810.0, 5388330.0],
                        [168780.0, 5388330.0],
                    ]
                ],
            },
        },
        1084: {
            "attributes": {"g_id": (5626, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388360.0],
                        [168780.0, 5388390.0],
                        [168810.0, 5388390.0],
                        [168810.0, 5388360.0],
                        [168780.0, 5388360.0],
                    ]
                ],
            },
        },
        1085: {
            "attributes": {"g_id": (5626, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388390.0],
                        [168780.0, 5388420.0],
                        [168810.0, 5388420.0],
                        [168810.0, 5388390.0],
                        [168780.0, 5388390.0],
                    ]
                ],
            },
        },
        1086: {
            "attributes": {"g_id": (5626, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388420.0],
                        [168780.0, 5388450.0],
                        [168810.0, 5388450.0],
                        [168810.0, 5388420.0],
                        [168780.0, 5388420.0],
                    ]
                ],
            },
        },
        1087: {
            "attributes": {"g_id": (5626, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388450.0],
                        [168780.0, 5388480.0],
                        [168810.0, 5388480.0],
                        [168810.0, 5388450.0],
                        [168780.0, 5388450.0],
                    ]
                ],
            },
        },
        1088: {
            "attributes": {"g_id": (5626, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388480.0],
                        [168780.0, 5388510.0],
                        [168810.0, 5388510.0],
                        [168810.0, 5388480.0],
                        [168780.0, 5388480.0],
                    ]
                ],
            },
        },
        1089: {
            "attributes": {"g_id": (5626, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388510.0],
                        [168780.0, 5388540.0],
                        [168810.0, 5388540.0],
                        [168810.0, 5388510.0],
                        [168780.0, 5388510.0],
                    ]
                ],
            },
        },
        1090: {
            "attributes": {"g_id": (5626, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388540.0],
                        [168780.0, 5388570.0],
                        [168810.0, 5388570.0],
                        [168810.0, 5388540.0],
                        [168780.0, 5388540.0],
                    ]
                ],
            },
        },
        1091: {
            "attributes": {"g_id": (5626, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388570.0],
                        [168780.0, 5388600.0],
                        [168810.0, 5388600.0],
                        [168810.0, 5388570.0],
                        [168780.0, 5388570.0],
                    ]
                ],
            },
        },
        1092: {
            "attributes": {"g_id": (5626, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388600.0],
                        [168780.0, 5388630.0],
                        [168810.0, 5388630.0],
                        [168810.0, 5388600.0],
                        [168780.0, 5388600.0],
                    ]
                ],
            },
        },
        1093: {
            "attributes": {"g_id": (5626, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388630.0],
                        [168780.0, 5388660.0],
                        [168810.0, 5388660.0],
                        [168810.0, 5388630.0],
                        [168780.0, 5388630.0],
                    ]
                ],
            },
        },
        1094: {
            "attributes": {"g_id": (5626, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388660.0],
                        [168780.0, 5388690.0],
                        [168810.0, 5388690.0],
                        [168810.0, 5388660.0],
                        [168780.0, 5388660.0],
                    ]
                ],
            },
        },
        1095: {
            "attributes": {"g_id": (5626, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388690.0],
                        [168780.0, 5388720.0],
                        [168810.0, 5388720.0],
                        [168810.0, 5388690.0],
                        [168780.0, 5388690.0],
                    ]
                ],
            },
        },
        1096: {
            "attributes": {"g_id": (5626, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388720.0],
                        [168780.0, 5388750.0],
                        [168810.0, 5388750.0],
                        [168810.0, 5388720.0],
                        [168780.0, 5388720.0],
                    ]
                ],
            },
        },
        1097: {
            "attributes": {"g_id": (5626, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388750.0],
                        [168780.0, 5388780.0],
                        [168810.0, 5388780.0],
                        [168810.0, 5388750.0],
                        [168780.0, 5388750.0],
                    ]
                ],
            },
        },
        1098: {
            "attributes": {"g_id": (5626, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388780.0],
                        [168780.0, 5388810.0],
                        [168810.0, 5388810.0],
                        [168810.0, 5388780.0],
                        [168780.0, 5388780.0],
                    ]
                ],
            },
        },
        1099: {
            "attributes": {"g_id": (5626, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388810.0],
                        [168780.0, 5388840.0],
                        [168810.0, 5388840.0],
                        [168810.0, 5388810.0],
                        [168780.0, 5388810.0],
                    ]
                ],
            },
        },
        1100: {
            "attributes": {"g_id": (5626, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388840.0],
                        [168780.0, 5388870.0],
                        [168810.0, 5388870.0],
                        [168810.0, 5388840.0],
                        [168780.0, 5388840.0],
                    ]
                ],
            },
        },
        1101: {
            "attributes": {"g_id": (5626, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388870.0],
                        [168780.0, 5388900.0],
                        [168810.0, 5388900.0],
                        [168810.0, 5388870.0],
                        [168780.0, 5388870.0],
                    ]
                ],
            },
        },
        1102: {
            "attributes": {"g_id": (5626, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388900.0],
                        [168780.0, 5388930.0],
                        [168810.0, 5388930.0],
                        [168810.0, 5388900.0],
                        [168780.0, 5388900.0],
                    ]
                ],
            },
        },
        1103: {
            "attributes": {"g_id": (5626, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168780.0, 5388930.0],
                        [168780.0, 5388960.0],
                        [168810.0, 5388960.0],
                        [168810.0, 5388930.0],
                        [168780.0, 5388930.0],
                    ]
                ],
            },
        },
        1104: {
            "attributes": {"g_id": (5627, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387520.0],
                        [168810.0, 5387550.0],
                        [168840.0, 5387550.0],
                        [168840.0, 5387520.0],
                        [168810.0, 5387520.0],
                    ]
                ],
            },
        },
        1105: {
            "attributes": {"g_id": (5627, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387550.0],
                        [168810.0, 5387580.0],
                        [168840.0, 5387580.0],
                        [168840.0, 5387550.0],
                        [168810.0, 5387550.0],
                    ]
                ],
            },
        },
        1106: {
            "attributes": {"g_id": (5627, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387580.0],
                        [168810.0, 5387610.0],
                        [168840.0, 5387610.0],
                        [168840.0, 5387580.0],
                        [168810.0, 5387580.0],
                    ]
                ],
            },
        },
        1107: {
            "attributes": {"g_id": (5627, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387610.0],
                        [168810.0, 5387640.0],
                        [168840.0, 5387640.0],
                        [168840.0, 5387610.0],
                        [168810.0, 5387610.0],
                    ]
                ],
            },
        },
        1108: {
            "attributes": {"g_id": (5627, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387640.0],
                        [168810.0, 5387670.0],
                        [168840.0, 5387670.0],
                        [168840.0, 5387640.0],
                        [168810.0, 5387640.0],
                    ]
                ],
            },
        },
        1109: {
            "attributes": {"g_id": (5627, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387670.0],
                        [168810.0, 5387700.0],
                        [168840.0, 5387700.0],
                        [168840.0, 5387670.0],
                        [168810.0, 5387670.0],
                    ]
                ],
            },
        },
        1110: {
            "attributes": {"g_id": (5627, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387700.0],
                        [168810.0, 5387730.0],
                        [168840.0, 5387730.0],
                        [168840.0, 5387700.0],
                        [168810.0, 5387700.0],
                    ]
                ],
            },
        },
        1111: {
            "attributes": {"g_id": (5627, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387730.0],
                        [168810.0, 5387760.0],
                        [168840.0, 5387760.0],
                        [168840.0, 5387730.0],
                        [168810.0, 5387730.0],
                    ]
                ],
            },
        },
        1112: {
            "attributes": {"g_id": (5627, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387760.0],
                        [168810.0, 5387790.0],
                        [168840.0, 5387790.0],
                        [168840.0, 5387760.0],
                        [168810.0, 5387760.0],
                    ]
                ],
            },
        },
        1113: {
            "attributes": {"g_id": (5627, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387790.0],
                        [168810.0, 5387820.0],
                        [168840.0, 5387820.0],
                        [168840.0, 5387790.0],
                        [168810.0, 5387790.0],
                    ]
                ],
            },
        },
        1114: {
            "attributes": {"g_id": (5627, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387820.0],
                        [168810.0, 5387850.0],
                        [168840.0, 5387850.0],
                        [168840.0, 5387820.0],
                        [168810.0, 5387820.0],
                    ]
                ],
            },
        },
        1115: {
            "attributes": {"g_id": (5627, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387850.0],
                        [168810.0, 5387880.0],
                        [168840.0, 5387880.0],
                        [168840.0, 5387850.0],
                        [168810.0, 5387850.0],
                    ]
                ],
            },
        },
        1116: {
            "attributes": {"g_id": (5627, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387880.0],
                        [168810.0, 5387910.0],
                        [168840.0, 5387910.0],
                        [168840.0, 5387880.0],
                        [168810.0, 5387880.0],
                    ]
                ],
            },
        },
        1117: {
            "attributes": {"g_id": (5627, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387910.0],
                        [168810.0, 5387940.0],
                        [168840.0, 5387940.0],
                        [168840.0, 5387910.0],
                        [168810.0, 5387910.0],
                    ]
                ],
            },
        },
        1118: {
            "attributes": {"g_id": (5627, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387940.0],
                        [168810.0, 5387970.0],
                        [168840.0, 5387970.0],
                        [168840.0, 5387940.0],
                        [168810.0, 5387940.0],
                    ]
                ],
            },
        },
        1119: {
            "attributes": {"g_id": (5627, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5387970.0],
                        [168810.0, 5388000.0],
                        [168840.0, 5388000.0],
                        [168840.0, 5387970.0],
                        [168810.0, 5387970.0],
                    ]
                ],
            },
        },
        1120: {
            "attributes": {"g_id": (5627, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388000.0],
                        [168810.0, 5388030.0],
                        [168840.0, 5388030.0],
                        [168840.0, 5388000.0],
                        [168810.0, 5388000.0],
                    ]
                ],
            },
        },
        1121: {
            "attributes": {"g_id": (5627, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388030.0],
                        [168810.0, 5388060.0],
                        [168840.0, 5388060.0],
                        [168840.0, 5388030.0],
                        [168810.0, 5388030.0],
                    ]
                ],
            },
        },
        1122: {
            "attributes": {"g_id": (5627, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388060.0],
                        [168810.0, 5388090.0],
                        [168840.0, 5388090.0],
                        [168840.0, 5388060.0],
                        [168810.0, 5388060.0],
                    ]
                ],
            },
        },
        1123: {
            "attributes": {"g_id": (5627, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388090.0],
                        [168810.0, 5388120.0],
                        [168840.0, 5388120.0],
                        [168840.0, 5388090.0],
                        [168810.0, 5388090.0],
                    ]
                ],
            },
        },
        1124: {
            "attributes": {"g_id": (5627, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388120.0],
                        [168810.0, 5388150.0],
                        [168840.0, 5388150.0],
                        [168840.0, 5388120.0],
                        [168810.0, 5388120.0],
                    ]
                ],
            },
        },
        1125: {
            "attributes": {"g_id": (5627, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388150.0],
                        [168810.0, 5388180.0],
                        [168840.0, 5388180.0],
                        [168840.0, 5388150.0],
                        [168810.0, 5388150.0],
                    ]
                ],
            },
        },
        1126: {
            "attributes": {"g_id": (5627, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388180.0],
                        [168810.0, 5388210.0],
                        [168840.0, 5388210.0],
                        [168840.0, 5388180.0],
                        [168810.0, 5388180.0],
                    ]
                ],
            },
        },
        1127: {
            "attributes": {"g_id": (5627, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388210.0],
                        [168810.0, 5388240.0],
                        [168840.0, 5388240.0],
                        [168840.0, 5388210.0],
                        [168810.0, 5388210.0],
                    ]
                ],
            },
        },
        1128: {
            "attributes": {"g_id": (5627, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388240.0],
                        [168810.0, 5388270.0],
                        [168840.0, 5388270.0],
                        [168840.0, 5388240.0],
                        [168810.0, 5388240.0],
                    ]
                ],
            },
        },
        1129: {
            "attributes": {"g_id": (5627, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388270.0],
                        [168810.0, 5388300.0],
                        [168840.0, 5388300.0],
                        [168840.0, 5388270.0],
                        [168810.0, 5388270.0],
                    ]
                ],
            },
        },
        1130: {
            "attributes": {"g_id": (5627, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388300.0],
                        [168810.0, 5388330.0],
                        [168840.0, 5388330.0],
                        [168840.0, 5388300.0],
                        [168810.0, 5388300.0],
                    ]
                ],
            },
        },
        1131: {
            "attributes": {"g_id": (5627, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388330.0],
                        [168810.0, 5388360.0],
                        [168840.0, 5388360.0],
                        [168840.0, 5388330.0],
                        [168810.0, 5388330.0],
                    ]
                ],
            },
        },
        1132: {
            "attributes": {"g_id": (5627, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388360.0],
                        [168810.0, 5388390.0],
                        [168840.0, 5388390.0],
                        [168840.0, 5388360.0],
                        [168810.0, 5388360.0],
                    ]
                ],
            },
        },
        1133: {
            "attributes": {"g_id": (5627, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388390.0],
                        [168810.0, 5388420.0],
                        [168840.0, 5388420.0],
                        [168840.0, 5388390.0],
                        [168810.0, 5388390.0],
                    ]
                ],
            },
        },
        1134: {
            "attributes": {"g_id": (5627, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388420.0],
                        [168810.0, 5388450.0],
                        [168840.0, 5388450.0],
                        [168840.0, 5388420.0],
                        [168810.0, 5388420.0],
                    ]
                ],
            },
        },
        1135: {
            "attributes": {"g_id": (5627, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388450.0],
                        [168810.0, 5388480.0],
                        [168840.0, 5388480.0],
                        [168840.0, 5388450.0],
                        [168810.0, 5388450.0],
                    ]
                ],
            },
        },
        1136: {
            "attributes": {"g_id": (5627, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388480.0],
                        [168810.0, 5388510.0],
                        [168840.0, 5388510.0],
                        [168840.0, 5388480.0],
                        [168810.0, 5388480.0],
                    ]
                ],
            },
        },
        1137: {
            "attributes": {"g_id": (5627, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388510.0],
                        [168810.0, 5388540.0],
                        [168840.0, 5388540.0],
                        [168840.0, 5388510.0],
                        [168810.0, 5388510.0],
                    ]
                ],
            },
        },
        1138: {
            "attributes": {"g_id": (5627, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388540.0],
                        [168810.0, 5388570.0],
                        [168840.0, 5388570.0],
                        [168840.0, 5388540.0],
                        [168810.0, 5388540.0],
                    ]
                ],
            },
        },
        1139: {
            "attributes": {"g_id": (5627, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388570.0],
                        [168810.0, 5388600.0],
                        [168840.0, 5388600.0],
                        [168840.0, 5388570.0],
                        [168810.0, 5388570.0],
                    ]
                ],
            },
        },
        1140: {
            "attributes": {"g_id": (5627, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388600.0],
                        [168810.0, 5388630.0],
                        [168840.0, 5388630.0],
                        [168840.0, 5388600.0],
                        [168810.0, 5388600.0],
                    ]
                ],
            },
        },
        1141: {
            "attributes": {"g_id": (5627, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388630.0],
                        [168810.0, 5388660.0],
                        [168840.0, 5388660.0],
                        [168840.0, 5388630.0],
                        [168810.0, 5388630.0],
                    ]
                ],
            },
        },
        1142: {
            "attributes": {"g_id": (5627, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388660.0],
                        [168810.0, 5388690.0],
                        [168840.0, 5388690.0],
                        [168840.0, 5388660.0],
                        [168810.0, 5388660.0],
                    ]
                ],
            },
        },
        1143: {
            "attributes": {"g_id": (5627, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388690.0],
                        [168810.0, 5388720.0],
                        [168840.0, 5388720.0],
                        [168840.0, 5388690.0],
                        [168810.0, 5388690.0],
                    ]
                ],
            },
        },
        1144: {
            "attributes": {"g_id": (5627, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388720.0],
                        [168810.0, 5388750.0],
                        [168840.0, 5388750.0],
                        [168840.0, 5388720.0],
                        [168810.0, 5388720.0],
                    ]
                ],
            },
        },
        1145: {
            "attributes": {"g_id": (5627, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388750.0],
                        [168810.0, 5388780.0],
                        [168840.0, 5388780.0],
                        [168840.0, 5388750.0],
                        [168810.0, 5388750.0],
                    ]
                ],
            },
        },
        1146: {
            "attributes": {"g_id": (5627, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388780.0],
                        [168810.0, 5388810.0],
                        [168840.0, 5388810.0],
                        [168840.0, 5388780.0],
                        [168810.0, 5388780.0],
                    ]
                ],
            },
        },
        1147: {
            "attributes": {"g_id": (5627, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388810.0],
                        [168810.0, 5388840.0],
                        [168840.0, 5388840.0],
                        [168840.0, 5388810.0],
                        [168810.0, 5388810.0],
                    ]
                ],
            },
        },
        1148: {
            "attributes": {"g_id": (5627, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388840.0],
                        [168810.0, 5388870.0],
                        [168840.0, 5388870.0],
                        [168840.0, 5388840.0],
                        [168810.0, 5388840.0],
                    ]
                ],
            },
        },
        1149: {
            "attributes": {"g_id": (5627, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388870.0],
                        [168810.0, 5388900.0],
                        [168840.0, 5388900.0],
                        [168840.0, 5388870.0],
                        [168810.0, 5388870.0],
                    ]
                ],
            },
        },
        1150: {
            "attributes": {"g_id": (5627, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388900.0],
                        [168810.0, 5388930.0],
                        [168840.0, 5388930.0],
                        [168840.0, 5388900.0],
                        [168810.0, 5388900.0],
                    ]
                ],
            },
        },
        1151: {
            "attributes": {"g_id": (5627, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168810.0, 5388930.0],
                        [168810.0, 5388960.0],
                        [168840.0, 5388960.0],
                        [168840.0, 5388930.0],
                        [168810.0, 5388930.0],
                    ]
                ],
            },
        },
        1152: {
            "attributes": {"g_id": (5628, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387520.0],
                        [168840.0, 5387550.0],
                        [168870.0, 5387550.0],
                        [168870.0, 5387520.0],
                        [168840.0, 5387520.0],
                    ]
                ],
            },
        },
        1153: {
            "attributes": {"g_id": (5628, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387550.0],
                        [168840.0, 5387580.0],
                        [168870.0, 5387580.0],
                        [168870.0, 5387550.0],
                        [168840.0, 5387550.0],
                    ]
                ],
            },
        },
        1154: {
            "attributes": {"g_id": (5628, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387580.0],
                        [168840.0, 5387610.0],
                        [168870.0, 5387610.0],
                        [168870.0, 5387580.0],
                        [168840.0, 5387580.0],
                    ]
                ],
            },
        },
        1155: {
            "attributes": {"g_id": (5628, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387610.0],
                        [168840.0, 5387640.0],
                        [168870.0, 5387640.0],
                        [168870.0, 5387610.0],
                        [168840.0, 5387610.0],
                    ]
                ],
            },
        },
        1156: {
            "attributes": {"g_id": (5628, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387640.0],
                        [168840.0, 5387670.0],
                        [168870.0, 5387670.0],
                        [168870.0, 5387640.0],
                        [168840.0, 5387640.0],
                    ]
                ],
            },
        },
        1157: {
            "attributes": {"g_id": (5628, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387670.0],
                        [168840.0, 5387700.0],
                        [168870.0, 5387700.0],
                        [168870.0, 5387670.0],
                        [168840.0, 5387670.0],
                    ]
                ],
            },
        },
        1158: {
            "attributes": {"g_id": (5628, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387700.0],
                        [168840.0, 5387730.0],
                        [168870.0, 5387730.0],
                        [168870.0, 5387700.0],
                        [168840.0, 5387700.0],
                    ]
                ],
            },
        },
        1159: {
            "attributes": {"g_id": (5628, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387730.0],
                        [168840.0, 5387760.0],
                        [168870.0, 5387760.0],
                        [168870.0, 5387730.0],
                        [168840.0, 5387730.0],
                    ]
                ],
            },
        },
        1160: {
            "attributes": {"g_id": (5628, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387760.0],
                        [168840.0, 5387790.0],
                        [168870.0, 5387790.0],
                        [168870.0, 5387760.0],
                        [168840.0, 5387760.0],
                    ]
                ],
            },
        },
        1161: {
            "attributes": {"g_id": (5628, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387790.0],
                        [168840.0, 5387820.0],
                        [168870.0, 5387820.0],
                        [168870.0, 5387790.0],
                        [168840.0, 5387790.0],
                    ]
                ],
            },
        },
        1162: {
            "attributes": {"g_id": (5628, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387820.0],
                        [168840.0, 5387850.0],
                        [168870.0, 5387850.0],
                        [168870.0, 5387820.0],
                        [168840.0, 5387820.0],
                    ]
                ],
            },
        },
        1163: {
            "attributes": {"g_id": (5628, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387850.0],
                        [168840.0, 5387880.0],
                        [168870.0, 5387880.0],
                        [168870.0, 5387850.0],
                        [168840.0, 5387850.0],
                    ]
                ],
            },
        },
        1164: {
            "attributes": {"g_id": (5628, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387880.0],
                        [168840.0, 5387910.0],
                        [168870.0, 5387910.0],
                        [168870.0, 5387880.0],
                        [168840.0, 5387880.0],
                    ]
                ],
            },
        },
        1165: {
            "attributes": {"g_id": (5628, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387910.0],
                        [168840.0, 5387940.0],
                        [168870.0, 5387940.0],
                        [168870.0, 5387910.0],
                        [168840.0, 5387910.0],
                    ]
                ],
            },
        },
        1166: {
            "attributes": {"g_id": (5628, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387940.0],
                        [168840.0, 5387970.0],
                        [168870.0, 5387970.0],
                        [168870.0, 5387940.0],
                        [168840.0, 5387940.0],
                    ]
                ],
            },
        },
        1167: {
            "attributes": {"g_id": (5628, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5387970.0],
                        [168840.0, 5388000.0],
                        [168870.0, 5388000.0],
                        [168870.0, 5387970.0],
                        [168840.0, 5387970.0],
                    ]
                ],
            },
        },
        1168: {
            "attributes": {"g_id": (5628, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388000.0],
                        [168840.0, 5388030.0],
                        [168870.0, 5388030.0],
                        [168870.0, 5388000.0],
                        [168840.0, 5388000.0],
                    ]
                ],
            },
        },
        1169: {
            "attributes": {"g_id": (5628, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388030.0],
                        [168840.0, 5388060.0],
                        [168870.0, 5388060.0],
                        [168870.0, 5388030.0],
                        [168840.0, 5388030.0],
                    ]
                ],
            },
        },
        1170: {
            "attributes": {"g_id": (5628, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388060.0],
                        [168840.0, 5388090.0],
                        [168870.0, 5388090.0],
                        [168870.0, 5388060.0],
                        [168840.0, 5388060.0],
                    ]
                ],
            },
        },
        1171: {
            "attributes": {"g_id": (5628, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388090.0],
                        [168840.0, 5388120.0],
                        [168870.0, 5388120.0],
                        [168870.0, 5388090.0],
                        [168840.0, 5388090.0],
                    ]
                ],
            },
        },
        1172: {
            "attributes": {"g_id": (5628, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388120.0],
                        [168840.0, 5388150.0],
                        [168870.0, 5388150.0],
                        [168870.0, 5388120.0],
                        [168840.0, 5388120.0],
                    ]
                ],
            },
        },
        1173: {
            "attributes": {"g_id": (5628, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388150.0],
                        [168840.0, 5388180.0],
                        [168870.0, 5388180.0],
                        [168870.0, 5388150.0],
                        [168840.0, 5388150.0],
                    ]
                ],
            },
        },
        1174: {
            "attributes": {"g_id": (5628, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388180.0],
                        [168840.0, 5388210.0],
                        [168870.0, 5388210.0],
                        [168870.0, 5388180.0],
                        [168840.0, 5388180.0],
                    ]
                ],
            },
        },
        1175: {
            "attributes": {"g_id": (5628, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388210.0],
                        [168840.0, 5388240.0],
                        [168870.0, 5388240.0],
                        [168870.0, 5388210.0],
                        [168840.0, 5388210.0],
                    ]
                ],
            },
        },
        1176: {
            "attributes": {"g_id": (5628, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388240.0],
                        [168840.0, 5388270.0],
                        [168870.0, 5388270.0],
                        [168870.0, 5388240.0],
                        [168840.0, 5388240.0],
                    ]
                ],
            },
        },
        1177: {
            "attributes": {"g_id": (5628, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388270.0],
                        [168840.0, 5388300.0],
                        [168870.0, 5388300.0],
                        [168870.0, 5388270.0],
                        [168840.0, 5388270.0],
                    ]
                ],
            },
        },
        1178: {
            "attributes": {"g_id": (5628, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388300.0],
                        [168840.0, 5388330.0],
                        [168870.0, 5388330.0],
                        [168870.0, 5388300.0],
                        [168840.0, 5388300.0],
                    ]
                ],
            },
        },
        1179: {
            "attributes": {"g_id": (5628, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388330.0],
                        [168840.0, 5388360.0],
                        [168870.0, 5388360.0],
                        [168870.0, 5388330.0],
                        [168840.0, 5388330.0],
                    ]
                ],
            },
        },
        1180: {
            "attributes": {"g_id": (5628, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388360.0],
                        [168840.0, 5388390.0],
                        [168870.0, 5388390.0],
                        [168870.0, 5388360.0],
                        [168840.0, 5388360.0],
                    ]
                ],
            },
        },
        1181: {
            "attributes": {"g_id": (5628, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388390.0],
                        [168840.0, 5388420.0],
                        [168870.0, 5388420.0],
                        [168870.0, 5388390.0],
                        [168840.0, 5388390.0],
                    ]
                ],
            },
        },
        1182: {
            "attributes": {"g_id": (5628, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388420.0],
                        [168840.0, 5388450.0],
                        [168870.0, 5388450.0],
                        [168870.0, 5388420.0],
                        [168840.0, 5388420.0],
                    ]
                ],
            },
        },
        1183: {
            "attributes": {"g_id": (5628, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388450.0],
                        [168840.0, 5388480.0],
                        [168870.0, 5388480.0],
                        [168870.0, 5388450.0],
                        [168840.0, 5388450.0],
                    ]
                ],
            },
        },
        1184: {
            "attributes": {"g_id": (5628, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388480.0],
                        [168840.0, 5388510.0],
                        [168870.0, 5388510.0],
                        [168870.0, 5388480.0],
                        [168840.0, 5388480.0],
                    ]
                ],
            },
        },
        1185: {
            "attributes": {"g_id": (5628, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388510.0],
                        [168840.0, 5388540.0],
                        [168870.0, 5388540.0],
                        [168870.0, 5388510.0],
                        [168840.0, 5388510.0],
                    ]
                ],
            },
        },
        1186: {
            "attributes": {"g_id": (5628, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388540.0],
                        [168840.0, 5388570.0],
                        [168870.0, 5388570.0],
                        [168870.0, 5388540.0],
                        [168840.0, 5388540.0],
                    ]
                ],
            },
        },
        1187: {
            "attributes": {"g_id": (5628, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388570.0],
                        [168840.0, 5388600.0],
                        [168870.0, 5388600.0],
                        [168870.0, 5388570.0],
                        [168840.0, 5388570.0],
                    ]
                ],
            },
        },
        1188: {
            "attributes": {"g_id": (5628, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388600.0],
                        [168840.0, 5388630.0],
                        [168870.0, 5388630.0],
                        [168870.0, 5388600.0],
                        [168840.0, 5388600.0],
                    ]
                ],
            },
        },
        1189: {
            "attributes": {"g_id": (5628, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388630.0],
                        [168840.0, 5388660.0],
                        [168870.0, 5388660.0],
                        [168870.0, 5388630.0],
                        [168840.0, 5388630.0],
                    ]
                ],
            },
        },
        1190: {
            "attributes": {"g_id": (5628, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388660.0],
                        [168840.0, 5388690.0],
                        [168870.0, 5388690.0],
                        [168870.0, 5388660.0],
                        [168840.0, 5388660.0],
                    ]
                ],
            },
        },
        1191: {
            "attributes": {"g_id": (5628, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388690.0],
                        [168840.0, 5388720.0],
                        [168870.0, 5388720.0],
                        [168870.0, 5388690.0],
                        [168840.0, 5388690.0],
                    ]
                ],
            },
        },
        1192: {
            "attributes": {"g_id": (5628, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388720.0],
                        [168840.0, 5388750.0],
                        [168870.0, 5388750.0],
                        [168870.0, 5388720.0],
                        [168840.0, 5388720.0],
                    ]
                ],
            },
        },
        1193: {
            "attributes": {"g_id": (5628, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388750.0],
                        [168840.0, 5388780.0],
                        [168870.0, 5388780.0],
                        [168870.0, 5388750.0],
                        [168840.0, 5388750.0],
                    ]
                ],
            },
        },
        1194: {
            "attributes": {"g_id": (5628, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388780.0],
                        [168840.0, 5388810.0],
                        [168870.0, 5388810.0],
                        [168870.0, 5388780.0],
                        [168840.0, 5388780.0],
                    ]
                ],
            },
        },
        1195: {
            "attributes": {"g_id": (5628, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388810.0],
                        [168840.0, 5388840.0],
                        [168870.0, 5388840.0],
                        [168870.0, 5388810.0],
                        [168840.0, 5388810.0],
                    ]
                ],
            },
        },
        1196: {
            "attributes": {"g_id": (5628, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388840.0],
                        [168840.0, 5388870.0],
                        [168870.0, 5388870.0],
                        [168870.0, 5388840.0],
                        [168840.0, 5388840.0],
                    ]
                ],
            },
        },
        1197: {
            "attributes": {"g_id": (5628, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388870.0],
                        [168840.0, 5388900.0],
                        [168870.0, 5388900.0],
                        [168870.0, 5388870.0],
                        [168840.0, 5388870.0],
                    ]
                ],
            },
        },
        1198: {
            "attributes": {"g_id": (5628, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388900.0],
                        [168840.0, 5388930.0],
                        [168870.0, 5388930.0],
                        [168870.0, 5388900.0],
                        [168840.0, 5388900.0],
                    ]
                ],
            },
        },
        1199: {
            "attributes": {"g_id": (5628, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168840.0, 5388930.0],
                        [168840.0, 5388960.0],
                        [168870.0, 5388960.0],
                        [168870.0, 5388930.0],
                        [168840.0, 5388930.0],
                    ]
                ],
            },
        },
        1200: {
            "attributes": {"g_id": (5629, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387520.0],
                        [168870.0, 5387550.0],
                        [168900.0, 5387550.0],
                        [168900.0, 5387520.0],
                        [168870.0, 5387520.0],
                    ]
                ],
            },
        },
        1201: {
            "attributes": {"g_id": (5629, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387550.0],
                        [168870.0, 5387580.0],
                        [168900.0, 5387580.0],
                        [168900.0, 5387550.0],
                        [168870.0, 5387550.0],
                    ]
                ],
            },
        },
        1202: {
            "attributes": {"g_id": (5629, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387580.0],
                        [168870.0, 5387610.0],
                        [168900.0, 5387610.0],
                        [168900.0, 5387580.0],
                        [168870.0, 5387580.0],
                    ]
                ],
            },
        },
        1203: {
            "attributes": {"g_id": (5629, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387610.0],
                        [168870.0, 5387640.0],
                        [168900.0, 5387640.0],
                        [168900.0, 5387610.0],
                        [168870.0, 5387610.0],
                    ]
                ],
            },
        },
        1204: {
            "attributes": {"g_id": (5629, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387640.0],
                        [168870.0, 5387670.0],
                        [168900.0, 5387670.0],
                        [168900.0, 5387640.0],
                        [168870.0, 5387640.0],
                    ]
                ],
            },
        },
        1205: {
            "attributes": {"g_id": (5629, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387670.0],
                        [168870.0, 5387700.0],
                        [168900.0, 5387700.0],
                        [168900.0, 5387670.0],
                        [168870.0, 5387670.0],
                    ]
                ],
            },
        },
        1206: {
            "attributes": {"g_id": (5629, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387700.0],
                        [168870.0, 5387730.0],
                        [168900.0, 5387730.0],
                        [168900.0, 5387700.0],
                        [168870.0, 5387700.0],
                    ]
                ],
            },
        },
        1207: {
            "attributes": {"g_id": (5629, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387730.0],
                        [168870.0, 5387760.0],
                        [168900.0, 5387760.0],
                        [168900.0, 5387730.0],
                        [168870.0, 5387730.0],
                    ]
                ],
            },
        },
        1208: {
            "attributes": {"g_id": (5629, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387760.0],
                        [168870.0, 5387790.0],
                        [168900.0, 5387790.0],
                        [168900.0, 5387760.0],
                        [168870.0, 5387760.0],
                    ]
                ],
            },
        },
        1209: {
            "attributes": {"g_id": (5629, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387790.0],
                        [168870.0, 5387820.0],
                        [168900.0, 5387820.0],
                        [168900.0, 5387790.0],
                        [168870.0, 5387790.0],
                    ]
                ],
            },
        },
        1210: {
            "attributes": {"g_id": (5629, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387820.0],
                        [168870.0, 5387850.0],
                        [168900.0, 5387850.0],
                        [168900.0, 5387820.0],
                        [168870.0, 5387820.0],
                    ]
                ],
            },
        },
        1211: {
            "attributes": {"g_id": (5629, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387850.0],
                        [168870.0, 5387880.0],
                        [168900.0, 5387880.0],
                        [168900.0, 5387850.0],
                        [168870.0, 5387850.0],
                    ]
                ],
            },
        },
        1212: {
            "attributes": {"g_id": (5629, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387880.0],
                        [168870.0, 5387910.0],
                        [168900.0, 5387910.0],
                        [168900.0, 5387880.0],
                        [168870.0, 5387880.0],
                    ]
                ],
            },
        },
        1213: {
            "attributes": {"g_id": (5629, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387910.0],
                        [168870.0, 5387940.0],
                        [168900.0, 5387940.0],
                        [168900.0, 5387910.0],
                        [168870.0, 5387910.0],
                    ]
                ],
            },
        },
        1214: {
            "attributes": {"g_id": (5629, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387940.0],
                        [168870.0, 5387970.0],
                        [168900.0, 5387970.0],
                        [168900.0, 5387940.0],
                        [168870.0, 5387940.0],
                    ]
                ],
            },
        },
        1215: {
            "attributes": {"g_id": (5629, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5387970.0],
                        [168870.0, 5388000.0],
                        [168900.0, 5388000.0],
                        [168900.0, 5387970.0],
                        [168870.0, 5387970.0],
                    ]
                ],
            },
        },
        1216: {
            "attributes": {"g_id": (5629, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388000.0],
                        [168870.0, 5388030.0],
                        [168900.0, 5388030.0],
                        [168900.0, 5388000.0],
                        [168870.0, 5388000.0],
                    ]
                ],
            },
        },
        1217: {
            "attributes": {"g_id": (5629, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388030.0],
                        [168870.0, 5388060.0],
                        [168900.0, 5388060.0],
                        [168900.0, 5388030.0],
                        [168870.0, 5388030.0],
                    ]
                ],
            },
        },
        1218: {
            "attributes": {"g_id": (5629, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388060.0],
                        [168870.0, 5388090.0],
                        [168900.0, 5388090.0],
                        [168900.0, 5388060.0],
                        [168870.0, 5388060.0],
                    ]
                ],
            },
        },
        1219: {
            "attributes": {"g_id": (5629, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388090.0],
                        [168870.0, 5388120.0],
                        [168900.0, 5388120.0],
                        [168900.0, 5388090.0],
                        [168870.0, 5388090.0],
                    ]
                ],
            },
        },
        1220: {
            "attributes": {"g_id": (5629, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388120.0],
                        [168870.0, 5388150.0],
                        [168900.0, 5388150.0],
                        [168900.0, 5388120.0],
                        [168870.0, 5388120.0],
                    ]
                ],
            },
        },
        1221: {
            "attributes": {"g_id": (5629, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388150.0],
                        [168870.0, 5388180.0],
                        [168900.0, 5388180.0],
                        [168900.0, 5388150.0],
                        [168870.0, 5388150.0],
                    ]
                ],
            },
        },
        1222: {
            "attributes": {"g_id": (5629, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388180.0],
                        [168870.0, 5388210.0],
                        [168900.0, 5388210.0],
                        [168900.0, 5388180.0],
                        [168870.0, 5388180.0],
                    ]
                ],
            },
        },
        1223: {
            "attributes": {"g_id": (5629, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388210.0],
                        [168870.0, 5388240.0],
                        [168900.0, 5388240.0],
                        [168900.0, 5388210.0],
                        [168870.0, 5388210.0],
                    ]
                ],
            },
        },
        1224: {
            "attributes": {"g_id": (5629, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388240.0],
                        [168870.0, 5388270.0],
                        [168900.0, 5388270.0],
                        [168900.0, 5388240.0],
                        [168870.0, 5388240.0],
                    ]
                ],
            },
        },
        1225: {
            "attributes": {"g_id": (5629, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388270.0],
                        [168870.0, 5388300.0],
                        [168900.0, 5388300.0],
                        [168900.0, 5388270.0],
                        [168870.0, 5388270.0],
                    ]
                ],
            },
        },
        1226: {
            "attributes": {"g_id": (5629, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388300.0],
                        [168870.0, 5388330.0],
                        [168900.0, 5388330.0],
                        [168900.0, 5388300.0],
                        [168870.0, 5388300.0],
                    ]
                ],
            },
        },
        1227: {
            "attributes": {"g_id": (5629, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388330.0],
                        [168870.0, 5388360.0],
                        [168900.0, 5388360.0],
                        [168900.0, 5388330.0],
                        [168870.0, 5388330.0],
                    ]
                ],
            },
        },
        1228: {
            "attributes": {"g_id": (5629, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388360.0],
                        [168870.0, 5388390.0],
                        [168900.0, 5388390.0],
                        [168900.0, 5388360.0],
                        [168870.0, 5388360.0],
                    ]
                ],
            },
        },
        1229: {
            "attributes": {"g_id": (5629, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388390.0],
                        [168870.0, 5388420.0],
                        [168900.0, 5388420.0],
                        [168900.0, 5388390.0],
                        [168870.0, 5388390.0],
                    ]
                ],
            },
        },
        1230: {
            "attributes": {"g_id": (5629, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388420.0],
                        [168870.0, 5388450.0],
                        [168900.0, 5388450.0],
                        [168900.0, 5388420.0],
                        [168870.0, 5388420.0],
                    ]
                ],
            },
        },
        1231: {
            "attributes": {"g_id": (5629, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388450.0],
                        [168870.0, 5388480.0],
                        [168900.0, 5388480.0],
                        [168900.0, 5388450.0],
                        [168870.0, 5388450.0],
                    ]
                ],
            },
        },
        1232: {
            "attributes": {"g_id": (5629, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388480.0],
                        [168870.0, 5388510.0],
                        [168900.0, 5388510.0],
                        [168900.0, 5388480.0],
                        [168870.0, 5388480.0],
                    ]
                ],
            },
        },
        1233: {
            "attributes": {"g_id": (5629, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388510.0],
                        [168870.0, 5388540.0],
                        [168900.0, 5388540.0],
                        [168900.0, 5388510.0],
                        [168870.0, 5388510.0],
                    ]
                ],
            },
        },
        1234: {
            "attributes": {"g_id": (5629, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388540.0],
                        [168870.0, 5388570.0],
                        [168900.0, 5388570.0],
                        [168900.0, 5388540.0],
                        [168870.0, 5388540.0],
                    ]
                ],
            },
        },
        1235: {
            "attributes": {"g_id": (5629, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388570.0],
                        [168870.0, 5388600.0],
                        [168900.0, 5388600.0],
                        [168900.0, 5388570.0],
                        [168870.0, 5388570.0],
                    ]
                ],
            },
        },
        1236: {
            "attributes": {"g_id": (5629, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388600.0],
                        [168870.0, 5388630.0],
                        [168900.0, 5388630.0],
                        [168900.0, 5388600.0],
                        [168870.0, 5388600.0],
                    ]
                ],
            },
        },
        1237: {
            "attributes": {"g_id": (5629, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388630.0],
                        [168870.0, 5388660.0],
                        [168900.0, 5388660.0],
                        [168900.0, 5388630.0],
                        [168870.0, 5388630.0],
                    ]
                ],
            },
        },
        1238: {
            "attributes": {"g_id": (5629, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388660.0],
                        [168870.0, 5388690.0],
                        [168900.0, 5388690.0],
                        [168900.0, 5388660.0],
                        [168870.0, 5388660.0],
                    ]
                ],
            },
        },
        1239: {
            "attributes": {"g_id": (5629, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388690.0],
                        [168870.0, 5388720.0],
                        [168900.0, 5388720.0],
                        [168900.0, 5388690.0],
                        [168870.0, 5388690.0],
                    ]
                ],
            },
        },
        1240: {
            "attributes": {"g_id": (5629, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388720.0],
                        [168870.0, 5388750.0],
                        [168900.0, 5388750.0],
                        [168900.0, 5388720.0],
                        [168870.0, 5388720.0],
                    ]
                ],
            },
        },
        1241: {
            "attributes": {"g_id": (5629, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388750.0],
                        [168870.0, 5388780.0],
                        [168900.0, 5388780.0],
                        [168900.0, 5388750.0],
                        [168870.0, 5388750.0],
                    ]
                ],
            },
        },
        1242: {
            "attributes": {"g_id": (5629, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388780.0],
                        [168870.0, 5388810.0],
                        [168900.0, 5388810.0],
                        [168900.0, 5388780.0],
                        [168870.0, 5388780.0],
                    ]
                ],
            },
        },
        1243: {
            "attributes": {"g_id": (5629, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388810.0],
                        [168870.0, 5388840.0],
                        [168900.0, 5388840.0],
                        [168900.0, 5388810.0],
                        [168870.0, 5388810.0],
                    ]
                ],
            },
        },
        1244: {
            "attributes": {"g_id": (5629, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388840.0],
                        [168870.0, 5388870.0],
                        [168900.0, 5388870.0],
                        [168900.0, 5388840.0],
                        [168870.0, 5388840.0],
                    ]
                ],
            },
        },
        1245: {
            "attributes": {"g_id": (5629, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388870.0],
                        [168870.0, 5388900.0],
                        [168900.0, 5388900.0],
                        [168900.0, 5388870.0],
                        [168870.0, 5388870.0],
                    ]
                ],
            },
        },
        1246: {
            "attributes": {"g_id": (5629, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388900.0],
                        [168870.0, 5388930.0],
                        [168900.0, 5388930.0],
                        [168900.0, 5388900.0],
                        [168870.0, 5388900.0],
                    ]
                ],
            },
        },
        1247: {
            "attributes": {"g_id": (5629, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168870.0, 5388930.0],
                        [168870.0, 5388960.0],
                        [168900.0, 5388960.0],
                        [168900.0, 5388930.0],
                        [168870.0, 5388930.0],
                    ]
                ],
            },
        },
        1248: {
            "attributes": {"g_id": (5630, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387520.0],
                        [168900.0, 5387550.0],
                        [168930.0, 5387550.0],
                        [168930.0, 5387520.0],
                        [168900.0, 5387520.0],
                    ]
                ],
            },
        },
        1249: {
            "attributes": {"g_id": (5630, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387550.0],
                        [168900.0, 5387580.0],
                        [168930.0, 5387580.0],
                        [168930.0, 5387550.0],
                        [168900.0, 5387550.0],
                    ]
                ],
            },
        },
        1250: {
            "attributes": {"g_id": (5630, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387580.0],
                        [168900.0, 5387610.0],
                        [168930.0, 5387610.0],
                        [168930.0, 5387580.0],
                        [168900.0, 5387580.0],
                    ]
                ],
            },
        },
        1251: {
            "attributes": {"g_id": (5630, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387610.0],
                        [168900.0, 5387640.0],
                        [168930.0, 5387640.0],
                        [168930.0, 5387610.0],
                        [168900.0, 5387610.0],
                    ]
                ],
            },
        },
        1252: {
            "attributes": {"g_id": (5630, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387640.0],
                        [168900.0, 5387670.0],
                        [168930.0, 5387670.0],
                        [168930.0, 5387640.0],
                        [168900.0, 5387640.0],
                    ]
                ],
            },
        },
        1253: {
            "attributes": {"g_id": (5630, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387670.0],
                        [168900.0, 5387700.0],
                        [168930.0, 5387700.0],
                        [168930.0, 5387670.0],
                        [168900.0, 5387670.0],
                    ]
                ],
            },
        },
        1254: {
            "attributes": {"g_id": (5630, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387700.0],
                        [168900.0, 5387730.0],
                        [168930.0, 5387730.0],
                        [168930.0, 5387700.0],
                        [168900.0, 5387700.0],
                    ]
                ],
            },
        },
        1255: {
            "attributes": {"g_id": (5630, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387730.0],
                        [168900.0, 5387760.0],
                        [168930.0, 5387760.0],
                        [168930.0, 5387730.0],
                        [168900.0, 5387730.0],
                    ]
                ],
            },
        },
        1256: {
            "attributes": {"g_id": (5630, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387760.0],
                        [168900.0, 5387790.0],
                        [168930.0, 5387790.0],
                        [168930.0, 5387760.0],
                        [168900.0, 5387760.0],
                    ]
                ],
            },
        },
        1257: {
            "attributes": {"g_id": (5630, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387790.0],
                        [168900.0, 5387820.0],
                        [168930.0, 5387820.0],
                        [168930.0, 5387790.0],
                        [168900.0, 5387790.0],
                    ]
                ],
            },
        },
        1258: {
            "attributes": {"g_id": (5630, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387820.0],
                        [168900.0, 5387850.0],
                        [168930.0, 5387850.0],
                        [168930.0, 5387820.0],
                        [168900.0, 5387820.0],
                    ]
                ],
            },
        },
        1259: {
            "attributes": {"g_id": (5630, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387850.0],
                        [168900.0, 5387880.0],
                        [168930.0, 5387880.0],
                        [168930.0, 5387850.0],
                        [168900.0, 5387850.0],
                    ]
                ],
            },
        },
        1260: {
            "attributes": {"g_id": (5630, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387880.0],
                        [168900.0, 5387910.0],
                        [168930.0, 5387910.0],
                        [168930.0, 5387880.0],
                        [168900.0, 5387880.0],
                    ]
                ],
            },
        },
        1261: {
            "attributes": {"g_id": (5630, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387910.0],
                        [168900.0, 5387940.0],
                        [168930.0, 5387940.0],
                        [168930.0, 5387910.0],
                        [168900.0, 5387910.0],
                    ]
                ],
            },
        },
        1262: {
            "attributes": {"g_id": (5630, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387940.0],
                        [168900.0, 5387970.0],
                        [168930.0, 5387970.0],
                        [168930.0, 5387940.0],
                        [168900.0, 5387940.0],
                    ]
                ],
            },
        },
        1263: {
            "attributes": {"g_id": (5630, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5387970.0],
                        [168900.0, 5388000.0],
                        [168930.0, 5388000.0],
                        [168930.0, 5387970.0],
                        [168900.0, 5387970.0],
                    ]
                ],
            },
        },
        1264: {
            "attributes": {"g_id": (5630, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388000.0],
                        [168900.0, 5388030.0],
                        [168930.0, 5388030.0],
                        [168930.0, 5388000.0],
                        [168900.0, 5388000.0],
                    ]
                ],
            },
        },
        1265: {
            "attributes": {"g_id": (5630, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388030.0],
                        [168900.0, 5388060.0],
                        [168930.0, 5388060.0],
                        [168930.0, 5388030.0],
                        [168900.0, 5388030.0],
                    ]
                ],
            },
        },
        1266: {
            "attributes": {"g_id": (5630, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388060.0],
                        [168900.0, 5388090.0],
                        [168930.0, 5388090.0],
                        [168930.0, 5388060.0],
                        [168900.0, 5388060.0],
                    ]
                ],
            },
        },
        1267: {
            "attributes": {"g_id": (5630, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388090.0],
                        [168900.0, 5388120.0],
                        [168930.0, 5388120.0],
                        [168930.0, 5388090.0],
                        [168900.0, 5388090.0],
                    ]
                ],
            },
        },
        1268: {
            "attributes": {"g_id": (5630, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388120.0],
                        [168900.0, 5388150.0],
                        [168930.0, 5388150.0],
                        [168930.0, 5388120.0],
                        [168900.0, 5388120.0],
                    ]
                ],
            },
        },
        1269: {
            "attributes": {"g_id": (5630, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388150.0],
                        [168900.0, 5388180.0],
                        [168930.0, 5388180.0],
                        [168930.0, 5388150.0],
                        [168900.0, 5388150.0],
                    ]
                ],
            },
        },
        1270: {
            "attributes": {"g_id": (5630, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388180.0],
                        [168900.0, 5388210.0],
                        [168930.0, 5388210.0],
                        [168930.0, 5388180.0],
                        [168900.0, 5388180.0],
                    ]
                ],
            },
        },
        1271: {
            "attributes": {"g_id": (5630, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388210.0],
                        [168900.0, 5388240.0],
                        [168930.0, 5388240.0],
                        [168930.0, 5388210.0],
                        [168900.0, 5388210.0],
                    ]
                ],
            },
        },
        1272: {
            "attributes": {"g_id": (5630, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388240.0],
                        [168900.0, 5388270.0],
                        [168930.0, 5388270.0],
                        [168930.0, 5388240.0],
                        [168900.0, 5388240.0],
                    ]
                ],
            },
        },
        1273: {
            "attributes": {"g_id": (5630, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388270.0],
                        [168900.0, 5388300.0],
                        [168930.0, 5388300.0],
                        [168930.0, 5388270.0],
                        [168900.0, 5388270.0],
                    ]
                ],
            },
        },
        1274: {
            "attributes": {"g_id": (5630, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388300.0],
                        [168900.0, 5388330.0],
                        [168930.0, 5388330.0],
                        [168930.0, 5388300.0],
                        [168900.0, 5388300.0],
                    ]
                ],
            },
        },
        1275: {
            "attributes": {"g_id": (5630, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388330.0],
                        [168900.0, 5388360.0],
                        [168930.0, 5388360.0],
                        [168930.0, 5388330.0],
                        [168900.0, 5388330.0],
                    ]
                ],
            },
        },
        1276: {
            "attributes": {"g_id": (5630, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388360.0],
                        [168900.0, 5388390.0],
                        [168930.0, 5388390.0],
                        [168930.0, 5388360.0],
                        [168900.0, 5388360.0],
                    ]
                ],
            },
        },
        1277: {
            "attributes": {"g_id": (5630, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388390.0],
                        [168900.0, 5388420.0],
                        [168930.0, 5388420.0],
                        [168930.0, 5388390.0],
                        [168900.0, 5388390.0],
                    ]
                ],
            },
        },
        1278: {
            "attributes": {"g_id": (5630, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388420.0],
                        [168900.0, 5388450.0],
                        [168930.0, 5388450.0],
                        [168930.0, 5388420.0],
                        [168900.0, 5388420.0],
                    ]
                ],
            },
        },
        1279: {
            "attributes": {"g_id": (5630, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388450.0],
                        [168900.0, 5388480.0],
                        [168930.0, 5388480.0],
                        [168930.0, 5388450.0],
                        [168900.0, 5388450.0],
                    ]
                ],
            },
        },
        1280: {
            "attributes": {"g_id": (5630, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388480.0],
                        [168900.0, 5388510.0],
                        [168930.0, 5388510.0],
                        [168930.0, 5388480.0],
                        [168900.0, 5388480.0],
                    ]
                ],
            },
        },
        1281: {
            "attributes": {"g_id": (5630, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388510.0],
                        [168900.0, 5388540.0],
                        [168930.0, 5388540.0],
                        [168930.0, 5388510.0],
                        [168900.0, 5388510.0],
                    ]
                ],
            },
        },
        1282: {
            "attributes": {"g_id": (5630, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388540.0],
                        [168900.0, 5388570.0],
                        [168930.0, 5388570.0],
                        [168930.0, 5388540.0],
                        [168900.0, 5388540.0],
                    ]
                ],
            },
        },
        1283: {
            "attributes": {"g_id": (5630, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388570.0],
                        [168900.0, 5388600.0],
                        [168930.0, 5388600.0],
                        [168930.0, 5388570.0],
                        [168900.0, 5388570.0],
                    ]
                ],
            },
        },
        1284: {
            "attributes": {"g_id": (5630, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388600.0],
                        [168900.0, 5388630.0],
                        [168930.0, 5388630.0],
                        [168930.0, 5388600.0],
                        [168900.0, 5388600.0],
                    ]
                ],
            },
        },
        1285: {
            "attributes": {"g_id": (5630, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388630.0],
                        [168900.0, 5388660.0],
                        [168930.0, 5388660.0],
                        [168930.0, 5388630.0],
                        [168900.0, 5388630.0],
                    ]
                ],
            },
        },
        1286: {
            "attributes": {"g_id": (5630, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388660.0],
                        [168900.0, 5388690.0],
                        [168930.0, 5388690.0],
                        [168930.0, 5388660.0],
                        [168900.0, 5388660.0],
                    ]
                ],
            },
        },
        1287: {
            "attributes": {"g_id": (5630, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388690.0],
                        [168900.0, 5388720.0],
                        [168930.0, 5388720.0],
                        [168930.0, 5388690.0],
                        [168900.0, 5388690.0],
                    ]
                ],
            },
        },
        1288: {
            "attributes": {"g_id": (5630, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388720.0],
                        [168900.0, 5388750.0],
                        [168930.0, 5388750.0],
                        [168930.0, 5388720.0],
                        [168900.0, 5388720.0],
                    ]
                ],
            },
        },
        1289: {
            "attributes": {"g_id": (5630, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388750.0],
                        [168900.0, 5388780.0],
                        [168930.0, 5388780.0],
                        [168930.0, 5388750.0],
                        [168900.0, 5388750.0],
                    ]
                ],
            },
        },
        1290: {
            "attributes": {"g_id": (5630, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388780.0],
                        [168900.0, 5388810.0],
                        [168930.0, 5388810.0],
                        [168930.0, 5388780.0],
                        [168900.0, 5388780.0],
                    ]
                ],
            },
        },
        1291: {
            "attributes": {"g_id": (5630, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388810.0],
                        [168900.0, 5388840.0],
                        [168930.0, 5388840.0],
                        [168930.0, 5388810.0],
                        [168900.0, 5388810.0],
                    ]
                ],
            },
        },
        1292: {
            "attributes": {"g_id": (5630, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388840.0],
                        [168900.0, 5388870.0],
                        [168930.0, 5388870.0],
                        [168930.0, 5388840.0],
                        [168900.0, 5388840.0],
                    ]
                ],
            },
        },
        1293: {
            "attributes": {"g_id": (5630, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388870.0],
                        [168900.0, 5388900.0],
                        [168930.0, 5388900.0],
                        [168930.0, 5388870.0],
                        [168900.0, 5388870.0],
                    ]
                ],
            },
        },
        1294: {
            "attributes": {"g_id": (5630, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388900.0],
                        [168900.0, 5388930.0],
                        [168930.0, 5388930.0],
                        [168930.0, 5388900.0],
                        [168900.0, 5388900.0],
                    ]
                ],
            },
        },
        1295: {
            "attributes": {"g_id": (5630, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168900.0, 5388930.0],
                        [168900.0, 5388960.0],
                        [168930.0, 5388960.0],
                        [168930.0, 5388930.0],
                        [168900.0, 5388930.0],
                    ]
                ],
            },
        },
        1296: {
            "attributes": {"g_id": (5631, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387520.0],
                        [168930.0, 5387550.0],
                        [168960.0, 5387550.0],
                        [168960.0, 5387520.0],
                        [168930.0, 5387520.0],
                    ]
                ],
            },
        },
        1297: {
            "attributes": {"g_id": (5631, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387550.0],
                        [168930.0, 5387580.0],
                        [168960.0, 5387580.0],
                        [168960.0, 5387550.0],
                        [168930.0, 5387550.0],
                    ]
                ],
            },
        },
        1298: {
            "attributes": {"g_id": (5631, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387580.0],
                        [168930.0, 5387610.0],
                        [168960.0, 5387610.0],
                        [168960.0, 5387580.0],
                        [168930.0, 5387580.0],
                    ]
                ],
            },
        },
        1299: {
            "attributes": {"g_id": (5631, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387610.0],
                        [168930.0, 5387640.0],
                        [168960.0, 5387640.0],
                        [168960.0, 5387610.0],
                        [168930.0, 5387610.0],
                    ]
                ],
            },
        },
        1300: {
            "attributes": {"g_id": (5631, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387640.0],
                        [168930.0, 5387670.0],
                        [168960.0, 5387670.0],
                        [168960.0, 5387640.0],
                        [168930.0, 5387640.0],
                    ]
                ],
            },
        },
        1301: {
            "attributes": {"g_id": (5631, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387670.0],
                        [168930.0, 5387700.0],
                        [168960.0, 5387700.0],
                        [168960.0, 5387670.0],
                        [168930.0, 5387670.0],
                    ]
                ],
            },
        },
        1302: {
            "attributes": {"g_id": (5631, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387700.0],
                        [168930.0, 5387730.0],
                        [168960.0, 5387730.0],
                        [168960.0, 5387700.0],
                        [168930.0, 5387700.0],
                    ]
                ],
            },
        },
        1303: {
            "attributes": {"g_id": (5631, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387730.0],
                        [168930.0, 5387760.0],
                        [168960.0, 5387760.0],
                        [168960.0, 5387730.0],
                        [168930.0, 5387730.0],
                    ]
                ],
            },
        },
        1304: {
            "attributes": {"g_id": (5631, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387760.0],
                        [168930.0, 5387790.0],
                        [168960.0, 5387790.0],
                        [168960.0, 5387760.0],
                        [168930.0, 5387760.0],
                    ]
                ],
            },
        },
        1305: {
            "attributes": {"g_id": (5631, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387790.0],
                        [168930.0, 5387820.0],
                        [168960.0, 5387820.0],
                        [168960.0, 5387790.0],
                        [168930.0, 5387790.0],
                    ]
                ],
            },
        },
        1306: {
            "attributes": {"g_id": (5631, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387820.0],
                        [168930.0, 5387850.0],
                        [168960.0, 5387850.0],
                        [168960.0, 5387820.0],
                        [168930.0, 5387820.0],
                    ]
                ],
            },
        },
        1307: {
            "attributes": {"g_id": (5631, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387850.0],
                        [168930.0, 5387880.0],
                        [168960.0, 5387880.0],
                        [168960.0, 5387850.0],
                        [168930.0, 5387850.0],
                    ]
                ],
            },
        },
        1308: {
            "attributes": {"g_id": (5631, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387880.0],
                        [168930.0, 5387910.0],
                        [168960.0, 5387910.0],
                        [168960.0, 5387880.0],
                        [168930.0, 5387880.0],
                    ]
                ],
            },
        },
        1309: {
            "attributes": {"g_id": (5631, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387910.0],
                        [168930.0, 5387940.0],
                        [168960.0, 5387940.0],
                        [168960.0, 5387910.0],
                        [168930.0, 5387910.0],
                    ]
                ],
            },
        },
        1310: {
            "attributes": {"g_id": (5631, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387940.0],
                        [168930.0, 5387970.0],
                        [168960.0, 5387970.0],
                        [168960.0, 5387940.0],
                        [168930.0, 5387940.0],
                    ]
                ],
            },
        },
        1311: {
            "attributes": {"g_id": (5631, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5387970.0],
                        [168930.0, 5388000.0],
                        [168960.0, 5388000.0],
                        [168960.0, 5387970.0],
                        [168930.0, 5387970.0],
                    ]
                ],
            },
        },
        1312: {
            "attributes": {"g_id": (5631, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388000.0],
                        [168930.0, 5388030.0],
                        [168960.0, 5388030.0],
                        [168960.0, 5388000.0],
                        [168930.0, 5388000.0],
                    ]
                ],
            },
        },
        1313: {
            "attributes": {"g_id": (5631, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388030.0],
                        [168930.0, 5388060.0],
                        [168960.0, 5388060.0],
                        [168960.0, 5388030.0],
                        [168930.0, 5388030.0],
                    ]
                ],
            },
        },
        1314: {
            "attributes": {"g_id": (5631, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388060.0],
                        [168930.0, 5388090.0],
                        [168960.0, 5388090.0],
                        [168960.0, 5388060.0],
                        [168930.0, 5388060.0],
                    ]
                ],
            },
        },
        1315: {
            "attributes": {"g_id": (5631, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388090.0],
                        [168930.0, 5388120.0],
                        [168960.0, 5388120.0],
                        [168960.0, 5388090.0],
                        [168930.0, 5388090.0],
                    ]
                ],
            },
        },
        1316: {
            "attributes": {"g_id": (5631, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388120.0],
                        [168930.0, 5388150.0],
                        [168960.0, 5388150.0],
                        [168960.0, 5388120.0],
                        [168930.0, 5388120.0],
                    ]
                ],
            },
        },
        1317: {
            "attributes": {"g_id": (5631, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388150.0],
                        [168930.0, 5388180.0],
                        [168960.0, 5388180.0],
                        [168960.0, 5388150.0],
                        [168930.0, 5388150.0],
                    ]
                ],
            },
        },
        1318: {
            "attributes": {"g_id": (5631, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388180.0],
                        [168930.0, 5388210.0],
                        [168960.0, 5388210.0],
                        [168960.0, 5388180.0],
                        [168930.0, 5388180.0],
                    ]
                ],
            },
        },
        1319: {
            "attributes": {"g_id": (5631, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388210.0],
                        [168930.0, 5388240.0],
                        [168960.0, 5388240.0],
                        [168960.0, 5388210.0],
                        [168930.0, 5388210.0],
                    ]
                ],
            },
        },
        1320: {
            "attributes": {"g_id": (5631, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388240.0],
                        [168930.0, 5388270.0],
                        [168960.0, 5388270.0],
                        [168960.0, 5388240.0],
                        [168930.0, 5388240.0],
                    ]
                ],
            },
        },
        1321: {
            "attributes": {"g_id": (5631, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388270.0],
                        [168930.0, 5388300.0],
                        [168960.0, 5388300.0],
                        [168960.0, 5388270.0],
                        [168930.0, 5388270.0],
                    ]
                ],
            },
        },
        1322: {
            "attributes": {"g_id": (5631, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388300.0],
                        [168930.0, 5388330.0],
                        [168960.0, 5388330.0],
                        [168960.0, 5388300.0],
                        [168930.0, 5388300.0],
                    ]
                ],
            },
        },
        1323: {
            "attributes": {"g_id": (5631, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388330.0],
                        [168930.0, 5388360.0],
                        [168960.0, 5388360.0],
                        [168960.0, 5388330.0],
                        [168930.0, 5388330.0],
                    ]
                ],
            },
        },
        1324: {
            "attributes": {"g_id": (5631, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388360.0],
                        [168930.0, 5388390.0],
                        [168960.0, 5388390.0],
                        [168960.0, 5388360.0],
                        [168930.0, 5388360.0],
                    ]
                ],
            },
        },
        1325: {
            "attributes": {"g_id": (5631, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388390.0],
                        [168930.0, 5388420.0],
                        [168960.0, 5388420.0],
                        [168960.0, 5388390.0],
                        [168930.0, 5388390.0],
                    ]
                ],
            },
        },
        1326: {
            "attributes": {"g_id": (5631, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388420.0],
                        [168930.0, 5388450.0],
                        [168960.0, 5388450.0],
                        [168960.0, 5388420.0],
                        [168930.0, 5388420.0],
                    ]
                ],
            },
        },
        1327: {
            "attributes": {"g_id": (5631, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388450.0],
                        [168930.0, 5388480.0],
                        [168960.0, 5388480.0],
                        [168960.0, 5388450.0],
                        [168930.0, 5388450.0],
                    ]
                ],
            },
        },
        1328: {
            "attributes": {"g_id": (5631, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388480.0],
                        [168930.0, 5388510.0],
                        [168960.0, 5388510.0],
                        [168960.0, 5388480.0],
                        [168930.0, 5388480.0],
                    ]
                ],
            },
        },
        1329: {
            "attributes": {"g_id": (5631, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388510.0],
                        [168930.0, 5388540.0],
                        [168960.0, 5388540.0],
                        [168960.0, 5388510.0],
                        [168930.0, 5388510.0],
                    ]
                ],
            },
        },
        1330: {
            "attributes": {"g_id": (5631, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388540.0],
                        [168930.0, 5388570.0],
                        [168960.0, 5388570.0],
                        [168960.0, 5388540.0],
                        [168930.0, 5388540.0],
                    ]
                ],
            },
        },
        1331: {
            "attributes": {"g_id": (5631, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388570.0],
                        [168930.0, 5388600.0],
                        [168960.0, 5388600.0],
                        [168960.0, 5388570.0],
                        [168930.0, 5388570.0],
                    ]
                ],
            },
        },
        1332: {
            "attributes": {"g_id": (5631, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388600.0],
                        [168930.0, 5388630.0],
                        [168960.0, 5388630.0],
                        [168960.0, 5388600.0],
                        [168930.0, 5388600.0],
                    ]
                ],
            },
        },
        1333: {
            "attributes": {"g_id": (5631, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388630.0],
                        [168930.0, 5388660.0],
                        [168960.0, 5388660.0],
                        [168960.0, 5388630.0],
                        [168930.0, 5388630.0],
                    ]
                ],
            },
        },
        1334: {
            "attributes": {"g_id": (5631, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388660.0],
                        [168930.0, 5388690.0],
                        [168960.0, 5388690.0],
                        [168960.0, 5388660.0],
                        [168930.0, 5388660.0],
                    ]
                ],
            },
        },
        1335: {
            "attributes": {"g_id": (5631, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388690.0],
                        [168930.0, 5388720.0],
                        [168960.0, 5388720.0],
                        [168960.0, 5388690.0],
                        [168930.0, 5388690.0],
                    ]
                ],
            },
        },
        1336: {
            "attributes": {"g_id": (5631, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388720.0],
                        [168930.0, 5388750.0],
                        [168960.0, 5388750.0],
                        [168960.0, 5388720.0],
                        [168930.0, 5388720.0],
                    ]
                ],
            },
        },
        1337: {
            "attributes": {"g_id": (5631, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388750.0],
                        [168930.0, 5388780.0],
                        [168960.0, 5388780.0],
                        [168960.0, 5388750.0],
                        [168930.0, 5388750.0],
                    ]
                ],
            },
        },
        1338: {
            "attributes": {"g_id": (5631, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388780.0],
                        [168930.0, 5388810.0],
                        [168960.0, 5388810.0],
                        [168960.0, 5388780.0],
                        [168930.0, 5388780.0],
                    ]
                ],
            },
        },
        1339: {
            "attributes": {"g_id": (5631, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388810.0],
                        [168930.0, 5388840.0],
                        [168960.0, 5388840.0],
                        [168960.0, 5388810.0],
                        [168930.0, 5388810.0],
                    ]
                ],
            },
        },
        1340: {
            "attributes": {"g_id": (5631, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388840.0],
                        [168930.0, 5388870.0],
                        [168960.0, 5388870.0],
                        [168960.0, 5388840.0],
                        [168930.0, 5388840.0],
                    ]
                ],
            },
        },
        1341: {
            "attributes": {"g_id": (5631, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388870.0],
                        [168930.0, 5388900.0],
                        [168960.0, 5388900.0],
                        [168960.0, 5388870.0],
                        [168930.0, 5388870.0],
                    ]
                ],
            },
        },
        1342: {
            "attributes": {"g_id": (5631, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388900.0],
                        [168930.0, 5388930.0],
                        [168960.0, 5388930.0],
                        [168960.0, 5388900.0],
                        [168930.0, 5388900.0],
                    ]
                ],
            },
        },
        1343: {
            "attributes": {"g_id": (5631, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168930.0, 5388930.0],
                        [168930.0, 5388960.0],
                        [168960.0, 5388960.0],
                        [168960.0, 5388930.0],
                        [168930.0, 5388930.0],
                    ]
                ],
            },
        },
        1344: {
            "attributes": {"g_id": (5632, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387520.0],
                        [168960.0, 5387550.0],
                        [168990.0, 5387550.0],
                        [168990.0, 5387520.0],
                        [168960.0, 5387520.0],
                    ]
                ],
            },
        },
        1345: {
            "attributes": {"g_id": (5632, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387550.0],
                        [168960.0, 5387580.0],
                        [168990.0, 5387580.0],
                        [168990.0, 5387550.0],
                        [168960.0, 5387550.0],
                    ]
                ],
            },
        },
        1346: {
            "attributes": {"g_id": (5632, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387580.0],
                        [168960.0, 5387610.0],
                        [168990.0, 5387610.0],
                        [168990.0, 5387580.0],
                        [168960.0, 5387580.0],
                    ]
                ],
            },
        },
        1347: {
            "attributes": {"g_id": (5632, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387610.0],
                        [168960.0, 5387640.0],
                        [168990.0, 5387640.0],
                        [168990.0, 5387610.0],
                        [168960.0, 5387610.0],
                    ]
                ],
            },
        },
        1348: {
            "attributes": {"g_id": (5632, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387640.0],
                        [168960.0, 5387670.0],
                        [168990.0, 5387670.0],
                        [168990.0, 5387640.0],
                        [168960.0, 5387640.0],
                    ]
                ],
            },
        },
        1349: {
            "attributes": {"g_id": (5632, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387670.0],
                        [168960.0, 5387700.0],
                        [168990.0, 5387700.0],
                        [168990.0, 5387670.0],
                        [168960.0, 5387670.0],
                    ]
                ],
            },
        },
        1350: {
            "attributes": {"g_id": (5632, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387700.0],
                        [168960.0, 5387730.0],
                        [168990.0, 5387730.0],
                        [168990.0, 5387700.0],
                        [168960.0, 5387700.0],
                    ]
                ],
            },
        },
        1351: {
            "attributes": {"g_id": (5632, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387730.0],
                        [168960.0, 5387760.0],
                        [168990.0, 5387760.0],
                        [168990.0, 5387730.0],
                        [168960.0, 5387730.0],
                    ]
                ],
            },
        },
        1352: {
            "attributes": {"g_id": (5632, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387760.0],
                        [168960.0, 5387790.0],
                        [168990.0, 5387790.0],
                        [168990.0, 5387760.0],
                        [168960.0, 5387760.0],
                    ]
                ],
            },
        },
        1353: {
            "attributes": {"g_id": (5632, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387790.0],
                        [168960.0, 5387820.0],
                        [168990.0, 5387820.0],
                        [168990.0, 5387790.0],
                        [168960.0, 5387790.0],
                    ]
                ],
            },
        },
        1354: {
            "attributes": {"g_id": (5632, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387820.0],
                        [168960.0, 5387850.0],
                        [168990.0, 5387850.0],
                        [168990.0, 5387820.0],
                        [168960.0, 5387820.0],
                    ]
                ],
            },
        },
        1355: {
            "attributes": {"g_id": (5632, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387850.0],
                        [168960.0, 5387880.0],
                        [168990.0, 5387880.0],
                        [168990.0, 5387850.0],
                        [168960.0, 5387850.0],
                    ]
                ],
            },
        },
        1356: {
            "attributes": {"g_id": (5632, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387880.0],
                        [168960.0, 5387910.0],
                        [168990.0, 5387910.0],
                        [168990.0, 5387880.0],
                        [168960.0, 5387880.0],
                    ]
                ],
            },
        },
        1357: {
            "attributes": {"g_id": (5632, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387910.0],
                        [168960.0, 5387940.0],
                        [168990.0, 5387940.0],
                        [168990.0, 5387910.0],
                        [168960.0, 5387910.0],
                    ]
                ],
            },
        },
        1358: {
            "attributes": {"g_id": (5632, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387940.0],
                        [168960.0, 5387970.0],
                        [168990.0, 5387970.0],
                        [168990.0, 5387940.0],
                        [168960.0, 5387940.0],
                    ]
                ],
            },
        },
        1359: {
            "attributes": {"g_id": (5632, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5387970.0],
                        [168960.0, 5388000.0],
                        [168990.0, 5388000.0],
                        [168990.0, 5387970.0],
                        [168960.0, 5387970.0],
                    ]
                ],
            },
        },
        1360: {
            "attributes": {"g_id": (5632, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388000.0],
                        [168960.0, 5388030.0],
                        [168990.0, 5388030.0],
                        [168990.0, 5388000.0],
                        [168960.0, 5388000.0],
                    ]
                ],
            },
        },
        1361: {
            "attributes": {"g_id": (5632, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388030.0],
                        [168960.0, 5388060.0],
                        [168990.0, 5388060.0],
                        [168990.0, 5388030.0],
                        [168960.0, 5388030.0],
                    ]
                ],
            },
        },
        1362: {
            "attributes": {"g_id": (5632, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388060.0],
                        [168960.0, 5388090.0],
                        [168990.0, 5388090.0],
                        [168990.0, 5388060.0],
                        [168960.0, 5388060.0],
                    ]
                ],
            },
        },
        1363: {
            "attributes": {"g_id": (5632, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388090.0],
                        [168960.0, 5388120.0],
                        [168990.0, 5388120.0],
                        [168990.0, 5388090.0],
                        [168960.0, 5388090.0],
                    ]
                ],
            },
        },
        1364: {
            "attributes": {"g_id": (5632, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388120.0],
                        [168960.0, 5388150.0],
                        [168990.0, 5388150.0],
                        [168990.0, 5388120.0],
                        [168960.0, 5388120.0],
                    ]
                ],
            },
        },
        1365: {
            "attributes": {"g_id": (5632, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388150.0],
                        [168960.0, 5388180.0],
                        [168990.0, 5388180.0],
                        [168990.0, 5388150.0],
                        [168960.0, 5388150.0],
                    ]
                ],
            },
        },
        1366: {
            "attributes": {"g_id": (5632, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388180.0],
                        [168960.0, 5388210.0],
                        [168990.0, 5388210.0],
                        [168990.0, 5388180.0],
                        [168960.0, 5388180.0],
                    ]
                ],
            },
        },
        1367: {
            "attributes": {"g_id": (5632, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388210.0],
                        [168960.0, 5388240.0],
                        [168990.0, 5388240.0],
                        [168990.0, 5388210.0],
                        [168960.0, 5388210.0],
                    ]
                ],
            },
        },
        1368: {
            "attributes": {"g_id": (5632, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388240.0],
                        [168960.0, 5388270.0],
                        [168990.0, 5388270.0],
                        [168990.0, 5388240.0],
                        [168960.0, 5388240.0],
                    ]
                ],
            },
        },
        1369: {
            "attributes": {"g_id": (5632, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388270.0],
                        [168960.0, 5388300.0],
                        [168990.0, 5388300.0],
                        [168990.0, 5388270.0],
                        [168960.0, 5388270.0],
                    ]
                ],
            },
        },
        1370: {
            "attributes": {"g_id": (5632, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388300.0],
                        [168960.0, 5388330.0],
                        [168990.0, 5388330.0],
                        [168990.0, 5388300.0],
                        [168960.0, 5388300.0],
                    ]
                ],
            },
        },
        1371: {
            "attributes": {"g_id": (5632, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388330.0],
                        [168960.0, 5388360.0],
                        [168990.0, 5388360.0],
                        [168990.0, 5388330.0],
                        [168960.0, 5388330.0],
                    ]
                ],
            },
        },
        1372: {
            "attributes": {"g_id": (5632, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388360.0],
                        [168960.0, 5388390.0],
                        [168990.0, 5388390.0],
                        [168990.0, 5388360.0],
                        [168960.0, 5388360.0],
                    ]
                ],
            },
        },
        1373: {
            "attributes": {"g_id": (5632, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388390.0],
                        [168960.0, 5388420.0],
                        [168990.0, 5388420.0],
                        [168990.0, 5388390.0],
                        [168960.0, 5388390.0],
                    ]
                ],
            },
        },
        1374: {
            "attributes": {"g_id": (5632, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388420.0],
                        [168960.0, 5388450.0],
                        [168990.0, 5388450.0],
                        [168990.0, 5388420.0],
                        [168960.0, 5388420.0],
                    ]
                ],
            },
        },
        1375: {
            "attributes": {"g_id": (5632, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388450.0],
                        [168960.0, 5388480.0],
                        [168990.0, 5388480.0],
                        [168990.0, 5388450.0],
                        [168960.0, 5388450.0],
                    ]
                ],
            },
        },
        1376: {
            "attributes": {"g_id": (5632, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388480.0],
                        [168960.0, 5388510.0],
                        [168990.0, 5388510.0],
                        [168990.0, 5388480.0],
                        [168960.0, 5388480.0],
                    ]
                ],
            },
        },
        1377: {
            "attributes": {"g_id": (5632, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388510.0],
                        [168960.0, 5388540.0],
                        [168990.0, 5388540.0],
                        [168990.0, 5388510.0],
                        [168960.0, 5388510.0],
                    ]
                ],
            },
        },
        1378: {
            "attributes": {"g_id": (5632, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388540.0],
                        [168960.0, 5388570.0],
                        [168990.0, 5388570.0],
                        [168990.0, 5388540.0],
                        [168960.0, 5388540.0],
                    ]
                ],
            },
        },
        1379: {
            "attributes": {"g_id": (5632, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388570.0],
                        [168960.0, 5388600.0],
                        [168990.0, 5388600.0],
                        [168990.0, 5388570.0],
                        [168960.0, 5388570.0],
                    ]
                ],
            },
        },
        1380: {
            "attributes": {"g_id": (5632, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388600.0],
                        [168960.0, 5388630.0],
                        [168990.0, 5388630.0],
                        [168990.0, 5388600.0],
                        [168960.0, 5388600.0],
                    ]
                ],
            },
        },
        1381: {
            "attributes": {"g_id": (5632, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388630.0],
                        [168960.0, 5388660.0],
                        [168990.0, 5388660.0],
                        [168990.0, 5388630.0],
                        [168960.0, 5388630.0],
                    ]
                ],
            },
        },
        1382: {
            "attributes": {"g_id": (5632, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388660.0],
                        [168960.0, 5388690.0],
                        [168990.0, 5388690.0],
                        [168990.0, 5388660.0],
                        [168960.0, 5388660.0],
                    ]
                ],
            },
        },
        1383: {
            "attributes": {"g_id": (5632, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388690.0],
                        [168960.0, 5388720.0],
                        [168990.0, 5388720.0],
                        [168990.0, 5388690.0],
                        [168960.0, 5388690.0],
                    ]
                ],
            },
        },
        1384: {
            "attributes": {"g_id": (5632, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388720.0],
                        [168960.0, 5388750.0],
                        [168990.0, 5388750.0],
                        [168990.0, 5388720.0],
                        [168960.0, 5388720.0],
                    ]
                ],
            },
        },
        1385: {
            "attributes": {"g_id": (5632, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388750.0],
                        [168960.0, 5388780.0],
                        [168990.0, 5388780.0],
                        [168990.0, 5388750.0],
                        [168960.0, 5388750.0],
                    ]
                ],
            },
        },
        1386: {
            "attributes": {"g_id": (5632, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388780.0],
                        [168960.0, 5388810.0],
                        [168990.0, 5388810.0],
                        [168990.0, 5388780.0],
                        [168960.0, 5388780.0],
                    ]
                ],
            },
        },
        1387: {
            "attributes": {"g_id": (5632, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388810.0],
                        [168960.0, 5388840.0],
                        [168990.0, 5388840.0],
                        [168990.0, 5388810.0],
                        [168960.0, 5388810.0],
                    ]
                ],
            },
        },
        1388: {
            "attributes": {"g_id": (5632, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388840.0],
                        [168960.0, 5388870.0],
                        [168990.0, 5388870.0],
                        [168990.0, 5388840.0],
                        [168960.0, 5388840.0],
                    ]
                ],
            },
        },
        1389: {
            "attributes": {"g_id": (5632, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388870.0],
                        [168960.0, 5388900.0],
                        [168990.0, 5388900.0],
                        [168990.0, 5388870.0],
                        [168960.0, 5388870.0],
                    ]
                ],
            },
        },
        1390: {
            "attributes": {"g_id": (5632, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388900.0],
                        [168960.0, 5388930.0],
                        [168990.0, 5388930.0],
                        [168990.0, 5388900.0],
                        [168960.0, 5388900.0],
                    ]
                ],
            },
        },
        1391: {
            "attributes": {"g_id": (5632, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168960.0, 5388930.0],
                        [168960.0, 5388960.0],
                        [168990.0, 5388960.0],
                        [168990.0, 5388930.0],
                        [168960.0, 5388930.0],
                    ]
                ],
            },
        },
        1392: {
            "attributes": {"g_id": (5633, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387520.0],
                        [168990.0, 5387550.0],
                        [169020.0, 5387550.0],
                        [169020.0, 5387520.0],
                        [168990.0, 5387520.0],
                    ]
                ],
            },
        },
        1393: {
            "attributes": {"g_id": (5633, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387550.0],
                        [168990.0, 5387580.0],
                        [169020.0, 5387580.0],
                        [169020.0, 5387550.0],
                        [168990.0, 5387550.0],
                    ]
                ],
            },
        },
        1394: {
            "attributes": {"g_id": (5633, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387580.0],
                        [168990.0, 5387610.0],
                        [169020.0, 5387610.0],
                        [169020.0, 5387580.0],
                        [168990.0, 5387580.0],
                    ]
                ],
            },
        },
        1395: {
            "attributes": {"g_id": (5633, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387610.0],
                        [168990.0, 5387640.0],
                        [169020.0, 5387640.0],
                        [169020.0, 5387610.0],
                        [168990.0, 5387610.0],
                    ]
                ],
            },
        },
        1396: {
            "attributes": {"g_id": (5633, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387640.0],
                        [168990.0, 5387670.0],
                        [169020.0, 5387670.0],
                        [169020.0, 5387640.0],
                        [168990.0, 5387640.0],
                    ]
                ],
            },
        },
        1397: {
            "attributes": {"g_id": (5633, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387670.0],
                        [168990.0, 5387700.0],
                        [169020.0, 5387700.0],
                        [169020.0, 5387670.0],
                        [168990.0, 5387670.0],
                    ]
                ],
            },
        },
        1398: {
            "attributes": {"g_id": (5633, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387700.0],
                        [168990.0, 5387730.0],
                        [169020.0, 5387730.0],
                        [169020.0, 5387700.0],
                        [168990.0, 5387700.0],
                    ]
                ],
            },
        },
        1399: {
            "attributes": {"g_id": (5633, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387730.0],
                        [168990.0, 5387760.0],
                        [169020.0, 5387760.0],
                        [169020.0, 5387730.0],
                        [168990.0, 5387730.0],
                    ]
                ],
            },
        },
        1400: {
            "attributes": {"g_id": (5633, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387760.0],
                        [168990.0, 5387790.0],
                        [169020.0, 5387790.0],
                        [169020.0, 5387760.0],
                        [168990.0, 5387760.0],
                    ]
                ],
            },
        },
        1401: {
            "attributes": {"g_id": (5633, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387790.0],
                        [168990.0, 5387820.0],
                        [169020.0, 5387820.0],
                        [169020.0, 5387790.0],
                        [168990.0, 5387790.0],
                    ]
                ],
            },
        },
        1402: {
            "attributes": {"g_id": (5633, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387820.0],
                        [168990.0, 5387850.0],
                        [169020.0, 5387850.0],
                        [169020.0, 5387820.0],
                        [168990.0, 5387820.0],
                    ]
                ],
            },
        },
        1403: {
            "attributes": {"g_id": (5633, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387850.0],
                        [168990.0, 5387880.0],
                        [169020.0, 5387880.0],
                        [169020.0, 5387850.0],
                        [168990.0, 5387850.0],
                    ]
                ],
            },
        },
        1404: {
            "attributes": {"g_id": (5633, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387880.0],
                        [168990.0, 5387910.0],
                        [169020.0, 5387910.0],
                        [169020.0, 5387880.0],
                        [168990.0, 5387880.0],
                    ]
                ],
            },
        },
        1405: {
            "attributes": {"g_id": (5633, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387910.0],
                        [168990.0, 5387940.0],
                        [169020.0, 5387940.0],
                        [169020.0, 5387910.0],
                        [168990.0, 5387910.0],
                    ]
                ],
            },
        },
        1406: {
            "attributes": {"g_id": (5633, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387940.0],
                        [168990.0, 5387970.0],
                        [169020.0, 5387970.0],
                        [169020.0, 5387940.0],
                        [168990.0, 5387940.0],
                    ]
                ],
            },
        },
        1407: {
            "attributes": {"g_id": (5633, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5387970.0],
                        [168990.0, 5388000.0],
                        [169020.0, 5388000.0],
                        [169020.0, 5387970.0],
                        [168990.0, 5387970.0],
                    ]
                ],
            },
        },
        1408: {
            "attributes": {"g_id": (5633, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388000.0],
                        [168990.0, 5388030.0],
                        [169020.0, 5388030.0],
                        [169020.0, 5388000.0],
                        [168990.0, 5388000.0],
                    ]
                ],
            },
        },
        1409: {
            "attributes": {"g_id": (5633, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388030.0],
                        [168990.0, 5388060.0],
                        [169020.0, 5388060.0],
                        [169020.0, 5388030.0],
                        [168990.0, 5388030.0],
                    ]
                ],
            },
        },
        1410: {
            "attributes": {"g_id": (5633, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388060.0],
                        [168990.0, 5388090.0],
                        [169020.0, 5388090.0],
                        [169020.0, 5388060.0],
                        [168990.0, 5388060.0],
                    ]
                ],
            },
        },
        1411: {
            "attributes": {"g_id": (5633, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388090.0],
                        [168990.0, 5388120.0],
                        [169020.0, 5388120.0],
                        [169020.0, 5388090.0],
                        [168990.0, 5388090.0],
                    ]
                ],
            },
        },
        1412: {
            "attributes": {"g_id": (5633, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388120.0],
                        [168990.0, 5388150.0],
                        [169020.0, 5388150.0],
                        [169020.0, 5388120.0],
                        [168990.0, 5388120.0],
                    ]
                ],
            },
        },
        1413: {
            "attributes": {"g_id": (5633, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388150.0],
                        [168990.0, 5388180.0],
                        [169020.0, 5388180.0],
                        [169020.0, 5388150.0],
                        [168990.0, 5388150.0],
                    ]
                ],
            },
        },
        1414: {
            "attributes": {"g_id": (5633, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388180.0],
                        [168990.0, 5388210.0],
                        [169020.0, 5388210.0],
                        [169020.0, 5388180.0],
                        [168990.0, 5388180.0],
                    ]
                ],
            },
        },
        1415: {
            "attributes": {"g_id": (5633, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388210.0],
                        [168990.0, 5388240.0],
                        [169020.0, 5388240.0],
                        [169020.0, 5388210.0],
                        [168990.0, 5388210.0],
                    ]
                ],
            },
        },
        1416: {
            "attributes": {"g_id": (5633, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388240.0],
                        [168990.0, 5388270.0],
                        [169020.0, 5388270.0],
                        [169020.0, 5388240.0],
                        [168990.0, 5388240.0],
                    ]
                ],
            },
        },
        1417: {
            "attributes": {"g_id": (5633, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388270.0],
                        [168990.0, 5388300.0],
                        [169020.0, 5388300.0],
                        [169020.0, 5388270.0],
                        [168990.0, 5388270.0],
                    ]
                ],
            },
        },
        1418: {
            "attributes": {"g_id": (5633, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388300.0],
                        [168990.0, 5388330.0],
                        [169020.0, 5388330.0],
                        [169020.0, 5388300.0],
                        [168990.0, 5388300.0],
                    ]
                ],
            },
        },
        1419: {
            "attributes": {"g_id": (5633, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388330.0],
                        [168990.0, 5388360.0],
                        [169020.0, 5388360.0],
                        [169020.0, 5388330.0],
                        [168990.0, 5388330.0],
                    ]
                ],
            },
        },
        1420: {
            "attributes": {"g_id": (5633, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388360.0],
                        [168990.0, 5388390.0],
                        [169020.0, 5388390.0],
                        [169020.0, 5388360.0],
                        [168990.0, 5388360.0],
                    ]
                ],
            },
        },
        1421: {
            "attributes": {"g_id": (5633, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388390.0],
                        [168990.0, 5388420.0],
                        [169020.0, 5388420.0],
                        [169020.0, 5388390.0],
                        [168990.0, 5388390.0],
                    ]
                ],
            },
        },
        1422: {
            "attributes": {"g_id": (5633, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388420.0],
                        [168990.0, 5388450.0],
                        [169020.0, 5388450.0],
                        [169020.0, 5388420.0],
                        [168990.0, 5388420.0],
                    ]
                ],
            },
        },
        1423: {
            "attributes": {"g_id": (5633, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388450.0],
                        [168990.0, 5388480.0],
                        [169020.0, 5388480.0],
                        [169020.0, 5388450.0],
                        [168990.0, 5388450.0],
                    ]
                ],
            },
        },
        1424: {
            "attributes": {"g_id": (5633, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388480.0],
                        [168990.0, 5388510.0],
                        [169020.0, 5388510.0],
                        [169020.0, 5388480.0],
                        [168990.0, 5388480.0],
                    ]
                ],
            },
        },
        1425: {
            "attributes": {"g_id": (5633, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388510.0],
                        [168990.0, 5388540.0],
                        [169020.0, 5388540.0],
                        [169020.0, 5388510.0],
                        [168990.0, 5388510.0],
                    ]
                ],
            },
        },
        1426: {
            "attributes": {"g_id": (5633, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388540.0],
                        [168990.0, 5388570.0],
                        [169020.0, 5388570.0],
                        [169020.0, 5388540.0],
                        [168990.0, 5388540.0],
                    ]
                ],
            },
        },
        1427: {
            "attributes": {"g_id": (5633, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388570.0],
                        [168990.0, 5388600.0],
                        [169020.0, 5388600.0],
                        [169020.0, 5388570.0],
                        [168990.0, 5388570.0],
                    ]
                ],
            },
        },
        1428: {
            "attributes": {"g_id": (5633, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388600.0],
                        [168990.0, 5388630.0],
                        [169020.0, 5388630.0],
                        [169020.0, 5388600.0],
                        [168990.0, 5388600.0],
                    ]
                ],
            },
        },
        1429: {
            "attributes": {"g_id": (5633, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388630.0],
                        [168990.0, 5388660.0],
                        [169020.0, 5388660.0],
                        [169020.0, 5388630.0],
                        [168990.0, 5388630.0],
                    ]
                ],
            },
        },
        1430: {
            "attributes": {"g_id": (5633, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388660.0],
                        [168990.0, 5388690.0],
                        [169020.0, 5388690.0],
                        [169020.0, 5388660.0],
                        [168990.0, 5388660.0],
                    ]
                ],
            },
        },
        1431: {
            "attributes": {"g_id": (5633, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388690.0],
                        [168990.0, 5388720.0],
                        [169020.0, 5388720.0],
                        [169020.0, 5388690.0],
                        [168990.0, 5388690.0],
                    ]
                ],
            },
        },
        1432: {
            "attributes": {"g_id": (5633, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388720.0],
                        [168990.0, 5388750.0],
                        [169020.0, 5388750.0],
                        [169020.0, 5388720.0],
                        [168990.0, 5388720.0],
                    ]
                ],
            },
        },
        1433: {
            "attributes": {"g_id": (5633, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388750.0],
                        [168990.0, 5388780.0],
                        [169020.0, 5388780.0],
                        [169020.0, 5388750.0],
                        [168990.0, 5388750.0],
                    ]
                ],
            },
        },
        1434: {
            "attributes": {"g_id": (5633, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388780.0],
                        [168990.0, 5388810.0],
                        [169020.0, 5388810.0],
                        [169020.0, 5388780.0],
                        [168990.0, 5388780.0],
                    ]
                ],
            },
        },
        1435: {
            "attributes": {"g_id": (5633, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388810.0],
                        [168990.0, 5388840.0],
                        [169020.0, 5388840.0],
                        [169020.0, 5388810.0],
                        [168990.0, 5388810.0],
                    ]
                ],
            },
        },
        1436: {
            "attributes": {"g_id": (5633, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388840.0],
                        [168990.0, 5388870.0],
                        [169020.0, 5388870.0],
                        [169020.0, 5388840.0],
                        [168990.0, 5388840.0],
                    ]
                ],
            },
        },
        1437: {
            "attributes": {"g_id": (5633, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388870.0],
                        [168990.0, 5388900.0],
                        [169020.0, 5388900.0],
                        [169020.0, 5388870.0],
                        [168990.0, 5388870.0],
                    ]
                ],
            },
        },
        1438: {
            "attributes": {"g_id": (5633, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388900.0],
                        [168990.0, 5388930.0],
                        [169020.0, 5388930.0],
                        [169020.0, 5388900.0],
                        [168990.0, 5388900.0],
                    ]
                ],
            },
        },
        1439: {
            "attributes": {"g_id": (5633, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [168990.0, 5388930.0],
                        [168990.0, 5388960.0],
                        [169020.0, 5388960.0],
                        [169020.0, 5388930.0],
                        [168990.0, 5388930.0],
                    ]
                ],
            },
        },
        1440: {
            "attributes": {"g_id": (5634, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387520.0],
                        [169020.0, 5387550.0],
                        [169050.0, 5387550.0],
                        [169050.0, 5387520.0],
                        [169020.0, 5387520.0],
                    ]
                ],
            },
        },
        1441: {
            "attributes": {"g_id": (5634, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387550.0],
                        [169020.0, 5387580.0],
                        [169050.0, 5387580.0],
                        [169050.0, 5387550.0],
                        [169020.0, 5387550.0],
                    ]
                ],
            },
        },
        1442: {
            "attributes": {"g_id": (5634, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387580.0],
                        [169020.0, 5387610.0],
                        [169050.0, 5387610.0],
                        [169050.0, 5387580.0],
                        [169020.0, 5387580.0],
                    ]
                ],
            },
        },
        1443: {
            "attributes": {"g_id": (5634, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387610.0],
                        [169020.0, 5387640.0],
                        [169050.0, 5387640.0],
                        [169050.0, 5387610.0],
                        [169020.0, 5387610.0],
                    ]
                ],
            },
        },
        1444: {
            "attributes": {"g_id": (5634, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387640.0],
                        [169020.0, 5387670.0],
                        [169050.0, 5387670.0],
                        [169050.0, 5387640.0],
                        [169020.0, 5387640.0],
                    ]
                ],
            },
        },
        1445: {
            "attributes": {"g_id": (5634, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387670.0],
                        [169020.0, 5387700.0],
                        [169050.0, 5387700.0],
                        [169050.0, 5387670.0],
                        [169020.0, 5387670.0],
                    ]
                ],
            },
        },
        1446: {
            "attributes": {"g_id": (5634, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387700.0],
                        [169020.0, 5387730.0],
                        [169050.0, 5387730.0],
                        [169050.0, 5387700.0],
                        [169020.0, 5387700.0],
                    ]
                ],
            },
        },
        1447: {
            "attributes": {"g_id": (5634, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387730.0],
                        [169020.0, 5387760.0],
                        [169050.0, 5387760.0],
                        [169050.0, 5387730.0],
                        [169020.0, 5387730.0],
                    ]
                ],
            },
        },
        1448: {
            "attributes": {"g_id": (5634, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387760.0],
                        [169020.0, 5387790.0],
                        [169050.0, 5387790.0],
                        [169050.0, 5387760.0],
                        [169020.0, 5387760.0],
                    ]
                ],
            },
        },
        1449: {
            "attributes": {"g_id": (5634, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387790.0],
                        [169020.0, 5387820.0],
                        [169050.0, 5387820.0],
                        [169050.0, 5387790.0],
                        [169020.0, 5387790.0],
                    ]
                ],
            },
        },
        1450: {
            "attributes": {"g_id": (5634, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387820.0],
                        [169020.0, 5387850.0],
                        [169050.0, 5387850.0],
                        [169050.0, 5387820.0],
                        [169020.0, 5387820.0],
                    ]
                ],
            },
        },
        1451: {
            "attributes": {"g_id": (5634, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387850.0],
                        [169020.0, 5387880.0],
                        [169050.0, 5387880.0],
                        [169050.0, 5387850.0],
                        [169020.0, 5387850.0],
                    ]
                ],
            },
        },
        1452: {
            "attributes": {"g_id": (5634, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387880.0],
                        [169020.0, 5387910.0],
                        [169050.0, 5387910.0],
                        [169050.0, 5387880.0],
                        [169020.0, 5387880.0],
                    ]
                ],
            },
        },
        1453: {
            "attributes": {"g_id": (5634, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387910.0],
                        [169020.0, 5387940.0],
                        [169050.0, 5387940.0],
                        [169050.0, 5387910.0],
                        [169020.0, 5387910.0],
                    ]
                ],
            },
        },
        1454: {
            "attributes": {"g_id": (5634, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387940.0],
                        [169020.0, 5387970.0],
                        [169050.0, 5387970.0],
                        [169050.0, 5387940.0],
                        [169020.0, 5387940.0],
                    ]
                ],
            },
        },
        1455: {
            "attributes": {"g_id": (5634, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5387970.0],
                        [169020.0, 5388000.0],
                        [169050.0, 5388000.0],
                        [169050.0, 5387970.0],
                        [169020.0, 5387970.0],
                    ]
                ],
            },
        },
        1456: {
            "attributes": {"g_id": (5634, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388000.0],
                        [169020.0, 5388030.0],
                        [169050.0, 5388030.0],
                        [169050.0, 5388000.0],
                        [169020.0, 5388000.0],
                    ]
                ],
            },
        },
        1457: {
            "attributes": {"g_id": (5634, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388030.0],
                        [169020.0, 5388060.0],
                        [169050.0, 5388060.0],
                        [169050.0, 5388030.0],
                        [169020.0, 5388030.0],
                    ]
                ],
            },
        },
        1458: {
            "attributes": {"g_id": (5634, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388060.0],
                        [169020.0, 5388090.0],
                        [169050.0, 5388090.0],
                        [169050.0, 5388060.0],
                        [169020.0, 5388060.0],
                    ]
                ],
            },
        },
        1459: {
            "attributes": {"g_id": (5634, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388090.0],
                        [169020.0, 5388120.0],
                        [169050.0, 5388120.0],
                        [169050.0, 5388090.0],
                        [169020.0, 5388090.0],
                    ]
                ],
            },
        },
        1460: {
            "attributes": {"g_id": (5634, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388120.0],
                        [169020.0, 5388150.0],
                        [169050.0, 5388150.0],
                        [169050.0, 5388120.0],
                        [169020.0, 5388120.0],
                    ]
                ],
            },
        },
        1461: {
            "attributes": {"g_id": (5634, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388150.0],
                        [169020.0, 5388180.0],
                        [169050.0, 5388180.0],
                        [169050.0, 5388150.0],
                        [169020.0, 5388150.0],
                    ]
                ],
            },
        },
        1462: {
            "attributes": {"g_id": (5634, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388180.0],
                        [169020.0, 5388210.0],
                        [169050.0, 5388210.0],
                        [169050.0, 5388180.0],
                        [169020.0, 5388180.0],
                    ]
                ],
            },
        },
        1463: {
            "attributes": {"g_id": (5634, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388210.0],
                        [169020.0, 5388240.0],
                        [169050.0, 5388240.0],
                        [169050.0, 5388210.0],
                        [169020.0, 5388210.0],
                    ]
                ],
            },
        },
        1464: {
            "attributes": {"g_id": (5634, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388240.0],
                        [169020.0, 5388270.0],
                        [169050.0, 5388270.0],
                        [169050.0, 5388240.0],
                        [169020.0, 5388240.0],
                    ]
                ],
            },
        },
        1465: {
            "attributes": {"g_id": (5634, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388270.0],
                        [169020.0, 5388300.0],
                        [169050.0, 5388300.0],
                        [169050.0, 5388270.0],
                        [169020.0, 5388270.0],
                    ]
                ],
            },
        },
        1466: {
            "attributes": {"g_id": (5634, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388300.0],
                        [169020.0, 5388330.0],
                        [169050.0, 5388330.0],
                        [169050.0, 5388300.0],
                        [169020.0, 5388300.0],
                    ]
                ],
            },
        },
        1467: {
            "attributes": {"g_id": (5634, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388330.0],
                        [169020.0, 5388360.0],
                        [169050.0, 5388360.0],
                        [169050.0, 5388330.0],
                        [169020.0, 5388330.0],
                    ]
                ],
            },
        },
        1468: {
            "attributes": {"g_id": (5634, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388360.0],
                        [169020.0, 5388390.0],
                        [169050.0, 5388390.0],
                        [169050.0, 5388360.0],
                        [169020.0, 5388360.0],
                    ]
                ],
            },
        },
        1469: {
            "attributes": {"g_id": (5634, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388390.0],
                        [169020.0, 5388420.0],
                        [169050.0, 5388420.0],
                        [169050.0, 5388390.0],
                        [169020.0, 5388390.0],
                    ]
                ],
            },
        },
        1470: {
            "attributes": {"g_id": (5634, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388420.0],
                        [169020.0, 5388450.0],
                        [169050.0, 5388450.0],
                        [169050.0, 5388420.0],
                        [169020.0, 5388420.0],
                    ]
                ],
            },
        },
        1471: {
            "attributes": {"g_id": (5634, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388450.0],
                        [169020.0, 5388480.0],
                        [169050.0, 5388480.0],
                        [169050.0, 5388450.0],
                        [169020.0, 5388450.0],
                    ]
                ],
            },
        },
        1472: {
            "attributes": {"g_id": (5634, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388480.0],
                        [169020.0, 5388510.0],
                        [169050.0, 5388510.0],
                        [169050.0, 5388480.0],
                        [169020.0, 5388480.0],
                    ]
                ],
            },
        },
        1473: {
            "attributes": {"g_id": (5634, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388510.0],
                        [169020.0, 5388540.0],
                        [169050.0, 5388540.0],
                        [169050.0, 5388510.0],
                        [169020.0, 5388510.0],
                    ]
                ],
            },
        },
        1474: {
            "attributes": {"g_id": (5634, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388540.0],
                        [169020.0, 5388570.0],
                        [169050.0, 5388570.0],
                        [169050.0, 5388540.0],
                        [169020.0, 5388540.0],
                    ]
                ],
            },
        },
        1475: {
            "attributes": {"g_id": (5634, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388570.0],
                        [169020.0, 5388600.0],
                        [169050.0, 5388600.0],
                        [169050.0, 5388570.0],
                        [169020.0, 5388570.0],
                    ]
                ],
            },
        },
        1476: {
            "attributes": {"g_id": (5634, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388600.0],
                        [169020.0, 5388630.0],
                        [169050.0, 5388630.0],
                        [169050.0, 5388600.0],
                        [169020.0, 5388600.0],
                    ]
                ],
            },
        },
        1477: {
            "attributes": {"g_id": (5634, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388630.0],
                        [169020.0, 5388660.0],
                        [169050.0, 5388660.0],
                        [169050.0, 5388630.0],
                        [169020.0, 5388630.0],
                    ]
                ],
            },
        },
        1478: {
            "attributes": {"g_id": (5634, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388660.0],
                        [169020.0, 5388690.0],
                        [169050.0, 5388690.0],
                        [169050.0, 5388660.0],
                        [169020.0, 5388660.0],
                    ]
                ],
            },
        },
        1479: {
            "attributes": {"g_id": (5634, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388690.0],
                        [169020.0, 5388720.0],
                        [169050.0, 5388720.0],
                        [169050.0, 5388690.0],
                        [169020.0, 5388690.0],
                    ]
                ],
            },
        },
        1480: {
            "attributes": {"g_id": (5634, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388720.0],
                        [169020.0, 5388750.0],
                        [169050.0, 5388750.0],
                        [169050.0, 5388720.0],
                        [169020.0, 5388720.0],
                    ]
                ],
            },
        },
        1481: {
            "attributes": {"g_id": (5634, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388750.0],
                        [169020.0, 5388780.0],
                        [169050.0, 5388780.0],
                        [169050.0, 5388750.0],
                        [169020.0, 5388750.0],
                    ]
                ],
            },
        },
        1482: {
            "attributes": {"g_id": (5634, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388780.0],
                        [169020.0, 5388810.0],
                        [169050.0, 5388810.0],
                        [169050.0, 5388780.0],
                        [169020.0, 5388780.0],
                    ]
                ],
            },
        },
        1483: {
            "attributes": {"g_id": (5634, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388810.0],
                        [169020.0, 5388840.0],
                        [169050.0, 5388840.0],
                        [169050.0, 5388810.0],
                        [169020.0, 5388810.0],
                    ]
                ],
            },
        },
        1484: {
            "attributes": {"g_id": (5634, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388840.0],
                        [169020.0, 5388870.0],
                        [169050.0, 5388870.0],
                        [169050.0, 5388840.0],
                        [169020.0, 5388840.0],
                    ]
                ],
            },
        },
        1485: {
            "attributes": {"g_id": (5634, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388870.0],
                        [169020.0, 5388900.0],
                        [169050.0, 5388900.0],
                        [169050.0, 5388870.0],
                        [169020.0, 5388870.0],
                    ]
                ],
            },
        },
        1486: {
            "attributes": {"g_id": (5634, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388900.0],
                        [169020.0, 5388930.0],
                        [169050.0, 5388930.0],
                        [169050.0, 5388900.0],
                        [169020.0, 5388900.0],
                    ]
                ],
            },
        },
        1487: {
            "attributes": {"g_id": (5634, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169020.0, 5388930.0],
                        [169020.0, 5388960.0],
                        [169050.0, 5388960.0],
                        [169050.0, 5388930.0],
                        [169020.0, 5388930.0],
                    ]
                ],
            },
        },
        1488: {
            "attributes": {"g_id": (5635, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387520.0],
                        [169050.0, 5387550.0],
                        [169080.0, 5387550.0],
                        [169080.0, 5387520.0],
                        [169050.0, 5387520.0],
                    ]
                ],
            },
        },
        1489: {
            "attributes": {"g_id": (5635, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387550.0],
                        [169050.0, 5387580.0],
                        [169080.0, 5387580.0],
                        [169080.0, 5387550.0],
                        [169050.0, 5387550.0],
                    ]
                ],
            },
        },
        1490: {
            "attributes": {"g_id": (5635, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387580.0],
                        [169050.0, 5387610.0],
                        [169080.0, 5387610.0],
                        [169080.0, 5387580.0],
                        [169050.0, 5387580.0],
                    ]
                ],
            },
        },
        1491: {
            "attributes": {"g_id": (5635, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387610.0],
                        [169050.0, 5387640.0],
                        [169080.0, 5387640.0],
                        [169080.0, 5387610.0],
                        [169050.0, 5387610.0],
                    ]
                ],
            },
        },
        1492: {
            "attributes": {"g_id": (5635, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387640.0],
                        [169050.0, 5387670.0],
                        [169080.0, 5387670.0],
                        [169080.0, 5387640.0],
                        [169050.0, 5387640.0],
                    ]
                ],
            },
        },
        1493: {
            "attributes": {"g_id": (5635, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387670.0],
                        [169050.0, 5387700.0],
                        [169080.0, 5387700.0],
                        [169080.0, 5387670.0],
                        [169050.0, 5387670.0],
                    ]
                ],
            },
        },
        1494: {
            "attributes": {"g_id": (5635, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387700.0],
                        [169050.0, 5387730.0],
                        [169080.0, 5387730.0],
                        [169080.0, 5387700.0],
                        [169050.0, 5387700.0],
                    ]
                ],
            },
        },
        1495: {
            "attributes": {"g_id": (5635, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387730.0],
                        [169050.0, 5387760.0],
                        [169080.0, 5387760.0],
                        [169080.0, 5387730.0],
                        [169050.0, 5387730.0],
                    ]
                ],
            },
        },
        1496: {
            "attributes": {"g_id": (5635, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387760.0],
                        [169050.0, 5387790.0],
                        [169080.0, 5387790.0],
                        [169080.0, 5387760.0],
                        [169050.0, 5387760.0],
                    ]
                ],
            },
        },
        1497: {
            "attributes": {"g_id": (5635, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387790.0],
                        [169050.0, 5387820.0],
                        [169080.0, 5387820.0],
                        [169080.0, 5387790.0],
                        [169050.0, 5387790.0],
                    ]
                ],
            },
        },
        1498: {
            "attributes": {"g_id": (5635, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387820.0],
                        [169050.0, 5387850.0],
                        [169080.0, 5387850.0],
                        [169080.0, 5387820.0],
                        [169050.0, 5387820.0],
                    ]
                ],
            },
        },
        1499: {
            "attributes": {"g_id": (5635, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387850.0],
                        [169050.0, 5387880.0],
                        [169080.0, 5387880.0],
                        [169080.0, 5387850.0],
                        [169050.0, 5387850.0],
                    ]
                ],
            },
        },
        1500: {
            "attributes": {"g_id": (5635, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387880.0],
                        [169050.0, 5387910.0],
                        [169080.0, 5387910.0],
                        [169080.0, 5387880.0],
                        [169050.0, 5387880.0],
                    ]
                ],
            },
        },
        1501: {
            "attributes": {"g_id": (5635, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387910.0],
                        [169050.0, 5387940.0],
                        [169080.0, 5387940.0],
                        [169080.0, 5387910.0],
                        [169050.0, 5387910.0],
                    ]
                ],
            },
        },
        1502: {
            "attributes": {"g_id": (5635, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387940.0],
                        [169050.0, 5387970.0],
                        [169080.0, 5387970.0],
                        [169080.0, 5387940.0],
                        [169050.0, 5387940.0],
                    ]
                ],
            },
        },
        1503: {
            "attributes": {"g_id": (5635, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5387970.0],
                        [169050.0, 5388000.0],
                        [169080.0, 5388000.0],
                        [169080.0, 5387970.0],
                        [169050.0, 5387970.0],
                    ]
                ],
            },
        },
        1504: {
            "attributes": {"g_id": (5635, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388000.0],
                        [169050.0, 5388030.0],
                        [169080.0, 5388030.0],
                        [169080.0, 5388000.0],
                        [169050.0, 5388000.0],
                    ]
                ],
            },
        },
        1505: {
            "attributes": {"g_id": (5635, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388030.0],
                        [169050.0, 5388060.0],
                        [169080.0, 5388060.0],
                        [169080.0, 5388030.0],
                        [169050.0, 5388030.0],
                    ]
                ],
            },
        },
        1506: {
            "attributes": {"g_id": (5635, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388060.0],
                        [169050.0, 5388090.0],
                        [169080.0, 5388090.0],
                        [169080.0, 5388060.0],
                        [169050.0, 5388060.0],
                    ]
                ],
            },
        },
        1507: {
            "attributes": {"g_id": (5635, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388090.0],
                        [169050.0, 5388120.0],
                        [169080.0, 5388120.0],
                        [169080.0, 5388090.0],
                        [169050.0, 5388090.0],
                    ]
                ],
            },
        },
        1508: {
            "attributes": {"g_id": (5635, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388120.0],
                        [169050.0, 5388150.0],
                        [169080.0, 5388150.0],
                        [169080.0, 5388120.0],
                        [169050.0, 5388120.0],
                    ]
                ],
            },
        },
        1509: {
            "attributes": {"g_id": (5635, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388150.0],
                        [169050.0, 5388180.0],
                        [169080.0, 5388180.0],
                        [169080.0, 5388150.0],
                        [169050.0, 5388150.0],
                    ]
                ],
            },
        },
        1510: {
            "attributes": {"g_id": (5635, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388180.0],
                        [169050.0, 5388210.0],
                        [169080.0, 5388210.0],
                        [169080.0, 5388180.0],
                        [169050.0, 5388180.0],
                    ]
                ],
            },
        },
        1511: {
            "attributes": {"g_id": (5635, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388210.0],
                        [169050.0, 5388240.0],
                        [169080.0, 5388240.0],
                        [169080.0, 5388210.0],
                        [169050.0, 5388210.0],
                    ]
                ],
            },
        },
        1512: {
            "attributes": {"g_id": (5635, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388240.0],
                        [169050.0, 5388270.0],
                        [169080.0, 5388270.0],
                        [169080.0, 5388240.0],
                        [169050.0, 5388240.0],
                    ]
                ],
            },
        },
        1513: {
            "attributes": {"g_id": (5635, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388270.0],
                        [169050.0, 5388300.0],
                        [169080.0, 5388300.0],
                        [169080.0, 5388270.0],
                        [169050.0, 5388270.0],
                    ]
                ],
            },
        },
        1514: {
            "attributes": {"g_id": (5635, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388300.0],
                        [169050.0, 5388330.0],
                        [169080.0, 5388330.0],
                        [169080.0, 5388300.0],
                        [169050.0, 5388300.0],
                    ]
                ],
            },
        },
        1515: {
            "attributes": {"g_id": (5635, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388330.0],
                        [169050.0, 5388360.0],
                        [169080.0, 5388360.0],
                        [169080.0, 5388330.0],
                        [169050.0, 5388330.0],
                    ]
                ],
            },
        },
        1516: {
            "attributes": {"g_id": (5635, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388360.0],
                        [169050.0, 5388390.0],
                        [169080.0, 5388390.0],
                        [169080.0, 5388360.0],
                        [169050.0, 5388360.0],
                    ]
                ],
            },
        },
        1517: {
            "attributes": {"g_id": (5635, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388390.0],
                        [169050.0, 5388420.0],
                        [169080.0, 5388420.0],
                        [169080.0, 5388390.0],
                        [169050.0, 5388390.0],
                    ]
                ],
            },
        },
        1518: {
            "attributes": {"g_id": (5635, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388420.0],
                        [169050.0, 5388450.0],
                        [169080.0, 5388450.0],
                        [169080.0, 5388420.0],
                        [169050.0, 5388420.0],
                    ]
                ],
            },
        },
        1519: {
            "attributes": {"g_id": (5635, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388450.0],
                        [169050.0, 5388480.0],
                        [169080.0, 5388480.0],
                        [169080.0, 5388450.0],
                        [169050.0, 5388450.0],
                    ]
                ],
            },
        },
        1520: {
            "attributes": {"g_id": (5635, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388480.0],
                        [169050.0, 5388510.0],
                        [169080.0, 5388510.0],
                        [169080.0, 5388480.0],
                        [169050.0, 5388480.0],
                    ]
                ],
            },
        },
        1521: {
            "attributes": {"g_id": (5635, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388510.0],
                        [169050.0, 5388540.0],
                        [169080.0, 5388540.0],
                        [169080.0, 5388510.0],
                        [169050.0, 5388510.0],
                    ]
                ],
            },
        },
        1522: {
            "attributes": {"g_id": (5635, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388540.0],
                        [169050.0, 5388570.0],
                        [169080.0, 5388570.0],
                        [169080.0, 5388540.0],
                        [169050.0, 5388540.0],
                    ]
                ],
            },
        },
        1523: {
            "attributes": {"g_id": (5635, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388570.0],
                        [169050.0, 5388600.0],
                        [169080.0, 5388600.0],
                        [169080.0, 5388570.0],
                        [169050.0, 5388570.0],
                    ]
                ],
            },
        },
        1524: {
            "attributes": {"g_id": (5635, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388600.0],
                        [169050.0, 5388630.0],
                        [169080.0, 5388630.0],
                        [169080.0, 5388600.0],
                        [169050.0, 5388600.0],
                    ]
                ],
            },
        },
        1525: {
            "attributes": {"g_id": (5635, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388630.0],
                        [169050.0, 5388660.0],
                        [169080.0, 5388660.0],
                        [169080.0, 5388630.0],
                        [169050.0, 5388630.0],
                    ]
                ],
            },
        },
        1526: {
            "attributes": {"g_id": (5635, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388660.0],
                        [169050.0, 5388690.0],
                        [169080.0, 5388690.0],
                        [169080.0, 5388660.0],
                        [169050.0, 5388660.0],
                    ]
                ],
            },
        },
        1527: {
            "attributes": {"g_id": (5635, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388690.0],
                        [169050.0, 5388720.0],
                        [169080.0, 5388720.0],
                        [169080.0, 5388690.0],
                        [169050.0, 5388690.0],
                    ]
                ],
            },
        },
        1528: {
            "attributes": {"g_id": (5635, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388720.0],
                        [169050.0, 5388750.0],
                        [169080.0, 5388750.0],
                        [169080.0, 5388720.0],
                        [169050.0, 5388720.0],
                    ]
                ],
            },
        },
        1529: {
            "attributes": {"g_id": (5635, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388750.0],
                        [169050.0, 5388780.0],
                        [169080.0, 5388780.0],
                        [169080.0, 5388750.0],
                        [169050.0, 5388750.0],
                    ]
                ],
            },
        },
        1530: {
            "attributes": {"g_id": (5635, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388780.0],
                        [169050.0, 5388810.0],
                        [169080.0, 5388810.0],
                        [169080.0, 5388780.0],
                        [169050.0, 5388780.0],
                    ]
                ],
            },
        },
        1531: {
            "attributes": {"g_id": (5635, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388810.0],
                        [169050.0, 5388840.0],
                        [169080.0, 5388840.0],
                        [169080.0, 5388810.0],
                        [169050.0, 5388810.0],
                    ]
                ],
            },
        },
        1532: {
            "attributes": {"g_id": (5635, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388840.0],
                        [169050.0, 5388870.0],
                        [169080.0, 5388870.0],
                        [169080.0, 5388840.0],
                        [169050.0, 5388840.0],
                    ]
                ],
            },
        },
        1533: {
            "attributes": {"g_id": (5635, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388870.0],
                        [169050.0, 5388900.0],
                        [169080.0, 5388900.0],
                        [169080.0, 5388870.0],
                        [169050.0, 5388870.0],
                    ]
                ],
            },
        },
        1534: {
            "attributes": {"g_id": (5635, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388900.0],
                        [169050.0, 5388930.0],
                        [169080.0, 5388930.0],
                        [169080.0, 5388900.0],
                        [169050.0, 5388900.0],
                    ]
                ],
            },
        },
        1535: {
            "attributes": {"g_id": (5635, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169050.0, 5388930.0],
                        [169050.0, 5388960.0],
                        [169080.0, 5388960.0],
                        [169080.0, 5388930.0],
                        [169050.0, 5388930.0],
                    ]
                ],
            },
        },
        1536: {
            "attributes": {"g_id": (5636, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387520.0],
                        [169080.0, 5387550.0],
                        [169110.0, 5387550.0],
                        [169110.0, 5387520.0],
                        [169080.0, 5387520.0],
                    ]
                ],
            },
        },
        1537: {
            "attributes": {"g_id": (5636, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387550.0],
                        [169080.0, 5387580.0],
                        [169110.0, 5387580.0],
                        [169110.0, 5387550.0],
                        [169080.0, 5387550.0],
                    ]
                ],
            },
        },
        1538: {
            "attributes": {"g_id": (5636, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387580.0],
                        [169080.0, 5387610.0],
                        [169110.0, 5387610.0],
                        [169110.0, 5387580.0],
                        [169080.0, 5387580.0],
                    ]
                ],
            },
        },
        1539: {
            "attributes": {"g_id": (5636, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387610.0],
                        [169080.0, 5387640.0],
                        [169110.0, 5387640.0],
                        [169110.0, 5387610.0],
                        [169080.0, 5387610.0],
                    ]
                ],
            },
        },
        1540: {
            "attributes": {"g_id": (5636, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387640.0],
                        [169080.0, 5387670.0],
                        [169110.0, 5387670.0],
                        [169110.0, 5387640.0],
                        [169080.0, 5387640.0],
                    ]
                ],
            },
        },
        1541: {
            "attributes": {"g_id": (5636, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387670.0],
                        [169080.0, 5387700.0],
                        [169110.0, 5387700.0],
                        [169110.0, 5387670.0],
                        [169080.0, 5387670.0],
                    ]
                ],
            },
        },
        1542: {
            "attributes": {"g_id": (5636, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387700.0],
                        [169080.0, 5387730.0],
                        [169110.0, 5387730.0],
                        [169110.0, 5387700.0],
                        [169080.0, 5387700.0],
                    ]
                ],
            },
        },
        1543: {
            "attributes": {"g_id": (5636, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387730.0],
                        [169080.0, 5387760.0],
                        [169110.0, 5387760.0],
                        [169110.0, 5387730.0],
                        [169080.0, 5387730.0],
                    ]
                ],
            },
        },
        1544: {
            "attributes": {"g_id": (5636, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387760.0],
                        [169080.0, 5387790.0],
                        [169110.0, 5387790.0],
                        [169110.0, 5387760.0],
                        [169080.0, 5387760.0],
                    ]
                ],
            },
        },
        1545: {
            "attributes": {"g_id": (5636, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387790.0],
                        [169080.0, 5387820.0],
                        [169110.0, 5387820.0],
                        [169110.0, 5387790.0],
                        [169080.0, 5387790.0],
                    ]
                ],
            },
        },
        1546: {
            "attributes": {"g_id": (5636, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387820.0],
                        [169080.0, 5387850.0],
                        [169110.0, 5387850.0],
                        [169110.0, 5387820.0],
                        [169080.0, 5387820.0],
                    ]
                ],
            },
        },
        1547: {
            "attributes": {"g_id": (5636, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387850.0],
                        [169080.0, 5387880.0],
                        [169110.0, 5387880.0],
                        [169110.0, 5387850.0],
                        [169080.0, 5387850.0],
                    ]
                ],
            },
        },
        1548: {
            "attributes": {"g_id": (5636, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387880.0],
                        [169080.0, 5387910.0],
                        [169110.0, 5387910.0],
                        [169110.0, 5387880.0],
                        [169080.0, 5387880.0],
                    ]
                ],
            },
        },
        1549: {
            "attributes": {"g_id": (5636, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387910.0],
                        [169080.0, 5387940.0],
                        [169110.0, 5387940.0],
                        [169110.0, 5387910.0],
                        [169080.0, 5387910.0],
                    ]
                ],
            },
        },
        1550: {
            "attributes": {"g_id": (5636, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387940.0],
                        [169080.0, 5387970.0],
                        [169110.0, 5387970.0],
                        [169110.0, 5387940.0],
                        [169080.0, 5387940.0],
                    ]
                ],
            },
        },
        1551: {
            "attributes": {"g_id": (5636, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5387970.0],
                        [169080.0, 5388000.0],
                        [169110.0, 5388000.0],
                        [169110.0, 5387970.0],
                        [169080.0, 5387970.0],
                    ]
                ],
            },
        },
        1552: {
            "attributes": {"g_id": (5636, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388000.0],
                        [169080.0, 5388030.0],
                        [169110.0, 5388030.0],
                        [169110.0, 5388000.0],
                        [169080.0, 5388000.0],
                    ]
                ],
            },
        },
        1553: {
            "attributes": {"g_id": (5636, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388030.0],
                        [169080.0, 5388060.0],
                        [169110.0, 5388060.0],
                        [169110.0, 5388030.0],
                        [169080.0, 5388030.0],
                    ]
                ],
            },
        },
        1554: {
            "attributes": {"g_id": (5636, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388060.0],
                        [169080.0, 5388090.0],
                        [169110.0, 5388090.0],
                        [169110.0, 5388060.0],
                        [169080.0, 5388060.0],
                    ]
                ],
            },
        },
        1555: {
            "attributes": {"g_id": (5636, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388090.0],
                        [169080.0, 5388120.0],
                        [169110.0, 5388120.0],
                        [169110.0, 5388090.0],
                        [169080.0, 5388090.0],
                    ]
                ],
            },
        },
        1556: {
            "attributes": {"g_id": (5636, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388120.0],
                        [169080.0, 5388150.0],
                        [169110.0, 5388150.0],
                        [169110.0, 5388120.0],
                        [169080.0, 5388120.0],
                    ]
                ],
            },
        },
        1557: {
            "attributes": {"g_id": (5636, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388150.0],
                        [169080.0, 5388180.0],
                        [169110.0, 5388180.0],
                        [169110.0, 5388150.0],
                        [169080.0, 5388150.0],
                    ]
                ],
            },
        },
        1558: {
            "attributes": {"g_id": (5636, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388180.0],
                        [169080.0, 5388210.0],
                        [169110.0, 5388210.0],
                        [169110.0, 5388180.0],
                        [169080.0, 5388180.0],
                    ]
                ],
            },
        },
        1559: {
            "attributes": {"g_id": (5636, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388210.0],
                        [169080.0, 5388240.0],
                        [169110.0, 5388240.0],
                        [169110.0, 5388210.0],
                        [169080.0, 5388210.0],
                    ]
                ],
            },
        },
        1560: {
            "attributes": {"g_id": (5636, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388240.0],
                        [169080.0, 5388270.0],
                        [169110.0, 5388270.0],
                        [169110.0, 5388240.0],
                        [169080.0, 5388240.0],
                    ]
                ],
            },
        },
        1561: {
            "attributes": {"g_id": (5636, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388270.0],
                        [169080.0, 5388300.0],
                        [169110.0, 5388300.0],
                        [169110.0, 5388270.0],
                        [169080.0, 5388270.0],
                    ]
                ],
            },
        },
        1562: {
            "attributes": {"g_id": (5636, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388300.0],
                        [169080.0, 5388330.0],
                        [169110.0, 5388330.0],
                        [169110.0, 5388300.0],
                        [169080.0, 5388300.0],
                    ]
                ],
            },
        },
        1563: {
            "attributes": {"g_id": (5636, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388330.0],
                        [169080.0, 5388360.0],
                        [169110.0, 5388360.0],
                        [169110.0, 5388330.0],
                        [169080.0, 5388330.0],
                    ]
                ],
            },
        },
        1564: {
            "attributes": {"g_id": (5636, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388360.0],
                        [169080.0, 5388390.0],
                        [169110.0, 5388390.0],
                        [169110.0, 5388360.0],
                        [169080.0, 5388360.0],
                    ]
                ],
            },
        },
        1565: {
            "attributes": {"g_id": (5636, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388390.0],
                        [169080.0, 5388420.0],
                        [169110.0, 5388420.0],
                        [169110.0, 5388390.0],
                        [169080.0, 5388390.0],
                    ]
                ],
            },
        },
        1566: {
            "attributes": {"g_id": (5636, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388420.0],
                        [169080.0, 5388450.0],
                        [169110.0, 5388450.0],
                        [169110.0, 5388420.0],
                        [169080.0, 5388420.0],
                    ]
                ],
            },
        },
        1567: {
            "attributes": {"g_id": (5636, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388450.0],
                        [169080.0, 5388480.0],
                        [169110.0, 5388480.0],
                        [169110.0, 5388450.0],
                        [169080.0, 5388450.0],
                    ]
                ],
            },
        },
        1568: {
            "attributes": {"g_id": (5636, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388480.0],
                        [169080.0, 5388510.0],
                        [169110.0, 5388510.0],
                        [169110.0, 5388480.0],
                        [169080.0, 5388480.0],
                    ]
                ],
            },
        },
        1569: {
            "attributes": {"g_id": (5636, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388510.0],
                        [169080.0, 5388540.0],
                        [169110.0, 5388540.0],
                        [169110.0, 5388510.0],
                        [169080.0, 5388510.0],
                    ]
                ],
            },
        },
        1570: {
            "attributes": {"g_id": (5636, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388540.0],
                        [169080.0, 5388570.0],
                        [169110.0, 5388570.0],
                        [169110.0, 5388540.0],
                        [169080.0, 5388540.0],
                    ]
                ],
            },
        },
        1571: {
            "attributes": {"g_id": (5636, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388570.0],
                        [169080.0, 5388600.0],
                        [169110.0, 5388600.0],
                        [169110.0, 5388570.0],
                        [169080.0, 5388570.0],
                    ]
                ],
            },
        },
        1572: {
            "attributes": {"g_id": (5636, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388600.0],
                        [169080.0, 5388630.0],
                        [169110.0, 5388630.0],
                        [169110.0, 5388600.0],
                        [169080.0, 5388600.0],
                    ]
                ],
            },
        },
        1573: {
            "attributes": {"g_id": (5636, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388630.0],
                        [169080.0, 5388660.0],
                        [169110.0, 5388660.0],
                        [169110.0, 5388630.0],
                        [169080.0, 5388630.0],
                    ]
                ],
            },
        },
        1574: {
            "attributes": {"g_id": (5636, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388660.0],
                        [169080.0, 5388690.0],
                        [169110.0, 5388690.0],
                        [169110.0, 5388660.0],
                        [169080.0, 5388660.0],
                    ]
                ],
            },
        },
        1575: {
            "attributes": {"g_id": (5636, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388690.0],
                        [169080.0, 5388720.0],
                        [169110.0, 5388720.0],
                        [169110.0, 5388690.0],
                        [169080.0, 5388690.0],
                    ]
                ],
            },
        },
        1576: {
            "attributes": {"g_id": (5636, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388720.0],
                        [169080.0, 5388750.0],
                        [169110.0, 5388750.0],
                        [169110.0, 5388720.0],
                        [169080.0, 5388720.0],
                    ]
                ],
            },
        },
        1577: {
            "attributes": {"g_id": (5636, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388750.0],
                        [169080.0, 5388780.0],
                        [169110.0, 5388780.0],
                        [169110.0, 5388750.0],
                        [169080.0, 5388750.0],
                    ]
                ],
            },
        },
        1578: {
            "attributes": {"g_id": (5636, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388780.0],
                        [169080.0, 5388810.0],
                        [169110.0, 5388810.0],
                        [169110.0, 5388780.0],
                        [169080.0, 5388780.0],
                    ]
                ],
            },
        },
        1579: {
            "attributes": {"g_id": (5636, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388810.0],
                        [169080.0, 5388840.0],
                        [169110.0, 5388840.0],
                        [169110.0, 5388810.0],
                        [169080.0, 5388810.0],
                    ]
                ],
            },
        },
        1580: {
            "attributes": {"g_id": (5636, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388840.0],
                        [169080.0, 5388870.0],
                        [169110.0, 5388870.0],
                        [169110.0, 5388840.0],
                        [169080.0, 5388840.0],
                    ]
                ],
            },
        },
        1581: {
            "attributes": {"g_id": (5636, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388870.0],
                        [169080.0, 5388900.0],
                        [169110.0, 5388900.0],
                        [169110.0, 5388870.0],
                        [169080.0, 5388870.0],
                    ]
                ],
            },
        },
        1582: {
            "attributes": {"g_id": (5636, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388900.0],
                        [169080.0, 5388930.0],
                        [169110.0, 5388930.0],
                        [169110.0, 5388900.0],
                        [169080.0, 5388900.0],
                    ]
                ],
            },
        },
        1583: {
            "attributes": {"g_id": (5636, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169080.0, 5388930.0],
                        [169080.0, 5388960.0],
                        [169110.0, 5388960.0],
                        [169110.0, 5388930.0],
                        [169080.0, 5388930.0],
                    ]
                ],
            },
        },
        1584: {
            "attributes": {"g_id": (5637, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387520.0],
                        [169110.0, 5387550.0],
                        [169140.0, 5387550.0],
                        [169140.0, 5387520.0],
                        [169110.0, 5387520.0],
                    ]
                ],
            },
        },
        1585: {
            "attributes": {"g_id": (5637, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387550.0],
                        [169110.0, 5387580.0],
                        [169140.0, 5387580.0],
                        [169140.0, 5387550.0],
                        [169110.0, 5387550.0],
                    ]
                ],
            },
        },
        1586: {
            "attributes": {"g_id": (5637, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387580.0],
                        [169110.0, 5387610.0],
                        [169140.0, 5387610.0],
                        [169140.0, 5387580.0],
                        [169110.0, 5387580.0],
                    ]
                ],
            },
        },
        1587: {
            "attributes": {"g_id": (5637, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387610.0],
                        [169110.0, 5387640.0],
                        [169140.0, 5387640.0],
                        [169140.0, 5387610.0],
                        [169110.0, 5387610.0],
                    ]
                ],
            },
        },
        1588: {
            "attributes": {"g_id": (5637, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387640.0],
                        [169110.0, 5387670.0],
                        [169140.0, 5387670.0],
                        [169140.0, 5387640.0],
                        [169110.0, 5387640.0],
                    ]
                ],
            },
        },
        1589: {
            "attributes": {"g_id": (5637, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387670.0],
                        [169110.0, 5387700.0],
                        [169140.0, 5387700.0],
                        [169140.0, 5387670.0],
                        [169110.0, 5387670.0],
                    ]
                ],
            },
        },
        1590: {
            "attributes": {"g_id": (5637, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387700.0],
                        [169110.0, 5387730.0],
                        [169140.0, 5387730.0],
                        [169140.0, 5387700.0],
                        [169110.0, 5387700.0],
                    ]
                ],
            },
        },
        1591: {
            "attributes": {"g_id": (5637, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387730.0],
                        [169110.0, 5387760.0],
                        [169140.0, 5387760.0],
                        [169140.0, 5387730.0],
                        [169110.0, 5387730.0],
                    ]
                ],
            },
        },
        1592: {
            "attributes": {"g_id": (5637, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387760.0],
                        [169110.0, 5387790.0],
                        [169140.0, 5387790.0],
                        [169140.0, 5387760.0],
                        [169110.0, 5387760.0],
                    ]
                ],
            },
        },
        1593: {
            "attributes": {"g_id": (5637, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387790.0],
                        [169110.0, 5387820.0],
                        [169140.0, 5387820.0],
                        [169140.0, 5387790.0],
                        [169110.0, 5387790.0],
                    ]
                ],
            },
        },
        1594: {
            "attributes": {"g_id": (5637, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387820.0],
                        [169110.0, 5387850.0],
                        [169140.0, 5387850.0],
                        [169140.0, 5387820.0],
                        [169110.0, 5387820.0],
                    ]
                ],
            },
        },
        1595: {
            "attributes": {"g_id": (5637, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387850.0],
                        [169110.0, 5387880.0],
                        [169140.0, 5387880.0],
                        [169140.0, 5387850.0],
                        [169110.0, 5387850.0],
                    ]
                ],
            },
        },
        1596: {
            "attributes": {"g_id": (5637, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387880.0],
                        [169110.0, 5387910.0],
                        [169140.0, 5387910.0],
                        [169140.0, 5387880.0],
                        [169110.0, 5387880.0],
                    ]
                ],
            },
        },
        1597: {
            "attributes": {"g_id": (5637, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387910.0],
                        [169110.0, 5387940.0],
                        [169140.0, 5387940.0],
                        [169140.0, 5387910.0],
                        [169110.0, 5387910.0],
                    ]
                ],
            },
        },
        1598: {
            "attributes": {"g_id": (5637, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387940.0],
                        [169110.0, 5387970.0],
                        [169140.0, 5387970.0],
                        [169140.0, 5387940.0],
                        [169110.0, 5387940.0],
                    ]
                ],
            },
        },
        1599: {
            "attributes": {"g_id": (5637, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5387970.0],
                        [169110.0, 5388000.0],
                        [169140.0, 5388000.0],
                        [169140.0, 5387970.0],
                        [169110.0, 5387970.0],
                    ]
                ],
            },
        },
        1600: {
            "attributes": {"g_id": (5637, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388000.0],
                        [169110.0, 5388030.0],
                        [169140.0, 5388030.0],
                        [169140.0, 5388000.0],
                        [169110.0, 5388000.0],
                    ]
                ],
            },
        },
        1601: {
            "attributes": {"g_id": (5637, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388030.0],
                        [169110.0, 5388060.0],
                        [169140.0, 5388060.0],
                        [169140.0, 5388030.0],
                        [169110.0, 5388030.0],
                    ]
                ],
            },
        },
        1602: {
            "attributes": {"g_id": (5637, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388060.0],
                        [169110.0, 5388090.0],
                        [169140.0, 5388090.0],
                        [169140.0, 5388060.0],
                        [169110.0, 5388060.0],
                    ]
                ],
            },
        },
        1603: {
            "attributes": {"g_id": (5637, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388090.0],
                        [169110.0, 5388120.0],
                        [169140.0, 5388120.0],
                        [169140.0, 5388090.0],
                        [169110.0, 5388090.0],
                    ]
                ],
            },
        },
        1604: {
            "attributes": {"g_id": (5637, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388120.0],
                        [169110.0, 5388150.0],
                        [169140.0, 5388150.0],
                        [169140.0, 5388120.0],
                        [169110.0, 5388120.0],
                    ]
                ],
            },
        },
        1605: {
            "attributes": {"g_id": (5637, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388150.0],
                        [169110.0, 5388180.0],
                        [169140.0, 5388180.0],
                        [169140.0, 5388150.0],
                        [169110.0, 5388150.0],
                    ]
                ],
            },
        },
        1606: {
            "attributes": {"g_id": (5637, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388180.0],
                        [169110.0, 5388210.0],
                        [169140.0, 5388210.0],
                        [169140.0, 5388180.0],
                        [169110.0, 5388180.0],
                    ]
                ],
            },
        },
        1607: {
            "attributes": {"g_id": (5637, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388210.0],
                        [169110.0, 5388240.0],
                        [169140.0, 5388240.0],
                        [169140.0, 5388210.0],
                        [169110.0, 5388210.0],
                    ]
                ],
            },
        },
        1608: {
            "attributes": {"g_id": (5637, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388240.0],
                        [169110.0, 5388270.0],
                        [169140.0, 5388270.0],
                        [169140.0, 5388240.0],
                        [169110.0, 5388240.0],
                    ]
                ],
            },
        },
        1609: {
            "attributes": {"g_id": (5637, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388270.0],
                        [169110.0, 5388300.0],
                        [169140.0, 5388300.0],
                        [169140.0, 5388270.0],
                        [169110.0, 5388270.0],
                    ]
                ],
            },
        },
        1610: {
            "attributes": {"g_id": (5637, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388300.0],
                        [169110.0, 5388330.0],
                        [169140.0, 5388330.0],
                        [169140.0, 5388300.0],
                        [169110.0, 5388300.0],
                    ]
                ],
            },
        },
        1611: {
            "attributes": {"g_id": (5637, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388330.0],
                        [169110.0, 5388360.0],
                        [169140.0, 5388360.0],
                        [169140.0, 5388330.0],
                        [169110.0, 5388330.0],
                    ]
                ],
            },
        },
        1612: {
            "attributes": {"g_id": (5637, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388360.0],
                        [169110.0, 5388390.0],
                        [169140.0, 5388390.0],
                        [169140.0, 5388360.0],
                        [169110.0, 5388360.0],
                    ]
                ],
            },
        },
        1613: {
            "attributes": {"g_id": (5637, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388390.0],
                        [169110.0, 5388420.0],
                        [169140.0, 5388420.0],
                        [169140.0, 5388390.0],
                        [169110.0, 5388390.0],
                    ]
                ],
            },
        },
        1614: {
            "attributes": {"g_id": (5637, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388420.0],
                        [169110.0, 5388450.0],
                        [169140.0, 5388450.0],
                        [169140.0, 5388420.0],
                        [169110.0, 5388420.0],
                    ]
                ],
            },
        },
        1615: {
            "attributes": {"g_id": (5637, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388450.0],
                        [169110.0, 5388480.0],
                        [169140.0, 5388480.0],
                        [169140.0, 5388450.0],
                        [169110.0, 5388450.0],
                    ]
                ],
            },
        },
        1616: {
            "attributes": {"g_id": (5637, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388480.0],
                        [169110.0, 5388510.0],
                        [169140.0, 5388510.0],
                        [169140.0, 5388480.0],
                        [169110.0, 5388480.0],
                    ]
                ],
            },
        },
        1617: {
            "attributes": {"g_id": (5637, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388510.0],
                        [169110.0, 5388540.0],
                        [169140.0, 5388540.0],
                        [169140.0, 5388510.0],
                        [169110.0, 5388510.0],
                    ]
                ],
            },
        },
        1618: {
            "attributes": {"g_id": (5637, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388540.0],
                        [169110.0, 5388570.0],
                        [169140.0, 5388570.0],
                        [169140.0, 5388540.0],
                        [169110.0, 5388540.0],
                    ]
                ],
            },
        },
        1619: {
            "attributes": {"g_id": (5637, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388570.0],
                        [169110.0, 5388600.0],
                        [169140.0, 5388600.0],
                        [169140.0, 5388570.0],
                        [169110.0, 5388570.0],
                    ]
                ],
            },
        },
        1620: {
            "attributes": {"g_id": (5637, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388600.0],
                        [169110.0, 5388630.0],
                        [169140.0, 5388630.0],
                        [169140.0, 5388600.0],
                        [169110.0, 5388600.0],
                    ]
                ],
            },
        },
        1621: {
            "attributes": {"g_id": (5637, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388630.0],
                        [169110.0, 5388660.0],
                        [169140.0, 5388660.0],
                        [169140.0, 5388630.0],
                        [169110.0, 5388630.0],
                    ]
                ],
            },
        },
        1622: {
            "attributes": {"g_id": (5637, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388660.0],
                        [169110.0, 5388690.0],
                        [169140.0, 5388690.0],
                        [169140.0, 5388660.0],
                        [169110.0, 5388660.0],
                    ]
                ],
            },
        },
        1623: {
            "attributes": {"g_id": (5637, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388690.0],
                        [169110.0, 5388720.0],
                        [169140.0, 5388720.0],
                        [169140.0, 5388690.0],
                        [169110.0, 5388690.0],
                    ]
                ],
            },
        },
        1624: {
            "attributes": {"g_id": (5637, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388720.0],
                        [169110.0, 5388750.0],
                        [169140.0, 5388750.0],
                        [169140.0, 5388720.0],
                        [169110.0, 5388720.0],
                    ]
                ],
            },
        },
        1625: {
            "attributes": {"g_id": (5637, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388750.0],
                        [169110.0, 5388780.0],
                        [169140.0, 5388780.0],
                        [169140.0, 5388750.0],
                        [169110.0, 5388750.0],
                    ]
                ],
            },
        },
        1626: {
            "attributes": {"g_id": (5637, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388780.0],
                        [169110.0, 5388810.0],
                        [169140.0, 5388810.0],
                        [169140.0, 5388780.0],
                        [169110.0, 5388780.0],
                    ]
                ],
            },
        },
        1627: {
            "attributes": {"g_id": (5637, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388810.0],
                        [169110.0, 5388840.0],
                        [169140.0, 5388840.0],
                        [169140.0, 5388810.0],
                        [169110.0, 5388810.0],
                    ]
                ],
            },
        },
        1628: {
            "attributes": {"g_id": (5637, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388840.0],
                        [169110.0, 5388870.0],
                        [169140.0, 5388870.0],
                        [169140.0, 5388840.0],
                        [169110.0, 5388840.0],
                    ]
                ],
            },
        },
        1629: {
            "attributes": {"g_id": (5637, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388870.0],
                        [169110.0, 5388900.0],
                        [169140.0, 5388900.0],
                        [169140.0, 5388870.0],
                        [169110.0, 5388870.0],
                    ]
                ],
            },
        },
        1630: {
            "attributes": {"g_id": (5637, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388900.0],
                        [169110.0, 5388930.0],
                        [169140.0, 5388930.0],
                        [169140.0, 5388900.0],
                        [169110.0, 5388900.0],
                    ]
                ],
            },
        },
        1631: {
            "attributes": {"g_id": (5637, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169110.0, 5388930.0],
                        [169110.0, 5388960.0],
                        [169140.0, 5388960.0],
                        [169140.0, 5388930.0],
                        [169110.0, 5388930.0],
                    ]
                ],
            },
        },
        1632: {
            "attributes": {"g_id": (5638, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387520.0],
                        [169140.0, 5387550.0],
                        [169170.0, 5387550.0],
                        [169170.0, 5387520.0],
                        [169140.0, 5387520.0],
                    ]
                ],
            },
        },
        1633: {
            "attributes": {"g_id": (5638, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387550.0],
                        [169140.0, 5387580.0],
                        [169170.0, 5387580.0],
                        [169170.0, 5387550.0],
                        [169140.0, 5387550.0],
                    ]
                ],
            },
        },
        1634: {
            "attributes": {"g_id": (5638, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387580.0],
                        [169140.0, 5387610.0],
                        [169170.0, 5387610.0],
                        [169170.0, 5387580.0],
                        [169140.0, 5387580.0],
                    ]
                ],
            },
        },
        1635: {
            "attributes": {"g_id": (5638, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387610.0],
                        [169140.0, 5387640.0],
                        [169170.0, 5387640.0],
                        [169170.0, 5387610.0],
                        [169140.0, 5387610.0],
                    ]
                ],
            },
        },
        1636: {
            "attributes": {"g_id": (5638, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387640.0],
                        [169140.0, 5387670.0],
                        [169170.0, 5387670.0],
                        [169170.0, 5387640.0],
                        [169140.0, 5387640.0],
                    ]
                ],
            },
        },
        1637: {
            "attributes": {"g_id": (5638, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387670.0],
                        [169140.0, 5387700.0],
                        [169170.0, 5387700.0],
                        [169170.0, 5387670.0],
                        [169140.0, 5387670.0],
                    ]
                ],
            },
        },
        1638: {
            "attributes": {"g_id": (5638, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387700.0],
                        [169140.0, 5387730.0],
                        [169170.0, 5387730.0],
                        [169170.0, 5387700.0],
                        [169140.0, 5387700.0],
                    ]
                ],
            },
        },
        1639: {
            "attributes": {"g_id": (5638, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387730.0],
                        [169140.0, 5387760.0],
                        [169170.0, 5387760.0],
                        [169170.0, 5387730.0],
                        [169140.0, 5387730.0],
                    ]
                ],
            },
        },
        1640: {
            "attributes": {"g_id": (5638, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387760.0],
                        [169140.0, 5387790.0],
                        [169170.0, 5387790.0],
                        [169170.0, 5387760.0],
                        [169140.0, 5387760.0],
                    ]
                ],
            },
        },
        1641: {
            "attributes": {"g_id": (5638, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387790.0],
                        [169140.0, 5387820.0],
                        [169170.0, 5387820.0],
                        [169170.0, 5387790.0],
                        [169140.0, 5387790.0],
                    ]
                ],
            },
        },
        1642: {
            "attributes": {"g_id": (5638, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387820.0],
                        [169140.0, 5387850.0],
                        [169170.0, 5387850.0],
                        [169170.0, 5387820.0],
                        [169140.0, 5387820.0],
                    ]
                ],
            },
        },
        1643: {
            "attributes": {"g_id": (5638, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387850.0],
                        [169140.0, 5387880.0],
                        [169170.0, 5387880.0],
                        [169170.0, 5387850.0],
                        [169140.0, 5387850.0],
                    ]
                ],
            },
        },
        1644: {
            "attributes": {"g_id": (5638, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387880.0],
                        [169140.0, 5387910.0],
                        [169170.0, 5387910.0],
                        [169170.0, 5387880.0],
                        [169140.0, 5387880.0],
                    ]
                ],
            },
        },
        1645: {
            "attributes": {"g_id": (5638, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387910.0],
                        [169140.0, 5387940.0],
                        [169170.0, 5387940.0],
                        [169170.0, 5387910.0],
                        [169140.0, 5387910.0],
                    ]
                ],
            },
        },
        1646: {
            "attributes": {"g_id": (5638, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387940.0],
                        [169140.0, 5387970.0],
                        [169170.0, 5387970.0],
                        [169170.0, 5387940.0],
                        [169140.0, 5387940.0],
                    ]
                ],
            },
        },
        1647: {
            "attributes": {"g_id": (5638, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5387970.0],
                        [169140.0, 5388000.0],
                        [169170.0, 5388000.0],
                        [169170.0, 5387970.0],
                        [169140.0, 5387970.0],
                    ]
                ],
            },
        },
        1648: {
            "attributes": {"g_id": (5638, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388000.0],
                        [169140.0, 5388030.0],
                        [169170.0, 5388030.0],
                        [169170.0, 5388000.0],
                        [169140.0, 5388000.0],
                    ]
                ],
            },
        },
        1649: {
            "attributes": {"g_id": (5638, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388030.0],
                        [169140.0, 5388060.0],
                        [169170.0, 5388060.0],
                        [169170.0, 5388030.0],
                        [169140.0, 5388030.0],
                    ]
                ],
            },
        },
        1650: {
            "attributes": {"g_id": (5638, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388060.0],
                        [169140.0, 5388090.0],
                        [169170.0, 5388090.0],
                        [169170.0, 5388060.0],
                        [169140.0, 5388060.0],
                    ]
                ],
            },
        },
        1651: {
            "attributes": {"g_id": (5638, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388090.0],
                        [169140.0, 5388120.0],
                        [169170.0, 5388120.0],
                        [169170.0, 5388090.0],
                        [169140.0, 5388090.0],
                    ]
                ],
            },
        },
        1652: {
            "attributes": {"g_id": (5638, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388120.0],
                        [169140.0, 5388150.0],
                        [169170.0, 5388150.0],
                        [169170.0, 5388120.0],
                        [169140.0, 5388120.0],
                    ]
                ],
            },
        },
        1653: {
            "attributes": {"g_id": (5638, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388150.0],
                        [169140.0, 5388180.0],
                        [169170.0, 5388180.0],
                        [169170.0, 5388150.0],
                        [169140.0, 5388150.0],
                    ]
                ],
            },
        },
        1654: {
            "attributes": {"g_id": (5638, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388180.0],
                        [169140.0, 5388210.0],
                        [169170.0, 5388210.0],
                        [169170.0, 5388180.0],
                        [169140.0, 5388180.0],
                    ]
                ],
            },
        },
        1655: {
            "attributes": {"g_id": (5638, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388210.0],
                        [169140.0, 5388240.0],
                        [169170.0, 5388240.0],
                        [169170.0, 5388210.0],
                        [169140.0, 5388210.0],
                    ]
                ],
            },
        },
        1656: {
            "attributes": {"g_id": (5638, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388240.0],
                        [169140.0, 5388270.0],
                        [169170.0, 5388270.0],
                        [169170.0, 5388240.0],
                        [169140.0, 5388240.0],
                    ]
                ],
            },
        },
        1657: {
            "attributes": {"g_id": (5638, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388270.0],
                        [169140.0, 5388300.0],
                        [169170.0, 5388300.0],
                        [169170.0, 5388270.0],
                        [169140.0, 5388270.0],
                    ]
                ],
            },
        },
        1658: {
            "attributes": {"g_id": (5638, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388300.0],
                        [169140.0, 5388330.0],
                        [169170.0, 5388330.0],
                        [169170.0, 5388300.0],
                        [169140.0, 5388300.0],
                    ]
                ],
            },
        },
        1659: {
            "attributes": {"g_id": (5638, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388330.0],
                        [169140.0, 5388360.0],
                        [169170.0, 5388360.0],
                        [169170.0, 5388330.0],
                        [169140.0, 5388330.0],
                    ]
                ],
            },
        },
        1660: {
            "attributes": {"g_id": (5638, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388360.0],
                        [169140.0, 5388390.0],
                        [169170.0, 5388390.0],
                        [169170.0, 5388360.0],
                        [169140.0, 5388360.0],
                    ]
                ],
            },
        },
        1661: {
            "attributes": {"g_id": (5638, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388390.0],
                        [169140.0, 5388420.0],
                        [169170.0, 5388420.0],
                        [169170.0, 5388390.0],
                        [169140.0, 5388390.0],
                    ]
                ],
            },
        },
        1662: {
            "attributes": {"g_id": (5638, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388420.0],
                        [169140.0, 5388450.0],
                        [169170.0, 5388450.0],
                        [169170.0, 5388420.0],
                        [169140.0, 5388420.0],
                    ]
                ],
            },
        },
        1663: {
            "attributes": {"g_id": (5638, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388450.0],
                        [169140.0, 5388480.0],
                        [169170.0, 5388480.0],
                        [169170.0, 5388450.0],
                        [169140.0, 5388450.0],
                    ]
                ],
            },
        },
        1664: {
            "attributes": {"g_id": (5638, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388480.0],
                        [169140.0, 5388510.0],
                        [169170.0, 5388510.0],
                        [169170.0, 5388480.0],
                        [169140.0, 5388480.0],
                    ]
                ],
            },
        },
        1665: {
            "attributes": {"g_id": (5638, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388510.0],
                        [169140.0, 5388540.0],
                        [169170.0, 5388540.0],
                        [169170.0, 5388510.0],
                        [169140.0, 5388510.0],
                    ]
                ],
            },
        },
        1666: {
            "attributes": {"g_id": (5638, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388540.0],
                        [169140.0, 5388570.0],
                        [169170.0, 5388570.0],
                        [169170.0, 5388540.0],
                        [169140.0, 5388540.0],
                    ]
                ],
            },
        },
        1667: {
            "attributes": {"g_id": (5638, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388570.0],
                        [169140.0, 5388600.0],
                        [169170.0, 5388600.0],
                        [169170.0, 5388570.0],
                        [169140.0, 5388570.0],
                    ]
                ],
            },
        },
        1668: {
            "attributes": {"g_id": (5638, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388600.0],
                        [169140.0, 5388630.0],
                        [169170.0, 5388630.0],
                        [169170.0, 5388600.0],
                        [169140.0, 5388600.0],
                    ]
                ],
            },
        },
        1669: {
            "attributes": {"g_id": (5638, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388630.0],
                        [169140.0, 5388660.0],
                        [169170.0, 5388660.0],
                        [169170.0, 5388630.0],
                        [169140.0, 5388630.0],
                    ]
                ],
            },
        },
        1670: {
            "attributes": {"g_id": (5638, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388660.0],
                        [169140.0, 5388690.0],
                        [169170.0, 5388690.0],
                        [169170.0, 5388660.0],
                        [169140.0, 5388660.0],
                    ]
                ],
            },
        },
        1671: {
            "attributes": {"g_id": (5638, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388690.0],
                        [169140.0, 5388720.0],
                        [169170.0, 5388720.0],
                        [169170.0, 5388690.0],
                        [169140.0, 5388690.0],
                    ]
                ],
            },
        },
        1672: {
            "attributes": {"g_id": (5638, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388720.0],
                        [169140.0, 5388750.0],
                        [169170.0, 5388750.0],
                        [169170.0, 5388720.0],
                        [169140.0, 5388720.0],
                    ]
                ],
            },
        },
        1673: {
            "attributes": {"g_id": (5638, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388750.0],
                        [169140.0, 5388780.0],
                        [169170.0, 5388780.0],
                        [169170.0, 5388750.0],
                        [169140.0, 5388750.0],
                    ]
                ],
            },
        },
        1674: {
            "attributes": {"g_id": (5638, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388780.0],
                        [169140.0, 5388810.0],
                        [169170.0, 5388810.0],
                        [169170.0, 5388780.0],
                        [169140.0, 5388780.0],
                    ]
                ],
            },
        },
        1675: {
            "attributes": {"g_id": (5638, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388810.0],
                        [169140.0, 5388840.0],
                        [169170.0, 5388840.0],
                        [169170.0, 5388810.0],
                        [169140.0, 5388810.0],
                    ]
                ],
            },
        },
        1676: {
            "attributes": {"g_id": (5638, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388840.0],
                        [169140.0, 5388870.0],
                        [169170.0, 5388870.0],
                        [169170.0, 5388840.0],
                        [169140.0, 5388840.0],
                    ]
                ],
            },
        },
        1677: {
            "attributes": {"g_id": (5638, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388870.0],
                        [169140.0, 5388900.0],
                        [169170.0, 5388900.0],
                        [169170.0, 5388870.0],
                        [169140.0, 5388870.0],
                    ]
                ],
            },
        },
        1678: {
            "attributes": {"g_id": (5638, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388900.0],
                        [169140.0, 5388930.0],
                        [169170.0, 5388930.0],
                        [169170.0, 5388900.0],
                        [169140.0, 5388900.0],
                    ]
                ],
            },
        },
        1679: {
            "attributes": {"g_id": (5638, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169140.0, 5388930.0],
                        [169140.0, 5388960.0],
                        [169170.0, 5388960.0],
                        [169170.0, 5388930.0],
                        [169140.0, 5388930.0],
                    ]
                ],
            },
        },
        1680: {
            "attributes": {"g_id": (5639, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387520.0],
                        [169170.0, 5387550.0],
                        [169200.0, 5387550.0],
                        [169200.0, 5387520.0],
                        [169170.0, 5387520.0],
                    ]
                ],
            },
        },
        1681: {
            "attributes": {"g_id": (5639, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387550.0],
                        [169170.0, 5387580.0],
                        [169200.0, 5387580.0],
                        [169200.0, 5387550.0],
                        [169170.0, 5387550.0],
                    ]
                ],
            },
        },
        1682: {
            "attributes": {"g_id": (5639, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387580.0],
                        [169170.0, 5387610.0],
                        [169200.0, 5387610.0],
                        [169200.0, 5387580.0],
                        [169170.0, 5387580.0],
                    ]
                ],
            },
        },
        1683: {
            "attributes": {"g_id": (5639, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387610.0],
                        [169170.0, 5387640.0],
                        [169200.0, 5387640.0],
                        [169200.0, 5387610.0],
                        [169170.0, 5387610.0],
                    ]
                ],
            },
        },
        1684: {
            "attributes": {"g_id": (5639, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387640.0],
                        [169170.0, 5387670.0],
                        [169200.0, 5387670.0],
                        [169200.0, 5387640.0],
                        [169170.0, 5387640.0],
                    ]
                ],
            },
        },
        1685: {
            "attributes": {"g_id": (5639, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387670.0],
                        [169170.0, 5387700.0],
                        [169200.0, 5387700.0],
                        [169200.0, 5387670.0],
                        [169170.0, 5387670.0],
                    ]
                ],
            },
        },
        1686: {
            "attributes": {"g_id": (5639, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387700.0],
                        [169170.0, 5387730.0],
                        [169200.0, 5387730.0],
                        [169200.0, 5387700.0],
                        [169170.0, 5387700.0],
                    ]
                ],
            },
        },
        1687: {
            "attributes": {"g_id": (5639, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387730.0],
                        [169170.0, 5387760.0],
                        [169200.0, 5387760.0],
                        [169200.0, 5387730.0],
                        [169170.0, 5387730.0],
                    ]
                ],
            },
        },
        1688: {
            "attributes": {"g_id": (5639, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387760.0],
                        [169170.0, 5387790.0],
                        [169200.0, 5387790.0],
                        [169200.0, 5387760.0],
                        [169170.0, 5387760.0],
                    ]
                ],
            },
        },
        1689: {
            "attributes": {"g_id": (5639, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387790.0],
                        [169170.0, 5387820.0],
                        [169200.0, 5387820.0],
                        [169200.0, 5387790.0],
                        [169170.0, 5387790.0],
                    ]
                ],
            },
        },
        1690: {
            "attributes": {"g_id": (5639, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387820.0],
                        [169170.0, 5387850.0],
                        [169200.0, 5387850.0],
                        [169200.0, 5387820.0],
                        [169170.0, 5387820.0],
                    ]
                ],
            },
        },
        1691: {
            "attributes": {"g_id": (5639, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387850.0],
                        [169170.0, 5387880.0],
                        [169200.0, 5387880.0],
                        [169200.0, 5387850.0],
                        [169170.0, 5387850.0],
                    ]
                ],
            },
        },
        1692: {
            "attributes": {"g_id": (5639, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387880.0],
                        [169170.0, 5387910.0],
                        [169200.0, 5387910.0],
                        [169200.0, 5387880.0],
                        [169170.0, 5387880.0],
                    ]
                ],
            },
        },
        1693: {
            "attributes": {"g_id": (5639, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387910.0],
                        [169170.0, 5387940.0],
                        [169200.0, 5387940.0],
                        [169200.0, 5387910.0],
                        [169170.0, 5387910.0],
                    ]
                ],
            },
        },
        1694: {
            "attributes": {"g_id": (5639, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387940.0],
                        [169170.0, 5387970.0],
                        [169200.0, 5387970.0],
                        [169200.0, 5387940.0],
                        [169170.0, 5387940.0],
                    ]
                ],
            },
        },
        1695: {
            "attributes": {"g_id": (5639, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5387970.0],
                        [169170.0, 5388000.0],
                        [169200.0, 5388000.0],
                        [169200.0, 5387970.0],
                        [169170.0, 5387970.0],
                    ]
                ],
            },
        },
        1696: {
            "attributes": {"g_id": (5639, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388000.0],
                        [169170.0, 5388030.0],
                        [169200.0, 5388030.0],
                        [169200.0, 5388000.0],
                        [169170.0, 5388000.0],
                    ]
                ],
            },
        },
        1697: {
            "attributes": {"g_id": (5639, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388030.0],
                        [169170.0, 5388060.0],
                        [169200.0, 5388060.0],
                        [169200.0, 5388030.0],
                        [169170.0, 5388030.0],
                    ]
                ],
            },
        },
        1698: {
            "attributes": {"g_id": (5639, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388060.0],
                        [169170.0, 5388090.0],
                        [169200.0, 5388090.0],
                        [169200.0, 5388060.0],
                        [169170.0, 5388060.0],
                    ]
                ],
            },
        },
        1699: {
            "attributes": {"g_id": (5639, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388090.0],
                        [169170.0, 5388120.0],
                        [169200.0, 5388120.0],
                        [169200.0, 5388090.0],
                        [169170.0, 5388090.0],
                    ]
                ],
            },
        },
        1700: {
            "attributes": {"g_id": (5639, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388120.0],
                        [169170.0, 5388150.0],
                        [169200.0, 5388150.0],
                        [169200.0, 5388120.0],
                        [169170.0, 5388120.0],
                    ]
                ],
            },
        },
        1701: {
            "attributes": {"g_id": (5639, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388150.0],
                        [169170.0, 5388180.0],
                        [169200.0, 5388180.0],
                        [169200.0, 5388150.0],
                        [169170.0, 5388150.0],
                    ]
                ],
            },
        },
        1702: {
            "attributes": {"g_id": (5639, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388180.0],
                        [169170.0, 5388210.0],
                        [169200.0, 5388210.0],
                        [169200.0, 5388180.0],
                        [169170.0, 5388180.0],
                    ]
                ],
            },
        },
        1703: {
            "attributes": {"g_id": (5639, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388210.0],
                        [169170.0, 5388240.0],
                        [169200.0, 5388240.0],
                        [169200.0, 5388210.0],
                        [169170.0, 5388210.0],
                    ]
                ],
            },
        },
        1704: {
            "attributes": {"g_id": (5639, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388240.0],
                        [169170.0, 5388270.0],
                        [169200.0, 5388270.0],
                        [169200.0, 5388240.0],
                        [169170.0, 5388240.0],
                    ]
                ],
            },
        },
        1705: {
            "attributes": {"g_id": (5639, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388270.0],
                        [169170.0, 5388300.0],
                        [169200.0, 5388300.0],
                        [169200.0, 5388270.0],
                        [169170.0, 5388270.0],
                    ]
                ],
            },
        },
        1706: {
            "attributes": {"g_id": (5639, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388300.0],
                        [169170.0, 5388330.0],
                        [169200.0, 5388330.0],
                        [169200.0, 5388300.0],
                        [169170.0, 5388300.0],
                    ]
                ],
            },
        },
        1707: {
            "attributes": {"g_id": (5639, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388330.0],
                        [169170.0, 5388360.0],
                        [169200.0, 5388360.0],
                        [169200.0, 5388330.0],
                        [169170.0, 5388330.0],
                    ]
                ],
            },
        },
        1708: {
            "attributes": {"g_id": (5639, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388360.0],
                        [169170.0, 5388390.0],
                        [169200.0, 5388390.0],
                        [169200.0, 5388360.0],
                        [169170.0, 5388360.0],
                    ]
                ],
            },
        },
        1709: {
            "attributes": {"g_id": (5639, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388390.0],
                        [169170.0, 5388420.0],
                        [169200.0, 5388420.0],
                        [169200.0, 5388390.0],
                        [169170.0, 5388390.0],
                    ]
                ],
            },
        },
        1710: {
            "attributes": {"g_id": (5639, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388420.0],
                        [169170.0, 5388450.0],
                        [169200.0, 5388450.0],
                        [169200.0, 5388420.0],
                        [169170.0, 5388420.0],
                    ]
                ],
            },
        },
        1711: {
            "attributes": {"g_id": (5639, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388450.0],
                        [169170.0, 5388480.0],
                        [169200.0, 5388480.0],
                        [169200.0, 5388450.0],
                        [169170.0, 5388450.0],
                    ]
                ],
            },
        },
        1712: {
            "attributes": {"g_id": (5639, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388480.0],
                        [169170.0, 5388510.0],
                        [169200.0, 5388510.0],
                        [169200.0, 5388480.0],
                        [169170.0, 5388480.0],
                    ]
                ],
            },
        },
        1713: {
            "attributes": {"g_id": (5639, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388510.0],
                        [169170.0, 5388540.0],
                        [169200.0, 5388540.0],
                        [169200.0, 5388510.0],
                        [169170.0, 5388510.0],
                    ]
                ],
            },
        },
        1714: {
            "attributes": {"g_id": (5639, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388540.0],
                        [169170.0, 5388570.0],
                        [169200.0, 5388570.0],
                        [169200.0, 5388540.0],
                        [169170.0, 5388540.0],
                    ]
                ],
            },
        },
        1715: {
            "attributes": {"g_id": (5639, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388570.0],
                        [169170.0, 5388600.0],
                        [169200.0, 5388600.0],
                        [169200.0, 5388570.0],
                        [169170.0, 5388570.0],
                    ]
                ],
            },
        },
        1716: {
            "attributes": {"g_id": (5639, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388600.0],
                        [169170.0, 5388630.0],
                        [169200.0, 5388630.0],
                        [169200.0, 5388600.0],
                        [169170.0, 5388600.0],
                    ]
                ],
            },
        },
        1717: {
            "attributes": {"g_id": (5639, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388630.0],
                        [169170.0, 5388660.0],
                        [169200.0, 5388660.0],
                        [169200.0, 5388630.0],
                        [169170.0, 5388630.0],
                    ]
                ],
            },
        },
        1718: {
            "attributes": {"g_id": (5639, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388660.0],
                        [169170.0, 5388690.0],
                        [169200.0, 5388690.0],
                        [169200.0, 5388660.0],
                        [169170.0, 5388660.0],
                    ]
                ],
            },
        },
        1719: {
            "attributes": {"g_id": (5639, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388690.0],
                        [169170.0, 5388720.0],
                        [169200.0, 5388720.0],
                        [169200.0, 5388690.0],
                        [169170.0, 5388690.0],
                    ]
                ],
            },
        },
        1720: {
            "attributes": {"g_id": (5639, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388720.0],
                        [169170.0, 5388750.0],
                        [169200.0, 5388750.0],
                        [169200.0, 5388720.0],
                        [169170.0, 5388720.0],
                    ]
                ],
            },
        },
        1721: {
            "attributes": {"g_id": (5639, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388750.0],
                        [169170.0, 5388780.0],
                        [169200.0, 5388780.0],
                        [169200.0, 5388750.0],
                        [169170.0, 5388750.0],
                    ]
                ],
            },
        },
        1722: {
            "attributes": {"g_id": (5639, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388780.0],
                        [169170.0, 5388810.0],
                        [169200.0, 5388810.0],
                        [169200.0, 5388780.0],
                        [169170.0, 5388780.0],
                    ]
                ],
            },
        },
        1723: {
            "attributes": {"g_id": (5639, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388810.0],
                        [169170.0, 5388840.0],
                        [169200.0, 5388840.0],
                        [169200.0, 5388810.0],
                        [169170.0, 5388810.0],
                    ]
                ],
            },
        },
        1724: {
            "attributes": {"g_id": (5639, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388840.0],
                        [169170.0, 5388870.0],
                        [169200.0, 5388870.0],
                        [169200.0, 5388840.0],
                        [169170.0, 5388840.0],
                    ]
                ],
            },
        },
        1725: {
            "attributes": {"g_id": (5639, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388870.0],
                        [169170.0, 5388900.0],
                        [169200.0, 5388900.0],
                        [169200.0, 5388870.0],
                        [169170.0, 5388870.0],
                    ]
                ],
            },
        },
        1726: {
            "attributes": {"g_id": (5639, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388900.0],
                        [169170.0, 5388930.0],
                        [169200.0, 5388930.0],
                        [169200.0, 5388900.0],
                        [169170.0, 5388900.0],
                    ]
                ],
            },
        },
        1727: {
            "attributes": {"g_id": (5639, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169170.0, 5388930.0],
                        [169170.0, 5388960.0],
                        [169200.0, 5388960.0],
                        [169200.0, 5388930.0],
                        [169170.0, 5388930.0],
                    ]
                ],
            },
        },
        1728: {
            "attributes": {"g_id": (5640, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387520.0],
                        [169200.0, 5387550.0],
                        [169230.0, 5387550.0],
                        [169230.0, 5387520.0],
                        [169200.0, 5387520.0],
                    ]
                ],
            },
        },
        1729: {
            "attributes": {"g_id": (5640, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387550.0],
                        [169200.0, 5387580.0],
                        [169230.0, 5387580.0],
                        [169230.0, 5387550.0],
                        [169200.0, 5387550.0],
                    ]
                ],
            },
        },
        1730: {
            "attributes": {"g_id": (5640, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387580.0],
                        [169200.0, 5387610.0],
                        [169230.0, 5387610.0],
                        [169230.0, 5387580.0],
                        [169200.0, 5387580.0],
                    ]
                ],
            },
        },
        1731: {
            "attributes": {"g_id": (5640, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387610.0],
                        [169200.0, 5387640.0],
                        [169230.0, 5387640.0],
                        [169230.0, 5387610.0],
                        [169200.0, 5387610.0],
                    ]
                ],
            },
        },
        1732: {
            "attributes": {"g_id": (5640, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387640.0],
                        [169200.0, 5387670.0],
                        [169230.0, 5387670.0],
                        [169230.0, 5387640.0],
                        [169200.0, 5387640.0],
                    ]
                ],
            },
        },
        1733: {
            "attributes": {"g_id": (5640, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387670.0],
                        [169200.0, 5387700.0],
                        [169230.0, 5387700.0],
                        [169230.0, 5387670.0],
                        [169200.0, 5387670.0],
                    ]
                ],
            },
        },
        1734: {
            "attributes": {"g_id": (5640, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387700.0],
                        [169200.0, 5387730.0],
                        [169230.0, 5387730.0],
                        [169230.0, 5387700.0],
                        [169200.0, 5387700.0],
                    ]
                ],
            },
        },
        1735: {
            "attributes": {"g_id": (5640, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387730.0],
                        [169200.0, 5387760.0],
                        [169230.0, 5387760.0],
                        [169230.0, 5387730.0],
                        [169200.0, 5387730.0],
                    ]
                ],
            },
        },
        1736: {
            "attributes": {"g_id": (5640, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387760.0],
                        [169200.0, 5387790.0],
                        [169230.0, 5387790.0],
                        [169230.0, 5387760.0],
                        [169200.0, 5387760.0],
                    ]
                ],
            },
        },
        1737: {
            "attributes": {"g_id": (5640, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387790.0],
                        [169200.0, 5387820.0],
                        [169230.0, 5387820.0],
                        [169230.0, 5387790.0],
                        [169200.0, 5387790.0],
                    ]
                ],
            },
        },
        1738: {
            "attributes": {"g_id": (5640, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387820.0],
                        [169200.0, 5387850.0],
                        [169230.0, 5387850.0],
                        [169230.0, 5387820.0],
                        [169200.0, 5387820.0],
                    ]
                ],
            },
        },
        1739: {
            "attributes": {"g_id": (5640, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387850.0],
                        [169200.0, 5387880.0],
                        [169230.0, 5387880.0],
                        [169230.0, 5387850.0],
                        [169200.0, 5387850.0],
                    ]
                ],
            },
        },
        1740: {
            "attributes": {"g_id": (5640, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387880.0],
                        [169200.0, 5387910.0],
                        [169230.0, 5387910.0],
                        [169230.0, 5387880.0],
                        [169200.0, 5387880.0],
                    ]
                ],
            },
        },
        1741: {
            "attributes": {"g_id": (5640, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387910.0],
                        [169200.0, 5387940.0],
                        [169230.0, 5387940.0],
                        [169230.0, 5387910.0],
                        [169200.0, 5387910.0],
                    ]
                ],
            },
        },
        1742: {
            "attributes": {"g_id": (5640, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387940.0],
                        [169200.0, 5387970.0],
                        [169230.0, 5387970.0],
                        [169230.0, 5387940.0],
                        [169200.0, 5387940.0],
                    ]
                ],
            },
        },
        1743: {
            "attributes": {"g_id": (5640, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5387970.0],
                        [169200.0, 5388000.0],
                        [169230.0, 5388000.0],
                        [169230.0, 5387970.0],
                        [169200.0, 5387970.0],
                    ]
                ],
            },
        },
        1744: {
            "attributes": {"g_id": (5640, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388000.0],
                        [169200.0, 5388030.0],
                        [169230.0, 5388030.0],
                        [169230.0, 5388000.0],
                        [169200.0, 5388000.0],
                    ]
                ],
            },
        },
        1745: {
            "attributes": {"g_id": (5640, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388030.0],
                        [169200.0, 5388060.0],
                        [169230.0, 5388060.0],
                        [169230.0, 5388030.0],
                        [169200.0, 5388030.0],
                    ]
                ],
            },
        },
        1746: {
            "attributes": {"g_id": (5640, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388060.0],
                        [169200.0, 5388090.0],
                        [169230.0, 5388090.0],
                        [169230.0, 5388060.0],
                        [169200.0, 5388060.0],
                    ]
                ],
            },
        },
        1747: {
            "attributes": {"g_id": (5640, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388090.0],
                        [169200.0, 5388120.0],
                        [169230.0, 5388120.0],
                        [169230.0, 5388090.0],
                        [169200.0, 5388090.0],
                    ]
                ],
            },
        },
        1748: {
            "attributes": {"g_id": (5640, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388120.0],
                        [169200.0, 5388150.0],
                        [169230.0, 5388150.0],
                        [169230.0, 5388120.0],
                        [169200.0, 5388120.0],
                    ]
                ],
            },
        },
        1749: {
            "attributes": {"g_id": (5640, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388150.0],
                        [169200.0, 5388180.0],
                        [169230.0, 5388180.0],
                        [169230.0, 5388150.0],
                        [169200.0, 5388150.0],
                    ]
                ],
            },
        },
        1750: {
            "attributes": {"g_id": (5640, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388180.0],
                        [169200.0, 5388210.0],
                        [169230.0, 5388210.0],
                        [169230.0, 5388180.0],
                        [169200.0, 5388180.0],
                    ]
                ],
            },
        },
        1751: {
            "attributes": {"g_id": (5640, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388210.0],
                        [169200.0, 5388240.0],
                        [169230.0, 5388240.0],
                        [169230.0, 5388210.0],
                        [169200.0, 5388210.0],
                    ]
                ],
            },
        },
        1752: {
            "attributes": {"g_id": (5640, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388240.0],
                        [169200.0, 5388270.0],
                        [169230.0, 5388270.0],
                        [169230.0, 5388240.0],
                        [169200.0, 5388240.0],
                    ]
                ],
            },
        },
        1753: {
            "attributes": {"g_id": (5640, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388270.0],
                        [169200.0, 5388300.0],
                        [169230.0, 5388300.0],
                        [169230.0, 5388270.0],
                        [169200.0, 5388270.0],
                    ]
                ],
            },
        },
        1754: {
            "attributes": {"g_id": (5640, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388300.0],
                        [169200.0, 5388330.0],
                        [169230.0, 5388330.0],
                        [169230.0, 5388300.0],
                        [169200.0, 5388300.0],
                    ]
                ],
            },
        },
        1755: {
            "attributes": {"g_id": (5640, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388330.0],
                        [169200.0, 5388360.0],
                        [169230.0, 5388360.0],
                        [169230.0, 5388330.0],
                        [169200.0, 5388330.0],
                    ]
                ],
            },
        },
        1756: {
            "attributes": {"g_id": (5640, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388360.0],
                        [169200.0, 5388390.0],
                        [169230.0, 5388390.0],
                        [169230.0, 5388360.0],
                        [169200.0, 5388360.0],
                    ]
                ],
            },
        },
        1757: {
            "attributes": {"g_id": (5640, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388390.0],
                        [169200.0, 5388420.0],
                        [169230.0, 5388420.0],
                        [169230.0, 5388390.0],
                        [169200.0, 5388390.0],
                    ]
                ],
            },
        },
        1758: {
            "attributes": {"g_id": (5640, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388420.0],
                        [169200.0, 5388450.0],
                        [169230.0, 5388450.0],
                        [169230.0, 5388420.0],
                        [169200.0, 5388420.0],
                    ]
                ],
            },
        },
        1759: {
            "attributes": {"g_id": (5640, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388450.0],
                        [169200.0, 5388480.0],
                        [169230.0, 5388480.0],
                        [169230.0, 5388450.0],
                        [169200.0, 5388450.0],
                    ]
                ],
            },
        },
        1760: {
            "attributes": {"g_id": (5640, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388480.0],
                        [169200.0, 5388510.0],
                        [169230.0, 5388510.0],
                        [169230.0, 5388480.0],
                        [169200.0, 5388480.0],
                    ]
                ],
            },
        },
        1761: {
            "attributes": {"g_id": (5640, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388510.0],
                        [169200.0, 5388540.0],
                        [169230.0, 5388540.0],
                        [169230.0, 5388510.0],
                        [169200.0, 5388510.0],
                    ]
                ],
            },
        },
        1762: {
            "attributes": {"g_id": (5640, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388540.0],
                        [169200.0, 5388570.0],
                        [169230.0, 5388570.0],
                        [169230.0, 5388540.0],
                        [169200.0, 5388540.0],
                    ]
                ],
            },
        },
        1763: {
            "attributes": {"g_id": (5640, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388570.0],
                        [169200.0, 5388600.0],
                        [169230.0, 5388600.0],
                        [169230.0, 5388570.0],
                        [169200.0, 5388570.0],
                    ]
                ],
            },
        },
        1764: {
            "attributes": {"g_id": (5640, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388600.0],
                        [169200.0, 5388630.0],
                        [169230.0, 5388630.0],
                        [169230.0, 5388600.0],
                        [169200.0, 5388600.0],
                    ]
                ],
            },
        },
        1765: {
            "attributes": {"g_id": (5640, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388630.0],
                        [169200.0, 5388660.0],
                        [169230.0, 5388660.0],
                        [169230.0, 5388630.0],
                        [169200.0, 5388630.0],
                    ]
                ],
            },
        },
        1766: {
            "attributes": {"g_id": (5640, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388660.0],
                        [169200.0, 5388690.0],
                        [169230.0, 5388690.0],
                        [169230.0, 5388660.0],
                        [169200.0, 5388660.0],
                    ]
                ],
            },
        },
        1767: {
            "attributes": {"g_id": (5640, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388690.0],
                        [169200.0, 5388720.0],
                        [169230.0, 5388720.0],
                        [169230.0, 5388690.0],
                        [169200.0, 5388690.0],
                    ]
                ],
            },
        },
        1768: {
            "attributes": {"g_id": (5640, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388720.0],
                        [169200.0, 5388750.0],
                        [169230.0, 5388750.0],
                        [169230.0, 5388720.0],
                        [169200.0, 5388720.0],
                    ]
                ],
            },
        },
        1769: {
            "attributes": {"g_id": (5640, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388750.0],
                        [169200.0, 5388780.0],
                        [169230.0, 5388780.0],
                        [169230.0, 5388750.0],
                        [169200.0, 5388750.0],
                    ]
                ],
            },
        },
        1770: {
            "attributes": {"g_id": (5640, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388780.0],
                        [169200.0, 5388810.0],
                        [169230.0, 5388810.0],
                        [169230.0, 5388780.0],
                        [169200.0, 5388780.0],
                    ]
                ],
            },
        },
        1771: {
            "attributes": {"g_id": (5640, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388810.0],
                        [169200.0, 5388840.0],
                        [169230.0, 5388840.0],
                        [169230.0, 5388810.0],
                        [169200.0, 5388810.0],
                    ]
                ],
            },
        },
        1772: {
            "attributes": {"g_id": (5640, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388840.0],
                        [169200.0, 5388870.0],
                        [169230.0, 5388870.0],
                        [169230.0, 5388840.0],
                        [169200.0, 5388840.0],
                    ]
                ],
            },
        },
        1773: {
            "attributes": {"g_id": (5640, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388870.0],
                        [169200.0, 5388900.0],
                        [169230.0, 5388900.0],
                        [169230.0, 5388870.0],
                        [169200.0, 5388870.0],
                    ]
                ],
            },
        },
        1774: {
            "attributes": {"g_id": (5640, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388900.0],
                        [169200.0, 5388930.0],
                        [169230.0, 5388930.0],
                        [169230.0, 5388900.0],
                        [169200.0, 5388900.0],
                    ]
                ],
            },
        },
        1775: {
            "attributes": {"g_id": (5640, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169200.0, 5388930.0],
                        [169200.0, 5388960.0],
                        [169230.0, 5388960.0],
                        [169230.0, 5388930.0],
                        [169200.0, 5388930.0],
                    ]
                ],
            },
        },
        1776: {
            "attributes": {"g_id": (5641, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387520.0],
                        [169230.0, 5387550.0],
                        [169260.0, 5387550.0],
                        [169260.0, 5387520.0],
                        [169230.0, 5387520.0],
                    ]
                ],
            },
        },
        1777: {
            "attributes": {"g_id": (5641, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387550.0],
                        [169230.0, 5387580.0],
                        [169260.0, 5387580.0],
                        [169260.0, 5387550.0],
                        [169230.0, 5387550.0],
                    ]
                ],
            },
        },
        1778: {
            "attributes": {"g_id": (5641, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387580.0],
                        [169230.0, 5387610.0],
                        [169260.0, 5387610.0],
                        [169260.0, 5387580.0],
                        [169230.0, 5387580.0],
                    ]
                ],
            },
        },
        1779: {
            "attributes": {"g_id": (5641, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387610.0],
                        [169230.0, 5387640.0],
                        [169260.0, 5387640.0],
                        [169260.0, 5387610.0],
                        [169230.0, 5387610.0],
                    ]
                ],
            },
        },
        1780: {
            "attributes": {"g_id": (5641, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387640.0],
                        [169230.0, 5387670.0],
                        [169260.0, 5387670.0],
                        [169260.0, 5387640.0],
                        [169230.0, 5387640.0],
                    ]
                ],
            },
        },
        1781: {
            "attributes": {"g_id": (5641, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387670.0],
                        [169230.0, 5387700.0],
                        [169260.0, 5387700.0],
                        [169260.0, 5387670.0],
                        [169230.0, 5387670.0],
                    ]
                ],
            },
        },
        1782: {
            "attributes": {"g_id": (5641, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387700.0],
                        [169230.0, 5387730.0],
                        [169260.0, 5387730.0],
                        [169260.0, 5387700.0],
                        [169230.0, 5387700.0],
                    ]
                ],
            },
        },
        1783: {
            "attributes": {"g_id": (5641, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387730.0],
                        [169230.0, 5387760.0],
                        [169260.0, 5387760.0],
                        [169260.0, 5387730.0],
                        [169230.0, 5387730.0],
                    ]
                ],
            },
        },
        1784: {
            "attributes": {"g_id": (5641, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387760.0],
                        [169230.0, 5387790.0],
                        [169260.0, 5387790.0],
                        [169260.0, 5387760.0],
                        [169230.0, 5387760.0],
                    ]
                ],
            },
        },
        1785: {
            "attributes": {"g_id": (5641, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387790.0],
                        [169230.0, 5387820.0],
                        [169260.0, 5387820.0],
                        [169260.0, 5387790.0],
                        [169230.0, 5387790.0],
                    ]
                ],
            },
        },
        1786: {
            "attributes": {"g_id": (5641, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387820.0],
                        [169230.0, 5387850.0],
                        [169260.0, 5387850.0],
                        [169260.0, 5387820.0],
                        [169230.0, 5387820.0],
                    ]
                ],
            },
        },
        1787: {
            "attributes": {"g_id": (5641, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387850.0],
                        [169230.0, 5387880.0],
                        [169260.0, 5387880.0],
                        [169260.0, 5387850.0],
                        [169230.0, 5387850.0],
                    ]
                ],
            },
        },
        1788: {
            "attributes": {"g_id": (5641, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387880.0],
                        [169230.0, 5387910.0],
                        [169260.0, 5387910.0],
                        [169260.0, 5387880.0],
                        [169230.0, 5387880.0],
                    ]
                ],
            },
        },
        1789: {
            "attributes": {"g_id": (5641, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387910.0],
                        [169230.0, 5387940.0],
                        [169260.0, 5387940.0],
                        [169260.0, 5387910.0],
                        [169230.0, 5387910.0],
                    ]
                ],
            },
        },
        1790: {
            "attributes": {"g_id": (5641, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387940.0],
                        [169230.0, 5387970.0],
                        [169260.0, 5387970.0],
                        [169260.0, 5387940.0],
                        [169230.0, 5387940.0],
                    ]
                ],
            },
        },
        1791: {
            "attributes": {"g_id": (5641, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5387970.0],
                        [169230.0, 5388000.0],
                        [169260.0, 5388000.0],
                        [169260.0, 5387970.0],
                        [169230.0, 5387970.0],
                    ]
                ],
            },
        },
        1792: {
            "attributes": {"g_id": (5641, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388000.0],
                        [169230.0, 5388030.0],
                        [169260.0, 5388030.0],
                        [169260.0, 5388000.0],
                        [169230.0, 5388000.0],
                    ]
                ],
            },
        },
        1793: {
            "attributes": {"g_id": (5641, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388030.0],
                        [169230.0, 5388060.0],
                        [169260.0, 5388060.0],
                        [169260.0, 5388030.0],
                        [169230.0, 5388030.0],
                    ]
                ],
            },
        },
        1794: {
            "attributes": {"g_id": (5641, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388060.0],
                        [169230.0, 5388090.0],
                        [169260.0, 5388090.0],
                        [169260.0, 5388060.0],
                        [169230.0, 5388060.0],
                    ]
                ],
            },
        },
        1795: {
            "attributes": {"g_id": (5641, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388090.0],
                        [169230.0, 5388120.0],
                        [169260.0, 5388120.0],
                        [169260.0, 5388090.0],
                        [169230.0, 5388090.0],
                    ]
                ],
            },
        },
        1796: {
            "attributes": {"g_id": (5641, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388120.0],
                        [169230.0, 5388150.0],
                        [169260.0, 5388150.0],
                        [169260.0, 5388120.0],
                        [169230.0, 5388120.0],
                    ]
                ],
            },
        },
        1797: {
            "attributes": {"g_id": (5641, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388150.0],
                        [169230.0, 5388180.0],
                        [169260.0, 5388180.0],
                        [169260.0, 5388150.0],
                        [169230.0, 5388150.0],
                    ]
                ],
            },
        },
        1798: {
            "attributes": {"g_id": (5641, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388180.0],
                        [169230.0, 5388210.0],
                        [169260.0, 5388210.0],
                        [169260.0, 5388180.0],
                        [169230.0, 5388180.0],
                    ]
                ],
            },
        },
        1799: {
            "attributes": {"g_id": (5641, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388210.0],
                        [169230.0, 5388240.0],
                        [169260.0, 5388240.0],
                        [169260.0, 5388210.0],
                        [169230.0, 5388210.0],
                    ]
                ],
            },
        },
        1800: {
            "attributes": {"g_id": (5641, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388240.0],
                        [169230.0, 5388270.0],
                        [169260.0, 5388270.0],
                        [169260.0, 5388240.0],
                        [169230.0, 5388240.0],
                    ]
                ],
            },
        },
        1801: {
            "attributes": {"g_id": (5641, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388270.0],
                        [169230.0, 5388300.0],
                        [169260.0, 5388300.0],
                        [169260.0, 5388270.0],
                        [169230.0, 5388270.0],
                    ]
                ],
            },
        },
        1802: {
            "attributes": {"g_id": (5641, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388300.0],
                        [169230.0, 5388330.0],
                        [169260.0, 5388330.0],
                        [169260.0, 5388300.0],
                        [169230.0, 5388300.0],
                    ]
                ],
            },
        },
        1803: {
            "attributes": {"g_id": (5641, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388330.0],
                        [169230.0, 5388360.0],
                        [169260.0, 5388360.0],
                        [169260.0, 5388330.0],
                        [169230.0, 5388330.0],
                    ]
                ],
            },
        },
        1804: {
            "attributes": {"g_id": (5641, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388360.0],
                        [169230.0, 5388390.0],
                        [169260.0, 5388390.0],
                        [169260.0, 5388360.0],
                        [169230.0, 5388360.0],
                    ]
                ],
            },
        },
        1805: {
            "attributes": {"g_id": (5641, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388390.0],
                        [169230.0, 5388420.0],
                        [169260.0, 5388420.0],
                        [169260.0, 5388390.0],
                        [169230.0, 5388390.0],
                    ]
                ],
            },
        },
        1806: {
            "attributes": {"g_id": (5641, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388420.0],
                        [169230.0, 5388450.0],
                        [169260.0, 5388450.0],
                        [169260.0, 5388420.0],
                        [169230.0, 5388420.0],
                    ]
                ],
            },
        },
        1807: {
            "attributes": {"g_id": (5641, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388450.0],
                        [169230.0, 5388480.0],
                        [169260.0, 5388480.0],
                        [169260.0, 5388450.0],
                        [169230.0, 5388450.0],
                    ]
                ],
            },
        },
        1808: {
            "attributes": {"g_id": (5641, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388480.0],
                        [169230.0, 5388510.0],
                        [169260.0, 5388510.0],
                        [169260.0, 5388480.0],
                        [169230.0, 5388480.0],
                    ]
                ],
            },
        },
        1809: {
            "attributes": {"g_id": (5641, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388510.0],
                        [169230.0, 5388540.0],
                        [169260.0, 5388540.0],
                        [169260.0, 5388510.0],
                        [169230.0, 5388510.0],
                    ]
                ],
            },
        },
        1810: {
            "attributes": {"g_id": (5641, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388540.0],
                        [169230.0, 5388570.0],
                        [169260.0, 5388570.0],
                        [169260.0, 5388540.0],
                        [169230.0, 5388540.0],
                    ]
                ],
            },
        },
        1811: {
            "attributes": {"g_id": (5641, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388570.0],
                        [169230.0, 5388600.0],
                        [169260.0, 5388600.0],
                        [169260.0, 5388570.0],
                        [169230.0, 5388570.0],
                    ]
                ],
            },
        },
        1812: {
            "attributes": {"g_id": (5641, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388600.0],
                        [169230.0, 5388630.0],
                        [169260.0, 5388630.0],
                        [169260.0, 5388600.0],
                        [169230.0, 5388600.0],
                    ]
                ],
            },
        },
        1813: {
            "attributes": {"g_id": (5641, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388630.0],
                        [169230.0, 5388660.0],
                        [169260.0, 5388660.0],
                        [169260.0, 5388630.0],
                        [169230.0, 5388630.0],
                    ]
                ],
            },
        },
        1814: {
            "attributes": {"g_id": (5641, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388660.0],
                        [169230.0, 5388690.0],
                        [169260.0, 5388690.0],
                        [169260.0, 5388660.0],
                        [169230.0, 5388660.0],
                    ]
                ],
            },
        },
        1815: {
            "attributes": {"g_id": (5641, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388690.0],
                        [169230.0, 5388720.0],
                        [169260.0, 5388720.0],
                        [169260.0, 5388690.0],
                        [169230.0, 5388690.0],
                    ]
                ],
            },
        },
        1816: {
            "attributes": {"g_id": (5641, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388720.0],
                        [169230.0, 5388750.0],
                        [169260.0, 5388750.0],
                        [169260.0, 5388720.0],
                        [169230.0, 5388720.0],
                    ]
                ],
            },
        },
        1817: {
            "attributes": {"g_id": (5641, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388750.0],
                        [169230.0, 5388780.0],
                        [169260.0, 5388780.0],
                        [169260.0, 5388750.0],
                        [169230.0, 5388750.0],
                    ]
                ],
            },
        },
        1818: {
            "attributes": {"g_id": (5641, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388780.0],
                        [169230.0, 5388810.0],
                        [169260.0, 5388810.0],
                        [169260.0, 5388780.0],
                        [169230.0, 5388780.0],
                    ]
                ],
            },
        },
        1819: {
            "attributes": {"g_id": (5641, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388810.0],
                        [169230.0, 5388840.0],
                        [169260.0, 5388840.0],
                        [169260.0, 5388810.0],
                        [169230.0, 5388810.0],
                    ]
                ],
            },
        },
        1820: {
            "attributes": {"g_id": (5641, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388840.0],
                        [169230.0, 5388870.0],
                        [169260.0, 5388870.0],
                        [169260.0, 5388840.0],
                        [169230.0, 5388840.0],
                    ]
                ],
            },
        },
        1821: {
            "attributes": {"g_id": (5641, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388870.0],
                        [169230.0, 5388900.0],
                        [169260.0, 5388900.0],
                        [169260.0, 5388870.0],
                        [169230.0, 5388870.0],
                    ]
                ],
            },
        },
        1822: {
            "attributes": {"g_id": (5641, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388900.0],
                        [169230.0, 5388930.0],
                        [169260.0, 5388930.0],
                        [169260.0, 5388900.0],
                        [169230.0, 5388900.0],
                    ]
                ],
            },
        },
        1823: {
            "attributes": {"g_id": (5641, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169230.0, 5388930.0],
                        [169230.0, 5388960.0],
                        [169260.0, 5388960.0],
                        [169260.0, 5388930.0],
                        [169230.0, 5388930.0],
                    ]
                ],
            },
        },
        1824: {
            "attributes": {"g_id": (5642, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387520.0],
                        [169260.0, 5387550.0],
                        [169290.0, 5387550.0],
                        [169290.0, 5387520.0],
                        [169260.0, 5387520.0],
                    ]
                ],
            },
        },
        1825: {
            "attributes": {"g_id": (5642, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387550.0],
                        [169260.0, 5387580.0],
                        [169290.0, 5387580.0],
                        [169290.0, 5387550.0],
                        [169260.0, 5387550.0],
                    ]
                ],
            },
        },
        1826: {
            "attributes": {"g_id": (5642, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387580.0],
                        [169260.0, 5387610.0],
                        [169290.0, 5387610.0],
                        [169290.0, 5387580.0],
                        [169260.0, 5387580.0],
                    ]
                ],
            },
        },
        1827: {
            "attributes": {"g_id": (5642, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387610.0],
                        [169260.0, 5387640.0],
                        [169290.0, 5387640.0],
                        [169290.0, 5387610.0],
                        [169260.0, 5387610.0],
                    ]
                ],
            },
        },
        1828: {
            "attributes": {"g_id": (5642, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387640.0],
                        [169260.0, 5387670.0],
                        [169290.0, 5387670.0],
                        [169290.0, 5387640.0],
                        [169260.0, 5387640.0],
                    ]
                ],
            },
        },
        1829: {
            "attributes": {"g_id": (5642, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387670.0],
                        [169260.0, 5387700.0],
                        [169290.0, 5387700.0],
                        [169290.0, 5387670.0],
                        [169260.0, 5387670.0],
                    ]
                ],
            },
        },
        1830: {
            "attributes": {"g_id": (5642, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387700.0],
                        [169260.0, 5387730.0],
                        [169290.0, 5387730.0],
                        [169290.0, 5387700.0],
                        [169260.0, 5387700.0],
                    ]
                ],
            },
        },
        1831: {
            "attributes": {"g_id": (5642, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387730.0],
                        [169260.0, 5387760.0],
                        [169290.0, 5387760.0],
                        [169290.0, 5387730.0],
                        [169260.0, 5387730.0],
                    ]
                ],
            },
        },
        1832: {
            "attributes": {"g_id": (5642, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387760.0],
                        [169260.0, 5387790.0],
                        [169290.0, 5387790.0],
                        [169290.0, 5387760.0],
                        [169260.0, 5387760.0],
                    ]
                ],
            },
        },
        1833: {
            "attributes": {"g_id": (5642, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387790.0],
                        [169260.0, 5387820.0],
                        [169290.0, 5387820.0],
                        [169290.0, 5387790.0],
                        [169260.0, 5387790.0],
                    ]
                ],
            },
        },
        1834: {
            "attributes": {"g_id": (5642, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387820.0],
                        [169260.0, 5387850.0],
                        [169290.0, 5387850.0],
                        [169290.0, 5387820.0],
                        [169260.0, 5387820.0],
                    ]
                ],
            },
        },
        1835: {
            "attributes": {"g_id": (5642, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387850.0],
                        [169260.0, 5387880.0],
                        [169290.0, 5387880.0],
                        [169290.0, 5387850.0],
                        [169260.0, 5387850.0],
                    ]
                ],
            },
        },
        1836: {
            "attributes": {"g_id": (5642, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387880.0],
                        [169260.0, 5387910.0],
                        [169290.0, 5387910.0],
                        [169290.0, 5387880.0],
                        [169260.0, 5387880.0],
                    ]
                ],
            },
        },
        1837: {
            "attributes": {"g_id": (5642, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387910.0],
                        [169260.0, 5387940.0],
                        [169290.0, 5387940.0],
                        [169290.0, 5387910.0],
                        [169260.0, 5387910.0],
                    ]
                ],
            },
        },
        1838: {
            "attributes": {"g_id": (5642, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387940.0],
                        [169260.0, 5387970.0],
                        [169290.0, 5387970.0],
                        [169290.0, 5387940.0],
                        [169260.0, 5387940.0],
                    ]
                ],
            },
        },
        1839: {
            "attributes": {"g_id": (5642, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5387970.0],
                        [169260.0, 5388000.0],
                        [169290.0, 5388000.0],
                        [169290.0, 5387970.0],
                        [169260.0, 5387970.0],
                    ]
                ],
            },
        },
        1840: {
            "attributes": {"g_id": (5642, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388000.0],
                        [169260.0, 5388030.0],
                        [169290.0, 5388030.0],
                        [169290.0, 5388000.0],
                        [169260.0, 5388000.0],
                    ]
                ],
            },
        },
        1841: {
            "attributes": {"g_id": (5642, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388030.0],
                        [169260.0, 5388060.0],
                        [169290.0, 5388060.0],
                        [169290.0, 5388030.0],
                        [169260.0, 5388030.0],
                    ]
                ],
            },
        },
        1842: {
            "attributes": {"g_id": (5642, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388060.0],
                        [169260.0, 5388090.0],
                        [169290.0, 5388090.0],
                        [169290.0, 5388060.0],
                        [169260.0, 5388060.0],
                    ]
                ],
            },
        },
        1843: {
            "attributes": {"g_id": (5642, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388090.0],
                        [169260.0, 5388120.0],
                        [169290.0, 5388120.0],
                        [169290.0, 5388090.0],
                        [169260.0, 5388090.0],
                    ]
                ],
            },
        },
        1844: {
            "attributes": {"g_id": (5642, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388120.0],
                        [169260.0, 5388150.0],
                        [169290.0, 5388150.0],
                        [169290.0, 5388120.0],
                        [169260.0, 5388120.0],
                    ]
                ],
            },
        },
        1845: {
            "attributes": {"g_id": (5642, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388150.0],
                        [169260.0, 5388180.0],
                        [169290.0, 5388180.0],
                        [169290.0, 5388150.0],
                        [169260.0, 5388150.0],
                    ]
                ],
            },
        },
        1846: {
            "attributes": {"g_id": (5642, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388180.0],
                        [169260.0, 5388210.0],
                        [169290.0, 5388210.0],
                        [169290.0, 5388180.0],
                        [169260.0, 5388180.0],
                    ]
                ],
            },
        },
        1847: {
            "attributes": {"g_id": (5642, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388210.0],
                        [169260.0, 5388240.0],
                        [169290.0, 5388240.0],
                        [169290.0, 5388210.0],
                        [169260.0, 5388210.0],
                    ]
                ],
            },
        },
        1848: {
            "attributes": {"g_id": (5642, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388240.0],
                        [169260.0, 5388270.0],
                        [169290.0, 5388270.0],
                        [169290.0, 5388240.0],
                        [169260.0, 5388240.0],
                    ]
                ],
            },
        },
        1849: {
            "attributes": {"g_id": (5642, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388270.0],
                        [169260.0, 5388300.0],
                        [169290.0, 5388300.0],
                        [169290.0, 5388270.0],
                        [169260.0, 5388270.0],
                    ]
                ],
            },
        },
        1850: {
            "attributes": {"g_id": (5642, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388300.0],
                        [169260.0, 5388330.0],
                        [169290.0, 5388330.0],
                        [169290.0, 5388300.0],
                        [169260.0, 5388300.0],
                    ]
                ],
            },
        },
        1851: {
            "attributes": {"g_id": (5642, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388330.0],
                        [169260.0, 5388360.0],
                        [169290.0, 5388360.0],
                        [169290.0, 5388330.0],
                        [169260.0, 5388330.0],
                    ]
                ],
            },
        },
        1852: {
            "attributes": {"g_id": (5642, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388360.0],
                        [169260.0, 5388390.0],
                        [169290.0, 5388390.0],
                        [169290.0, 5388360.0],
                        [169260.0, 5388360.0],
                    ]
                ],
            },
        },
        1853: {
            "attributes": {"g_id": (5642, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388390.0],
                        [169260.0, 5388420.0],
                        [169290.0, 5388420.0],
                        [169290.0, 5388390.0],
                        [169260.0, 5388390.0],
                    ]
                ],
            },
        },
        1854: {
            "attributes": {"g_id": (5642, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388420.0],
                        [169260.0, 5388450.0],
                        [169290.0, 5388450.0],
                        [169290.0, 5388420.0],
                        [169260.0, 5388420.0],
                    ]
                ],
            },
        },
        1855: {
            "attributes": {"g_id": (5642, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388450.0],
                        [169260.0, 5388480.0],
                        [169290.0, 5388480.0],
                        [169290.0, 5388450.0],
                        [169260.0, 5388450.0],
                    ]
                ],
            },
        },
        1856: {
            "attributes": {"g_id": (5642, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388480.0],
                        [169260.0, 5388510.0],
                        [169290.0, 5388510.0],
                        [169290.0, 5388480.0],
                        [169260.0, 5388480.0],
                    ]
                ],
            },
        },
        1857: {
            "attributes": {"g_id": (5642, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388510.0],
                        [169260.0, 5388540.0],
                        [169290.0, 5388540.0],
                        [169290.0, 5388510.0],
                        [169260.0, 5388510.0],
                    ]
                ],
            },
        },
        1858: {
            "attributes": {"g_id": (5642, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388540.0],
                        [169260.0, 5388570.0],
                        [169290.0, 5388570.0],
                        [169290.0, 5388540.0],
                        [169260.0, 5388540.0],
                    ]
                ],
            },
        },
        1859: {
            "attributes": {"g_id": (5642, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388570.0],
                        [169260.0, 5388600.0],
                        [169290.0, 5388600.0],
                        [169290.0, 5388570.0],
                        [169260.0, 5388570.0],
                    ]
                ],
            },
        },
        1860: {
            "attributes": {"g_id": (5642, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388600.0],
                        [169260.0, 5388630.0],
                        [169290.0, 5388630.0],
                        [169290.0, 5388600.0],
                        [169260.0, 5388600.0],
                    ]
                ],
            },
        },
        1861: {
            "attributes": {"g_id": (5642, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388630.0],
                        [169260.0, 5388660.0],
                        [169290.0, 5388660.0],
                        [169290.0, 5388630.0],
                        [169260.0, 5388630.0],
                    ]
                ],
            },
        },
        1862: {
            "attributes": {"g_id": (5642, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388660.0],
                        [169260.0, 5388690.0],
                        [169290.0, 5388690.0],
                        [169290.0, 5388660.0],
                        [169260.0, 5388660.0],
                    ]
                ],
            },
        },
        1863: {
            "attributes": {"g_id": (5642, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388690.0],
                        [169260.0, 5388720.0],
                        [169290.0, 5388720.0],
                        [169290.0, 5388690.0],
                        [169260.0, 5388690.0],
                    ]
                ],
            },
        },
        1864: {
            "attributes": {"g_id": (5642, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388720.0],
                        [169260.0, 5388750.0],
                        [169290.0, 5388750.0],
                        [169290.0, 5388720.0],
                        [169260.0, 5388720.0],
                    ]
                ],
            },
        },
        1865: {
            "attributes": {"g_id": (5642, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388750.0],
                        [169260.0, 5388780.0],
                        [169290.0, 5388780.0],
                        [169290.0, 5388750.0],
                        [169260.0, 5388750.0],
                    ]
                ],
            },
        },
        1866: {
            "attributes": {"g_id": (5642, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388780.0],
                        [169260.0, 5388810.0],
                        [169290.0, 5388810.0],
                        [169290.0, 5388780.0],
                        [169260.0, 5388780.0],
                    ]
                ],
            },
        },
        1867: {
            "attributes": {"g_id": (5642, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388810.0],
                        [169260.0, 5388840.0],
                        [169290.0, 5388840.0],
                        [169290.0, 5388810.0],
                        [169260.0, 5388810.0],
                    ]
                ],
            },
        },
        1868: {
            "attributes": {"g_id": (5642, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388840.0],
                        [169260.0, 5388870.0],
                        [169290.0, 5388870.0],
                        [169290.0, 5388840.0],
                        [169260.0, 5388840.0],
                    ]
                ],
            },
        },
        1869: {
            "attributes": {"g_id": (5642, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388870.0],
                        [169260.0, 5388900.0],
                        [169290.0, 5388900.0],
                        [169290.0, 5388870.0],
                        [169260.0, 5388870.0],
                    ]
                ],
            },
        },
        1870: {
            "attributes": {"g_id": (5642, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388900.0],
                        [169260.0, 5388930.0],
                        [169290.0, 5388930.0],
                        [169290.0, 5388900.0],
                        [169260.0, 5388900.0],
                    ]
                ],
            },
        },
        1871: {
            "attributes": {"g_id": (5642, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169260.0, 5388930.0],
                        [169260.0, 5388960.0],
                        [169290.0, 5388960.0],
                        [169290.0, 5388930.0],
                        [169260.0, 5388930.0],
                    ]
                ],
            },
        },
        1872: {
            "attributes": {"g_id": (5643, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387520.0],
                        [169290.0, 5387550.0],
                        [169320.0, 5387550.0],
                        [169320.0, 5387520.0],
                        [169290.0, 5387520.0],
                    ]
                ],
            },
        },
        1873: {
            "attributes": {"g_id": (5643, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387550.0],
                        [169290.0, 5387580.0],
                        [169320.0, 5387580.0],
                        [169320.0, 5387550.0],
                        [169290.0, 5387550.0],
                    ]
                ],
            },
        },
        1874: {
            "attributes": {"g_id": (5643, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387580.0],
                        [169290.0, 5387610.0],
                        [169320.0, 5387610.0],
                        [169320.0, 5387580.0],
                        [169290.0, 5387580.0],
                    ]
                ],
            },
        },
        1875: {
            "attributes": {"g_id": (5643, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387610.0],
                        [169290.0, 5387640.0],
                        [169320.0, 5387640.0],
                        [169320.0, 5387610.0],
                        [169290.0, 5387610.0],
                    ]
                ],
            },
        },
        1876: {
            "attributes": {"g_id": (5643, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387640.0],
                        [169290.0, 5387670.0],
                        [169320.0, 5387670.0],
                        [169320.0, 5387640.0],
                        [169290.0, 5387640.0],
                    ]
                ],
            },
        },
        1877: {
            "attributes": {"g_id": (5643, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387670.0],
                        [169290.0, 5387700.0],
                        [169320.0, 5387700.0],
                        [169320.0, 5387670.0],
                        [169290.0, 5387670.0],
                    ]
                ],
            },
        },
        1878: {
            "attributes": {"g_id": (5643, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387700.0],
                        [169290.0, 5387730.0],
                        [169320.0, 5387730.0],
                        [169320.0, 5387700.0],
                        [169290.0, 5387700.0],
                    ]
                ],
            },
        },
        1879: {
            "attributes": {"g_id": (5643, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387730.0],
                        [169290.0, 5387760.0],
                        [169320.0, 5387760.0],
                        [169320.0, 5387730.0],
                        [169290.0, 5387730.0],
                    ]
                ],
            },
        },
        1880: {
            "attributes": {"g_id": (5643, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387760.0],
                        [169290.0, 5387790.0],
                        [169320.0, 5387790.0],
                        [169320.0, 5387760.0],
                        [169290.0, 5387760.0],
                    ]
                ],
            },
        },
        1881: {
            "attributes": {"g_id": (5643, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387790.0],
                        [169290.0, 5387820.0],
                        [169320.0, 5387820.0],
                        [169320.0, 5387790.0],
                        [169290.0, 5387790.0],
                    ]
                ],
            },
        },
        1882: {
            "attributes": {"g_id": (5643, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387820.0],
                        [169290.0, 5387850.0],
                        [169320.0, 5387850.0],
                        [169320.0, 5387820.0],
                        [169290.0, 5387820.0],
                    ]
                ],
            },
        },
        1883: {
            "attributes": {"g_id": (5643, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387850.0],
                        [169290.0, 5387880.0],
                        [169320.0, 5387880.0],
                        [169320.0, 5387850.0],
                        [169290.0, 5387850.0],
                    ]
                ],
            },
        },
        1884: {
            "attributes": {"g_id": (5643, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387880.0],
                        [169290.0, 5387910.0],
                        [169320.0, 5387910.0],
                        [169320.0, 5387880.0],
                        [169290.0, 5387880.0],
                    ]
                ],
            },
        },
        1885: {
            "attributes": {"g_id": (5643, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387910.0],
                        [169290.0, 5387940.0],
                        [169320.0, 5387940.0],
                        [169320.0, 5387910.0],
                        [169290.0, 5387910.0],
                    ]
                ],
            },
        },
        1886: {
            "attributes": {"g_id": (5643, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387940.0],
                        [169290.0, 5387970.0],
                        [169320.0, 5387970.0],
                        [169320.0, 5387940.0],
                        [169290.0, 5387940.0],
                    ]
                ],
            },
        },
        1887: {
            "attributes": {"g_id": (5643, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5387970.0],
                        [169290.0, 5388000.0],
                        [169320.0, 5388000.0],
                        [169320.0, 5387970.0],
                        [169290.0, 5387970.0],
                    ]
                ],
            },
        },
        1888: {
            "attributes": {"g_id": (5643, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388000.0],
                        [169290.0, 5388030.0],
                        [169320.0, 5388030.0],
                        [169320.0, 5388000.0],
                        [169290.0, 5388000.0],
                    ]
                ],
            },
        },
        1889: {
            "attributes": {"g_id": (5643, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388030.0],
                        [169290.0, 5388060.0],
                        [169320.0, 5388060.0],
                        [169320.0, 5388030.0],
                        [169290.0, 5388030.0],
                    ]
                ],
            },
        },
        1890: {
            "attributes": {"g_id": (5643, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388060.0],
                        [169290.0, 5388090.0],
                        [169320.0, 5388090.0],
                        [169320.0, 5388060.0],
                        [169290.0, 5388060.0],
                    ]
                ],
            },
        },
        1891: {
            "attributes": {"g_id": (5643, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388090.0],
                        [169290.0, 5388120.0],
                        [169320.0, 5388120.0],
                        [169320.0, 5388090.0],
                        [169290.0, 5388090.0],
                    ]
                ],
            },
        },
        1892: {
            "attributes": {"g_id": (5643, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388120.0],
                        [169290.0, 5388150.0],
                        [169320.0, 5388150.0],
                        [169320.0, 5388120.0],
                        [169290.0, 5388120.0],
                    ]
                ],
            },
        },
        1893: {
            "attributes": {"g_id": (5643, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388150.0],
                        [169290.0, 5388180.0],
                        [169320.0, 5388180.0],
                        [169320.0, 5388150.0],
                        [169290.0, 5388150.0],
                    ]
                ],
            },
        },
        1894: {
            "attributes": {"g_id": (5643, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388180.0],
                        [169290.0, 5388210.0],
                        [169320.0, 5388210.0],
                        [169320.0, 5388180.0],
                        [169290.0, 5388180.0],
                    ]
                ],
            },
        },
        1895: {
            "attributes": {"g_id": (5643, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388210.0],
                        [169290.0, 5388240.0],
                        [169320.0, 5388240.0],
                        [169320.0, 5388210.0],
                        [169290.0, 5388210.0],
                    ]
                ],
            },
        },
        1896: {
            "attributes": {"g_id": (5643, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388240.0],
                        [169290.0, 5388270.0],
                        [169320.0, 5388270.0],
                        [169320.0, 5388240.0],
                        [169290.0, 5388240.0],
                    ]
                ],
            },
        },
        1897: {
            "attributes": {"g_id": (5643, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388270.0],
                        [169290.0, 5388300.0],
                        [169320.0, 5388300.0],
                        [169320.0, 5388270.0],
                        [169290.0, 5388270.0],
                    ]
                ],
            },
        },
        1898: {
            "attributes": {"g_id": (5643, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388300.0],
                        [169290.0, 5388330.0],
                        [169320.0, 5388330.0],
                        [169320.0, 5388300.0],
                        [169290.0, 5388300.0],
                    ]
                ],
            },
        },
        1899: {
            "attributes": {"g_id": (5643, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388330.0],
                        [169290.0, 5388360.0],
                        [169320.0, 5388360.0],
                        [169320.0, 5388330.0],
                        [169290.0, 5388330.0],
                    ]
                ],
            },
        },
        1900: {
            "attributes": {"g_id": (5643, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388360.0],
                        [169290.0, 5388390.0],
                        [169320.0, 5388390.0],
                        [169320.0, 5388360.0],
                        [169290.0, 5388360.0],
                    ]
                ],
            },
        },
        1901: {
            "attributes": {"g_id": (5643, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388390.0],
                        [169290.0, 5388420.0],
                        [169320.0, 5388420.0],
                        [169320.0, 5388390.0],
                        [169290.0, 5388390.0],
                    ]
                ],
            },
        },
        1902: {
            "attributes": {"g_id": (5643, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388420.0],
                        [169290.0, 5388450.0],
                        [169320.0, 5388450.0],
                        [169320.0, 5388420.0],
                        [169290.0, 5388420.0],
                    ]
                ],
            },
        },
        1903: {
            "attributes": {"g_id": (5643, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388450.0],
                        [169290.0, 5388480.0],
                        [169320.0, 5388480.0],
                        [169320.0, 5388450.0],
                        [169290.0, 5388450.0],
                    ]
                ],
            },
        },
        1904: {
            "attributes": {"g_id": (5643, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388480.0],
                        [169290.0, 5388510.0],
                        [169320.0, 5388510.0],
                        [169320.0, 5388480.0],
                        [169290.0, 5388480.0],
                    ]
                ],
            },
        },
        1905: {
            "attributes": {"g_id": (5643, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388510.0],
                        [169290.0, 5388540.0],
                        [169320.0, 5388540.0],
                        [169320.0, 5388510.0],
                        [169290.0, 5388510.0],
                    ]
                ],
            },
        },
        1906: {
            "attributes": {"g_id": (5643, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388540.0],
                        [169290.0, 5388570.0],
                        [169320.0, 5388570.0],
                        [169320.0, 5388540.0],
                        [169290.0, 5388540.0],
                    ]
                ],
            },
        },
        1907: {
            "attributes": {"g_id": (5643, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388570.0],
                        [169290.0, 5388600.0],
                        [169320.0, 5388600.0],
                        [169320.0, 5388570.0],
                        [169290.0, 5388570.0],
                    ]
                ],
            },
        },
        1908: {
            "attributes": {"g_id": (5643, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388600.0],
                        [169290.0, 5388630.0],
                        [169320.0, 5388630.0],
                        [169320.0, 5388600.0],
                        [169290.0, 5388600.0],
                    ]
                ],
            },
        },
        1909: {
            "attributes": {"g_id": (5643, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388630.0],
                        [169290.0, 5388660.0],
                        [169320.0, 5388660.0],
                        [169320.0, 5388630.0],
                        [169290.0, 5388630.0],
                    ]
                ],
            },
        },
        1910: {
            "attributes": {"g_id": (5643, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388660.0],
                        [169290.0, 5388690.0],
                        [169320.0, 5388690.0],
                        [169320.0, 5388660.0],
                        [169290.0, 5388660.0],
                    ]
                ],
            },
        },
        1911: {
            "attributes": {"g_id": (5643, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388690.0],
                        [169290.0, 5388720.0],
                        [169320.0, 5388720.0],
                        [169320.0, 5388690.0],
                        [169290.0, 5388690.0],
                    ]
                ],
            },
        },
        1912: {
            "attributes": {"g_id": (5643, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388720.0],
                        [169290.0, 5388750.0],
                        [169320.0, 5388750.0],
                        [169320.0, 5388720.0],
                        [169290.0, 5388720.0],
                    ]
                ],
            },
        },
        1913: {
            "attributes": {"g_id": (5643, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388750.0],
                        [169290.0, 5388780.0],
                        [169320.0, 5388780.0],
                        [169320.0, 5388750.0],
                        [169290.0, 5388750.0],
                    ]
                ],
            },
        },
        1914: {
            "attributes": {"g_id": (5643, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388780.0],
                        [169290.0, 5388810.0],
                        [169320.0, 5388810.0],
                        [169320.0, 5388780.0],
                        [169290.0, 5388780.0],
                    ]
                ],
            },
        },
        1915: {
            "attributes": {"g_id": (5643, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388810.0],
                        [169290.0, 5388840.0],
                        [169320.0, 5388840.0],
                        [169320.0, 5388810.0],
                        [169290.0, 5388810.0],
                    ]
                ],
            },
        },
        1916: {
            "attributes": {"g_id": (5643, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388840.0],
                        [169290.0, 5388870.0],
                        [169320.0, 5388870.0],
                        [169320.0, 5388840.0],
                        [169290.0, 5388840.0],
                    ]
                ],
            },
        },
        1917: {
            "attributes": {"g_id": (5643, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388870.0],
                        [169290.0, 5388900.0],
                        [169320.0, 5388900.0],
                        [169320.0, 5388870.0],
                        [169290.0, 5388870.0],
                    ]
                ],
            },
        },
        1918: {
            "attributes": {"g_id": (5643, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388900.0],
                        [169290.0, 5388930.0],
                        [169320.0, 5388930.0],
                        [169320.0, 5388900.0],
                        [169290.0, 5388900.0],
                    ]
                ],
            },
        },
        1919: {
            "attributes": {"g_id": (5643, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169290.0, 5388930.0],
                        [169290.0, 5388960.0],
                        [169320.0, 5388960.0],
                        [169320.0, 5388930.0],
                        [169290.0, 5388930.0],
                    ]
                ],
            },
        },
        1920: {
            "attributes": {"g_id": (5644, 179584), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387520.0],
                        [169320.0, 5387550.0],
                        [169350.0, 5387550.0],
                        [169350.0, 5387520.0],
                        [169320.0, 5387520.0],
                    ]
                ],
            },
        },
        1921: {
            "attributes": {"g_id": (5644, 179585), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387550.0],
                        [169320.0, 5387580.0],
                        [169350.0, 5387580.0],
                        [169350.0, 5387550.0],
                        [169320.0, 5387550.0],
                    ]
                ],
            },
        },
        1922: {
            "attributes": {"g_id": (5644, 179586), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387580.0],
                        [169320.0, 5387610.0],
                        [169350.0, 5387610.0],
                        [169350.0, 5387580.0],
                        [169320.0, 5387580.0],
                    ]
                ],
            },
        },
        1923: {
            "attributes": {"g_id": (5644, 179587), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387610.0],
                        [169320.0, 5387640.0],
                        [169350.0, 5387640.0],
                        [169350.0, 5387610.0],
                        [169320.0, 5387610.0],
                    ]
                ],
            },
        },
        1924: {
            "attributes": {"g_id": (5644, 179588), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387640.0],
                        [169320.0, 5387670.0],
                        [169350.0, 5387670.0],
                        [169350.0, 5387640.0],
                        [169320.0, 5387640.0],
                    ]
                ],
            },
        },
        1925: {
            "attributes": {"g_id": (5644, 179589), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387670.0],
                        [169320.0, 5387700.0],
                        [169350.0, 5387700.0],
                        [169350.0, 5387670.0],
                        [169320.0, 5387670.0],
                    ]
                ],
            },
        },
        1926: {
            "attributes": {"g_id": (5644, 179590), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387700.0],
                        [169320.0, 5387730.0],
                        [169350.0, 5387730.0],
                        [169350.0, 5387700.0],
                        [169320.0, 5387700.0],
                    ]
                ],
            },
        },
        1927: {
            "attributes": {"g_id": (5644, 179591), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387730.0],
                        [169320.0, 5387760.0],
                        [169350.0, 5387760.0],
                        [169350.0, 5387730.0],
                        [169320.0, 5387730.0],
                    ]
                ],
            },
        },
        1928: {
            "attributes": {"g_id": (5644, 179592), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387760.0],
                        [169320.0, 5387790.0],
                        [169350.0, 5387790.0],
                        [169350.0, 5387760.0],
                        [169320.0, 5387760.0],
                    ]
                ],
            },
        },
        1929: {
            "attributes": {"g_id": (5644, 179593), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387790.0],
                        [169320.0, 5387820.0],
                        [169350.0, 5387820.0],
                        [169350.0, 5387790.0],
                        [169320.0, 5387790.0],
                    ]
                ],
            },
        },
        1930: {
            "attributes": {"g_id": (5644, 179594), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387820.0],
                        [169320.0, 5387850.0],
                        [169350.0, 5387850.0],
                        [169350.0, 5387820.0],
                        [169320.0, 5387820.0],
                    ]
                ],
            },
        },
        1931: {
            "attributes": {"g_id": (5644, 179595), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387850.0],
                        [169320.0, 5387880.0],
                        [169350.0, 5387880.0],
                        [169350.0, 5387850.0],
                        [169320.0, 5387850.0],
                    ]
                ],
            },
        },
        1932: {
            "attributes": {"g_id": (5644, 179596), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387880.0],
                        [169320.0, 5387910.0],
                        [169350.0, 5387910.0],
                        [169350.0, 5387880.0],
                        [169320.0, 5387880.0],
                    ]
                ],
            },
        },
        1933: {
            "attributes": {"g_id": (5644, 179597), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387910.0],
                        [169320.0, 5387940.0],
                        [169350.0, 5387940.0],
                        [169350.0, 5387910.0],
                        [169320.0, 5387910.0],
                    ]
                ],
            },
        },
        1934: {
            "attributes": {"g_id": (5644, 179598), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387940.0],
                        [169320.0, 5387970.0],
                        [169350.0, 5387970.0],
                        [169350.0, 5387940.0],
                        [169320.0, 5387940.0],
                    ]
                ],
            },
        },
        1935: {
            "attributes": {"g_id": (5644, 179599), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5387970.0],
                        [169320.0, 5388000.0],
                        [169350.0, 5388000.0],
                        [169350.0, 5387970.0],
                        [169320.0, 5387970.0],
                    ]
                ],
            },
        },
        1936: {
            "attributes": {"g_id": (5644, 179600), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388000.0],
                        [169320.0, 5388030.0],
                        [169350.0, 5388030.0],
                        [169350.0, 5388000.0],
                        [169320.0, 5388000.0],
                    ]
                ],
            },
        },
        1937: {
            "attributes": {"g_id": (5644, 179601), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388030.0],
                        [169320.0, 5388060.0],
                        [169350.0, 5388060.0],
                        [169350.0, 5388030.0],
                        [169320.0, 5388030.0],
                    ]
                ],
            },
        },
        1938: {
            "attributes": {"g_id": (5644, 179602), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388060.0],
                        [169320.0, 5388090.0],
                        [169350.0, 5388090.0],
                        [169350.0, 5388060.0],
                        [169320.0, 5388060.0],
                    ]
                ],
            },
        },
        1939: {
            "attributes": {"g_id": (5644, 179603), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388090.0],
                        [169320.0, 5388120.0],
                        [169350.0, 5388120.0],
                        [169350.0, 5388090.0],
                        [169320.0, 5388090.0],
                    ]
                ],
            },
        },
        1940: {
            "attributes": {"g_id": (5644, 179604), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388120.0],
                        [169320.0, 5388150.0],
                        [169350.0, 5388150.0],
                        [169350.0, 5388120.0],
                        [169320.0, 5388120.0],
                    ]
                ],
            },
        },
        1941: {
            "attributes": {"g_id": (5644, 179605), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388150.0],
                        [169320.0, 5388180.0],
                        [169350.0, 5388180.0],
                        [169350.0, 5388150.0],
                        [169320.0, 5388150.0],
                    ]
                ],
            },
        },
        1942: {
            "attributes": {"g_id": (5644, 179606), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388180.0],
                        [169320.0, 5388210.0],
                        [169350.0, 5388210.0],
                        [169350.0, 5388180.0],
                        [169320.0, 5388180.0],
                    ]
                ],
            },
        },
        1943: {
            "attributes": {"g_id": (5644, 179607), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388210.0],
                        [169320.0, 5388240.0],
                        [169350.0, 5388240.0],
                        [169350.0, 5388210.0],
                        [169320.0, 5388210.0],
                    ]
                ],
            },
        },
        1944: {
            "attributes": {"g_id": (5644, 179608), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388240.0],
                        [169320.0, 5388270.0],
                        [169350.0, 5388270.0],
                        [169350.0, 5388240.0],
                        [169320.0, 5388240.0],
                    ]
                ],
            },
        },
        1945: {
            "attributes": {"g_id": (5644, 179609), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388270.0],
                        [169320.0, 5388300.0],
                        [169350.0, 5388300.0],
                        [169350.0, 5388270.0],
                        [169320.0, 5388270.0],
                    ]
                ],
            },
        },
        1946: {
            "attributes": {"g_id": (5644, 179610), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388300.0],
                        [169320.0, 5388330.0],
                        [169350.0, 5388330.0],
                        [169350.0, 5388300.0],
                        [169320.0, 5388300.0],
                    ]
                ],
            },
        },
        1947: {
            "attributes": {"g_id": (5644, 179611), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388330.0],
                        [169320.0, 5388360.0],
                        [169350.0, 5388360.0],
                        [169350.0, 5388330.0],
                        [169320.0, 5388330.0],
                    ]
                ],
            },
        },
        1948: {
            "attributes": {"g_id": (5644, 179612), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388360.0],
                        [169320.0, 5388390.0],
                        [169350.0, 5388390.0],
                        [169350.0, 5388360.0],
                        [169320.0, 5388360.0],
                    ]
                ],
            },
        },
        1949: {
            "attributes": {"g_id": (5644, 179613), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388390.0],
                        [169320.0, 5388420.0],
                        [169350.0, 5388420.0],
                        [169350.0, 5388390.0],
                        [169320.0, 5388390.0],
                    ]
                ],
            },
        },
        1950: {
            "attributes": {"g_id": (5644, 179614), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388420.0],
                        [169320.0, 5388450.0],
                        [169350.0, 5388450.0],
                        [169350.0, 5388420.0],
                        [169320.0, 5388420.0],
                    ]
                ],
            },
        },
        1951: {
            "attributes": {"g_id": (5644, 179615), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388450.0],
                        [169320.0, 5388480.0],
                        [169350.0, 5388480.0],
                        [169350.0, 5388450.0],
                        [169320.0, 5388450.0],
                    ]
                ],
            },
        },
        1952: {
            "attributes": {"g_id": (5644, 179616), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388480.0],
                        [169320.0, 5388510.0],
                        [169350.0, 5388510.0],
                        [169350.0, 5388480.0],
                        [169320.0, 5388480.0],
                    ]
                ],
            },
        },
        1953: {
            "attributes": {"g_id": (5644, 179617), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388510.0],
                        [169320.0, 5388540.0],
                        [169350.0, 5388540.0],
                        [169350.0, 5388510.0],
                        [169320.0, 5388510.0],
                    ]
                ],
            },
        },
        1954: {
            "attributes": {"g_id": (5644, 179618), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388540.0],
                        [169320.0, 5388570.0],
                        [169350.0, 5388570.0],
                        [169350.0, 5388540.0],
                        [169320.0, 5388540.0],
                    ]
                ],
            },
        },
        1955: {
            "attributes": {"g_id": (5644, 179619), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388570.0],
                        [169320.0, 5388600.0],
                        [169350.0, 5388600.0],
                        [169350.0, 5388570.0],
                        [169320.0, 5388570.0],
                    ]
                ],
            },
        },
        1956: {
            "attributes": {"g_id": (5644, 179620), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388600.0],
                        [169320.0, 5388630.0],
                        [169350.0, 5388630.0],
                        [169350.0, 5388600.0],
                        [169320.0, 5388600.0],
                    ]
                ],
            },
        },
        1957: {
            "attributes": {"g_id": (5644, 179621), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388630.0],
                        [169320.0, 5388660.0],
                        [169350.0, 5388660.0],
                        [169350.0, 5388630.0],
                        [169320.0, 5388630.0],
                    ]
                ],
            },
        },
        1958: {
            "attributes": {"g_id": (5644, 179622), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388660.0],
                        [169320.0, 5388690.0],
                        [169350.0, 5388690.0],
                        [169350.0, 5388660.0],
                        [169320.0, 5388660.0],
                    ]
                ],
            },
        },
        1959: {
            "attributes": {"g_id": (5644, 179623), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388690.0],
                        [169320.0, 5388720.0],
                        [169350.0, 5388720.0],
                        [169350.0, 5388690.0],
                        [169320.0, 5388690.0],
                    ]
                ],
            },
        },
        1960: {
            "attributes": {"g_id": (5644, 179624), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388720.0],
                        [169320.0, 5388750.0],
                        [169350.0, 5388750.0],
                        [169350.0, 5388720.0],
                        [169320.0, 5388720.0],
                    ]
                ],
            },
        },
        1961: {
            "attributes": {"g_id": (5644, 179625), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388750.0],
                        [169320.0, 5388780.0],
                        [169350.0, 5388780.0],
                        [169350.0, 5388750.0],
                        [169320.0, 5388750.0],
                    ]
                ],
            },
        },
        1962: {
            "attributes": {"g_id": (5644, 179626), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388780.0],
                        [169320.0, 5388810.0],
                        [169350.0, 5388810.0],
                        [169350.0, 5388780.0],
                        [169320.0, 5388780.0],
                    ]
                ],
            },
        },
        1963: {
            "attributes": {"g_id": (5644, 179627), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388810.0],
                        [169320.0, 5388840.0],
                        [169350.0, 5388840.0],
                        [169350.0, 5388810.0],
                        [169320.0, 5388810.0],
                    ]
                ],
            },
        },
        1964: {
            "attributes": {"g_id": (5644, 179628), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388840.0],
                        [169320.0, 5388870.0],
                        [169350.0, 5388870.0],
                        [169350.0, 5388840.0],
                        [169320.0, 5388840.0],
                    ]
                ],
            },
        },
        1965: {
            "attributes": {"g_id": (5644, 179629), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388870.0],
                        [169320.0, 5388900.0],
                        [169350.0, 5388900.0],
                        [169350.0, 5388870.0],
                        [169320.0, 5388870.0],
                    ]
                ],
            },
        },
        1966: {
            "attributes": {"g_id": (5644, 179630), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388900.0],
                        [169320.0, 5388930.0],
                        [169350.0, 5388930.0],
                        [169350.0, 5388900.0],
                        [169320.0, 5388900.0],
                    ]
                ],
            },
        },
        1967: {
            "attributes": {"g_id": (5644, 179631), "i_feat_list": [0]},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [169320.0, 5388930.0],
                        [169320.0, 5388960.0],
                        [169350.0, 5388960.0],
                        [169350.0, 5388930.0],
                        [169320.0, 5388930.0],
                    ]
                ],
            },
        },
    },
    "metadata": {
        "name": "grid_index",
        "geometry_ref": {"type": "Polygon"},
        "fields": {
            "g_id": {"type": "String", "width": 254, "index": 0},
            "i_feat_list": {"type": "IntegerList", "index": 1},
        },
    },
}

geolayer_paris_velib = {'metadata': {'name': 'velib-disponibilite-en-temps-reel', 'fields': {'Identifiant station': {'type': 'Integer', 'index': 0}, 'Nom station': {'type': 'String', 'width': 45, 'index': 1}, 'Station en fonctionnement': {'type': 'String', 'width': 3, 'index': 2}, 'Capacité de la station': {'type': 'Integer', 'index': 3}, 'Nombre bornettes libres': {'type': 'Integer', 'index': 4}, 'Nombre total vélos disponibles': {'type': 'Integer', 'index': 5}, 'Vélos mécaniques disponibles': {'type': 'Integer', 'index': 6}, 'Vélos électriques disponibles': {'type': 'Integer', 'index': 7}, 'Borne de paiement disponible': {'type': 'String', 'width': 3, 'index': 8}, 'Retour vélib possible': {'type': 'String', 'width': 3, 'index': 9}, 'Actualisation de la donnée': {'type': 'DateTime', 'index': 10}, 'Coordonnées géographiques': {'type': 'RealList', 'width': 13, 'precision': 11, 'index': 11}, 'Nom communes équipées': {'type': 'String', 'width': 20, 'index': 12}}}, 'features': {0: {'attributes': {'Identifiant station': 11104, 'Nom station': 'Charonne - Robert et Sonia Delauney', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.855907556, 2.39257067442], 'Nom communes équipées': 'Paris'}}, 1: {'attributes': {'Identifiant station': 16107, 'Nom station': 'Benjamin Godard - Victor Hugo', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.865983, 2.275725], 'Nom communes équipées': 'Paris'}}, 2: {'attributes': {'Identifiant station': 14014, 'Nom station': 'Jourdan - Stade Charléty', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 60, 'Nombre bornettes libres': 56, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8194283334, 2.34333537519], 'Nom communes équipées': 'Paris'}}, 3: {'attributes': {'Identifiant station': 9020, 'Nom station': 'Toudouze - Clauzel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8792959173, 2.33736008406], 'Nom communes équipées': 'Paris'}}, 4: {'attributes': {'Identifiant station': 13007, 'Nom station': 'Le Brun - Gobelins', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 48, 'Nombre bornettes libres': 38, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8350927878, 2.35346813513], 'Nom communes équipées': 'Paris'}}, 5: {'attributes': {'Identifiant station': 12109, 'Nom station': 'Mairie du 12ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8408553118, 2.38755494356], 'Nom communes équipées': 'Paris'}}, 6: {'attributes': {'Identifiant station': 6108, 'Nom station': 'Saint-Romain - Cherche-Midi', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8470815908, 2.32137478888], 'Nom communes équipées': 'Paris'}}, 7: {'attributes': {'Identifiant station': 5001, 'Nom station': 'Harpe - Saint-Germain', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.851518815, 2.34367031604], 'Nom communes équipées': 'Paris'}}, 8: {'attributes': {'Identifiant station': 42016, 'Nom station': 'Pierre et Marie Curie - Maurice Thorez', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8158022636, 2.37680498511], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 9: {'attributes': {'Identifiant station': 17026, 'Nom station': "Jouffroy d'Abbans - Wagram", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8819732984, 2.30113215744], 'Nom communes équipées': 'Paris'}}, 10: {'attributes': {'Identifiant station': 41301, 'Nom station': 'Bois de Vincennes - Gare', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 46, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8360222429, 2.47083399508], 'Nom communes équipées': 'Nogent-sur-Marne'}}, 11: {'attributes': {'Identifiant station': 10013, 'Nom station': 'Alibert - Jemmapes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 60, 'Nombre bornettes libres': 53, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.871044052, 2.36610446199], 'Nom communes équipées': 'Paris'}}, 12: {'attributes': {'Identifiant station': 21010, 'Nom station': 'Silly - Galliéni', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 24, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8355838387, 2.23255008459], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 13: {'attributes': {'Identifiant station': 8026, 'Nom station': 'Messine - Place Du Pérou', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 12, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.875448034, 2.31550801901], 'Nom communes équipées': 'Paris'}}, 14: {'attributes': {'Identifiant station': 25006, 'Nom station': 'Place Nelson Mandela', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8620909377, 2.19657629728], 'Nom communes équipées': 'Rueil-Malmaison'}}, 15: {'attributes': {'Identifiant station': 14111, 'Nom station': 'Cassini - Denfert-Rochereau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8375258391, 2.33603540808], 'Nom communes équipées': 'Paris'}}, 16: {'attributes': {'Identifiant station': 7001, 'Nom station': 'Invalides - Duroc', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8477924733, 2.31619209051], 'Nom communes équipées': 'Paris'}}, 17: {'attributes': {'Identifiant station': 6003, 'Nom station': 'Saint-Sulpice', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8516538318, 2.33080778271], 'Nom communes équipées': 'Paris'}}, 18: {'attributes': {'Identifiant station': 11025, 'Nom station': 'Froment - Bréguet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8570414505, 2.37289470307], 'Nom communes équipées': 'Paris'}}, 19: {'attributes': {'Identifiant station': 5110, 'Nom station': 'Lacépède - Monge', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8438928653, 2.35196638852], 'Nom communes équipées': 'Paris'}}, 20: {'attributes': {'Identifiant station': 13101, 'Nom station': 'Croulebarde - Corvisart', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8309816593, 2.34816469252], 'Nom communes équipées': 'Paris'}}, 21: {'attributes': {'Identifiant station': 33006, 'Nom station': 'André Karman - République', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9103987576, 2.38513559103], 'Nom communes équipées': 'Aubervilliers'}}, 22: {'attributes': {'Identifiant station': 14108, 'Nom station': 'Le Brix et Mesmin - Jourdan', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8223409659, 2.3278616533], 'Nom communes équipées': 'Paris'}}, 23: {'attributes': {'Identifiant station': 6021, 'Nom station': 'Beaux-Arts - Bonaparte', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8564519854, 2.33485188335], 'Nom communes équipées': 'Paris'}}, 24: {'attributes': {'Identifiant station': 47007, 'Nom station': 'Liberté - Vert-de-Maisons', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7901478096, 2.43286281824], 'Nom communes équipées': 'Maisons-Alfort'}}, 25: {'attributes': {'Identifiant station': 17038, 'Nom station': 'Grande Armée - Brunel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 61, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 11, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.876116, 2.288124], 'Nom communes équipées': 'Paris'}}, 26: {'attributes': {'Identifiant station': 8050, 'Nom station': 'Boétie - Ponthieu', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 31, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 5, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8714172844, 2.3076787591], 'Nom communes équipées': 'Paris'}}, 27: {'attributes': {'Identifiant station': 17041, 'Nom station': 'Guersant - Gouvion-Saint-Cyr', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8828777518, 2.28766737081], 'Nom communes équipées': 'Paris'}}, 28: {'attributes': {'Identifiant station': 17044, 'Nom station': 'Porte de Saint-Ouen - Bessières', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 11, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8979224085, 2.32851451022], 'Nom communes équipées': 'Paris'}}, 29: {'attributes': {'Identifiant station': 7003, 'Nom station': 'Square Boucicaut', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 60, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 26, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8512964337, 2.32506182045], 'Nom communes équipées': 'Paris'}}, 30: {'attributes': {'Identifiant station': 16118, 'Nom station': 'Michel-Ange - Parent de Rosan', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8405799237, 2.25841194391], 'Nom communes équipées': 'Paris'}}, 31: {'attributes': {'Identifiant station': 7002, 'Nom station': 'Vaneau - Sèvres', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8485632331, 2.32042182593], 'Nom communes équipées': 'Paris'}}, 32: {'attributes': {'Identifiant station': 43002, 'Nom station': 'Lagny - Victor Basch', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8488414719, 2.42632541893], 'Nom communes équipées': 'Vincennes'}}, 33: {'attributes': {'Identifiant station': 5016, 'Nom station': 'Thouin - Cardinal Lemoine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8450471666, 2.34946478513], 'Nom communes équipées': 'Paris'}}, 34: {'attributes': {'Identifiant station': 21216, 'Nom station': 'Louis Lejeune - Barbès', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8180741898, 2.32670098543], 'Nom communes équipées': 'Montrouge'}}, 35: {'attributes': {'Identifiant station': 15047, 'Nom station': 'Morillons - Dantzig', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 52, 'Nombre bornettes libres': 51, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8331014995, 2.29938000441], 'Nom communes équipées': 'Paris'}}, 36: {'attributes': {'Identifiant station': 22207, 'Nom station': 'Rond-point du Dr Albert Schweitzer', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7898091899, 2.31439400464], 'Nom communes équipées': 'Bagneux'}}, 37: {'attributes': {'Identifiant station': 17048, 'Nom station': 'De Toqueville - Terrasse', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.882428, 2.314002], 'Nom communes équipées': 'Paris'}}, 38: {'attributes': {'Identifiant station': 13030, 'Nom station': 'Italie - Tolbiac', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8255722946, 2.35761713237], 'Nom communes équipées': 'Paris'}}, 39: {'attributes': {'Identifiant station': 17025, 'Nom station': 'Chazelles - Courcelles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.879406605, 2.30344556272], 'Nom communes équipées': 'Paris'}}, 40: {'attributes': {'Identifiant station': 19027, 'Nom station': 'Serrurier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8804934032, 2.39806417376], 'Nom communes équipées': 'Paris'}}, 41: {'attributes': {'Identifiant station': 20143, 'Nom station': 'Ramponeau - Belleville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 44, 'Nombre bornettes libres': 41, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8710311376, 2.37905685211], 'Nom communes équipées': 'Paris'}}, 42: {'attributes': {'Identifiant station': 44017, 'Nom station': "Jules Guesde - Pont du Port à l'Anglais", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7962880709, 2.41721212864], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 43: {'attributes': {'Identifiant station': 31024, 'Nom station': 'Romainville - Vaillant-Couturier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.86785, 2.446748], 'Nom communes équipées': 'Montreuil'}}, 44: {'attributes': {'Identifiant station': 48003, 'Nom station': 'Youri Gagarine - Karl Marx', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7844848508, 2.3590843007], 'Nom communes équipées': 'Villejuif'}}, 45: {'attributes': {'Identifiant station': 15026, 'Nom station': 'Bir Hakeim', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 32, 'Vélos mécaniques disponibles': 27, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8538440543, 2.28970534822], 'Nom communes équipées': 'Paris'}}, 46: {'attributes': {'Identifiant station': 18134, 'Nom station': 'Ney - Porte de Clignancourt', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8978867, 2.3458001], 'Nom communes équipées': 'Paris'}}, 47: {'attributes': {'Identifiant station': 19033, 'Nom station': 'Cambrai - Benjamin Constant', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8959473685, 2.38113380225], 'Nom communes équipées': 'Paris'}}, 48: {'attributes': {'Identifiant station': 14002, 'Nom station': 'Edgar Quinet - Raspail', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 37, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8394026667, 2.32971478254], 'Nom communes équipées': 'Paris'}}, 49: {'attributes': {'Identifiant station': 16138, 'Nom station': 'Saint-Cloud - Hippodrome', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8577214889, 2.2549109906], 'Nom communes équipées': 'Paris'}}, 50: {'attributes': {'Identifiant station': 9014, 'Nom station': 'Faubourg Montmartre - La Fayette', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8752545491, 2.3408536613], 'Nom communes équipées': 'Paris'}}, 51: {'attributes': {'Identifiant station': 15068, 'Nom station': 'Général Martial Valin - Pont du Garigliano', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8382309427, 2.27050602436], 'Nom communes équipées': 'Paris'}}, 52: {'attributes': {'Identifiant station': 8053, 'Nom station': 'Place Georges Guillaumin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.874750204, 2.30177143533], 'Nom communes équipées': 'Paris'}}, 53: {'attributes': {'Identifiant station': 7004, 'Nom station': 'Raspail - Varenne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8531478165, 2.32639051974], 'Nom communes équipées': 'Paris'}}, 54: {'attributes': {'Identifiant station': 17032, 'Nom station': 'Jean Oestreicher - Porte de Champerret', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8865558007, 2.28864043951], 'Nom communes équipées': 'Paris'}}, 55: {'attributes': {'Identifiant station': 92002, 'Nom station': 'Théâtre des Amandiers - Palais des sports', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8926485268, 2.21369415522], 'Nom communes équipées': 'Nanterre'}}, 56: {'attributes': {'Identifiant station': 10029, 'Nom station': 'Dunkerque - Rocroy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.880726, 2.351464], 'Nom communes équipées': 'Paris'}}, 57: {'attributes': {'Identifiant station': 6029, 'Nom station': 'Vaugirard - Monsieur le Prince', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8489514416, 2.34113294631], 'Nom communes équipées': 'Paris'}}, 58: {'attributes': {'Identifiant station': 13118, 'Nom station': 'Regnault - Patay', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 63, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 46, 'Vélos mécaniques disponibles': 42, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8243578222, 2.37706515938], 'Nom communes équipées': 'Paris'}}, 59: {'attributes': {'Identifiant station': 20035, 'Nom station': 'Pyrénées - Ménilmontant', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8693268535, 2.39508457482], 'Nom communes équipées': 'Paris'}}, 60: {'attributes': {'Identifiant station': 12031, 'Nom station': 'Gabriel Lamé', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8351212784, 2.38553456962], 'Nom communes équipées': 'Paris'}}, 61: {'attributes': {'Identifiant station': 2017, 'Nom station': 'Caire - Dussoubs', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 35, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 14, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8678534211, 2.3493643105], 'Nom communes équipées': 'Paris'}}, 62: {'attributes': {'Identifiant station': 5022, 'Nom station': 'Boulangers - Cardinal Lemoine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8462904, 2.3521495], 'Nom communes équipées': 'Paris'}}, 63: {'attributes': {'Identifiant station': 9008, 'Nom station': "Tour d'Auvergne - Rodier", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.879161, 2.343698], 'Nom communes équipées': 'Paris'}}, 64: {'attributes': {'Identifiant station': 9106, 'Nom station': 'Boudreau - Auber', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 14, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.87219, 2.329423], 'Nom communes équipées': 'Paris'}}, 65: {'attributes': {'Identifiant station': 8103, 'Nom station': 'Artois - Berri', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8739352124, 2.30642583221], 'Nom communes équipées': 'Paris'}}, 66: {'attributes': {'Identifiant station': 19105, 'Nom station': 'Ambroise Rendu - Porte Brunet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.883667, 2.395733], 'Nom communes équipées': 'Paris'}}, 67: {'attributes': {'Identifiant station': 8048, 'Nom station': 'Marceau - Chaillot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8688339499, 2.29899678379], 'Nom communes équipées': 'Paris'}}, 68: {'attributes': {'Identifiant station': 31103, 'Nom station': 'Gare Rosny-Bois-Perrier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 11, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8811502334, 2.48043682426], 'Nom communes équipées': 'Rosny-sous-Bois'}}, 69: {'attributes': {'Identifiant station': 14013, 'Nom station': 'Liard - Amiral Mouchez', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 14, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8214169, 2.342167], 'Nom communes équipées': 'Paris'}}, 70: {'attributes': {'Identifiant station': 6001, 'Nom station': 'Institut de France', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8576156786, 2.33583135304], 'Nom communes équipées': 'Paris'}}, 71: {'attributes': {'Identifiant station': 7011, 'Nom station': 'Las Cases - Bourgogne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8590911047, 2.3185685277], 'Nom communes équipées': 'Paris'}}, 72: {'attributes': {'Identifiant station': 22409, 'Nom station': 'Henri Barbusse - Stalingrad', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8141837487, 2.28728625923], 'Nom communes équipées': 'Malakoff'}}, 73: {'attributes': {'Identifiant station': 18111, 'Nom station': 'Félix Ziem - Armand Gauthier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8894963049, 2.3333936588], 'Nom communes équipées': 'Paris'}}, 74: {'attributes': {'Identifiant station': 19015, 'Nom station': 'Thionville - Ourcq', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8891761374, 2.38336533308], 'Nom communes équipées': 'Paris'}}, 75: {'attributes': {'Identifiant station': 17010, 'Nom station': 'Jonquière - Docteur Paul Brousse', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8947998167, 2.31870521217], 'Nom communes équipées': 'Paris'}}, 76: {'attributes': {'Identifiant station': 17012, 'Nom station': 'Mairie du 17ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8841618487, 2.3220794311], 'Nom communes équipées': 'Paris'}}, 77: {'attributes': {'Identifiant station': 41303, 'Nom station': 'Gare de Nogent-le-Perreux', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8384255335, 2.49369330704], 'Nom communes équipées': 'Nogent-sur-Marne'}}, 78: {'attributes': {'Identifiant station': 8036, 'Nom station': 'Lisbonne - Monceau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 13, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8774556768, 2.30969422142], 'Nom communes équipées': 'Paris'}}, 79: {'attributes': {'Identifiant station': 16110, 'Nom station': 'Octave Feuillet - Albéric Magnard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8610690295, 2.27302830666], 'Nom communes équipées': 'Paris'}}, 80: {'attributes': {'Identifiant station': 1122, 'Nom station': 'Pont-Neuf - Quai du Louvre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 1, 22, 22, 50, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8582958983, 2.34237145633], 'Nom communes équipées': 'Paris'}}, 81: {'attributes': {'Identifiant station': 43009, 'Nom station': 'Place Jean-Spire-Lemaître', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 46, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8468428627, 2.45206207037], 'Nom communes équipées': 'Vincennes'}}, 82: {'attributes': {'Identifiant station': 12151, 'Nom station': 'Gare de Lyon - Place Louis Armand', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 60, 'Nombre bornettes libres': 38, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8455150898, 2.37264984463], 'Nom communes équipées': 'Paris'}}, 83: {'attributes': {'Identifiant station': 9001, 'Nom station': 'Bellefond - Poissonnière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8775989705, 2.34846375883], 'Nom communes équipées': 'Paris'}}, 84: {'attributes': {'Identifiant station': 9022, 'Nom station': 'Rossini - Laffitte', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 31, 'Vélos mécaniques disponibles': 26, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8733462751, 2.3379808677], 'Nom communes équipées': 'Paris'}}, 85: {'attributes': {'Identifiant station': 8016, 'Nom station': 'Square Louis XVI', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 27, 'Vélos mécaniques disponibles': 19, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.873204401, 2.32365883074], 'Nom communes équipées': 'Paris'}}, 86: {'attributes': {'Identifiant station': 16026, 'Nom station': 'Ranelagh - Mozart', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8554439, 2.2704418], 'Nom communes équipées': 'Paris'}}, 87: {'attributes': {'Identifiant station': 10162, 'Nom station': 'Alexandre Parodi - Quai de Valmy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8793594194, 2.36659616232], 'Nom communes équipées': 'Paris'}}, 88: {'attributes': {'Identifiant station': 11027, 'Nom station': 'Saint-Ambroise - Parmentier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8621478445, 2.3772348091], 'Nom communes équipées': 'Paris'}}, 89: {'attributes': {'Identifiant station': 10115, 'Nom station': 'Granges aux Belles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8761373391, 2.36808449794], 'Nom communes équipées': 'Paris'}}, 90: {'attributes': {'Identifiant station': 17043, 'Nom station': 'Belidor - Gouvion-Saint-Cyr', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8802215405, 2.2854680568], 'Nom communes équipées': 'Paris'}}, 91: {'attributes': {'Identifiant station': 5005, 'Nom station': 'Gay Lussac - Saint-Jacques', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 11, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8447302561, 2.34192394487], 'Nom communes équipées': 'Paris'}}, 92: {'attributes': {'Identifiant station': 19026, 'Nom station': 'Place de Rhin et Danube', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8819276586, 2.39258676767], 'Nom communes équipées': 'Paris'}}, 93: {'attributes': {'Identifiant station': 18024, 'Nom station': 'Clignancourt - Ordener', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8914577262, 2.34863647806], 'Nom communes équipées': 'Paris'}}, 94: {'attributes': {'Identifiant station': 45003, 'Nom station': '11 Novembre 1918 - 8 Mai 1945', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8088953377, 2.53824211657], 'Nom communes équipées': 'Champigny-sur-Marne'}}, 95: {'attributes': {'Identifiant station': 7018, 'Nom station': "Ségur - d'Estrées", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.851419979, 2.30970449746], 'Nom communes équipées': 'Paris'}}, 96: {'attributes': {'Identifiant station': 12111, 'Nom station': 'Claude Decaen - Cannebière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8376141704, 2.3972537822], 'Nom communes équipées': 'Paris'}}, 97: {'attributes': {'Identifiant station': 9002, 'Nom station': 'Abbeville - Faubourg Poissonnière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 14, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.879223, 2.349147], 'Nom communes équipées': 'Paris'}}, 98: {'attributes': {'Identifiant station': 8034, 'Nom station': 'Berri - Haussmann', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8747460474, 2.3082953319], 'Nom communes équipées': 'Paris'}}, 99: {'attributes': {'Identifiant station': 10161, 'Nom station': "Gare de l'Est - Faubourg-Saint-Martin", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8756426475, 2.35945880413], 'Nom communes équipées': 'Paris'}}, 100: {'attributes': {'Identifiant station': 19043, 'Nom station': 'Delphine Seyrig - Général Leclerc', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 53, 'Nombre bornettes libres': 47, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8939239631, 2.39790290594], 'Nom communes équipées': 'Pantin'}}, 101: {'attributes': {'Identifiant station': 17040, 'Nom station': 'Pereire - Ternes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 48, 'Nombre bornettes libres': 44, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8797395483, 2.28801213205], 'Nom communes équipées': 'Paris'}}, 102: {'attributes': {'Identifiant station': 41302, 'Nom station': 'Charles de Gaulle - Beauté', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 49, 'Nombre bornettes libres': 43, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8364960231, 2.47956265115], 'Nom communes équipées': 'Nogent-sur-Marne'}}, 103: {'attributes': {'Identifiant station': 9038, 'Nom station': 'Douai - Bruxelles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8832078412, 2.3308449949], 'Nom communes équipées': 'Paris'}}, 104: {'attributes': {'Identifiant station': 15067, 'Nom station': 'Desaix - Edgar Faure', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 12, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8528046049, 2.29307702986], 'Nom communes équipées': 'Paris'}}, 105: {'attributes': {'Identifiant station': 8009, 'Nom station': 'Gare Saint-Lazare - Isly', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8747776776, 2.32651580478], 'Nom communes équipées': 'Paris'}}, 106: {'attributes': {'Identifiant station': 43401, 'Nom station': 'Gare RER - Canadiens', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 11, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8199530796, 2.46417675246], 'Nom communes équipées': 'Joinville-le-Pont'}}, 107: {'attributes': {'Identifiant station': 1001, 'Nom station': "Quai de l'Horloge - Pont Neuf", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8570587391, 2.34179828394], 'Nom communes équipées': 'Paris'}}, 108: {'attributes': {'Identifiant station': 12128, 'Nom station': 'Pyramide - Ecole du Breuil', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8223797715, 2.45868712664], 'Nom communes équipées': 'Paris'}}, 109: {'attributes': {'Identifiant station': 17008, 'Nom station': 'Porte Pouchet-Bessières', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8978765, 2.3230632], 'Nom communes équipées': 'Paris'}}, 110: {'attributes': {'Identifiant station': 31022, 'Nom station': 'Mairie de Montreuil', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 52, 'Nombre bornettes libres': 46, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8613145276, 2.44061440229], 'Nom communes équipées': 'Montreuil'}}, 111: {'attributes': {'Identifiant station': 15112, 'Nom station': 'Vigée-Lebrun - Anselme Payen', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8386839416, 2.3110422492], 'Nom communes équipées': 'Paris'}}, 112: {'attributes': {'Identifiant station': 46002, 'Nom station': 'Rouget de Lisle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 30, 'Vélos mécaniques disponibles': 27, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7646154834, 2.40711897612], 'Nom communes équipées': 'Choisy-le-Roi'}}, 113: {'attributes': {'Identifiant station': 7025, 'Nom station': 'Octave Gréard - Tour Eiffel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 31, 'Vélos mécaniques disponibles': 29, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8565029777, 2.29317875433], 'Nom communes équipées': 'Paris'}}, 114: {'attributes': {'Identifiant station': 11113, 'Nom station': 'Richard Lenoir - Republique', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 58, 'Nombre bornettes libres': 55, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8667175, 2.3692541], 'Nom communes équipées': 'Paris'}}, 115: {'attributes': {'Identifiant station': 31706, 'Nom station': 'Château - République', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 59, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8629238006, 2.4155035615], 'Nom communes équipées': 'Bagnolet'}}, 116: {'attributes': {'Identifiant station': 15042, 'Nom station': 'Henry Farman', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8337243028, 2.27138545364], 'Nom communes équipées': 'Paris'}}, 117: {'attributes': {'Identifiant station': 10025, 'Nom station': 'Jemmapes - Ecluses Saint-Martin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 15, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8774851919, 2.36600674689], 'Nom communes équipées': 'Paris'}}, 118: {'attributes': {'Identifiant station': 19044, 'Nom station': 'Grands Moulins de Pantin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8973836972, 2.39599853754], 'Nom communes équipées': 'Paris'}}, 119: {'attributes': {'Identifiant station': 26003, 'Nom station': 'Argenteuil - Voltaire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9186486596, 2.28141121566], 'Nom communes équipées': 'Asnières-sur-Seine'}}, 120: {'attributes': {'Identifiant station': 43008, 'Nom station': 'Fontenay - Libération', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8478030636, 2.44514197111], 'Nom communes équipées': 'Vincennes'}}, 121: {'attributes': {'Identifiant station': 17004, 'Nom station': 'Legendre - Clichy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 35, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8889616186, 2.32252897509], 'Nom communes équipées': 'Paris'}}, 122: {'attributes': {'Identifiant station': 33102, 'Nom station': 'Gabriel Péri - Francisco Ferrer', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8863371024, 2.40733891726], 'Nom communes équipées': 'Le Pré-Saint-Gervais'}}, 123: {'attributes': {'Identifiant station': 19003, 'Nom station': 'Quai de la Seine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8844922384, 2.37039417028], 'Nom communes équipées': 'Paris'}}, 124: {'attributes': {'Identifiant station': 8045, 'Nom station': 'Place de la Reine Astrid', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 58, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 46, 'Vélos mécaniques disponibles': 27, 'Vélos électriques disponibles': 19, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8649209825, 2.30254918239], 'Nom communes équipées': 'Paris'}}, 125: {'attributes': {'Identifiant station': 10008, 'Nom station': "Cité Riverin - Château d'Eau", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8707024236, 2.3587398639], 'Nom communes équipées': 'Paris'}}, 126: {'attributes': {'Identifiant station': 21312, 'Nom station': 'Lucie Aubrac - Fort', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.818144834, 2.27118462324], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 127: {'attributes': {'Identifiant station': 18107, 'Nom station': 'Chartres - La Charbonnière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8847601, 2.3534657], 'Nom communes équipées': 'Paris'}}, 128: {'attributes': {'Identifiant station': 18011, 'Nom station': 'Pajol - Place Herbert', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 38, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8927825, 2.3635598], 'Nom communes équipées': 'Paris'}}, 129: {'attributes': {'Identifiant station': 5020, 'Nom station': 'Institut du Monde Arabe - Saint-Germain', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 36, 'Vélos mécaniques disponibles': 25, 'Vélos électriques disponibles': 11, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8490632995, 2.35580865294], 'Nom communes équipées': 'Paris'}}, 130: {'attributes': {'Identifiant station': 19103, 'Nom station': "Manin - Carrières d'Amérique", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8846721421, 2.3919698596], 'Nom communes équipées': 'Paris'}}, 131: {'attributes': {'Identifiant station': 92007, 'Nom station': 'Pesaro - Préfecture', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8957704628, 2.22390834242], 'Nom communes équipées': 'Nanterre'}}, 132: {'attributes': {'Identifiant station': 21306, 'Nom station': 'Place Lafayette', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 62, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 58, 'Vélos mécaniques disponibles': 44, 'Vélos électriques disponibles': 14, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8296549755, 2.26386755705], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 133: {'attributes': {'Identifiant station': 9028, 'Nom station': 'Moncey - Blanche', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8800658747, 2.33108103275], 'Nom communes équipées': 'Paris'}}, 134: {'attributes': {'Identifiant station': 14138, 'Nom station': 'Porte de Vanves', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8280128494, 2.30568788946], 'Nom communes équipées': 'Paris'}}, 135: {'attributes': {'Identifiant station': 10107, 'Nom station': 'Gare du Nord - Hôpital Lariboisière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 49, 'Nombre bornettes libres': 45, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.881949, 2.352339], 'Nom communes équipées': 'Paris'}}, 136: {'attributes': {'Identifiant station': 46003, 'Nom station': 'Conservatoire de Musique', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 19, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7709358667, 2.40340143442], 'Nom communes équipées': 'Choisy-le-Roi'}}, 137: {'attributes': {'Identifiant station': 20042, 'Nom station': 'Pyrénées - Emmery', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 39, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8720102813, 2.39203959703], 'Nom communes équipées': 'Paris'}}, 138: {'attributes': {'Identifiant station': 41209, 'Nom station': 'Cimetière de Fontenay-sous-Bois', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 15, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8470617336, 2.47878223658], 'Nom communes équipées': 'Fontenay-sous-Bois'}}, 139: {'attributes': {'Identifiant station': 11002, 'Nom station': 'Roquette - Thiéré', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8550793058, 2.37336863187], 'Nom communes équipées': 'Paris'}}, 140: {'attributes': {'Identifiant station': 10024, 'Nom station': 'Buisson Saint-Louis - Saint-Maur', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8717468493, 2.37225283288], 'Nom communes équipées': 'Paris'}}, 141: {'attributes': {'Identifiant station': 16039, 'Nom station': 'Exelmans - Versailles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 49, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 18, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8409749109, 2.26439025253], 'Nom communes équipées': 'Paris'}}, 142: {'attributes': {'Identifiant station': 20037, 'Nom station': 'Gambetta - Saint Fargeau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8712446104, 2.40390568972], 'Nom communes équipées': 'Paris'}}, 143: {'attributes': {'Identifiant station': 11023, 'Nom station': 'Saint-Maur - Roquette', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 44, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8586887741, 2.38355399457], 'Nom communes équipées': 'Paris'}}, 144: {'attributes': {'Identifiant station': 5018, 'Nom station': 'Mutualité', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 32, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8482255727, 2.35028732568], 'Nom communes équipées': 'Paris'}}, 145: {'attributes': {'Identifiant station': 31009, 'Nom station': 'Place Du Général De Gaulle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8687591, 2.4329931], 'Nom communes équipées': 'Montreuil'}}, 146: {'attributes': {'Identifiant station': 8017, 'Nom station': 'Rocher - Laborde', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 65, 'Nombre bornettes libres': 49, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8759024071, 2.32261631638], 'Nom communes équipées': 'Paris'}}, 147: {'attributes': {'Identifiant station': 11047, 'Nom station': 'Amelot - Saint-Sébastien', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8611421628, 2.36779961811], 'Nom communes équipées': 'Paris'}}, 148: {'attributes': {'Identifiant station': 15054, 'Nom station': 'Lecourbe - Jean Maridor', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8379636615, 2.28773921728], 'Nom communes équipées': 'Paris'}}, 149: {'attributes': {'Identifiant station': 17030, 'Nom station': 'Pereire - Rennequin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8836478289, 2.29524470866], 'Nom communes équipées': 'Paris'}}, 150: {'attributes': {'Identifiant station': 15106, 'Nom station': 'Grenelle - Violet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 19, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8499303551, 2.29461204261], 'Nom communes équipées': 'Paris'}}, 151: {'attributes': {'Identifiant station': 5032, 'Nom station': 'Panthéon - Valette', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 27, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 24, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8470205717, 2.34649826197], 'Nom communes équipées': 'Paris'}}, 152: {'attributes': {'Identifiant station': 14008, 'Nom station': 'Mouton Duvernet - Général Leclerc', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8316325183, 2.32931211591], 'Nom communes équipées': 'Paris'}}, 153: {'attributes': {'Identifiant station': 14020, 'Nom station': 'Cimetière de Montrouge', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 58, 'Nombre bornettes libres': 56, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.822009, 2.322188699], 'Nom communes équipées': 'Paris'}}, 154: {'attributes': {'Identifiant station': 14106, 'Nom station': 'Marc Sangnier - Porte de Vanves', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8252186836, 2.31071166694], 'Nom communes équipées': 'Paris'}}, 155: {'attributes': {'Identifiant station': 26002, 'Nom station': 'Redoute - Les Courtilles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9298671945, 2.28366963565], 'Nom communes équipées': 'Asnières-sur-Seine'}}, 156: {'attributes': {'Identifiant station': 22406, 'Nom station': 'Guy Môquet - Etienne Dolet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8152700999, 2.298078277], 'Nom communes équipées': 'Malakoff'}}, 157: {'attributes': {'Identifiant station': 6006, 'Nom station': 'Notre-Dame-des-Champs - Stanislas', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8447992886, 2.32933775589], 'Nom communes équipées': 'Paris'}}, 158: {'attributes': {'Identifiant station': 35002, 'Nom station': 'Jean Jaurès - Presles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.907134415, 2.39600572805], 'Nom communes équipées': 'Pantin'}}, 159: {'attributes': {'Identifiant station': 8010, 'Nom station': 'Liège - Moscou', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8795431, 2.3264628], 'Nom communes équipées': 'Paris'}}, 160: {'attributes': {'Identifiant station': 2111, 'Nom station': 'Chabanais - Petits Champs', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 26, 'Vélos mécaniques disponibles': 19, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8670374019, 2.33671601862], 'Nom communes équipées': 'Paris'}}, 161: {'attributes': {'Identifiant station': 19101, 'Nom station': 'Square Simon Bolivar', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8750898883, 2.38263040781], 'Nom communes équipées': 'Paris'}}, 162: {'attributes': {'Identifiant station': 8054, 'Nom station': 'Bertie Albrecht - Hoche', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8761403983, 2.30130313831], 'Nom communes équipées': 'Paris'}}, 163: {'attributes': {'Identifiant station': 14003, 'Nom station': 'Schoelcher - Raspail', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 35, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8368173353, 2.33127940804], 'Nom communes équipées': 'Paris'}}, 164: {'attributes': {'Identifiant station': 35010, 'Nom station': 'Hoche - Jean Lolive', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 2, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.891102392, 2.40284353495], 'Nom communes équipées': 'Pantin'}}, 165: {'attributes': {'Identifiant station': 18003, 'Nom station': 'Cimetière de Montmartre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 42, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8864724661, 2.33289018273], 'Nom communes équipées': 'Paris'}}, 166: {'attributes': {'Identifiant station': 15110, 'Nom station': 'Place Robert Guillemard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8361917281, 2.28112671601], 'Nom communes équipées': 'Paris'}}, 167: {'attributes': {'Identifiant station': 10016, 'Nom station': 'Récollets - Square Villemin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8750346559, 2.3598012386], 'Nom communes équipées': 'Paris'}}, 168: {'attributes': {'Identifiant station': 10042, 'Nom station': 'Enghien - Faubourg Poissonnière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 2, 21, 21, 15, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8724200631, 2.34839523628], 'Nom communes équipées': 'Paris'}}, 169: {'attributes': {'Identifiant station': 21212, 'Nom station': 'Marne - Germain Dardan', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.811542704, 2.30307072401], 'Nom communes équipées': 'Montrouge'}}, 170: {'attributes': {'Identifiant station': 31008, 'Nom station': 'François Debergue - Croix de Chavaux', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 57, 'Nombre bornettes libres': 46, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8574217081, 2.43757814169], 'Nom communes équipées': 'Montreuil'}}, 171: {'attributes': {'Identifiant station': 42706, 'Nom station': 'Roger Salengro - Fontainebleau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8145486973, 2.36116636544], 'Nom communes équipées': 'Le Kremlin-Bicêtre'}}, 172: {'attributes': {'Identifiant station': 21015, 'Nom station': 'Nationale - Place Jules Guesde', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8275396246, 2.24160522223], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 173: {'attributes': {'Identifiant station': 33017, 'Nom station': 'Anatole France - President Roosevelt', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9155156, 2.3814103], 'Nom communes équipées': 'Aubervilliers'}}, 174: {'attributes': {'Identifiant station': 41404, 'Nom station': 'Carnot - Pâquerettes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 18, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7959293943, 2.42618411779], 'Nom communes équipées': 'Alfortville'}}, 175: {'attributes': {'Identifiant station': 20112, 'Nom station': 'Jourdain - Place des Grandes Rigoles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8740311373, 2.38959476352], 'Nom communes équipées': 'Paris'}}, 176: {'attributes': {'Identifiant station': 22203, 'Nom station': 'Tertres - Fontaine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7875536547, 2.30623573065], 'Nom communes équipées': 'Bagneux'}}, 177: {'attributes': {'Identifiant station': 13001, 'Nom station': 'Port-Royal - Hôpital du Val-de-Grâce', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8376682728, 2.34481383466], 'Nom communes équipées': 'Paris'}}, 178: {'attributes': {'Identifiant station': 41202, 'Nom station': 'Gare Rer de Fontenay sous Bois', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 61, 'Nombre bornettes libres': 53, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8436279, 2.4630465], 'Nom communes équipées': 'Fontenay-sous-Bois'}}, 179: {'attributes': {'Identifiant station': 15104, 'Nom station': 'Hôpital Européen Georges Pompidou', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 64, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 50, 'Vélos mécaniques disponibles': 42, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8376953192, 2.27537423372], 'Nom communes équipées': 'Paris'}}, 180: {'attributes': {'Identifiant station': 35020, 'Nom station': 'Marcelle - Thalie', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8835462, 2.4137883], 'Nom communes équipées': 'Pantin'}}, 181: {'attributes': {'Identifiant station': 44006, 'Nom station': 'Concorde - Stalingrad', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7985844, 2.3821302], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 182: {'attributes': {'Identifiant station': 25003, 'Nom station': 'Henri Ste-Claire Déville - Paul Héroult', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.892663, 2.172411], 'Nom communes équipées': 'Rueil-Malmaison'}}, 183: {'attributes': {'Identifiant station': 22202, 'Nom station': 'Jean Marin Naudin - Stalingrad', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8057138974, 2.32020232826], 'Nom communes équipées': 'Bagneux'}}, 184: {'attributes': {'Identifiant station': 31003, 'Nom station': 'Paris - Emile Zola', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8548980958, 2.41866856813], 'Nom communes équipées': 'Montreuil'}}, 185: {'attributes': {'Identifiant station': 19120, 'Nom station': 'Carducci - Place Hannah Arendt', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8774366815, 2.38607436419], 'Nom communes équipées': 'Paris'}}, 186: {'attributes': {'Identifiant station': 9116, 'Nom station': "Victoire - Chaussée d'Antin", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 12, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.875065791, 2.33167380095], 'Nom communes équipées': 'Paris'}}, 187: {'attributes': {'Identifiant station': 5004, 'Nom station': 'Saint-Jacques - Val de Grâce', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8417514, 2.3413893], 'Nom communes équipées': 'Paris'}}, 188: {'attributes': {'Identifiant station': 21008, 'Nom station': 'Victor Hugo - Galliéni', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 12, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8380201544, 2.24608451128], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 189: {'attributes': {'Identifiant station': 12114, 'Nom station': 'Victor Hugo - Jeanne Jugan', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.843949064, 2.41508139483], 'Nom communes équipées': 'Paris'}}, 190: {'attributes': {'Identifiant station': 1119, 'Nom station': 'Cossonnerie - Sébastopol', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8613266591, 2.34941963106], 'Nom communes équipées': 'Paris'}}, 191: {'attributes': {'Identifiant station': 18028, 'Nom station': 'Championnet - Vauvenargues', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8940187474, 2.33198728412], 'Nom communes équipées': 'Paris'}}, 192: {'attributes': {'Identifiant station': 11039, 'Nom station': 'Richard Lenoir - Jean-Pierre Timbaud', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8658042767, 2.36933707497], 'Nom communes équipées': 'Paris'}}, 193: {'attributes': {'Identifiant station': 15105, 'Nom station': 'Fédération - Suffren', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 18, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8528281534, 2.29787763208], 'Nom communes équipées': 'Paris'}}, 194: {'attributes': {'Identifiant station': 17031, 'Nom station': 'Place de la Porte de Champerret', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8856174163, 2.29078141128], 'Nom communes équipées': 'Paris'}}, 195: {'attributes': {'Identifiant station': 18031, 'Nom station': 'Place Albert Kahn', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 44, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8957969798, 2.34558558413], 'Nom communes équipées': 'Paris'}}, 196: {'attributes': {'Identifiant station': 13050, 'Nom station': 'Quai Panhard et Levassor', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 57, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 52, 'Vélos mécaniques disponibles': 50, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8283748281, 2.38427374581], 'Nom communes équipées': 'Paris'}}, 197: {'attributes': {'Identifiant station': 42014, 'Nom station': 'Place Marcel Cachin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 29, 'Vélos mécaniques disponibles': 22, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8141144236, 2.39090770483], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 198: {'attributes': {'Identifiant station': 11030, 'Nom station': 'Oberkampf - République', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8653160871, 2.37611478491], 'Nom communes équipées': 'Paris'}}, 199: {'attributes': {'Identifiant station': 5124, 'Nom station': 'Erasme - Ulm', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8426437527, 2.34459467232], 'Nom communes équipées': 'Paris'}}, 200: {'attributes': {'Identifiant station': 19024, 'Nom station': 'Botzaris - Parc des Buttes Chaumont', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.877951, 2.381464], 'Nom communes équipées': 'Paris'}}, 201: {'attributes': {'Identifiant station': 18046, 'Nom station': 'Etienne Jodelle - Saint Ouen', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8879401639, 2.32612442031], 'Nom communes équipées': 'Paris'}}, 202: {'attributes': {'Identifiant station': 17105, 'Nom station': 'Place Madeleine Daniélou', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 39, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.883498, 2.282015], 'Nom communes équipées': 'Paris'}}, 203: {'attributes': {'Identifiant station': 31102, 'Nom station': 'Boissière - Etienne Dolet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.876111, 2.4677312], 'Nom communes équipées': 'Montreuil'}}, 204: {'attributes': {'Identifiant station': 5012, 'Nom station': 'Estrapade - Ulm', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 2, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8451277283, 2.34545681704], 'Nom communes équipées': 'Paris'}}, 205: {'attributes': {'Identifiant station': 44013, 'Nom station': 'Edouard Tremblay - Julian Grimau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7813838251, 2.37235084176], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 206: {'attributes': {'Identifiant station': 20134, 'Nom station': 'Lieutenant Chauré - Etienne Marey', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 2, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8674619541, 2.40597099066], 'Nom communes équipées': 'Paris'}}, 207: {'attributes': {'Identifiant station': 15109, 'Nom station': 'Cevennes - Bergers', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8424076513, 2.28103771806], 'Nom communes équipées': 'Paris'}}, 208: {'attributes': {'Identifiant station': 15103, 'Nom station': 'Cauchy - Cévennes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 51, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 40, 'Vélos mécaniques disponibles': 38, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.843372571, 2.27513283491], 'Nom communes équipées': 'Paris'}}, 209: {'attributes': {'Identifiant station': 21209, 'Nom station': 'Molière - République', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8111158139, 2.31457386244], 'Nom communes équipées': 'Montrouge'}}, 210: {'attributes': {'Identifiant station': 18009, 'Nom station': 'Jean-François Lépine - Stephenson', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 39, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.886114668, 2.35685562753], 'Nom communes équipées': 'Paris'}}, 211: {'attributes': {'Identifiant station': 21005, 'Nom station': 'Hôtel de Ville de Boulogne Billancourt', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8349588517, 2.24168032408], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 212: {'attributes': {'Identifiant station': 16108, 'Nom station': 'Sablons - Georges Mandel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 35, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.863875, 2.28189], 'Nom communes équipées': 'Paris'}}, 213: {'attributes': {'Identifiant station': 14124, 'Nom station': 'David Weill - Parc Montsouris', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8210923743, 2.33372569084], 'Nom communes équipées': 'Paris'}}, 214: {'attributes': {'Identifiant station': 11024, 'Nom station': 'Mairie du 11ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 51, 'Nombre bornettes libres': 37, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.858925, 2.3789759], 'Nom communes équipées': 'Paris'}}, 215: {'attributes': {'Identifiant station': 15133, 'Nom station': 'Saint Lambert - Blomet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8365914815, 2.29305569082], 'Nom communes équipées': 'Paris'}}, 216: {'attributes': {'Identifiant station': 24001, 'Nom station': 'Arche - Léonard de Vinci', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8979097249, 2.23837230436], 'Nom communes équipées': 'Courbevoie'}}, 217: {'attributes': {'Identifiant station': 18022, 'Nom station': 'Marcadet - Barbès', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 35, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.890451121, 2.34905739297], 'Nom communes équipées': 'Paris'}}, 218: {'attributes': {'Identifiant station': 8025, 'Nom station': 'Miromesnil - La Boétie', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8737224129, 2.31584809721], 'Nom communes équipées': 'Paris'}}, 219: {'attributes': {'Identifiant station': 21006, 'Nom station': 'Marcelin Berthelot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8415473059, 2.23316162825], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 220: {'attributes': {'Identifiant station': 10014, 'Nom station': 'Marseille - Beaurepaire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 55, 'Nombre bornettes libres': 46, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.871624307, 2.36392650827], 'Nom communes équipées': 'Paris'}}, 221: {'attributes': {'Identifiant station': 20131, 'Nom station': 'Repos - Cimetière du Père Lachaise', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8593585236, 2.38961313668], 'Nom communes équipées': 'Paris'}}, 222: {'attributes': {'Identifiant station': 9032, 'Nom station': 'Mathurins - Auber', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 3, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8729888407, 2.32949842123], 'Nom communes équipées': 'Paris'}}, 223: {'attributes': {'Identifiant station': 14012, 'Nom station': 'Sibelle - Alésia', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8267017585, 2.33849834651], 'Nom communes équipées': 'Paris'}}, 224: {'attributes': {'Identifiant station': 5123, 'Nom station': 'Saint-Germain - Thenard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 26, 'Vélos mécaniques disponibles': 23, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.850571, 2.3471015], 'Nom communes équipées': 'Paris'}}, 225: {'attributes': {'Identifiant station': 44003, 'Nom station': 'Gabriel Péri - Danielle Casanova', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 5, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.790114, 2.4025777], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 226: {'attributes': {'Identifiant station': 13002, 'Nom station': 'Arago - Glacière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8348605478, 2.34457705243], 'Nom communes équipées': 'Paris'}}, 227: {'attributes': {'Identifiant station': 15034, 'Nom station': 'Lakanal - Commerce', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 53, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 31, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 43, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8444246279, 2.29405917227], 'Nom communes équipées': 'Paris'}}, 228: {'attributes': {'Identifiant station': 46001, 'Nom station': 'Place Paul Eluard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7672039733, 2.41449773312], 'Nom communes équipées': 'Choisy-le-Roi'}}, 229: {'attributes': {'Identifiant station': 48010, 'Nom station': 'Karl Marx.', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7845279373, 2.36466679795], 'Nom communes équipées': 'Villejuif'}}, 230: {'attributes': {'Identifiant station': 17028, 'Nom station': 'Reims - Raymond Pitet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8888, 2.296319], 'Nom communes équipées': 'Paris'}}, 231: {'attributes': {'Identifiant station': 42021, 'Nom station': 'Raspail - Manufacture des Oeillets', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8094458, 2.3903009], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 232: {'attributes': {'Identifiant station': 42302, 'Nom station': 'Aristide Briand - Gare de Bagneux', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7938726966, 2.32114881277], 'Nom communes équipées': 'Cachan'}}, 233: {'attributes': {'Identifiant station': 21112, 'Nom station': 'Claude Debussy - Marc Bloch', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 10, 13, 22, 8, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.9103881805, 2.31177717447], 'Nom communes équipées': 'Clichy'}}, 234: {'attributes': {'Identifiant station': 21110, 'Nom station': 'Parc Roger Salengro', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9053038586, 2.31055945158], 'Nom communes équipées': 'Clichy'}}, 235: {'attributes': {'Identifiant station': 31708, 'Nom station': 'Noisy le Sec - Jean-Baptiste Clément', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8745153896, 2.41528362036], 'Nom communes équipées': 'Bagnolet'}}, 236: {'attributes': {'Identifiant station': 11042, 'Nom station': 'Grand Prieuré - Crussol', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 49, 'Nombre bornettes libres': 42, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8646606279, 2.36922655318], 'Nom communes équipées': 'Paris'}}, 237: {'attributes': {'Identifiant station': 20115, 'Nom station': 'Louis Lumière - Porte de Bagnolet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.86431136, 2.41052102298], 'Nom communes équipées': 'Paris'}}, 238: {'attributes': {'Identifiant station': 7102, 'Nom station': 'Mairie du 7ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 19, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8577967, 2.3192664], 'Nom communes équipées': 'Paris'}}, 239: {'attributes': {'Identifiant station': 22402, 'Nom station': 'Jean Jaurès - Salvador Allende', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 38, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8214755947, 2.30265766382], 'Nom communes équipées': 'Malakoff'}}, 240: {'attributes': {'Identifiant station': 15013, 'Nom station': 'Volontaire - Lecourbe', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.843513788, 2.30655223131], 'Nom communes équipées': 'Paris'}}, 241: {'attributes': {'Identifiant station': 17013, 'Nom station': 'Legendre - Dulong', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8853251694, 2.31642376631], 'Nom communes équipées': 'Paris'}}, 242: {'attributes': {'Identifiant station': 1002, 'Nom station': 'Victoria - Place du Chatelet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8579925915, 2.34691679478], 'Nom communes équipées': 'Paris'}}, 243: {'attributes': {'Identifiant station': 16008, 'Nom station': 'Galilée - Kléber', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8674118657, 2.29078138011], 'Nom communes équipées': 'Paris'}}, 244: {'attributes': {'Identifiant station': 11004, 'Nom station': 'Charonne - Faubourg Saint-Antoine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8521919097, 2.37409327179], 'Nom communes équipées': 'Paris'}}, 245: {'attributes': {'Identifiant station': 5106, 'Nom station': 'Cujas - Saint-Michel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8481913486, 2.34183165485], 'Nom communes équipées': 'Paris'}}, 246: {'attributes': {'Identifiant station': 42301, 'Nom station': 'Gare Arcueil - Cachan', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.799824, 2.328171], 'Nom communes équipées': 'Arcueil'}}, 247: {'attributes': {'Identifiant station': 11026, 'Nom station': 'Chemin Vert - Saint-Maur', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 39, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8611246141, 2.38129902631], 'Nom communes équipées': 'Paris'}}, 248: {'attributes': {'Identifiant station': 20121, 'Nom station': 'Boyer - Ménilmontant', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8688951627, 2.39198241456], 'Nom communes équipées': 'Paris'}}, 249: {'attributes': {'Identifiant station': 32604, 'Nom station': 'Square du Théâtre du Garde-Chasse', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 46, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8812873751, 2.42011964321], 'Nom communes équipées': 'Les Lilas'}}, 250: {'attributes': {'Identifiant station': 42015, 'Nom station': 'Place Jean Ferrat', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8184945213, 2.37441748381], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 251: {'attributes': {'Identifiant station': 8057, 'Nom station': 'Hoche - Tilsitt', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8748364578, 2.29713231325], 'Nom communes équipées': 'Paris'}}, 252: {'attributes': {'Identifiant station': 19008, 'Nom station': 'Flandre - Alphonse Karr', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.89465, 2.381868], 'Nom communes équipées': 'Paris'}}, 253: {'attributes': {'Identifiant station': 9010, 'Nom station': 'Rochechouart lamartine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8767198951, 2.34439302185], 'Nom communes équipées': 'Paris'}}, 254: {'attributes': {'Identifiant station': 32016, 'Nom station': 'Place de la Porte de Paris', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 24, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9300936321, 2.35640376806], 'Nom communes équipées': 'Saint-Denis'}}, 255: {'attributes': {'Identifiant station': 15021, 'Nom station': 'Joseph Liouville - Croix Nivert', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 64, 'Nombre bornettes libres': 42, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 19, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8448354712, 2.29750849307], 'Nom communes équipées': 'Paris'}}, 256: {'attributes': {'Identifiant station': 15201, 'Nom station': 'Place Amédée Gordini - Lefebvre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8306173775, 2.29206342083], 'Nom communes équipées': 'Paris'}}, 257: {'attributes': {'Identifiant station': 11105, 'Nom station': 'Parmentier - République', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 42, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 2, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8644954, 2.3750124], 'Nom communes équipées': 'Paris'}}, 258: {'attributes': {'Identifiant station': 8013, 'Nom station': 'Marignan - Champs-Elysées', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 21, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8694956612, 2.3066138139], 'Nom communes équipées': 'Paris'}}, 259: {'attributes': {'Identifiant station': 21701, 'Nom station': 'Larmeroux - Baudelaire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8169399113, 2.28098948048], 'Nom communes équipées': 'Vanves'}}, 260: {'attributes': {'Identifiant station': 5006, 'Nom station': 'Saint-Jacques - Soufflot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8465255, 2.343099], 'Nom communes équipées': 'Paris'}}, 261: {'attributes': {'Identifiant station': 22003, 'Nom station': 'Beffroy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8865910745, 2.26139456034], 'Nom communes équipées': 'Neuilly-sur-Seine'}}, 262: {'attributes': {'Identifiant station': 43003, 'Nom station': 'Hôpital Bégin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8457167211, 2.42724627256], 'Nom communes équipées': 'Vincennes'}}, 263: {'attributes': {'Identifiant station': 16038, 'Nom station': 'Molitor - Chardon-Lagache', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 59, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8452088018, 2.2656840831], 'Nom communes équipées': 'Paris'}}, 264: {'attributes': {'Identifiant station': 12009, 'Nom station': 'Charenton - Diderot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8461714612, 2.37930681556], 'Nom communes équipées': 'Paris'}}, 265: {'attributes': {'Identifiant station': 41601, 'Nom station': 'Général De Gaulle - Alouette', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.839109, 2.4175664], 'Nom communes équipées': 'Saint-Mandé'}}, 266: {'attributes': {'Identifiant station': 11036, 'Nom station': 'Jules Ferry - Faubourg du Temple Paris', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 58, 'Nombre bornettes libres': 51, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.868397, 2.3678933], 'Nom communes équipées': 'Paris'}}, 267: {'attributes': {'Identifiant station': 14110, 'Nom station': 'Abbé Carton - Plantes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8276738232, 2.3209181428], 'Nom communes équipées': 'Paris'}}, 268: {'attributes': {'Identifiant station': 14036, 'Nom station': 'Boulard - Daguerre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 43, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8344647268, 2.32933860272], 'Nom communes équipées': 'Paris'}}, 269: {'attributes': {'Identifiant station': 17125, 'Nom station': 'Marie-Georges Picquart - Pereire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8886366935, 2.31117065996], 'Nom communes équipées': 'Paris'}}, 270: {'attributes': {'Identifiant station': 23009, 'Nom station': 'Anatole France - Louise Michel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 38, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 14, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8887321493, 2.28815764189], 'Nom communes équipées': 'Levallois-Perret'}}, 271: {'attributes': {'Identifiant station': 43005, 'Nom station': 'Gare RER de Vincennes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 51, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 13, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8475630151, 2.43338316679], 'Nom communes équipées': 'Vincennes'}}, 272: {'attributes': {'Identifiant station': 11007, 'Nom station': 'Jean Macé - Faidherbe', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 63, 'Nombre bornettes libres': 62, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8533754843, 2.38286208361], 'Nom communes équipées': 'Paris'}}, 273: {'attributes': {'Identifiant station': 19032, 'Nom station': 'Villette - Belleville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 48, 'Nombre bornettes libres': 44, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8726877314, 2.37646132708], 'Nom communes équipées': 'Paris'}}, 274: {'attributes': {'Identifiant station': 32303, 'Nom station': 'Gaston Roussel - Commune de Paris', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8943203455, 2.43255227442], 'Nom communes équipées': 'Romainville'}}, 275: {'attributes': {'Identifiant station': 16136, 'Nom station': 'Théophile Gautier - François Millet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8503482132, 2.27302998304], 'Nom communes équipées': 'Paris'}}, 276: {'attributes': {'Identifiant station': 8035, 'Nom station': 'Bienfaisance - Place de Narvik', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 30, 'Vélos mécaniques disponibles': 24, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8766016346, 2.31297712773], 'Nom communes équipées': 'Paris'}}, 277: {'attributes': {'Identifiant station': 19022, 'Nom station': 'Marché Secrétan', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8812549638, 2.37346328795], 'Nom communes équipées': 'Paris'}}, 278: {'attributes': {'Identifiant station': 12027, 'Nom station': 'Montgallet - Charenton', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8426477787, 2.38616492781], 'Nom communes équipées': 'Paris'}}, 279: {'attributes': {'Identifiant station': 22016, 'Nom station': 'Charles de Gaulle - Sablonville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 18, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8799192486, 2.27637704462], 'Nom communes équipées': 'Neuilly-sur-Seine'}}, 280: {'attributes': {'Identifiant station': 21313, 'Nom station': 'Guynemer - Séverine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8309174337, 2.27907065302], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 281: {'attributes': {'Identifiant station': 14136, 'Nom station': 'Auguste Cain - Jean Moulin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 40, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8257702893, 2.32104152441], 'Nom communes équipées': 'Paris'}}, 282: {'attributes': {'Identifiant station': 6013, 'Nom station': 'Jacques Callot - Mazarine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 19, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8552801753, 2.3375743255], 'Nom communes équipées': 'Paris'}}, 283: {'attributes': {'Identifiant station': 32302, 'Nom station': 'Chemin Vert - Fraternité', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8771412084, 2.44615651667], 'Nom communes équipées': 'Romainville'}}, 284: {'attributes': {'Identifiant station': 28001, 'Nom station': 'Richard Wallace - Place du 8 Mai 1945', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8786679479, 2.24203437567], 'Nom communes équipées': 'Puteaux'}}, 285: {'attributes': {'Identifiant station': 11035, 'Nom station': 'Parmentier - Abel Rabaud', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 52, 'Nombre bornettes libres': 47, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8692127384, 2.37168882124], 'Nom communes équipées': 'Paris'}}, 286: {'attributes': {'Identifiant station': 7015, 'Nom station': 'Varenne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 47, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 28, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8572026618, 2.31527712196], 'Nom communes équipées': 'Paris'}}, 287: {'attributes': {'Identifiant station': 42010, 'Nom station': "Mairie d'Ivry-sur-Seine", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 53, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 45, 'Vélos mécaniques disponibles': 40, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8105712097, 2.38428264856], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 288: {'attributes': {'Identifiant station': 14021, 'Nom station': 'Place de la Porte de Châtillon', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 59, 'Nombre bornettes libres': 59, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8242469, 2.3186306], 'Nom communes équipées': 'Paris'}}, 289: {'attributes': {'Identifiant station': 15046, 'Nom station': 'Parc Georges Brassens - Brancion', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8322782927, 2.30225700885], 'Nom communes équipées': 'Paris'}}, 290: {'attributes': {'Identifiant station': 7112, 'Nom station': "Quai d'Orsay - Invalides", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 1, 23, 13, 2, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8630969001, 2.31698962102], 'Nom communes équipées': 'Paris'}}, 291: {'attributes': {'Identifiant station': 16040, 'Nom station': 'Exelmans - Michel-Ange', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8430948665, 2.25977708007], 'Nom communes équipées': 'Paris'}}, 292: {'attributes': {'Identifiant station': 11115, 'Nom station': 'Montreuil - Gonnet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8505005649, 2.38908562879], 'Nom communes équipées': 'Paris'}}, 293: {'attributes': {'Identifiant station': 22006, 'Nom station': 'Pierret - Madrid', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 14, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8829765769, 2.26012576725], 'Nom communes équipées': 'Neuilly-sur-Seine'}}, 294: {'attributes': {'Identifiant station': 12013, 'Nom station': 'Montgallet - Reuilly', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.844291, 2.3896481], 'Nom communes équipées': 'Paris'}}, 295: {'attributes': {'Identifiant station': 21301, 'Nom station': 'Saint-Vincent - Verdun', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 29, 'Vélos mécaniques disponibles': 26, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8192115492, 2.25400775671], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 296: {'attributes': {'Identifiant station': 15025, 'Nom station': 'Dupleix - du Guesclin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 47, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 46, 'Vélos mécaniques disponibles': 44, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8514791041, 2.29670953006], 'Nom communes équipées': 'Paris'}}, 297: {'attributes': {'Identifiant station': 19029, 'Nom station': 'Seine - Flandre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8867857415, 2.37461932003], 'Nom communes équipées': 'Paris'}}, 298: {'attributes': {'Identifiant station': 32602, 'Nom station': 'Mairie des Lilas', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 47, 'Nombre bornettes libres': 45, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8792834, 2.4160182], 'Nom communes équipées': 'Les Lilas'}}, 299: {'attributes': {'Identifiant station': 32003, 'Nom station': 'Wilson - Métallurgie', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9094230101, 2.35844358802], 'Nom communes équipées': 'Saint-Denis'}}, 300: {'attributes': {'Identifiant station': 12115, 'Nom station': 'Porte de Vincennes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.846454, 2.415514], 'Nom communes équipées': 'Paris'}}, 301: {'attributes': {'Identifiant station': 21104, 'Nom station': 'Madame de Sanzillon - Mozart', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9037982077, 2.31874018908], 'Nom communes équipées': 'Clichy'}}, 302: {'attributes': {'Identifiant station': 17011, 'Nom station': 'Fragonard - Porte de Clichy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 56, 'Nombre bornettes libres': 37, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8941556334, 2.31438931078], 'Nom communes équipées': 'Paris'}}, 303: {'attributes': {'Identifiant station': 19016, 'Nom station': 'Hautpoul - Jean Jaurès', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8863936794, 2.38654145444], 'Nom communes équipées': 'Paris'}}, 304: {'attributes': {'Identifiant station': 43402, 'Nom station': 'Place de Verdun', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 37, 'Vélos mécaniques disponibles': 36, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8202823, 2.473775], 'Nom communes équipées': 'Joinville-le-Pont'}}, 305: {'attributes': {'Identifiant station': 9005, 'Nom station': "Square d'Anvers", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8826334843, 2.34488440621], 'Nom communes équipées': 'Paris'}}, 306: {'attributes': {'Identifiant station': 33018, 'Nom station': 'Edouard Poisson - Victor Hugo', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9111855705, 2.37986475269], 'Nom communes équipées': 'Aubervilliers'}}, 307: {'attributes': {'Identifiant station': 19013, 'Nom station': 'Laumière - Petit', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8846736854, 2.38003905863], 'Nom communes équipées': 'Paris'}}, 308: {'attributes': {'Identifiant station': 16121, 'Nom station': "Jardin d'Acclimatation", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8787722425, 2.27077223361], 'Nom communes équipées': 'Paris'}}, 309: {'attributes': {'Identifiant station': 42504, 'Nom station': 'Gare RER de Gentilly', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 2, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8142804504, 2.34103471041], 'Nom communes équipées': 'Gentilly'}}, 310: {'attributes': {'Identifiant station': 13049, 'Nom station': 'Vincent Auriol - Louise Weiss', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 12, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8340019355, 2.37038880587], 'Nom communes équipées': 'Paris'}}, 311: {'attributes': {'Identifiant station': 20111, 'Nom station': "Villiers de l'Isle Adam - Pyrénées", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.867362, 2.396222], 'Nom communes équipées': 'Paris'}}, 312: {'attributes': {'Identifiant station': 11102, 'Nom station': 'Alexandre Dumas - Voltaire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 59, 'Nombre bornettes libres': 51, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8528886704, 2.38914764081], 'Nom communes équipées': 'Paris'}}, 313: {'attributes': {'Identifiant station': 9021, 'Nom station': "Saint Georges - d'Aumale", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.877786396, 2.33742982149], 'Nom communes équipées': 'Paris'}}, 314: {'attributes': {'Identifiant station': 19004, 'Nom station': 'Quai de la Loire - Jean Jaurès', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8833973385, 2.37107452298], 'Nom communes équipées': 'Paris'}}, 315: {'attributes': {'Identifiant station': 31010, 'Nom station': 'Stalingrad - Désiré Chevalier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 40, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8567016974, 2.44507223368], 'Nom communes équipées': 'Montreuil'}}, 316: {'attributes': {'Identifiant station': 16124, 'Nom station': 'Route de Sèvres - Porte de Bagatelle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 66, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 39, 'Vélos mécaniques disponibles': 38, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8752598413, 2.24941581488], 'Nom communes équipées': 'Paris'}}, 317: {'attributes': {'Identifiant station': 92008, 'Nom station': 'Hôtel de Ville de Nanterre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8923718983, 2.20554895699], 'Nom communes équipées': 'Nanterre'}}, 318: {'attributes': {'Identifiant station': 33003, 'Nom station': 'Bordier - Félix Faure', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 48, 'Nombre bornettes libres': 43, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9039075188, 2.38300591707], 'Nom communes équipées': 'Aubervilliers'}}, 319: {'attributes': {'Identifiant station': 41402, 'Nom station': 'Victor Hugo - Ecoles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8047600794, 2.41803739303], 'Nom communes équipées': 'Alfortville'}}, 320: {'attributes': {'Identifiant station': 20043, 'Nom station': 'Charonne - Avron', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8516026652, 2.39840304704], 'Nom communes équipées': 'Paris'}}, 321: {'attributes': {'Identifiant station': 7005, 'Nom station': 'Raspail - Bac', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 52, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 42, 'Vélos mécaniques disponibles': 38, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8556287211, 2.32566028833], 'Nom communes équipées': 'Paris'}}, 322: {'attributes': {'Identifiant station': 22505, 'Nom station': 'Mairie de La Garenne Colombes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9066382347, 2.24679596722], 'Nom communes équipées': 'La Garenne-Colombes'}}, 323: {'attributes': {'Identifiant station': 18012, 'Nom station': 'Stephenson - Doudeauville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 35, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8885370489, 2.35616102815], 'Nom communes équipées': 'Paris'}}, 324: {'attributes': {'Identifiant station': 13045, 'Nom station': 'Jean Colly - Tolbiac', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 24, 'Vélos mécaniques disponibles': 22, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8278172916, 2.37054906785], 'Nom communes équipées': 'Paris'}}, 325: {'attributes': {'Identifiant station': 19125, 'Nom station': "Place de l'Édit de Nantes", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.88879, 2.3785], 'Nom communes équipées': 'Paris'}}, 326: {'attributes': {'Identifiant station': 32010, 'Nom station': 'Place aux Étoiles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 24, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 15, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9183562941, 2.35234256834], 'Nom communes équipées': 'Saint-Denis'}}, 327: {'attributes': {'Identifiant station': 18201, 'Nom station': 'Tristan Tzara -  Tchaïkovski', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8949022175, 2.36484903842], 'Nom communes équipées': 'Paris'}}, 328: {'attributes': {'Identifiant station': 16018, 'Nom station': 'Flandrin - Henri Martin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8643275, 2.2724174], 'Nom communes équipées': 'Paris'}}, 329: {'attributes': {'Identifiant station': 11009, 'Nom station': 'Boulets - Voltaire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8521870562, 2.38903015852], 'Nom communes équipées': 'Paris'}}, 330: {'attributes': {'Identifiant station': 15010, 'Nom station': 'Square Cambronne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 59, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 33, 'Vélos mécaniques disponibles': 29, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8475665453, 2.30256915092], 'Nom communes équipées': 'Paris'}}, 331: {'attributes': {'Identifiant station': 42029, 'Nom station': 'Stalingrad - Germaine Tailleferre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8025720048, 2.37866912037], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 332: {'attributes': {'Identifiant station': 7009, 'Nom station': 'Assemblée Nationale', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 63, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 55, 'Vélos mécaniques disponibles': 39, 'Vélos électriques disponibles': 16, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8614110147, 2.32012141852], 'Nom communes équipées': 'Paris'}}, 333: {'attributes': {'Identifiant station': 21305, 'Nom station': 'Place Jacques Madaule', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 24, 'Vélos mécaniques disponibles': 20, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8239337873, 2.26069182158], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 334: {'attributes': {'Identifiant station': 17042, 'Nom station': 'Porte Maillot - Pereire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 52, 'Nombre bornettes libres': 45, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.877775, 2.284444], 'Nom communes équipées': 'Paris'}}, 335: {'attributes': {'Identifiant station': 10113, 'Nom station': 'Parmentier - Hôpital Saint-Louis', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 42, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8713798743, 2.36989638367], 'Nom communes équipées': 'Paris'}}, 336: {'attributes': {'Identifiant station': 33019, 'Nom station': 'Madeleine Vionnet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9025190741, 2.37408958375], 'Nom communes équipées': 'Aubervilliers'}}, 337: {'attributes': {'Identifiant station': 26007, 'Nom station': 'Square Gilbert Thomain', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9071737381, 2.26954009384], 'Nom communes équipées': 'Asnières-sur-Seine'}}, 338: {'attributes': {'Identifiant station': 8033, 'Nom station': 'Commandant Rivière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 26, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 12, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.872933, 2.309796], 'Nom communes équipées': 'Paris'}}, 339: {'attributes': {'Identifiant station': 17102, 'Nom station': 'Alfred Roll - Berthier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8878085705, 2.30009529818], 'Nom communes équipées': 'Paris'}}, 340: {'attributes': {'Identifiant station': 33001, 'Nom station': 'Victor Hugo - Magasins Généraux', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9032814011, 2.36807238311], 'Nom communes équipées': 'Aubervilliers'}}, 341: {'attributes': {'Identifiant station': 16111, 'Nom station': 'Faustin Hélie - Desbordes-Valmore', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 59, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.859403, 2.276522], 'Nom communes équipées': 'Paris'}}, 342: {'attributes': {'Identifiant station': 12029, 'Nom station': 'Dugommier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 55, 'Nombre bornettes libres': 45, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8388110488, 2.38965779543], 'Nom communes équipées': 'Paris'}}, 343: {'attributes': {'Identifiant station': 19005, 'Nom station': 'Flandre - Riquet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8880132814, 2.37375959552], 'Nom communes équipées': 'Paris'}}, 344: {'attributes': {'Identifiant station': 20119, 'Nom station': 'Square des Saint-Simoniens', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8701796, 2.3967273], 'Nom communes équipées': 'Paris'}}, 345: {'attributes': {'Identifiant station': 35018, 'Nom station': 'Ancien Canal - Hélène Brion', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.894847993, 2.41883385926], 'Nom communes équipées': 'Pantin'}}, 346: {'attributes': {'Identifiant station': 42207, 'Nom station': 'Place des Marseillais', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8266390725, 2.40592807531], 'Nom communes équipées': 'Charenton-le-Pont'}}, 347: {'attributes': {'Identifiant station': 18110, 'Nom station': 'Departement - Pajol', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8866748497, 2.36136116087], 'Nom communes équipées': 'Paris'}}, 348: {'attributes': {'Identifiant station': 12030, 'Nom station': 'Charenton - Wattignies', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8366757964, 2.39179585556], 'Nom communes équipées': 'Paris'}}, 349: {'attributes': {'Identifiant station': 41606, 'Nom station': 'Pasteur - Gambetta', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8457279738, 2.42360115051], 'Nom communes équipées': 'Saint-Mandé'}}, 350: {'attributes': {'Identifiant station': 20036, 'Nom station': 'Ménilmontant - Pelleport', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8706826338, 2.39895960233], 'Nom communes équipées': 'Paris'}}, 351: {'attributes': {'Identifiant station': 17045, 'Nom station': 'Place des Ternes.', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8784386312, 2.29772776365], 'Nom communes équipées': 'Paris'}}, 352: {'attributes': {'Identifiant station': 35009, 'Nom station': 'Jean Lolive - Parc Stalingrad', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.892061744, 2.40917354822], 'Nom communes équipées': 'Pantin'}}, 353: {'attributes': {'Identifiant station': 24002, 'Nom station': 'Gare de Courbevoie', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 32, 'Vélos mécaniques disponibles': 29, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8993867, 2.2506506], 'Nom communes équipées': 'Courbevoie'}}, 354: {'attributes': {'Identifiant station': 27002, 'Nom station': 'Parc Pierre Lagravère', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9265048813, 2.23906967849], 'Nom communes équipées': 'Colombes'}}, 355: {'attributes': {'Identifiant station': 14117, 'Nom station': 'Gare Montparnasse - René Mouchotte', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 35, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8392038433, 2.32088394463], 'Nom communes équipées': 'Paris'}}, 356: {'attributes': {'Identifiant station': 21109, 'Nom station': 'Guichet - Jean Jaurès', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9051698681, 2.3023840785], 'Nom communes équipées': 'Clichy'}}, 357: {'attributes': {'Identifiant station': 18141, 'Nom station': 'Clignancourt - Sofia', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8851199231, 2.34719488842], 'Nom communes équipées': 'Paris'}}, 358: {'attributes': {'Identifiant station': 20120, 'Nom station': 'Evariste Galois', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8730875373, 2.41333801299], 'Nom communes équipées': 'Paris'}}, 359: {'attributes': {'Identifiant station': 23001, 'Nom station': 'Paul Vaillant Couturier - Chaptal', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 33, 'Vélos mécaniques disponibles': 26, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8934795762, 2.27752536535], 'Nom communes équipées': 'Levallois-Perret'}}, 360: {'attributes': {'Identifiant station': 15126, 'Nom station': 'Ernest Renan - Parc des Expositions', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 9, 9, 23, 21, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8308975701, 2.2853584215], 'Nom communes équipées': 'Paris'}}, 361: {'attributes': {'Identifiant station': 41102, 'Nom station': 'Gare de Laplace', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 41, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8089319911, 2.33494609594], 'Nom communes équipées': 'Arcueil'}}, 362: {'attributes': {'Identifiant station': 21502, 'Nom station': 'Verdun - Place Henri IV', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8707973756, 2.22770031542], 'Nom communes équipées': 'Suresnes'}}, 363: {'attributes': {'Identifiant station': 17037, 'Nom station': 'Bayen - Pereire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 14, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8818044091, 2.29210887104], 'Nom communes équipées': 'Paris'}}, 364: {'attributes': {'Identifiant station': 14001, 'Nom station': 'Edgar Quinet - Gaité', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.841160929, 2.32459545135], 'Nom communes équipées': 'Paris'}}, 365: {'attributes': {'Identifiant station': 33009, 'Nom station': 'Félix Faure - Sadi Carnot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9081211023, 2.37862318754], 'Nom communes équipées': 'Aubervilliers'}}, 366: {'attributes': {'Identifiant station': 31012, 'Nom station': 'Place Carnot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8559393219, 2.44153171778], 'Nom communes équipées': 'Montreuil'}}, 367: {'attributes': {'Identifiant station': 5021, 'Nom station': 'Jussieu - Fossés Saint-Bernard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 36, 'Vélos mécaniques disponibles': 26, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.84718551, 2.35348585993], 'Nom communes équipées': 'Paris'}}, 368: {'attributes': {'Identifiant station': 5116, 'Nom station': 'Place Valhubert', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 25, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8441825773, 2.36403666437], 'Nom communes équipées': 'Paris'}}, 369: {'attributes': {'Identifiant station': 25005, 'Nom station': '18 juin 1940 - Buzenval', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.868807, 2.185428], 'Nom communes équipées': 'Rueil-Malmaison'}}, 370: {'attributes': {'Identifiant station': 20113, 'Nom station': 'Piat - Parc de Belleville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8719562664, 2.38498139028], 'Nom communes équipées': 'Paris'}}, 371: {'attributes': {'Identifiant station': 19115, 'Nom station': 'Porte de la Villette', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8983519655, 2.38615013659], 'Nom communes équipées': 'Paris'}}, 372: {'attributes': {'Identifiant station': 41604, 'Nom station': 'Lagny - Joffre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8493068708, 2.41794973612], 'Nom communes équipées': 'Saint-Mandé'}}, 373: {'attributes': {'Identifiant station': 14034, 'Nom station': 'Ouest - Château', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8361728601, 2.31939214392], 'Nom communes équipées': 'Paris'}}, 374: {'attributes': {'Identifiant station': 15017, 'Nom station': 'Place Falguière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8362997394, 2.31044646353], 'Nom communes équipées': 'Paris'}}, 375: {'attributes': {'Identifiant station': 12034, 'Nom station': 'Pirogues de Bercy - Libourne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 49, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 47, 'Vélos mécaniques disponibles': 41, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.832308, 2.38637], 'Nom communes équipées': 'Paris'}}, 376: {'attributes': {'Identifiant station': 31702, 'Nom station': 'Pierre et Marie Curie - Julien Grimau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8707140149, 2.4240970239], 'Nom communes équipées': 'Bagnolet'}}, 377: {'attributes': {'Identifiant station': 15033, 'Nom station': 'Place Violet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8448789384, 2.29057565331], 'Nom communes équipées': 'Paris'}}, 378: {'attributes': {'Identifiant station': 5002, 'Nom station': 'Sommerard - Saint-Jacques', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 27, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8503104329, 2.3450158353], 'Nom communes équipées': 'Paris'}}, 379: {'attributes': {'Identifiant station': 15122, 'Nom station': 'Place Charles Vallin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 3, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.835523812, 2.30259865522], 'Nom communes équipées': 'Paris'}}, 380: {'attributes': {'Identifiant station': 15028, 'Nom station': 'Grenelle - Dr Finlay', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 71, 'Nombre bornettes libres': 71, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 3, 29, 9, 17, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8513154067, 2.29195028543], 'Nom communes équipées': 'Paris'}}, 381: {'attributes': {'Identifiant station': 16034, 'Nom station': "Porte d'Auteuil", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8479394142, 2.26083967835], 'Nom communes équipées': 'Paris'}}, 382: {'attributes': {'Identifiant station': 92004, 'Nom station': 'Nanterre - Université', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 18, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.901365, 2.212693], 'Nom communes équipées': 'Nanterre'}}, 383: {'attributes': {'Identifiant station': 14019, 'Nom station': 'Sarrette - Général Leclerc', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8250308529, 2.3264128694], 'Nom communes équipées': 'Paris'}}, 384: {'attributes': {'Identifiant station': 20136, 'Nom station': 'Henri Chevreau - Couronnes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8703320531, 2.3887776956], 'Nom communes équipées': 'Paris'}}, 385: {'attributes': {'Identifiant station': 20132, 'Nom station': 'Réunion - Avron', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.852566752, 2.40392806871], 'Nom communes équipées': 'Paris'}}, 386: {'attributes': {'Identifiant station': 8012, 'Nom station': 'Batignolles - Place de Clichy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 47, 'Nombre bornettes libres': 41, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8834745358, 2.32649311423], 'Nom communes équipées': 'Paris'}}, 387: {'attributes': {'Identifiant station': 5114, 'Nom station': 'Claude Bernard - Arbalète', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8397667698, 2.34751090407], 'Nom communes équipées': 'Paris'}}, 388: {'attributes': {'Identifiant station': 21111, 'Nom station': 'Professeur René Leriche - Général Leclerc', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 39, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9078284548, 2.3078879714], 'Nom communes équipées': 'Clichy'}}, 389: {'attributes': {'Identifiant station': 19019, 'Nom station': 'Petit - Manin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 2, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8865068753, 2.39381554013], 'Nom communes équipées': 'Paris'}}, 390: {'attributes': {'Identifiant station': 7006, 'Nom station': 'Quai Voltaire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 26, 'Vélos mécaniques disponibles': 24, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 3, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8589429862, 2.33141820471], 'Nom communes équipées': 'Paris'}}, 391: {'attributes': {'Identifiant station': 21403, 'Nom station': 'Gare de Clamart', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 14, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.813835356, 2.27227896452], 'Nom communes équipées': 'Clamart'}}, 392: {'attributes': {'Identifiant station': 11114, 'Nom station': 'Ledru-Rollin - Charonne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8534585561, 2.37724635819], 'Nom communes équipées': 'Paris'}}, 393: {'attributes': {'Identifiant station': 42707, 'Nom station': 'Place Jean-Baptiste Clément', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 2, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8124046687, 2.36170917749], 'Nom communes équipées': 'Le Kremlin-Bicêtre'}}, 394: {'attributes': {'Identifiant station': 12011, 'Nom station': 'Charenton - Place du Col Bourgoin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8448673196, 2.38257893505], 'Nom communes équipées': 'Paris'}}, 395: {'attributes': {'Identifiant station': 92003, 'Nom station': 'Chanzy - Sud-Est', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8889120377, 2.19563215971], 'Nom communes équipées': 'Nanterre'}}, 396: {'attributes': {'Identifiant station': 10023, 'Nom station': "Gare de l'Est - Verdun", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 14, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8761560543, 2.36091490832], 'Nom communes équipées': 'Paris'}}, 397: {'attributes': {'Identifiant station': 5119, 'Nom station': 'Jussieu - Guy de la Brosse', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 35, 'Vélos mécaniques disponibles': 30, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.845665, 2.355653], 'Nom communes équipées': 'Paris'}}, 398: {'attributes': {'Identifiant station': 23002, 'Nom station': 'Pont de Levallois-bécon', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 46, 'Vélos mécaniques disponibles': 41, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8984179, 2.2791311], 'Nom communes équipées': 'Levallois-Perret'}}, 399: {'attributes': {'Identifiant station': 31705, 'Nom station': 'Champeaux - Gallieni', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 49, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8644591347, 2.41617780179], 'Nom communes équipées': 'Bagnolet'}}, 400: {'attributes': {'Identifiant station': 14139, 'Nom station': 'Pierre Larousse - Raymond Losserand', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8305956423, 2.31191832572], 'Nom communes équipées': 'Paris'}}, 401: {'attributes': {'Identifiant station': 8055, 'Nom station': 'Courcelles - Place des Ternes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8781999289, 2.29928601887], 'Nom communes équipées': 'Paris'}}, 402: {'attributes': {'Identifiant station': 13102, 'Nom station': 'Pitié-Salpêtrière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 51, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 45, 'Vélos mécaniques disponibles': 36, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8374881032, 2.36521381885], 'Nom communes équipées': 'Paris'}}, 403: {'attributes': {'Identifiant station': 13014, 'Nom station': "Gare d'Austerlitz", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 14, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.842524, 2.364044], 'Nom communes équipées': 'Paris'}}, 404: {'attributes': {'Identifiant station': 11101, 'Nom station': 'Charles Delescluze', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 49, 'Nombre bornettes libres': 41, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8524220085, 2.38063249737], 'Nom communes équipées': 'Paris'}}, 405: {'attributes': {'Identifiant station': 23701, 'Nom station': 'Parvis Robert Schuman - Roger Salengro', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8126574706, 2.19217512757], 'Nom communes équipées': 'Chaville'}}, 406: {'attributes': {'Identifiant station': 14130, 'Nom station': 'Jonquoy - Didot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8289045482, 2.31648076326], 'Nom communes équipées': 'Paris'}}, 407: {'attributes': {'Identifiant station': 12126, 'Nom station': 'Gravelle - Route du Bac', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 51, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 18, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.824407, 2.418455], 'Nom communes équipées': 'Paris'}}, 408: {'attributes': {'Identifiant station': 10015, 'Nom station': 'Lucien Sampaix - Récollets', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8743303776, 2.36212290823], 'Nom communes équipées': 'Paris'}}, 409: {'attributes': {'Identifiant station': 42023, 'Nom station': 'Place Léon Gambetta', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8146125024, 2.40292936563], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 410: {'attributes': {'Identifiant station': 41207, 'Nom station': 'Place du Général de Gaulle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8525288, 2.4877332], 'Nom communes équipées': 'Fontenay-sous-Bois'}}, 411: {'attributes': {'Identifiant station': 16129, 'Nom station': 'Ceinture du Lac Inferieure - Saint-Cloud', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8621807093, 2.26178716868], 'Nom communes équipées': 'Paris'}}, 412: {'attributes': {'Identifiant station': 7014, 'Nom station': 'Saint-François-Xavier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8515448477, 2.31461495161], 'Nom communes équipées': 'Paris'}}, 413: {'attributes': {'Identifiant station': 35019, 'Nom station': 'Anatole France - Jean Lolive', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8949922, 2.4253764], 'Nom communes équipées': 'Pantin'}}, 414: {'attributes': {'Identifiant station': 8101, 'Nom station': 'Londres - Amsterdam', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 12, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8780344615, 2.32648354097], 'Nom communes équipées': 'Paris'}}, 415: {'attributes': {'Identifiant station': 14104, 'Nom station': 'Boyer-Barret - Raymond Losserand', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8332988023, 2.31733839959], 'Nom communes équipées': 'Paris'}}, 416: {'attributes': {'Identifiant station': 6018, 'Nom station': 'Michelet - Assas', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8427415028, 2.3350211978], 'Nom communes équipées': 'Paris'}}, 417: {'attributes': {'Identifiant station': 17018, 'Nom station': 'Parc Monceau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8810779141, 2.30954926461], 'Nom communes équipées': 'Paris'}}, 418: {'attributes': {'Identifiant station': 19037, 'Nom station': 'Belleville - Porte des Lilas', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8766039189, 2.40476969113], 'Nom communes équipées': 'Paris'}}, 419: {'attributes': {'Identifiant station': 14018, 'Nom station': "Porte d'Orléans", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 48, 'Nombre bornettes libres': 47, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8227639, 2.3249313], 'Nom communes équipées': 'Paris'}}, 420: {'attributes': {'Identifiant station': 20109, 'Nom station': 'Haxo - Surmelin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8694370761, 2.40539657538], 'Nom communes équipées': 'Paris'}}, 421: {'attributes': {'Identifiant station': 15029, 'Nom station': 'Emeriau - Place de Brazzaville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 51, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 48, 'Vélos mécaniques disponibles': 44, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8507852617, 2.28715851903], 'Nom communes équipées': 'Paris'}}, 422: {'attributes': {'Identifiant station': 15038, 'Nom station': 'Place Adolphe Cherioux - Vaugirard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 63, 'Nombre bornettes libres': 58, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.839594428, 2.30106577277], 'Nom communes équipées': 'Paris'}}, 423: {'attributes': {'Identifiant station': 18005, 'Nom station': 'Tardieu - Chappe', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8841467598, 2.34184540808], 'Nom communes équipées': 'Paris'}}, 424: {'attributes': {'Identifiant station': 19109, 'Nom station': 'Tanger - Place du Maroc', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8862524447, 2.36892968416], 'Nom communes équipées': 'Paris'}}, 425: {'attributes': {'Identifiant station': 10005, 'Nom station': 'Petites Ecuries - Faubourg Poissonnière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8739721724, 2.3483887575], 'Nom communes équipées': 'Paris'}}, 426: {'attributes': {'Identifiant station': 31019, 'Nom station': 'Vincennes - Douy-Delcupe', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 37, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 14, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8549369216, 2.43604928255], 'Nom communes équipées': 'Montreuil'}}, 427: {'attributes': {'Identifiant station': 12024, 'Nom station': 'Gare de Bercy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 31, 'Vélos mécaniques disponibles': 25, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8397945739, 2.38260660321], 'Nom communes équipées': 'Paris'}}, 428: {'attributes': {'Identifiant station': 18032, 'Nom station': 'Porte de Clignancourt.', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 56, 'Nombre bornettes libres': 48, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.897488, 2.343996], 'Nom communes équipées': 'Paris'}}, 429: {'attributes': {'Identifiant station': 18020, 'Nom station': 'Francoeur - Marcadet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8910412326, 2.34001458544], 'Nom communes équipées': 'Paris'}}, 430: {'attributes': {'Identifiant station': 17027, 'Nom station': 'Place du Maréchal Juin - Péreire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 44, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.885263, 2.298286], 'Nom communes équipées': 'Paris'}}, 431: {'attributes': {'Identifiant station': 5026, 'Nom station': 'Pascal - Claude Bernard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 49, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8390619522, 2.34997820109], 'Nom communes équipées': 'Paris'}}, 432: {'attributes': {'Identifiant station': 22101, 'Nom station': 'Gare Routière de Saint Cloud', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.842881839, 2.22224503756], 'Nom communes équipées': 'Saint-Cloud'}}, 433: {'attributes': {'Identifiant station': 43006, 'Nom station': 'Vignerons - Minimes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.841486, 2.430793], 'Nom communes équipées': 'Vincennes'}}, 434: {'attributes': {'Identifiant station': 13016, 'Nom station': 'Primo Levi - Frigos', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 24, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 3, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8309484485, 2.37965351593], 'Nom communes équipées': 'Paris'}}, 435: {'attributes': {'Identifiant station': 13104, 'Nom station': "Gare d'Austerlitz - Quai Saint-Bernard", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8436903086, 2.36571639776], 'Nom communes équipées': 'Paris'}}, 436: {'attributes': {'Identifiant station': 11020, 'Nom station': 'Belfort - Voltaire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8563358, 2.3829901], 'Nom communes équipées': 'Paris'}}, 437: {'attributes': {'Identifiant station': 8102, 'Nom station': 'Lamennais - Washington', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 12, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8735299003, 2.30344489217], 'Nom communes équipées': 'Paris'}}, 438: {'attributes': {'Identifiant station': 8051, 'Nom station': 'Batignolles - Rome', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 15, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8818233706, 2.32025898993], 'Nom communes équipées': 'Paris'}}, 439: {'attributes': {'Identifiant station': 21013, 'Nom station': 'Pierre Grenier - Pont de Billancourt', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8258020743, 2.2487989068], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 440: {'attributes': {'Identifiant station': 13043, 'Nom station': 'Dunois - Clisson', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 51, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 26, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.832294, 2.3674523], 'Nom communes équipées': 'Paris'}}, 441: {'attributes': {'Identifiant station': 2010, 'Nom station': 'Uzès - Montmartre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 24, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8707909802, 2.34310101718], 'Nom communes équipées': 'Paris'}}, 442: {'attributes': {'Identifiant station': 14122, 'Nom station': 'Colonel Monteil - Brune', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8265029331, 2.30927479983], 'Nom communes équipées': 'Paris'}}, 443: {'attributes': {'Identifiant station': 23011, 'Nom station': 'Bineau - Louise Michel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8866616221, 2.28432744741], 'Nom communes équipées': 'Levallois-Perret'}}, 444: {'attributes': {'Identifiant station': 19039, 'Nom station': 'Edouard Pailleron - Simon Bolivar', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8797988605, 2.37483926117], 'Nom communes équipées': 'Paris'}}, 445: {'attributes': {'Identifiant station': 21308, 'Nom station': 'Bara - Henri Farman', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 35, 'Vélos mécaniques disponibles': 26, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8332180275, 2.26775676012], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 446: {'attributes': {'Identifiant station': 18025, 'Nom station': 'Mairie du 18ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8929392969, 2.34455108643], 'Nom communes équipées': 'Paris'}}, 447: {'attributes': {'Identifiant station': 17007, 'Nom station': 'Navier - Epinettes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8958778077, 2.32267767191], 'Nom communes équipées': 'Paris'}}, 448: {'attributes': {'Identifiant station': 9023, 'Nom station': 'Laffitte - Italiens', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 31, 'Vélos mécaniques disponibles': 24, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8718157374, 2.33720154785], 'Nom communes équipées': 'Paris'}}, 449: {'attributes': {'Identifiant station': 20040, 'Nom station': 'Parc de Belleville - Julien Lacroix', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8703459, 2.3842214], 'Nom communes équipées': 'Paris'}}, 450: {'attributes': {'Identifiant station': 20021, 'Nom station': 'Indre - Prairies', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8625439851, 2.40329045802], 'Nom communes équipées': 'Paris'}}, 451: {'attributes': {'Identifiant station': 42503, 'Nom station': 'Vladimir Ilitch Lénine - Andrée Chedid', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8131995, 2.33214586973], 'Nom communes équipées': 'Arcueil'}}, 452: {'attributes': {'Identifiant station': 26001, 'Nom station': "Gare d'Asnières sur Seine", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9063645922, 2.28350669146], 'Nom communes équipées': 'Asnières-sur-Seine'}}, 453: {'attributes': {'Identifiant station': 17017, 'Nom station': 'Gare de Pont-Cardinet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8871390148, 2.31449414399], 'Nom communes équipées': 'Paris'}}, 454: {'attributes': {'Identifiant station': 17047, 'Nom station': 'Courcelles - Pierre Demours', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8837697333, 2.29858466343], 'Nom communes équipées': 'Paris'}}, 455: {'attributes': {'Identifiant station': 21105, 'Nom station': 'Morice - Général Leclerc', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9024547198, 2.31289297342], 'Nom communes équipées': 'Clichy'}}, 456: {'attributes': {'Identifiant station': 11040, 'Nom station': 'Temple - Jean-Pierre Timbaud', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.864465972, 2.36605972052], 'Nom communes équipées': 'Paris'}}, 457: {'attributes': {'Identifiant station': 13026, 'Nom station': 'Place Jean Delay', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.823316, 2.354336], 'Nom communes équipées': 'Paris'}}, 458: {'attributes': {'Identifiant station': 41204, 'Nom station': 'République - Danton', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 43, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8524917218, 2.46203452349], 'Nom communes équipées': 'Fontenay-sous-Bois'}}, 459: {'attributes': {'Identifiant station': 15055, 'Nom station': 'Vasco de Gama', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 54, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 45, 'Vélos mécaniques disponibles': 41, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8367178347, 2.28366408565], 'Nom communes équipées': 'Paris'}}, 460: {'attributes': {'Identifiant station': 19009, 'Nom station': "Cité des Sciences et de l'Industrie", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8965957118, 2.38459914923], 'Nom communes équipées': 'Paris'}}, 461: {'attributes': {'Identifiant station': 34002, 'Nom station': 'Capitaine Glarner - Gabriel Péri', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9024499004, 2.33048075334], 'Nom communes équipées': 'Saint-Ouen-sur-Seine'}}, 462: {'attributes': {'Identifiant station': 32017, 'Nom station': 'Basilique', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9362689106, 2.35886668202], 'Nom communes équipées': 'Saint-Denis'}}, 463: {'attributes': {'Identifiant station': 16006, 'Nom station': 'Lauriston - Copernic', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 1, 8, 6, 44, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.86926, 2.289918], 'Nom communes équipées': 'Paris'}}, 464: {'attributes': {'Identifiant station': 10020, 'Nom station': "Chabrol - d'Hauteville", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 11, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.877018388, 2.35131796449], 'Nom communes équipées': 'Paris'}}, 465: {'attributes': {'Identifiant station': 21019, 'Nom station': 'Edouard Vaillant - Paul-Adolphe Souriau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 18, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8348458588, 2.24794596434], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 466: {'attributes': {'Identifiant station': 11116, 'Nom station': 'Faubourg St-Antoine - Place de la Nation', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 8, 28, 4, 57, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8487659, 2.3940436], 'Nom communes équipées': 'Paris'}}, 467: {'attributes': {'Identifiant station': 12032, 'Nom station': 'Porte Dorée', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8354337, 2.4077132], 'Nom communes équipées': 'Paris'}}, 468: {'attributes': {'Identifiant station': 16004, 'Nom station': 'Université Paris Dauphine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 2, 11, 10, 14, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.87070784, 2.27486561984], 'Nom communes équipées': 'Paris'}}, 469: {'attributes': {'Identifiant station': 21032, 'Nom station': 'Sèvres-Gallieni', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.837809409, 2.22815193236], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 470: {'attributes': {'Identifiant station': 19038, 'Nom station': 'Lassus - Delouvain', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8755044846, 2.38903273255], 'Nom communes équipées': 'Paris'}}, 471: {'attributes': {'Identifiant station': 6107, 'Nom station': 'Place Alphonse Deville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 18, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8501882627, 2.32759617269], 'Nom communes équipées': 'Paris'}}, 472: {'attributes': {'Identifiant station': 21703, 'Nom station': "Square du Carrefour de l'Insurrection", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8208645598, 2.29111343622], 'Nom communes équipées': 'Vanves'}}, 473: {'attributes': {'Identifiant station': 6103, 'Nom station': 'Mézières - Rennes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 55, 'Nombre bornettes libres': 47, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8502345932, 2.33038198203], 'Nom communes équipées': 'Paris'}}, 474: {'attributes': {'Identifiant station': 21706, 'Nom station': 'Jean Bleuzen - Square du 11 Novembre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 49, 'Nombre bornettes libres': 42, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8232579, 2.2965213], 'Nom communes équipées': 'Vanves'}}, 475: {'attributes': {'Identifiant station': 12001, 'Nom station': 'Bastille - Biscornet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 62, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 33, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8513557796, 2.36922003329], 'Nom communes équipées': 'Paris'}}, 476: {'attributes': {'Identifiant station': 16021, 'Nom station': 'François Ponsard - La Muette', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8583666, 2.2744515], 'Nom communes équipées': 'Paris'}}, 477: {'attributes': {'Identifiant station': 32014, 'Nom station': 'Place du Cornillon - Stade de France', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9254069584, 2.36444268376], 'Nom communes équipées': 'Saint-Denis'}}, 478: {'attributes': {'Identifiant station': 9031, 'Nom station': "Provence - Chaussée d'Antin", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8742844754, 2.33297651156], 'Nom communes équipées': 'Paris'}}, 479: {'attributes': {'Identifiant station': 17124, 'Nom station': 'Porte de Saint-Ouen - Henri Huchard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8987083664, 2.32953608036], 'Nom communes équipées': 'Paris'}}, 480: {'attributes': {'Identifiant station': 6030, 'Nom station': 'Assas - Vaugirard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 30, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 16, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8482741114, 2.32924137264], 'Nom communes équipées': 'Paris'}}, 481: {'attributes': {'Identifiant station': 44012, 'Nom station': 'Génie - Mozart', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7949023812, 2.37341836095], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 482: {'attributes': {'Identifiant station': 18023, 'Nom station': 'Ordener - Poissonniers', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8912138908, 2.35128866622], 'Nom communes équipées': 'Paris'}}, 483: {'attributes': {'Identifiant station': 10165, 'Nom station': "Gare de l'Est - Place du 11 Novembre 1918", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 43, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.876327, 2.3585464], 'Nom communes équipées': 'Paris'}}, 484: {'attributes': {'Identifiant station': 18049, 'Nom station': "Porte d'Aubervilliers", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 55, 'Nombre bornettes libres': 45, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8985966194, 2.36897226423], 'Nom communes équipées': 'Paris'}}, 485: {'attributes': {'Identifiant station': 31016, 'Nom station': 'Lucie Aubrac - Franklin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8622391601, 2.44358628988], 'Nom communes équipées': 'Montreuil'}}, 486: {'attributes': {'Identifiant station': 31703, 'Nom station': 'Sadi Carnot - François Mitterrand', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8753268767, 2.4241617322], 'Nom communes équipées': 'Bagnolet'}}, 487: {'attributes': {'Identifiant station': 51003, 'Nom station': "Hôtel de Ville d'Argenteuil", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.946504, 2.2496396], 'Nom communes équipées': 'Argenteuil'}}, 488: {'attributes': {'Identifiant station': 15031, 'Nom station': 'Théâtre - Lourmel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 53, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 33, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8482070398, 2.28986587375], 'Nom communes équipées': 'Paris'}}, 489: {'attributes': {'Identifiant station': 44102, 'Nom station': 'Verdun - Adrien Damalix', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 24, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 14, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8222334179, 2.42153283209], 'Nom communes équipées': 'Saint-Maurice'}}, 490: {'attributes': {'Identifiant station': 21014, 'Nom station': 'Hameau Fleuri - Marché', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8314488926, 2.2412404418], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 491: {'attributes': {'Identifiant station': 9115, 'Nom station': 'Milton - Manuel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.878249, 2.341181], 'Nom communes équipées': 'Paris'}}, 492: {'attributes': {'Identifiant station': 35014, 'Nom station': 'Charles de Gaulle - Jean Lolive', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8932644351, 2.4126765132], 'Nom communes équipées': 'Pantin'}}, 493: {'attributes': {'Identifiant station': 16123, 'Nom station': 'Mahatma Gandhi', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 15, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.876356, 2.264205], 'Nom communes équipées': 'Paris'}}, 494: {'attributes': {'Identifiant station': 15049, 'Nom station': 'Porte de Versailles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8321538173, 2.2871491313], 'Nom communes équipées': 'Paris'}}, 495: {'attributes': {'Identifiant station': 16032, 'Nom station': "Eglise d'Auteuil", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8474984177, 2.26848387135], 'Nom communes équipées': 'Paris'}}, 496: {'attributes': {'Identifiant station': 11041, 'Nom station': 'Square Saint-Ambroise', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.860934, 2.375841], 'Nom communes équipées': 'Paris'}}, 497: {'attributes': {'Identifiant station': 18113, 'Nom station': "Lepic - Armée d'Orient", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8875364038, 2.3343077302], 'Nom communes équipées': 'Paris'}}, 498: {'attributes': {'Identifiant station': 11006, 'Nom station': 'Ledru-Rollin - Place du Père Chaillet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8564337069, 2.37902058776], 'Nom communes équipées': 'Paris'}}, 499: {'attributes': {'Identifiant station': 21003, 'Nom station': 'Jean Jaurès - Jean Baptiste Clément', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8457626145, 2.23696500063], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 500: {'attributes': {'Identifiant station': 31027, 'Nom station': 'Place de la République.', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 10, 3, 22, 46, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.852988, 2.42467], 'Nom communes équipées': 'Montreuil'}}, 501: {'attributes': {'Identifiant station': 22001, 'Nom station': 'Château - Bineau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8904040286, 2.27010637522], 'Nom communes équipées': 'Neuilly-sur-Seine'}}, 502: {'attributes': {'Identifiant station': 19127, 'Nom station': 'Aubervilliers - Curial', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8908830523, 2.36885793507], 'Nom communes équipées': 'Paris'}}, 503: {'attributes': {'Identifiant station': 17122, 'Nom station': 'Bridaine - Lamandé', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8862074, 2.3206239], 'Nom communes équipées': 'Paris'}}, 504: {'attributes': {'Identifiant station': 16015, 'Nom station': 'Bassano -  Iéna', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 15, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8681445105, 2.29614960099], 'Nom communes équipées': 'Paris'}}, 505: {'attributes': {'Identifiant station': 20027, 'Nom station': 'Porte de Ménilmontant', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 15, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8691543609, 2.40932246879], 'Nom communes équipées': 'Paris'}}, 506: {'attributes': {'Identifiant station': 47004, 'Nom station': 'Stade Auguste Delaune', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8089637868, 2.43452042341], 'Nom communes équipées': 'Maisons-Alfort'}}, 507: {'attributes': {'Identifiant station': 15032, 'Nom station': 'Félix Faure - Lourmel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 58, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 57, 'Vélos mécaniques disponibles': 56, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8386890171, 2.2818081826], 'Nom communes équipées': 'Paris'}}, 508: {'attributes': {'Identifiant station': 16122, 'Nom station': 'Route de la Muette - Porte de Neuilly', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 45, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8798926, 2.2587671], 'Nom communes équipées': 'Paris'}}, 509: {'attributes': {'Identifiant station': 15108, 'Nom station': 'Lourmel - Marie Skobtsov', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.844974, 2.287283], 'Nom communes équipées': 'Paris'}}, 510: {'attributes': {'Identifiant station': 20116, 'Nom station': 'Haies - Réunion', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8538689677, 2.40242616224], 'Nom communes équipées': 'Paris'}}, 511: {'attributes': {'Identifiant station': 19040, 'Nom station': 'Belleville - Télégraphe', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 41, 'Nombre bornettes libres': 41, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8756045, 2.3995549], 'Nom communes équipées': 'Paris'}}, 512: {'attributes': {'Identifiant station': 15002, 'Nom station': 'Maine - Antoine Bourdelle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8430853175, 2.32026394678], 'Nom communes équipées': 'Paris'}}, 513: {'attributes': {'Identifiant station': 6016, 'Nom station': 'Danton - Serpente', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8522973626, 2.34141323715], 'Nom communes équipées': 'Paris'}}, 514: {'attributes': {'Identifiant station': 23007, 'Nom station': 'Président Wilson - Jean Jaures', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8903405405, 2.29206830263], 'Nom communes équipées': 'Levallois-Perret'}}, 515: {'attributes': {'Identifiant station': 17046, 'Nom station': 'Mac-Mahon - Brey', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 14, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8775320841, 2.29451253605], 'Nom communes équipées': 'Paris'}}, 516: {'attributes': {'Identifiant station': 10010, 'Nom station': 'Place Jacques Bonsergent', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8709482186, 2.3612049222], 'Nom communes équipées': 'Paris'}}, 517: {'attributes': {'Identifiant station': 17121, 'Nom station': 'Square des Epinettes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8938390978, 2.32595399022], 'Nom communes équipées': 'Paris'}}, 518: {'attributes': {'Identifiant station': 5121, 'Nom station': 'Square René Viviani - Montebello', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 31, 'Vélos mécaniques disponibles': 26, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8519048914, 2.34784215689], 'Nom communes équipées': 'Paris'}}, 519: {'attributes': {'Identifiant station': 47006, 'Nom station': 'Les Planètes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8099849921, 2.45808530599], 'Nom communes équipées': 'Maisons-Alfort'}}, 520: {'attributes': {'Identifiant station': 16027, 'Nom station': 'Mozart - Jasmin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 62, 'Nombre bornettes libres': 58, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 41, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8528612487, 2.26848357065], 'Nom communes équipées': 'Paris'}}, 521: {'attributes': {'Identifiant station': 18109, 'Nom station': 'Pajol - Riquet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 47, 'Nombre bornettes libres': 45, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8896002813, 2.36285347492], 'Nom communes équipées': 'Paris'}}, 522: {'attributes': {'Identifiant station': 1117, 'Nom station': 'Pont Neuf - Rivoli', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8600898926, 2.34405755997], 'Nom communes équipées': 'Paris'}}, 523: {'attributes': {'Identifiant station': 18029, 'Nom station': 'Emile Blemont - Poteau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 38, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8943782, 2.3415122], 'Nom communes équipées': 'Paris'}}, 524: {'attributes': {'Identifiant station': 11043, 'Nom station': 'Place Pasdeloup', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8626542156, 2.36703250628], 'Nom communes équipées': 'Paris'}}, 525: {'attributes': {'Identifiant station': 20118, 'Nom station': 'Orteaux - Mouraud', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8555864285, 2.40876290776], 'Nom communes équipées': 'Paris'}}, 526: {'attributes': {'Identifiant station': 1120, 'Nom station': 'Lavandieres Sainte Opportune - Rivoli', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 23, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8592797025, 2.34662175179], 'Nom communes équipées': 'Paris'}}, 527: {'attributes': {'Identifiant station': 20023, 'Nom station': 'Pelleport - Belgrand', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 15, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8650008185, 2.40398083638], 'Nom communes équipées': 'Paris'}}, 528: {'attributes': {'Identifiant station': 9017, 'Nom station': 'Lallier - Trudaine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8810924662, 2.34082248062], 'Nom communes équipées': 'Paris'}}, 529: {'attributes': {'Identifiant station': 15035, 'Nom station': 'Place Etienne Pernet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 20, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8424345713, 2.29218196124], 'Nom communes équipées': 'Paris'}}, 530: {'attributes': {'Identifiant station': 32001, 'Nom station': 'Proudhon - Président Wilson', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9064427635, 2.35849052668], 'Nom communes équipées': 'Saint-Denis'}}, 531: {'attributes': {'Identifiant station': 35017, 'Nom station': 'Toffier Decaux - Cartier Bresson', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 15, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 59, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9016507255, 2.40175455809], 'Nom communes équipées': 'Pantin'}}, 532: {'attributes': {'Identifiant station': 42703, 'Nom station': 'Fontainebleau - Edmond Michelet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 44, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8095643684, 2.36269086599], 'Nom communes équipées': 'Le Kremlin-Bicêtre'}}, 533: {'attributes': {'Identifiant station': 9117, 'Nom station': 'Bellefond - Maubeuge', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8782553976, 2.34526421875], 'Nom communes équipées': 'Paris'}}, 534: {'attributes': {'Identifiant station': 13046, 'Nom station': 'Eugène Oudiné - Dessous Des Berges', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 44, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 30, 'Vélos mécaniques disponibles': 30, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8256423, 2.3750226], 'Nom communes équipées': 'Paris'}}, 535: {'attributes': {'Identifiant station': 35003, 'Nom station': 'Gabrielle Josserand - Edouard Vaillant', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9027156626, 2.39332705736], 'Nom communes équipées': 'Pantin'}}, 536: {'attributes': {'Identifiant station': 22102, 'Nom station': 'Mont Valérien - Michel Salles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8561915, 2.2152966], 'Nom communes équipées': 'Saint-Cloud'}}, 537: {'attributes': {'Identifiant station': 47005, 'Nom station': 'Ecole Vétérinaire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 33, 'Vélos mécaniques disponibles': 31, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8148885982, 2.42243621578], 'Nom communes équipées': 'Maisons-Alfort'}}, 538: {'attributes': {'Identifiant station': 20030, 'Nom station': 'Ménilmontant - Père Lachaise', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 56, 'Nombre bornettes libres': 50, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8632872915, 2.3869940266], 'Nom communes équipées': 'Paris'}}, 539: {'attributes': {'Identifiant station': 15070, 'Nom station': 'Balard - Montagne de la Fage', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 30, 'Vélos mécaniques disponibles': 25, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8385074033, 2.27828711271], 'Nom communes équipées': 'Paris'}}, 540: {'attributes': {'Identifiant station': 4110, 'Nom station': 'Neuve Saint-Pierre - Beautreillis', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8535535144, 2.36391227692], 'Nom communes équipées': 'Paris'}}, 541: {'attributes': {'Identifiant station': 12107, 'Nom station': 'Diderot - Pierre Bourdan', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8476486208, 2.39025827497], 'Nom communes équipées': 'Paris'}}, 542: {'attributes': {'Identifiant station': 18202, 'Nom station': 'Diane Arbus', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 35, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8988007172, 2.35240224749], 'Nom communes équipées': 'Paris'}}, 543: {'attributes': {'Identifiant station': 14112, 'Nom station': 'Méchain - Faubourg Saint-Jacques', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.835867721, 2.33798034198], 'Nom communes équipées': 'Paris'}}, 544: {'attributes': {'Identifiant station': 14030, 'Nom station': 'Pernety - Raymond Losserand', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.834254628, 2.31760933939], 'Nom communes équipées': 'Paris'}}, 545: {'attributes': {'Identifiant station': 9026, 'Nom station': 'Jean-Baptiste Pigalle - La Bruyere', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8793982645, 2.33370917334], 'Nom communes équipées': 'Paris'}}, 546: {'attributes': {'Identifiant station': 7020, 'Nom station': 'Champ de Mars - Cler', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 41, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 19, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8566915501, 2.30664677918], 'Nom communes équipées': 'Paris'}}, 547: {'attributes': {'Identifiant station': 21001, 'Nom station': 'Transvaal - Charles de Gaulle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8482804036, 2.23734201209], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 548: {'attributes': {'Identifiant station': 13103, 'Nom station': 'Hôpital Pitié Salpêtrière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 37, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8377016, 2.3604946], 'Nom communes équipées': 'Paris'}}, 549: {'attributes': {'Identifiant station': 18033, 'Nom station': 'Belliard - Damrémont', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8966923878, 2.33802345039], 'Nom communes équipées': 'Paris'}}, 550: {'attributes': {'Identifiant station': 6017, 'Nom station': 'Sénat - Condé', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 31, 'Vélos mécaniques disponibles': 31, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8495422779, 2.3379394412], 'Nom communes équipées': 'Paris'}}, 551: {'attributes': {'Identifiant station': 11010, 'Nom station': 'Boulets - Faubourg Saint-Antoine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 37, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8492592621, 2.39175461419], 'Nom communes équipées': 'Paris'}}, 552: {'attributes': {'Identifiant station': 10036, 'Nom station': 'Aqueduc - Villette', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8838860639, 2.36719880331], 'Nom communes équipées': 'Paris'}}, 553: {'attributes': {'Identifiant station': 21310, 'Nom station': 'Kléber - Gouverneur Général Eboué', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 20, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8268858431, 2.27234132588], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 554: {'attributes': {'Identifiant station': 14028, 'Nom station': 'Alésia - Suisses', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 47, 'Nombre bornettes libres': 47, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8316825207, 2.3149869226], 'Nom communes équipées': 'Paris'}}, 555: {'attributes': {'Identifiant station': 20016, 'Nom station': 'Pyrénées - Maraîchers', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.857140014, 2.40435093641], 'Nom communes équipées': 'Paris'}}, 556: {'attributes': {'Identifiant station': 20137, 'Nom station': 'Borrégo - Gambetta', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8733049705, 2.40141458809], 'Nom communes équipées': 'Paris'}}, 557: {'attributes': {'Identifiant station': 32015, 'Nom station': 'Gare RER de Saint-Denis', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 33, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9355432425, 2.34714880586], 'Nom communes équipées': 'Saint-Denis'}}, 558: {'attributes': {'Identifiant station': 12161, 'Nom station': 'Diderot - Reuilly', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 44, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 40, 'Vélos mécaniques disponibles': 30, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8472084561, 2.38612733781], 'Nom communes équipées': 'Paris'}}, 559: {'attributes': {'Identifiant station': 16043, 'Nom station': 'Place de Passy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 15, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8578029171, 2.27750054694], 'Nom communes équipées': 'Paris'}}, 560: {'attributes': {'Identifiant station': 7013, 'Nom station': 'Duroc - Place de Breteuil', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8474606411, 2.31261402369], 'Nom communes équipées': 'Paris'}}, 561: {'attributes': {'Identifiant station': 18043, 'Nom station': 'Clichy - Place Blanche', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8836010883, 2.33322378248], 'Nom communes équipées': 'Paris'}}, 562: {'attributes': {'Identifiant station': 2013, 'Nom station': 'Favart - Italiens', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8714463539, 2.33829010585], 'Nom communes équipées': 'Paris'}}, 563: {'attributes': {'Identifiant station': 18008, 'Nom station': 'Square Léon', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8866739679, 2.35320556909], 'Nom communes équipées': 'Paris'}}, 564: {'attributes': {'Identifiant station': 12041, 'Nom station': 'Bois de Vincennes.', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 60, 'Nombre bornettes libres': 40, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8338064, 2.4132221], 'Nom communes équipées': 'Saint-Mandé'}}, 565: {'attributes': {'Identifiant station': 32004, 'Nom station': 'Wilson - Landy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 18, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9151672622, 2.35763322562], 'Nom communes équipées': 'Saint-Denis'}}, 566: {'attributes': {'Identifiant station': 15059, 'Nom station': 'Parc André Citroën', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 65, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 59, 'Vélos mécaniques disponibles': 56, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8406399443, 2.27806180716], 'Nom communes équipées': 'Paris'}}, 567: {'attributes': {'Identifiant station': 16029, 'Nom station': 'Maison de Radio - France', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 47, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 39, 'Vélos mécaniques disponibles': 39, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8513288643, 2.27778553963], 'Nom communes équipées': 'Paris'}}, 568: {'attributes': {'Identifiant station': 8038, 'Nom station': 'François 1er - Montaigne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 13, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.86688765, 2.30667557572], 'Nom communes équipées': 'Paris'}}, 569: {'attributes': {'Identifiant station': 18017, 'Nom station': 'Caulaincourt - Place Constantin Pecqueur', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 44, 'Nombre bornettes libres': 41, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.889677905, 2.33817871443], 'Nom communes équipées': 'Paris'}}, 570: {'attributes': {'Identifiant station': 18138, 'Nom station': 'Ney - Charles Hermite', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8987480398, 2.3645754531], 'Nom communes équipées': 'Paris'}}, 571: {'attributes': {'Identifiant station': 8019, 'Nom station': 'Naples - Rome', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8799607008, 2.32135869563], 'Nom communes équipées': 'Paris'}}, 572: {'attributes': {'Identifiant station': 32603, 'Nom station': 'Calmette.', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.881049251, 2.42499321699], 'Nom communes équipées': 'Les Lilas'}}, 573: {'attributes': {'Identifiant station': 8001, 'Nom station': 'Petit Palais', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 35, 'Vélos mécaniques disponibles': 27, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8667692, 2.3157655], 'Nom communes équipées': 'Paris'}}, 574: {'attributes': {'Identifiant station': 4011, 'Nom station': 'Hôtel de Ville - Pont-Marie', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8538996455, 2.35694725066], 'Nom communes équipées': 'Paris'}}, 575: {'attributes': {'Identifiant station': 7008, 'Nom station': 'Villersexel - Saint-Germain', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8582970012, 2.32379347086], 'Nom communes équipées': 'Paris'}}, 576: {'attributes': {'Identifiant station': 45001, 'Nom station': 'Hôtel de Ville de Champigny sur Marne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 18, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8138106284, 2.50967592001], 'Nom communes équipées': 'Champigny-sur-Marne'}}, 577: {'attributes': {'Identifiant station': 20201, 'Nom station': 'Eugène Reisz - Davout', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8563187463, 2.41039697081], 'Nom communes équipées': 'Paris'}}, 578: {'attributes': {'Identifiant station': 10163, 'Nom station': 'Gare du Nord - Demarquay', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.881154, 2.358411], 'Nom communes équipées': 'Paris'}}, 579: {'attributes': {'Identifiant station': 16112, 'Nom station': 'Chernoviz - Raynouard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 33, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8569864146, 2.28217321814], 'Nom communes équipées': 'Paris'}}, 580: {'attributes': {'Identifiant station': 2001, 'Nom station': 'Greneta - Sebastopol', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8652423332, 2.35166698694], 'Nom communes équipées': 'Paris'}}, 581: {'attributes': {'Identifiant station': 12004, 'Nom station': 'Traversière - Ledru-Rollin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8507358241, 2.37587011171], 'Nom communes équipées': 'Paris'}}, 582: {'attributes': {'Identifiant station': 7101, 'Nom station': 'Raspail - Chomel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 49, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 24, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8522331643, 2.32665807009], 'Nom communes équipées': 'Paris'}}, 583: {'attributes': {'Identifiant station': 15003, 'Nom station': 'Gare Montparnasse - Vaugirard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 66, 'Nombre bornettes libres': 60, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8418071799, 2.3195100269], 'Nom communes équipées': 'Paris'}}, 584: {'attributes': {'Identifiant station': 18137, 'Nom station': 'Poissonniers - Nord', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8925112135, 2.35222287476], 'Nom communes équipées': 'Paris'}}, 585: {'attributes': {'Identifiant station': 6032, 'Nom station': 'Sabot - Rennes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 5, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8525751129, 2.33155276626], 'Nom communes équipées': 'Paris'}}, 586: {'attributes': {'Identifiant station': 17116, 'Nom station': "Reims - Porte d'Asnières", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8918344482, 2.30014465995], 'Nom communes équipées': 'Paris'}}, 587: {'attributes': {'Identifiant station': 17022, 'Nom station': 'Pereire - Place de Wagram', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 41, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8870706076, 2.30411112453], 'Nom communes équipées': 'Paris'}}, 588: {'attributes': {'Identifiant station': 14116, 'Nom station': 'Romain Rolland', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8202152297, 2.32341442344], 'Nom communes équipées': 'Montrouge'}}, 589: {'attributes': {'Identifiant station': 10114, 'Nom station': 'Granges aux Belles - Hôpital Saint-Louis', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 60, 'Nombre bornettes libres': 59, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8747949948, 2.36666943698], 'Nom communes équipées': 'Paris'}}, 590: {'attributes': {'Identifiant station': 18001, 'Nom station': 'Damrémont - Caulaincourt', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8871450908, 2.33258809894], 'Nom communes équipées': 'Paris'}}, 591: {'attributes': {'Identifiant station': 15058, 'Nom station': 'Place du Moulin de Javel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 31, 'Vélos mécaniques disponibles': 23, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8402167092, 2.27160606533], 'Nom communes équipées': 'Paris'}}, 592: {'attributes': {'Identifiant station': 5019, 'Nom station': 'Poissy - Saint-Germain', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8496964943, 2.35296852887], 'Nom communes équipées': 'Paris'}}, 593: {'attributes': {'Identifiant station': 13122, 'Nom station': 'Parc de Choisy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8281562516, 2.3585135329], 'Nom communes équipées': 'Paris'}}, 594: {'attributes': {'Identifiant station': 18019, 'Nom station': 'Square Raymond Souplex', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 52, 'Nombre bornettes libres': 51, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8918372148, 2.33542610317], 'Nom communes équipées': 'Paris'}}, 595: {'attributes': {'Identifiant station': 19041, 'Nom station': 'Belleville - Pyrénées', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 39, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8741908, 2.3860039], 'Nom communes équipées': 'Paris'}}, 596: {'attributes': {'Identifiant station': 15063, 'Nom station': 'Convention - Saint-Charles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8431561111, 2.28310905397], 'Nom communes équipées': 'Paris'}}, 597: {'attributes': {'Identifiant station': 21114, 'Nom station': 'Gustave Eiffel - Parc des Impressionnistes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9039061965, 2.2946908325], 'Nom communes équipées': 'Clichy'}}, 598: {'attributes': {'Identifiant station': 11003, 'Nom station': 'Keller - La Roquette', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8555508, 2.3752778], 'Nom communes équipées': 'Paris'}}, 599: {'attributes': {'Identifiant station': 15107, 'Nom station': 'Palais des Sports', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 19, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8334685174, 2.28587206453], 'Nom communes équipées': 'Paris'}}, 600: {'attributes': {'Identifiant station': 11034, 'Nom station': 'Parmentier - Fontaine au Roi', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 48, 'Nombre bornettes libres': 39, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8678767055, 2.37270597867], 'Nom communes équipées': 'Paris'}}, 601: {'attributes': {'Identifiant station': 10033, 'Nom station': 'Gare du Nord - Saint-Vincent de Paul', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8809557416, 2.35243051484], 'Nom communes équipées': 'Paris'}}, 602: {'attributes': {'Identifiant station': 41206, 'Nom station': 'Place des Larris', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8568363, 2.4780236], 'Nom communes équipées': 'Fontenay-sous-Bois'}}, 603: {'attributes': {'Identifiant station': 31704, 'Nom station': 'Jeanne Hornet - Girardot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8738026814, 2.42782026529], 'Nom communes équipées': 'Bagnolet'}}, 604: {'attributes': {'Identifiant station': 11019, 'Nom station': 'Charonne - Philippe-Auguste', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.855603162, 2.39054639062], 'Nom communes équipées': 'Paris'}}, 605: {'attributes': {'Identifiant station': 21024, 'Nom station': 'Billancourt - Général Leclerc', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8322649, 2.2380413], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 606: {'attributes': {'Identifiant station': 8015, 'Nom station': 'Roquépine - Malesherbes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 32, 'Vélos mécaniques disponibles': 27, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8734507339, 2.32040449977], 'Nom communes équipées': 'Paris'}}, 607: {'attributes': {'Identifiant station': 19023, 'Nom station': 'Manin - Simon Bolivar', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8764996541, 2.3794073626], 'Nom communes équipées': 'Paris'}}, 608: {'attributes': {'Identifiant station': 16016, 'Nom station': 'Paul Doumer - La Tour', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8601161096, 2.28084048191], 'Nom communes équipées': 'Paris'}}, 609: {'attributes': {'Identifiant station': 21311, 'Nom station': 'Parvis Corentin Celton', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 55, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 53, 'Vélos mécaniques disponibles': 49, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8274725252, 2.278431952], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 610: {'attributes': {'Identifiant station': 21108, 'Nom station': 'Mairie de Clichy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 51, 'Nombre bornettes libres': 41, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9036536347, 2.30593532324], 'Nom communes équipées': 'Clichy'}}, 611: {'attributes': {'Identifiant station': 8056, 'Nom station': 'Beaujon - Wagram', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.875352456, 2.29644164443], 'Nom communes équipées': 'Paris'}}, 612: {'attributes': {'Identifiant station': 6014, 'Nom station': 'Pont de Lodi - Dauphine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8552968986, 2.33994730757], 'Nom communes équipées': 'Paris'}}, 613: {'attributes': {'Identifiant station': 7016, 'Nom station': 'Surcouf - Université', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 58, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 47, 'Vélos mécaniques disponibles': 47, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8612730163, 2.30945355091], 'Nom communes équipées': 'Paris'}}, 614: {'attributes': {'Identifiant station': 47002, 'Nom station': 'Les Juilliottes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8043072, 2.4437362], 'Nom communes équipées': 'Maisons-Alfort'}}, 615: {'attributes': {'Identifiant station': 7113, 'Nom station': "Berges de Seine - Musée d'Orsay", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 1, 27, 7, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8627885937, 2.30645935982], 'Nom communes équipées': 'Paris'}}, 616: {'attributes': {'Identifiant station': 1022, 'Nom station': 'Danielle Casanova - Place Vendôme', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 27, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8682664982, 2.33048390597], 'Nom communes équipées': 'Paris'}}, 617: {'attributes': {'Identifiant station': 14006, 'Nom station': 'Saint-Jacques - Tombe Issoire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 2, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8332063305, 2.33691081405], 'Nom communes équipées': 'Paris'}}, 618: {'attributes': {'Identifiant station': 11046, 'Nom station': 'Place de la République - Voltaire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8668309689, 2.36548103392], 'Nom communes équipées': 'Paris'}}, 619: {'attributes': {'Identifiant station': 13041, 'Nom station': "Porte d'Ivry", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 44, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8217687476, 2.36887067556], 'Nom communes équipées': 'Paris'}}, 620: {'attributes': {'Identifiant station': 5030, 'Nom station': 'Sorbonne - Ecoles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 45, 'Vélos mécaniques disponibles': 31, 'Vélos électriques disponibles': 14, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8496865662, 2.34349966049], 'Nom communes équipées': 'Paris'}}, 621: {'attributes': {'Identifiant station': 9118, 'Nom station': 'Faubourg Poissonniere - Delta', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8825558076, 2.34974820167], 'Nom communes équipées': 'Paris'}}, 622: {'attributes': {'Identifiant station': 11107, 'Nom station': 'Faidherbe - Dahomey', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 44, 'Nombre bornettes libres': 37, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8513583298, 2.3838231348], 'Nom communes équipées': 'Paris'}}, 623: {'attributes': {'Identifiant station': 4013, 'Nom station': 'Vieille du Temple - Francs Bourgeois', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8582564134, 2.35821794719], 'Nom communes équipées': 'Paris'}}, 624: {'attributes': {'Identifiant station': 16041, 'Nom station': 'Versailles - Claude Terrasse', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 24, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8402193381, 2.26383419115], 'Nom communes équipées': 'Paris'}}, 625: {'attributes': {'Identifiant station': 31023, 'Nom station': 'Place François Mitterand', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 33, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8649278202, 2.4429583177], 'Nom communes équipées': 'Montreuil'}}, 626: {'attributes': {'Identifiant station': 4109, 'Nom station': 'Mornay - Sully', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 15, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8497039955, 2.36446078867], 'Nom communes équipées': 'Paris'}}, 627: {'attributes': {'Identifiant station': 18042, 'Nom station': 'Place Pigalle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8827555, 2.3361566], 'Nom communes équipées': 'Paris'}}, 628: {'attributes': {'Identifiant station': 3101, 'Nom station': 'Porte Saint-Martin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8686017211, 2.3557010293], 'Nom communes équipées': 'Paris'}}, 629: {'attributes': {'Identifiant station': 20019, 'Nom station': 'Place du Maquis du Vercors', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8773215795, 2.40837827325], 'Nom communes équipées': 'Paris'}}, 630: {'attributes': {'Identifiant station': 11048, 'Nom station': 'Place Mireille Havet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.850533, 2.383756], 'Nom communes équipées': 'Paris'}}, 631: {'attributes': {'Identifiant station': 31013, 'Nom station': 'Centenaire - Sorins', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 42, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8585352336, 2.42735087872], 'Nom communes équipées': 'Montreuil'}}, 632: {'attributes': {'Identifiant station': 6031, 'Nom station': 'Pierre Sarrazin - Saint-Michel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 32, 'Vélos mécaniques disponibles': 27, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8509081464, 2.34213072807], 'Nom communes équipées': 'Paris'}}, 633: {'attributes': {'Identifiant station': 32018, 'Nom station': 'Franciade - Gabriel Péri', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9326149933, 2.35595449805], 'Nom communes équipées': 'Saint-Denis'}}, 634: {'attributes': {'Identifiant station': 8105, 'Nom station': 'François 1er - Lincoln', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 16, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8695179793, 2.30242621884], 'Nom communes équipées': 'Paris'}}, 635: {'attributes': {'Identifiant station': 17016, 'Nom station': 'Brochant - Square des Batignolles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8881869858, 2.31686114315], 'Nom communes équipées': 'Paris'}}, 636: {'attributes': {'Identifiant station': 2101, 'Nom station': 'Bachaumont - Montmartre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 32, 'Vélos mécaniques disponibles': 26, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8661739508, 2.34497755766], 'Nom communes équipées': 'Paris'}}, 637: {'attributes': {'Identifiant station': 13113, 'Nom station': 'Vistule - Choisy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 33, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8236554366, 2.36157238483], 'Nom communes équipées': 'Paris'}}, 638: {'attributes': {'Identifiant station': 14007, 'Nom station': 'Ferrus - Saint-Jacques', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 33, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8313878, 2.3408563], 'Nom communes équipées': 'Paris'}}, 639: {'attributes': {'Identifiant station': 12108, 'Nom station': 'Bercy.', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 66, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 62, 'Vélos mécaniques disponibles': 45, 'Vélos électriques disponibles': 17, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.839966915, 2.3788625747], 'Nom communes équipées': 'Paris'}}, 640: {'attributes': {'Identifiant station': 10026, 'Nom station': "Gare de l'Est - Chateau Landon", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 59, 'Nombre bornettes libres': 56, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.879305, 2.362424], 'Nom communes équipées': 'Paris'}}, 641: {'attributes': {'Identifiant station': 7110, 'Nom station': "Quai Anatole France - Musée d'Orsay", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 12, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 1, 22, 21, 1, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8612117406, 2.32424609363], 'Nom communes équipées': 'Paris'}}, 642: {'attributes': {'Identifiant station': 9015, 'Nom station': 'Lamartine - Faubourg Montmartre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8766139829, 2.33976569027], 'Nom communes équipées': 'Paris'}}, 643: {'attributes': {'Identifiant station': 9012, 'Nom station': 'Rougemont - Poissonnière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8716115694, 2.34587308019], 'Nom communes équipées': 'Paris'}}, 644: {'attributes': {'Identifiant station': 19021, 'Nom station': 'Mairie du 19ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8825626425, 2.38126248121], 'Nom communes équipées': 'Paris'}}, 645: {'attributes': {'Identifiant station': 20026, 'Nom station': 'Gambetta - Hôpital Tenon', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 35, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.867925154, 2.40097662355], 'Nom communes équipées': 'Paris'}}, 646: {'attributes': {'Identifiant station': 14127, 'Nom station': 'Odessa - Départ', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 3, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8428855901, 2.32442814857], 'Nom communes équipées': 'Paris'}}, 647: {'attributes': {'Identifiant station': 15020, 'Nom station': 'Mairie du 15ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8417762, 2.2986539], 'Nom communes équipées': 'Paris'}}, 648: {'attributes': {'Identifiant station': 23105, 'Nom station': 'Sèvres Rive Gauche', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 43, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8214432183, 2.21507853069], 'Nom communes équipées': 'Sèvres'}}, 649: {'attributes': {'Identifiant station': 19031, 'Nom station': 'Manin - Crimée', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8831217744, 2.38636706024], 'Nom communes équipées': 'Paris'}}, 650: {'attributes': {'Identifiant station': 19030, 'Nom station': 'Melun - Jean Jaurès', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8837796723, 2.37647742033], 'Nom communes équipées': 'Paris'}}, 651: {'attributes': {'Identifiant station': 13114, 'Nom station': 'Nationale - Marcel Duchamp', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8246374934, 2.36780148], 'Nom communes équipées': 'Paris'}}, 652: {'attributes': {'Identifiant station': 21004, 'Nom station': 'Jean Jaurès - Reine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8400962256, 2.2398725152], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 653: {'attributes': {'Identifiant station': 15061, 'Nom station': 'Square Desnouettes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8347683793, 2.28414229739], 'Nom communes équipées': 'Paris'}}, 654: {'attributes': {'Identifiant station': 16031, 'Nom station': 'Vignes - Boulainvilliers', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.856399484, 2.27524045855], 'Nom communes équipées': 'Paris'}}, 655: {'attributes': {'Identifiant station': 20017, 'Nom station': 'Saint-Blaise - Mouraud', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.856961776, 2.40869477391], 'Nom communes équipées': 'Paris'}}, 656: {'attributes': {'Identifiant station': 12127, 'Nom station': 'Tremblay - Lac des Minimes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 48, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 39, 'Vélos mécaniques disponibles': 31, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8341312615, 2.45475165546], 'Nom communes équipées': 'Paris'}}, 657: {'attributes': {'Identifiant station': 23008, 'Nom station': 'Hôtel de Ville de Levallois Perret', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.89314452, 2.28888183832], 'Nom communes équipées': 'Levallois-Perret'}}, 658: {'attributes': {'Identifiant station': 12153, 'Nom station': 'Terroirs de France - Lheureux', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 55, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 54, 'Vélos mécaniques disponibles': 41, 'Vélos électriques disponibles': 13, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8321202707, 2.38861810416], 'Nom communes équipées': 'Paris'}}, 659: {'attributes': {'Identifiant station': 1016, 'Nom station': 'Thérèse - Opéra', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 12, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8664835145, 2.33443342122], 'Nom communes équipées': 'Paris'}}, 660: {'attributes': {'Identifiant station': 5028, 'Nom station': 'Censier - Jardin des Plantes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8410717816, 2.35547538847], 'Nom communes équipées': 'Paris'}}, 661: {'attributes': {'Identifiant station': 7026, 'Nom station': 'Place de Finlande', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.862408336, 2.3113624379], 'Nom communes équipées': 'Paris'}}, 662: {'attributes': {'Identifiant station': 17005, 'Nom station': 'Brochant - Clichy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 53, 'Nombre bornettes libres': 47, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8903299891, 2.31973055993], 'Nom communes équipées': 'Paris'}}, 663: {'attributes': {'Identifiant station': 31018, 'Nom station': 'Gambetta - Michelet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8527379238, 2.42996603251], 'Nom communes équipées': 'Montreuil'}}, 664: {'attributes': {'Identifiant station': 8032, 'Nom station': 'Matignon - Faubourg Saint-Honoré', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 36, 'Vélos mécaniques disponibles': 29, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8714785912, 2.31397792697], 'Nom communes équipées': 'Paris'}}, 665: {'attributes': {'Identifiant station': 33005, 'Nom station': 'République - Henri Barbusse', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9066155363, 2.38943248987], 'Nom communes équipées': 'Aubervilliers'}}, 666: {'attributes': {'Identifiant station': 14016, 'Nom station': 'René Coty - Parc Montsouris', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 55, 'Nombre bornettes libres': 55, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.824791345, 2.33612492681], 'Nom communes équipées': 'Paris'}}, 667: {'attributes': {'Identifiant station': 2110, 'Nom station': 'Jeûneurs - Mulhouse', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.868997, 2.346925], 'Nom communes équipées': 'Paris'}}, 668: {'attributes': {'Identifiant station': 11044, 'Nom station': 'Square du Bataclan', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8633762, 2.3711327], 'Nom communes équipées': 'Paris'}}, 669: {'attributes': {'Identifiant station': 13019, 'Nom station': 'Fernand Braudel - Vincent Auriol', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 32, 'Vélos mécaniques disponibles': 22, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8363785232, 2.3723404482], 'Nom communes équipées': 'Paris'}}, 670: {'attributes': {'Identifiant station': 18142, 'Nom station': 'Jessaint - Marx Dormoy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8848454311, 2.35893040895], 'Nom communes équipées': 'Paris'}}, 671: {'attributes': {'Identifiant station': 8049, 'Nom station': 'Georges V - François 1er', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.87034, 2.30082], 'Nom communes équipées': 'Paris'}}, 672: {'attributes': {'Identifiant station': 21017, 'Nom station': 'Place Marcel Sembat', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 56, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 39, 'Vélos mécaniques disponibles': 31, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8332886503, 2.24417477846], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 673: {'attributes': {'Identifiant station': 9113, 'Nom station': 'Bleue - Trévise', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8757890656, 2.34728962183], 'Nom communes équipées': 'Paris'}}, 674: {'attributes': {'Identifiant station': 42011, 'Nom station': 'Quai Marcel Boyer - Victor Hugo', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8225563667, 2.39203959703], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 675: {'attributes': {'Identifiant station': 20103, 'Nom station': 'Le Vau - Maurice Bertaux', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8680343904, 2.41103061245], 'Nom communes équipées': 'Paris'}}, 676: {'attributes': {'Identifiant station': 15009, 'Nom station': 'Suffren - Pérignon', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 30, 'Vélos mécaniques disponibles': 27, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8471756149, 2.3070521476], 'Nom communes équipées': 'Paris'}}, 677: {'attributes': {'Identifiant station': 2014, 'Nom station': 'Daunou - Louis le Grand', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.869113, 2.332515], 'Nom communes équipées': 'Paris'}}, 678: {'attributes': {'Identifiant station': 4107, 'Nom station': 'Beaumarchais - Pas de la Mule', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.855580507, 2.36843336734], 'Nom communes équipées': 'Paris'}}, 679: {'attributes': {'Identifiant station': 23302, 'Nom station': 'Parc de Sceaux - Centre-Ville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7769529826, 2.29572147131], 'Nom communes équipées': 'Sceaux'}}, 680: {'attributes': {'Identifiant station': 9013, 'Nom station': 'Mairie du 9ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8732671376, 2.34075344805], 'Nom communes équipées': 'Paris'}}, 681: {'attributes': {'Identifiant station': 21952, 'Nom station': 'Place Tony de Graaff', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 5, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8181645, 2.2458109], 'Nom communes équipées': 'Meudon'}}, 682: {'attributes': {'Identifiant station': 31017, 'Nom station': 'Jean Moulin - Paul Doumer', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8622179855, 2.45533436537], 'Nom communes équipées': 'Montreuil'}}, 683: {'attributes': {'Identifiant station': 12154, 'Nom station': 'Square Georges Melies', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8407023921, 2.40966774523], 'Nom communes équipées': 'Paris'}}, 684: {'attributes': {'Identifiant station': 13047, 'Nom station': 'Boutroux - Porte de Vitry', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 49, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 30, 'Vélos mécaniques disponibles': 29, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.822632, 2.377738], 'Nom communes équipées': 'Paris'}}, 685: {'attributes': {'Identifiant station': 10164, 'Nom station': 'Claude Vellefaux - Sambre et Meuse', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 59, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.875022, 2.370478], 'Nom communes équipées': 'Paris'}}, 686: {'attributes': {'Identifiant station': 13117, 'Nom station': 'Stade Georges Carpentier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 37, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8204824384, 2.36683219671], 'Nom communes équipées': 'Paris'}}, 687: {'attributes': {'Identifiant station': 8029, 'Nom station': 'Place du Canada', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 32, 'Vélos mécaniques disponibles': 23, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8653462694, 2.31019378796], 'Nom communes équipées': 'Paris'}}, 688: {'attributes': {'Identifiant station': 92005, 'Nom station': 'Gare de Nanterre Ville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8950295, 2.1957932], 'Nom communes équipées': 'Nanterre'}}, 689: {'attributes': {'Identifiant station': 17104, 'Nom station': 'Jardin Marguerite Long', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 18, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8816985774, 2.28365756571], 'Nom communes équipées': 'Paris'}}, 690: {'attributes': {'Identifiant station': 22103, 'Nom station': 'Senard - Charles Blum', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8573158248, 2.22109168768], 'Nom communes équipées': 'Saint-Cloud'}}, 691: {'attributes': {'Identifiant station': 22002, 'Nom station': 'Argenson - Château', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8885593147, 2.26420015097], 'Nom communes équipées': 'Neuilly-sur-Seine'}}, 692: {'attributes': {'Identifiant station': 21120, 'Nom station': 'Docteur Calmette - Martre', 'Station en fonctionnement': 'NON', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 7, 22, 18, 48, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8989693585, 2.30865132524], 'Nom communes équipées': 'Clichy'}}, 693: {'attributes': {'Identifiant station': 35008, 'Nom station': "Place de l'Eglise - Pantin", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8928094615, 2.41243511438], 'Nom communes équipées': 'Pantin'}}, 694: {'attributes': {'Identifiant station': 14128, 'Nom station': 'Gergovie - Vercingétorix', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8345341388, 2.31315454743], 'Nom communes équipées': 'Paris'}}, 695: {'attributes': {'Identifiant station': 13022, 'Nom station': 'Butte aux Cailles - Esperance', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 35, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8275877419, 2.34921511263], 'Nom communes équipées': 'Paris'}}, 696: {'attributes': {'Identifiant station': 12125, 'Nom station': 'Pyramide - Parc Floral', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 37, 'Vélos mécaniques disponibles': 32, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.837442, 2.440452], 'Nom communes équipées': 'Paris'}}, 697: {'attributes': {'Identifiant station': 20045, 'Nom station': 'Louis Ganne - Davout', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8633872069, 2.409725748], 'Nom communes équipées': 'Paris'}}, 698: {'attributes': {'Identifiant station': 31101, 'Nom station': 'Jules Guesde - Laennec', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 14, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.876289839, 2.50069420785], 'Nom communes équipées': 'Rosny-sous-Bois'}}, 699: {'attributes': {'Identifiant station': 11110, 'Nom station': 'Roquette - Folie Regnault', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 52, 'Nombre bornettes libres': 46, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8592469592, 2.38655362358], 'Nom communes équipées': 'Paris'}}, 700: {'attributes': {'Identifiant station': 12129, 'Nom station': 'Bois de Vincennes - Porte Jaune', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8387512467, 2.46054891497], 'Nom communes équipées': 'Paris'}}, 701: {'attributes': {'Identifiant station': 22401, 'Nom station': 'Charles de Gaulle - Plateau de Vanves', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8223308897, 2.29857180341], 'Nom communes équipées': 'Malakoff'}}, 702: {'attributes': {'Identifiant station': 17001, 'Nom station': 'Guy Môquet - Saint-Ouen', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 53, 'Nombre bornettes libres': 48, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.892777719, 2.32703357935], 'Nom communes équipées': 'Paris'}}, 703: {'attributes': {'Identifiant station': 12033, 'Nom station': 'François Truffaut - Saint Emilion', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 66, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 57, 'Vélos mécaniques disponibles': 54, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8337235434, 2.38660190327], 'Nom communes équipées': 'Paris'}}, 704: {'attributes': {'Identifiant station': 10039, 'Nom station': 'Sambre et Meuse - Villette', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8744056662, 2.373807238], 'Nom communes équipées': 'Paris'}}, 705: {'attributes': {'Identifiant station': 12120, 'Nom station': 'Lac Daumesnil - Saint-Maurice', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 47, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8277455574, 2.41836547852], 'Nom communes équipées': 'Paris'}}, 706: {'attributes': {'Identifiant station': 8011, 'Nom station': 'Clapeyron - Moscou', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 15, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8807881951, 2.32481505722], 'Nom communes équipées': 'Paris'}}, 707: {'attributes': {'Identifiant station': 22012, 'Nom station': 'Madeleine Michelis - Roule', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 24, 'Vélos mécaniques disponibles': 18, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.882604313, 2.27736342698], 'Nom communes équipées': 'Neuilly-sur-Seine'}}, 708: {'attributes': {'Identifiant station': 13021, 'Nom station': 'Boussingault - Tolbiac', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8260351661, 2.342176808], 'Nom communes équipées': 'Paris'}}, 709: {'attributes': {'Identifiant station': 18014, 'Nom station': 'Léon - Doudeauville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 13, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8883242185, 2.35357337264], 'Nom communes équipées': 'Paris'}}, 710: {'attributes': {'Identifiant station': 13020, 'Nom station': 'Paul Klee - Place Augusta Holmes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 38, 'Vélos mécaniques disponibles': 20, 'Vélos électriques disponibles': 18, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8389464174, 2.3704235664], 'Nom communes équipées': 'Paris'}}, 711: {'attributes': {'Identifiant station': 18035, 'Nom station': 'René Binet - Porte de Montmartre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8990510995, 2.3365534097], 'Nom communes équipées': 'Paris'}}, 712: {'attributes': {'Identifiant station': 11031, 'Nom station': 'Jacquard - Ternaux', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8646605, 2.373162], 'Nom communes équipées': 'Paris'}}, 713: {'attributes': {'Identifiant station': 13110, 'Nom station': 'Moulin de la Pointe', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8209294605, 2.35631793737], 'Nom communes équipées': 'Paris'}}, 714: {'attributes': {'Identifiant station': 19124, 'Nom station': 'Alexander Fleming - Belvédère', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 40, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.881790642, 2.40305993776], 'Nom communes équipées': 'Paris'}}, 715: {'attributes': {'Identifiant station': 11012, 'Nom station': 'Montreuil - Voltaire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8505070118, 2.39307555827], 'Nom communes équipées': 'Paris'}}, 716: {'attributes': {'Identifiant station': 35011, 'Nom station': "Honoré d'Estienne d'Ovres - Grilles", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8905639474, 2.40648318438], 'Nom communes équipées': 'Pantin'}}, 717: {'attributes': {'Identifiant station': 23005, 'Nom station': 'Gare de Clichy-Levallois', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8967982748, 2.29846805334], 'Nom communes équipées': 'Levallois-Perret'}}, 718: {'attributes': {'Identifiant station': 15019, 'Nom station': 'Paul Barruel - Vaugirard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8401491855, 2.30443328619], 'Nom communes équipées': 'Paris'}}, 719: {'attributes': {'Identifiant station': 12010, 'Nom station': 'Daumesnil - Picpus', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8374688, 2.401765], 'Nom communes équipées': 'Paris'}}, 720: {'attributes': {'Identifiant station': 16007, 'Nom station': "Longchamp - Place d'Iéna", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8649172334, 2.29267481714], 'Nom communes équipées': 'Paris'}}, 721: {'attributes': {'Identifiant station': 20117, 'Nom station': 'Saint-Fargeau - Mortier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8726665612, 2.40791290998], 'Nom communes équipées': 'Paris'}}, 722: {'attributes': {'Identifiant station': 10110, 'Nom station': 'Louis Blanc - Jemmapes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8793307553, 2.36844856292], 'Nom communes équipées': 'Paris'}}, 723: {'attributes': {'Identifiant station': 48009, 'Nom station': 'République - Auguste Perret', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7869528095, 2.35185943544], 'Nom communes équipées': 'Villejuif'}}, 724: {'attributes': {'Identifiant station': 13515, 'Nom station': 'Auguste Blanqui - Glacière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 60, 'Nombre bornettes libres': 53, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.831151603, 2.34382219613], 'Nom communes équipées': 'Paris'}}, 725: {'attributes': {'Identifiant station': 14125, 'Nom station': 'Dareau - Broussais', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8307758, 2.3361804], 'Nom communes équipées': 'Paris'}}, 726: {'attributes': {'Identifiant station': 13010, 'Nom station': "Place d'Italie - Vincent Auriol", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8314818745, 2.35675502923], 'Nom communes équipées': 'Paris'}}, 727: {'attributes': {'Identifiant station': 21009, 'Nom station': 'Le Corbusier - Jean Jaurès', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 30, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8378083056, 2.24083811045], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 728: {'attributes': {'Identifiant station': 13027, 'Nom station': 'Mazagran - Pierre de Coubertin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 15, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8162186403, 2.34422687441], 'Nom communes équipées': 'Paris'}}, 729: {'attributes': {'Identifiant station': 21951, 'Nom station': 'Place Centrale', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7874016769, 2.2278830409], 'Nom communes équipées': 'Meudon'}}, 730: {'attributes': {'Identifiant station': 17029, 'Nom station': 'Pierre Demours - Laugier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8813072, 2.294846], 'Nom communes équipées': 'Paris'}}, 731: {'attributes': {'Identifiant station': 23006, 'Nom station': 'Jules Guesde - Alsace', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8905662755, 2.29507774115], 'Nom communes équipées': 'Levallois-Perret'}}, 732: {'attributes': {'Identifiant station': 10004, 'Nom station': 'Metz - Faubourg Saint-Denis', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 48, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 36, 'Vélos mécaniques disponibles': 33, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.870893, 2.353521], 'Nom communes équipées': 'Paris'}}, 733: {'attributes': {'Identifiant station': 18114, 'Nom station': 'Véron - Lepic', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.885261234, 2.33455985785], 'Nom communes équipées': 'Paris'}}, 734: {'attributes': {'Identifiant station': 6024, 'Nom station': 'Saint-Germain-des-Prés', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 24, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8541851082, 2.33030587435], 'Nom communes équipées': 'Paris'}}, 735: {'attributes': {'Identifiant station': 44103, 'Nom station': 'Pont De Charenton', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8179685, 2.4199531], 'Nom communes équipées': 'Saint-Maurice'}}, 736: {'attributes': {'Identifiant station': 11103, 'Nom station': 'Pelée - Alphonse Baudin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8600265, 2.3710541], 'Nom communes équipées': 'Paris'}}, 737: {'attributes': {'Identifiant station': 22403, 'Nom station': 'Pierre Brossolette - Adolphe Pinard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8217722795, 2.31315582991], 'Nom communes équipées': 'Malakoff'}}, 738: {'attributes': {'Identifiant station': 10153, 'Nom station': 'Gare du Nord - Denain', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8795957876, 2.35447257757], 'Nom communes équipées': 'Paris'}}, 739: {'attributes': {'Identifiant station': 22009, 'Nom station': 'Charles de Gaulle - Orléans', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.880922251, 2.27158159018], 'Nom communes équipées': 'Neuilly-sur-Seine'}}, 740: {'attributes': {'Identifiant station': 8008, 'Nom station': 'Joseph Sansboeuf - Saint Lazare', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8753156307, 2.32290197164], 'Nom communes équipées': 'Paris'}}, 741: {'attributes': {'Identifiant station': 13035, 'Nom station': 'Choisy - Tolbiac', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8259115559, 2.36023932695], 'Nom communes équipées': 'Paris'}}, 742: {'attributes': {'Identifiant station': 21203, 'Nom station': 'Place Jules Ferry', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8121079284, 2.32583731413], 'Nom communes équipées': 'Montrouge'}}, 743: {'attributes': {'Identifiant station': 20047, 'Nom station': 'Cours de Vincennes - Davout', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 48, 'Nombre bornettes libres': 45, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.84723803, 2.41070423362], 'Nom communes équipées': 'Paris'}}, 744: {'attributes': {'Identifiant station': 21115, 'Nom station': 'Chancé Milly - Cimetière Communal Sud', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8968718938, 2.30392232537], 'Nom communes équipées': 'Clichy'}}, 745: {'attributes': {'Identifiant station': 7024, 'Nom station': 'Rapp - Place du Général Gouraud', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 24, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8581028851, 2.30040729046], 'Nom communes équipées': 'Paris'}}, 746: {'attributes': {'Identifiant station': 19025, 'Nom station': 'Botzaris - Crimée', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8795969869, 2.38879133375], 'Nom communes équipées': 'Paris'}}, 747: {'attributes': {'Identifiant station': 31026, 'Nom station': 'Demi-Lune - Aristide Briand', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8762638192, 2.4562818557], 'Nom communes équipées': 'Montreuil'}}, 748: {'attributes': {'Identifiant station': 16009, 'Nom station': 'Saint-Didier - Raymond Poincaré', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8664231202, 2.28586349719], 'Nom communes équipées': 'Paris'}}, 749: {'attributes': {'Identifiant station': 20004, 'Nom station': "Commandant l'Herminier - Gallieni", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8470405526, 2.41604536772], 'Nom communes équipées': 'Paris'}}, 750: {'attributes': {'Identifiant station': 7111, 'Nom station': 'Ecole Militaire - Place Joffre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 18, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 18, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8520602036, 2.30179935694], 'Nom communes équipées': 'Paris'}}, 751: {'attributes': {'Identifiant station': 19113, 'Nom station': 'Pré Saint-Gervais - Lilas', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.877870627, 2.39590734243], 'Nom communes équipées': 'Paris'}}, 752: {'attributes': {'Identifiant station': 20039, 'Nom station': 'Belleville - Couronnes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 47, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8688310829, 2.38098621368], 'Nom communes équipées': 'Paris'}}, 753: {'attributes': {'Identifiant station': 9016, 'Nom station': 'Choron - Martyrs', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8778514435, 2.33967013657], 'Nom communes équipées': 'Paris'}}, 754: {'attributes': {'Identifiant station': 13048, 'Nom station': 'Wurtz - Tolbiac', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8263119562, 2.34458595514], 'Nom communes équipées': 'Paris'}}, 755: {'attributes': {'Identifiant station': 35004, 'Nom station': 'Place du Président Salvador Allende', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 60, 'Nombre bornettes libres': 52, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8973073104, 2.40079153681], 'Nom communes équipées': 'Pantin'}}, 756: {'attributes': {'Identifiant station': 11109, 'Nom station': 'Pierre Levée - Fontaine Timbaud', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8663272337, 2.37111635506], 'Nom communes équipées': 'Paris'}}, 757: {'attributes': {'Identifiant station': 33104, 'Nom station': 'Square Edmond Pépin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8836562069, 2.40335315466], 'Nom communes équipées': 'Le Pré-Saint-Gervais'}}, 758: {'attributes': {'Identifiant station': 21316, 'Nom station': "Cimetière d'Issy-les-Moulineaux", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8176997739, 2.26093120873], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 759: {'attributes': {'Identifiant station': 22010, 'Nom station': 'Charles de Gaulle - Louis-Philippe', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 14, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8816213111, 2.27128300904], 'Nom communes équipées': 'Neuilly-sur-Seine'}}, 760: {'attributes': {'Identifiant station': 31021, 'Nom station': 'Résistance - Hoche', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8642734237, 2.43578441441], 'Nom communes équipées': 'Montreuil'}}, 761: {'attributes': {'Identifiant station': 19036, 'Nom station': 'Hôpital Robert Debré', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.880052222, 2.40086503463], 'Nom communes équipées': 'Paris'}}, 762: {'attributes': {'Identifiant station': 44002, 'Nom station': 'Paul Vaillant-Couturier - Gare RER', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 18, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8010815907, 2.40209721029], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 763: {'attributes': {'Identifiant station': 14113, 'Nom station': 'Santé - Arago', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8350765756, 2.3415737262], 'Nom communes équipées': 'Paris'}}, 764: {'attributes': {'Identifiant station': 13121, 'Nom station': "Bruneseau - Quai d'Ivry", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 48, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 41, 'Vélos mécaniques disponibles': 37, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8249456376, 2.38856613636], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 765: {'attributes': {'Identifiant station': 35015, 'Nom station': 'Averroès - Aimé Césaire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9140532343, 2.41263695061], 'Nom communes équipées': 'Pantin'}}, 766: {'attributes': {'Identifiant station': 21030, 'Nom station': 'Fessart - Château', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.84245, 2.23989], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 767: {'attributes': {'Identifiant station': 32021, 'Nom station': 'Place du Front Populaire - George Sand', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 20, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.906807137, 2.36454121499], 'Nom communes équipées': 'Saint-Denis'}}, 768: {'attributes': {'Identifiant station': 21031, 'Nom station': 'Cimetière de Boulogne-Billancourt', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 36, 'Vélos mécaniques disponibles': 31, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.830764, 2.252024], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 769: {'attributes': {'Identifiant station': 12006, 'Nom station': 'Gare de Lyon - Van Gogh', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 51, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 37, 'Vélos mécaniques disponibles': 27, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8438397856, 2.37044502404], 'Nom communes équipées': 'Paris'}}, 770: {'attributes': {'Identifiant station': 16024, 'Nom station': 'Ranelagh - Maison de la Radio', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8529360317, 2.28044964373], 'Nom communes équipées': 'Paris'}}, 771: {'attributes': {'Identifiant station': 15060, 'Nom station': 'Place de la Montagne du Goulet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8443415, 2.2773844], 'Nom communes équipées': 'Paris'}}, 772: {'attributes': {'Identifiant station': 9101, 'Nom station': 'Cadet - La Fayette', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8756016327, 2.34357040375], 'Nom communes équipées': 'Paris'}}, 773: {'attributes': {'Identifiant station': 15131, 'Nom station': 'Charles Lecocq - Lecourbe', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.84044, 2.294867], 'Nom communes équipées': 'Paris'}}, 774: {'attributes': {'Identifiant station': 20009, 'Nom station': 'Docteurs Déjérine - Pte de Montreuil', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8539844813, 2.41208288897], 'Nom communes équipées': 'Paris'}}, 775: {'attributes': {'Identifiant station': 10037, 'Nom station': 'Quai de Valmy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 43, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8812695158, 2.36818604171], 'Nom communes équipées': 'Paris'}}, 776: {'attributes': {'Identifiant station': 11032, 'Nom station': 'Jean-Pierre Timbaud - Vaucouleurs', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 47, 'Nombre bornettes libres': 46, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.867937, 2.377885], 'Nom communes équipées': 'Paris'}}, 777: {'attributes': {'Identifiant station': 20002, 'Nom station': 'Square de la rue Pixérécourt', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 15, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8736824, 2.3960278], 'Nom communes équipées': 'Paris'}}, 778: {'attributes': {'Identifiant station': 17033, 'Nom station': 'Carnot - Général Lanrezac', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 33, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 10, 25, 10, 41, 59, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8754438707, 2.2938242935], 'Nom communes équipées': 'Paris'}}, 779: {'attributes': {'Identifiant station': 5029, 'Nom station': 'Georges Bernanos - Port Royal', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8401452135, 2.33735740185], 'Nom communes équipées': 'Paris'}}, 780: {'attributes': {'Identifiant station': 16003, 'Nom station': 'André Maurois - Joseph et Marie Hackin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8778983, 2.2789591], 'Nom communes équipées': 'Paris'}}, 781: {'attributes': {'Identifiant station': 9036, 'Nom station': 'Athènes - Clichy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8781137879, 2.32956021946], 'Nom communes équipées': 'Paris'}}, 782: {'attributes': {'Identifiant station': 10012, 'Nom station': 'Vinaigriers - Magenta', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8731167, 2.3593052], 'Nom communes équipées': 'Paris'}}, 783: {'attributes': {'Identifiant station': 12150, 'Nom station': 'Bastille - Quai de la Rapée', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8468241086, 2.36693646759], 'Nom communes équipées': 'Paris'}}, 784: {'attributes': {'Identifiant station': 21028, 'Nom station': 'Pierre Lefaucheux - Traversière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.824352083, 2.24179130048], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 785: {'attributes': {'Identifiant station': 15125, 'Nom station': 'Parc Suzanne Lenglen', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 52, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 39, 'Vélos mécaniques disponibles': 38, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8332506905, 2.27654032409], 'Nom communes équipées': 'Paris'}}, 786: {'attributes': {'Identifiant station': 20007, 'Nom station': 'Buzenval - Avron', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 42, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.851617205, 2.40151359731], 'Nom communes équipées': 'Paris'}}, 787: {'attributes': {'Identifiant station': 31707, 'Nom station': 'Edouard Vaillant - Galliéni', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 49, 'Nombre bornettes libres': 49, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2020, 8, 14, 4, 12, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8584628817, 2.41479545832], 'Nom communes équipées': 'Bagnolet'}}, 788: {'attributes': {'Identifiant station': 32301, 'Nom station': 'Mairie de Romainville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.885109, 2.435679], 'Nom communes équipées': 'Romainville'}}, 789: {'attributes': {'Identifiant station': 10152, 'Nom station': 'Gare du Nord - Place de Valenciennes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.879089, 2.354235], 'Nom communes équipées': 'Paris'}}, 790: {'attributes': {'Identifiant station': 13011, 'Nom station': 'Hôpital - Campo-Formio', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.835405, 2.358218], 'Nom communes équipées': 'Paris'}}, 791: {'attributes': {'Identifiant station': 16014, 'Nom station': 'Eylau - Trocadéro', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8633541224, 2.28642091155], 'Nom communes équipées': 'Paris'}}, 792: {'attributes': {'Identifiant station': 22501, 'Nom station': 'Théâtre de La Garenne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9057683093, 2.23539210223], 'Nom communes équipées': 'La Garenne-Colombes'}}, 793: {'attributes': {'Identifiant station': 13039, 'Nom station': 'Ivry - Simone Weil', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.823283932, 2.36590951681], 'Nom communes équipées': 'Paris'}}, 794: {'attributes': {'Identifiant station': 16030, 'Nom station': 'Place de Barcelone - Mirabeau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 55, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 44, 'Vélos mécaniques disponibles': 37, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8476230274, 2.27356642485], 'Nom communes équipées': 'Paris'}}, 795: {'attributes': {'Identifiant station': 21118, 'Nom station': 'Neuilly - Petit', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8989653613, 2.29627937078], 'Nom communes équipées': 'Clichy'}}, 796: {'attributes': {'Identifiant station': 21219, 'Nom station': 'Amaury Duval - Gabriel Péri', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 35, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8171875967, 2.32384175062], 'Nom communes équipées': 'Montrouge'}}, 797: {'attributes': {'Identifiant station': 5034, 'Nom station': 'Censier - Santeuil', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 69, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 37, 'Vélos mécaniques disponibles': 34, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8406589214, 2.35375240445], 'Nom communes équipées': 'Paris'}}, 798: {'attributes': {'Identifiant station': 10018, 'Nom station': 'Square Alban Satragne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8753405484, 2.35588476062], 'Nom communes équipées': 'Paris'}}, 799: {'attributes': {'Identifiant station': 22013, 'Nom station': 'Mairie de Neuilly sur Seine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8849472877, 2.26770311594], 'Nom communes équipées': 'Neuilly-sur-Seine'}}, 800: {'attributes': {'Identifiant station': 22008, 'Nom station': 'Charles de Gaulle - Graviers', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8831517592, 2.26488143206], 'Nom communes équipées': 'Neuilly-sur-Seine'}}, 801: {'attributes': {'Identifiant station': 14101, 'Nom station': 'Gare Montparnasse - Edgar Quinet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8414878511, 2.32332322828], 'Nom communes équipées': 'Paris'}}, 802: {'attributes': {'Identifiant station': 5014, 'Nom station': 'Jean Calvin - Tournefort', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8415885693, 2.34848957509], 'Nom communes équipées': 'Paris'}}, 803: {'attributes': {'Identifiant station': 21029, 'Nom station': "Cours de l'Ile Seguin - Parc Billancourt", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8271405584, 2.23423451185], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 804: {'attributes': {'Identifiant station': 8549, 'Nom station': 'George V - Christophe Colomb', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 12, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 43, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.87046, 2.30104], 'Nom communes équipées': 'Paris'}}, 805: {'attributes': {'Identifiant station': 15072, 'Nom station': 'Place Fernand Forest', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8492691438, 2.28186316788], 'Nom communes équipées': 'Paris'}}, 806: {'attributes': {'Identifiant station': 13033, 'Nom station': "Porte d'Italie", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 13, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8171619877, 2.36023630947], 'Nom communes équipées': 'Paris'}}, 807: {'attributes': {'Identifiant station': 21708, 'Nom station': 'Marcel Yol - Jullien', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.825376118, 2.28723501806], 'Nom communes équipées': 'Vanves'}}, 808: {'attributes': {'Identifiant station': 20122, 'Nom station': 'Vitruve - Davout', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8605316115, 2.40915928126], 'Nom communes équipées': 'Paris'}}, 809: {'attributes': {'Identifiant station': 10017, 'Nom station': "Gare de l'Est - Fidélité", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 15, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 43, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.874575, 2.356796], 'Nom communes équipées': 'Paris'}}, 810: {'attributes': {'Identifiant station': 11038, 'Nom station': 'Temple - Charlot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8657093438, 2.36527372507], 'Nom communes équipées': 'Paris'}}, 811: {'attributes': {'Identifiant station': 41210, 'Nom station': 'Hôtel de ville de Fontenay-sous-Bois', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8494974911, 2.47501641512], 'Nom communes équipées': 'Fontenay-sous-Bois'}}, 812: {'attributes': {'Identifiant station': 4003, 'Nom station': 'Quai aux Fleurs - Pont Saint-Louis', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8529159562, 2.35219571739], 'Nom communes équipées': 'Paris'}}, 813: {'attributes': {'Identifiant station': 42304, 'Nom station': 'Marc Sangnier - Division Leclerc', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 22, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7859965231, 2.3299555108], 'Nom communes équipées': 'Cachan'}}, 814: {'attributes': {'Identifiant station': 12043, 'Nom station': 'Daumesnil - Hébrard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 31, 'Vélos mécaniques disponibles': 28, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8429, 2.38474], 'Nom communes équipées': 'Paris'}}, 815: {'attributes': {'Identifiant station': 4111, 'Nom station': "Hôtel de Ville - Pont d'Arcole", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 1, 22, 23, 26, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8558764518, 2.35089216381], 'Nom communes équipées': 'Paris'}}, 816: {'attributes': {'Identifiant station': 8020, 'Nom station': 'Batignolles - Beudant', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 44, 'Nombre bornettes libres': 38, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8821640147, 2.31985297054], 'Nom communes équipées': 'Paris'}}, 817: {'attributes': {'Identifiant station': 15052, 'Nom station': 'Convention - Vaugirard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 48, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 20, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8376768223, 2.29558692602], 'Nom communes équipées': 'Paris'}}, 818: {'attributes': {'Identifiant station': 18027, 'Nom station': 'Damrémont - Ordener', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.893400393, 2.33626652391], 'Nom communes équipées': 'Paris'}}, 819: {'attributes': {'Identifiant station': 16115, 'Nom station': 'Raffet - Montmorency', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 38, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8525867081, 2.26292524562], 'Nom communes équipées': 'Paris'}}, 820: {'attributes': {'Identifiant station': 15023, 'Nom station': 'La Motte-Picquet - Grenelle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8485832134, 2.2993478179], 'Nom communes équipées': 'Paris'}}, 821: {'attributes': {'Identifiant station': 15123, 'Nom station': 'Commerce - Letellier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 44, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 26, 'Vélos mécaniques disponibles': 26, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8479766037, 2.29662451966], 'Nom communes équipées': 'Paris'}}, 822: {'attributes': {'Identifiant station': 13128, 'Nom station': 'Quai de la Gare - Pont de Bercy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 14, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8373078797, 2.37408612606], 'Nom communes équipées': 'Paris'}}, 823: {'attributes': {'Identifiant station': 21307, 'Nom station': 'Victor Cresson - Telles de la Poterie', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.822698843, 2.2686498275], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 824: {'attributes': {'Identifiant station': 23003, 'Nom station': 'Président Wilson - Baudin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 26, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 12, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.898268871, 2.28493362665], 'Nom communes équipées': 'Levallois-Perret'}}, 825: {'attributes': {'Identifiant station': 15043, 'Nom station': 'Brancion - Lefebvre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 11, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8290022279, 2.30143103816], 'Nom communes équipées': 'Paris'}}, 826: {'attributes': {'Identifiant station': 4016, 'Nom station': 'Lobau - Hôtel de Ville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 60, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 39, 'Vélos mécaniques disponibles': 35, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8562118, 2.3532536], 'Nom communes équipées': 'Paris'}}, 827: {'attributes': {'Identifiant station': 17039, 'Nom station': 'Place Saint-Ferdinand', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8781552979, 2.28851804786], 'Nom communes équipées': 'Paris'}}, 828: {'attributes': {'Identifiant station': 11021, 'Nom station': 'Charonne - Pierre Bayle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 5, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8584450543, 2.39039812239], 'Nom communes équipées': 'Paris'}}, 829: {'attributes': {'Identifiant station': 5107, 'Nom station': 'Pontoise - La Tournelle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8504580092, 2.35245427549], 'Nom communes équipées': 'Paris'}}, 830: {'attributes': {'Identifiant station': 19108, 'Nom station': 'Gaston Tessier - Rosa Parks RER', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8963765, 2.3743692], 'Nom communes équipées': 'Paris'}}, 831: {'attributes': {'Identifiant station': 22604, 'Nom station': 'Hôtel de Ville de Fontenay-aux-Roses', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7908446035, 2.28714559303], 'Nom communes équipées': 'Fontenay-aux-Roses'}}, 832: {'attributes': {'Identifiant station': 42501, 'Nom station': 'Raspail - Albert Schweitzer', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 24, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.81031545, 2.34395871], 'Nom communes équipées': 'Gentilly'}}, 833: {'attributes': {'Identifiant station': 19034, 'Nom station': 'Porte de Pantin - Petits Ponts', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 52, 'Nombre bornettes libres': 48, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2019, 6, 6, 9, 56, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8904108624, 2.39819392562], 'Nom communes équipées': 'Paris'}}, 834: {'attributes': {'Identifiant station': 35001, 'Nom station': 'Jean Jaurès - Ouvrières Pivéreuses', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9104234375, 2.39957660437], 'Nom communes équipées': 'Pantin'}}, 835: {'attributes': {'Identifiant station': 21213, 'Nom station': 'Vanne - Général de Gaulle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.815007, 2.328318], 'Nom communes équipées': 'Montrouge'}}, 836: {'attributes': {'Identifiant station': 42209, 'Nom station': "Place de l'Europe - Quai de Bercy", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8252985884, 2.39456791431], 'Nom communes équipées': 'Charenton-le-Pont'}}, 837: {'attributes': {'Identifiant station': 16101, 'Nom station': 'Malakoff - Place de la Porte Maillot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8767334966, 2.28345774114], 'Nom communes équipées': 'Paris'}}, 838: {'attributes': {'Identifiant station': 22206, 'Nom station': 'Louis Pasteur - Albert Petit', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.796261, 2.317289], 'Nom communes équipées': 'Bagneux'}}, 839: {'attributes': {'Identifiant station': 21702, 'Nom station': 'Gare SNCF Vanves-Malakoff', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 51, 'Nombre bornettes libres': 51, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8190021638, 2.2921500051], 'Nom communes équipées': 'Vanves'}}, 840: {'attributes': {'Identifiant station': 13106, 'Nom station': 'Bobillot - Paulin Mery', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8298998417, 2.35433567667], 'Nom communes équipées': 'Paris'}}, 841: {'attributes': {'Identifiant station': 42305, 'Nom station': 'Vanne - Saussaies', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.786404, 2.338848], 'Nom communes équipées': 'Cachan'}}, 842: {'attributes': {'Identifiant station': 17003, 'Nom station': 'Jacques Kellner - Saint-Ouen', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 38, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 2, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8958094858, 2.32789740916], 'Nom communes équipées': 'Paris'}}, 843: {'attributes': {'Identifiant station': 42003, 'Nom station': "Place de l'Insurrection d'Août 1944", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 26, 'Vélos mécaniques disponibles': 22, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8147785275, 2.39807456732], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 844: {'attributes': {'Identifiant station': 15064, 'Nom station': 'Javel - André Citroën', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 42, 'Vélos mécaniques disponibles': 34, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8462764855, 2.27863680571], 'Nom communes équipées': 'Paris'}}, 845: {'attributes': {'Identifiant station': 31025, 'Nom station': 'Paul Signac - Aristide Briand', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8695026216, 2.4502260983], 'Nom communes équipées': 'Montreuil'}}, 846: {'attributes': {'Identifiant station': 15030, 'Nom station': 'Emeriau - Beaugrenelle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 74, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 70, 'Vélos mécaniques disponibles': 64, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8479124971, 2.28430598974], 'Nom communes équipées': 'Paris'}}, 847: {'attributes': {'Identifiant station': 32011, 'Nom station': 'Stade de France', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 44, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9191419536, 2.36206557602], 'Nom communes équipées': 'Saint-Denis'}}, 848: {'attributes': {'Identifiant station': 16011, 'Nom station': 'Victor Hugo - La Pompe', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 41, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8679522247, 2.28146318346], 'Nom communes équipées': 'Paris'}}, 849: {'attributes': {'Identifiant station': 42701, 'Nom station': 'Charles Gide - Fort de Bicêtre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8071937957, 2.35398441553], 'Nom communes équipées': 'Le Kremlin-Bicêtre'}}, 850: {'attributes': {'Identifiant station': 12007, 'Nom station': 'Gare de Lyon - Chalon', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8456661942, 2.37414523959], 'Nom communes équipées': 'Paris'}}, 851: {'attributes': {'Identifiant station': 20108, 'Nom station': "Jardin de l'Hospice Debrousse", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8614510619, 2.40568600595], 'Nom communes équipées': 'Paris'}}, 852: {'attributes': {'Identifiant station': 47001, 'Nom station': 'Place de la Gare', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 39, 'Vélos mécaniques disponibles': 34, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8025335801, 2.42750376463], 'Nom communes équipées': 'Maisons-Alfort'}}, 853: {'attributes': {'Identifiant station': 16020, 'Nom station': 'Place de la Porte de Passy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8573410955, 2.26441655388], 'Nom communes équipées': 'Paris'}}, 854: {'attributes': {'Identifiant station': 41101, 'Nom station': 'Paul Doumer - Stalingrad', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8065189922, 2.33688801527], 'Nom communes équipées': 'Arcueil'}}, 855: {'attributes': {'Identifiant station': 13126, 'Nom station': 'Place Nationale - Château-des-Rentiers', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.829566, 2.365053], 'Nom communes équipées': 'Paris'}}, 856: {'attributes': {'Identifiant station': 27004, 'Nom station': 'Gare de la Garenne-Colombes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.909752, 2.239145], 'Nom communes équipées': 'Colombes'}}, 857: {'attributes': {'Identifiant station': 17015, 'Nom station': 'Place de Lévis', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 48, 'Nombre bornettes libres': 43, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8835468515, 2.31319338083], 'Nom communes équipées': 'Paris'}}, 858: {'attributes': {'Identifiant station': 18038, 'Nom station': 'Rond-Point de la Chapelle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 49, 'Nombre bornettes libres': 42, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8951266093, 2.35997110605], 'Nom communes équipées': 'Paris'}}, 859: {'attributes': {'Identifiant station': 4001, 'Nom station': 'Arcole - Notre-Dame', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 25, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8538835413, 2.3493797332], 'Nom communes équipées': 'Paris'}}, 860: {'attributes': {'Identifiant station': 43010, 'Nom station': 'La Jarry - Place Diderot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8497269404, 2.45201379061], 'Nom communes équipées': 'Vincennes'}}, 861: {'attributes': {'Identifiant station': 14131, 'Nom station': 'Jourdan - Monticelli', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.822032, 2.329095], 'Nom communes équipées': 'Paris'}}, 862: {'attributes': {'Identifiant station': 17107, 'Nom station': 'Berthier - Place Stuart Merril', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8859279224, 2.29334503412], 'Nom communes équipées': 'Paris'}}, 863: {'attributes': {'Identifiant station': 17023, 'Nom station': "Porte d'Asnières", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 33, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8903581761, 2.30362862349], 'Nom communes équipées': 'Paris'}}, 864: {'attributes': {'Identifiant station': 15004, 'Nom station': 'Vaugirard - Pasteur', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.840408688, 2.3154367134], 'Nom communes équipées': 'Paris'}}, 865: {'attributes': {'Identifiant station': 44007, 'Nom station': 'Camille Groult - Guy Môquet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 18, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7874827465, 2.39767257124], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 866: {'attributes': {'Identifiant station': 15057, 'Nom station': 'Lucien Bossoutrot - Général Martial Vallin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 29, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 12, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8369560463, 2.27410722524], 'Nom communes équipées': 'Paris'}}, 867: {'attributes': {'Identifiant station': 6008, 'Nom station': 'Assas - Luxembourg', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8437344688, 2.33342838188], 'Nom communes équipées': 'Paris'}}, 868: {'attributes': {'Identifiant station': 17108, 'Nom station': 'Bessières - Porte de Clichy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8961176245, 2.31790903956], 'Nom communes équipées': 'Paris'}}, 869: {'attributes': {'Identifiant station': 14010, 'Nom station': 'Jean Moulin - Place Victor et Hélène Basch', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 41, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8276491024, 2.32619673014], 'Nom communes équipées': 'Paris'}}, 870: {'attributes': {'Identifiant station': 4020, 'Nom station': 'Place Edmond Michelet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8601348905, 2.3500636965], 'Nom communes équipées': 'Paris'}}, 871: {'attributes': {'Identifiant station': 10105, 'Nom station': 'Mazagran - Bonne Nouvelle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8700471237, 2.35112484545], 'Nom communes équipées': 'Paris'}}, 872: {'attributes': {'Identifiant station': 15022, 'Nom station': 'Emile Zola - Fondary', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8470511431, 2.29534059763], 'Nom communes équipées': 'Paris'}}, 873: {'attributes': {'Identifiant station': 13109, 'Nom station': 'Brillat-Savarin - Küss', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.822203, 2.350422], 'Nom communes équipées': 'Paris'}}, 874: {'attributes': {'Identifiant station': 9024, 'Nom station': 'Taitbout - La Fayette', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 53, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 47, 'Vélos mécaniques disponibles': 36, 'Vélos électriques disponibles': 11, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8733298892, 2.33526259661], 'Nom communes équipées': 'Paris'}}, 875: {'attributes': {'Identifiant station': 4002, 'Nom station': 'Marché aux fleurs', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8552581154, 2.34737545252], 'Nom communes équipées': 'Paris'}}, 876: {'attributes': {'Identifiant station': 16001, 'Nom station': 'Portugais - Kléber', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 12, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8712148, 2.2937349], 'Nom communes équipées': 'Paris'}}, 877: {'attributes': {'Identifiant station': 21204, 'Nom station': "Estienne d'Orves - Henri Ginoux", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8153882204, 2.32088679076], 'Nom communes équipées': 'Montrouge'}}, 878: {'attributes': {'Identifiant station': 33015, 'Nom station': 'Charles Tillon - Cimetière Communal', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 11, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9184402364, 2.39356309175], 'Nom communes équipées': 'Aubervilliers'}}, 879: {'attributes': {'Identifiant station': 43001, 'Nom station': 'Paris - Laitières', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8463027415, 2.42075532675], 'Nom communes équipées': 'Vincennes'}}, 880: {'attributes': {'Identifiant station': 15311, 'Nom station': 'Quai du Président Roosevelt - Pont Aval', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8343883676, 2.26568207145], 'Nom communes équipées': 'Paris'}}, 881: {'attributes': {'Identifiant station': 21217, 'Nom station': 'Louis Rolland - République', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8171088974, 2.3193106435], 'Nom communes équipées': 'Montrouge'}}, 882: {'attributes': {'Identifiant station': 35006, 'Nom station': 'Delizy - Louis Nadot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8962128456, 2.40945786238], 'Nom communes équipées': 'Pantin'}}, 883: {'attributes': {'Identifiant station': 9114, 'Nom station': 'Faubourg Poissonnière - Dunkerque', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 14, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8808834454, 2.34993427992], 'Nom communes équipées': 'Paris'}}, 884: {'attributes': {'Identifiant station': 15011, 'Nom station': 'Cambronne - Villa Croix Nivert', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 62, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 26, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8458475614, 2.30175744742], 'Nom communes équipées': 'Paris'}}, 885: {'attributes': {'Identifiant station': 25001, 'Nom station': 'Colmar - Victor Hugo', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.886522511, 2.1739609167], 'Nom communes équipées': 'Rueil-Malmaison'}}, 886: {'attributes': {'Identifiant station': 1021, 'Nom station': 'Halles - Bourdonnais', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 59, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8604489926, 2.3460162431], 'Nom communes équipées': 'Paris'}}, 887: {'attributes': {'Identifiant station': 35007, 'Nom station': 'Benjamin Delessert - Jean Lolive', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8939204362, 2.41798192263], 'Nom communes équipées': 'Pantin'}}, 888: {'attributes': {'Identifiant station': 12018, 'Nom station': "Saint-Mandé - Fabre d'Eglantine", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8453579, 2.3965022], 'Nom communes équipées': 'Paris'}}, 889: {'attributes': {'Identifiant station': 44101, 'Nom station': 'Place Mongolfier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 35, 'Vélos mécaniques disponibles': 31, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8151600301, 2.45928794146], 'Nom communes équipées': 'Saint-Maurice'}}, 890: {'attributes': {'Identifiant station': 17111, 'Nom station': 'Lemercier - Cardinet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8900971689, 2.3173776269], 'Nom communes équipées': 'Paris'}}, 891: {'attributes': {'Identifiant station': 31007, 'Nom station': 'Paris - Désiré Préaux', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 49, 'Nombre bornettes libres': 44, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8574287669, 2.43230491877], 'Nom communes équipées': 'Montreuil'}}, 892: {'attributes': {'Identifiant station': 17119, 'Nom station': 'Toqueville - Cardinet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 3, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8849018711, 2.31093931943], 'Nom communes équipées': 'Paris'}}, 893: {'attributes': {'Identifiant station': 13032, 'Nom station': 'Italie - Jardin Joan Miro', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8199091405, 2.3592948541], 'Nom communes équipées': 'Paris'}}, 894: {'attributes': {'Identifiant station': 18041, 'Nom station': 'Rochechouart - Martyrs', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8821937796, 2.34054956585], 'Nom communes équipées': 'Paris'}}, 895: {'attributes': {'Identifiant station': 8037, 'Nom station': 'Monceau - Malesherbes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8795079, 2.3145456], 'Nom communes équipées': 'Paris'}}, 896: {'attributes': {'Identifiant station': 7017, 'Nom station': 'La Motte-Picquet - La Tour Maubourg', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 35, 'Vélos mécaniques disponibles': 32, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8572316, 2.3094136], 'Nom communes équipées': 'Paris'}}, 897: {'attributes': {'Identifiant station': 22303, 'Nom station': 'Lucien Lanternier - Le Luth', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9315943056, 2.28897839785], 'Nom communes équipées': 'Gennevilliers'}}, 898: {'attributes': {'Identifiant station': 15071, 'Nom station': 'Suffren - Champ de Mars', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 19, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8547194, 2.2952168], 'Nom communes équipées': 'Paris'}}, 899: {'attributes': {'Identifiant station': 92001, 'Nom station': 'Mairie de quartier du Plateau - Mont-Valérien', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8799933, 2.2072437], 'Nom communes équipées': 'Nanterre'}}, 900: {'attributes': {'Identifiant station': 9004, 'Nom station': 'Gérando - Rochechouart', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8829434084, 2.34648462385], 'Nom communes équipées': 'Paris'}}, 901: {'attributes': {'Identifiant station': 23004, 'Nom station': 'Paul Vaillant-Couturier - Victor Hugo', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8981970226, 2.29384801797], 'Nom communes équipées': 'Levallois-Perret'}}, 902: {'attributes': {'Identifiant station': 4021, 'Nom station': 'Centre Georges Pompidou', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.861419, 2.352581], 'Nom communes équipées': 'Paris'}}, 903: {'attributes': {'Identifiant station': 24003, 'Nom station': 'Place Charles de Gaulle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 9, 17, 1, 16, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8966431015, 2.25257009268], 'Nom communes équipées': 'Courbevoie'}}, 904: {'attributes': {'Identifiant station': 15102, 'Nom station': 'Square Bela Bartok', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8512015683, 2.28465367109], 'Nom communes équipées': 'Paris'}}, 905: {'attributes': {'Identifiant station': 16002, 'Nom station': 'Pergolèse - Malakoff', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8752108878, 2.28427816182], 'Nom communes équipées': 'Paris'}}, 906: {'attributes': {'Identifiant station': 12026, 'Nom station': 'Place Ginette Hamelin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 51, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 42, 'Vélos mécaniques disponibles': 37, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.837853, 2.382064], 'Nom communes équipées': 'Paris'}}, 907: {'attributes': {'Identifiant station': 23010, 'Nom station': 'Voltaire - Anatole France', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8919993603, 2.28426273912], 'Nom communes équipées': 'Levallois-Perret'}}, 908: {'attributes': {'Identifiant station': 13009, 'Nom station': 'Auguste Blanqui - Corvisart', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8300989, 2.3502726], 'Nom communes équipées': 'Paris'}}, 909: {'attributes': {'Identifiant station': 18007, 'Nom station': "Goutte d'or - Barbès", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8851243236, 2.34969623387], 'Nom communes équipées': 'Paris'}}, 910: {'attributes': {'Identifiant station': 21501, 'Nom station': 'Pont de Suresnes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 52, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8674150923, 2.22589466796], 'Nom communes équipées': 'Suresnes'}}, 911: {'attributes': {'Identifiant station': 15005, 'Nom station': 'Place Jacques et Thérèse Tréfouël', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 38, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8427264983, 2.31256574392], 'Nom communes équipées': 'Paris'}}, 912: {'attributes': {'Identifiant station': 9102, 'Nom station': "Londres - Place d'Estienne d'Orves", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 24, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8766938058, 2.33053620905], 'Nom communes équipées': 'Paris'}}, 913: {'attributes': {'Identifiant station': 19128, 'Nom station': "Quai de l'Oise - Aisne", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8906231, 2.3833701], 'Nom communes équipées': 'Paris'}}, 914: {'attributes': {'Identifiant station': 12038, 'Nom station': 'Picpus - Louis Braille', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 44, 'Nombre bornettes libres': 37, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8399887889, 2.40052566855], 'Nom communes équipées': 'Paris'}}, 915: {'attributes': {'Identifiant station': 22408, 'Nom station': 'Place du 14 Juillet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8192669595, 2.30119988322], 'Nom communes équipées': 'Malakoff'}}, 916: {'attributes': {'Identifiant station': 18030, 'Nom station': 'Joseph Dijon - Ornano', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8937445335, 2.3476678133], 'Nom communes équipées': 'Paris'}}, 917: {'attributes': {'Identifiant station': 11017, 'Nom station': 'Charonne - Bureau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 47, 'Nombre bornettes libres': 42, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 3, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.854176, 2.396093], 'Nom communes équipées': 'Paris'}}, 918: {'attributes': {'Identifiant station': 5024, 'Nom station': 'Place Monge', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8427618031, 2.35240794718], 'Nom communes équipées': 'Paris'}}, 919: {'attributes': {'Identifiant station': 18006, 'Nom station': 'Square Louise Michel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8846873545, 2.34413869679], 'Nom communes équipées': 'Paris'}}, 920: {'attributes': {'Identifiant station': 14137, 'Nom station': "Maurice d'Ocagne - General Maudhuy", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8241428252, 2.31587894261], 'Nom communes équipées': 'Paris'}}, 921: {'attributes': {'Identifiant station': 6027, 'Nom station': 'Blaise-Desgoffe - Vaugirard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8461288777, 2.32426721603], 'Nom communes équipées': 'Paris'}}, 922: {'attributes': {'Identifiant station': 12155, 'Nom station': 'Rottembourg - Général Michel Bizot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8378782599, 2.40440525115], 'Nom communes équipées': 'Paris'}}, 923: {'attributes': {'Identifiant station': 21401, 'Nom station': 'Mairie de Clamart', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.800612, 2.262151], 'Nom communes équipées': 'Clamart'}}, 924: {'attributes': {'Identifiant station': 12028, 'Nom station': 'Vivaldi - Antoine-Julien Hénard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8419236633, 2.38972897947], 'Nom communes équipées': 'Paris'}}, 925: {'attributes': {'Identifiant station': 13044, 'Nom station': "Place Jeanne d'Arc", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.829575, 2.369126], 'Nom communes équipées': 'Paris'}}, 926: {'attributes': {'Identifiant station': 9034, 'Nom station': 'Godot de Mauroy - Madeleine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.869789, 2.326347], 'Nom communes équipées': 'Paris'}}, 927: {'attributes': {'Identifiant station': 9027, 'Nom station': 'Douai - Pierre Fontaine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8822910115, 2.33324121684], 'Nom communes équipées': 'Paris'}}, 928: {'attributes': {'Identifiant station': 12035, 'Nom station': 'Madagascar - Meuniers', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8346046447, 2.3970650509], 'Nom communes équipées': 'Paris'}}, 929: {'attributes': {'Identifiant station': 41205, 'Nom station': 'Maréchal Joffre - Verdun', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 41, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.85385, 2.470307], 'Nom communes équipées': 'Fontenay-sous-Bois'}}, 930: {'attributes': {'Identifiant station': 1003, 'Nom station': 'Saint-Denis - Rivoli', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 27, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8590593408, 2.34769430012], 'Nom communes équipées': 'Paris'}}, 931: {'attributes': {'Identifiant station': 22015, 'Nom station': 'Île de la Jatte', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8940361613, 2.26663392037], 'Nom communes équipées': 'Neuilly-sur-Seine'}}, 932: {'attributes': {'Identifiant station': 41211, 'Nom station': 'Louis Xavier de Ricard - Ancienne Mairie', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 33, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 14, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8462989907, 2.47242037207], 'Nom communes équipées': 'Fontenay-sous-Bois'}}, 933: {'attributes': {'Identifiant station': 44016, 'Nom station': 'Balzac - Olympes de Gouges', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7785033249, 2.400678657], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 934: {'attributes': {'Identifiant station': 44011, 'Nom station': 'Lebrun - Colonel Fabien', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7826687073, 2.38131780177], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 935: {'attributes': {'Identifiant station': 44005, 'Nom station': 'Place Saint-Just', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7903795187, 2.38952267915], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 936: {'attributes': {'Identifiant station': 4019, 'Nom station': 'Cloître Saint-Merri - Renard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 20, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8588354489, 2.35182791948], 'Nom communes équipées': 'Paris'}}, 937: {'attributes': {'Identifiant station': 51002, 'Nom station': 'Placette Grégoire Collas', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.94342014, 2.24874760956], 'Nom communes équipées': 'Argenteuil'}}, 938: {'attributes': {'Identifiant station': 8116, 'Nom station': 'Champs-Elysees - Bassano', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 2, 21, 20, 9, 18, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8718118078, 2.30023898184], 'Nom communes équipées': 'Paris'}}, 939: {'attributes': {'Identifiant station': 32601, 'Nom station': 'Porte des Lilas - Faidherbe', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 35, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8786185567, 2.41203278303], 'Nom communes équipées': 'Les Lilas'}}, 940: {'attributes': {'Identifiant station': 20015, 'Nom station': 'Bagnolet - Orteaux', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 33, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8570864103, 2.39841252565], 'Nom communes équipées': 'Paris'}}, 941: {'attributes': {'Identifiant station': 33016, 'Nom station': "Fort d'Aubervilliers", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9150238237, 2.4035281688], 'Nom communes équipées': 'Aubervilliers'}}, 942: {'attributes': {'Identifiant station': 51001, 'Nom station': "Gare d'Argenteuil", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9470224368, 2.2568981722], 'Nom communes équipées': 'Argenteuil'}}, 943: {'attributes': {'Identifiant station': 6012, 'Nom station': 'Saint-Germain - Ciseaux', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8533052776, 2.33446546188], 'Nom communes équipées': 'Paris'}}, 944: {'attributes': {'Identifiant station': 8027, 'Nom station': 'Mairie du 8ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 14, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.877932, 2.318264], 'Nom communes équipées': 'Paris'}}, 945: {'attributes': {'Identifiant station': 12005, 'Nom station': 'Crozatier - Faubourg Saint-Antoine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 24, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8504400134, 2.37890136621], 'Nom communes équipées': 'Paris'}}, 946: {'attributes': {'Identifiant station': 2112, 'Nom station': 'Bibliothèque Nationale', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8682889935, 2.33808930963], 'Nom communes équipées': 'Paris'}}, 947: {'attributes': {'Identifiant station': 27005, 'Nom station': 'Gare du Stade', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9312676, 2.2603778], 'Nom communes équipées': 'Colombes'}}, 948: {'attributes': {'Identifiant station': 41403, 'Nom station': 'Gare RER - Malleret-Joinville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.802936, 2.42531], 'Nom communes équipées': 'Alfortville'}}, 949: {'attributes': {'Identifiant station': 17123, 'Nom station': 'Mstislav Rostropovitch - Berthier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8921208212, 2.30923410505], 'Nom communes équipées': 'Paris'}}, 950: {'attributes': {'Identifiant station': 21303, 'Nom station': 'Jean Monnet - Iles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 23, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8233939841, 2.24970731949], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 951: {'attributes': {'Identifiant station': 21319, 'Nom station': 'Esplanade du Foncet - Camille Desmoulins', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8270902335, 2.2606857866], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 952: {'attributes': {'Identifiant station': 33014, 'Nom station': 'Parc Eli Lotar', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9184499305, 2.37270858139], 'Nom communes équipées': 'Aubervilliers'}}, 953: {'attributes': {'Identifiant station': 18044, 'Nom station': 'Clichy - Douai', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8841183189, 2.32872873545], 'Nom communes équipées': 'Paris'}}, 954: {'attributes': {'Identifiant station': 13052, 'Nom station': 'Leredde - Tolbiac', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 44, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 37, 'Vélos mécaniques disponibles': 30, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8288762966, 2.37443827093], 'Nom communes équipées': 'Paris'}}, 955: {'attributes': {'Identifiant station': 9006, 'Nom station': 'Dunkerque - Trudaine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8821516677, 2.34632536769], 'Nom communes équipées': 'Paris'}}, 956: {'attributes': {'Identifiant station': 15127, 'Nom station': 'Place Charles Michels', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 32, 'Vélos mécaniques disponibles': 30, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 5, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.846401, 2.28502], 'Nom communes équipées': 'Paris'}}, 957: {'attributes': {'Identifiant station': 21314, 'Nom station': 'Chemin de Bretagne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 27, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.825147, 2.25936], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 958: {'attributes': {'Identifiant station': 11011, 'Nom station': 'Richard Lenoir - Voltaire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8616427388, 2.3727491498], 'Nom communes équipées': 'Paris'}}, 959: {'attributes': {'Identifiant station': 92006, 'Nom station': 'Place Nelson Mandela', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8978743182, 2.21828207374], 'Nom communes équipées': 'Nanterre'}}, 960: {'attributes': {'Identifiant station': 15036, 'Nom station': 'Croix Nivert - Square Saint-Lambert', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 47, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 32, 'Vélos mécaniques disponibles': 31, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8430515216, 2.29520045221], 'Nom communes équipées': 'Paris'}}, 961: {'attributes': {'Identifiant station': 31005, 'Nom station': 'Square Denise Buisson', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 41, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8562322731, 2.42643624544], 'Nom communes équipées': 'Montreuil'}}, 962: {'attributes': {'Identifiant station': 20105, 'Nom station': 'Benoît Frachon - République', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8523072901, 2.41560548544], 'Nom communes équipées': 'Paris'}}, 963: {'attributes': {'Identifiant station': 13516, 'Nom station': 'Glacière - Soeur Catherine Marie', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 38, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8295229894, 2.34357040375], 'Nom communes équipées': 'Paris'}}, 964: {'attributes': {'Identifiant station': 15018, 'Nom station': 'Dutot - Place Alleray', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 56, 'Nombre bornettes libres': 52, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8368260918, 2.30717010956], 'Nom communes équipées': 'Paris'}}, 965: {'attributes': {'Identifiant station': 2108, 'Nom station': 'Vivienne - Montmartre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 36, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 21, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8713475974, 2.34145883471], 'Nom communes équipées': 'Paris'}}, 966: {'attributes': {'Identifiant station': 6002, 'Nom station': 'Saint-Benoît - Jacob', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 2, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8554513601, 2.33347222209], 'Nom communes équipées': 'Paris'}}, 967: {'attributes': {'Identifiant station': 9011, 'Nom station': 'Geoffroy - Marie', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8737819406, 2.34451387634], 'Nom communes équipées': 'Paris'}}, 968: {'attributes': {'Identifiant station': 18039, 'Nom station': "Jardins d'Eole", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8869703, 2.3668488], 'Nom communes équipées': 'Paris'}}, 969: {'attributes': {'Identifiant station': 7109, 'Nom station': 'Place Joffre - Frédéric le Play', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8537544869, 2.30442423373], 'Nom communes équipées': 'Paris'}}, 970: {'attributes': {'Identifiant station': 14031, 'Nom station': 'Didot - Place de Moro-Giafferi', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8337, 2.3215], 'Nom communes équipées': 'Paris'}}, 971: {'attributes': {'Identifiant station': 13025, 'Nom station': 'Place de Rungis', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8224559476, 2.34738272948], 'Nom communes équipées': 'Paris'}}, 972: {'attributes': {'Identifiant station': 20044, 'Nom station': 'Davout - Volga', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.85291968, 2.41052290978], 'Nom communes équipées': 'Paris'}}, 973: {'attributes': {'Identifiant station': 5115, 'Nom station': 'Buffon - Jardin des Plantes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8426684661, 2.36185871065], 'Nom communes équipées': 'Paris'}}, 974: {'attributes': {'Identifiant station': 16019, 'Nom station': 'Porte de la Muette', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8630231784, 2.26871881702], 'Nom communes équipées': 'Paris'}}, 975: {'attributes': {'Identifiant station': 10021, 'Nom station': 'Marché Saint-Quentin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 15, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8773560577, 2.35451144564], 'Nom communes équipées': 'Paris'}}, 976: {'attributes': {'Identifiant station': 33012, 'Nom station': 'Landy - Heurtault', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9150965345, 2.37595137209], 'Nom communes équipées': 'Aubervilliers'}}, 977: {'attributes': {'Identifiant station': 4010, 'Nom station': 'Saint-Antoine Sévigné', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8550222, 2.3612322], 'Nom communes équipées': 'Paris'}}, 978: {'attributes': {'Identifiant station': 10019, 'Nom station': 'Paradis - Hauteville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8749999547, 2.35227574019], 'Nom communes équipées': 'Paris'}}, 979: {'attributes': {'Identifiant station': 21707, 'Nom station': 'Square Etienne Jarousse', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.824851826, 2.29259334505], 'Nom communes équipées': 'Vanves'}}, 980: {'attributes': {'Identifiant station': 13018, 'Nom station': 'Bruant - Vincent Auriol', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8348484529, 2.36682649963], 'Nom communes équipées': 'Paris'}}, 981: {'attributes': {'Identifiant station': 15129, 'Nom station': 'Hôpital Necker', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 29, 'Vélos mécaniques disponibles': 23, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8460434901, 2.31324501336], 'Nom communes équipées': 'Paris'}}, 982: {'attributes': {'Identifiant station': 14032, 'Nom station': 'Mairie du 14ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 14, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8325457812, 2.32549231499], 'Nom communes équipées': 'Paris'}}, 983: {'attributes': {'Identifiant station': 3102, 'Nom station': "Bourg l'Abbé - Saint-Martin", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 26, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8639333195, 2.3524062708], 'Nom communes équipées': 'Paris'}}, 984: {'attributes': {'Identifiant station': 14011, 'Nom station': "Place des Droits de l'Enfant", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8274804, 2.3319217], 'Nom communes équipées': 'Paris'}}, 985: {'attributes': {'Identifiant station': 15039, 'Nom station': 'Abbé Groult - Convention', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 12, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8359692787, 2.30206403951], 'Nom communes équipées': 'Paris'}}, 986: {'attributes': {'Identifiant station': 11033, 'Nom station': 'Bréguet - Sabin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8569316783, 2.37067494621], 'Nom communes équipées': 'Paris'}}, 987: {'attributes': {'Identifiant station': 21315, 'Nom station': 'Jules Guesdes - Général de Gaulle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8193058129, 2.27893419564], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 988: {'attributes': {'Identifiant station': 16005, 'Nom station': 'Raymond Poincaré - Place Victor Hugo', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8703061244, 2.28507454142], 'Nom communes équipées': 'Paris'}}, 989: {'attributes': {'Identifiant station': 6022, 'Nom station': 'Marché Saint-Germain', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8516562586, 2.33550030738], 'Nom communes équipées': 'Paris'}}, 990: {'attributes': {'Identifiant station': 15008, 'Nom station': 'Sèvres - Lecourbe', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 62, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 23, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8447677, 2.3110086], 'Nom communes équipées': 'Paris'}}, 991: {'attributes': {'Identifiant station': 15006, 'Nom station': 'Cherche-Midi - Hôpital Necker', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8446106331, 2.3179140687], 'Nom communes équipées': 'Paris'}}, 992: {'attributes': {'Identifiant station': 5007, 'Nom station': 'Ecoles - Carmes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 23, 'Vélos électriques disponibles': 11, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.848904448, 2.34734863043], 'Nom communes équipées': 'Paris'}}, 993: {'attributes': {'Identifiant station': 12101, 'Nom station': 'Charenton - Prague', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.84817513, 2.37635844943], 'Nom communes équipées': 'Paris'}}, 994: {'attributes': {'Identifiant station': 4006, 'Nom station': 'Bourdon - Pont Morland', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 8, 29, 9, 39, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8478103, 2.3659356], 'Nom communes équipées': 'Paris'}}, 995: {'attributes': {'Identifiant station': 21402, 'Nom station': 'Place Aimé Césaire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7831947131, 2.24291279912], 'Nom communes équipées': 'Clamart'}}, 996: {'attributes': {'Identifiant station': 12015, 'Nom station': 'Place de la Nation - Picpus', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8468216815, 2.40011304617], 'Nom communes équipées': 'Paris'}}, 997: {'attributes': {'Identifiant station': 9007, 'Nom station': 'Condorcet - Turgot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8798515579, 2.34523236752], 'Nom communes équipées': 'Paris'}}, 998: {'attributes': {'Identifiant station': 1020, 'Nom station': 'Cambon - Rivoli', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 33, 'Vélos mécaniques disponibles': 22, 'Vélos électriques disponibles': 11, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.866143391, 2.32518006471], 'Nom communes équipées': 'Paris'}}, 999: {'attributes': {'Identifiant station': 23102, 'Nom station': 'Hôtel de Ville de Sèvres', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8243635613, 2.21310473979], 'Nom communes équipées': 'Sèvres'}}, 1000: {'attributes': {'Identifiant station': 15001, 'Nom station': 'Gare Montparnasse - Arrivée', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8438279943, 2.32258111238], 'Nom communes équipées': 'Paris'}}, 1001: {'attributes': {'Identifiant station': 11001, 'Nom station': 'Richard Lenoir - Place de la Bastille', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 53, 'Nombre bornettes libres': 33, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8538327, 2.3697222], 'Nom communes équipées': 'Paris'}}, 1002: {'attributes': {'Identifiant station': 6010, 'Nom station': 'Chevreuse - Montparnasse', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 15, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8417391612, 2.33155221674], 'Nom communes équipées': 'Paris'}}, 1003: {'attributes': {'Identifiant station': 17024, 'Nom station': 'Villiers - Batignolles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8812525385, 2.31653843075], 'Nom communes équipées': 'Paris'}}, 1004: {'attributes': {'Identifiant station': 10032, 'Nom station': 'Claude Vellefaux - Hôpital Saint-Louis', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.872859139, 2.37007170699], 'Nom communes équipées': 'Paris'}}, 1005: {'attributes': {'Identifiant station': 33103, 'Nom station': 'Jean Jaurès - Jules Jacquemin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.884330536, 2.40829024552], 'Nom communes équipées': 'Le Pré-Saint-Gervais'}}, 1006: {'attributes': {'Identifiant station': 45002, 'Nom station': 'Roger Salengro - Général de Gaulle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8200290104, 2.48467605561], 'Nom communes équipées': 'Champigny-sur-Marne'}}, 1007: {'attributes': {'Identifiant station': 41401, 'Nom station': 'Marne - Charles de Gaulle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8150717195, 2.41216689348], 'Nom communes équipées': 'Alfortville'}}, 1008: {'attributes': {'Identifiant station': 16042, 'Nom station': 'Murat - Porte de Saint-Cloud', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8375086262, 2.25776519626], 'Nom communes équipées': 'Paris'}}, 1009: {'attributes': {'Identifiant station': 41203, 'Nom station': 'Stalingrad - Rigollots', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8499740384, 2.45557576418], 'Nom communes équipées': 'Fontenay-sous-Bois'}}, 1010: {'attributes': {'Identifiant station': 17019, 'Nom station': 'Phalsbourg - Villiers', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.882423079, 2.30904534459], 'Nom communes équipées': 'Paris'}}, 1011: {'attributes': {'Identifiant station': 17009, 'Nom station': 'Paul Bodin - Clichy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8923961462, 2.31714259833], 'Nom communes équipées': 'Paris'}}, 1012: {'attributes': {'Identifiant station': 16037, 'Nom station': 'Molitor - Michel-Ange', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 52, 'Nombre bornettes libres': 33, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8451820815, 2.26211277357], 'Nom communes équipées': 'Paris'}}, 1013: {'attributes': {'Identifiant station': 20022, 'Nom station': 'Porte de Bagnolet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8647071391, 2.40834463735], 'Nom communes équipées': 'Paris'}}, 1014: {'attributes': {'Identifiant station': 47003, 'Nom station': 'Charentonneau - Maisons Alfort', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8099329805, 2.4432884565], 'Nom communes équipées': 'Maisons-Alfort'}}, 1015: {'attributes': {'Identifiant station': 18026, 'Nom station': 'Ruisseau - Ordener', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8929946257, 2.34014488757], 'Nom communes équipées': 'Paris'}}, 1016: {'attributes': {'Identifiant station': 20034, 'Nom station': 'Sorbier - Ménilmontant', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8685423957, 2.38977212459], 'Nom communes équipées': 'Paris'}}, 1017: {'attributes': {'Identifiant station': 14026, 'Nom station': 'Plantes - Moulin Vert', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 43, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8300575531, 2.32323020697], 'Nom communes équipées': 'Paris'}}, 1018: {'attributes': {'Identifiant station': 7012, 'Nom station': 'Cité Vaneau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8542210666, 2.31961492449], 'Nom communes équipées': 'Paris'}}, 1019: {'attributes': {'Identifiant station': 16105, 'Nom station': 'Crevaux - Bugeaud', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8708094287, 2.28114900769], 'Nom communes équipées': 'Paris'}}, 1020: {'attributes': {'Identifiant station': 10038, 'Nom station': 'Place du Colonel Fabien', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8775683209, 2.36987918615], 'Nom communes équipées': 'Paris'}}, 1021: {'attributes': {'Identifiant station': 16012, 'Nom station': 'Flandrin - Longchamp', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 31, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 20, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8687784918, 2.27435453918], 'Nom communes équipées': 'Paris'}}, 1022: {'attributes': {'Identifiant station': 16102, 'Nom station': 'Pergolèse - Marbeau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8735687069, 2.28151252253], 'Nom communes équipées': 'Paris'}}, 1023: {'attributes': {'Identifiant station': 16036, 'Nom station': 'Porte Molitor', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 60, 'Nombre bornettes libres': 38, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 20, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8452476351, 2.25690152496], 'Nom communes équipées': 'Paris'}}, 1024: {'attributes': {'Identifiant station': 14135, 'Nom station': 'Jourdan - Cité Universitaire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 39, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8198973432, 2.33970079641], 'Nom communes équipées': 'Paris'}}, 1025: {'attributes': {'Identifiant station': 42505, 'Nom station': 'Charles Frérot - Albert Guilpin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8155839184, 2.35050290823], 'Nom communes équipées': 'Gentilly'}}, 1026: {'attributes': {'Identifiant station': 21309, 'Nom station': "Mairie d'Issy", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 41, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8237677925, 2.27231115103], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 1027: {'attributes': {'Identifiant station': 8117, 'Nom station': 'Place de la Madeleine - Royale', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 18, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8692280526, 2.32442110777], 'Nom communes équipées': 'Paris'}}, 1028: {'attributes': {'Identifiant station': 20024, 'Nom station': 'Gambetta - Père Lachaise', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8642956962, 2.39822866728], 'Nom communes équipées': 'Paris'}}, 1029: {'attributes': {'Identifiant station': 12036, 'Nom station': 'Reuilly - Place Felix Eboue', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 40, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.840302745, 2.39463039872], 'Nom communes équipées': 'Paris'}}, 1030: {'attributes': {'Identifiant station': 12156, 'Nom station': 'Niger - Soult', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8436990637, 2.41036123722], 'Nom communes équipées': 'Paris'}}, 1031: {'attributes': {'Identifiant station': 26004, 'Nom station': 'Gare RER les Grésillons', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9198986336, 2.31458459136], 'Nom communes équipées': 'Asnières-sur-Seine'}}, 1032: {'attributes': {'Identifiant station': 4108, 'Nom station': 'Quai de Béthune - Pont de la Tournelle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8509743319, 2.35630720854], 'Nom communes équipées': 'Paris'}}, 1033: {'attributes': {'Identifiant station': 18139, 'Nom station': 'Poissonniers - Championnet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 59, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8954329973, 2.35301781446], 'Nom communes équipées': 'Paris'}}, 1034: {'attributes': {'Identifiant station': 19002, 'Nom station': 'Centquatre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 40, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8893335385, 2.37078543752], 'Nom communes équipées': 'Paris'}}, 1035: {'attributes': {'Identifiant station': 1006, 'Nom station': 'Grande Truanderie - Saint-Denis', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 29, 'Vélos mécaniques disponibles': 22, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8626302, 2.3498075], 'Nom communes équipées': 'Paris'}}, 1036: {'attributes': {'Identifiant station': 9009, 'Nom station': 'Square Montholon', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8766203775, 2.34576411545], 'Nom communes équipées': 'Paris'}}, 1037: {'attributes': {'Identifiant station': 11112, 'Nom station': 'Jules Vallès - Charonne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8542843799, 2.38476745784], 'Nom communes équipées': 'Paris'}}, 1038: {'attributes': {'Identifiant station': 14103, 'Nom station': 'Maine - Liancourt', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.835064364, 2.32373144997], 'Nom communes équipées': 'Paris'}}, 1039: {'attributes': {'Identifiant station': 19110, 'Nom station': 'Square Claude Bernard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8987601622, 2.37532574683], 'Nom communes équipées': 'Paris'}}, 1040: {'attributes': {'Identifiant station': 3010, 'Nom station': 'Beaubourg - Michel-le-Comte', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 2, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8625349418, 2.35431030393], 'Nom communes équipées': 'Paris'}}, 1041: {'attributes': {'Identifiant station': 22201, 'Nom station': 'Henri Ravera - Gabriel Péri', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7972227383, 2.3021158576], 'Nom communes équipées': 'Bagneux'}}, 1042: {'attributes': {'Identifiant station': 9037, 'Nom station': 'Parme - Clichy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8812664242, 2.32819709345], 'Nom communes équipées': 'Paris'}}, 1043: {'attributes': {'Identifiant station': 3008, 'Nom station': 'Perle - Vieille du Temple', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8601346, 2.3611206], 'Nom communes équipées': 'Paris'}}, 1044: {'attributes': {'Identifiant station': 13005, 'Nom station': 'Cordelières - Arago', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.835480719, 2.3485724617], 'Nom communes équipées': 'Paris'}}, 1045: {'attributes': {'Identifiant station': 5015, 'Nom station': 'Epée de Bois - Mouffetard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 63, 'Nombre bornettes libres': 52, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8415086904, 2.35006671399], 'Nom communes équipées': 'Paris'}}, 1046: {'attributes': {'Identifiant station': 19010, 'Nom station': 'Emile Reynaud - Porte de la Villette', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9009895363, 2.38870225847], 'Nom communes équipées': 'Paris'}}, 1047: {'attributes': {'Identifiant station': 48004, 'Nom station': 'Maxime Gorki - Louis Aragon', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7890981473, 2.36838787794], 'Nom communes équipées': 'Villejuif'}}, 1048: {'attributes': {'Identifiant station': 15132, 'Nom station': 'Bargue -  Dutot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.837884, 2.308876], 'Nom communes équipées': 'Paris'}}, 1049: {'attributes': {'Identifiant station': 15128, 'Nom station': 'Cadix - Hameau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8340107632, 2.28951618075], 'Nom communes équipées': 'Paris'}}, 1050: {'attributes': {'Identifiant station': 41405, 'Nom station': "Place de l'Europe - Alfortville", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7891370241, 2.42848008871], 'Nom communes équipées': 'Alfortville'}}, 1051: {'attributes': {'Identifiant station': 44014, 'Nom station': 'Camille Risch - Paul Armangot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.774703, 2.373685], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 1052: {'attributes': {'Identifiant station': 20006, 'Nom station': 'Pyrénées - Plaine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 12, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8502670246, 2.4062821269], 'Nom communes équipées': 'Paris'}}, 1053: {'attributes': {'Identifiant station': 12037, 'Nom station': 'Reuilly - Daumesnil', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8395385, 2.3972516], 'Nom communes équipées': 'Paris'}}, 1054: {'attributes': {'Identifiant station': 21026, 'Nom station': 'Sèvres - Général Leclerc', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 27, 'Vélos mécaniques disponibles': 26, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8301215591, 2.23142623901], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 1055: {'attributes': {'Identifiant station': 18004, 'Nom station': 'Place des Abbesses', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 12, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8845427261, 2.33868677169], 'Nom communes équipées': 'Paris'}}, 1056: {'attributes': {'Identifiant station': 32013, 'Nom station': 'Saint-Denis - Université', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9456343095, 2.363829799], 'Nom communes équipées': 'Saint-Denis'}}, 1057: {'attributes': {'Identifiant station': 16013, 'Nom station': 'Mairie du 16ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8639368485, 2.27659530938], 'Nom communes équipées': 'Paris'}}, 1058: {'attributes': {'Identifiant station': 7021, 'Nom station': 'Bosquet - Saint-Dominique', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 19, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8586517095, 2.30371658023], 'Nom communes équipées': 'Paris'}}, 1059: {'attributes': {'Identifiant station': 20008, 'Nom station': 'Pyrénées - Avron', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8532468982, 2.40587104453], 'Nom communes équipées': 'Paris'}}, 1060: {'attributes': {'Identifiant station': 16103, 'Nom station': 'Traktir - Foch', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8727553346, 2.29154978025], 'Nom communes équipées': 'Paris'}}, 1061: {'attributes': {'Identifiant station': 18034, 'Nom station': 'Leibniz - Jean Dollfus', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8963283485, 2.33339042383], 'Nom communes équipées': 'Paris'}}, 1062: {'attributes': {'Identifiant station': 11037, 'Nom station': 'Faubourg Du Temple - Republique', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8678724847, 2.36489821581], 'Nom communes équipées': 'Paris'}}, 1063: {'attributes': {'Identifiant station': 6104, 'Nom station': 'Herschel - Observatoire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8434237622, 2.33766451478], 'Nom communes équipées': 'Paris'}}, 1064: {'attributes': {'Identifiant station': 31709, 'Nom station': 'Place de la Fraternité', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 18, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8571640584, 2.42284744978], 'Nom communes équipées': 'Montreuil'}}, 1065: {'attributes': {'Identifiant station': 10034, 'Nom station': 'Louis Blanc - La Chapelle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8840089645, 2.35993355513], 'Nom communes équipées': 'Paris'}}, 1066: {'attributes': {'Identifiant station': 10006, 'Nom station': 'Petites Ecuries - Faubourg Saint-Denis', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8728712059, 2.35421508551], 'Nom communes équipées': 'Paris'}}, 1067: {'attributes': {'Identifiant station': 12019, 'Nom station': 'Gare de Reuilly - Picpus', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8426735412, 2.39729437977], 'Nom communes équipées': 'Paris'}}, 1068: {'attributes': {'Identifiant station': 8028, 'Nom station': 'Arsène Houssaye - Champs-Elysées', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8734092763, 2.29779046029], 'Nom communes équipées': 'Paris'}}, 1069: {'attributes': {'Identifiant station': 20104, 'Nom station': 'Louis Lumière.', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8621619807, 2.41112578186], 'Nom communes équipées': 'Paris'}}, 1070: {'attributes': {'Identifiant station': 10011, 'Nom station': 'Bourse du Travail', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 3, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8689766388, 2.3623438552], 'Nom communes équipées': 'Paris'}}, 1071: {'attributes': {'Identifiant station': 20001, 'Nom station': 'Place de la Nation - Charonne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 33, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8483646678, 2.39976238598], 'Nom communes équipées': 'Paris'}}, 1072: {'attributes': {'Identifiant station': 20029, 'Nom station': 'Frères Flavien - Porte des Lilas', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8780992764, 2.41104629853], 'Nom communes équipées': 'Paris'}}, 1073: {'attributes': {'Identifiant station': 18140, 'Nom station': 'Porte de Clignancourt - Ney', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.899641124, 2.34432142228], 'Nom communes équipées': 'Paris'}}, 1074: {'attributes': {'Identifiant station': 2003, 'Nom station': 'Saint-Denis - Réaumur', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8661116316, 2.35077906755], 'Nom communes équipées': 'Paris'}}, 1075: {'attributes': {'Identifiant station': 8110, 'Nom station': 'Gare Saint-Lazare - Stockholm', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8771273167, 2.32241716236], 'Nom communes équipées': 'Paris'}}, 1076: {'attributes': {'Identifiant station': 1102, 'Nom station': 'Française - Etienne Marcel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 33, 'Vélos mécaniques disponibles': 25, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8640533047, 2.34758701175], 'Nom communes équipées': 'Paris'}}, 1077: {'attributes': {'Identifiant station': 1007, 'Nom station': 'Turbigo - Française', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 43, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8635885811, 2.34777946025], 'Nom communes équipées': 'Paris'}}, 1078: {'attributes': {'Identifiant station': 5122, 'Nom station': 'Saint-Michel - Luxembourg', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 3, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8450150763, 2.3394941166], 'Nom communes équipées': 'Paris'}}, 1079: {'attributes': {'Identifiant station': 13051, 'Nom station': 'Halle Freyssinet - Parvis Alan Turing', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 12, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8328468145, 2.37114217132], 'Nom communes équipées': 'Paris'}}, 1080: {'attributes': {'Identifiant station': 13017, 'Nom station': 'Nationale - Vincent Auriol', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.832527084, 2.36235441039], 'Nom communes équipées': 'Paris'}}, 1081: {'attributes': {'Identifiant station': 19201, 'Nom station': 'Petit - Georges Thill', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 3, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.88603132, 2.3895753175], 'Nom communes équipées': 'Paris'}}, 1082: {'attributes': {'Identifiant station': 2002, 'Nom station': 'Etienne Marcel - Montorgueil', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 24, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8645056726, 2.3461959511], 'Nom communes équipées': 'Paris'}}, 1083: {'attributes': {'Identifiant station': 22301, 'Nom station': 'Chausson - Gabriel Péri', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9218910035, 2.29523062706], 'Nom communes équipées': 'Gennevilliers'}}, 1084: {'attributes': {'Identifiant station': 15111, 'Nom station': 'Olivier de Serres - Leriche', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8346022172, 2.2957482934], 'Nom communes équipées': 'Paris'}}, 1085: {'attributes': {'Identifiant station': 10009, 'Nom station': 'Mairie du 10ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8720894499, 2.35758259892], 'Nom communes équipées': 'Paris'}}, 1086: {'attributes': {'Identifiant station': 3005, 'Nom station': 'Sainte-Elisabeth - Turbigo', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 43, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8661033743, 2.35964924097], 'Nom communes équipées': 'Paris'}}, 1087: {'attributes': {'Identifiant station': 2015, 'Nom station': 'Louis le Grand - Italiens', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8705089372, 2.33405446194], 'Nom communes équipées': 'Paris'}}, 1088: {'attributes': {'Identifiant station': 19129, 'Nom station': 'Solitaires - Place des Fêtes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.876601, 2.391769], 'Nom communes équipées': 'Paris'}}, 1089: {'attributes': {'Identifiant station': 12002, 'Nom station': 'Lacuée - Lyon', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.849268482, 2.3704585433], 'Nom communes équipées': 'Paris'}}, 1090: {'attributes': {'Identifiant station': 2021, 'Nom station': 'Cléry - Montmartre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 44, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 38, 'Vélos mécaniques disponibles': 32, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.867458, 2.3445211], 'Nom communes équipées': 'Paris'}}, 1091: {'attributes': {'Identifiant station': 21953, 'Nom station': 'Gare de Bellevue', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8192404686, 2.22953092307], 'Nom communes équipées': 'Meudon'}}, 1092: {'attributes': {'Identifiant station': 24006, 'Nom station': 'Adolphe Lalyre - Armand Silvestre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9040626696, 2.27341622114], 'Nom communes équipées': 'Courbevoie'}}, 1093: {'attributes': {'Identifiant station': 3001, 'Nom station': 'Beaumarchais - Saint-Gilles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 51, 'Nombre bornettes libres': 44, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8580378, 2.3679352], 'Nom communes équipées': 'Paris'}}, 1094: {'attributes': {'Identifiant station': 26005, 'Nom station': "Hôtel de Ville d'Asnières sur Seine", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9099933003, 2.28823810816], 'Nom communes équipées': 'Asnières-sur-Seine'}}, 1095: {'attributes': {'Identifiant station': 12112, 'Nom station': 'Charenton - Jardiniers', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8338217729, 2.39469266411], 'Nom communes équipées': 'Paris'}}, 1096: {'attributes': {'Identifiant station': 3012, 'Nom station': 'Square Emile - Chautemps', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 43, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8674337242, 2.35363572836], 'Nom communes équipées': 'Paris'}}, 1097: {'attributes': {'Identifiant station': 3011, 'Nom station': 'Turbigo - Réaumur', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 22, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8655740475, 2.35641986132], 'Nom communes équipées': 'Paris'}}, 1098: {'attributes': {'Identifiant station': 14022, 'Nom station': 'Didot - Mariniers', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8265054361, 2.31313619827], 'Nom communes équipées': 'Paris'}}, 1099: {'attributes': {'Identifiant station': 10027, 'Nom station': 'Dunkerque - Alsace', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.879355, 2.358379], 'Nom communes équipées': 'Paris'}}, 1100: {'attributes': {'Identifiant station': 19012, 'Nom station': 'Lally-Tollendal - Armand Carrel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8827006622, 2.37471755594], 'Nom communes équipées': 'Paris'}}, 1101: {'attributes': {'Identifiant station': 21506, 'Nom station': 'Maurice Payret-Dortail', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8781195706, 2.22336251289], 'Nom communes équipées': 'Suresnes'}}, 1102: {'attributes': {'Identifiant station': 31001, 'Nom station': 'Lagny - Auguste Blanqui', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8492256805, 2.42186039686], 'Nom communes équipées': 'Montreuil'}}, 1103: {'attributes': {'Identifiant station': 4103, 'Nom station': 'Archives - Rivoli', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8573467074, 2.35397670418], 'Nom communes équipées': 'Paris'}}, 1104: {'attributes': {'Identifiant station': 14023, 'Nom station': 'Adolphe Pinard - Victor Hugo', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 52, 'Nombre bornettes libres': 49, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8236825875, 2.30778437108], 'Nom communes équipées': 'Malakoff'}}, 1105: {'attributes': {'Identifiant station': 22503, 'Nom station': 'Joffre - Aigle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.904033, 2.245096], 'Nom communes équipées': 'La Garenne-Colombes'}}, 1106: {'attributes': {'Identifiant station': 22502, 'Nom station': 'Kléber - National', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9063338987, 2.23939105868], 'Nom communes équipées': 'La Garenne-Colombes'}}, 1107: {'attributes': {'Identifiant station': 12020, 'Nom station': 'Cours de Vincennes - Soult', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 41, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8469187616, 2.4105220288], 'Nom communes équipées': 'Paris'}}, 1108: {'attributes': {'Identifiant station': 9111, 'Nom station': 'Victoire - Laffitte', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8751437349, 2.33811850514], 'Nom communes équipées': 'Paris'}}, 1109: {'attributes': {'Identifiant station': 2005, 'Nom station': 'Montmartre - Etienne Marcel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8650219458, 2.34506844014], 'Nom communes équipées': 'Paris'}}, 1110: {'attributes': {'Identifiant station': 48005, 'Nom station': 'Eugène Varlin - Place de la Paix', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.792862, 2.363093], 'Nom communes équipées': 'Villejuif'}}, 1111: {'attributes': {'Identifiant station': 3009, 'Nom station': 'Temple - Haudriettes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8616368802, 2.35669602065], 'Nom communes équipées': 'Paris'}}, 1112: {'attributes': {'Identifiant station': 41208, 'Nom station': 'Alouettes - Maréchal de Lattre de Tassigny', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8551100929, 2.49307002872], 'Nom communes équipées': 'Fontenay-sous-Bois'}}, 1113: {'attributes': {'Identifiant station': 14132, 'Nom station': 'Bézout - Général Leclerc', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.828848045, 2.32859730721], 'Nom communes équipées': 'Paris'}}, 1114: {'attributes': {'Identifiant station': 3003, 'Nom station': 'Fille du Calvaire - Turenne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 24, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 11, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8621050541, 2.36496001482], 'Nom communes équipées': 'Paris'}}, 1115: {'attributes': {'Identifiant station': 4014, 'Nom station': 'Blancs-Manteaux - Archives', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.859418007, 2.35594712198], 'Nom communes équipées': 'Paris'}}, 1116: {'attributes': {'Identifiant station': 4017, 'Nom station': "Place de l'Hôtel de Ville", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 14, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 5, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8573314, 2.351458], 'Nom communes équipées': 'Paris'}}, 1117: {'attributes': {'Identifiant station': 23702, 'Nom station': 'Hôtel de Ville de Chaville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8088714909, 2.18813404441], 'Nom communes équipées': 'Chaville'}}, 1118: {'attributes': {'Identifiant station': 10041, 'Nom station': 'Gare du Nord - La Chapelle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 14, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8841782873, 2.35610570759], 'Nom communes équipées': 'Paris'}}, 1119: {'attributes': {'Identifiant station': 22306, 'Nom station': 'Place du 17 Octobre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9196269003, 2.29782861342], 'Nom communes équipées': 'Gennevilliers'}}, 1120: {'attributes': {'Identifiant station': 4009, 'Nom station': 'Village Saint-Paul', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8526398666, 2.3607301712], 'Nom communes équipées': 'Paris'}}, 1121: {'attributes': {'Identifiant station': 13125, 'Nom station': "Gare d'Austerlitz - Pont Charles de Gaulle", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 55, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 43, 'Vélos mécaniques disponibles': 36, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 43, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8423198301, 2.36703000963], 'Nom communes équipées': 'Paris'}}, 1122: {'attributes': {'Identifiant station': 4015, 'Nom station': 'Mairie du 4ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 44, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 23, 'Vélos électriques disponibles': 11, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8559499105, 2.35638231039], 'Nom communes équipées': 'Paris'}}, 1123: {'attributes': {'Identifiant station': 21507, 'Nom station': 'Gardenat-Lapostol - Velettes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.87387, 2.224185], 'Nom communes équipées': 'Suresnes'}}, 1124: {'attributes': {'Identifiant station': 12163, 'Nom station': 'Hippodrome Paris-Vincennes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8204838776, 2.44627118111], 'Nom communes équipées': 'Paris'}}, 1125: {'attributes': {'Identifiant station': 3006, 'Nom station': 'Mairie du 3ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8644617814, 2.36176282167], 'Nom communes équipées': 'Paris'}}, 1126: {'attributes': {'Identifiant station': 3004, 'Nom station': 'Place de la République - Temple', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8672112019, 2.36354643705], 'Nom communes équipées': 'Paris'}}, 1127: {'attributes': {'Identifiant station': 15062, 'Nom station': 'Convention - Lourmel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 27, 'Vélos mécaniques disponibles': 23, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8420550417, 2.28599108756], 'Nom communes équipées': 'Paris'}}, 1128: {'attributes': {'Identifiant station': 3002, 'Nom station': 'Saint-Gilles - Turenne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8581589568, 2.3647149921], 'Nom communes équipées': 'Paris'}}, 1129: {'attributes': {'Identifiant station': 18136, 'Nom station': 'Poulet - Barbès', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.887148, 2.349064], 'Nom communes équipées': 'Paris'}}, 1130: {'attributes': {'Identifiant station': 4101, 'Nom station': 'Bastille - Beaumarchais', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8538334639, 2.36857026815], 'Nom communes équipées': 'Paris'}}, 1131: {'attributes': {'Identifiant station': 22504, 'Nom station': 'Château - République', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.905928, 2.253629], 'Nom communes équipées': 'La Garenne-Colombes'}}, 1132: {'attributes': {'Identifiant station': 1012, 'Nom station': 'Jean-Jacques Rousseau - Deux Ecus', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8629780597, 2.34146352857], 'Nom communes équipées': 'Paris'}}, 1133: {'attributes': {'Identifiant station': 1014, 'Nom station': 'Rivoli - Musée du Louvre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8636463685, 2.33409583569], 'Nom communes équipées': 'Paris'}}, 1134: {'attributes': {'Identifiant station': 19130, 'Nom station': 'Place Paul Delouvrier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8919762143, 2.38512888551], 'Nom communes équipées': 'Paris'}}, 1135: {'attributes': {'Identifiant station': 14133, 'Nom station': 'Froidevaux - Boulard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8351473196, 2.32944119722], 'Nom communes équipées': 'Paris'}}, 1136: {'attributes': {'Identifiant station': 35012, 'Nom station': 'Candale - Mehul', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8893935006, 2.41437971592], 'Nom communes équipées': 'Pantin'}}, 1137: {'attributes': {'Identifiant station': 42211, 'Nom station': 'Séminaire de Conflans - Liberté', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.823194, 2.404179], 'Nom communes équipées': 'Charenton-le-Pont'}}, 1138: {'attributes': {'Identifiant station': 31104, 'Nom station': 'Mairie de Rosny-sous-Bois', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.871256519, 2.48658075929], 'Nom communes équipées': 'Rosny-sous-Bois'}}, 1139: {'attributes': {'Identifiant station': 21215, 'Nom station': 'Verdier - Jean Jaurès', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8161404871, 2.31235619634], 'Nom communes équipées': 'Montrouge'}}, 1140: {'attributes': {'Identifiant station': 22506, 'Nom station': 'Gare des Vallées', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.913179, 2.257726], 'Nom communes équipées': 'La Garenne-Colombes'}}, 1141: {'attributes': {'Identifiant station': 42704, 'Nom station': 'Mairie du Kremlin-Bicêtre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8119198154, 2.35701899976], 'Nom communes équipées': 'Le Kremlin-Bicêtre'}}, 1142: {'attributes': {'Identifiant station': 13038, 'Nom station': 'Choisy - Malmaisons', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.821919703, 2.36334688341], 'Nom communes équipées': 'Paris'}}, 1143: {'attributes': {'Identifiant station': 4012, 'Nom station': 'Tiron - Rivoli', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8556922532, 2.35790580511], 'Nom communes équipées': 'Paris'}}, 1144: {'attributes': {'Identifiant station': 1025, 'Nom station': 'Oratoire - Rivoli', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8613427609, 2.34000474215], 'Nom communes équipées': 'Paris'}}, 1145: {'attributes': {'Identifiant station': 26006, 'Nom station': 'Bas - Gabriel Péri', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9158558054, 2.29480952024], 'Nom communes équipées': 'Asnières-sur-Seine'}}, 1146: {'attributes': {'Identifiant station': 4018, 'Nom station': 'Saint-Bon - Rivoli', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8579996433, 2.35021728098], 'Nom communes équipées': 'Paris'}}, 1147: {'attributes': {'Identifiant station': 42026, 'Nom station': 'Barbès - Monique Maunoury', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 5, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8155271882, 2.36871693349], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 1148: {'attributes': {'Identifiant station': 32019, 'Nom station': 'Place du 8 mai 1945', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9388822721, 2.35662437975], 'Nom communes équipées': 'Saint-Denis'}}, 1149: {'attributes': {'Identifiant station': 2012, 'Nom station': 'Choiseul - Quatre Septembre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 44, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 40, 'Vélos mécaniques disponibles': 23, 'Vélos électriques disponibles': 17, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8699355332, 2.33599752188], 'Nom communes équipées': 'Paris'}}, 1150: {'attributes': {'Identifiant station': 21954, 'Nom station': 'Val-Fleury', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.807671408, 2.24072176963], 'Nom communes équipées': 'Meudon'}}, 1151: {'attributes': {'Identifiant station': 16044, 'Nom station': 'Porte de Saint Cloud - Parc des Princes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 24, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8386627571, 2.2525164485], 'Nom communes équipées': 'Paris'}}, 1152: {'attributes': {'Identifiant station': 1018, 'Nom station': 'Alger - Rivoli', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8647735535, 2.32944939361], 'Nom communes équipées': 'Paris'}}, 1153: {'attributes': {'Identifiant station': 44001, 'Nom station': 'Quai Jules Guesde - Saint-Simon', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 29, 'Vélos mécaniques disponibles': 25, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8039148722, 2.40896616203], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 1154: {'attributes': {'Identifiant station': 15040, 'Nom station': 'Saint-Amand - Labrouste', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8340972748, 2.30876773596], 'Nom communes équipées': 'Paris'}}, 1155: {'attributes': {'Identifiant station': 19111, 'Nom station': 'Cesaria Evora - Rosa Parks', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 40, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 1, 3, 11, 28, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8971767283, 2.37451739609], 'Nom communes équipées': 'Paris'}}, 1156: {'attributes': {'Identifiant station': 13054, 'Nom station': 'Pau Casals - Neuve Tolbiac', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 53, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 49, 'Vélos mécaniques disponibles': 33, 'Vélos électriques disponibles': 16, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8311602, 2.3774395], 'Nom communes équipées': 'Paris'}}, 1157: {'attributes': {'Identifiant station': 21509, 'Nom station': 'Gardenat Lapostol - Franklin Roosevelt', 'Station en fonctionnement': 'NON', 'Capacité de la station': 24, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.871839, 2.222505], 'Nom communes équipées': 'Suresnes'}}, 1158: {'attributes': {'Identifiant station': 14134, 'Nom station': 'Château - Place de Catalogne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8368055423, 2.31852930039], 'Nom communes équipées': 'Paris'}}, 1159: {'attributes': {'Identifiant station': 4007, 'Nom station': 'Place de la Bastille - Bourdon', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8523643553, 2.3681089282], 'Nom communes équipées': 'Paris'}}, 1160: {'attributes': {'Identifiant station': 12165, 'Nom station': 'Champs de Manoeuvre - Cartoucherie', 'Station en fonctionnement': 'NON', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2019, 3, 21, 10, 50, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.835261, 2.447974], 'Nom communes équipées': 'Paris'}}, 1161: {'attributes': {'Identifiant station': 2004, 'Nom station': 'Square Pierre Lazareff', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 15, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8671417207, 2.34839335084], 'Nom communes équipées': 'Paris'}}, 1162: {'attributes': {'Identifiant station': 6026, 'Nom station': 'Saint-Placide - Cherche Midi', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 66, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 41, 'Vélos mécaniques disponibles': 37, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8490535919, 2.32531797141], 'Nom communes équipées': 'Paris'}}, 1163: {'attributes': {'Identifiant station': 18015, 'Nom station': 'Custine - Ramey', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 41, 'Nombre bornettes libres': 39, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8887358969, 2.34695266932], 'Nom communes équipées': 'Paris'}}, 1164: {'attributes': {'Identifiant station': 1023, 'Nom station': 'Saint-Honoré - Musée du Louvre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8634431328, 2.33487818464], 'Nom communes équipées': 'Paris'}}, 1165: {'attributes': {'Identifiant station': 21505, 'Nom station': 'Édouard Vaillant - Place Jean Jaurès', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.86265538, 2.20890373191], 'Nom communes équipées': 'Suresnes'}}, 1166: {'attributes': {'Identifiant station': 48008, 'Nom station': 'Guy Môquet - Parvis Georges Marchais', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.795188401, 2.35943466425], 'Nom communes équipées': 'Villejuif'}}, 1167: {'attributes': {'Identifiant station': 2016, 'Nom station': 'Aboukir - Alexandrie', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8686834444, 2.35020432695], 'Nom communes équipées': 'Paris'}}, 1168: {'attributes': {'Identifiant station': 5003, 'Nom station': 'Le Goff - Gay-Lussac', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.846102401, 2.34122078866], 'Nom communes équipées': 'Paris'}}, 1169: {'attributes': {'Identifiant station': 9039, 'Nom station': "Chaussée d'Antin - Haussmann", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.87243, 2.33359], 'Nom communes équipées': 'Paris'}}, 1170: {'attributes': {'Identifiant station': 6007, 'Nom station': 'Bréa - Raspail', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8427384783, 2.32979422176], 'Nom communes équipées': 'Paris'}}, 1171: {'attributes': {'Identifiant station': 33020, 'Nom station': 'Waldeck Rochet - Germaine Tillon', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.911059602, 2.36763015389], 'Nom communes équipées': 'Aubervilliers'}}, 1172: {'attributes': {'Identifiant station': 3007, 'Nom station': 'Archives - Pastourelle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 26, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8623615, 2.3595041], 'Nom communes équipées': 'Paris'}}, 1173: {'attributes': {'Identifiant station': 2008, 'Nom station': 'Mairie du 2ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8672190355, 2.34046254377], 'Nom communes équipées': 'Paris'}}, 1174: {'attributes': {'Identifiant station': 31020, 'Nom station': 'Résistance - Parmentier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8596513813, 2.43670508265], 'Nom communes équipées': 'Montreuil'}}, 1175: {'attributes': {'Identifiant station': 1026, 'Nom station': 'Place du Lieutenant Henri Karcher', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 20, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8634908717, 2.34006710351], 'Nom communes équipées': 'Paris'}}, 1176: {'attributes': {'Identifiant station': 16139, 'Nom station': 'Suresnes - Muette à Neuilly', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8685704043, 2.26600091904], 'Nom communes équipées': 'Paris'}}, 1177: {'attributes': {'Identifiant station': 23101, 'Nom station': 'Grande Rue - Europe', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8218104688, 2.20587987453], 'Nom communes équipées': 'Sèvres'}}, 1178: {'attributes': {'Identifiant station': 27006, 'Nom station': 'Victor Hugo - Gare', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9231339527, 2.26009704173], 'Nom communes équipées': 'Colombes'}}, 1179: {'attributes': {'Identifiant station': 2009, 'Nom station': 'Filles Saint-Thomas - Place de la Bourse', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 58, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 48, 'Vélos mécaniques disponibles': 36, 'Vélos électriques disponibles': 12, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8688998909, 2.33975438452], 'Nom communes équipées': 'Paris'}}, 1180: {'attributes': {'Identifiant station': 42028, 'Nom station': 'Verdun - Carnot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8084393761, 2.37379521132], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 1181: {'attributes': {'Identifiant station': 1116, 'Nom station': 'Ventadour - Opéra', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 18, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8668106533, 2.33438844825], 'Nom communes équipées': 'Paris'}}, 1182: {'attributes': {'Identifiant station': 12166, 'Nom station': 'Parc Floral du Bel Air', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8392857119, 2.43856016546], 'Nom communes équipées': 'Paris'}}, 1183: {'attributes': {'Identifiant station': 27001, 'Nom station': 'Square Victor Basch', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9143048623, 2.22891569138], 'Nom communes équipées': 'Colombes'}}, 1184: {'attributes': {'Identifiant station': 1013, 'Nom station': 'Place du Palais Royal', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 67, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 62, 'Vélos mécaniques disponibles': 48, 'Vélos électriques disponibles': 14, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8623829706, 2.33848225325], 'Nom communes équipées': 'Paris'}}, 1185: {'attributes': {'Identifiant station': 12157, 'Nom station': "Fabre d'Eglantine - Place de la Nation", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 68, 'Nombre bornettes libres': 59, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8471116, 2.3956204], 'Nom communes équipées': 'Paris'}}, 1186: {'attributes': {'Identifiant station': 22603, 'Nom station': 'Place de la Division Leclerc', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 35, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7938249886, 2.27311404306], 'Nom communes équipées': 'Fontenay-aux-Roses'}}, 1187: {'attributes': {'Identifiant station': 12164, 'Nom station': 'Stade Pershing', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 20, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8291078269, 2.45835352689], 'Nom communes équipées': 'Paris'}}, 1188: {'attributes': {'Identifiant station': 2102, 'Nom station': 'Saint-Marc - Feydeau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 36, 'Vélos mécaniques disponibles': 20, 'Vélos électriques disponibles': 16, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8703106614, 2.34197918326], 'Nom communes équipées': 'Paris'}}, 1189: {'attributes': {'Identifiant station': 1017, 'Nom station': 'Saint-Honoré - 29 juillet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8655711428, 2.33056741758], 'Nom communes équipées': 'Paris'}}, 1190: {'attributes': {'Identifiant station': 2006, 'Nom station': 'Place des Victoires', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8660354, 2.3420333], 'Nom communes équipées': 'Paris'}}, 1191: {'attributes': {'Identifiant station': 17106, 'Nom station': 'Berthier - Porte de Clichy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8942226878, 2.31258720068], 'Nom communes équipées': 'Paris'}}, 1192: {'attributes': {'Identifiant station': 21119, 'Nom station': 'Belfort - Victor Hugo', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9018492, 2.3172724], 'Nom communes équipées': 'Clichy'}}, 1193: {'attributes': {'Identifiant station': 17021, 'Nom station': "Jouffroy d'Abbans - Villiers", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.884215547, 2.30565335602], 'Nom communes équipées': 'Paris'}}, 1194: {'attributes': {'Identifiant station': 9019, 'Nom station': 'Victor Massé - Jean-Baptiste Pigalle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8811043724, 2.33669556677], 'Nom communes équipées': 'Paris'}}, 1195: {'attributes': {'Identifiant station': 22601, 'Nom station': 'Marx Dormoy - Château Sainte-Barbe', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7917823411, 2.29785684496], 'Nom communes équipées': 'Fontenay-aux-Roses'}}, 1196: {'attributes': {'Identifiant station': 23042, 'Nom station': 'André Theuriet - Blagis', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7797732575, 2.31305256486], 'Nom communes équipées': 'Bourg-la-Reine'}}, 1197: {'attributes': {'Identifiant station': 42018, 'Nom station': "Amédée Huon - Fort d'Ivry", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8051313769, 2.3884896934], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 1198: {'attributes': {'Identifiant station': 2109, 'Nom station': 'Réaumur - Montmartre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8682309998, 2.34382603389], 'Nom communes équipées': 'Paris'}}, 1199: {'attributes': {'Identifiant station': 28005, 'Nom station': 'Jean Jaurès - Paul Lafargue', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8849331777, 2.24844887853], 'Nom communes équipées': 'Puteaux'}}, 1200: {'attributes': {'Identifiant station': 44004, 'Nom station': "Place de l'Eglise - Vitry-sur-Seine", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7919228199, 2.39505775273], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 1201: {'attributes': {'Identifiant station': 35005, 'Nom station': "Quai de l'Aisne - Général Leclerc", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.895713, 2.400571], 'Nom communes équipées': 'Pantin'}}, 1202: {'attributes': {'Identifiant station': 19118, 'Nom station': 'Simon Bolivar - Henri Turot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 5, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.876887, 2.374454], 'Nom communes équipées': 'Paris'}}, 1203: {'attributes': {'Identifiant station': 18112, 'Nom station': 'Arthur Ranc - Hôpital Bichat', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8978712501, 2.33352184296], 'Nom communes équipées': 'Paris'}}, 1204: {'attributes': {'Identifiant station': 10166, 'Nom station': 'Strasbourg - Saint-Denis', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8696016424, 2.35429085791], 'Nom communes équipées': 'Paris'}}, 1205: {'attributes': {'Identifiant station': 10201, 'Nom station': 'Magenta - Chabrol', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8764953509, 2.35609900206], 'Nom communes équipées': 'Paris'}}, 1206: {'attributes': {'Identifiant station': 44008, 'Nom station': 'Youri Gagarine - Commune de Paris', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 29, 'Vélos mécaniques disponibles': 26, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.782427463, 2.39397346973], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 1207: {'attributes': {'Identifiant station': 48007, 'Nom station': 'Parc Départemental des Hautes-Bruyères', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7934579014, 2.35090121627], 'Nom communes équipées': 'Villejuif'}}, 1208: {'attributes': {'Identifiant station': 20140, 'Nom station': 'Paganini - Davout', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.85135, 2.412021], 'Nom communes équipées': 'Paris'}}, 1209: {'attributes': {'Identifiant station': 12113, 'Nom station': 'Cardinal Lavigerie', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 33, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8329341937, 2.4026367494], 'Nom communes équipées': 'Paris'}}, 1210: {'attributes': {'Identifiant station': 45004, 'Nom station': 'République - Louis Loucheur', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 24, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 11, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8235492612, 2.51018654555], 'Nom communes équipées': 'Champigny-sur-Marne'}}, 1211: {'attributes': {'Identifiant station': 1015, 'Nom station': 'Place André Malraux', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 47, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8639251587, 2.33562268317], 'Nom communes équipées': 'Paris'}}, 1212: {'attributes': {'Identifiant station': 23202, 'Nom station': 'Henri Barbusse - Gabriel Péri', 'Station en fonctionnement': 'NON', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 7, 27, 14, 53, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8017737958, 2.28741663622], 'Nom communes équipées': 'Châtillon'}}, 1213: {'attributes': {'Identifiant station': 19046, 'Nom station': 'Porte de Pantin.', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8891997256, 2.39501819015], 'Nom communes équipées': 'Paris'}}, 1214: {'attributes': {'Identifiant station': 2022, 'Nom station': 'Saint-Fiacre - Poissonière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 20, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 14, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8708289116, 2.34592135996], 'Nom communes équipées': 'Paris'}}, 1215: {'attributes': {'Identifiant station': 16010, 'Nom station': 'Belles Feuilles - Place de Mexico', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8658047465, 2.28292062879], 'Nom communes équipées': 'Paris'}}, 1216: {'attributes': {'Identifiant station': 17020, 'Nom station': "Jouffroy d'Abbans - Malesherbes", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8850134, 2.3070717], 'Nom communes équipées': 'Paris'}}, 1217: {'attributes': {'Identifiant station': 42019, 'Nom station': 'Lampes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 43, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 33, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8115248199, 2.40179311484], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 1218: {'attributes': {'Identifiant station': 23206, 'Nom station': 'Béranger - Félix Faure', 'Station en fonctionnement': 'NON', 'Capacité de la station': 28, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 10, 15, 10, 1, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8040482862, 2.28482633737], 'Nom communes équipées': 'Châtillon'}}, 1219: {'attributes': {'Identifiant station': 1008, 'Nom station': 'Les Halles - Saint-Eustache', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 29, 'Vélos mécaniques disponibles': 26, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.863626959, 2.34262391925], 'Nom communes équipées': 'Paris'}}, 1220: {'attributes': {'Identifiant station': 10202, 'Nom station': 'Bichat - Grange aux Belles', 'Station en fonctionnement': 'NON', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 8, 16, 15, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.873688, 2.365015], 'Nom communes équipées': 'Paris'}}, 1221: {'attributes': {'Identifiant station': 1024, 'Nom station': 'Coquillière - Louvre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8637959097, 2.34257362783], 'Nom communes équipées': 'Paris'}}, 1222: {'attributes': {'Identifiant station': 25007, 'Nom station': 'Bons Raisins - Voltaire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8728245146, 2.19471555709], 'Nom communes équipées': 'Rueil-Malmaison'}}, 1223: {'attributes': {'Identifiant station': 32020, 'Nom station': 'Jules Guesdes - Carnot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 18, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9382816993, 2.35089920461], 'Nom communes équipées': 'Saint-Denis'}}, 1224: {'attributes': {'Identifiant station': 42024, 'Nom station': 'Danielle Casanova - Ledru-Rollin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8157834979, 2.3832597211], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 1225: {'attributes': {'Identifiant station': 2020, 'Nom station': 'Danielle Casanova - Opéra', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8676432423, 2.33338940889], 'Nom communes équipées': 'Paris'}}, 1226: {'attributes': {'Identifiant station': 18135, 'Nom station': 'Ney - Porte de Montmartre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.897918, 2.3364872], 'Nom communes équipées': 'Paris'}}, 1227: {'attributes': {'Identifiant station': 20010, 'Nom station': 'Sorbier - Gasnier-Guy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.865217631, 2.39365965128], 'Nom communes équipées': 'Paris'}}, 1228: {'attributes': {'Identifiant station': 8104, 'Nom station': 'Chateaubriand - Friedland', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 9, 24, 20, 54, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8740144756, 2.29991625787], 'Nom communes équipées': 'Paris'}}, 1229: {'attributes': {'Identifiant station': 21011, 'Nom station': 'Silly - Louis Blériot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8321904537, 2.23401993513], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 1230: {'attributes': {'Identifiant station': 21002, 'Nom station': 'Place Denfert-Rochereau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8432525363, 2.24598795176], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 1231: {'attributes': {'Identifiant station': 12016, 'Nom station': 'Saint-Mande - Picpus', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8451277283, 2.40156862965], 'Nom communes équipées': 'Paris'}}, 1232: {'attributes': {'Identifiant station': 8018, 'Nom station': 'Saint-Augustin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 31, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8762009747, 2.31978625059], 'Nom communes équipées': 'Paris'}}, 1233: {'attributes': {'Identifiant station': 23301, 'Nom station': 'Les Blagis', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.785205, 2.302526], 'Nom communes équipées': 'Sceaux'}}, 1234: {'attributes': {'Identifiant station': 13015, 'Nom station': 'Chevaleret - Louise Weiss', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 31, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8325865134, 2.37121138812], 'Nom communes équipées': 'Paris'}}, 1235: {'attributes': {'Identifiant station': 34013, 'Nom station': 'Blanqui - Docteur Bauer', 'Station en fonctionnement': 'NON', 'Capacité de la station': 24, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 8, 13, 11, 36, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.9077354891, 2.34071928146], 'Nom communes équipées': 'Saint-Ouen-sur-Seine'}}, 1236: {'attributes': {'Identifiant station': 13116, 'Nom station': 'Place du Docteur Yersin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8201049499, 2.37255837768], 'Nom communes équipées': 'Paris'}}, 1237: {'attributes': {'Identifiant station': 23103, 'Nom station': 'Manufacture Nationale', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 2, 28, 15, 26, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.828785141, 2.22372226417], 'Nom communes équipées': 'Sèvres'}}, 1238: {'attributes': {'Identifiant station': 11111, 'Nom station': 'Léon Frot - Charonne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8549638003, 2.38721708537], 'Nom communes équipées': 'Paris'}}, 1239: {'attributes': {'Identifiant station': 42210, 'Nom station': 'Victor Hugo - Paul Eluard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 26, 'Vélos mécaniques disponibles': 22, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8196025549, 2.41143196731], 'Nom communes équipées': 'Charenton-le-Pont'}}, 1240: {'attributes': {'Identifiant station': 13006, 'Nom station': "Duméril - Jeanne d'Arc", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 41, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8381516081, 2.35710844956], 'Nom communes équipées': 'Paris'}}, 1241: {'attributes': {'Identifiant station': 48012, 'Nom station': 'Stalingrad - Henri Luisette', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7836309246, 2.36693594676], 'Nom communes équipées': 'Villejuif'}}, 1242: {'attributes': {'Identifiant station': 12119, 'Nom station': 'Dom-Pérignon - Gravelle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 41, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8254815744, 2.4098803103], 'Nom communes équipées': 'Paris'}}, 1243: {'attributes': {'Identifiant station': 20048, 'Nom station': 'Bagnolet - Place Saint-Blaise', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.85973, 2.40334], 'Nom communes équipées': 'Paris'}}, 1244: {'attributes': {'Identifiant station': 48006, 'Nom station': 'Maxime Gorki - Paul Vaillant-Couturier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.796735088, 2.36833423376], 'Nom communes équipées': 'Villejuif'}}, 1245: {'attributes': {'Identifiant station': 23303, 'Nom station': 'Gare de Robinson', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7799259212, 2.28182863444], 'Nom communes équipées': 'Sceaux'}}, 1246: {'attributes': {'Identifiant station': 18016, 'Nom station': 'Francoeur - Caulaincourt', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 33, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.889994441, 2.34252970666], 'Nom communes équipées': 'Paris'}}, 1247: {'attributes': {'Identifiant station': 42022, 'Nom station': 'Gare RER - Maurice Grandcoing', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 29, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.813881, 2.392429], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 1248: {'attributes': {'Identifiant station': 18010, 'Nom station': 'Riquet - Marx Dormoy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.890210037, 2.36017562449], 'Nom communes équipées': 'Paris'}}, 1249: {'attributes': {'Identifiant station': 6015, 'Nom station': 'André Mazet - Saint-André des Arts', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 55, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 30, 'Vélos mécaniques disponibles': 24, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8537558106, 2.33909580857], 'Nom communes équipées': 'Paris'}}, 1250: {'attributes': {'Identifiant station': 34010, 'Nom station': 'Hôtel de Ville de Saint-Ouen', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 48, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 24, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9113801858, 2.33400768076], 'Nom communes équipées': 'Saint-Ouen-sur-Seine'}}, 1251: {'attributes': {'Identifiant station': 23703, 'Nom station': 'Gare de Chaville Rive Gauche', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8051603043, 2.18827720731], 'Nom communes équipées': 'Chaville'}}, 1252: {'attributes': {'Identifiant station': 10003, 'Nom station': 'Hauteville - Bonne Nouvelle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8707743602, 2.34964963683], 'Nom communes équipées': 'Paris'}}, 1253: {'attributes': {'Identifiant station': 5027, 'Nom station': 'Gobelins - Port-Royal', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 26, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8371359003, 2.351520136], 'Nom communes équipées': 'Paris'}}, 1254: {'attributes': {'Identifiant station': 15024, 'Nom station': 'Suffren - La Motte-Piquet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 18, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8508812309, 2.30121999979], 'Nom communes équipées': 'Paris'}}, 1255: {'attributes': {'Identifiant station': 20142, 'Nom station': 'Mûriers - Père Lachaise', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8641305097, 2.39164982067], 'Nom communes équipées': 'Paris'}}, 1256: {'attributes': {'Identifiant station': 13024, 'Nom station': 'Bobillot - Tolbiac', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8255961335, 2.3502041772], 'Nom communes équipées': 'Paris'}}, 1257: {'attributes': {'Identifiant station': 21704, 'Nom station': 'Place du Maréchal de Lattre de Tassigny', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8217077153, 2.28534314854], 'Nom communes équipées': 'Vanves'}}, 1258: {'attributes': {'Identifiant station': 9025, 'Nom station': 'Taitbout - Châteaudun', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 19, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8763839694, 2.33535667793], 'Nom communes équipées': 'Paris'}}, 1259: {'attributes': {'Identifiant station': 19121, 'Nom station': 'Belleville -  Pré Saint-Gervais', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 25, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8755659101, 2.39304609597], 'Nom communes équipées': 'Paris'}}, 1260: {'attributes': {'Identifiant station': 13028, 'Nom station': 'Gouthière - Poterne des Peupliers', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.820507, 2.351342], 'Nom communes équipées': 'Paris'}}, 1261: {'attributes': {'Identifiant station': 34011, 'Nom station': 'Clef des Champs - Albert Dhalenne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.914938, 2.330665], 'Nom communes équipées': 'Saint-Ouen-sur-Seine'}}, 1262: {'attributes': {'Identifiant station': 11014, 'Nom station': 'Place de la Nation - Taillebourg', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 47, 'Nombre bornettes libres': 43, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8488408, 2.3973043], 'Nom communes équipées': 'Paris'}}, 1263: {'attributes': {'Identifiant station': 24101, 'Nom station': 'Place Charles de Gaulle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.82654572, 2.18925887361], 'Nom communes équipées': "Ville-d'Avray"}}, 1264: {'attributes': {'Identifiant station': 14027, 'Nom station': 'Gergovie - Alésia', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8308871746, 2.31888210121], 'Nom communes équipées': 'Paris'}}, 1265: {'attributes': {'Identifiant station': 19114, 'Nom station': 'Manin - Secrétan', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 15, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8791665, 2.3787959], 'Nom communes équipées': 'Paris'}}, 1266: {'attributes': {'Identifiant station': 42303, 'Nom station': 'Place Eustache Deschamps', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.793855, 2.336637], 'Nom communes équipées': 'Cachan'}}, 1267: {'attributes': {'Identifiant station': 12123, 'Nom station': 'Château de Vincennes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 52, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 18, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 5, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8442319326, 2.43920953771], 'Nom communes équipées': 'Paris'}}, 1268: {'attributes': {'Identifiant station': 41605, 'Nom station': 'Général De Gaulle - Vivien', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8360742009, 2.41891667247], 'Nom communes équipées': 'Saint-Mandé'}}, 1269: {'attributes': {'Identifiant station': 24004, 'Nom station': 'Paix - Verdun', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.900504, 2.261173], 'Nom communes équipées': 'Courbevoie'}}, 1270: {'attributes': {'Identifiant station': 12102, 'Nom station': 'Bercy - Traversiére', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8461926426, 2.37038008869], 'Nom communes équipées': 'Paris'}}, 1271: {'attributes': {'Identifiant station': 19006, 'Nom station': 'Mathis - Flandre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 55, 'Nombre bornettes libres': 48, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8906429899, 2.37625546753], 'Nom communes équipées': 'Paris'}}, 1272: {'attributes': {'Identifiant station': 19119, 'Nom station': 'Edouard Pailleron - Bouret', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8805976941, 2.37694948912], 'Nom communes équipées': 'Paris'}}, 1273: {'attributes': {'Identifiant station': 20003, 'Nom station': 'Pyrénées - Cours de Vincennes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8477967799, 2.40617215633], 'Nom communes équipées': 'Paris'}}, 1274: {'attributes': {'Identifiant station': 35013, 'Nom station': 'Guillaume Tell - Parc Henri Barbusse', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8895892589, 2.41988629103], 'Nom communes équipées': 'Pantin'}}, 1275: {'attributes': {'Identifiant station': 8052, 'Nom station': 'Balzac - Champs Elysées', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 13, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8726996396, 2.30019539595], 'Nom communes équipées': 'Paris'}}, 1276: {'attributes': {'Identifiant station': 24005, 'Nom station': 'Edith Cavell - Volta', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9016165643, 2.26935435086], 'Nom communes équipées': 'Courbevoie'}}, 1277: {'attributes': {'Identifiant station': 23041, 'Nom station': 'Général Leclerc - Bièvre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7824144287, 2.31666952372], 'Nom communes équipées': 'Bourg-la-Reine'}}, 1278: {'attributes': {'Identifiant station': 43007, 'Nom station': 'Hôtel de Ville de Vincennes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8482760971, 2.43989557028], 'Nom communes équipées': 'Vincennes'}}, 1279: {'attributes': {'Identifiant station': 15069, 'Nom station': 'Félix Faure - Convention', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 44, 'Nombre bornettes libres': 9, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 33, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.841249336, 2.28838660085], 'Nom communes équipées': 'Paris'}}, 1280: {'attributes': {'Identifiant station': 14004, 'Nom station': 'Port Royal - Hôpital Cochin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 59, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8383732338, 2.34071787447], 'Nom communes équipées': 'Paris'}}, 1281: {'attributes': {'Identifiant station': 15012, 'Nom station': 'Cambronne - Mademoiselle', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 67, 'Nombre bornettes libres': 56, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.84327733, 2.30230766886], 'Nom communes équipées': 'Paris'}}, 1282: {'attributes': {'Identifiant station': 14035, 'Nom station': 'Maine - Jean Zay', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 50, 'Nombre bornettes libres': 47, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8379252639, 2.32254289091], 'Nom communes équipées': 'Paris'}}, 1283: {'attributes': {'Identifiant station': 19117, 'Nom station': 'Euryale Dehaynin - Loire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8862489173, 2.37745910883], 'Nom communes équipées': 'Paris'}}, 1284: {'attributes': {'Identifiant station': 20032, 'Nom station': 'Square des Amandiers', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 12, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8661563067, 2.38913241774], 'Nom communes équipées': 'Paris'}}, 1285: {'attributes': {'Identifiant station': 15053, 'Nom station': 'Lecourbe - Convention', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8392331924, 2.29151979089], 'Nom communes équipées': 'Paris'}}, 1286: {'attributes': {'Identifiant station': 6025, 'Nom station': 'Assas - Rennes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8492084999, 2.32851454623], 'Nom communes équipées': 'Paris'}}, 1287: {'attributes': {'Identifiant station': 8040, 'Nom station': 'Pierre Charron - Champs-Elysées', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 27, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 10, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8701033599, 2.30435013771], 'Nom communes équipées': 'Paris'}}, 1288: {'attributes': {'Identifiant station': 14009, 'Nom station': 'Tombe Issoire - René Coty', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8305128767, 2.33436137438], 'Nom communes équipées': 'Paris'}}, 1289: {'attributes': {'Identifiant station': 21022, 'Nom station': 'Dôme - République', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 20, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 18, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.832681, 2.250257], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 1290: {'attributes': {'Identifiant station': 22404, 'Nom station': 'Pierre Brossolette - 12 Février 1934', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8176962417, 2.30761237442], 'Nom communes équipées': 'Malakoff'}}, 1291: {'attributes': {'Identifiant station': 15041, 'Nom station': 'Franquet - Labrouste', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 36, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 14, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8328770501, 2.30676278472], 'Nom communes équipées': 'Paris'}}, 1292: {'attributes': {'Identifiant station': 16017, 'Nom station': 'Siam - La Pompe', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.861720329, 2.27534656856], 'Nom communes équipées': 'Paris'}}, 1293: {'attributes': {'Identifiant station': 11022, 'Nom station': 'Roquette - Auguste Laurent', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.858001906, 2.38179770265], 'Nom communes équipées': 'Paris'}}, 1294: {'attributes': {'Identifiant station': 17006, 'Nom station': 'Guy Môquet - Compoint', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8921355904, 2.32321947813], 'Nom communes équipées': 'Paris'}}, 1295: {'attributes': {'Identifiant station': 41103, 'Nom station': 'Place de la vache noire', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8104051706, 2.32706040144], 'Nom communes équipées': 'Arcueil'}}, 1296: {'attributes': {'Identifiant station': 12022, 'Nom station': 'Sahel - Général Michel Bizot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8408729648, 2.40444447845], 'Nom communes équipées': 'Paris'}}, 1297: {'attributes': {'Identifiant station': 41602, 'Nom station': 'Place Charles Digeon', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8435420313, 2.41842716932], 'Nom communes équipées': 'Saint-Mandé'}}, 1298: {'attributes': {'Identifiant station': 15027, 'Nom station': 'Duhamel - André Gide', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8365662244, 2.3127071211], 'Nom communes équipées': 'Paris'}}, 1299: {'attributes': {'Identifiant station': 20110, 'Nom station': 'Gambetta - Tourelles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8753379023, 2.40599211305], 'Nom communes équipées': 'Paris'}}, 1300: {'attributes': {'Identifiant station': 21021, 'Nom station': 'Enfants du Paradis - Peupliers', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 35, 'Vélos mécaniques disponibles': 23, 'Vélos électriques disponibles': 12, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8331226865, 2.25714594126], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 1301: {'attributes': {'Identifiant station': 48002, 'Nom station': 'LCL Campus - Place Oscar Niemeyer', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8053424806, 2.36398369074], 'Nom communes équipées': 'Villejuif'}}, 1302: {'attributes': {'Identifiant station': 31105, 'Nom station': 'Lavoisier - Missak Manouchian', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.862864689, 2.49567646533], 'Nom communes équipées': 'Rosny-sous-Bois'}}, 1303: {'attributes': {'Identifiant station': 20012, 'Nom station': 'Buzenval - Vignoles', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8538581717, 2.40000408143], 'Nom communes équipées': 'Paris'}}, 1304: {'attributes': {'Identifiant station': 17117, 'Nom station': 'Pereire - Saussure', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 57, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8882290755, 2.31012962759], 'Nom communes équipées': 'Paris'}}, 1305: {'attributes': {'Identifiant station': 13120, 'Nom station': 'Pierre Joseph Desault', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 12, 'Vélos mécaniques disponibles': 9, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.821102308, 2.3787754029], 'Nom communes équipées': 'Paris'}}, 1306: {'attributes': {'Identifiant station': 42205, 'Nom station': 'Place Aristide Briand', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8218923658, 2.41300374269], 'Nom communes équipées': 'Charenton-le-Pont'}}, 1307: {'attributes': {'Identifiant station': 20020, 'Nom station': 'Pyrénées - Charles Renouvier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8609593323, 2.40042185204], 'Nom communes équipées': 'Paris'}}, 1308: {'attributes': {'Identifiant station': 7023, 'Nom station': 'Bourdonnais - Tour Eiffel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 60, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 41, 'Vélos mécaniques disponibles': 37, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8608977979, 2.29559959845], 'Nom communes équipées': 'Paris'}}, 1309: {'attributes': {'Identifiant station': 12008, 'Nom station': 'Hector Malot - Daumesnil', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 54, 'Nombre bornettes libres': 51, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8469355299, 2.37708762288], 'Nom communes équipées': 'Paris'}}, 1310: {'attributes': {'Identifiant station': 8007, 'Nom station': 'Rome - Provence', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 22, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 18, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8742300556, 2.32557011172], 'Nom communes équipées': 'Paris'}}, 1311: {'attributes': {'Identifiant station': 13036, 'Nom station': 'Olympiades', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 35, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8267646652, 2.36508607864], 'Nom communes équipées': 'Paris'}}, 1312: {'attributes': {'Identifiant station': 11029, 'Nom station': 'Ménilmontant - Oberkampf', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 35, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8666176587, 2.38301344042], 'Nom communes équipées': 'Paris'}}, 1313: {'attributes': {'Identifiant station': 15050, 'Nom station': 'Dominique Pado - Croix Nivert', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 7, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8364879816, 2.28972505778], 'Nom communes équipées': 'Paris'}}, 1314: {'attributes': {'Identifiant station': 20005, 'Nom station': 'Reynaldo Hahn - Lagny', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8492660436, 2.41233032884], 'Nom communes équipées': 'Paris'}}, 1315: {'attributes': {'Identifiant station': 16025, 'Nom station': 'Jean de la Fontaine - Boulainvilliers', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 68, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 38, 'Vélos mécaniques disponibles': 31, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8526479146, 2.27586641908], 'Nom communes équipées': 'Paris'}}, 1316: {'attributes': {'Identifiant station': 15120, 'Nom station': 'Colonel Pierre Avia', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8296843301, 2.27534258685], 'Nom communes équipées': 'Paris'}}, 1317: {'attributes': {'Identifiant station': 18021, 'Nom station': 'Marcadet - Ramey', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.890875, 2.3449019], 'Nom communes équipées': 'Paris'}}, 1318: {'attributes': {'Identifiant station': 13055, 'Nom station': 'M.- A. Lagroua Weill-Hallé -  F. Dolto', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 60, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 60, 'Vélos mécaniques disponibles': 35, 'Vélos électriques disponibles': 25, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8285952839, 2.38022060627], 'Nom communes équipées': 'Paris'}}, 1319: {'attributes': {'Identifiant station': 16116, 'Nom station': 'George Sand - Jean de La Fontaine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 28, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8498691876, 2.26825738753], 'Nom communes équipées': 'Paris'}}, 1320: {'attributes': {'Identifiant station': 21205, 'Nom station': 'Arthur Auger - Jean Jaurès', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 3, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8137364456, 2.30692774057], 'Nom communes équipées': 'Montrouge'}}, 1321: {'attributes': {'Identifiant station': 20013, 'Nom station': 'Alexandre Dumas - Place de la Réunion', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 44, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 38, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8553371074, 2.39977036876], 'Nom communes équipées': 'Paris'}}, 1322: {'attributes': {'Identifiant station': 11028, 'Nom station': 'Guillaume Bertrand - République', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 37, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 14, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8638251743, 2.38090637715], 'Nom communes équipées': 'Paris'}}, 1323: {'attributes': {'Identifiant station': 15045, 'Nom station': 'Porte de la Plaine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8280516961, 2.29242771864], 'Nom communes équipées': 'Paris'}}, 1324: {'attributes': {'Identifiant station': 19014, 'Nom station': 'Lorraine - Jean Jaurès', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8863865, 2.3825772], 'Nom communes équipées': 'Paris'}}, 1325: {'attributes': {'Identifiant station': 25004, 'Nom station': 'Marcel Pourtout - Édouard Manet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 9, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8755562076, 2.16559678316], 'Nom communes équipées': 'Rueil-Malmaison'}}, 1326: {'attributes': {'Identifiant station': 16023, 'Nom station': 'Passy - Place du Costa Rica', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 7, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8584893, 2.2839655], 'Nom communes équipées': 'Paris'}}, 1327: {'attributes': {'Identifiant station': 14005, 'Nom station': 'René Coty - Place Denfert-Rochereau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 2, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8330564773, 2.3329411447], 'Nom communes équipées': 'Paris'}}, 1328: {'attributes': {'Identifiant station': 22014, 'Nom station': 'Victor Hugo - Bineau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 10, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8875822589, 2.27838903666], 'Nom communes équipées': 'Neuilly-sur-Seine'}}, 1329: {'attributes': {'Identifiant station': 21023, 'Nom station': 'Ancienne Mairie - André Morizet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 24, 'Vélos mécaniques disponibles': 20, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.838311, 2.234077], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 1330: {'attributes': {'Identifiant station': 11016, 'Nom station': 'Philippe-Auguste', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 48, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8518534883, 2.39329453558], 'Nom communes équipées': 'Paris'}}, 1331: {'attributes': {'Identifiant station': 43004, 'Nom station': 'Montreuil - République', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8502634946, 2.43452042341], 'Nom communes équipées': 'Vincennes'}}, 1332: {'attributes': {'Identifiant station': 10031, 'Nom station': 'Louis Blanc - Aqueduc', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 33, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8820301826, 2.36356157809], 'Nom communes équipées': 'Paris'}}, 1333: {'attributes': {'Identifiant station': 8046, 'Nom station': 'Marceau - Président Wilson', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 21, 'Vélos mécaniques disponibles': 19, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8652359, 2.3002513], 'Nom communes équipées': 'Paris'}}, 1334: {'attributes': {'Identifiant station': 21027, 'Nom station': 'Fief - République', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8317088817, 2.25577197969], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 1335: {'attributes': {'Identifiant station': 20014, 'Nom station': 'Charonne - Alexandre Dumas', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 33, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8560774151, 2.39486932755], 'Nom communes équipées': 'Paris'}}, 1336: {'attributes': {'Identifiant station': 15044, 'Nom station': 'Dantzig - Lefebvre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 43, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8301392159, 2.29593403637], 'Nom communes équipées': 'Paris'}}, 1337: {'attributes': {'Identifiant station': 17110, 'Nom station': 'Legendre - Nollet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8877421868, 2.32029307988], 'Nom communes équipées': 'Paris'}}, 1338: {'attributes': {'Identifiant station': 12040, 'Nom station': 'Porte de Charenton', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 67, 'Nombre bornettes libres': 59, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8312401059, 2.39891074598], 'Nom communes équipées': 'Paris'}}, 1339: {'attributes': {'Identifiant station': 7010, 'Nom station': 'Constantine - Université', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 65, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 60, 'Vélos mécaniques disponibles': 43, 'Vélos électriques disponibles': 17, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8606804737, 2.31484819558], 'Nom communes équipées': 'Paris'}}, 1340: {'attributes': {'Identifiant station': 18018, 'Nom station': 'Carpeaux - Place Jacques Froment', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8906542, 2.3309428], 'Nom communes équipées': 'Paris'}}, 1341: {'attributes': {'Identifiant station': 31701, 'Nom station': 'Mairie de Bagnolet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8684213186, 2.41795878857], 'Nom communes équipées': 'Bagnolet'}}, 1342: {'attributes': {'Identifiant station': 19001, 'Nom station': 'Crimée - Ourcq', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 56, 'Nombre bornettes libres': 51, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8941041403, 2.37294608847], 'Nom communes équipées': 'Paris'}}, 1343: {'attributes': {'Identifiant station': 19102, 'Nom station': 'Rampal - Belleville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8730216016, 2.37991098315], 'Nom communes équipées': 'Paris'}}, 1344: {'attributes': {'Identifiant station': 16022, 'Nom station': 'Jean Bologne - Passy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 19, 'Nombre bornettes libres': 12, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8575668898, 2.28002739531], 'Nom communes équipées': 'Paris'}}, 1345: {'attributes': {'Identifiant station': 5033, 'Nom station': 'Saint-Séverin - Saint-Michel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8527313055, 2.34409276396], 'Nom communes équipées': 'Paris'}}, 1346: {'attributes': {'Identifiant station': 20011, 'Nom station': 'Pyrénées - Dagorno', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8554257707, 2.4051643163], 'Nom communes équipées': 'Paris'}}, 1347: {'attributes': {'Identifiant station': 18101, 'Nom station': 'Championnet.', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 5, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8953643, 2.3498166], 'Nom communes équipées': 'Paris'}}, 1348: {'attributes': {'Identifiant station': 13013, 'Nom station': 'Saint-Marcel - Hôpital', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 15, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8395044246, 2.36098941702], 'Nom communes équipées': 'Paris'}}, 1349: {'attributes': {'Identifiant station': 21207, 'Nom station': 'Verdier - République', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 27, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8154037663, 2.31739372015], 'Nom communes équipées': 'Montrouge'}}, 1350: {'attributes': {'Identifiant station': 13023, 'Nom station': 'Bobillot - Moulin des Prés', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8286573462, 2.35302064632], 'Nom communes équipées': 'Paris'}}, 1351: {'attributes': {'Identifiant station': 12122, 'Nom station': 'Bois de Vincennes - Polygone', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 51, 'Nombre bornettes libres': 46, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8354449162, 2.43142108144], 'Nom communes équipées': 'Paris'}}, 1352: {'attributes': {'Identifiant station': 21007, 'Nom station': 'Rond-Point Rhin et Danube', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 53, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 42, 'Vélos mécaniques disponibles': 39, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.840452821, 2.22802251577], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 1353: {'attributes': {'Identifiant station': 24007, 'Nom station': 'Jean-Baptiste Charcot - Sainte-Geneviève', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 18, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9031088372, 2.27889295667], 'Nom communes équipées': 'Courbevoie'}}, 1354: {'attributes': {'Identifiant station': 5105, 'Nom station': 'Geoffroy-Saint-Hilaire - Saint-Marcel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 21, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8390782642, 2.35687734067], 'Nom communes équipées': 'Paris'}}, 1355: {'attributes': {'Identifiant station': 16033, 'Nom station': "Marché d'Auteuil", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 11, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8484276137, 2.26525382116], 'Nom communes équipées': 'Paris'}}, 1356: {'attributes': {'Identifiant station': 12039, 'Nom station': 'Général Michel Bizot - Claude Decaen', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8348126298, 2.40092778813], 'Nom communes équipées': 'Paris'}}, 1357: {'attributes': {'Identifiant station': 6028, 'Nom station': "Quatre Vents - Carrefour de l'Odéon", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 22, 'Vélos mécaniques disponibles': 18, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 43, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8517513745, 2.33816169513], 'Nom communes équipées': 'Paris'}}, 1358: {'attributes': {'Identifiant station': 12017, 'Nom station': 'Saint-Mandé - Docteur Arnold Netter', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 4, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8446258577, 2.40494638681], 'Nom communes équipées': 'Paris'}}, 1359: {'attributes': {'Identifiant station': 20106, 'Nom station': 'Mairie du 20ème', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 27, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 11, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8653023243, 2.39906698465], 'Nom communes équipées': 'Paris'}}, 1360: {'attributes': {'Identifiant station': 8039, 'Nom station': 'Colisée - Champs-Elysées', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 24, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 23, 'Vélos mécaniques disponibles': 18, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8704753994, 2.30761907995], 'Nom communes équipées': 'Paris'}}, 1361: {'attributes': {'Identifiant station': 15014, 'Nom station': 'Volontaires - Vaugirard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 11, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8413205018, 2.30808901725], 'Nom communes équipées': 'Paris'}}, 1362: {'attributes': {'Identifiant station': 21033, 'Nom station': 'Sèvres - Maréchal Juin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 18, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 14, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 34, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8324305, 2.2292812], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 1363: {'attributes': {'Identifiant station': 15114, 'Nom station': 'Pasteur - Cotentin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8389284, 2.316356], 'Nom communes équipées': 'Paris'}}, 1364: {'attributes': {'Identifiant station': 17036, 'Nom station': 'Place Tristan Bernard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 41, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8791964, 2.2915403], 'Nom communes équipées': 'Paris'}}, 1365: {'attributes': {'Identifiant station': 6004, 'Nom station': 'Regard - Rennes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 46, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.847385405, 2.32685554773], 'Nom communes équipées': 'Paris'}}, 1366: {'attributes': {'Identifiant station': 17050, 'Nom station': 'Batignolles - Abel Truchet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8831039046, 2.32383474909], 'Nom communes équipées': 'Paris'}}, 1367: {'attributes': {'Identifiant station': 25002, 'Nom station': 'Hôtel de Ville de Rueil-Malmaison', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 12, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 45, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.878194, 2.181724], 'Nom communes équipées': 'Rueil-Malmaison'}}, 1368: {'attributes': {'Identifiant station': 19018, 'Nom station': 'Cité de la Musique', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 60, 'Nombre bornettes libres': 57, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8887965565, 2.39258119319], 'Nom communes équipées': 'Paris'}}, 1369: {'attributes': {'Identifiant station': 15113, 'Nom station': 'Arsonval - Falguière', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8404004037, 2.3131178673], 'Nom communes équipées': 'Paris'}}, 1370: {'attributes': {'Identifiant station': 10001, 'Nom station': 'René Boulanger - Lancry', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 62, 'Nombre bornettes libres': 46, 'Nombre total vélos disponibles': 11, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8685433, 2.3600032], 'Nom communes équipées': 'Paris'}}, 1371: {'attributes': {'Identifiant station': 20033, 'Nom station': 'Etienne Dolet - Place Maurice Chevalier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 32, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8679830336, 2.38514053111], 'Nom communes équipées': 'Paris'}}, 1372: {'attributes': {'Identifiant station': 44009, 'Nom station': 'Gare RER les Ardoines', 'Station en fonctionnement': 'NON', 'Capacité de la station': 49, 'Nombre bornettes libres': 49, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2019, 5, 17, 9, 28, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.7838563635, 2.40790937175], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 1373: {'attributes': {'Identifiant station': 6005, 'Nom station': 'Montparnasse -  Square Ozanam', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 49, 'Nombre bornettes libres': 43, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8433105678, 2.32652429491], 'Nom communes équipées': 'Paris'}}, 1374: {'attributes': {'Identifiant station': 10151, 'Nom station': 'Gare du Nord - Faubourg Saint-Denis', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 28, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 3, 'Vélos mécaniques disponibles': 2, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 4, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8796127655, 2.35684029758], 'Nom communes équipées': 'Paris'}}, 1375: {'attributes': {'Identifiant station': 14025, 'Nom station': 'Jacquier - Didot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8295333629, 2.31817927212], 'Nom communes équipées': 'Paris'}}, 1376: {'attributes': {'Identifiant station': 21508, 'Nom station': 'Georges Pompidou - Verdun', 'Station en fonctionnement': 'NON', 'Capacité de la station': 26, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2019, 9, 5, 14, 10, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.875481, 2.232443], 'Nom communes équipées': 'Suresnes'}}, 1377: {'attributes': {'Identifiant station': 11018, 'Nom station': 'Saint-Maur - République', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 38, 'Nombre bornettes libres': 37, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 23, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8641858615, 2.37831741571], 'Nom communes équipées': 'Paris'}}, 1378: {'attributes': {'Identifiant station': 4005, 'Nom station': 'Quai des Célestins - Henri IV', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 14, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 8, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8512971, 2.3624535], 'Nom communes équipées': 'Paris'}}, 1379: {'attributes': {'Identifiant station': 19007, 'Nom station': 'Flandre - Ourcq', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 29, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8926015919, 2.37909156829], 'Nom communes équipées': 'Paris'}}, 1380: {'attributes': {'Identifiant station': 8003, 'Nom station': 'Galilée - Vernet', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 17, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8717158787, 2.2985444963], 'Nom communes équipées': 'Paris'}}, 1381: {'attributes': {'Identifiant station': 21016, 'Nom station': 'Point du Jour - Jean Jaurès', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 6, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8287531414, 2.24591146701], 'Nom communes équipées': 'Boulogne-Billancourt'}}, 1382: {'attributes': {'Identifiant station': 19045, 'Nom station': 'Macdonald - Césaria Evora', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 67, 'Nombre bornettes libres': 67, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2020, 6, 29, 14, 52, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.89883, 2.37933], 'Nom communes équipées': 'Paris'}}, 1383: {'attributes': {'Identifiant station': 6020, 'Nom station': 'Place Saint-André des Arts', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 37, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 34, 'Vélos mécaniques disponibles': 28, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8528861739, 2.34265442938], 'Nom communes équipées': 'Paris'}}, 1384: {'attributes': {'Identifiant station': 12105, 'Nom station': 'Bercy - Villot', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 45, 'Vélos mécaniques disponibles': 32, 'Vélos électriques disponibles': 13, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.841795009, 2.37678501848], 'Nom communes équipées': 'Paris'}}, 1385: {'attributes': {'Identifiant station': 13008, 'Nom station': "Place d'Italie - Soeur Rosalie", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 52, 'Nombre bornettes libres': 47, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8319085204, 2.35469434481], 'Nom communes équipées': 'Paris'}}, 1386: {'attributes': {'Identifiant station': 8004, 'Nom station': 'Malesherbes - Place de la Madeleine', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 67, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 78, 'Vélos mécaniques disponibles': 57, 'Vélos électriques disponibles': 21, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 39, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8704060285, 2.32324350981], 'Nom communes équipées': 'Paris'}}, 1387: {'attributes': {'Identifiant station': 19123, 'Nom station': 'Metz - Marne', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 31, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 33, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.891163, 2.3865705], 'Nom communes équipées': 'Paris'}}, 1388: {'attributes': {'Identifiant station': 41212, 'Nom station': 'Nungesser - Santé', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 21, 'Nombre total vélos disponibles': 1, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.852335, 2.448967], 'Nom communes équipées': 'Fontenay-sous-Bois'}}, 1389: {'attributes': {'Identifiant station': 21107, 'Nom station': 'Bertrand Sincholle - Henri Barbusse', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 20, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8992598239, 2.3042294383], 'Nom communes équipées': 'Clichy'}}, 1390: {'attributes': {'Identifiant station': 13053, 'Nom station': 'Chevaleret - Tolbiac', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 21, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 18, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 46, 3, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.82941, 2.375934], 'Nom communes équipées': 'Paris'}}, 1391: {'attributes': {'Identifiant station': 16104, 'Nom station': 'Paul Valery - Victor Hugo', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 25, 'Nombre total vélos disponibles': 10, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 44, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8713740608, 2.28871285915], 'Nom communes équipées': 'Paris'}}, 1392: {'attributes': {'Identifiant station': 18047, 'Nom station': 'St-Ouen - Lamarck', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 42, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8910830403, 2.32673569484], 'Nom communes équipées': 'Paris'}}, 1393: {'attributes': {'Identifiant station': 5031, 'Nom station': 'Lacépède - Linné', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 54, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 19, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 4, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 16, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8437035477, 2.35465127975], 'Nom communes équipées': 'Paris'}}, 1394: {'attributes': {'Identifiant station': 8002, 'Nom station': 'Gare Saint-Lazare - Cour du Havre', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 8, 'Nombre total vélos disponibles': 32, 'Vélos mécaniques disponibles': 29, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 56, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8756744009, 2.32655983418], 'Nom communes équipées': 'Paris'}}, 1395: {'attributes': {'Identifiant station': 22007, 'Nom station': 'Commandant Charcot - Bretteville', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8753057077, 2.25584607571], 'Nom communes équipées': 'Neuilly-sur-Seine'}}, 1396: {'attributes': {'Identifiant station': 13123, 'Nom station': 'Bibliothèque Nationale de France', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 42, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 75, 'Vélos mécaniques disponibles': 56, 'Vélos électriques disponibles': 19, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 27, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8350268238, 2.3760157451], 'Nom communes équipées': 'Paris'}}, 1397: {'attributes': {'Identifiant station': 13037, 'Nom station': 'Ivry - Baudricourt', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 20, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 4, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 14, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8246959954, 2.36310607358], 'Nom communes équipées': 'Paris'}}, 1398: {'attributes': {'Identifiant station': 34009, 'Nom station': 'Dora Maar - Dulcie September', 'Station en fonctionnement': 'NON', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 5, 28, 18, 31, 59, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.906438, 2.32159], 'Nom communes équipées': 'Saint-Ouen-sur-Seine'}}, 1399: {'attributes': {'Identifiant station': 6009, 'Nom station': 'Guynemer - Jardin du Luxembourg', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8466127, 2.3325478], 'Nom communes équipées': 'Paris'}}, 1400: {'attributes': {'Identifiant station': 9029, 'Nom station': "Saint-Lazare - Place d'Estienne d'Orves", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 39, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 17, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 46, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8768345, 2.3329067], 'Nom communes équipées': 'Paris'}}, 1401: {'attributes': {'Identifiant station': 42027, 'Nom station': 'Verdun - Pierre et Marie Curie', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 26, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 42, 58, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8128223983, 2.37071067095], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 1402: {'attributes': {'Identifiant station': 4104, 'Nom station': 'Sebastopol - Rambuteau', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 19, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8618180927, 2.3501381278], 'Nom communes équipées': 'Paris'}}, 1403: {'attributes': {'Identifiant station': 8044, 'Nom station': 'Alfred de Vigny - Place du Général Brocard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 16, 'Nombre total vélos disponibles': 17, 'Vélos mécaniques disponibles': 14, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8783613836, 2.30525444822], 'Nom communes équipées': 'Paris'}}, 1404: {'attributes': {'Identifiant station': 42004, 'Nom station': 'Westermeyer - Paul Vaillant-Couturier', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 25, 'Vélos mécaniques disponibles': 16, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8191161816, 2.39666372538], 'Nom communes équipées': 'Ivry-sur-Seine'}}, 1405: {'attributes': {'Identifiant station': 8031, 'Nom station': 'Jean Mermoz - Champs-Elysées', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 33, 'Nombre bornettes libres': 2, 'Nombre total vélos disponibles': 30, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 9, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8696898569, 2.31073245406], 'Nom communes équipées': 'Paris'}}, 1406: {'attributes': {'Identifiant station': 22305, 'Nom station': 'Jean Jaurès - Place du Dr Roux', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 35, 'Nombre bornettes libres': 6, 'Nombre total vélos disponibles': 28, 'Vélos mécaniques disponibles': 23, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 3, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9275670328, 2.29473370174], 'Nom communes équipées': 'Gennevilliers'}}, 1407: {'attributes': {'Identifiant station': 16140, 'Nom station': 'Longchamp - Suresnes', 'Station en fonctionnement': 'NON', 'Capacité de la station': 30, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 5, 26, 11, 51, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8641322653, 2.23990537226], 'Nom communes équipées': 'Paris'}}, 1408: {'attributes': {'Identifiant station': 34008, 'Nom station': 'Garibaldi - Gabriel Péri', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 19, 'Nombre total vélos disponibles': 8, 'Vélos mécaniques disponibles': 6, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 8, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9057419114, 2.33160442081], 'Nom communes équipées': 'Saint-Ouen-sur-Seine'}}, 1409: {'attributes': {'Identifiant station': 10028, 'Nom station': 'Gare du Nord - Dunkerque', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 29, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8804235084, 2.35276367515], 'Nom communes équipées': 'Paris'}}, 1410: {'attributes': {'Identifiant station': 22507, 'Nom station': 'Châteaudun - Place de la Liberté', 'Station en fonctionnement': 'NON', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 10, 20, 9, 43, 31, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.909708303, 2.24544864806], 'Nom communes équipées': 'La Garenne-Colombes'}}, 1411: {'attributes': {'Identifiant station': 44010, 'Nom station': '8 Mai 1945 - 10 Juillet 1940', 'Station en fonctionnement': 'NON', 'Capacité de la station': 30, 'Nombre bornettes libres': 30, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2019, 5, 17, 12, 8, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.7845670297, 2.39790357649], 'Nom communes équipées': 'Vitry-sur-Seine'}}, 1412: {'attributes': {'Identifiant station': 12106, 'Nom station': 'Gare de Lyon - Roland Barthes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 68, 'Nombre bornettes libres': 13, 'Nombre total vélos disponibles': 54, 'Vélos mécaniques disponibles': 46, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 51, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8439572949, 2.37908821553], 'Nom communes équipées': 'Paris'}}, 1413: {'attributes': {'Identifiant station': 7007, 'Nom station': "Musée d'Orsay", 'Station en fonctionnement': 'OUI', 'Capacité de la station': 65, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 60, 'Vélos mécaniques disponibles': 55, 'Vélos électriques disponibles': 5, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8597261579, 2.32586044818], 'Nom communes équipées': 'Paris'}}, 1414: {'attributes': {'Identifiant station': 22005, 'Nom station': 'Charles de Gaulle - Madrid', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 7, 'Nombre total vélos disponibles': 13, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8844322704, 2.26080447435], 'Nom communes équipées': 'Neuilly-sur-Seine'}}, 1415: {'attributes': {'Identifiant station': 35021, 'Nom station': 'Cartier Bresson - Denis Papin', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 24, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 48, 6, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.902616, 2.397814], 'Nom communes équipées': 'Pantin'}}, 1416: {'attributes': {'Identifiant station': 23203, 'Nom station': 'Division Leclerc - Kléber', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 22, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7981166342, 2.2826877223], 'Nom communes équipées': 'Châtillon'}}, 1417: {'attributes': {'Identifiant station': 18045, 'Nom station': 'Pierre Ginier - Clichy', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 16, 'Nombre bornettes libres': 14, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 54, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8867667, 2.326228], 'Nom communes équipées': 'Paris'}}, 1418: {'attributes': {'Identifiant station': 9035, 'Nom station': 'Gare Saint-Lazare - Place de Budapest', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 44, 'Nombre bornettes libres': 4, 'Nombre total vélos disponibles': 39, 'Vélos mécaniques disponibles': 31, 'Vélos électriques disponibles': 8, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 48, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.877753, 2.327339], 'Nom communes équipées': 'Paris'}}, 1419: {'attributes': {'Identifiant station': 21302, 'Nom station': 'Aristide Briand - Place de la Résistance', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 25, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 29, 'Vélos mécaniques disponibles': 22, 'Vélos électriques disponibles': 7, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.821242484, 2.25110024214], 'Nom communes équipées': 'Issy-les-Moulineaux'}}, 1420: {'attributes': {'Identifiant station': 9104, 'Nom station': 'Caumartin - Provence', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 27, 'Vélos mécaniques disponibles': 13, 'Vélos électriques disponibles': 14, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 50, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8744227734, 2.32846856117], 'Nom communes équipées': 'Paris'}}, 1421: {'attributes': {'Identifiant station': 32023, 'Nom station': 'Guy Môquet - Jean Moulin', 'Station en fonctionnement': 'NON', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 7, 28, 11, 33, 19, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.940931521, 2.36394899993], 'Nom communes équipées': 'Saint-Denis'}}, 1422: {'attributes': {'Identifiant station': 15056, 'Nom station': 'Place Balard', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 22, 'Nombre bornettes libres': 1, 'Nombre total vélos disponibles': 36, 'Vélos mécaniques disponibles': 33, 'Vélos électriques disponibles': 3, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8363957364, 2.27841921151], 'Nom communes équipées': 'Paris'}}, 1423: {'attributes': {'Identifiant station': 22602, 'Nom station': 'Place Henri Dunant - Gare RER', 'Station en fonctionnement': 'NON', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 7, 28, 9, 28, 26, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.787543, 2.2907628], 'Nom communes équipées': 'Fontenay-aux-Roses'}}, 1424: {'attributes': {'Identifiant station': 21514, 'Nom station': 'Place de la Paix - Abbé Saint-Pierre', 'Station en fonctionnement': 'NON', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 10, 22, 10, 34, 12, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8646272446, 2.20475703385], 'Nom communes équipées': 'Suresnes'}}, 1425: {'attributes': {'Identifiant station': 23201, 'Nom station': 'Avenir', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 45, 'Nombre bornettes libres': 39, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 5, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8103438, 2.3003776], 'Nom communes équipées': 'Châtillon'}}, 1426: {'attributes': {'Identifiant station': 23204, 'Nom station': 'Roland Garros - Jean Bouin', 'Station en fonctionnement': 'NON', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 10, 22, 10, 43, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8094608572, 2.28531986383], 'Nom communes équipées': 'Châtillon'}}, 1427: {'attributes': {'Identifiant station': 23205, 'Nom station': 'Parc des Pierrettes', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 23, 'Nombre bornettes libres': 23, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 32, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.7973062, 2.2909969], 'Nom communes équipées': 'Châtillon'}}, 1428: {'attributes': {'Identifiant station': 26008, 'Nom station': 'Henri Barbusse - Bourguignons', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 34, 'Nombre bornettes libres': 17, 'Nombre total vélos disponibles': 16, 'Vélos mécaniques disponibles': 10, 'Vélos électriques disponibles': 6, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 47, 45, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.913017228, 2.27342122708], 'Nom communes équipées': 'Asnières-sur-Seine'}}, 1429: {'attributes': {'Identifiant station': 17127, 'Nom station': 'Porte de Clichy - Place des Nations Unies', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 40, 'Nombre bornettes libres': 34, 'Nombre total vélos disponibles': 5, 'Vélos mécaniques disponibles': 3, 'Vélos électriques disponibles': 2, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 44, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8967372633, 2.3105429992], 'Nom communes équipées': 'Paris'}}, 1430: {'attributes': {'Identifiant station': 17126, 'Nom station': 'Hélène et François Missoffe - Emile Borel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 32, 'Nombre bornettes libres': 3, 'Nombre total vélos disponibles': 26, 'Vélos mécaniques disponibles': 15, 'Vélos électriques disponibles': 11, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 51, 2, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.8999065384, 2.32154399167], 'Nom communes équipées': 'Paris'}}, 1431: {'attributes': {'Identifiant station': 32012, 'Nom station': 'Carrefour Pleyel', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 36, 'Nombre bornettes libres': 5, 'Nombre total vélos disponibles': 32, 'Vélos mécaniques disponibles': 21, 'Vélos électriques disponibles': 11, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 49, 52, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.9203415696, 2.34385438263], 'Nom communes équipées': 'Saint-Denis'}}, 1432: {'attributes': {'Identifiant station': 34005, 'Nom station': 'Adrien Lesesne - Docteur Bauer', 'Station en fonctionnement': 'OUI', 'Capacité de la station': 30, 'Nombre bornettes libres': 28, 'Nombre total vélos disponibles': 2, 'Vélos mécaniques disponibles': 1, 'Vélos électriques disponibles': 1, 'Borne de paiement disponible': 'OUI', 'Retour vélib possible': 'OUI', 'Actualisation de la donnée': datetime.datetime(2021, 11, 3, 13, 50, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))), 'Coordonnées géographiques': [48.902638673, 2.34824806343], 'Nom communes équipées': 'Saint-Ouen-sur-Seine'}}, 1433: {'attributes': {'Identifiant station': 26009, 'Nom station': 'Marcel Delorme - Pierre Brossolette', 'Station en fonctionnement': 'NON', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 7, 5, 12, 21, 40, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.9120606286, 2.29589742357], 'Nom communes équipées': 'Asnières-sur-Seine'}}, 1434: {'attributes': {'Identifiant station': 34012, 'Nom station': 'Bateliers - Quai de Seine', 'Station en fonctionnement': 'NON', 'Capacité de la station': 26, 'Nombre bornettes libres': 26, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2018, 10, 11, 5, 6, 49, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.9158175855, 2.32395332986], 'Nom communes équipées': 'Saint-Ouen-sur-Seine'}}, 1435: {'attributes': {'Identifiant station': 31301, 'Nom station': 'Gare RER', 'Station en fonctionnement': 'NON', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 10, 22, 10, 52, 10, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.8962414123, 2.4595758301], 'Nom communes équipées': 'Noisy-le-Sec'}}, 1436: {'attributes': {'Identifiant station': 31303, 'Nom station': 'Simone Signoret - Paris', 'Station en fonctionnement': 'NON', 'Capacité de la station': 0, 'Nombre bornettes libres': 0, 'Nombre total vélos disponibles': 0, 'Vélos mécaniques disponibles': 0, 'Vélos électriques disponibles': 0, 'Borne de paiement disponible': 'NON', 'Retour vélib possible': 'NON', 'Actualisation de la donnée': datetime.datetime(2021, 10, 22, 10, 59, 55, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))), 'Coordonnées géographiques': [48.9036006247, 2.46540659627], 'Nom communes équipées': 'Noisy-le-Sec'}}}}


geolayer_paris_velib_str = {
    "metadata": {
        "name": "velib-disponibilite-en-temps-reel",
        "fields": {
            "Identifiant station": {"type": "String", "width": 5, "index": 0},
            "Nom station": {"type": "String", "width": 45, "index": 1},
            "Station en fonctionnement": {"type": "String", "width": 3, "index": 2},
            "Capacité de la station": {"type": "String", "width": 2, "index": 3},
            "Nombre bornettes libres": {"type": "String", "width": 2, "index": 4},
            "Nombre total vélos disponibles": {
                "type": "String",
                "width": 2,
                "index": 5,
            },
            "Vélos mécaniques disponibles": {"type": "String", "width": 2, "index": 6},
            "Vélos électriques disponibles": {"type": "String", "width": 2, "index": 7},
            "Borne de paiement disponible": {"type": "String", "width": 3, "index": 8},
            "Retour vélib possible": {"type": "String", "width": 3, "index": 9},
            "Actualisation de la donnée": {"type": "String", "width": 25, "index": 10},
            "Coordonnées géographiques": {"type": "String", "width": 27, "index": 11},
            "Nom communes équipées": {"type": "String", "width": 20, "index": 12},
            "Code INSEE communes équipées": {"type": "String", "width": 0, "index": 13},
        },
    },
    "features": {
        0: {
            "attributes": {
                "Identifiant station": "11104",
                "Nom station": "Charonne - Robert et Sonia Delauney",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:31+01:00",
                "Coordonnées géographiques": "48.855907556,2.39257067442",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1: {
            "attributes": {
                "Identifiant station": "16107",
                "Nom station": "Benjamin Godard - Victor Hugo",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:17+01:00",
                "Coordonnées géographiques": "48.865983,2.275725",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        2: {
            "attributes": {
                "Identifiant station": "14014",
                "Nom station": "Jourdan - Stade Charléty",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "60",
                "Nombre bornettes libres": "56",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:57+01:00",
                "Coordonnées géographiques": "48.8194283334,2.34333537519",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        3: {
            "attributes": {
                "Identifiant station": "9020",
                "Nom station": "Toudouze - Clauzel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8792959173,2.33736008406",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        4: {
            "attributes": {
                "Identifiant station": "13007",
                "Nom station": "Le Brun - Gobelins",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "48",
                "Nombre bornettes libres": "38",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:50+01:00",
                "Coordonnées géographiques": "48.8350927878,2.35346813513",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        5: {
            "attributes": {
                "Identifiant station": "12109",
                "Nom station": "Mairie du 12ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:39+01:00",
                "Coordonnées géographiques": "48.8408553118,2.38755494356",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        6: {
            "attributes": {
                "Identifiant station": "6108",
                "Nom station": "Saint-Romain - Cherche-Midi",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:24+01:00",
                "Coordonnées géographiques": "48.8470815908,2.32137478888",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        7: {
            "attributes": {
                "Identifiant station": "5001",
                "Nom station": "Harpe - Saint-Germain",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:51+01:00",
                "Coordonnées géographiques": "48.851518815,2.34367031604",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        8: {
            "attributes": {
                "Identifiant station": "42016",
                "Nom station": "Pierre et Marie Curie - Maurice Thorez",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:41+01:00",
                "Coordonnées géographiques": "48.8158022636,2.37680498511",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        9: {
            "attributes": {
                "Identifiant station": "17026",
                "Nom station": "Jouffroy d'Abbans - Wagram",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:56+01:00",
                "Coordonnées géographiques": "48.8819732984,2.30113215744",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        10: {
            "attributes": {
                "Identifiant station": "41301",
                "Nom station": "Bois de Vincennes - Gare",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "46",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:54+01:00",
                "Coordonnées géographiques": "48.8360222429,2.47083399508",
                "Nom communes équipées": "Nogent-sur-Marne",
                "Code INSEE communes équipées": "",
            }
        },
        11: {
            "attributes": {
                "Identifiant station": "10013",
                "Nom station": "Alibert - Jemmapes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "60",
                "Nombre bornettes libres": "53",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:46+01:00",
                "Coordonnées géographiques": "48.871044052,2.36610446199",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        12: {
            "attributes": {
                "Identifiant station": "21010",
                "Nom station": "Silly - Galliéni",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "24",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8355838387,2.23255008459",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        13: {
            "attributes": {
                "Identifiant station": "8026",
                "Nom station": "Messine - Place Du Pérou",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "12",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:12+01:00",
                "Coordonnées géographiques": "48.875448034,2.31550801901",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        14: {
            "attributes": {
                "Identifiant station": "25006",
                "Nom station": "Place Nelson Mandela",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:09+01:00",
                "Coordonnées géographiques": "48.8620909377,2.19657629728",
                "Nom communes équipées": "Rueil-Malmaison",
                "Code INSEE communes équipées": "",
            }
        },
        15: {
            "attributes": {
                "Identifiant station": "14111",
                "Nom station": "Cassini - Denfert-Rochereau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:17+01:00",
                "Coordonnées géographiques": "48.8375258391,2.33603540808",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        16: {
            "attributes": {
                "Identifiant station": "7001",
                "Nom station": "Invalides - Duroc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:45+01:00",
                "Coordonnées géographiques": "48.8477924733,2.31619209051",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        17: {
            "attributes": {
                "Identifiant station": "6003",
                "Nom station": "Saint-Sulpice",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:23+01:00",
                "Coordonnées géographiques": "48.8516538318,2.33080778271",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        18: {
            "attributes": {
                "Identifiant station": "11025",
                "Nom station": "Froment - Bréguet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:01+01:00",
                "Coordonnées géographiques": "48.8570414505,2.37289470307",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        19: {
            "attributes": {
                "Identifiant station": "5110",
                "Nom station": "Lacépède - Monge",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:37+01:00",
                "Coordonnées géographiques": "48.8438928653,2.35196638852",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        20: {
            "attributes": {
                "Identifiant station": "13101",
                "Nom station": "Croulebarde - Corvisart",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.8309816593,2.34816469252",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        21: {
            "attributes": {
                "Identifiant station": "33006",
                "Nom station": "André Karman - République",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:55+01:00",
                "Coordonnées géographiques": "48.9103987576,2.38513559103",
                "Nom communes équipées": "Aubervilliers",
                "Code INSEE communes équipées": "",
            }
        },
        22: {
            "attributes": {
                "Identifiant station": "14108",
                "Nom station": "Le Brix et Mesmin - Jourdan",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:29+01:00",
                "Coordonnées géographiques": "48.8223409659,2.3278616533",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        23: {
            "attributes": {
                "Identifiant station": "6021",
                "Nom station": "Beaux-Arts - Bonaparte",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:44+01:00",
                "Coordonnées géographiques": "48.8564519854,2.33485188335",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        24: {
            "attributes": {
                "Identifiant station": "47007",
                "Nom station": "Liberté - Vert-de-Maisons",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:56+01:00",
                "Coordonnées géographiques": "48.7901478096,2.43286281824",
                "Nom communes équipées": "Maisons-Alfort",
                "Code INSEE communes équipées": "",
            }
        },
        25: {
            "attributes": {
                "Identifiant station": "17038",
                "Nom station": "Grande Armée - Brunel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "61",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "11",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:51+01:00",
                "Coordonnées géographiques": "48.876116,2.288124",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        26: {
            "attributes": {
                "Identifiant station": "8050",
                "Nom station": "Boétie - Ponthieu",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "31",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:05+01:00",
                "Coordonnées géographiques": "48.8714172844,2.3076787591",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        27: {
            "attributes": {
                "Identifiant station": "17041",
                "Nom station": "Guersant - Gouvion-Saint-Cyr",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:10+01:00",
                "Coordonnées géographiques": "48.8828777518,2.28766737081",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        28: {
            "attributes": {
                "Identifiant station": "17044",
                "Nom station": "Porte de Saint-Ouen - Bessières",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "11",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:15+01:00",
                "Coordonnées géographiques": "48.8979224085,2.32851451022",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        29: {
            "attributes": {
                "Identifiant station": "7003",
                "Nom station": "Square Boucicaut",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "60",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "26",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.8512964337,2.32506182045",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        30: {
            "attributes": {
                "Identifiant station": "16118",
                "Nom station": "Michel-Ange - Parent de Rosan",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:31+01:00",
                "Coordonnées géographiques": "48.8405799237,2.25841194391",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        31: {
            "attributes": {
                "Identifiant station": "7002",
                "Nom station": "Vaneau - Sèvres",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:38+01:00",
                "Coordonnées géographiques": "48.8485632331,2.32042182593",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        32: {
            "attributes": {
                "Identifiant station": "43002",
                "Nom station": "Lagny - Victor Basch",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8488414719,2.42632541893",
                "Nom communes équipées": "Vincennes",
                "Code INSEE communes équipées": "",
            }
        },
        33: {
            "attributes": {
                "Identifiant station": "5016",
                "Nom station": "Thouin - Cardinal Lemoine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:27+01:00",
                "Coordonnées géographiques": "48.8450471666,2.34946478513",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        34: {
            "attributes": {
                "Identifiant station": "21216",
                "Nom station": "Louis Lejeune - Barbès",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:25+01:00",
                "Coordonnées géographiques": "48.8180741898,2.32670098543",
                "Nom communes équipées": "Montrouge",
                "Code INSEE communes équipées": "",
            }
        },
        35: {
            "attributes": {
                "Identifiant station": "15047",
                "Nom station": "Morillons - Dantzig",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "52",
                "Nombre bornettes libres": "51",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:25+01:00",
                "Coordonnées géographiques": "48.8331014995,2.29938000441",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        36: {
            "attributes": {
                "Identifiant station": "22207",
                "Nom station": "Rond-point du Dr Albert Schweitzer",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.7898091899,2.31439400464",
                "Nom communes équipées": "Bagneux",
                "Code INSEE communes équipées": "",
            }
        },
        37: {
            "attributes": {
                "Identifiant station": "17048",
                "Nom station": "De Toqueville - Terrasse",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:53+01:00",
                "Coordonnées géographiques": "48.882428,2.314002",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        38: {
            "attributes": {
                "Identifiant station": "13030",
                "Nom station": "Italie - Tolbiac",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:24+01:00",
                "Coordonnées géographiques": "48.8255722946,2.35761713237",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        39: {
            "attributes": {
                "Identifiant station": "17025",
                "Nom station": "Chazelles - Courcelles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:31+01:00",
                "Coordonnées géographiques": "48.879406605,2.30344556272",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        40: {
            "attributes": {
                "Identifiant station": "19027",
                "Nom station": "Serrurier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:35+01:00",
                "Coordonnées géographiques": "48.8804934032,2.39806417376",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        41: {
            "attributes": {
                "Identifiant station": "20143",
                "Nom station": "Ramponeau - Belleville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "44",
                "Nombre bornettes libres": "41",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8710311376,2.37905685211",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        42: {
            "attributes": {
                "Identifiant station": "44017",
                "Nom station": "Jules Guesde - Pont du Port à l'Anglais",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:57+01:00",
                "Coordonnées géographiques": "48.7962880709,2.41721212864",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        43: {
            "attributes": {
                "Identifiant station": "31024",
                "Nom station": "Romainville - Vaillant-Couturier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:12+01:00",
                "Coordonnées géographiques": "48.86785,2.446748",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        44: {
            "attributes": {
                "Identifiant station": "48003",
                "Nom station": "Youri Gagarine - Karl Marx",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:10+01:00",
                "Coordonnées géographiques": "48.7844848508,2.3590843007",
                "Nom communes équipées": "Villejuif",
                "Code INSEE communes équipées": "",
            }
        },
        45: {
            "attributes": {
                "Identifiant station": "15026",
                "Nom station": "Bir Hakeim",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "32",
                "Vélos mécaniques disponibles": "27",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:44+01:00",
                "Coordonnées géographiques": "48.8538440543,2.28970534822",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        46: {
            "attributes": {
                "Identifiant station": "18134",
                "Nom station": "Ney - Porte de Clignancourt",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:44+01:00",
                "Coordonnées géographiques": "48.8978867,2.3458001",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        47: {
            "attributes": {
                "Identifiant station": "19033",
                "Nom station": "Cambrai - Benjamin Constant",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:39+01:00",
                "Coordonnées géographiques": "48.8959473685,2.38113380225",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        48: {
            "attributes": {
                "Identifiant station": "14002",
                "Nom station": "Edgar Quinet - Raspail",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "37",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:08+01:00",
                "Coordonnées géographiques": "48.8394026667,2.32971478254",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        49: {
            "attributes": {
                "Identifiant station": "16138",
                "Nom station": "Saint-Cloud - Hippodrome",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:56+01:00",
                "Coordonnées géographiques": "48.8577214889,2.2549109906",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        50: {
            "attributes": {
                "Identifiant station": "9014",
                "Nom station": "Faubourg Montmartre - La Fayette",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:16+01:00",
                "Coordonnées géographiques": "48.8752545491,2.3408536613",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        51: {
            "attributes": {
                "Identifiant station": "15068",
                "Nom station": "Général Martial Valin - Pont du Garigliano",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:21+01:00",
                "Coordonnées géographiques": "48.8382309427,2.27050602436",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        52: {
            "attributes": {
                "Identifiant station": "8053",
                "Nom station": "Place Georges Guillaumin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:32+01:00",
                "Coordonnées géographiques": "48.874750204,2.30177143533",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        53: {
            "attributes": {
                "Identifiant station": "7004",
                "Nom station": "Raspail - Varenne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:23+01:00",
                "Coordonnées géographiques": "48.8531478165,2.32639051974",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        54: {
            "attributes": {
                "Identifiant station": "17032",
                "Nom station": "Jean Oestreicher - Porte de Champerret",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:27+01:00",
                "Coordonnées géographiques": "48.8865558007,2.28864043951",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        55: {
            "attributes": {
                "Identifiant station": "92002",
                "Nom station": "Théâtre des Amandiers - Palais des sports",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:28+01:00",
                "Coordonnées géographiques": "48.8926485268,2.21369415522",
                "Nom communes équipées": "Nanterre",
                "Code INSEE communes équipées": "",
            }
        },
        56: {
            "attributes": {
                "Identifiant station": "10029",
                "Nom station": "Dunkerque - Rocroy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:51+01:00",
                "Coordonnées géographiques": "48.880726,2.351464",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        57: {
            "attributes": {
                "Identifiant station": "6029",
                "Nom station": "Vaugirard - Monsieur le Prince",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:58+01:00",
                "Coordonnées géographiques": "48.8489514416,2.34113294631",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        58: {
            "attributes": {
                "Identifiant station": "13118",
                "Nom station": "Regnault - Patay",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "63",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "46",
                "Vélos mécaniques disponibles": "42",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:12+01:00",
                "Coordonnées géographiques": "48.8243578222,2.37706515938",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        59: {
            "attributes": {
                "Identifiant station": "20035",
                "Nom station": "Pyrénées - Ménilmontant",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:38+01:00",
                "Coordonnées géographiques": "48.8693268535,2.39508457482",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        60: {
            "attributes": {
                "Identifiant station": "12031",
                "Nom station": "Gabriel Lamé",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:26+01:00",
                "Coordonnées géographiques": "48.8351212784,2.38553456962",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        61: {
            "attributes": {
                "Identifiant station": "2017",
                "Nom station": "Caire - Dussoubs",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "35",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:14+01:00",
                "Coordonnées géographiques": "48.8678534211,2.3493643105",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        62: {
            "attributes": {
                "Identifiant station": "5022",
                "Nom station": "Boulangers - Cardinal Lemoine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:41+01:00",
                "Coordonnées géographiques": "48.8462904,2.3521495",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        63: {
            "attributes": {
                "Identifiant station": "9008",
                "Nom station": "Tour d'Auvergne - Rodier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:32+01:00",
                "Coordonnées géographiques": "48.879161,2.343698",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        64: {
            "attributes": {
                "Identifiant station": "9106",
                "Nom station": "Boudreau - Auber",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "14",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:26+01:00",
                "Coordonnées géographiques": "48.87219,2.329423",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        65: {
            "attributes": {
                "Identifiant station": "8103",
                "Nom station": "Artois - Berri",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:17+01:00",
                "Coordonnées géographiques": "48.8739352124,2.30642583221",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        66: {
            "attributes": {
                "Identifiant station": "19105",
                "Nom station": "Ambroise Rendu - Porte Brunet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:44+01:00",
                "Coordonnées géographiques": "48.883667,2.395733",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        67: {
            "attributes": {
                "Identifiant station": "8048",
                "Nom station": "Marceau - Chaillot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:46+01:00",
                "Coordonnées géographiques": "48.8688339499,2.29899678379",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        68: {
            "attributes": {
                "Identifiant station": "31103",
                "Nom station": "Gare Rosny-Bois-Perrier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "11",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:08+01:00",
                "Coordonnées géographiques": "48.8811502334,2.48043682426",
                "Nom communes équipées": "Rosny-sous-Bois",
                "Code INSEE communes équipées": "",
            }
        },
        69: {
            "attributes": {
                "Identifiant station": "14013",
                "Nom station": "Liard - Amiral Mouchez",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "14",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:50+01:00",
                "Coordonnées géographiques": "48.8214169,2.342167",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        70: {
            "attributes": {
                "Identifiant station": "6001",
                "Nom station": "Institut de France",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:06+01:00",
                "Coordonnées géographiques": "48.8576156786,2.33583135304",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        71: {
            "attributes": {
                "Identifiant station": "7011",
                "Nom station": "Las Cases - Bourgogne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:44+01:00",
                "Coordonnées géographiques": "48.8590911047,2.3185685277",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        72: {
            "attributes": {
                "Identifiant station": "22409",
                "Nom station": "Henri Barbusse - Stalingrad",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:39+01:00",
                "Coordonnées géographiques": "48.8141837487,2.28728625923",
                "Nom communes équipées": "Malakoff",
                "Code INSEE communes équipées": "",
            }
        },
        73: {
            "attributes": {
                "Identifiant station": "18111",
                "Nom station": "Félix Ziem - Armand Gauthier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:38+01:00",
                "Coordonnées géographiques": "48.8894963049,2.3333936588",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        74: {
            "attributes": {
                "Identifiant station": "19015",
                "Nom station": "Thionville - Ourcq",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:26+01:00",
                "Coordonnées géographiques": "48.8891761374,2.38336533308",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        75: {
            "attributes": {
                "Identifiant station": "17010",
                "Nom station": "Jonquière - Docteur Paul Brousse",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:36+01:00",
                "Coordonnées géographiques": "48.8947998167,2.31870521217",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        76: {
            "attributes": {
                "Identifiant station": "17012",
                "Nom station": "Mairie du 17ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:26+01:00",
                "Coordonnées géographiques": "48.8841618487,2.3220794311",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        77: {
            "attributes": {
                "Identifiant station": "41303",
                "Nom station": "Gare de Nogent-le-Perreux",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:29+01:00",
                "Coordonnées géographiques": "48.8384255335,2.49369330704",
                "Nom communes équipées": "Nogent-sur-Marne",
                "Code INSEE communes équipées": "",
            }
        },
        78: {
            "attributes": {
                "Identifiant station": "8036",
                "Nom station": "Lisbonne - Monceau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "13",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:36+01:00",
                "Coordonnées géographiques": "48.8774556768,2.30969422142",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        79: {
            "attributes": {
                "Identifiant station": "16110",
                "Nom station": "Octave Feuillet - Albéric Magnard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:54+01:00",
                "Coordonnées géographiques": "48.8610690295,2.27302830666",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        80: {
            "attributes": {
                "Identifiant station": "1122",
                "Nom station": "Pont-Neuf - Quai du Louvre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-01-22T22:50:04+01:00",
                "Coordonnées géographiques": "48.8582958983,2.34237145633",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        81: {
            "attributes": {
                "Identifiant station": "43009",
                "Nom station": "Place Jean-Spire-Lemaître",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "46",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:01+01:00",
                "Coordonnées géographiques": "48.8468428627,2.45206207037",
                "Nom communes équipées": "Vincennes",
                "Code INSEE communes équipées": "",
            }
        },
        82: {
            "attributes": {
                "Identifiant station": "12151",
                "Nom station": "Gare de Lyon - Place Louis Armand",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "60",
                "Nombre bornettes libres": "38",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:16+01:00",
                "Coordonnées géographiques": "48.8455150898,2.37264984463",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        83: {
            "attributes": {
                "Identifiant station": "9001",
                "Nom station": "Bellefond - Poissonnière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:17+01:00",
                "Coordonnées géographiques": "48.8775989705,2.34846375883",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        84: {
            "attributes": {
                "Identifiant station": "9022",
                "Nom station": "Rossini - Laffitte",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "31",
                "Vélos mécaniques disponibles": "26",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:37+01:00",
                "Coordonnées géographiques": "48.8733462751,2.3379808677",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        85: {
            "attributes": {
                "Identifiant station": "8016",
                "Nom station": "Square Louis XVI",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "27",
                "Vélos mécaniques disponibles": "19",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:09+01:00",
                "Coordonnées géographiques": "48.873204401,2.32365883074",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        86: {
            "attributes": {
                "Identifiant station": "16026",
                "Nom station": "Ranelagh - Mozart",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:36+01:00",
                "Coordonnées géographiques": "48.8554439,2.2704418",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        87: {
            "attributes": {
                "Identifiant station": "10162",
                "Nom station": "Alexandre Parodi - Quai de Valmy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:25+01:00",
                "Coordonnées géographiques": "48.8793594194,2.36659616232",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        88: {
            "attributes": {
                "Identifiant station": "11027",
                "Nom station": "Saint-Ambroise - Parmentier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:15+01:00",
                "Coordonnées géographiques": "48.8621478445,2.3772348091",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        89: {
            "attributes": {
                "Identifiant station": "10115",
                "Nom station": "Granges aux Belles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:32+01:00",
                "Coordonnées géographiques": "48.8761373391,2.36808449794",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        90: {
            "attributes": {
                "Identifiant station": "17043",
                "Nom station": "Belidor - Gouvion-Saint-Cyr",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:36+01:00",
                "Coordonnées géographiques": "48.8802215405,2.2854680568",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        91: {
            "attributes": {
                "Identifiant station": "5005",
                "Nom station": "Gay Lussac - Saint-Jacques",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "11",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:58+01:00",
                "Coordonnées géographiques": "48.8447302561,2.34192394487",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        92: {
            "attributes": {
                "Identifiant station": "19026",
                "Nom station": "Place de Rhin et Danube",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:25+01:00",
                "Coordonnées géographiques": "48.8819276586,2.39258676767",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        93: {
            "attributes": {
                "Identifiant station": "18024",
                "Nom station": "Clignancourt - Ordener",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:57+01:00",
                "Coordonnées géographiques": "48.8914577262,2.34863647806",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        94: {
            "attributes": {
                "Identifiant station": "45003",
                "Nom station": "11 Novembre 1918 - 8 Mai 1945",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:51+01:00",
                "Coordonnées géographiques": "48.8088953377,2.53824211657",
                "Nom communes équipées": "Champigny-sur-Marne",
                "Code INSEE communes équipées": "",
            }
        },
        95: {
            "attributes": {
                "Identifiant station": "7018",
                "Nom station": "Ségur - d'Estrées",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:21+01:00",
                "Coordonnées géographiques": "48.851419979,2.30970449746",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        96: {
            "attributes": {
                "Identifiant station": "12111",
                "Nom station": "Claude Decaen - Cannebière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:56+01:00",
                "Coordonnées géographiques": "48.8376141704,2.3972537822",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        97: {
            "attributes": {
                "Identifiant station": "9002",
                "Nom station": "Abbeville - Faubourg Poissonnière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "14",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:51+01:00",
                "Coordonnées géographiques": "48.879223,2.349147",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        98: {
            "attributes": {
                "Identifiant station": "8034",
                "Nom station": "Berri - Haussmann",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:01+01:00",
                "Coordonnées géographiques": "48.8747460474,2.3082953319",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        99: {
            "attributes": {
                "Identifiant station": "10161",
                "Nom station": "Gare de l'Est - Faubourg-Saint-Martin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:53+01:00",
                "Coordonnées géographiques": "48.8756426475,2.35945880413",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        100: {
            "attributes": {
                "Identifiant station": "19043",
                "Nom station": "Delphine Seyrig - Général Leclerc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "53",
                "Nombre bornettes libres": "47",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:45+01:00",
                "Coordonnées géographiques": "48.8939239631,2.39790290594",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        101: {
            "attributes": {
                "Identifiant station": "17040",
                "Nom station": "Pereire - Ternes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "48",
                "Nombre bornettes libres": "44",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:42+01:00",
                "Coordonnées géographiques": "48.8797395483,2.28801213205",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        102: {
            "attributes": {
                "Identifiant station": "41302",
                "Nom station": "Charles de Gaulle - Beauté",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "49",
                "Nombre bornettes libres": "43",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:01+01:00",
                "Coordonnées géographiques": "48.8364960231,2.47956265115",
                "Nom communes équipées": "Nogent-sur-Marne",
                "Code INSEE communes équipées": "",
            }
        },
        103: {
            "attributes": {
                "Identifiant station": "9038",
                "Nom station": "Douai - Bruxelles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:13+01:00",
                "Coordonnées géographiques": "48.8832078412,2.3308449949",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        104: {
            "attributes": {
                "Identifiant station": "15067",
                "Nom station": "Desaix - Edgar Faure",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "12",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:13+01:00",
                "Coordonnées géographiques": "48.8528046049,2.29307702986",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        105: {
            "attributes": {
                "Identifiant station": "8009",
                "Nom station": "Gare Saint-Lazare - Isly",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:31+01:00",
                "Coordonnées géographiques": "48.8747776776,2.32651580478",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        106: {
            "attributes": {
                "Identifiant station": "43401",
                "Nom station": "Gare RER - Canadiens",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "11",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:52+01:00",
                "Coordonnées géographiques": "48.8199530796,2.46417675246",
                "Nom communes équipées": "Joinville-le-Pont",
                "Code INSEE communes équipées": "",
            }
        },
        107: {
            "attributes": {
                "Identifiant station": "1001",
                "Nom station": "Quai de l'Horloge - Pont Neuf",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:19+01:00",
                "Coordonnées géographiques": "48.8570587391,2.34179828394",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        108: {
            "attributes": {
                "Identifiant station": "12128",
                "Nom station": "Pyramide - Ecole du Breuil",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:09+01:00",
                "Coordonnées géographiques": "48.8223797715,2.45868712664",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        109: {
            "attributes": {
                "Identifiant station": "17008",
                "Nom station": "Porte Pouchet-Bessières",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:25+01:00",
                "Coordonnées géographiques": "48.8978765,2.3230632",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        110: {
            "attributes": {
                "Identifiant station": "31022",
                "Nom station": "Mairie de Montreuil",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "52",
                "Nombre bornettes libres": "46",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:04+01:00",
                "Coordonnées géographiques": "48.8613145276,2.44061440229",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        111: {
            "attributes": {
                "Identifiant station": "15112",
                "Nom station": "Vigée-Lebrun - Anselme Payen",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:32+01:00",
                "Coordonnées géographiques": "48.8386839416,2.3110422492",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        112: {
            "attributes": {
                "Identifiant station": "46002",
                "Nom station": "Rouget de Lisle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "30",
                "Vélos mécaniques disponibles": "27",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:34+01:00",
                "Coordonnées géographiques": "48.7646154834,2.40711897612",
                "Nom communes équipées": "Choisy-le-Roi",
                "Code INSEE communes équipées": "",
            }
        },
        113: {
            "attributes": {
                "Identifiant station": "7025",
                "Nom station": "Octave Gréard - Tour Eiffel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "31",
                "Vélos mécaniques disponibles": "29",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:33+01:00",
                "Coordonnées géographiques": "48.8565029777,2.29317875433",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        114: {
            "attributes": {
                "Identifiant station": "11113",
                "Nom station": "Richard Lenoir - Republique",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "58",
                "Nombre bornettes libres": "55",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:01+01:00",
                "Coordonnées géographiques": "48.8667175,2.3692541",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        115: {
            "attributes": {
                "Identifiant station": "31706",
                "Nom station": "Château - République",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:59+01:00",
                "Coordonnées géographiques": "48.8629238006,2.4155035615",
                "Nom communes équipées": "Bagnolet",
                "Code INSEE communes équipées": "",
            }
        },
        116: {
            "attributes": {
                "Identifiant station": "15042",
                "Nom station": "Henry Farman",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8337243028,2.27138545364",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        117: {
            "attributes": {
                "Identifiant station": "10025",
                "Nom station": "Jemmapes - Ecluses Saint-Martin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "15",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:33+01:00",
                "Coordonnées géographiques": "48.8774851919,2.36600674689",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        118: {
            "attributes": {
                "Identifiant station": "19044",
                "Nom station": "Grands Moulins de Pantin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:29+01:00",
                "Coordonnées géographiques": "48.8973836972,2.39599853754",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        119: {
            "attributes": {
                "Identifiant station": "26003",
                "Nom station": "Argenteuil - Voltaire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:41+01:00",
                "Coordonnées géographiques": "48.9186486596,2.28141121566",
                "Nom communes équipées": "Asnières-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        120: {
            "attributes": {
                "Identifiant station": "43008",
                "Nom station": "Fontenay - Libération",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:22+01:00",
                "Coordonnées géographiques": "48.8478030636,2.44514197111",
                "Nom communes équipées": "Vincennes",
                "Code INSEE communes équipées": "",
            }
        },
        121: {
            "attributes": {
                "Identifiant station": "17004",
                "Nom station": "Legendre - Clichy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "35",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:52+01:00",
                "Coordonnées géographiques": "48.8889616186,2.32252897509",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        122: {
            "attributes": {
                "Identifiant station": "33102",
                "Nom station": "Gabriel Péri - Francisco Ferrer",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:37+01:00",
                "Coordonnées géographiques": "48.8863371024,2.40733891726",
                "Nom communes équipées": "Le Pré-Saint-Gervais",
                "Code INSEE communes équipées": "",
            }
        },
        123: {
            "attributes": {
                "Identifiant station": "19003",
                "Nom station": "Quai de la Seine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:21+01:00",
                "Coordonnées géographiques": "48.8844922384,2.37039417028",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        124: {
            "attributes": {
                "Identifiant station": "8045",
                "Nom station": "Place de la Reine Astrid",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "58",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "46",
                "Vélos mécaniques disponibles": "27",
                "Vélos électriques disponibles": "19",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:56+01:00",
                "Coordonnées géographiques": "48.8649209825,2.30254918239",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        125: {
            "attributes": {
                "Identifiant station": "10008",
                "Nom station": "Cité Riverin - Château d'Eau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:48+01:00",
                "Coordonnées géographiques": "48.8707024236,2.3587398639",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        126: {
            "attributes": {
                "Identifiant station": "21312",
                "Nom station": "Lucie Aubrac - Fort",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:53+01:00",
                "Coordonnées géographiques": "48.818144834,2.27118462324",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        127: {
            "attributes": {
                "Identifiant station": "18107",
                "Nom station": "Chartres - La Charbonnière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:10+01:00",
                "Coordonnées géographiques": "48.8847601,2.3534657",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        128: {
            "attributes": {
                "Identifiant station": "18011",
                "Nom station": "Pajol - Place Herbert",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "38",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8927825,2.3635598",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        129: {
            "attributes": {
                "Identifiant station": "5020",
                "Nom station": "Institut du Monde Arabe - Saint-Germain",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "36",
                "Vélos mécaniques disponibles": "25",
                "Vélos électriques disponibles": "11",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:45+01:00",
                "Coordonnées géographiques": "48.8490632995,2.35580865294",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        130: {
            "attributes": {
                "Identifiant station": "19103",
                "Nom station": "Manin - Carrières d'Amérique",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:11+01:00",
                "Coordonnées géographiques": "48.8846721421,2.3919698596",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        131: {
            "attributes": {
                "Identifiant station": "92007",
                "Nom station": "Pesaro - Préfecture",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.8957704628,2.22390834242",
                "Nom communes équipées": "Nanterre",
                "Code INSEE communes équipées": "",
            }
        },
        132: {
            "attributes": {
                "Identifiant station": "21306",
                "Nom station": "Place Lafayette",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "62",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "58",
                "Vélos mécaniques disponibles": "44",
                "Vélos électriques disponibles": "14",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:07+01:00",
                "Coordonnées géographiques": "48.8296549755,2.26386755705",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        133: {
            "attributes": {
                "Identifiant station": "9028",
                "Nom station": "Moncey - Blanche",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:21+01:00",
                "Coordonnées géographiques": "48.8800658747,2.33108103275",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        134: {
            "attributes": {
                "Identifiant station": "14138",
                "Nom station": "Porte de Vanves",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:25+01:00",
                "Coordonnées géographiques": "48.8280128494,2.30568788946",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        135: {
            "attributes": {
                "Identifiant station": "10107",
                "Nom station": "Gare du Nord - Hôpital Lariboisière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "49",
                "Nombre bornettes libres": "45",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:52+01:00",
                "Coordonnées géographiques": "48.881949,2.352339",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        136: {
            "attributes": {
                "Identifiant station": "46003",
                "Nom station": "Conservatoire de Musique",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "19",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:37+01:00",
                "Coordonnées géographiques": "48.7709358667,2.40340143442",
                "Nom communes équipées": "Choisy-le-Roi",
                "Code INSEE communes équipées": "",
            }
        },
        137: {
            "attributes": {
                "Identifiant station": "20042",
                "Nom station": "Pyrénées - Emmery",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "39",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:47+01:00",
                "Coordonnées géographiques": "48.8720102813,2.39203959703",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        138: {
            "attributes": {
                "Identifiant station": "41209",
                "Nom station": "Cimetière de Fontenay-sous-Bois",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "15",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:58+01:00",
                "Coordonnées géographiques": "48.8470617336,2.47878223658",
                "Nom communes équipées": "Fontenay-sous-Bois",
                "Code INSEE communes équipées": "",
            }
        },
        139: {
            "attributes": {
                "Identifiant station": "11002",
                "Nom station": "Roquette - Thiéré",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:41+01:00",
                "Coordonnées géographiques": "48.8550793058,2.37336863187",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        140: {
            "attributes": {
                "Identifiant station": "10024",
                "Nom station": "Buisson Saint-Louis - Saint-Maur",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:45+01:00",
                "Coordonnées géographiques": "48.8717468493,2.37225283288",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        141: {
            "attributes": {
                "Identifiant station": "16039",
                "Nom station": "Exelmans - Versailles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "49",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "18",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:54+01:00",
                "Coordonnées géographiques": "48.8409749109,2.26439025253",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        142: {
            "attributes": {
                "Identifiant station": "20037",
                "Nom station": "Gambetta - Saint Fargeau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:54+01:00",
                "Coordonnées géographiques": "48.8712446104,2.40390568972",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        143: {
            "attributes": {
                "Identifiant station": "11023",
                "Nom station": "Saint-Maur - Roquette",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "44",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:11+01:00",
                "Coordonnées géographiques": "48.8586887741,2.38355399457",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        144: {
            "attributes": {
                "Identifiant station": "5018",
                "Nom station": "Mutualité",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "32",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:34+01:00",
                "Coordonnées géographiques": "48.8482255727,2.35028732568",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        145: {
            "attributes": {
                "Identifiant station": "31009",
                "Nom station": "Place Du Général De Gaulle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:54+01:00",
                "Coordonnées géographiques": "48.8687591,2.4329931",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        146: {
            "attributes": {
                "Identifiant station": "8017",
                "Nom station": "Rocher - Laborde",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "65",
                "Nombre bornettes libres": "49",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.8759024071,2.32261631638",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        147: {
            "attributes": {
                "Identifiant station": "11047",
                "Nom station": "Amelot - Saint-Sébastien",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:24+01:00",
                "Coordonnées géographiques": "48.8611421628,2.36779961811",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        148: {
            "attributes": {
                "Identifiant station": "15054",
                "Nom station": "Lecourbe - Jean Maridor",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:33+01:00",
                "Coordonnées géographiques": "48.8379636615,2.28773921728",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        149: {
            "attributes": {
                "Identifiant station": "17030",
                "Nom station": "Pereire - Rennequin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:29+01:00",
                "Coordonnées géographiques": "48.8836478289,2.29524470866",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        150: {
            "attributes": {
                "Identifiant station": "15106",
                "Nom station": "Grenelle - Violet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "19",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:42+01:00",
                "Coordonnées géographiques": "48.8499303551,2.29461204261",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        151: {
            "attributes": {
                "Identifiant station": "5032",
                "Nom station": "Panthéon - Valette",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "27",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "24",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:40+01:00",
                "Coordonnées géographiques": "48.8470205717,2.34649826197",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        152: {
            "attributes": {
                "Identifiant station": "14008",
                "Nom station": "Mouton Duvernet - Général Leclerc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:08+01:00",
                "Coordonnées géographiques": "48.8316325183,2.32931211591",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        153: {
            "attributes": {
                "Identifiant station": "14020",
                "Nom station": "Cimetière de Montrouge",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "58",
                "Nombre bornettes libres": "56",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:29+01:00",
                "Coordonnées géographiques": "48.822009,2.322188699",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        154: {
            "attributes": {
                "Identifiant station": "14106",
                "Nom station": "Marc Sangnier - Porte de Vanves",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:29+01:00",
                "Coordonnées géographiques": "48.8252186836,2.31071166694",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        155: {
            "attributes": {
                "Identifiant station": "26002",
                "Nom station": "Redoute - Les Courtilles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:34+01:00",
                "Coordonnées géographiques": "48.9298671945,2.28366963565",
                "Nom communes équipées": "Asnières-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        156: {
            "attributes": {
                "Identifiant station": "22406",
                "Nom station": "Guy Môquet - Etienne Dolet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:21+01:00",
                "Coordonnées géographiques": "48.8152700999,2.298078277",
                "Nom communes équipées": "Malakoff",
                "Code INSEE communes équipées": "",
            }
        },
        157: {
            "attributes": {
                "Identifiant station": "6006",
                "Nom station": "Notre-Dame-des-Champs - Stanislas",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:35+01:00",
                "Coordonnées géographiques": "48.8447992886,2.32933775589",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        158: {
            "attributes": {
                "Identifiant station": "35002",
                "Nom station": "Jean Jaurès - Presles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:34+01:00",
                "Coordonnées géographiques": "48.907134415,2.39600572805",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        159: {
            "attributes": {
                "Identifiant station": "8010",
                "Nom station": "Liège - Moscou",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:20+01:00",
                "Coordonnées géographiques": "48.8795431,2.3264628",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        160: {
            "attributes": {
                "Identifiant station": "2111",
                "Nom station": "Chabanais - Petits Champs",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "26",
                "Vélos mécaniques disponibles": "19",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:27+01:00",
                "Coordonnées géographiques": "48.8670374019,2.33671601862",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        161: {
            "attributes": {
                "Identifiant station": "19101",
                "Nom station": "Square Simon Bolivar",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:46+01:00",
                "Coordonnées géographiques": "48.8750898883,2.38263040781",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        162: {
            "attributes": {
                "Identifiant station": "8054",
                "Nom station": "Bertie Albrecht - Hoche",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.8761403983,2.30130313831",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        163: {
            "attributes": {
                "Identifiant station": "14003",
                "Nom station": "Schoelcher - Raspail",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "35",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:27+01:00",
                "Coordonnées géographiques": "48.8368173353,2.33127940804",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        164: {
            "attributes": {
                "Identifiant station": "35010",
                "Nom station": "Hoche - Jean Lolive",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:02+01:00",
                "Coordonnées géographiques": "48.891102392,2.40284353495",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        165: {
            "attributes": {
                "Identifiant station": "18003",
                "Nom station": "Cimetière de Montmartre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "42",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:39+01:00",
                "Coordonnées géographiques": "48.8864724661,2.33289018273",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        166: {
            "attributes": {
                "Identifiant station": "15110",
                "Nom station": "Place Robert Guillemard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.8361917281,2.28112671601",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        167: {
            "attributes": {
                "Identifiant station": "10016",
                "Nom station": "Récollets - Square Villemin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:21+01:00",
                "Coordonnées géographiques": "48.8750346559,2.3598012386",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        168: {
            "attributes": {
                "Identifiant station": "10042",
                "Nom station": "Enghien - Faubourg Poissonnière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-02-21T21:15:22+01:00",
                "Coordonnées géographiques": "48.8724200631,2.34839523628",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        169: {
            "attributes": {
                "Identifiant station": "21212",
                "Nom station": "Marne - Germain Dardan",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:40+01:00",
                "Coordonnées géographiques": "48.811542704,2.30307072401",
                "Nom communes équipées": "Montrouge",
                "Code INSEE communes équipées": "",
            }
        },
        170: {
            "attributes": {
                "Identifiant station": "31008",
                "Nom station": "François Debergue - Croix de Chavaux",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "57",
                "Nombre bornettes libres": "46",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:12+01:00",
                "Coordonnées géographiques": "48.8574217081,2.43757814169",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        171: {
            "attributes": {
                "Identifiant station": "42706",
                "Nom station": "Roger Salengro - Fontainebleau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:30+01:00",
                "Coordonnées géographiques": "48.8145486973,2.36116636544",
                "Nom communes équipées": "Le Kremlin-Bicêtre",
                "Code INSEE communes équipées": "",
            }
        },
        172: {
            "attributes": {
                "Identifiant station": "21015",
                "Nom station": "Nationale - Place Jules Guesde",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:20+01:00",
                "Coordonnées géographiques": "48.8275396246,2.24160522223",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        173: {
            "attributes": {
                "Identifiant station": "33017",
                "Nom station": "Anatole France - President Roosevelt",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:34+01:00",
                "Coordonnées géographiques": "48.9155156,2.3814103",
                "Nom communes équipées": "Aubervilliers",
                "Code INSEE communes équipées": "",
            }
        },
        174: {
            "attributes": {
                "Identifiant station": "41404",
                "Nom station": "Carnot - Pâquerettes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:18+01:00",
                "Coordonnées géographiques": "48.7959293943,2.42618411779",
                "Nom communes équipées": "Alfortville",
                "Code INSEE communes équipées": "",
            }
        },
        175: {
            "attributes": {
                "Identifiant station": "20112",
                "Nom station": "Jourdain - Place des Grandes Rigoles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:20+01:00",
                "Coordonnées géographiques": "48.8740311373,2.38959476352",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        176: {
            "attributes": {
                "Identifiant station": "22203",
                "Nom station": "Tertres - Fontaine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:22+01:00",
                "Coordonnées géographiques": "48.7875536547,2.30623573065",
                "Nom communes équipées": "Bagneux",
                "Code INSEE communes équipées": "",
            }
        },
        177: {
            "attributes": {
                "Identifiant station": "13001",
                "Nom station": "Port-Royal - Hôpital du Val-de-Grâce",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:07+01:00",
                "Coordonnées géographiques": "48.8376682728,2.34481383466",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        178: {
            "attributes": {
                "Identifiant station": "41202",
                "Nom station": "Gare Rer de Fontenay sous Bois",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "61",
                "Nombre bornettes libres": "53",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:40+01:00",
                "Coordonnées géographiques": "48.8436279,2.4630465",
                "Nom communes équipées": "Fontenay-sous-Bois",
                "Code INSEE communes équipées": "",
            }
        },
        179: {
            "attributes": {
                "Identifiant station": "15104",
                "Nom station": "Hôpital Européen Georges Pompidou",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "64",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "50",
                "Vélos mécaniques disponibles": "42",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:39+01:00",
                "Coordonnées géographiques": "48.8376953192,2.27537423372",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        180: {
            "attributes": {
                "Identifiant station": "35020",
                "Nom station": "Marcelle - Thalie",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:44+01:00",
                "Coordonnées géographiques": "48.8835462,2.4137883",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        181: {
            "attributes": {
                "Identifiant station": "44006",
                "Nom station": "Concorde - Stalingrad",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:48+01:00",
                "Coordonnées géographiques": "48.7985844,2.3821302",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        182: {
            "attributes": {
                "Identifiant station": "25003",
                "Nom station": "Henri Ste-Claire Déville - Paul Héroult",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.892663,2.172411",
                "Nom communes équipées": "Rueil-Malmaison",
                "Code INSEE communes équipées": "",
            }
        },
        183: {
            "attributes": {
                "Identifiant station": "22202",
                "Nom station": "Jean Marin Naudin - Stalingrad",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:57+01:00",
                "Coordonnées géographiques": "48.8057138974,2.32020232826",
                "Nom communes équipées": "Bagneux",
                "Code INSEE communes équipées": "",
            }
        },
        184: {
            "attributes": {
                "Identifiant station": "31003",
                "Nom station": "Paris - Emile Zola",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8548980958,2.41866856813",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        185: {
            "attributes": {
                "Identifiant station": "19120",
                "Nom station": "Carducci - Place Hannah Arendt",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:08+01:00",
                "Coordonnées géographiques": "48.8774366815,2.38607436419",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        186: {
            "attributes": {
                "Identifiant station": "9116",
                "Nom station": "Victoire - Chaussée d'Antin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "12",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:45+01:00",
                "Coordonnées géographiques": "48.875065791,2.33167380095",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        187: {
            "attributes": {
                "Identifiant station": "5004",
                "Nom station": "Saint-Jacques - Val de Grâce",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:35+01:00",
                "Coordonnées géographiques": "48.8417514,2.3413893",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        188: {
            "attributes": {
                "Identifiant station": "21008",
                "Nom station": "Victor Hugo - Galliéni",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "12",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:20+01:00",
                "Coordonnées géographiques": "48.8380201544,2.24608451128",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        189: {
            "attributes": {
                "Identifiant station": "12114",
                "Nom station": "Victor Hugo - Jeanne Jugan",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:45+01:00",
                "Coordonnées géographiques": "48.843949064,2.41508139483",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        190: {
            "attributes": {
                "Identifiant station": "1119",
                "Nom station": "Cossonnerie - Sébastopol",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8613266591,2.34941963106",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        191: {
            "attributes": {
                "Identifiant station": "18028",
                "Nom station": "Championnet - Vauvenargues",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:24+01:00",
                "Coordonnées géographiques": "48.8940187474,2.33198728412",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        192: {
            "attributes": {
                "Identifiant station": "11039",
                "Nom station": "Richard Lenoir - Jean-Pierre Timbaud",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:48+01:00",
                "Coordonnées géographiques": "48.8658042767,2.36933707497",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        193: {
            "attributes": {
                "Identifiant station": "15105",
                "Nom station": "Fédération - Suffren",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "18",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.8528281534,2.29787763208",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        194: {
            "attributes": {
                "Identifiant station": "17031",
                "Nom station": "Place de la Porte de Champerret",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:55+01:00",
                "Coordonnées géographiques": "48.8856174163,2.29078141128",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        195: {
            "attributes": {
                "Identifiant station": "18031",
                "Nom station": "Place Albert Kahn",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "44",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:23+01:00",
                "Coordonnées géographiques": "48.8957969798,2.34558558413",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        196: {
            "attributes": {
                "Identifiant station": "13050",
                "Nom station": "Quai Panhard et Levassor",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "57",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "52",
                "Vélos mécaniques disponibles": "50",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:25+01:00",
                "Coordonnées géographiques": "48.8283748281,2.38427374581",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        197: {
            "attributes": {
                "Identifiant station": "42014",
                "Nom station": "Place Marcel Cachin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "29",
                "Vélos mécaniques disponibles": "22",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:51+01:00",
                "Coordonnées géographiques": "48.8141144236,2.39090770483",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        198: {
            "attributes": {
                "Identifiant station": "11030",
                "Nom station": "Oberkampf - République",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:17+01:00",
                "Coordonnées géographiques": "48.8653160871,2.37611478491",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        199: {
            "attributes": {
                "Identifiant station": "5124",
                "Nom station": "Erasme - Ulm",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:19+01:00",
                "Coordonnées géographiques": "48.8426437527,2.34459467232",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        200: {
            "attributes": {
                "Identifiant station": "19024",
                "Nom station": "Botzaris - Parc des Buttes Chaumont",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:25+01:00",
                "Coordonnées géographiques": "48.877951,2.381464",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        201: {
            "attributes": {
                "Identifiant station": "18046",
                "Nom station": "Etienne Jodelle - Saint Ouen",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:35+01:00",
                "Coordonnées géographiques": "48.8879401639,2.32612442031",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        202: {
            "attributes": {
                "Identifiant station": "17105",
                "Nom station": "Place Madeleine Daniélou",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "39",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:37+01:00",
                "Coordonnées géographiques": "48.883498,2.282015",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        203: {
            "attributes": {
                "Identifiant station": "31102",
                "Nom station": "Boissière - Etienne Dolet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:07+01:00",
                "Coordonnées géographiques": "48.876111,2.4677312",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        204: {
            "attributes": {
                "Identifiant station": "5012",
                "Nom station": "Estrapade - Ulm",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:02+01:00",
                "Coordonnées géographiques": "48.8451277283,2.34545681704",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        205: {
            "attributes": {
                "Identifiant station": "44013",
                "Nom station": "Edouard Tremblay - Julian Grimau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:42+01:00",
                "Coordonnées géographiques": "48.7813838251,2.37235084176",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        206: {
            "attributes": {
                "Identifiant station": "20134",
                "Nom station": "Lieutenant Chauré - Etienne Marey",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:02+01:00",
                "Coordonnées géographiques": "48.8674619541,2.40597099066",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        207: {
            "attributes": {
                "Identifiant station": "15109",
                "Nom station": "Cevennes - Bergers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8424076513,2.28103771806",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        208: {
            "attributes": {
                "Identifiant station": "15103",
                "Nom station": "Cauchy - Cévennes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "51",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "40",
                "Vélos mécaniques disponibles": "38",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:19+01:00",
                "Coordonnées géographiques": "48.843372571,2.27513283491",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        209: {
            "attributes": {
                "Identifiant station": "21209",
                "Nom station": "Molière - République",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:46+01:00",
                "Coordonnées géographiques": "48.8111158139,2.31457386244",
                "Nom communes équipées": "Montrouge",
                "Code INSEE communes équipées": "",
            }
        },
        210: {
            "attributes": {
                "Identifiant station": "18009",
                "Nom station": "Jean-François Lépine - Stephenson",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:39:50+01:00",
                "Coordonnées géographiques": "48.886114668,2.35685562753",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        211: {
            "attributes": {
                "Identifiant station": "21005",
                "Nom station": "Hôtel de Ville de Boulogne Billancourt",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:15+01:00",
                "Coordonnées géographiques": "48.8349588517,2.24168032408",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        212: {
            "attributes": {
                "Identifiant station": "16108",
                "Nom station": "Sablons - Georges Mandel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "35",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:44+01:00",
                "Coordonnées géographiques": "48.863875,2.28189",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        213: {
            "attributes": {
                "Identifiant station": "14124",
                "Nom station": "David Weill - Parc Montsouris",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:40+01:00",
                "Coordonnées géographiques": "48.8210923743,2.33372569084",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        214: {
            "attributes": {
                "Identifiant station": "11024",
                "Nom station": "Mairie du 11ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "51",
                "Nombre bornettes libres": "37",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.858925,2.3789759",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        215: {
            "attributes": {
                "Identifiant station": "15133",
                "Nom station": "Saint Lambert - Blomet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:51+01:00",
                "Coordonnées géographiques": "48.8365914815,2.29305569082",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        216: {
            "attributes": {
                "Identifiant station": "24001",
                "Nom station": "Arche - Léonard de Vinci",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:41+01:00",
                "Coordonnées géographiques": "48.8979097249,2.23837230436",
                "Nom communes équipées": "Courbevoie",
                "Code INSEE communes équipées": "",
            }
        },
        217: {
            "attributes": {
                "Identifiant station": "18022",
                "Nom station": "Marcadet - Barbès",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "35",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:46+01:00",
                "Coordonnées géographiques": "48.890451121,2.34905739297",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        218: {
            "attributes": {
                "Identifiant station": "8025",
                "Nom station": "Miromesnil - La Boétie",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:04+01:00",
                "Coordonnées géographiques": "48.8737224129,2.31584809721",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        219: {
            "attributes": {
                "Identifiant station": "21006",
                "Nom station": "Marcelin Berthelot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:45+01:00",
                "Coordonnées géographiques": "48.8415473059,2.23316162825",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        220: {
            "attributes": {
                "Identifiant station": "10014",
                "Nom station": "Marseille - Beaurepaire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "55",
                "Nombre bornettes libres": "46",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.871624307,2.36392650827",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        221: {
            "attributes": {
                "Identifiant station": "20131",
                "Nom station": "Repos - Cimetière du Père Lachaise",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:45+01:00",
                "Coordonnées géographiques": "48.8593585236,2.38961313668",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        222: {
            "attributes": {
                "Identifiant station": "9032",
                "Nom station": "Mathurins - Auber",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:03+01:00",
                "Coordonnées géographiques": "48.8729888407,2.32949842123",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        223: {
            "attributes": {
                "Identifiant station": "14012",
                "Nom station": "Sibelle - Alésia",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:38+01:00",
                "Coordonnées géographiques": "48.8267017585,2.33849834651",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        224: {
            "attributes": {
                "Identifiant station": "5123",
                "Nom station": "Saint-Germain - Thenard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "26",
                "Vélos mécaniques disponibles": "23",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.850571,2.3471015",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        225: {
            "attributes": {
                "Identifiant station": "44003",
                "Nom station": "Gabriel Péri - Danielle Casanova",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:05+01:00",
                "Coordonnées géographiques": "48.790114,2.4025777",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        226: {
            "attributes": {
                "Identifiant station": "13002",
                "Nom station": "Arago - Glacière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:31+01:00",
                "Coordonnées géographiques": "48.8348605478,2.34457705243",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        227: {
            "attributes": {
                "Identifiant station": "15034",
                "Nom station": "Lakanal - Commerce",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "53",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "31",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:43+01:00",
                "Coordonnées géographiques": "48.8444246279,2.29405917227",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        228: {
            "attributes": {
                "Identifiant station": "46001",
                "Nom station": "Place Paul Eluard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:54+01:00",
                "Coordonnées géographiques": "48.7672039733,2.41449773312",
                "Nom communes équipées": "Choisy-le-Roi",
                "Code INSEE communes équipées": "",
            }
        },
        229: {
            "attributes": {
                "Identifiant station": "48010",
                "Nom station": "Karl Marx.",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:31+01:00",
                "Coordonnées géographiques": "48.7845279373,2.36466679795",
                "Nom communes équipées": "Villejuif",
                "Code INSEE communes équipées": "",
            }
        },
        230: {
            "attributes": {
                "Identifiant station": "17028",
                "Nom station": "Reims - Raymond Pitet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:55+01:00",
                "Coordonnées géographiques": "48.8888,2.296319",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        231: {
            "attributes": {
                "Identifiant station": "42021",
                "Nom station": "Raspail - Manufacture des Oeillets",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:08+01:00",
                "Coordonnées géographiques": "48.8094458,2.3903009",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        232: {
            "attributes": {
                "Identifiant station": "42302",
                "Nom station": "Aristide Briand - Gare de Bagneux",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:06+01:00",
                "Coordonnées géographiques": "48.7938726966,2.32114881277",
                "Nom communes équipées": "Cachan",
                "Code INSEE communes équipées": "",
            }
        },
        233: {
            "attributes": {
                "Identifiant station": "21112",
                "Nom station": "Claude Debussy - Marc Bloch",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-10-13T22:08:33+02:00",
                "Coordonnées géographiques": "48.9103881805,2.31177717447",
                "Nom communes équipées": "Clichy",
                "Code INSEE communes équipées": "",
            }
        },
        234: {
            "attributes": {
                "Identifiant station": "21110",
                "Nom station": "Parc Roger Salengro",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:45+01:00",
                "Coordonnées géographiques": "48.9053038586,2.31055945158",
                "Nom communes équipées": "Clichy",
                "Code INSEE communes équipées": "",
            }
        },
        235: {
            "attributes": {
                "Identifiant station": "31708",
                "Nom station": "Noisy le Sec - Jean-Baptiste Clément",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:55+01:00",
                "Coordonnées géographiques": "48.8745153896,2.41528362036",
                "Nom communes équipées": "Bagnolet",
                "Code INSEE communes équipées": "",
            }
        },
        236: {
            "attributes": {
                "Identifiant station": "11042",
                "Nom station": "Grand Prieuré - Crussol",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "49",
                "Nombre bornettes libres": "42",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:45+01:00",
                "Coordonnées géographiques": "48.8646606279,2.36922655318",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        237: {
            "attributes": {
                "Identifiant station": "20115",
                "Nom station": "Louis Lumière - Porte de Bagnolet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:20+01:00",
                "Coordonnées géographiques": "48.86431136,2.41052102298",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        238: {
            "attributes": {
                "Identifiant station": "7102",
                "Nom station": "Mairie du 7ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "19",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:15+01:00",
                "Coordonnées géographiques": "48.8577967,2.3192664",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        239: {
            "attributes": {
                "Identifiant station": "22402",
                "Nom station": "Jean Jaurès - Salvador Allende",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "38",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:15+01:00",
                "Coordonnées géographiques": "48.8214755947,2.30265766382",
                "Nom communes équipées": "Malakoff",
                "Code INSEE communes équipées": "",
            }
        },
        240: {
            "attributes": {
                "Identifiant station": "15013",
                "Nom station": "Volontaire - Lecourbe",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:41+01:00",
                "Coordonnées géographiques": "48.843513788,2.30655223131",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        241: {
            "attributes": {
                "Identifiant station": "17013",
                "Nom station": "Legendre - Dulong",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:51+01:00",
                "Coordonnées géographiques": "48.8853251694,2.31642376631",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        242: {
            "attributes": {
                "Identifiant station": "1002",
                "Nom station": "Victoria - Place du Chatelet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:25+01:00",
                "Coordonnées géographiques": "48.8579925915,2.34691679478",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        243: {
            "attributes": {
                "Identifiant station": "16008",
                "Nom station": "Galilée - Kléber",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8674118657,2.29078138011",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        244: {
            "attributes": {
                "Identifiant station": "11004",
                "Nom station": "Charonne - Faubourg Saint-Antoine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8521919097,2.37409327179",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        245: {
            "attributes": {
                "Identifiant station": "5106",
                "Nom station": "Cujas - Saint-Michel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:37+01:00",
                "Coordonnées géographiques": "48.8481913486,2.34183165485",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        246: {
            "attributes": {
                "Identifiant station": "42301",
                "Nom station": "Gare Arcueil - Cachan",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:24+01:00",
                "Coordonnées géographiques": "48.799824,2.328171",
                "Nom communes équipées": "Arcueil",
                "Code INSEE communes équipées": "",
            }
        },
        247: {
            "attributes": {
                "Identifiant station": "11026",
                "Nom station": "Chemin Vert - Saint-Maur",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "39",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:49+01:00",
                "Coordonnées géographiques": "48.8611246141,2.38129902631",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        248: {
            "attributes": {
                "Identifiant station": "20121",
                "Nom station": "Boyer - Ménilmontant",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:44+01:00",
                "Coordonnées géographiques": "48.8688951627,2.39198241456",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        249: {
            "attributes": {
                "Identifiant station": "32604",
                "Nom station": "Square du Théâtre du Garde-Chasse",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "46",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:21+01:00",
                "Coordonnées géographiques": "48.8812873751,2.42011964321",
                "Nom communes équipées": "Les Lilas",
                "Code INSEE communes équipées": "",
            }
        },
        250: {
            "attributes": {
                "Identifiant station": "42015",
                "Nom station": "Place Jean Ferrat",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:23+01:00",
                "Coordonnées géographiques": "48.8184945213,2.37441748381",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        251: {
            "attributes": {
                "Identifiant station": "8057",
                "Nom station": "Hoche - Tilsitt",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:27+01:00",
                "Coordonnées géographiques": "48.8748364578,2.29713231325",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        252: {
            "attributes": {
                "Identifiant station": "19008",
                "Nom station": "Flandre - Alphonse Karr",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:07+01:00",
                "Coordonnées géographiques": "48.89465,2.381868",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        253: {
            "attributes": {
                "Identifiant station": "9010",
                "Nom station": "Rochechouart lamartine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:49+01:00",
                "Coordonnées géographiques": "48.8767198951,2.34439302185",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        254: {
            "attributes": {
                "Identifiant station": "32016",
                "Nom station": "Place de la Porte de Paris",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "24",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:38+01:00",
                "Coordonnées géographiques": "48.9300936321,2.35640376806",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        255: {
            "attributes": {
                "Identifiant station": "15021",
                "Nom station": "Joseph Liouville - Croix Nivert",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "64",
                "Nombre bornettes libres": "42",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "19",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:11+01:00",
                "Coordonnées géographiques": "48.8448354712,2.29750849307",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        256: {
            "attributes": {
                "Identifiant station": "15201",
                "Nom station": "Place Amédée Gordini - Lefebvre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:46+01:00",
                "Coordonnées géographiques": "48.8306173775,2.29206342083",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        257: {
            "attributes": {
                "Identifiant station": "11105",
                "Nom station": "Parmentier - République",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "42",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:02+01:00",
                "Coordonnées géographiques": "48.8644954,2.3750124",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        258: {
            "attributes": {
                "Identifiant station": "8013",
                "Nom station": "Marignan - Champs-Elysées",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "21",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:49+01:00",
                "Coordonnées géographiques": "48.8694956612,2.3066138139",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        259: {
            "attributes": {
                "Identifiant station": "21701",
                "Nom station": "Larmeroux - Baudelaire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8169399113,2.28098948048",
                "Nom communes équipées": "Vanves",
                "Code INSEE communes équipées": "",
            }
        },
        260: {
            "attributes": {
                "Identifiant station": "5006",
                "Nom station": "Saint-Jacques - Soufflot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:37+01:00",
                "Coordonnées géographiques": "48.8465255,2.343099",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        261: {
            "attributes": {
                "Identifiant station": "22003",
                "Nom station": "Beffroy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:49+01:00",
                "Coordonnées géographiques": "48.8865910745,2.26139456034",
                "Nom communes équipées": "Neuilly-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        262: {
            "attributes": {
                "Identifiant station": "43003",
                "Nom station": "Hôpital Bégin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:21+01:00",
                "Coordonnées géographiques": "48.8457167211,2.42724627256",
                "Nom communes équipées": "Vincennes",
                "Code INSEE communes équipées": "",
            }
        },
        263: {
            "attributes": {
                "Identifiant station": "16038",
                "Nom station": "Molitor - Chardon-Lagache",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:59+01:00",
                "Coordonnées géographiques": "48.8452088018,2.2656840831",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        264: {
            "attributes": {
                "Identifiant station": "12009",
                "Nom station": "Charenton - Diderot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:46+01:00",
                "Coordonnées géographiques": "48.8461714612,2.37930681556",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        265: {
            "attributes": {
                "Identifiant station": "41601",
                "Nom station": "Général De Gaulle - Alouette",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:49+01:00",
                "Coordonnées géographiques": "48.839109,2.4175664",
                "Nom communes équipées": "Saint-Mandé",
                "Code INSEE communes équipées": "",
            }
        },
        266: {
            "attributes": {
                "Identifiant station": "11036",
                "Nom station": "Jules Ferry - Faubourg du Temple Paris",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "58",
                "Nombre bornettes libres": "51",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.868397,2.3678933",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        267: {
            "attributes": {
                "Identifiant station": "14110",
                "Nom station": "Abbé Carton - Plantes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.8276738232,2.3209181428",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        268: {
            "attributes": {
                "Identifiant station": "14036",
                "Nom station": "Boulard - Daguerre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "43",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:06+01:00",
                "Coordonnées géographiques": "48.8344647268,2.32933860272",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        269: {
            "attributes": {
                "Identifiant station": "17125",
                "Nom station": "Marie-Georges Picquart - Pereire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:27+01:00",
                "Coordonnées géographiques": "48.8886366935,2.31117065996",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        270: {
            "attributes": {
                "Identifiant station": "23009",
                "Nom station": "Anatole France - Louise Michel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "38",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:14+01:00",
                "Coordonnées géographiques": "48.8887321493,2.28815764189",
                "Nom communes équipées": "Levallois-Perret",
                "Code INSEE communes équipées": "",
            }
        },
        271: {
            "attributes": {
                "Identifiant station": "43005",
                "Nom station": "Gare RER de Vincennes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "51",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "13",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:26+01:00",
                "Coordonnées géographiques": "48.8475630151,2.43338316679",
                "Nom communes équipées": "Vincennes",
                "Code INSEE communes équipées": "",
            }
        },
        272: {
            "attributes": {
                "Identifiant station": "11007",
                "Nom station": "Jean Macé - Faidherbe",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "63",
                "Nombre bornettes libres": "62",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:04+01:00",
                "Coordonnées géographiques": "48.8533754843,2.38286208361",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        273: {
            "attributes": {
                "Identifiant station": "19032",
                "Nom station": "Villette - Belleville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "48",
                "Nombre bornettes libres": "44",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:33+01:00",
                "Coordonnées géographiques": "48.8726877314,2.37646132708",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        274: {
            "attributes": {
                "Identifiant station": "32303",
                "Nom station": "Gaston Roussel - Commune de Paris",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:53+01:00",
                "Coordonnées géographiques": "48.8943203455,2.43255227442",
                "Nom communes équipées": "Romainville",
                "Code INSEE communes équipées": "",
            }
        },
        275: {
            "attributes": {
                "Identifiant station": "16136",
                "Nom station": "Théophile Gautier - François Millet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:27+01:00",
                "Coordonnées géographiques": "48.8503482132,2.27302998304",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        276: {
            "attributes": {
                "Identifiant station": "8035",
                "Nom station": "Bienfaisance - Place de Narvik",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "30",
                "Vélos mécaniques disponibles": "24",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:29+01:00",
                "Coordonnées géographiques": "48.8766016346,2.31297712773",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        277: {
            "attributes": {
                "Identifiant station": "19022",
                "Nom station": "Marché Secrétan",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:44+01:00",
                "Coordonnées géographiques": "48.8812549638,2.37346328795",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        278: {
            "attributes": {
                "Identifiant station": "12027",
                "Nom station": "Montgallet - Charenton",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:42+01:00",
                "Coordonnées géographiques": "48.8426477787,2.38616492781",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        279: {
            "attributes": {
                "Identifiant station": "22016",
                "Nom station": "Charles de Gaulle - Sablonville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:18+01:00",
                "Coordonnées géographiques": "48.8799192486,2.27637704462",
                "Nom communes équipées": "Neuilly-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        280: {
            "attributes": {
                "Identifiant station": "21313",
                "Nom station": "Guynemer - Séverine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:30+01:00",
                "Coordonnées géographiques": "48.8309174337,2.27907065302",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        281: {
            "attributes": {
                "Identifiant station": "14136",
                "Nom station": "Auguste Cain - Jean Moulin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "40",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:21+01:00",
                "Coordonnées géographiques": "48.8257702893,2.32104152441",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        282: {
            "attributes": {
                "Identifiant station": "6013",
                "Nom station": "Jacques Callot - Mazarine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "19",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:47+01:00",
                "Coordonnées géographiques": "48.8552801753,2.3375743255",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        283: {
            "attributes": {
                "Identifiant station": "32302",
                "Nom station": "Chemin Vert - Fraternité",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:38+01:00",
                "Coordonnées géographiques": "48.8771412084,2.44615651667",
                "Nom communes équipées": "Romainville",
                "Code INSEE communes équipées": "",
            }
        },
        284: {
            "attributes": {
                "Identifiant station": "28001",
                "Nom station": "Richard Wallace - Place du 8 Mai 1945",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:33+01:00",
                "Coordonnées géographiques": "48.8786679479,2.24203437567",
                "Nom communes équipées": "Puteaux",
                "Code INSEE communes équipées": "",
            }
        },
        285: {
            "attributes": {
                "Identifiant station": "11035",
                "Nom station": "Parmentier - Abel Rabaud",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "52",
                "Nombre bornettes libres": "47",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:42+01:00",
                "Coordonnées géographiques": "48.8692127384,2.37168882124",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        286: {
            "attributes": {
                "Identifiant station": "7015",
                "Nom station": "Varenne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "47",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "28",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:28+01:00",
                "Coordonnées géographiques": "48.8572026618,2.31527712196",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        287: {
            "attributes": {
                "Identifiant station": "42010",
                "Nom station": "Mairie d'Ivry-sur-Seine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "53",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "45",
                "Vélos mécaniques disponibles": "40",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:41+01:00",
                "Coordonnées géographiques": "48.8105712097,2.38428264856",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        288: {
            "attributes": {
                "Identifiant station": "14021",
                "Nom station": "Place de la Porte de Châtillon",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "59",
                "Nombre bornettes libres": "59",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:50+01:00",
                "Coordonnées géographiques": "48.8242469,2.3186306",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        289: {
            "attributes": {
                "Identifiant station": "15046",
                "Nom station": "Parc Georges Brassens - Brancion",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:22+01:00",
                "Coordonnées géographiques": "48.8322782927,2.30225700885",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        290: {
            "attributes": {
                "Identifiant station": "7112",
                "Nom station": "Quai d'Orsay - Invalides",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-01-23T13:02:41+01:00",
                "Coordonnées géographiques": "48.8630969001,2.31698962102",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        291: {
            "attributes": {
                "Identifiant station": "16040",
                "Nom station": "Exelmans - Michel-Ange",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:25+01:00",
                "Coordonnées géographiques": "48.8430948665,2.25977708007",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        292: {
            "attributes": {
                "Identifiant station": "11115",
                "Nom station": "Montreuil - Gonnet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:53+01:00",
                "Coordonnées géographiques": "48.8505005649,2.38908562879",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        293: {
            "attributes": {
                "Identifiant station": "22006",
                "Nom station": "Pierret - Madrid",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:14+01:00",
                "Coordonnées géographiques": "48.8829765769,2.26012576725",
                "Nom communes équipées": "Neuilly-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        294: {
            "attributes": {
                "Identifiant station": "12013",
                "Nom station": "Montgallet - Reuilly",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:20+01:00",
                "Coordonnées géographiques": "48.844291,2.3896481",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        295: {
            "attributes": {
                "Identifiant station": "21301",
                "Nom station": "Saint-Vincent - Verdun",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "29",
                "Vélos mécaniques disponibles": "26",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:37+01:00",
                "Coordonnées géographiques": "48.8192115492,2.25400775671",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        296: {
            "attributes": {
                "Identifiant station": "15025",
                "Nom station": "Dupleix - du Guesclin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "47",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "46",
                "Vélos mécaniques disponibles": "44",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:30+01:00",
                "Coordonnées géographiques": "48.8514791041,2.29670953006",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        297: {
            "attributes": {
                "Identifiant station": "19029",
                "Nom station": "Seine - Flandre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:34+01:00",
                "Coordonnées géographiques": "48.8867857415,2.37461932003",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        298: {
            "attributes": {
                "Identifiant station": "32602",
                "Nom station": "Mairie des Lilas",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "47",
                "Nombre bornettes libres": "45",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:47+01:00",
                "Coordonnées géographiques": "48.8792834,2.4160182",
                "Nom communes équipées": "Les Lilas",
                "Code INSEE communes équipées": "",
            }
        },
        299: {
            "attributes": {
                "Identifiant station": "32003",
                "Nom station": "Wilson - Métallurgie",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:31+01:00",
                "Coordonnées géographiques": "48.9094230101,2.35844358802",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        300: {
            "attributes": {
                "Identifiant station": "12115",
                "Nom station": "Porte de Vincennes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:28+01:00",
                "Coordonnées géographiques": "48.846454,2.415514",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        301: {
            "attributes": {
                "Identifiant station": "21104",
                "Nom station": "Madame de Sanzillon - Mozart",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:10+01:00",
                "Coordonnées géographiques": "48.9037982077,2.31874018908",
                "Nom communes équipées": "Clichy",
                "Code INSEE communes équipées": "",
            }
        },
        302: {
            "attributes": {
                "Identifiant station": "17011",
                "Nom station": "Fragonard - Porte de Clichy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "56",
                "Nombre bornettes libres": "37",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:29+01:00",
                "Coordonnées géographiques": "48.8941556334,2.31438931078",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        303: {
            "attributes": {
                "Identifiant station": "19016",
                "Nom station": "Hautpoul - Jean Jaurès",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:34+01:00",
                "Coordonnées géographiques": "48.8863936794,2.38654145444",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        304: {
            "attributes": {
                "Identifiant station": "43402",
                "Nom station": "Place de Verdun",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "37",
                "Vélos mécaniques disponibles": "36",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:25+01:00",
                "Coordonnées géographiques": "48.8202823,2.473775",
                "Nom communes équipées": "Joinville-le-Pont",
                "Code INSEE communes équipées": "",
            }
        },
        305: {
            "attributes": {
                "Identifiant station": "9005",
                "Nom station": "Square d'Anvers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:28+01:00",
                "Coordonnées géographiques": "48.8826334843,2.34488440621",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        306: {
            "attributes": {
                "Identifiant station": "33018",
                "Nom station": "Edouard Poisson - Victor Hugo",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:56+01:00",
                "Coordonnées géographiques": "48.9111855705,2.37986475269",
                "Nom communes équipées": "Aubervilliers",
                "Code INSEE communes équipées": "",
            }
        },
        307: {
            "attributes": {
                "Identifiant station": "19013",
                "Nom station": "Laumière - Petit",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:08+01:00",
                "Coordonnées géographiques": "48.8846736854,2.38003905863",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        308: {
            "attributes": {
                "Identifiant station": "16121",
                "Nom station": "Jardin d'Acclimatation",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:21+01:00",
                "Coordonnées géographiques": "48.8787722425,2.27077223361",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        309: {
            "attributes": {
                "Identifiant station": "42504",
                "Nom station": "Gare RER de Gentilly",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:02+01:00",
                "Coordonnées géographiques": "48.8142804504,2.34103471041",
                "Nom communes équipées": "Gentilly",
                "Code INSEE communes équipées": "",
            }
        },
        310: {
            "attributes": {
                "Identifiant station": "13049",
                "Nom station": "Vincent Auriol - Louise Weiss",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "12",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8340019355,2.37038880587",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        311: {
            "attributes": {
                "Identifiant station": "20111",
                "Nom station": "Villiers de l'Isle Adam - Pyrénées",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:30+01:00",
                "Coordonnées géographiques": "48.867362,2.396222",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        312: {
            "attributes": {
                "Identifiant station": "11102",
                "Nom station": "Alexandre Dumas - Voltaire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "59",
                "Nombre bornettes libres": "51",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:33+01:00",
                "Coordonnées géographiques": "48.8528886704,2.38914764081",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        313: {
            "attributes": {
                "Identifiant station": "9021",
                "Nom station": "Saint Georges - d'Aumale",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:16+01:00",
                "Coordonnées géographiques": "48.877786396,2.33742982149",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        314: {
            "attributes": {
                "Identifiant station": "19004",
                "Nom station": "Quai de la Loire - Jean Jaurès",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8833973385,2.37107452298",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        315: {
            "attributes": {
                "Identifiant station": "31010",
                "Nom station": "Stalingrad - Désiré Chevalier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "40",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:29+01:00",
                "Coordonnées géographiques": "48.8567016974,2.44507223368",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        316: {
            "attributes": {
                "Identifiant station": "16124",
                "Nom station": "Route de Sèvres - Porte de Bagatelle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "66",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "39",
                "Vélos mécaniques disponibles": "38",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:55+01:00",
                "Coordonnées géographiques": "48.8752598413,2.24941581488",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        317: {
            "attributes": {
                "Identifiant station": "92008",
                "Nom station": "Hôtel de Ville de Nanterre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:34+01:00",
                "Coordonnées géographiques": "48.8923718983,2.20554895699",
                "Nom communes équipées": "Nanterre",
                "Code INSEE communes équipées": "",
            }
        },
        318: {
            "attributes": {
                "Identifiant station": "33003",
                "Nom station": "Bordier - Félix Faure",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "48",
                "Nombre bornettes libres": "43",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:55+01:00",
                "Coordonnées géographiques": "48.9039075188,2.38300591707",
                "Nom communes équipées": "Aubervilliers",
                "Code INSEE communes équipées": "",
            }
        },
        319: {
            "attributes": {
                "Identifiant station": "41402",
                "Nom station": "Victor Hugo - Ecoles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:25+01:00",
                "Coordonnées géographiques": "48.8047600794,2.41803739303",
                "Nom communes équipées": "Alfortville",
                "Code INSEE communes équipées": "",
            }
        },
        320: {
            "attributes": {
                "Identifiant station": "20043",
                "Nom station": "Charonne - Avron",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:20+01:00",
                "Coordonnées géographiques": "48.8516026652,2.39840304704",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        321: {
            "attributes": {
                "Identifiant station": "7005",
                "Nom station": "Raspail - Bac",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "52",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "42",
                "Vélos mécaniques disponibles": "38",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:37+01:00",
                "Coordonnées géographiques": "48.8556287211,2.32566028833",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        322: {
            "attributes": {
                "Identifiant station": "22505",
                "Nom station": "Mairie de La Garenne Colombes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:07+01:00",
                "Coordonnées géographiques": "48.9066382347,2.24679596722",
                "Nom communes équipées": "La Garenne-Colombes",
                "Code INSEE communes équipées": "",
            }
        },
        323: {
            "attributes": {
                "Identifiant station": "18012",
                "Nom station": "Stephenson - Doudeauville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "35",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:44+01:00",
                "Coordonnées géographiques": "48.8885370489,2.35616102815",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        324: {
            "attributes": {
                "Identifiant station": "13045",
                "Nom station": "Jean Colly - Tolbiac",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "24",
                "Vélos mécaniques disponibles": "22",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:39+01:00",
                "Coordonnées géographiques": "48.8278172916,2.37054906785",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        325: {
            "attributes": {
                "Identifiant station": "19125",
                "Nom station": "Place de l'Édit de Nantes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.88879,2.3785",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        326: {
            "attributes": {
                "Identifiant station": "32010",
                "Nom station": "Place aux Étoiles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "24",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "15",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:19+01:00",
                "Coordonnées géographiques": "48.9183562941,2.35234256834",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        327: {
            "attributes": {
                "Identifiant station": "18201",
                "Nom station": "Tristan Tzara -  Tchaïkovski",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:54+01:00",
                "Coordonnées géographiques": "48.8949022175,2.36484903842",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        328: {
            "attributes": {
                "Identifiant station": "16018",
                "Nom station": "Flandrin - Henri Martin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:06+01:00",
                "Coordonnées géographiques": "48.8643275,2.2724174",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        329: {
            "attributes": {
                "Identifiant station": "11009",
                "Nom station": "Boulets - Voltaire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:04+01:00",
                "Coordonnées géographiques": "48.8521870562,2.38903015852",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        330: {
            "attributes": {
                "Identifiant station": "15010",
                "Nom station": "Square Cambronne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "59",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "33",
                "Vélos mécaniques disponibles": "29",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:28+01:00",
                "Coordonnées géographiques": "48.8475665453,2.30256915092",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        331: {
            "attributes": {
                "Identifiant station": "42029",
                "Nom station": "Stalingrad - Germaine Tailleferre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:11+01:00",
                "Coordonnées géographiques": "48.8025720048,2.37866912037",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        332: {
            "attributes": {
                "Identifiant station": "7009",
                "Nom station": "Assemblée Nationale",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "63",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "55",
                "Vélos mécaniques disponibles": "39",
                "Vélos électriques disponibles": "16",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:47+01:00",
                "Coordonnées géographiques": "48.8614110147,2.32012141852",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        333: {
            "attributes": {
                "Identifiant station": "21305",
                "Nom station": "Place Jacques Madaule",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "24",
                "Vélos mécaniques disponibles": "20",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:35+01:00",
                "Coordonnées géographiques": "48.8239337873,2.26069182158",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        334: {
            "attributes": {
                "Identifiant station": "17042",
                "Nom station": "Porte Maillot - Pereire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "52",
                "Nombre bornettes libres": "45",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:28+01:00",
                "Coordonnées géographiques": "48.877775,2.284444",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        335: {
            "attributes": {
                "Identifiant station": "10113",
                "Nom station": "Parmentier - Hôpital Saint-Louis",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "42",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:13+01:00",
                "Coordonnées géographiques": "48.8713798743,2.36989638367",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        336: {
            "attributes": {
                "Identifiant station": "33019",
                "Nom station": "Madeleine Vionnet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:47+01:00",
                "Coordonnées géographiques": "48.9025190741,2.37408958375",
                "Nom communes équipées": "Aubervilliers",
                "Code INSEE communes équipées": "",
            }
        },
        337: {
            "attributes": {
                "Identifiant station": "26007",
                "Nom station": "Square Gilbert Thomain",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:34+01:00",
                "Coordonnées géographiques": "48.9071737381,2.26954009384",
                "Nom communes équipées": "Asnières-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        338: {
            "attributes": {
                "Identifiant station": "8033",
                "Nom station": "Commandant Rivière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "26",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "12",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:24+01:00",
                "Coordonnées géographiques": "48.872933,2.309796",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        339: {
            "attributes": {
                "Identifiant station": "17102",
                "Nom station": "Alfred Roll - Berthier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:06+01:00",
                "Coordonnées géographiques": "48.8878085705,2.30009529818",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        340: {
            "attributes": {
                "Identifiant station": "33001",
                "Nom station": "Victor Hugo - Magasins Généraux",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:25+01:00",
                "Coordonnées géographiques": "48.9032814011,2.36807238311",
                "Nom communes équipées": "Aubervilliers",
                "Code INSEE communes équipées": "",
            }
        },
        341: {
            "attributes": {
                "Identifiant station": "16111",
                "Nom station": "Faustin Hélie - Desbordes-Valmore",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:59+01:00",
                "Coordonnées géographiques": "48.859403,2.276522",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        342: {
            "attributes": {
                "Identifiant station": "12029",
                "Nom station": "Dugommier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "55",
                "Nombre bornettes libres": "45",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:13+01:00",
                "Coordonnées géographiques": "48.8388110488,2.38965779543",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        343: {
            "attributes": {
                "Identifiant station": "19005",
                "Nom station": "Flandre - Riquet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8880132814,2.37375959552",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        344: {
            "attributes": {
                "Identifiant station": "20119",
                "Nom station": "Square des Saint-Simoniens",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:16+01:00",
                "Coordonnées géographiques": "48.8701796,2.3967273",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        345: {
            "attributes": {
                "Identifiant station": "35018",
                "Nom station": "Ancien Canal - Hélène Brion",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:32+01:00",
                "Coordonnées géographiques": "48.894847993,2.41883385926",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        346: {
            "attributes": {
                "Identifiant station": "42207",
                "Nom station": "Place des Marseillais",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8266390725,2.40592807531",
                "Nom communes équipées": "Charenton-le-Pont",
                "Code INSEE communes équipées": "",
            }
        },
        347: {
            "attributes": {
                "Identifiant station": "18110",
                "Nom station": "Departement - Pajol",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:45+01:00",
                "Coordonnées géographiques": "48.8866748497,2.36136116087",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        348: {
            "attributes": {
                "Identifiant station": "12030",
                "Nom station": "Charenton - Wattignies",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8366757964,2.39179585556",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        349: {
            "attributes": {
                "Identifiant station": "41606",
                "Nom station": "Pasteur - Gambetta",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:39+01:00",
                "Coordonnées géographiques": "48.8457279738,2.42360115051",
                "Nom communes équipées": "Saint-Mandé",
                "Code INSEE communes équipées": "",
            }
        },
        350: {
            "attributes": {
                "Identifiant station": "20036",
                "Nom station": "Ménilmontant - Pelleport",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:24+01:00",
                "Coordonnées géographiques": "48.8706826338,2.39895960233",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        351: {
            "attributes": {
                "Identifiant station": "17045",
                "Nom station": "Place des Ternes.",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:41+01:00",
                "Coordonnées géographiques": "48.8784386312,2.29772776365",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        352: {
            "attributes": {
                "Identifiant station": "35009",
                "Nom station": "Jean Lolive - Parc Stalingrad",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:37+01:00",
                "Coordonnées géographiques": "48.892061744,2.40917354822",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        353: {
            "attributes": {
                "Identifiant station": "24002",
                "Nom station": "Gare de Courbevoie",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "32",
                "Vélos mécaniques disponibles": "29",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8993867,2.2506506",
                "Nom communes équipées": "Courbevoie",
                "Code INSEE communes équipées": "",
            }
        },
        354: {
            "attributes": {
                "Identifiant station": "27002",
                "Nom station": "Parc Pierre Lagravère",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:00+01:00",
                "Coordonnées géographiques": "48.9265048813,2.23906967849",
                "Nom communes équipées": "Colombes",
                "Code INSEE communes équipées": "",
            }
        },
        355: {
            "attributes": {
                "Identifiant station": "14117",
                "Nom station": "Gare Montparnasse - René Mouchotte",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "35",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:37+01:00",
                "Coordonnées géographiques": "48.8392038433,2.32088394463",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        356: {
            "attributes": {
                "Identifiant station": "21109",
                "Nom station": "Guichet - Jean Jaurès",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:35+01:00",
                "Coordonnées géographiques": "48.9051698681,2.3023840785",
                "Nom communes équipées": "Clichy",
                "Code INSEE communes équipées": "",
            }
        },
        357: {
            "attributes": {
                "Identifiant station": "18141",
                "Nom station": "Clignancourt - Sofia",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:16+01:00",
                "Coordonnées géographiques": "48.8851199231,2.34719488842",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        358: {
            "attributes": {
                "Identifiant station": "20120",
                "Nom station": "Evariste Galois",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:57+01:00",
                "Coordonnées géographiques": "48.8730875373,2.41333801299",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        359: {
            "attributes": {
                "Identifiant station": "23001",
                "Nom station": "Paul Vaillant Couturier - Chaptal",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "33",
                "Vélos mécaniques disponibles": "26",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:57+01:00",
                "Coordonnées géographiques": "48.8934795762,2.27752536535",
                "Nom communes équipées": "Levallois-Perret",
                "Code INSEE communes équipées": "",
            }
        },
        360: {
            "attributes": {
                "Identifiant station": "15126",
                "Nom station": "Ernest Renan - Parc des Expositions",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-09-09T23:21:38+02:00",
                "Coordonnées géographiques": "48.8308975701,2.2853584215",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        361: {
            "attributes": {
                "Identifiant station": "41102",
                "Nom station": "Gare de Laplace",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "41",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:36+01:00",
                "Coordonnées géographiques": "48.8089319911,2.33494609594",
                "Nom communes équipées": "Arcueil",
                "Code INSEE communes équipées": "",
            }
        },
        362: {
            "attributes": {
                "Identifiant station": "21502",
                "Nom station": "Verdun - Place Henri IV",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:53+01:00",
                "Coordonnées géographiques": "48.8707973756,2.22770031542",
                "Nom communes équipées": "Suresnes",
                "Code INSEE communes équipées": "",
            }
        },
        363: {
            "attributes": {
                "Identifiant station": "17037",
                "Nom station": "Bayen - Pereire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:14+01:00",
                "Coordonnées géographiques": "48.8818044091,2.29210887104",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        364: {
            "attributes": {
                "Identifiant station": "14001",
                "Nom station": "Edgar Quinet - Gaité",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:16+01:00",
                "Coordonnées géographiques": "48.841160929,2.32459545135",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        365: {
            "attributes": {
                "Identifiant station": "33009",
                "Nom station": "Félix Faure - Sadi Carnot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:10+01:00",
                "Coordonnées géographiques": "48.9081211023,2.37862318754",
                "Nom communes équipées": "Aubervilliers",
                "Code INSEE communes équipées": "",
            }
        },
        366: {
            "attributes": {
                "Identifiant station": "31012",
                "Nom station": "Place Carnot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:40+01:00",
                "Coordonnées géographiques": "48.8559393219,2.44153171778",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        367: {
            "attributes": {
                "Identifiant station": "5021",
                "Nom station": "Jussieu - Fossés Saint-Bernard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "36",
                "Vélos mécaniques disponibles": "26",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.84718551,2.35348585993",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        368: {
            "attributes": {
                "Identifiant station": "5116",
                "Nom station": "Place Valhubert",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "25",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:17+01:00",
                "Coordonnées géographiques": "48.8441825773,2.36403666437",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        369: {
            "attributes": {
                "Identifiant station": "25005",
                "Nom station": "18 juin 1940 - Buzenval",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:38+01:00",
                "Coordonnées géographiques": "48.868807,2.185428",
                "Nom communes équipées": "Rueil-Malmaison",
                "Code INSEE communes équipées": "",
            }
        },
        370: {
            "attributes": {
                "Identifiant station": "20113",
                "Nom station": "Piat - Parc de Belleville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:51+01:00",
                "Coordonnées géographiques": "48.8719562664,2.38498139028",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        371: {
            "attributes": {
                "Identifiant station": "19115",
                "Nom station": "Porte de la Villette",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:32+01:00",
                "Coordonnées géographiques": "48.8983519655,2.38615013659",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        372: {
            "attributes": {
                "Identifiant station": "41604",
                "Nom station": "Lagny - Joffre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:01+01:00",
                "Coordonnées géographiques": "48.8493068708,2.41794973612",
                "Nom communes équipées": "Saint-Mandé",
                "Code INSEE communes équipées": "",
            }
        },
        373: {
            "attributes": {
                "Identifiant station": "14034",
                "Nom station": "Ouest - Château",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:45+01:00",
                "Coordonnées géographiques": "48.8361728601,2.31939214392",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        374: {
            "attributes": {
                "Identifiant station": "15017",
                "Nom station": "Place Falguière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:42+01:00",
                "Coordonnées géographiques": "48.8362997394,2.31044646353",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        375: {
            "attributes": {
                "Identifiant station": "12034",
                "Nom station": "Pirogues de Bercy - Libourne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "49",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "47",
                "Vélos mécaniques disponibles": "41",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:51+01:00",
                "Coordonnées géographiques": "48.832308,2.38637",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        376: {
            "attributes": {
                "Identifiant station": "31702",
                "Nom station": "Pierre et Marie Curie - Julien Grimau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:04+01:00",
                "Coordonnées géographiques": "48.8707140149,2.4240970239",
                "Nom communes équipées": "Bagnolet",
                "Code INSEE communes équipées": "",
            }
        },
        377: {
            "attributes": {
                "Identifiant station": "15033",
                "Nom station": "Place Violet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:56+01:00",
                "Coordonnées géographiques": "48.8448789384,2.29057565331",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        378: {
            "attributes": {
                "Identifiant station": "5002",
                "Nom station": "Sommerard - Saint-Jacques",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "27",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:49+01:00",
                "Coordonnées géographiques": "48.8503104329,2.3450158353",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        379: {
            "attributes": {
                "Identifiant station": "15122",
                "Nom station": "Place Charles Vallin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:03+01:00",
                "Coordonnées géographiques": "48.835523812,2.30259865522",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        380: {
            "attributes": {
                "Identifiant station": "15028",
                "Nom station": "Grenelle - Dr Finlay",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "71",
                "Nombre bornettes libres": "71",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-03-29T09:17:46+02:00",
                "Coordonnées géographiques": "48.8513154067,2.29195028543",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        381: {
            "attributes": {
                "Identifiant station": "16034",
                "Nom station": "Porte d'Auteuil",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:33+01:00",
                "Coordonnées géographiques": "48.8479394142,2.26083967835",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        382: {
            "attributes": {
                "Identifiant station": "92004",
                "Nom station": "Nanterre - Université",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:18+01:00",
                "Coordonnées géographiques": "48.901365,2.212693",
                "Nom communes équipées": "Nanterre",
                "Code INSEE communes équipées": "",
            }
        },
        383: {
            "attributes": {
                "Identifiant station": "14019",
                "Nom station": "Sarrette - Général Leclerc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:08+01:00",
                "Coordonnées géographiques": "48.8250308529,2.3264128694",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        384: {
            "attributes": {
                "Identifiant station": "20136",
                "Nom station": "Henri Chevreau - Couronnes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:23+01:00",
                "Coordonnées géographiques": "48.8703320531,2.3887776956",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        385: {
            "attributes": {
                "Identifiant station": "20132",
                "Nom station": "Réunion - Avron",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:00+01:00",
                "Coordonnées géographiques": "48.852566752,2.40392806871",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        386: {
            "attributes": {
                "Identifiant station": "8012",
                "Nom station": "Batignolles - Place de Clichy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "47",
                "Nombre bornettes libres": "41",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8834745358,2.32649311423",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        387: {
            "attributes": {
                "Identifiant station": "5114",
                "Nom station": "Claude Bernard - Arbalète",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:48+01:00",
                "Coordonnées géographiques": "48.8397667698,2.34751090407",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        388: {
            "attributes": {
                "Identifiant station": "21111",
                "Nom station": "Professeur René Leriche - Général Leclerc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "39",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:56+01:00",
                "Coordonnées géographiques": "48.9078284548,2.3078879714",
                "Nom communes équipées": "Clichy",
                "Code INSEE communes équipées": "",
            }
        },
        389: {
            "attributes": {
                "Identifiant station": "19019",
                "Nom station": "Petit - Manin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:02+01:00",
                "Coordonnées géographiques": "48.8865068753,2.39381554013",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        390: {
            "attributes": {
                "Identifiant station": "7006",
                "Nom station": "Quai Voltaire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "26",
                "Vélos mécaniques disponibles": "24",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:03+01:00",
                "Coordonnées géographiques": "48.8589429862,2.33141820471",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        391: {
            "attributes": {
                "Identifiant station": "21403",
                "Nom station": "Gare de Clamart",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "14",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:06+01:00",
                "Coordonnées géographiques": "48.813835356,2.27227896452",
                "Nom communes équipées": "Clamart",
                "Code INSEE communes équipées": "",
            }
        },
        392: {
            "attributes": {
                "Identifiant station": "11114",
                "Nom station": "Ledru-Rollin - Charonne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:19+01:00",
                "Coordonnées géographiques": "48.8534585561,2.37724635819",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        393: {
            "attributes": {
                "Identifiant station": "42707",
                "Nom station": "Place Jean-Baptiste Clément",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:02+01:00",
                "Coordonnées géographiques": "48.8124046687,2.36170917749",
                "Nom communes équipées": "Le Kremlin-Bicêtre",
                "Code INSEE communes équipées": "",
            }
        },
        394: {
            "attributes": {
                "Identifiant station": "12011",
                "Nom station": "Charenton - Place du Col Bourgoin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:17+01:00",
                "Coordonnées géographiques": "48.8448673196,2.38257893505",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        395: {
            "attributes": {
                "Identifiant station": "92003",
                "Nom station": "Chanzy - Sud-Est",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:24+01:00",
                "Coordonnées géographiques": "48.8889120377,2.19563215971",
                "Nom communes équipées": "Nanterre",
                "Code INSEE communes équipées": "",
            }
        },
        396: {
            "attributes": {
                "Identifiant station": "10023",
                "Nom station": "Gare de l'Est - Verdun",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "14",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:55+01:00",
                "Coordonnées géographiques": "48.8761560543,2.36091490832",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        397: {
            "attributes": {
                "Identifiant station": "5119",
                "Nom station": "Jussieu - Guy de la Brosse",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "35",
                "Vélos mécaniques disponibles": "30",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:07+01:00",
                "Coordonnées géographiques": "48.845665,2.355653",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        398: {
            "attributes": {
                "Identifiant station": "23002",
                "Nom station": "Pont de Levallois-bécon",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "46",
                "Vélos mécaniques disponibles": "41",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:20+01:00",
                "Coordonnées géographiques": "48.8984179,2.2791311",
                "Nom communes équipées": "Levallois-Perret",
                "Code INSEE communes équipées": "",
            }
        },
        399: {
            "attributes": {
                "Identifiant station": "31705",
                "Nom station": "Champeaux - Gallieni",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "49",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:07+01:00",
                "Coordonnées géographiques": "48.8644591347,2.41617780179",
                "Nom communes équipées": "Bagnolet",
                "Code INSEE communes équipées": "",
            }
        },
        400: {
            "attributes": {
                "Identifiant station": "14139",
                "Nom station": "Pierre Larousse - Raymond Losserand",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:39+01:00",
                "Coordonnées géographiques": "48.8305956423,2.31191832572",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        401: {
            "attributes": {
                "Identifiant station": "8055",
                "Nom station": "Courcelles - Place des Ternes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:12+01:00",
                "Coordonnées géographiques": "48.8781999289,2.29928601887",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        402: {
            "attributes": {
                "Identifiant station": "13102",
                "Nom station": "Pitié-Salpêtrière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "51",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "45",
                "Vélos mécaniques disponibles": "36",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:35+01:00",
                "Coordonnées géographiques": "48.8374881032,2.36521381885",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        403: {
            "attributes": {
                "Identifiant station": "13014",
                "Nom station": "Gare d'Austerlitz",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "14",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:35+01:00",
                "Coordonnées géographiques": "48.842524,2.364044",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        404: {
            "attributes": {
                "Identifiant station": "11101",
                "Nom station": "Charles Delescluze",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "49",
                "Nombre bornettes libres": "41",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:23+01:00",
                "Coordonnées géographiques": "48.8524220085,2.38063249737",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        405: {
            "attributes": {
                "Identifiant station": "23701",
                "Nom station": "Parvis Robert Schuman - Roger Salengro",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:06+01:00",
                "Coordonnées géographiques": "48.8126574706,2.19217512757",
                "Nom communes équipées": "Chaville",
                "Code INSEE communes équipées": "",
            }
        },
        406: {
            "attributes": {
                "Identifiant station": "14130",
                "Nom station": "Jonquoy - Didot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:45+01:00",
                "Coordonnées géographiques": "48.8289045482,2.31648076326",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        407: {
            "attributes": {
                "Identifiant station": "12126",
                "Nom station": "Gravelle - Route du Bac",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "51",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "18",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:22+01:00",
                "Coordonnées géographiques": "48.824407,2.418455",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        408: {
            "attributes": {
                "Identifiant station": "10015",
                "Nom station": "Lucien Sampaix - Récollets",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:27+01:00",
                "Coordonnées géographiques": "48.8743303776,2.36212290823",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        409: {
            "attributes": {
                "Identifiant station": "42023",
                "Nom station": "Place Léon Gambetta",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:23+01:00",
                "Coordonnées géographiques": "48.8146125024,2.40292936563",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        410: {
            "attributes": {
                "Identifiant station": "41207",
                "Nom station": "Place du Général de Gaulle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:09+01:00",
                "Coordonnées géographiques": "48.8525288,2.4877332",
                "Nom communes équipées": "Fontenay-sous-Bois",
                "Code INSEE communes équipées": "",
            }
        },
        411: {
            "attributes": {
                "Identifiant station": "16129",
                "Nom station": "Ceinture du Lac Inferieure - Saint-Cloud",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:54+01:00",
                "Coordonnées géographiques": "48.8621807093,2.26178716868",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        412: {
            "attributes": {
                "Identifiant station": "7014",
                "Nom station": "Saint-François-Xavier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:41+01:00",
                "Coordonnées géographiques": "48.8515448477,2.31461495161",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        413: {
            "attributes": {
                "Identifiant station": "35019",
                "Nom station": "Anatole France - Jean Lolive",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:25+01:00",
                "Coordonnées géographiques": "48.8949922,2.4253764",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        414: {
            "attributes": {
                "Identifiant station": "8101",
                "Nom station": "Londres - Amsterdam",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "12",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:15+01:00",
                "Coordonnées géographiques": "48.8780344615,2.32648354097",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        415: {
            "attributes": {
                "Identifiant station": "14104",
                "Nom station": "Boyer-Barret - Raymond Losserand",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:30+01:00",
                "Coordonnées géographiques": "48.8332988023,2.31733839959",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        416: {
            "attributes": {
                "Identifiant station": "6018",
                "Nom station": "Michelet - Assas",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:51+01:00",
                "Coordonnées géographiques": "48.8427415028,2.3350211978",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        417: {
            "attributes": {
                "Identifiant station": "17018",
                "Nom station": "Parc Monceau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:56+01:00",
                "Coordonnées géographiques": "48.8810779141,2.30954926461",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        418: {
            "attributes": {
                "Identifiant station": "19037",
                "Nom station": "Belleville - Porte des Lilas",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:28+01:00",
                "Coordonnées géographiques": "48.8766039189,2.40476969113",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        419: {
            "attributes": {
                "Identifiant station": "14018",
                "Nom station": "Porte d'Orléans",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "48",
                "Nombre bornettes libres": "47",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:47+01:00",
                "Coordonnées géographiques": "48.8227639,2.3249313",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        420: {
            "attributes": {
                "Identifiant station": "20109",
                "Nom station": "Haxo - Surmelin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:33+01:00",
                "Coordonnées géographiques": "48.8694370761,2.40539657538",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        421: {
            "attributes": {
                "Identifiant station": "15029",
                "Nom station": "Emeriau - Place de Brazzaville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "51",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "48",
                "Vélos mécaniques disponibles": "44",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:30+01:00",
                "Coordonnées géographiques": "48.8507852617,2.28715851903",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        422: {
            "attributes": {
                "Identifiant station": "15038",
                "Nom station": "Place Adolphe Cherioux - Vaugirard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "63",
                "Nombre bornettes libres": "58",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.839594428,2.30106577277",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        423: {
            "attributes": {
                "Identifiant station": "18005",
                "Nom station": "Tardieu - Chappe",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:31+01:00",
                "Coordonnées géographiques": "48.8841467598,2.34184540808",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        424: {
            "attributes": {
                "Identifiant station": "19109",
                "Nom station": "Tanger - Place du Maroc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:30+01:00",
                "Coordonnées géographiques": "48.8862524447,2.36892968416",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        425: {
            "attributes": {
                "Identifiant station": "10005",
                "Nom station": "Petites Ecuries - Faubourg Poissonnière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.8739721724,2.3483887575",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        426: {
            "attributes": {
                "Identifiant station": "31019",
                "Nom station": "Vincennes - Douy-Delcupe",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "37",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:14+01:00",
                "Coordonnées géographiques": "48.8549369216,2.43604928255",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        427: {
            "attributes": {
                "Identifiant station": "12024",
                "Nom station": "Gare de Bercy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "31",
                "Vélos mécaniques disponibles": "25",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:50+01:00",
                "Coordonnées géographiques": "48.8397945739,2.38260660321",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        428: {
            "attributes": {
                "Identifiant station": "18032",
                "Nom station": "Porte de Clignancourt.",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "56",
                "Nombre bornettes libres": "48",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:45+01:00",
                "Coordonnées géographiques": "48.897488,2.343996",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        429: {
            "attributes": {
                "Identifiant station": "18020",
                "Nom station": "Francoeur - Marcadet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:10+01:00",
                "Coordonnées géographiques": "48.8910412326,2.34001458544",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        430: {
            "attributes": {
                "Identifiant station": "17027",
                "Nom station": "Place du Maréchal Juin - Péreire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "44",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:27+01:00",
                "Coordonnées géographiques": "48.885263,2.298286",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        431: {
            "attributes": {
                "Identifiant station": "5026",
                "Nom station": "Pascal - Claude Bernard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "49",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8390619522,2.34997820109",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        432: {
            "attributes": {
                "Identifiant station": "22101",
                "Nom station": "Gare Routière de Saint Cloud",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.842881839,2.22224503756",
                "Nom communes équipées": "Saint-Cloud",
                "Code INSEE communes équipées": "",
            }
        },
        433: {
            "attributes": {
                "Identifiant station": "43006",
                "Nom station": "Vignerons - Minimes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:08+01:00",
                "Coordonnées géographiques": "48.841486,2.430793",
                "Nom communes équipées": "Vincennes",
                "Code INSEE communes équipées": "",
            }
        },
        434: {
            "attributes": {
                "Identifiant station": "13016",
                "Nom station": "Primo Levi - Frigos",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "24",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:03+01:00",
                "Coordonnées géographiques": "48.8309484485,2.37965351593",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        435: {
            "attributes": {
                "Identifiant station": "13104",
                "Nom station": "Gare d'Austerlitz - Quai Saint-Bernard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:47+01:00",
                "Coordonnées géographiques": "48.8436903086,2.36571639776",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        436: {
            "attributes": {
                "Identifiant station": "11020",
                "Nom station": "Belfort - Voltaire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:55+01:00",
                "Coordonnées géographiques": "48.8563358,2.3829901",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        437: {
            "attributes": {
                "Identifiant station": "8102",
                "Nom station": "Lamennais - Washington",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "12",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:08+01:00",
                "Coordonnées géographiques": "48.8735299003,2.30344489217",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        438: {
            "attributes": {
                "Identifiant station": "8051",
                "Nom station": "Batignolles - Rome",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "15",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:22+01:00",
                "Coordonnées géographiques": "48.8818233706,2.32025898993",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        439: {
            "attributes": {
                "Identifiant station": "21013",
                "Nom station": "Pierre Grenier - Pont de Billancourt",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:36+01:00",
                "Coordonnées géographiques": "48.8258020743,2.2487989068",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        440: {
            "attributes": {
                "Identifiant station": "13043",
                "Nom station": "Dunois - Clisson",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "51",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "26",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.832294,2.3674523",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        441: {
            "attributes": {
                "Identifiant station": "2010",
                "Nom station": "Uzès - Montmartre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "24",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:40+01:00",
                "Coordonnées géographiques": "48.8707909802,2.34310101718",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        442: {
            "attributes": {
                "Identifiant station": "14122",
                "Nom station": "Colonel Monteil - Brune",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:26+01:00",
                "Coordonnées géographiques": "48.8265029331,2.30927479983",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        443: {
            "attributes": {
                "Identifiant station": "23011",
                "Nom station": "Bineau - Louise Michel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:04+01:00",
                "Coordonnées géographiques": "48.8866616221,2.28432744741",
                "Nom communes équipées": "Levallois-Perret",
                "Code INSEE communes équipées": "",
            }
        },
        444: {
            "attributes": {
                "Identifiant station": "19039",
                "Nom station": "Edouard Pailleron - Simon Bolivar",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:47+01:00",
                "Coordonnées géographiques": "48.8797988605,2.37483926117",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        445: {
            "attributes": {
                "Identifiant station": "21308",
                "Nom station": "Bara - Henri Farman",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "35",
                "Vélos mécaniques disponibles": "26",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:56+01:00",
                "Coordonnées géographiques": "48.8332180275,2.26775676012",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        446: {
            "attributes": {
                "Identifiant station": "18025",
                "Nom station": "Mairie du 18ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:48+01:00",
                "Coordonnées géographiques": "48.8929392969,2.34455108643",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        447: {
            "attributes": {
                "Identifiant station": "17007",
                "Nom station": "Navier - Epinettes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:45+01:00",
                "Coordonnées géographiques": "48.8958778077,2.32267767191",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        448: {
            "attributes": {
                "Identifiant station": "9023",
                "Nom station": "Laffitte - Italiens",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "31",
                "Vélos mécaniques disponibles": "24",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:46+01:00",
                "Coordonnées géographiques": "48.8718157374,2.33720154785",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        449: {
            "attributes": {
                "Identifiant station": "20040",
                "Nom station": "Parc de Belleville - Julien Lacroix",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:46+01:00",
                "Coordonnées géographiques": "48.8703459,2.3842214",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        450: {
            "attributes": {
                "Identifiant station": "20021",
                "Nom station": "Indre - Prairies",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:21+01:00",
                "Coordonnées géographiques": "48.8625439851,2.40329045802",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        451: {
            "attributes": {
                "Identifiant station": "42503",
                "Nom station": "Vladimir Ilitch Lénine - Andrée Chedid",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:10+01:00",
                "Coordonnées géographiques": "48.8131995,2.33214586973",
                "Nom communes équipées": "Arcueil",
                "Code INSEE communes équipées": "",
            }
        },
        452: {
            "attributes": {
                "Identifiant station": "26001",
                "Nom station": "Gare d'Asnières sur Seine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:19+01:00",
                "Coordonnées géographiques": "48.9063645922,2.28350669146",
                "Nom communes équipées": "Asnières-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        453: {
            "attributes": {
                "Identifiant station": "17017",
                "Nom station": "Gare de Pont-Cardinet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:00+01:00",
                "Coordonnées géographiques": "48.8871390148,2.31449414399",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        454: {
            "attributes": {
                "Identifiant station": "17047",
                "Nom station": "Courcelles - Pierre Demours",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:54+01:00",
                "Coordonnées géographiques": "48.8837697333,2.29858466343",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        455: {
            "attributes": {
                "Identifiant station": "21105",
                "Nom station": "Morice - Général Leclerc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:21+01:00",
                "Coordonnées géographiques": "48.9024547198,2.31289297342",
                "Nom communes équipées": "Clichy",
                "Code INSEE communes équipées": "",
            }
        },
        456: {
            "attributes": {
                "Identifiant station": "11040",
                "Nom station": "Temple - Jean-Pierre Timbaud",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:15+01:00",
                "Coordonnées géographiques": "48.864465972,2.36605972052",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        457: {
            "attributes": {
                "Identifiant station": "13026",
                "Nom station": "Place Jean Delay",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:21+01:00",
                "Coordonnées géographiques": "48.823316,2.354336",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        458: {
            "attributes": {
                "Identifiant station": "41204",
                "Nom station": "République - Danton",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:43+01:00",
                "Coordonnées géographiques": "48.8524917218,2.46203452349",
                "Nom communes équipées": "Fontenay-sous-Bois",
                "Code INSEE communes équipées": "",
            }
        },
        459: {
            "attributes": {
                "Identifiant station": "15055",
                "Nom station": "Vasco de Gama",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "54",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "45",
                "Vélos mécaniques disponibles": "41",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8367178347,2.28366408565",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        460: {
            "attributes": {
                "Identifiant station": "19009",
                "Nom station": "Cité des Sciences et de l'Industrie",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.8965957118,2.38459914923",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        461: {
            "attributes": {
                "Identifiant station": "34002",
                "Nom station": "Capitaine Glarner - Gabriel Péri",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:44+01:00",
                "Coordonnées géographiques": "48.9024499004,2.33048075334",
                "Nom communes équipées": "Saint-Ouen-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        462: {
            "attributes": {
                "Identifiant station": "32017",
                "Nom station": "Basilique",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:52+01:00",
                "Coordonnées géographiques": "48.9362689106,2.35886668202",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        463: {
            "attributes": {
                "Identifiant station": "16006",
                "Nom station": "Lauriston - Copernic",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-01-08T06:44:26+01:00",
                "Coordonnées géographiques": "48.86926,2.289918",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        464: {
            "attributes": {
                "Identifiant station": "10020",
                "Nom station": "Chabrol - d'Hauteville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "11",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:46+01:00",
                "Coordonnées géographiques": "48.877018388,2.35131796449",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        465: {
            "attributes": {
                "Identifiant station": "21019",
                "Nom station": "Edouard Vaillant - Paul-Adolphe Souriau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "18",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:33+01:00",
                "Coordonnées géographiques": "48.8348458588,2.24794596434",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        466: {
            "attributes": {
                "Identifiant station": "11116",
                "Nom station": "Faubourg St-Antoine - Place de la Nation",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-08-28T04:57:31+02:00",
                "Coordonnées géographiques": "48.8487659,2.3940436",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        467: {
            "attributes": {
                "Identifiant station": "12032",
                "Nom station": "Porte Dorée",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8354337,2.4077132",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        468: {
            "attributes": {
                "Identifiant station": "16004",
                "Nom station": "Université Paris Dauphine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-02-11T10:14:41+01:00",
                "Coordonnées géographiques": "48.87070784,2.27486561984",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        469: {
            "attributes": {
                "Identifiant station": "21032",
                "Nom station": "Sèvres-Gallieni",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:41+01:00",
                "Coordonnées géographiques": "48.837809409,2.22815193236",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        470: {
            "attributes": {
                "Identifiant station": "19038",
                "Nom station": "Lassus - Delouvain",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:57+01:00",
                "Coordonnées géographiques": "48.8755044846,2.38903273255",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        471: {
            "attributes": {
                "Identifiant station": "6107",
                "Nom station": "Place Alphonse Deville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "18",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:39+01:00",
                "Coordonnées géographiques": "48.8501882627,2.32759617269",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        472: {
            "attributes": {
                "Identifiant station": "21703",
                "Nom station": "Square du Carrefour de l'Insurrection",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:38+01:00",
                "Coordonnées géographiques": "48.8208645598,2.29111343622",
                "Nom communes équipées": "Vanves",
                "Code INSEE communes équipées": "",
            }
        },
        473: {
            "attributes": {
                "Identifiant station": "6103",
                "Nom station": "Mézières - Rennes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "55",
                "Nombre bornettes libres": "47",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:35+01:00",
                "Coordonnées géographiques": "48.8502345932,2.33038198203",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        474: {
            "attributes": {
                "Identifiant station": "21706",
                "Nom station": "Jean Bleuzen - Square du 11 Novembre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "49",
                "Nombre bornettes libres": "42",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:31+01:00",
                "Coordonnées géographiques": "48.8232579,2.2965213",
                "Nom communes équipées": "Vanves",
                "Code INSEE communes équipées": "",
            }
        },
        475: {
            "attributes": {
                "Identifiant station": "12001",
                "Nom station": "Bastille - Biscornet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "62",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "33",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:35+01:00",
                "Coordonnées géographiques": "48.8513557796,2.36922003329",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        476: {
            "attributes": {
                "Identifiant station": "16021",
                "Nom station": "François Ponsard - La Muette",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:55+01:00",
                "Coordonnées géographiques": "48.8583666,2.2744515",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        477: {
            "attributes": {
                "Identifiant station": "32014",
                "Nom station": "Place du Cornillon - Stade de France",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:30+01:00",
                "Coordonnées géographiques": "48.9254069584,2.36444268376",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        478: {
            "attributes": {
                "Identifiant station": "9031",
                "Nom station": "Provence - Chaussée d'Antin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:38+01:00",
                "Coordonnées géographiques": "48.8742844754,2.33297651156",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        479: {
            "attributes": {
                "Identifiant station": "17124",
                "Nom station": "Porte de Saint-Ouen - Henri Huchard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:16+01:00",
                "Coordonnées géographiques": "48.8987083664,2.32953608036",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        480: {
            "attributes": {
                "Identifiant station": "6030",
                "Nom station": "Assas - Vaugirard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "30",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "16",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:24+01:00",
                "Coordonnées géographiques": "48.8482741114,2.32924137264",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        481: {
            "attributes": {
                "Identifiant station": "44012",
                "Nom station": "Génie - Mozart",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:38+01:00",
                "Coordonnées géographiques": "48.7949023812,2.37341836095",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        482: {
            "attributes": {
                "Identifiant station": "18023",
                "Nom station": "Ordener - Poissonniers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:35+01:00",
                "Coordonnées géographiques": "48.8912138908,2.35128866622",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        483: {
            "attributes": {
                "Identifiant station": "10165",
                "Nom station": "Gare de l'Est - Place du 11 Novembre 1918",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:43+01:00",
                "Coordonnées géographiques": "48.876327,2.3585464",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        484: {
            "attributes": {
                "Identifiant station": "18049",
                "Nom station": "Porte d'Aubervilliers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "55",
                "Nombre bornettes libres": "45",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:35+01:00",
                "Coordonnées géographiques": "48.8985966194,2.36897226423",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        485: {
            "attributes": {
                "Identifiant station": "31016",
                "Nom station": "Lucie Aubrac - Franklin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:28+01:00",
                "Coordonnées géographiques": "48.8622391601,2.44358628988",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        486: {
            "attributes": {
                "Identifiant station": "31703",
                "Nom station": "Sadi Carnot - François Mitterrand",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:47+01:00",
                "Coordonnées géographiques": "48.8753268767,2.4241617322",
                "Nom communes équipées": "Bagnolet",
                "Code INSEE communes équipées": "",
            }
        },
        487: {
            "attributes": {
                "Identifiant station": "51003",
                "Nom station": "Hôtel de Ville d'Argenteuil",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:47+01:00",
                "Coordonnées géographiques": "48.946504,2.2496396",
                "Nom communes équipées": "Argenteuil",
                "Code INSEE communes équipées": "",
            }
        },
        488: {
            "attributes": {
                "Identifiant station": "15031",
                "Nom station": "Théâtre - Lourmel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "53",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "33",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:35+01:00",
                "Coordonnées géographiques": "48.8482070398,2.28986587375",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        489: {
            "attributes": {
                "Identifiant station": "44102",
                "Nom station": "Verdun - Adrien Damalix",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "24",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "14",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:41+01:00",
                "Coordonnées géographiques": "48.8222334179,2.42153283209",
                "Nom communes équipées": "Saint-Maurice",
                "Code INSEE communes équipées": "",
            }
        },
        490: {
            "attributes": {
                "Identifiant station": "21014",
                "Nom station": "Hameau Fleuri - Marché",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:42+01:00",
                "Coordonnées géographiques": "48.8314488926,2.2412404418",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        491: {
            "attributes": {
                "Identifiant station": "9115",
                "Nom station": "Milton - Manuel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:58+01:00",
                "Coordonnées géographiques": "48.878249,2.341181",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        492: {
            "attributes": {
                "Identifiant station": "35014",
                "Nom station": "Charles de Gaulle - Jean Lolive",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:01+01:00",
                "Coordonnées géographiques": "48.8932644351,2.4126765132",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        493: {
            "attributes": {
                "Identifiant station": "16123",
                "Nom station": "Mahatma Gandhi",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "15",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:46+01:00",
                "Coordonnées géographiques": "48.876356,2.264205",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        494: {
            "attributes": {
                "Identifiant station": "15049",
                "Nom station": "Porte de Versailles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.8321538173,2.2871491313",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        495: {
            "attributes": {
                "Identifiant station": "16032",
                "Nom station": "Eglise d'Auteuil",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:54+01:00",
                "Coordonnées géographiques": "48.8474984177,2.26848387135",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        496: {
            "attributes": {
                "Identifiant station": "11041",
                "Nom station": "Square Saint-Ambroise",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:06+01:00",
                "Coordonnées géographiques": "48.860934,2.375841",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        497: {
            "attributes": {
                "Identifiant station": "18113",
                "Nom station": "Lepic - Armée d'Orient",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:19+01:00",
                "Coordonnées géographiques": "48.8875364038,2.3343077302",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        498: {
            "attributes": {
                "Identifiant station": "11006",
                "Nom station": "Ledru-Rollin - Place du Père Chaillet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:23+01:00",
                "Coordonnées géographiques": "48.8564337069,2.37902058776",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        499: {
            "attributes": {
                "Identifiant station": "21003",
                "Nom station": "Jean Jaurès - Jean Baptiste Clément",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:15+01:00",
                "Coordonnées géographiques": "48.8457626145,2.23696500063",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        500: {
            "attributes": {
                "Identifiant station": "31027",
                "Nom station": "Place de la République.",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-10-03T22:46:32+02:00",
                "Coordonnées géographiques": "48.852988,2.42467",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        501: {
            "attributes": {
                "Identifiant station": "22001",
                "Nom station": "Château - Bineau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:32+01:00",
                "Coordonnées géographiques": "48.8904040286,2.27010637522",
                "Nom communes équipées": "Neuilly-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        502: {
            "attributes": {
                "Identifiant station": "19127",
                "Nom station": "Aubervilliers - Curial",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:49+01:00",
                "Coordonnées géographiques": "48.8908830523,2.36885793507",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        503: {
            "attributes": {
                "Identifiant station": "17122",
                "Nom station": "Bridaine - Lamandé",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:13+01:00",
                "Coordonnées géographiques": "48.8862074,2.3206239",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        504: {
            "attributes": {
                "Identifiant station": "16015",
                "Nom station": "Bassano -  Iéna",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "15",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:26+01:00",
                "Coordonnées géographiques": "48.8681445105,2.29614960099",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        505: {
            "attributes": {
                "Identifiant station": "20027",
                "Nom station": "Porte de Ménilmontant",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "15",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:45+01:00",
                "Coordonnées géographiques": "48.8691543609,2.40932246879",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        506: {
            "attributes": {
                "Identifiant station": "47004",
                "Nom station": "Stade Auguste Delaune",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:32+01:00",
                "Coordonnées géographiques": "48.8089637868,2.43452042341",
                "Nom communes équipées": "Maisons-Alfort",
                "Code INSEE communes équipées": "",
            }
        },
        507: {
            "attributes": {
                "Identifiant station": "15032",
                "Nom station": "Félix Faure - Lourmel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "58",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "57",
                "Vélos mécaniques disponibles": "56",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:57+01:00",
                "Coordonnées géographiques": "48.8386890171,2.2818081826",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        508: {
            "attributes": {
                "Identifiant station": "16122",
                "Nom station": "Route de la Muette - Porte de Neuilly",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "45",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:37+01:00",
                "Coordonnées géographiques": "48.8798926,2.2587671",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        509: {
            "attributes": {
                "Identifiant station": "15108",
                "Nom station": "Lourmel - Marie Skobtsov",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:13+01:00",
                "Coordonnées géographiques": "48.844974,2.287283",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        510: {
            "attributes": {
                "Identifiant station": "20116",
                "Nom station": "Haies - Réunion",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:38+01:00",
                "Coordonnées géographiques": "48.8538689677,2.40242616224",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        511: {
            "attributes": {
                "Identifiant station": "19040",
                "Nom station": "Belleville - Télégraphe",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "41",
                "Nombre bornettes libres": "41",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.8756045,2.3995549",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        512: {
            "attributes": {
                "Identifiant station": "15002",
                "Nom station": "Maine - Antoine Bourdelle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:46+01:00",
                "Coordonnées géographiques": "48.8430853175,2.32026394678",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        513: {
            "attributes": {
                "Identifiant station": "6016",
                "Nom station": "Danton - Serpente",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:34+01:00",
                "Coordonnées géographiques": "48.8522973626,2.34141323715",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        514: {
            "attributes": {
                "Identifiant station": "23007",
                "Nom station": "Président Wilson - Jean Jaures",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:45+01:00",
                "Coordonnées géographiques": "48.8903405405,2.29206830263",
                "Nom communes équipées": "Levallois-Perret",
                "Code INSEE communes équipées": "",
            }
        },
        515: {
            "attributes": {
                "Identifiant station": "17046",
                "Nom station": "Mac-Mahon - Brey",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:14+01:00",
                "Coordonnées géographiques": "48.8775320841,2.29451253605",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        516: {
            "attributes": {
                "Identifiant station": "10010",
                "Nom station": "Place Jacques Bonsergent",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:47+01:00",
                "Coordonnées géographiques": "48.8709482186,2.3612049222",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        517: {
            "attributes": {
                "Identifiant station": "17121",
                "Nom station": "Square des Epinettes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:08+01:00",
                "Coordonnées géographiques": "48.8938390978,2.32595399022",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        518: {
            "attributes": {
                "Identifiant station": "5121",
                "Nom station": "Square René Viviani - Montebello",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "31",
                "Vélos mécaniques disponibles": "26",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.8519048914,2.34784215689",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        519: {
            "attributes": {
                "Identifiant station": "47006",
                "Nom station": "Les Planètes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:38+01:00",
                "Coordonnées géographiques": "48.8099849921,2.45808530599",
                "Nom communes équipées": "Maisons-Alfort",
                "Code INSEE communes équipées": "",
            }
        },
        520: {
            "attributes": {
                "Identifiant station": "16027",
                "Nom station": "Mozart - Jasmin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "62",
                "Nombre bornettes libres": "58",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:41:57+01:00",
                "Coordonnées géographiques": "48.8528612487,2.26848357065",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        521: {
            "attributes": {
                "Identifiant station": "18109",
                "Nom station": "Pajol - Riquet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "47",
                "Nombre bornettes libres": "45",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8896002813,2.36285347492",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        522: {
            "attributes": {
                "Identifiant station": "1117",
                "Nom station": "Pont Neuf - Rivoli",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:34+01:00",
                "Coordonnées géographiques": "48.8600898926,2.34405755997",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        523: {
            "attributes": {
                "Identifiant station": "18029",
                "Nom station": "Emile Blemont - Poteau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "38",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8943782,2.3415122",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        524: {
            "attributes": {
                "Identifiant station": "11043",
                "Nom station": "Place Pasdeloup",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:04+01:00",
                "Coordonnées géographiques": "48.8626542156,2.36703250628",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        525: {
            "attributes": {
                "Identifiant station": "20118",
                "Nom station": "Orteaux - Mouraud",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:04+01:00",
                "Coordonnées géographiques": "48.8555864285,2.40876290776",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        526: {
            "attributes": {
                "Identifiant station": "1120",
                "Nom station": "Lavandieres Sainte Opportune - Rivoli",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "23",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:32+01:00",
                "Coordonnées géographiques": "48.8592797025,2.34662175179",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        527: {
            "attributes": {
                "Identifiant station": "20023",
                "Nom station": "Pelleport - Belgrand",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "15",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:48+01:00",
                "Coordonnées géographiques": "48.8650008185,2.40398083638",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        528: {
            "attributes": {
                "Identifiant station": "9017",
                "Nom station": "Lallier - Trudaine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:17+01:00",
                "Coordonnées géographiques": "48.8810924662,2.34082248062",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        529: {
            "attributes": {
                "Identifiant station": "15035",
                "Nom station": "Place Etienne Pernet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "20",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:33+01:00",
                "Coordonnées géographiques": "48.8424345713,2.29218196124",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        530: {
            "attributes": {
                "Identifiant station": "32001",
                "Nom station": "Proudhon - Président Wilson",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:42+01:00",
                "Coordonnées géographiques": "48.9064427635,2.35849052668",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        531: {
            "attributes": {
                "Identifiant station": "35017",
                "Nom station": "Toffier Decaux - Cartier Bresson",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "15",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:59+01:00",
                "Coordonnées géographiques": "48.9016507255,2.40175455809",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        532: {
            "attributes": {
                "Identifiant station": "42703",
                "Nom station": "Fontainebleau - Edmond Michelet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "44",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:52+01:00",
                "Coordonnées géographiques": "48.8095643684,2.36269086599",
                "Nom communes équipées": "Le Kremlin-Bicêtre",
                "Code INSEE communes équipées": "",
            }
        },
        533: {
            "attributes": {
                "Identifiant station": "9117",
                "Nom station": "Bellefond - Maubeuge",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:44+01:00",
                "Coordonnées géographiques": "48.8782553976,2.34526421875",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        534: {
            "attributes": {
                "Identifiant station": "13046",
                "Nom station": "Eugène Oudiné - Dessous Des Berges",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "44",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "30",
                "Vélos mécaniques disponibles": "30",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:33+01:00",
                "Coordonnées géographiques": "48.8256423,2.3750226",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        535: {
            "attributes": {
                "Identifiant station": "35003",
                "Nom station": "Gabrielle Josserand - Edouard Vaillant",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:53+01:00",
                "Coordonnées géographiques": "48.9027156626,2.39332705736",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        536: {
            "attributes": {
                "Identifiant station": "22102",
                "Nom station": "Mont Valérien - Michel Salles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:11+01:00",
                "Coordonnées géographiques": "48.8561915,2.2152966",
                "Nom communes équipées": "Saint-Cloud",
                "Code INSEE communes équipées": "",
            }
        },
        537: {
            "attributes": {
                "Identifiant station": "47005",
                "Nom station": "Ecole Vétérinaire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "33",
                "Vélos mécaniques disponibles": "31",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:27+01:00",
                "Coordonnées géographiques": "48.8148885982,2.42243621578",
                "Nom communes équipées": "Maisons-Alfort",
                "Code INSEE communes équipées": "",
            }
        },
        538: {
            "attributes": {
                "Identifiant station": "20030",
                "Nom station": "Ménilmontant - Père Lachaise",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "56",
                "Nombre bornettes libres": "50",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:11+01:00",
                "Coordonnées géographiques": "48.8632872915,2.3869940266",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        539: {
            "attributes": {
                "Identifiant station": "15070",
                "Nom station": "Balard - Montagne de la Fage",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "30",
                "Vélos mécaniques disponibles": "25",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.8385074033,2.27828711271",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        540: {
            "attributes": {
                "Identifiant station": "4110",
                "Nom station": "Neuve Saint-Pierre - Beautreillis",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:27+01:00",
                "Coordonnées géographiques": "48.8535535144,2.36391227692",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        541: {
            "attributes": {
                "Identifiant station": "12107",
                "Nom station": "Diderot - Pierre Bourdan",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:37+01:00",
                "Coordonnées géographiques": "48.8476486208,2.39025827497",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        542: {
            "attributes": {
                "Identifiant station": "18202",
                "Nom station": "Diane Arbus",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "35",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:08+01:00",
                "Coordonnées géographiques": "48.8988007172,2.35240224749",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        543: {
            "attributes": {
                "Identifiant station": "14112",
                "Nom station": "Méchain - Faubourg Saint-Jacques",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:26+01:00",
                "Coordonnées géographiques": "48.835867721,2.33798034198",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        544: {
            "attributes": {
                "Identifiant station": "14030",
                "Nom station": "Pernety - Raymond Losserand",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:35+01:00",
                "Coordonnées géographiques": "48.834254628,2.31760933939",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        545: {
            "attributes": {
                "Identifiant station": "9026",
                "Nom station": "Jean-Baptiste Pigalle - La Bruyere",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:22+01:00",
                "Coordonnées géographiques": "48.8793982645,2.33370917334",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        546: {
            "attributes": {
                "Identifiant station": "7020",
                "Nom station": "Champ de Mars - Cler",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "41",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "19",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:04+01:00",
                "Coordonnées géographiques": "48.8566915501,2.30664677918",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        547: {
            "attributes": {
                "Identifiant station": "21001",
                "Nom station": "Transvaal - Charles de Gaulle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8482804036,2.23734201209",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        548: {
            "attributes": {
                "Identifiant station": "13103",
                "Nom station": "Hôpital Pitié Salpêtrière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "37",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:51+01:00",
                "Coordonnées géographiques": "48.8377016,2.3604946",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        549: {
            "attributes": {
                "Identifiant station": "18033",
                "Nom station": "Belliard - Damrémont",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:50+01:00",
                "Coordonnées géographiques": "48.8966923878,2.33802345039",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        550: {
            "attributes": {
                "Identifiant station": "6017",
                "Nom station": "Sénat - Condé",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "31",
                "Vélos mécaniques disponibles": "31",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:21+01:00",
                "Coordonnées géographiques": "48.8495422779,2.3379394412",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        551: {
            "attributes": {
                "Identifiant station": "11010",
                "Nom station": "Boulets - Faubourg Saint-Antoine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "37",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:37+01:00",
                "Coordonnées géographiques": "48.8492592621,2.39175461419",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        552: {
            "attributes": {
                "Identifiant station": "10036",
                "Nom station": "Aqueduc - Villette",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:36+01:00",
                "Coordonnées géographiques": "48.8838860639,2.36719880331",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        553: {
            "attributes": {
                "Identifiant station": "21310",
                "Nom station": "Kléber - Gouverneur Général Eboué",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "20",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8268858431,2.27234132588",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        554: {
            "attributes": {
                "Identifiant station": "14028",
                "Nom station": "Alésia - Suisses",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "47",
                "Nombre bornettes libres": "47",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:23+01:00",
                "Coordonnées géographiques": "48.8316825207,2.3149869226",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        555: {
            "attributes": {
                "Identifiant station": "20016",
                "Nom station": "Pyrénées - Maraîchers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.857140014,2.40435093641",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        556: {
            "attributes": {
                "Identifiant station": "20137",
                "Nom station": "Borrégo - Gambetta",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:57+01:00",
                "Coordonnées géographiques": "48.8733049705,2.40141458809",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        557: {
            "attributes": {
                "Identifiant station": "32015",
                "Nom station": "Gare RER de Saint-Denis",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "33",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:56+01:00",
                "Coordonnées géographiques": "48.9355432425,2.34714880586",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        558: {
            "attributes": {
                "Identifiant station": "12161",
                "Nom station": "Diderot - Reuilly",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "44",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "40",
                "Vélos mécaniques disponibles": "30",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:37+01:00",
                "Coordonnées géographiques": "48.8472084561,2.38612733781",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        559: {
            "attributes": {
                "Identifiant station": "16043",
                "Nom station": "Place de Passy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "15",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:48+01:00",
                "Coordonnées géographiques": "48.8578029171,2.27750054694",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        560: {
            "attributes": {
                "Identifiant station": "7013",
                "Nom station": "Duroc - Place de Breteuil",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:33+01:00",
                "Coordonnées géographiques": "48.8474606411,2.31261402369",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        561: {
            "attributes": {
                "Identifiant station": "18043",
                "Nom station": "Clichy - Place Blanche",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:38+01:00",
                "Coordonnées géographiques": "48.8836010883,2.33322378248",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        562: {
            "attributes": {
                "Identifiant station": "2013",
                "Nom station": "Favart - Italiens",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.8714463539,2.33829010585",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        563: {
            "attributes": {
                "Identifiant station": "18008",
                "Nom station": "Square Léon",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-11-03T13:51:22+01:00",
                "Coordonnées géographiques": "48.8866739679,2.35320556909",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        564: {
            "attributes": {
                "Identifiant station": "12041",
                "Nom station": "Bois de Vincennes.",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "60",
                "Nombre bornettes libres": "40",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:30+01:00",
                "Coordonnées géographiques": "48.8338064,2.4132221",
                "Nom communes équipées": "Saint-Mandé",
                "Code INSEE communes équipées": "",
            }
        },
        565: {
            "attributes": {
                "Identifiant station": "32004",
                "Nom station": "Wilson - Landy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "18",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:15+01:00",
                "Coordonnées géographiques": "48.9151672622,2.35763322562",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        566: {
            "attributes": {
                "Identifiant station": "15059",
                "Nom station": "Parc André Citroën",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "65",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "59",
                "Vélos mécaniques disponibles": "56",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:56+01:00",
                "Coordonnées géographiques": "48.8406399443,2.27806180716",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        567: {
            "attributes": {
                "Identifiant station": "16029",
                "Nom station": "Maison de Radio - France",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "47",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "39",
                "Vélos mécaniques disponibles": "39",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:07+01:00",
                "Coordonnées géographiques": "48.8513288643,2.27778553963",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        568: {
            "attributes": {
                "Identifiant station": "8038",
                "Nom station": "François 1er - Montaigne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "13",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.86688765,2.30667557572",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        569: {
            "attributes": {
                "Identifiant station": "18017",
                "Nom station": "Caulaincourt - Place Constantin Pecqueur",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "44",
                "Nombre bornettes libres": "41",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:46+01:00",
                "Coordonnées géographiques": "48.889677905,2.33817871443",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        570: {
            "attributes": {
                "Identifiant station": "18138",
                "Nom station": "Ney - Charles Hermite",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:50+01:00",
                "Coordonnées géographiques": "48.8987480398,2.3645754531",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        571: {
            "attributes": {
                "Identifiant station": "8019",
                "Nom station": "Naples - Rome",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:45+01:00",
                "Coordonnées géographiques": "48.8799607008,2.32135869563",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        572: {
            "attributes": {
                "Identifiant station": "32603",
                "Nom station": "Calmette.",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:19+01:00",
                "Coordonnées géographiques": "48.881049251,2.42499321699",
                "Nom communes équipées": "Les Lilas",
                "Code INSEE communes équipées": "",
            }
        },
        573: {
            "attributes": {
                "Identifiant station": "8001",
                "Nom station": "Petit Palais",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "35",
                "Vélos mécaniques disponibles": "27",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:19+01:00",
                "Coordonnées géographiques": "48.8667692,2.3157655",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        574: {
            "attributes": {
                "Identifiant station": "4011",
                "Nom station": "Hôtel de Ville - Pont-Marie",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:53+01:00",
                "Coordonnées géographiques": "48.8538996455,2.35694725066",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        575: {
            "attributes": {
                "Identifiant station": "7008",
                "Nom station": "Villersexel - Saint-Germain",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:11+01:00",
                "Coordonnées géographiques": "48.8582970012,2.32379347086",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        576: {
            "attributes": {
                "Identifiant station": "45001",
                "Nom station": "Hôtel de Ville de Champigny sur Marne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "18",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:15+01:00",
                "Coordonnées géographiques": "48.8138106284,2.50967592001",
                "Nom communes équipées": "Champigny-sur-Marne",
                "Code INSEE communes équipées": "",
            }
        },
        577: {
            "attributes": {
                "Identifiant station": "20201",
                "Nom station": "Eugène Reisz - Davout",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:37+01:00",
                "Coordonnées géographiques": "48.8563187463,2.41039697081",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        578: {
            "attributes": {
                "Identifiant station": "10163",
                "Nom station": "Gare du Nord - Demarquay",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:07+01:00",
                "Coordonnées géographiques": "48.881154,2.358411",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        579: {
            "attributes": {
                "Identifiant station": "16112",
                "Nom station": "Chernoviz - Raynouard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "33",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8569864146,2.28217321814",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        580: {
            "attributes": {
                "Identifiant station": "2001",
                "Nom station": "Greneta - Sebastopol",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.8652423332,2.35166698694",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        581: {
            "attributes": {
                "Identifiant station": "12004",
                "Nom station": "Traversière - Ledru-Rollin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:55+01:00",
                "Coordonnées géographiques": "48.8507358241,2.37587011171",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        582: {
            "attributes": {
                "Identifiant station": "7101",
                "Nom station": "Raspail - Chomel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "49",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "24",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:24+01:00",
                "Coordonnées géographiques": "48.8522331643,2.32665807009",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        583: {
            "attributes": {
                "Identifiant station": "15003",
                "Nom station": "Gare Montparnasse - Vaugirard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "66",
                "Nombre bornettes libres": "60",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:21+01:00",
                "Coordonnées géographiques": "48.8418071799,2.3195100269",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        584: {
            "attributes": {
                "Identifiant station": "18137",
                "Nom station": "Poissonniers - Nord",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:11+01:00",
                "Coordonnées géographiques": "48.8925112135,2.35222287476",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        585: {
            "attributes": {
                "Identifiant station": "6032",
                "Nom station": "Sabot - Rennes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:05+01:00",
                "Coordonnées géographiques": "48.8525751129,2.33155276626",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        586: {
            "attributes": {
                "Identifiant station": "17116",
                "Nom station": "Reims - Porte d'Asnières",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:28+01:00",
                "Coordonnées géographiques": "48.8918344482,2.30014465995",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        587: {
            "attributes": {
                "Identifiant station": "17022",
                "Nom station": "Pereire - Place de Wagram",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "41",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:06+01:00",
                "Coordonnées géographiques": "48.8870706076,2.30411112453",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        588: {
            "attributes": {
                "Identifiant station": "14116",
                "Nom station": "Romain Rolland",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:27+01:00",
                "Coordonnées géographiques": "48.8202152297,2.32341442344",
                "Nom communes équipées": "Montrouge",
                "Code INSEE communes équipées": "",
            }
        },
        589: {
            "attributes": {
                "Identifiant station": "10114",
                "Nom station": "Granges aux Belles - Hôpital Saint-Louis",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "60",
                "Nombre bornettes libres": "59",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:40+01:00",
                "Coordonnées géographiques": "48.8747949948,2.36666943698",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        590: {
            "attributes": {
                "Identifiant station": "18001",
                "Nom station": "Damrémont - Caulaincourt",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:08+01:00",
                "Coordonnées géographiques": "48.8871450908,2.33258809894",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        591: {
            "attributes": {
                "Identifiant station": "15058",
                "Nom station": "Place du Moulin de Javel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "31",
                "Vélos mécaniques disponibles": "23",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:09+01:00",
                "Coordonnées géographiques": "48.8402167092,2.27160606533",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        592: {
            "attributes": {
                "Identifiant station": "5019",
                "Nom station": "Poissy - Saint-Germain",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:23+01:00",
                "Coordonnées géographiques": "48.8496964943,2.35296852887",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        593: {
            "attributes": {
                "Identifiant station": "13122",
                "Nom station": "Parc de Choisy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:37+01:00",
                "Coordonnées géographiques": "48.8281562516,2.3585135329",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        594: {
            "attributes": {
                "Identifiant station": "18019",
                "Nom station": "Square Raymond Souplex",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "52",
                "Nombre bornettes libres": "51",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:04+01:00",
                "Coordonnées géographiques": "48.8918372148,2.33542610317",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        595: {
            "attributes": {
                "Identifiant station": "19041",
                "Nom station": "Belleville - Pyrénées",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "39",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:47+01:00",
                "Coordonnées géographiques": "48.8741908,2.3860039",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        596: {
            "attributes": {
                "Identifiant station": "15063",
                "Nom station": "Convention - Saint-Charles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:50+01:00",
                "Coordonnées géographiques": "48.8431561111,2.28310905397",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        597: {
            "attributes": {
                "Identifiant station": "21114",
                "Nom station": "Gustave Eiffel - Parc des Impressionnistes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:13+01:00",
                "Coordonnées géographiques": "48.9039061965,2.2946908325",
                "Nom communes équipées": "Clichy",
                "Code INSEE communes équipées": "",
            }
        },
        598: {
            "attributes": {
                "Identifiant station": "11003",
                "Nom station": "Keller - La Roquette",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:44+01:00",
                "Coordonnées géographiques": "48.8555508,2.3752778",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        599: {
            "attributes": {
                "Identifiant station": "15107",
                "Nom station": "Palais des Sports",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "19",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:16+01:00",
                "Coordonnées géographiques": "48.8334685174,2.28587206453",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        600: {
            "attributes": {
                "Identifiant station": "11034",
                "Nom station": "Parmentier - Fontaine au Roi",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "48",
                "Nombre bornettes libres": "39",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:56+01:00",
                "Coordonnées géographiques": "48.8678767055,2.37270597867",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        601: {
            "attributes": {
                "Identifiant station": "10033",
                "Nom station": "Gare du Nord - Saint-Vincent de Paul",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:57+01:00",
                "Coordonnées géographiques": "48.8809557416,2.35243051484",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        602: {
            "attributes": {
                "Identifiant station": "41206",
                "Nom station": "Place des Larris",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:13+01:00",
                "Coordonnées géographiques": "48.8568363,2.4780236",
                "Nom communes équipées": "Fontenay-sous-Bois",
                "Code INSEE communes équipées": "",
            }
        },
        603: {
            "attributes": {
                "Identifiant station": "31704",
                "Nom station": "Jeanne Hornet - Girardot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:25+01:00",
                "Coordonnées géographiques": "48.8738026814,2.42782026529",
                "Nom communes équipées": "Bagnolet",
                "Code INSEE communes équipées": "",
            }
        },
        604: {
            "attributes": {
                "Identifiant station": "11019",
                "Nom station": "Charonne - Philippe-Auguste",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:10+01:00",
                "Coordonnées géographiques": "48.855603162,2.39054639062",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        605: {
            "attributes": {
                "Identifiant station": "21024",
                "Nom station": "Billancourt - Général Leclerc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:23+01:00",
                "Coordonnées géographiques": "48.8322649,2.2380413",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        606: {
            "attributes": {
                "Identifiant station": "8015",
                "Nom station": "Roquépine - Malesherbes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "32",
                "Vélos mécaniques disponibles": "27",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.8734507339,2.32040449977",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        607: {
            "attributes": {
                "Identifiant station": "19023",
                "Nom station": "Manin - Simon Bolivar",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:21+01:00",
                "Coordonnées géographiques": "48.8764996541,2.3794073626",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        608: {
            "attributes": {
                "Identifiant station": "16016",
                "Nom station": "Paul Doumer - La Tour",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:19+01:00",
                "Coordonnées géographiques": "48.8601161096,2.28084048191",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        609: {
            "attributes": {
                "Identifiant station": "21311",
                "Nom station": "Parvis Corentin Celton",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "55",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "53",
                "Vélos mécaniques disponibles": "49",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8274725252,2.278431952",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        610: {
            "attributes": {
                "Identifiant station": "21108",
                "Nom station": "Mairie de Clichy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "51",
                "Nombre bornettes libres": "41",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:55+01:00",
                "Coordonnées géographiques": "48.9036536347,2.30593532324",
                "Nom communes équipées": "Clichy",
                "Code INSEE communes équipées": "",
            }
        },
        611: {
            "attributes": {
                "Identifiant station": "8056",
                "Nom station": "Beaujon - Wagram",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:12+01:00",
                "Coordonnées géographiques": "48.875352456,2.29644164443",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        612: {
            "attributes": {
                "Identifiant station": "6014",
                "Nom station": "Pont de Lodi - Dauphine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:36+01:00",
                "Coordonnées géographiques": "48.8552968986,2.33994730757",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        613: {
            "attributes": {
                "Identifiant station": "7016",
                "Nom station": "Surcouf - Université",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "58",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "47",
                "Vélos mécaniques disponibles": "47",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:46+01:00",
                "Coordonnées géographiques": "48.8612730163,2.30945355091",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        614: {
            "attributes": {
                "Identifiant station": "47002",
                "Nom station": "Les Juilliottes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:10+01:00",
                "Coordonnées géographiques": "48.8043072,2.4437362",
                "Nom communes équipées": "Maisons-Alfort",
                "Code INSEE communes équipées": "",
            }
        },
        615: {
            "attributes": {
                "Identifiant station": "7113",
                "Nom station": "Berges de Seine - Musée d'Orsay",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-01-27T07:51:48+01:00",
                "Coordonnées géographiques": "48.8627885937,2.30645935982",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        616: {
            "attributes": {
                "Identifiant station": "1022",
                "Nom station": "Danielle Casanova - Place Vendôme",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "27",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:39+01:00",
                "Coordonnées géographiques": "48.8682664982,2.33048390597",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        617: {
            "attributes": {
                "Identifiant station": "14006",
                "Nom station": "Saint-Jacques - Tombe Issoire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:02+01:00",
                "Coordonnées géographiques": "48.8332063305,2.33691081405",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        618: {
            "attributes": {
                "Identifiant station": "11046",
                "Nom station": "Place de la République - Voltaire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:11+01:00",
                "Coordonnées géographiques": "48.8668309689,2.36548103392",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        619: {
            "attributes": {
                "Identifiant station": "13041",
                "Nom station": "Porte d'Ivry",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "44",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:38+01:00",
                "Coordonnées géographiques": "48.8217687476,2.36887067556",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        620: {
            "attributes": {
                "Identifiant station": "5030",
                "Nom station": "Sorbonne - Ecoles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "45",
                "Vélos mécaniques disponibles": "31",
                "Vélos électriques disponibles": "14",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:42+01:00",
                "Coordonnées géographiques": "48.8496865662,2.34349966049",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        621: {
            "attributes": {
                "Identifiant station": "9118",
                "Nom station": "Faubourg Poissonniere - Delta",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:49+01:00",
                "Coordonnées géographiques": "48.8825558076,2.34974820167",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        622: {
            "attributes": {
                "Identifiant station": "11107",
                "Nom station": "Faidherbe - Dahomey",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "44",
                "Nombre bornettes libres": "37",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:38+01:00",
                "Coordonnées géographiques": "48.8513583298,2.3838231348",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        623: {
            "attributes": {
                "Identifiant station": "4013",
                "Nom station": "Vieille du Temple - Francs Bourgeois",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:01+01:00",
                "Coordonnées géographiques": "48.8582564134,2.35821794719",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        624: {
            "attributes": {
                "Identifiant station": "16041",
                "Nom station": "Versailles - Claude Terrasse",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "24",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:47+01:00",
                "Coordonnées géographiques": "48.8402193381,2.26383419115",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        625: {
            "attributes": {
                "Identifiant station": "31023",
                "Nom station": "Place François Mitterand",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "33",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:26+01:00",
                "Coordonnées géographiques": "48.8649278202,2.4429583177",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        626: {
            "attributes": {
                "Identifiant station": "4109",
                "Nom station": "Mornay - Sully",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "15",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:31+01:00",
                "Coordonnées géographiques": "48.8497039955,2.36446078867",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        627: {
            "attributes": {
                "Identifiant station": "18042",
                "Nom station": "Place Pigalle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8827555,2.3361566",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        628: {
            "attributes": {
                "Identifiant station": "3101",
                "Nom station": "Porte Saint-Martin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:46+01:00",
                "Coordonnées géographiques": "48.8686017211,2.3557010293",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        629: {
            "attributes": {
                "Identifiant station": "20019",
                "Nom station": "Place du Maquis du Vercors",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:28+01:00",
                "Coordonnées géographiques": "48.8773215795,2.40837827325",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        630: {
            "attributes": {
                "Identifiant station": "11048",
                "Nom station": "Place Mireille Havet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:50+01:00",
                "Coordonnées géographiques": "48.850533,2.383756",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        631: {
            "attributes": {
                "Identifiant station": "31013",
                "Nom station": "Centenaire - Sorins",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "42",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.8585352336,2.42735087872",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        632: {
            "attributes": {
                "Identifiant station": "6031",
                "Nom station": "Pierre Sarrazin - Saint-Michel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "32",
                "Vélos mécaniques disponibles": "27",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:00+01:00",
                "Coordonnées géographiques": "48.8509081464,2.34213072807",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        633: {
            "attributes": {
                "Identifiant station": "32018",
                "Nom station": "Franciade - Gabriel Péri",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:19+01:00",
                "Coordonnées géographiques": "48.9326149933,2.35595449805",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        634: {
            "attributes": {
                "Identifiant station": "8105",
                "Nom station": "François 1er - Lincoln",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "16",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.8695179793,2.30242621884",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        635: {
            "attributes": {
                "Identifiant station": "17016",
                "Nom station": "Brochant - Square des Batignolles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:09+01:00",
                "Coordonnées géographiques": "48.8881869858,2.31686114315",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        636: {
            "attributes": {
                "Identifiant station": "2101",
                "Nom station": "Bachaumont - Montmartre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "32",
                "Vélos mécaniques disponibles": "26",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:09+01:00",
                "Coordonnées géographiques": "48.8661739508,2.34497755766",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        637: {
            "attributes": {
                "Identifiant station": "13113",
                "Nom station": "Vistule - Choisy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "33",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-11-03T13:43:53+01:00",
                "Coordonnées géographiques": "48.8236554366,2.36157238483",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        638: {
            "attributes": {
                "Identifiant station": "14007",
                "Nom station": "Ferrus - Saint-Jacques",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "33",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:50+01:00",
                "Coordonnées géographiques": "48.8313878,2.3408563",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        639: {
            "attributes": {
                "Identifiant station": "12108",
                "Nom station": "Bercy.",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "66",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "62",
                "Vélos mécaniques disponibles": "45",
                "Vélos électriques disponibles": "17",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:40+01:00",
                "Coordonnées géographiques": "48.839966915,2.3788625747",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        640: {
            "attributes": {
                "Identifiant station": "10026",
                "Nom station": "Gare de l'Est - Chateau Landon",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "59",
                "Nombre bornettes libres": "56",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:39+01:00",
                "Coordonnées géographiques": "48.879305,2.362424",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        641: {
            "attributes": {
                "Identifiant station": "7110",
                "Nom station": "Quai Anatole France - Musée d'Orsay",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "12",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-01-22T21:01:53+01:00",
                "Coordonnées géographiques": "48.8612117406,2.32424609363",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        642: {
            "attributes": {
                "Identifiant station": "9015",
                "Nom station": "Lamartine - Faubourg Montmartre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:32+01:00",
                "Coordonnées géographiques": "48.8766139829,2.33976569027",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        643: {
            "attributes": {
                "Identifiant station": "9012",
                "Nom station": "Rougemont - Poissonnière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:24+01:00",
                "Coordonnées géographiques": "48.8716115694,2.34587308019",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        644: {
            "attributes": {
                "Identifiant station": "19021",
                "Nom station": "Mairie du 19ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8825626425,2.38126248121",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        645: {
            "attributes": {
                "Identifiant station": "20026",
                "Nom station": "Gambetta - Hôpital Tenon",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "35",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:30+01:00",
                "Coordonnées géographiques": "48.867925154,2.40097662355",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        646: {
            "attributes": {
                "Identifiant station": "14127",
                "Nom station": "Odessa - Départ",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:03+01:00",
                "Coordonnées géographiques": "48.8428855901,2.32442814857",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        647: {
            "attributes": {
                "Identifiant station": "15020",
                "Nom station": "Mairie du 15ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:53+01:00",
                "Coordonnées géographiques": "48.8417762,2.2986539",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        648: {
            "attributes": {
                "Identifiant station": "23105",
                "Nom station": "Sèvres Rive Gauche",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:43+01:00",
                "Coordonnées géographiques": "48.8214432183,2.21507853069",
                "Nom communes équipées": "Sèvres",
                "Code INSEE communes équipées": "",
            }
        },
        649: {
            "attributes": {
                "Identifiant station": "19031",
                "Nom station": "Manin - Crimée",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:37+01:00",
                "Coordonnées géographiques": "48.8831217744,2.38636706024",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        650: {
            "attributes": {
                "Identifiant station": "19030",
                "Nom station": "Melun - Jean Jaurès",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:32+01:00",
                "Coordonnées géographiques": "48.8837796723,2.37647742033",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        651: {
            "attributes": {
                "Identifiant station": "13114",
                "Nom station": "Nationale - Marcel Duchamp",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:30+01:00",
                "Coordonnées géographiques": "48.8246374934,2.36780148",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        652: {
            "attributes": {
                "Identifiant station": "21004",
                "Nom station": "Jean Jaurès - Reine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:25+01:00",
                "Coordonnées géographiques": "48.8400962256,2.2398725152",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        653: {
            "attributes": {
                "Identifiant station": "15061",
                "Nom station": "Square Desnouettes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.8347683793,2.28414229739",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        654: {
            "attributes": {
                "Identifiant station": "16031",
                "Nom station": "Vignes - Boulainvilliers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:27+01:00",
                "Coordonnées géographiques": "48.856399484,2.27524045855",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        655: {
            "attributes": {
                "Identifiant station": "20017",
                "Nom station": "Saint-Blaise - Mouraud",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:16+01:00",
                "Coordonnées géographiques": "48.856961776,2.40869477391",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        656: {
            "attributes": {
                "Identifiant station": "12127",
                "Nom station": "Tremblay - Lac des Minimes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "48",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "39",
                "Vélos mécaniques disponibles": "31",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:22+01:00",
                "Coordonnées géographiques": "48.8341312615,2.45475165546",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        657: {
            "attributes": {
                "Identifiant station": "23008",
                "Nom station": "Hôtel de Ville de Levallois Perret",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:36+01:00",
                "Coordonnées géographiques": "48.89314452,2.28888183832",
                "Nom communes équipées": "Levallois-Perret",
                "Code INSEE communes équipées": "",
            }
        },
        658: {
            "attributes": {
                "Identifiant station": "12153",
                "Nom station": "Terroirs de France - Lheureux",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "55",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "54",
                "Vélos mécaniques disponibles": "41",
                "Vélos électriques disponibles": "13",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:55+01:00",
                "Coordonnées géographiques": "48.8321202707,2.38861810416",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        659: {
            "attributes": {
                "Identifiant station": "1016",
                "Nom station": "Thérèse - Opéra",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "12",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.8664835145,2.33443342122",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        660: {
            "attributes": {
                "Identifiant station": "5028",
                "Nom station": "Censier - Jardin des Plantes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8410717816,2.35547538847",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        661: {
            "attributes": {
                "Identifiant station": "7026",
                "Nom station": "Place de Finlande",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:20+01:00",
                "Coordonnées géographiques": "48.862408336,2.3113624379",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        662: {
            "attributes": {
                "Identifiant station": "17005",
                "Nom station": "Brochant - Clichy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "53",
                "Nombre bornettes libres": "47",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:12+01:00",
                "Coordonnées géographiques": "48.8903299891,2.31973055993",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        663: {
            "attributes": {
                "Identifiant station": "31018",
                "Nom station": "Gambetta - Michelet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:24+01:00",
                "Coordonnées géographiques": "48.8527379238,2.42996603251",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        664: {
            "attributes": {
                "Identifiant station": "8032",
                "Nom station": "Matignon - Faubourg Saint-Honoré",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "36",
                "Vélos mécaniques disponibles": "29",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:29+01:00",
                "Coordonnées géographiques": "48.8714785912,2.31397792697",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        665: {
            "attributes": {
                "Identifiant station": "33005",
                "Nom station": "République - Henri Barbusse",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:50+01:00",
                "Coordonnées géographiques": "48.9066155363,2.38943248987",
                "Nom communes équipées": "Aubervilliers",
                "Code INSEE communes équipées": "",
            }
        },
        666: {
            "attributes": {
                "Identifiant station": "14016",
                "Nom station": "René Coty - Parc Montsouris",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "55",
                "Nombre bornettes libres": "55",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:12+01:00",
                "Coordonnées géographiques": "48.824791345,2.33612492681",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        667: {
            "attributes": {
                "Identifiant station": "2110",
                "Nom station": "Jeûneurs - Mulhouse",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:21+01:00",
                "Coordonnées géographiques": "48.868997,2.346925",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        668: {
            "attributes": {
                "Identifiant station": "11044",
                "Nom station": "Square du Bataclan",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:29+01:00",
                "Coordonnées géographiques": "48.8633762,2.3711327",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        669: {
            "attributes": {
                "Identifiant station": "13019",
                "Nom station": "Fernand Braudel - Vincent Auriol",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "32",
                "Vélos mécaniques disponibles": "22",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:16+01:00",
                "Coordonnées géographiques": "48.8363785232,2.3723404482",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        670: {
            "attributes": {
                "Identifiant station": "18142",
                "Nom station": "Jessaint - Marx Dormoy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:31+01:00",
                "Coordonnées géographiques": "48.8848454311,2.35893040895",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        671: {
            "attributes": {
                "Identifiant station": "8049",
                "Nom station": "Georges V - François 1er",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.87034,2.30082",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        672: {
            "attributes": {
                "Identifiant station": "21017",
                "Nom station": "Place Marcel Sembat",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "56",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "39",
                "Vélos mécaniques disponibles": "31",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:38+01:00",
                "Coordonnées géographiques": "48.8332886503,2.24417477846",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        673: {
            "attributes": {
                "Identifiant station": "9113",
                "Nom station": "Bleue - Trévise",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.8757890656,2.34728962183",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        674: {
            "attributes": {
                "Identifiant station": "42011",
                "Nom station": "Quai Marcel Boyer - Victor Hugo",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.8225563667,2.39203959703",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        675: {
            "attributes": {
                "Identifiant station": "20103",
                "Nom station": "Le Vau - Maurice Bertaux",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:56+01:00",
                "Coordonnées géographiques": "48.8680343904,2.41103061245",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        676: {
            "attributes": {
                "Identifiant station": "15009",
                "Nom station": "Suffren - Pérignon",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "30",
                "Vélos mécaniques disponibles": "27",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:45+01:00",
                "Coordonnées géographiques": "48.8471756149,2.3070521476",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        677: {
            "attributes": {
                "Identifiant station": "2014",
                "Nom station": "Daunou - Louis le Grand",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:47+01:00",
                "Coordonnées géographiques": "48.869113,2.332515",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        678: {
            "attributes": {
                "Identifiant station": "4107",
                "Nom station": "Beaumarchais - Pas de la Mule",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:16+01:00",
                "Coordonnées géographiques": "48.855580507,2.36843336734",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        679: {
            "attributes": {
                "Identifiant station": "23302",
                "Nom station": "Parc de Sceaux - Centre-Ville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:10+01:00",
                "Coordonnées géographiques": "48.7769529826,2.29572147131",
                "Nom communes équipées": "Sceaux",
                "Code INSEE communes équipées": "",
            }
        },
        680: {
            "attributes": {
                "Identifiant station": "9013",
                "Nom station": "Mairie du 9ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:55+01:00",
                "Coordonnées géographiques": "48.8732671376,2.34075344805",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        681: {
            "attributes": {
                "Identifiant station": "21952",
                "Nom station": "Place Tony de Graaff",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:05+01:00",
                "Coordonnées géographiques": "48.8181645,2.2458109",
                "Nom communes équipées": "Meudon",
                "Code INSEE communes équipées": "",
            }
        },
        682: {
            "attributes": {
                "Identifiant station": "31017",
                "Nom station": "Jean Moulin - Paul Doumer",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:45+01:00",
                "Coordonnées géographiques": "48.8622179855,2.45533436537",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        683: {
            "attributes": {
                "Identifiant station": "12154",
                "Nom station": "Square Georges Melies",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:23+01:00",
                "Coordonnées géographiques": "48.8407023921,2.40966774523",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        684: {
            "attributes": {
                "Identifiant station": "13047",
                "Nom station": "Boutroux - Porte de Vitry",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "49",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "30",
                "Vélos mécaniques disponibles": "29",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:50+01:00",
                "Coordonnées géographiques": "48.822632,2.377738",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        685: {
            "attributes": {
                "Identifiant station": "10164",
                "Nom station": "Claude Vellefaux - Sambre et Meuse",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:59+01:00",
                "Coordonnées géographiques": "48.875022,2.370478",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        686: {
            "attributes": {
                "Identifiant station": "13117",
                "Nom station": "Stade Georges Carpentier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "37",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:51+01:00",
                "Coordonnées géographiques": "48.8204824384,2.36683219671",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        687: {
            "attributes": {
                "Identifiant station": "8029",
                "Nom station": "Place du Canada",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "32",
                "Vélos mécaniques disponibles": "23",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:58+01:00",
                "Coordonnées géographiques": "48.8653462694,2.31019378796",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        688: {
            "attributes": {
                "Identifiant station": "92005",
                "Nom station": "Gare de Nanterre Ville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:53+01:00",
                "Coordonnées géographiques": "48.8950295,2.1957932",
                "Nom communes équipées": "Nanterre",
                "Code INSEE communes équipées": "",
            }
        },
        689: {
            "attributes": {
                "Identifiant station": "17104",
                "Nom station": "Jardin Marguerite Long",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:18+01:00",
                "Coordonnées géographiques": "48.8816985774,2.28365756571",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        690: {
            "attributes": {
                "Identifiant station": "22103",
                "Nom station": "Senard - Charles Blum",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:16+01:00",
                "Coordonnées géographiques": "48.8573158248,2.22109168768",
                "Nom communes équipées": "Saint-Cloud",
                "Code INSEE communes équipées": "",
            }
        },
        691: {
            "attributes": {
                "Identifiant station": "22002",
                "Nom station": "Argenson - Château",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8885593147,2.26420015097",
                "Nom communes équipées": "Neuilly-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        692: {
            "attributes": {
                "Identifiant station": "21120",
                "Nom station": "Docteur Calmette - Martre",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-07-22T18:48:35+02:00",
                "Coordonnées géographiques": "48.8989693585,2.30865132524",
                "Nom communes équipées": "Clichy",
                "Code INSEE communes équipées": "",
            }
        },
        693: {
            "attributes": {
                "Identifiant station": "35008",
                "Nom station": "Place de l'Eglise - Pantin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:22+01:00",
                "Coordonnées géographiques": "48.8928094615,2.41243511438",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        694: {
            "attributes": {
                "Identifiant station": "14128",
                "Nom station": "Gergovie - Vercingétorix",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:23+01:00",
                "Coordonnées géographiques": "48.8345341388,2.31315454743",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        695: {
            "attributes": {
                "Identifiant station": "13022",
                "Nom station": "Butte aux Cailles - Esperance",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "35",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:15+01:00",
                "Coordonnées géographiques": "48.8275877419,2.34921511263",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        696: {
            "attributes": {
                "Identifiant station": "12125",
                "Nom station": "Pyramide - Parc Floral",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "37",
                "Vélos mécaniques disponibles": "32",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:50+01:00",
                "Coordonnées géographiques": "48.837442,2.440452",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        697: {
            "attributes": {
                "Identifiant station": "20045",
                "Nom station": "Louis Ganne - Davout",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:23+01:00",
                "Coordonnées géographiques": "48.8633872069,2.409725748",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        698: {
            "attributes": {
                "Identifiant station": "31101",
                "Nom station": "Jules Guesde - Laennec",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:14+01:00",
                "Coordonnées géographiques": "48.876289839,2.50069420785",
                "Nom communes équipées": "Rosny-sous-Bois",
                "Code INSEE communes équipées": "",
            }
        },
        699: {
            "attributes": {
                "Identifiant station": "11110",
                "Nom station": "Roquette - Folie Regnault",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "52",
                "Nombre bornettes libres": "46",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:56+01:00",
                "Coordonnées géographiques": "48.8592469592,2.38655362358",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        700: {
            "attributes": {
                "Identifiant station": "12129",
                "Nom station": "Bois de Vincennes - Porte Jaune",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:22+01:00",
                "Coordonnées géographiques": "48.8387512467,2.46054891497",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        701: {
            "attributes": {
                "Identifiant station": "22401",
                "Nom station": "Charles de Gaulle - Plateau de Vanves",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:40+01:00",
                "Coordonnées géographiques": "48.8223308897,2.29857180341",
                "Nom communes équipées": "Malakoff",
                "Code INSEE communes équipées": "",
            }
        },
        702: {
            "attributes": {
                "Identifiant station": "17001",
                "Nom station": "Guy Môquet - Saint-Ouen",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "53",
                "Nombre bornettes libres": "48",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:55+01:00",
                "Coordonnées géographiques": "48.892777719,2.32703357935",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        703: {
            "attributes": {
                "Identifiant station": "12033",
                "Nom station": "François Truffaut - Saint Emilion",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "66",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "57",
                "Vélos mécaniques disponibles": "54",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8337235434,2.38660190327",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        704: {
            "attributes": {
                "Identifiant station": "10039",
                "Nom station": "Sambre et Meuse - Villette",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:22+01:00",
                "Coordonnées géographiques": "48.8744056662,2.373807238",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        705: {
            "attributes": {
                "Identifiant station": "12120",
                "Nom station": "Lac Daumesnil - Saint-Maurice",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "47",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:16+01:00",
                "Coordonnées géographiques": "48.8277455574,2.41836547852",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        706: {
            "attributes": {
                "Identifiant station": "8011",
                "Nom station": "Clapeyron - Moscou",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "15",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:06+01:00",
                "Coordonnées géographiques": "48.8807881951,2.32481505722",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        707: {
            "attributes": {
                "Identifiant station": "22012",
                "Nom station": "Madeleine Michelis - Roule",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "24",
                "Vélos mécaniques disponibles": "18",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:11+01:00",
                "Coordonnées géographiques": "48.882604313,2.27736342698",
                "Nom communes équipées": "Neuilly-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        708: {
            "attributes": {
                "Identifiant station": "13021",
                "Nom station": "Boussingault - Tolbiac",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:38+01:00",
                "Coordonnées géographiques": "48.8260351661,2.342176808",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        709: {
            "attributes": {
                "Identifiant station": "18014",
                "Nom station": "Léon - Doudeauville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "13",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:34+01:00",
                "Coordonnées géographiques": "48.8883242185,2.35357337264",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        710: {
            "attributes": {
                "Identifiant station": "13020",
                "Nom station": "Paul Klee - Place Augusta Holmes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "38",
                "Vélos mécaniques disponibles": "20",
                "Vélos électriques disponibles": "18",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:28+01:00",
                "Coordonnées géographiques": "48.8389464174,2.3704235664",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        711: {
            "attributes": {
                "Identifiant station": "18035",
                "Nom station": "René Binet - Porte de Montmartre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:42+01:00",
                "Coordonnées géographiques": "48.8990510995,2.3365534097",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        712: {
            "attributes": {
                "Identifiant station": "11031",
                "Nom station": "Jacquard - Ternaux",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:16+01:00",
                "Coordonnées géographiques": "48.8646605,2.373162",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        713: {
            "attributes": {
                "Identifiant station": "13110",
                "Nom station": "Moulin de la Pointe",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:34+01:00",
                "Coordonnées géographiques": "48.8209294605,2.35631793737",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        714: {
            "attributes": {
                "Identifiant station": "19124",
                "Nom station": "Alexander Fleming - Belvédère",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:40:19+01:00",
                "Coordonnées géographiques": "48.881790642,2.40305993776",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        715: {
            "attributes": {
                "Identifiant station": "11012",
                "Nom station": "Montreuil - Voltaire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:35+01:00",
                "Coordonnées géographiques": "48.8505070118,2.39307555827",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        716: {
            "attributes": {
                "Identifiant station": "35011",
                "Nom station": "Honoré d'Estienne d'Ovres - Grilles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:40+01:00",
                "Coordonnées géographiques": "48.8905639474,2.40648318438",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        717: {
            "attributes": {
                "Identifiant station": "23005",
                "Nom station": "Gare de Clichy-Levallois",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8967982748,2.29846805334",
                "Nom communes équipées": "Levallois-Perret",
                "Code INSEE communes équipées": "",
            }
        },
        718: {
            "attributes": {
                "Identifiant station": "15019",
                "Nom station": "Paul Barruel - Vaugirard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:10+01:00",
                "Coordonnées géographiques": "48.8401491855,2.30443328619",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        719: {
            "attributes": {
                "Identifiant station": "12010",
                "Nom station": "Daumesnil - Picpus",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:21+01:00",
                "Coordonnées géographiques": "48.8374688,2.401765",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        720: {
            "attributes": {
                "Identifiant station": "16007",
                "Nom station": "Longchamp - Place d'Iéna",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:12+01:00",
                "Coordonnées géographiques": "48.8649172334,2.29267481714",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        721: {
            "attributes": {
                "Identifiant station": "20117",
                "Nom station": "Saint-Fargeau - Mortier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:40+01:00",
                "Coordonnées géographiques": "48.8726665612,2.40791290998",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        722: {
            "attributes": {
                "Identifiant station": "10110",
                "Nom station": "Louis Blanc - Jemmapes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:54+01:00",
                "Coordonnées géographiques": "48.8793307553,2.36844856292",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        723: {
            "attributes": {
                "Identifiant station": "48009",
                "Nom station": "République - Auguste Perret",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:07+01:00",
                "Coordonnées géographiques": "48.7869528095,2.35185943544",
                "Nom communes équipées": "Villejuif",
                "Code INSEE communes équipées": "",
            }
        },
        724: {
            "attributes": {
                "Identifiant station": "13515",
                "Nom station": "Auguste Blanqui - Glacière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "60",
                "Nombre bornettes libres": "53",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.831151603,2.34382219613",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        725: {
            "attributes": {
                "Identifiant station": "14125",
                "Nom station": "Dareau - Broussais",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:27+01:00",
                "Coordonnées géographiques": "48.8307758,2.3361804",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        726: {
            "attributes": {
                "Identifiant station": "13010",
                "Nom station": "Place d'Italie - Vincent Auriol",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:29+01:00",
                "Coordonnées géographiques": "48.8314818745,2.35675502923",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        727: {
            "attributes": {
                "Identifiant station": "21009",
                "Nom station": "Le Corbusier - Jean Jaurès",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "30",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:46+01:00",
                "Coordonnées géographiques": "48.8378083056,2.24083811045",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        728: {
            "attributes": {
                "Identifiant station": "13027",
                "Nom station": "Mazagran - Pierre de Coubertin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "15",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:26+01:00",
                "Coordonnées géographiques": "48.8162186403,2.34422687441",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        729: {
            "attributes": {
                "Identifiant station": "21951",
                "Nom station": "Place Centrale",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:27+01:00",
                "Coordonnées géographiques": "48.7874016769,2.2278830409",
                "Nom communes équipées": "Meudon",
                "Code INSEE communes équipées": "",
            }
        },
        730: {
            "attributes": {
                "Identifiant station": "17029",
                "Nom station": "Pierre Demours - Laugier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:46+01:00",
                "Coordonnées géographiques": "48.8813072,2.294846",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        731: {
            "attributes": {
                "Identifiant station": "23006",
                "Nom station": "Jules Guesde - Alsace",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:26+01:00",
                "Coordonnées géographiques": "48.8905662755,2.29507774115",
                "Nom communes équipées": "Levallois-Perret",
                "Code INSEE communes équipées": "",
            }
        },
        732: {
            "attributes": {
                "Identifiant station": "10004",
                "Nom station": "Metz - Faubourg Saint-Denis",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "48",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "36",
                "Vélos mécaniques disponibles": "33",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:40+01:00",
                "Coordonnées géographiques": "48.870893,2.353521",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        733: {
            "attributes": {
                "Identifiant station": "18114",
                "Nom station": "Véron - Lepic",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:07+01:00",
                "Coordonnées géographiques": "48.885261234,2.33455985785",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        734: {
            "attributes": {
                "Identifiant station": "6024",
                "Nom station": "Saint-Germain-des-Prés",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "24",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:12+01:00",
                "Coordonnées géographiques": "48.8541851082,2.33030587435",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        735: {
            "attributes": {
                "Identifiant station": "44103",
                "Nom station": "Pont De Charenton",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:08+01:00",
                "Coordonnées géographiques": "48.8179685,2.4199531",
                "Nom communes équipées": "Saint-Maurice",
                "Code INSEE communes équipées": "",
            }
        },
        736: {
            "attributes": {
                "Identifiant station": "11103",
                "Nom station": "Pelée - Alphonse Baudin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:31+01:00",
                "Coordonnées géographiques": "48.8600265,2.3710541",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        737: {
            "attributes": {
                "Identifiant station": "22403",
                "Nom station": "Pierre Brossolette - Adolphe Pinard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:11+01:00",
                "Coordonnées géographiques": "48.8217722795,2.31315582991",
                "Nom communes équipées": "Malakoff",
                "Code INSEE communes équipées": "",
            }
        },
        738: {
            "attributes": {
                "Identifiant station": "10153",
                "Nom station": "Gare du Nord - Denain",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:12+01:00",
                "Coordonnées géographiques": "48.8795957876,2.35447257757",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        739: {
            "attributes": {
                "Identifiant station": "22009",
                "Nom station": "Charles de Gaulle - Orléans",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:38+01:00",
                "Coordonnées géographiques": "48.880922251,2.27158159018",
                "Nom communes équipées": "Neuilly-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        740: {
            "attributes": {
                "Identifiant station": "8008",
                "Nom station": "Joseph Sansboeuf - Saint Lazare",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:58+01:00",
                "Coordonnées géographiques": "48.8753156307,2.32290197164",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        741: {
            "attributes": {
                "Identifiant station": "13035",
                "Nom station": "Choisy - Tolbiac",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:37+01:00",
                "Coordonnées géographiques": "48.8259115559,2.36023932695",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        742: {
            "attributes": {
                "Identifiant station": "21203",
                "Nom station": "Place Jules Ferry",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:40+01:00",
                "Coordonnées géographiques": "48.8121079284,2.32583731413",
                "Nom communes équipées": "Montrouge",
                "Code INSEE communes équipées": "",
            }
        },
        743: {
            "attributes": {
                "Identifiant station": "20047",
                "Nom station": "Cours de Vincennes - Davout",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "48",
                "Nombre bornettes libres": "45",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:58+01:00",
                "Coordonnées géographiques": "48.84723803,2.41070423362",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        744: {
            "attributes": {
                "Identifiant station": "21115",
                "Nom station": "Chancé Milly - Cimetière Communal Sud",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:55+01:00",
                "Coordonnées géographiques": "48.8968718938,2.30392232537",
                "Nom communes équipées": "Clichy",
                "Code INSEE communes équipées": "",
            }
        },
        745: {
            "attributes": {
                "Identifiant station": "7024",
                "Nom station": "Rapp - Place du Général Gouraud",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "24",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:34+01:00",
                "Coordonnées géographiques": "48.8581028851,2.30040729046",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        746: {
            "attributes": {
                "Identifiant station": "19025",
                "Nom station": "Botzaris - Crimée",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:40+01:00",
                "Coordonnées géographiques": "48.8795969869,2.38879133375",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        747: {
            "attributes": {
                "Identifiant station": "31026",
                "Nom station": "Demi-Lune - Aristide Briand",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:58+01:00",
                "Coordonnées géographiques": "48.8762638192,2.4562818557",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        748: {
            "attributes": {
                "Identifiant station": "16009",
                "Nom station": "Saint-Didier - Raymond Poincaré",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:45+01:00",
                "Coordonnées géographiques": "48.8664231202,2.28586349719",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        749: {
            "attributes": {
                "Identifiant station": "20004",
                "Nom station": "Commandant l'Herminier - Gallieni",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:51+01:00",
                "Coordonnées géographiques": "48.8470405526,2.41604536772",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        750: {
            "attributes": {
                "Identifiant station": "7111",
                "Nom station": "Ecole Militaire - Place Joffre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "18",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:18+01:00",
                "Coordonnées géographiques": "48.8520602036,2.30179935694",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        751: {
            "attributes": {
                "Identifiant station": "19113",
                "Nom station": "Pré Saint-Gervais - Lilas",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:48+01:00",
                "Coordonnées géographiques": "48.877870627,2.39590734243",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        752: {
            "attributes": {
                "Identifiant station": "20039",
                "Nom station": "Belleville - Couronnes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "47",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:08+01:00",
                "Coordonnées géographiques": "48.8688310829,2.38098621368",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        753: {
            "attributes": {
                "Identifiant station": "9016",
                "Nom station": "Choron - Martyrs",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:45+01:00",
                "Coordonnées géographiques": "48.8778514435,2.33967013657",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        754: {
            "attributes": {
                "Identifiant station": "13048",
                "Nom station": "Wurtz - Tolbiac",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:15+01:00",
                "Coordonnées géographiques": "48.8263119562,2.34458595514",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        755: {
            "attributes": {
                "Identifiant station": "35004",
                "Nom station": "Place du Président Salvador Allende",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "60",
                "Nombre bornettes libres": "52",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:27+01:00",
                "Coordonnées géographiques": "48.8973073104,2.40079153681",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        756: {
            "attributes": {
                "Identifiant station": "11109",
                "Nom station": "Pierre Levée - Fontaine Timbaud",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:04+01:00",
                "Coordonnées géographiques": "48.8663272337,2.37111635506",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        757: {
            "attributes": {
                "Identifiant station": "33104",
                "Nom station": "Square Edmond Pépin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:16+01:00",
                "Coordonnées géographiques": "48.8836562069,2.40335315466",
                "Nom communes équipées": "Le Pré-Saint-Gervais",
                "Code INSEE communes équipées": "",
            }
        },
        758: {
            "attributes": {
                "Identifiant station": "21316",
                "Nom station": "Cimetière d'Issy-les-Moulineaux",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:16+01:00",
                "Coordonnées géographiques": "48.8176997739,2.26093120873",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        759: {
            "attributes": {
                "Identifiant station": "22010",
                "Nom station": "Charles de Gaulle - Louis-Philippe",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "14",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:23+01:00",
                "Coordonnées géographiques": "48.8816213111,2.27128300904",
                "Nom communes équipées": "Neuilly-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        760: {
            "attributes": {
                "Identifiant station": "31021",
                "Nom station": "Résistance - Hoche",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:46+01:00",
                "Coordonnées géographiques": "48.8642734237,2.43578441441",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        761: {
            "attributes": {
                "Identifiant station": "19036",
                "Nom station": "Hôpital Robert Debré",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:09+01:00",
                "Coordonnées géographiques": "48.880052222,2.40086503463",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        762: {
            "attributes": {
                "Identifiant station": "44002",
                "Nom station": "Paul Vaillant-Couturier - Gare RER",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:18+01:00",
                "Coordonnées géographiques": "48.8010815907,2.40209721029",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        763: {
            "attributes": {
                "Identifiant station": "14113",
                "Nom station": "Santé - Arago",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:01+01:00",
                "Coordonnées géographiques": "48.8350765756,2.3415737262",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        764: {
            "attributes": {
                "Identifiant station": "13121",
                "Nom station": "Bruneseau - Quai d'Ivry",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "48",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "41",
                "Vélos mécaniques disponibles": "37",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:12+01:00",
                "Coordonnées géographiques": "48.8249456376,2.38856613636",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        765: {
            "attributes": {
                "Identifiant station": "35015",
                "Nom station": "Averroès - Aimé Césaire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:10+01:00",
                "Coordonnées géographiques": "48.9140532343,2.41263695061",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        766: {
            "attributes": {
                "Identifiant station": "21030",
                "Nom station": "Fessart - Château",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:08+01:00",
                "Coordonnées géographiques": "48.84245,2.23989",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        767: {
            "attributes": {
                "Identifiant station": "32021",
                "Nom station": "Place du Front Populaire - George Sand",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "20",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:22+01:00",
                "Coordonnées géographiques": "48.906807137,2.36454121499",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        768: {
            "attributes": {
                "Identifiant station": "21031",
                "Nom station": "Cimetière de Boulogne-Billancourt",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "36",
                "Vélos mécaniques disponibles": "31",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.830764,2.252024",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        769: {
            "attributes": {
                "Identifiant station": "12006",
                "Nom station": "Gare de Lyon - Van Gogh",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "51",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "37",
                "Vélos mécaniques disponibles": "27",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:34+01:00",
                "Coordonnées géographiques": "48.8438397856,2.37044502404",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        770: {
            "attributes": {
                "Identifiant station": "16024",
                "Nom station": "Ranelagh - Maison de la Radio",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:45+01:00",
                "Coordonnées géographiques": "48.8529360317,2.28044964373",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        771: {
            "attributes": {
                "Identifiant station": "15060",
                "Nom station": "Place de la Montagne du Goulet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:47+01:00",
                "Coordonnées géographiques": "48.8443415,2.2773844",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        772: {
            "attributes": {
                "Identifiant station": "9101",
                "Nom station": "Cadet - La Fayette",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:15+01:00",
                "Coordonnées géographiques": "48.8756016327,2.34357040375",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        773: {
            "attributes": {
                "Identifiant station": "15131",
                "Nom station": "Charles Lecocq - Lecourbe",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:54+01:00",
                "Coordonnées géographiques": "48.84044,2.294867",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        774: {
            "attributes": {
                "Identifiant station": "20009",
                "Nom station": "Docteurs Déjérine - Pte de Montreuil",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:48+01:00",
                "Coordonnées géographiques": "48.8539844813,2.41208288897",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        775: {
            "attributes": {
                "Identifiant station": "10037",
                "Nom station": "Quai de Valmy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:43+01:00",
                "Coordonnées géographiques": "48.8812695158,2.36818604171",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        776: {
            "attributes": {
                "Identifiant station": "11032",
                "Nom station": "Jean-Pierre Timbaud - Vaucouleurs",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "47",
                "Nombre bornettes libres": "46",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:12+01:00",
                "Coordonnées géographiques": "48.867937,2.377885",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        777: {
            "attributes": {
                "Identifiant station": "20002",
                "Nom station": "Square de la rue Pixérécourt",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "15",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:28+01:00",
                "Coordonnées géographiques": "48.8736824,2.3960278",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        778: {
            "attributes": {
                "Identifiant station": "17033",
                "Nom station": "Carnot - Général Lanrezac",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "33",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-10-25T10:41:59+02:00",
                "Coordonnées géographiques": "48.8754438707,2.2938242935",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        779: {
            "attributes": {
                "Identifiant station": "5029",
                "Nom station": "Georges Bernanos - Port Royal",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:34+01:00",
                "Coordonnées géographiques": "48.8401452135,2.33735740185",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        780: {
            "attributes": {
                "Identifiant station": "16003",
                "Nom station": "André Maurois - Joseph et Marie Hackin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:01+01:00",
                "Coordonnées géographiques": "48.8778983,2.2789591",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        781: {
            "attributes": {
                "Identifiant station": "9036",
                "Nom station": "Athènes - Clichy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:39+01:00",
                "Coordonnées géographiques": "48.8781137879,2.32956021946",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        782: {
            "attributes": {
                "Identifiant station": "10012",
                "Nom station": "Vinaigriers - Magenta",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:52+01:00",
                "Coordonnées géographiques": "48.8731167,2.3593052",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        783: {
            "attributes": {
                "Identifiant station": "12150",
                "Nom station": "Bastille - Quai de la Rapée",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:28+01:00",
                "Coordonnées géographiques": "48.8468241086,2.36693646759",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        784: {
            "attributes": {
                "Identifiant station": "21028",
                "Nom station": "Pierre Lefaucheux - Traversière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:10+01:00",
                "Coordonnées géographiques": "48.824352083,2.24179130048",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        785: {
            "attributes": {
                "Identifiant station": "15125",
                "Nom station": "Parc Suzanne Lenglen",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "52",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "39",
                "Vélos mécaniques disponibles": "38",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:19+01:00",
                "Coordonnées géographiques": "48.8332506905,2.27654032409",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        786: {
            "attributes": {
                "Identifiant station": "20007",
                "Nom station": "Buzenval - Avron",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "42",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:45+01:00",
                "Coordonnées géographiques": "48.851617205,2.40151359731",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        787: {
            "attributes": {
                "Identifiant station": "31707",
                "Nom station": "Edouard Vaillant - Galliéni",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "49",
                "Nombre bornettes libres": "49",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2020-08-14T04:12:17+02:00",
                "Coordonnées géographiques": "48.8584628817,2.41479545832",
                "Nom communes équipées": "Bagnolet",
                "Code INSEE communes équipées": "",
            }
        },
        788: {
            "attributes": {
                "Identifiant station": "32301",
                "Nom station": "Mairie de Romainville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:58+01:00",
                "Coordonnées géographiques": "48.885109,2.435679",
                "Nom communes équipées": "Romainville",
                "Code INSEE communes équipées": "",
            }
        },
        789: {
            "attributes": {
                "Identifiant station": "10152",
                "Nom station": "Gare du Nord - Place de Valenciennes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:41+01:00",
                "Coordonnées géographiques": "48.879089,2.354235",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        790: {
            "attributes": {
                "Identifiant station": "13011",
                "Nom station": "Hôpital - Campo-Formio",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:27+01:00",
                "Coordonnées géographiques": "48.835405,2.358218",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        791: {
            "attributes": {
                "Identifiant station": "16014",
                "Nom station": "Eylau - Trocadéro",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:33+01:00",
                "Coordonnées géographiques": "48.8633541224,2.28642091155",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        792: {
            "attributes": {
                "Identifiant station": "22501",
                "Nom station": "Théâtre de La Garenne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:19+01:00",
                "Coordonnées géographiques": "48.9057683093,2.23539210223",
                "Nom communes équipées": "La Garenne-Colombes",
                "Code INSEE communes équipées": "",
            }
        },
        793: {
            "attributes": {
                "Identifiant station": "13039",
                "Nom station": "Ivry - Simone Weil",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:30+01:00",
                "Coordonnées géographiques": "48.823283932,2.36590951681",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        794: {
            "attributes": {
                "Identifiant station": "16030",
                "Nom station": "Place de Barcelone - Mirabeau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "55",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "44",
                "Vélos mécaniques disponibles": "37",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:27+01:00",
                "Coordonnées géographiques": "48.8476230274,2.27356642485",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        795: {
            "attributes": {
                "Identifiant station": "21118",
                "Nom station": "Neuilly - Petit",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:17+01:00",
                "Coordonnées géographiques": "48.8989653613,2.29627937078",
                "Nom communes équipées": "Clichy",
                "Code INSEE communes équipées": "",
            }
        },
        796: {
            "attributes": {
                "Identifiant station": "21219",
                "Nom station": "Amaury Duval - Gabriel Péri",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "35",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:54+01:00",
                "Coordonnées géographiques": "48.8171875967,2.32384175062",
                "Nom communes équipées": "Montrouge",
                "Code INSEE communes équipées": "",
            }
        },
        797: {
            "attributes": {
                "Identifiant station": "5034",
                "Nom station": "Censier - Santeuil",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "69",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "37",
                "Vélos mécaniques disponibles": "34",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8406589214,2.35375240445",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        798: {
            "attributes": {
                "Identifiant station": "10018",
                "Nom station": "Square Alban Satragne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:23+01:00",
                "Coordonnées géographiques": "48.8753405484,2.35588476062",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        799: {
            "attributes": {
                "Identifiant station": "22013",
                "Nom station": "Mairie de Neuilly sur Seine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:57+01:00",
                "Coordonnées géographiques": "48.8849472877,2.26770311594",
                "Nom communes équipées": "Neuilly-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        800: {
            "attributes": {
                "Identifiant station": "22008",
                "Nom station": "Charles de Gaulle - Graviers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:36+01:00",
                "Coordonnées géographiques": "48.8831517592,2.26488143206",
                "Nom communes équipées": "Neuilly-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        801: {
            "attributes": {
                "Identifiant station": "14101",
                "Nom station": "Gare Montparnasse - Edgar Quinet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:41+01:00",
                "Coordonnées géographiques": "48.8414878511,2.32332322828",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        802: {
            "attributes": {
                "Identifiant station": "5014",
                "Nom station": "Jean Calvin - Tournefort",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.8415885693,2.34848957509",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        803: {
            "attributes": {
                "Identifiant station": "21029",
                "Nom station": "Cours de l'Ile Seguin - Parc Billancourt",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:19+01:00",
                "Coordonnées géographiques": "48.8271405584,2.23423451185",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        804: {
            "attributes": {
                "Identifiant station": "8549",
                "Nom station": "George V - Christophe Colomb",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "12",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:43+01:00",
                "Coordonnées géographiques": "48.87046,2.30104",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        805: {
            "attributes": {
                "Identifiant station": "15072",
                "Nom station": "Place Fernand Forest",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:07+01:00",
                "Coordonnées géographiques": "48.8492691438,2.28186316788",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        806: {
            "attributes": {
                "Identifiant station": "13033",
                "Nom station": "Porte d'Italie",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "13",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:53+01:00",
                "Coordonnées géographiques": "48.8171619877,2.36023630947",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        807: {
            "attributes": {
                "Identifiant station": "21708",
                "Nom station": "Marcel Yol - Jullien",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:37+01:00",
                "Coordonnées géographiques": "48.825376118,2.28723501806",
                "Nom communes équipées": "Vanves",
                "Code INSEE communes équipées": "",
            }
        },
        808: {
            "attributes": {
                "Identifiant station": "20122",
                "Nom station": "Vitruve - Davout",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:28+01:00",
                "Coordonnées géographiques": "48.8605316115,2.40915928126",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        809: {
            "attributes": {
                "Identifiant station": "10017",
                "Nom station": "Gare de l'Est - Fidélité",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "15",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:43+01:00",
                "Coordonnées géographiques": "48.874575,2.356796",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        810: {
            "attributes": {
                "Identifiant station": "11038",
                "Nom station": "Temple - Charlot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:27+01:00",
                "Coordonnées géographiques": "48.8657093438,2.36527372507",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        811: {
            "attributes": {
                "Identifiant station": "41210",
                "Nom station": "Hôtel de ville de Fontenay-sous-Bois",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:55+01:00",
                "Coordonnées géographiques": "48.8494974911,2.47501641512",
                "Nom communes équipées": "Fontenay-sous-Bois",
                "Code INSEE communes équipées": "",
            }
        },
        812: {
            "attributes": {
                "Identifiant station": "4003",
                "Nom station": "Quai aux Fleurs - Pont Saint-Louis",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:36+01:00",
                "Coordonnées géographiques": "48.8529159562,2.35219571739",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        813: {
            "attributes": {
                "Identifiant station": "42304",
                "Nom station": "Marc Sangnier - Division Leclerc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "22",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.7859965231,2.3299555108",
                "Nom communes équipées": "Cachan",
                "Code INSEE communes équipées": "",
            }
        },
        814: {
            "attributes": {
                "Identifiant station": "12043",
                "Nom station": "Daumesnil - Hébrard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "31",
                "Vélos mécaniques disponibles": "28",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:52+01:00",
                "Coordonnées géographiques": "48.8429,2.38474",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        815: {
            "attributes": {
                "Identifiant station": "4111",
                "Nom station": "Hôtel de Ville - Pont d'Arcole",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-01-22T23:26:39+01:00",
                "Coordonnées géographiques": "48.8558764518,2.35089216381",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        816: {
            "attributes": {
                "Identifiant station": "8020",
                "Nom station": "Batignolles - Beudant",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "44",
                "Nombre bornettes libres": "38",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:22+01:00",
                "Coordonnées géographiques": "48.8821640147,2.31985297054",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        817: {
            "attributes": {
                "Identifiant station": "15052",
                "Nom station": "Convention - Vaugirard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "48",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "20",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.8376768223,2.29558692602",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        818: {
            "attributes": {
                "Identifiant station": "18027",
                "Nom station": "Damrémont - Ordener",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:21+01:00",
                "Coordonnées géographiques": "48.893400393,2.33626652391",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        819: {
            "attributes": {
                "Identifiant station": "16115",
                "Nom station": "Raffet - Montmorency",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "38",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:16+01:00",
                "Coordonnées géographiques": "48.8525867081,2.26292524562",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        820: {
            "attributes": {
                "Identifiant station": "15023",
                "Nom station": "La Motte-Picquet - Grenelle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:48+01:00",
                "Coordonnées géographiques": "48.8485832134,2.2993478179",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        821: {
            "attributes": {
                "Identifiant station": "15123",
                "Nom station": "Commerce - Letellier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "44",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "26",
                "Vélos mécaniques disponibles": "26",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:45+01:00",
                "Coordonnées géographiques": "48.8479766037,2.29662451966",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        822: {
            "attributes": {
                "Identifiant station": "13128",
                "Nom station": "Quai de la Gare - Pont de Bercy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "14",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:23+01:00",
                "Coordonnées géographiques": "48.8373078797,2.37408612606",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        823: {
            "attributes": {
                "Identifiant station": "21307",
                "Nom station": "Victor Cresson - Telles de la Poterie",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:11+01:00",
                "Coordonnées géographiques": "48.822698843,2.2686498275",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        824: {
            "attributes": {
                "Identifiant station": "23003",
                "Nom station": "Président Wilson - Baudin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "26",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "12",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:19+01:00",
                "Coordonnées géographiques": "48.898268871,2.28493362665",
                "Nom communes équipées": "Levallois-Perret",
                "Code INSEE communes équipées": "",
            }
        },
        825: {
            "attributes": {
                "Identifiant station": "15043",
                "Nom station": "Brancion - Lefebvre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "11",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:37+01:00",
                "Coordonnées géographiques": "48.8290022279,2.30143103816",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        826: {
            "attributes": {
                "Identifiant station": "4016",
                "Nom station": "Lobau - Hôtel de Ville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "60",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "39",
                "Vélos mécaniques disponibles": "35",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:33+01:00",
                "Coordonnées géographiques": "48.8562118,2.3532536",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        827: {
            "attributes": {
                "Identifiant station": "17039",
                "Nom station": "Place Saint-Ferdinand",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.8781552979,2.28851804786",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        828: {
            "attributes": {
                "Identifiant station": "11021",
                "Nom station": "Charonne - Pierre Bayle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:05+01:00",
                "Coordonnées géographiques": "48.8584450543,2.39039812239",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        829: {
            "attributes": {
                "Identifiant station": "5107",
                "Nom station": "Pontoise - La Tournelle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8504580092,2.35245427549",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        830: {
            "attributes": {
                "Identifiant station": "19108",
                "Nom station": "Gaston Tessier - Rosa Parks RER",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:30+01:00",
                "Coordonnées géographiques": "48.8963765,2.3743692",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        831: {
            "attributes": {
                "Identifiant station": "22604",
                "Nom station": "Hôtel de Ville de Fontenay-aux-Roses",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:16+01:00",
                "Coordonnées géographiques": "48.7908446035,2.28714559303",
                "Nom communes équipées": "Fontenay-aux-Roses",
                "Code INSEE communes équipées": "",
            }
        },
        832: {
            "attributes": {
                "Identifiant station": "42501",
                "Nom station": "Raspail - Albert Schweitzer",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "24",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:33+01:00",
                "Coordonnées géographiques": "48.81031545,2.34395871",
                "Nom communes équipées": "Gentilly",
                "Code INSEE communes équipées": "",
            }
        },
        833: {
            "attributes": {
                "Identifiant station": "19034",
                "Nom station": "Porte de Pantin - Petits Ponts",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "52",
                "Nombre bornettes libres": "48",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2019-06-06T09:56:04+02:00",
                "Coordonnées géographiques": "48.8904108624,2.39819392562",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        834: {
            "attributes": {
                "Identifiant station": "35001",
                "Nom station": "Jean Jaurès - Ouvrières Pivéreuses",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:19+01:00",
                "Coordonnées géographiques": "48.9104234375,2.39957660437",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        835: {
            "attributes": {
                "Identifiant station": "21213",
                "Nom station": "Vanne - Général de Gaulle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:28+01:00",
                "Coordonnées géographiques": "48.815007,2.328318",
                "Nom communes équipées": "Montrouge",
                "Code INSEE communes équipées": "",
            }
        },
        836: {
            "attributes": {
                "Identifiant station": "42209",
                "Nom station": "Place de l'Europe - Quai de Bercy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:15+01:00",
                "Coordonnées géographiques": "48.8252985884,2.39456791431",
                "Nom communes équipées": "Charenton-le-Pont",
                "Code INSEE communes équipées": "",
            }
        },
        837: {
            "attributes": {
                "Identifiant station": "16101",
                "Nom station": "Malakoff - Place de la Porte Maillot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:21+01:00",
                "Coordonnées géographiques": "48.8767334966,2.28345774114",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        838: {
            "attributes": {
                "Identifiant station": "22206",
                "Nom station": "Louis Pasteur - Albert Petit",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:36+01:00",
                "Coordonnées géographiques": "48.796261,2.317289",
                "Nom communes équipées": "Bagneux",
                "Code INSEE communes équipées": "",
            }
        },
        839: {
            "attributes": {
                "Identifiant station": "21702",
                "Nom station": "Gare SNCF Vanves-Malakoff",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "51",
                "Nombre bornettes libres": "51",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:06+01:00",
                "Coordonnées géographiques": "48.8190021638,2.2921500051",
                "Nom communes équipées": "Vanves",
                "Code INSEE communes équipées": "",
            }
        },
        840: {
            "attributes": {
                "Identifiant station": "13106",
                "Nom station": "Bobillot - Paulin Mery",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:56+01:00",
                "Coordonnées géographiques": "48.8298998417,2.35433567667",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        841: {
            "attributes": {
                "Identifiant station": "42305",
                "Nom station": "Vanne - Saussaies",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.786404,2.338848",
                "Nom communes équipées": "Cachan",
                "Code INSEE communes équipées": "",
            }
        },
        842: {
            "attributes": {
                "Identifiant station": "17003",
                "Nom station": "Jacques Kellner - Saint-Ouen",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "38",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:02+01:00",
                "Coordonnées géographiques": "48.8958094858,2.32789740916",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        843: {
            "attributes": {
                "Identifiant station": "42003",
                "Nom station": "Place de l'Insurrection d'Août 1944",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "26",
                "Vélos mécaniques disponibles": "22",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:37+01:00",
                "Coordonnées géographiques": "48.8147785275,2.39807456732",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        844: {
            "attributes": {
                "Identifiant station": "15064",
                "Nom station": "Javel - André Citroën",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "42",
                "Vélos mécaniques disponibles": "34",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:42+01:00",
                "Coordonnées géographiques": "48.8462764855,2.27863680571",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        845: {
            "attributes": {
                "Identifiant station": "31025",
                "Nom station": "Paul Signac - Aristide Briand",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:40+01:00",
                "Coordonnées géographiques": "48.8695026216,2.4502260983",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        846: {
            "attributes": {
                "Identifiant station": "15030",
                "Nom station": "Emeriau - Beaugrenelle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "74",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "70",
                "Vélos mécaniques disponibles": "64",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:30+01:00",
                "Coordonnées géographiques": "48.8479124971,2.28430598974",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        847: {
            "attributes": {
                "Identifiant station": "32011",
                "Nom station": "Stade de France",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "44",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:06+01:00",
                "Coordonnées géographiques": "48.9191419536,2.36206557602",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        848: {
            "attributes": {
                "Identifiant station": "16011",
                "Nom station": "Victor Hugo - La Pompe",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "41",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:01+01:00",
                "Coordonnées géographiques": "48.8679522247,2.28146318346",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        849: {
            "attributes": {
                "Identifiant station": "42701",
                "Nom station": "Charles Gide - Fort de Bicêtre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:22+01:00",
                "Coordonnées géographiques": "48.8071937957,2.35398441553",
                "Nom communes équipées": "Le Kremlin-Bicêtre",
                "Code INSEE communes équipées": "",
            }
        },
        850: {
            "attributes": {
                "Identifiant station": "12007",
                "Nom station": "Gare de Lyon - Chalon",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:13+01:00",
                "Coordonnées géographiques": "48.8456661942,2.37414523959",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        851: {
            "attributes": {
                "Identifiant station": "20108",
                "Nom station": "Jardin de l'Hospice Debrousse",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:29+01:00",
                "Coordonnées géographiques": "48.8614510619,2.40568600595",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        852: {
            "attributes": {
                "Identifiant station": "47001",
                "Nom station": "Place de la Gare",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "39",
                "Vélos mécaniques disponibles": "34",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:31+01:00",
                "Coordonnées géographiques": "48.8025335801,2.42750376463",
                "Nom communes équipées": "Maisons-Alfort",
                "Code INSEE communes équipées": "",
            }
        },
        853: {
            "attributes": {
                "Identifiant station": "16020",
                "Nom station": "Place de la Porte de Passy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:32+01:00",
                "Coordonnées géographiques": "48.8573410955,2.26441655388",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        854: {
            "attributes": {
                "Identifiant station": "41101",
                "Nom station": "Paul Doumer - Stalingrad",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:09+01:00",
                "Coordonnées géographiques": "48.8065189922,2.33688801527",
                "Nom communes équipées": "Arcueil",
                "Code INSEE communes équipées": "",
            }
        },
        855: {
            "attributes": {
                "Identifiant station": "13126",
                "Nom station": "Place Nationale - Château-des-Rentiers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:11+01:00",
                "Coordonnées géographiques": "48.829566,2.365053",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        856: {
            "attributes": {
                "Identifiant station": "27004",
                "Nom station": "Gare de la Garenne-Colombes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:53+01:00",
                "Coordonnées géographiques": "48.909752,2.239145",
                "Nom communes équipées": "Colombes",
                "Code INSEE communes équipées": "",
            }
        },
        857: {
            "attributes": {
                "Identifiant station": "17015",
                "Nom station": "Place de Lévis",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "48",
                "Nombre bornettes libres": "43",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:20+01:00",
                "Coordonnées géographiques": "48.8835468515,2.31319338083",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        858: {
            "attributes": {
                "Identifiant station": "18038",
                "Nom station": "Rond-Point de la Chapelle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "49",
                "Nombre bornettes libres": "42",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:45+01:00",
                "Coordonnées géographiques": "48.8951266093,2.35997110605",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        859: {
            "attributes": {
                "Identifiant station": "4001",
                "Nom station": "Arcole - Notre-Dame",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "25",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:50+01:00",
                "Coordonnées géographiques": "48.8538835413,2.3493797332",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        860: {
            "attributes": {
                "Identifiant station": "43010",
                "Nom station": "La Jarry - Place Diderot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8497269404,2.45201379061",
                "Nom communes équipées": "Vincennes",
                "Code INSEE communes équipées": "",
            }
        },
        861: {
            "attributes": {
                "Identifiant station": "14131",
                "Nom station": "Jourdan - Monticelli",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:30+01:00",
                "Coordonnées géographiques": "48.822032,2.329095",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        862: {
            "attributes": {
                "Identifiant station": "17107",
                "Nom station": "Berthier - Place Stuart Merril",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.8859279224,2.29334503412",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        863: {
            "attributes": {
                "Identifiant station": "17023",
                "Nom station": "Porte d'Asnières",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "33",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:06+01:00",
                "Coordonnées géographiques": "48.8903581761,2.30362862349",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        864: {
            "attributes": {
                "Identifiant station": "15004",
                "Nom station": "Vaugirard - Pasteur",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:33+01:00",
                "Coordonnées géographiques": "48.840408688,2.3154367134",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        865: {
            "attributes": {
                "Identifiant station": "44007",
                "Nom station": "Camille Groult - Guy Môquet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:18+01:00",
                "Coordonnées géographiques": "48.7874827465,2.39767257124",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        866: {
            "attributes": {
                "Identifiant station": "15057",
                "Nom station": "Lucien Bossoutrot - Général Martial Vallin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "29",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "12",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:27+01:00",
                "Coordonnées géographiques": "48.8369560463,2.27410722524",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        867: {
            "attributes": {
                "Identifiant station": "6008",
                "Nom station": "Assas - Luxembourg",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:46+01:00",
                "Coordonnées géographiques": "48.8437344688,2.33342838188",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        868: {
            "attributes": {
                "Identifiant station": "17108",
                "Nom station": "Bessières - Porte de Clichy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:36+01:00",
                "Coordonnées géographiques": "48.8961176245,2.31790903956",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        869: {
            "attributes": {
                "Identifiant station": "14010",
                "Nom station": "Jean Moulin - Place Victor et Hélène Basch",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "41",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8276491024,2.32619673014",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        870: {
            "attributes": {
                "Identifiant station": "4020",
                "Nom station": "Place Edmond Michelet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:22+01:00",
                "Coordonnées géographiques": "48.8601348905,2.3500636965",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        871: {
            "attributes": {
                "Identifiant station": "10105",
                "Nom station": "Mazagran - Bonne Nouvelle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:10+01:00",
                "Coordonnées géographiques": "48.8700471237,2.35112484545",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        872: {
            "attributes": {
                "Identifiant station": "15022",
                "Nom station": "Emile Zola - Fondary",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:51+01:00",
                "Coordonnées géographiques": "48.8470511431,2.29534059763",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        873: {
            "attributes": {
                "Identifiant station": "13109",
                "Nom station": "Brillat-Savarin - Küss",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:16+01:00",
                "Coordonnées géographiques": "48.822203,2.350422",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        874: {
            "attributes": {
                "Identifiant station": "9024",
                "Nom station": "Taitbout - La Fayette",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "53",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "47",
                "Vélos mécaniques disponibles": "36",
                "Vélos électriques disponibles": "11",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:13+01:00",
                "Coordonnées géographiques": "48.8733298892,2.33526259661",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        875: {
            "attributes": {
                "Identifiant station": "4002",
                "Nom station": "Marché aux fleurs",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:35+01:00",
                "Coordonnées géographiques": "48.8552581154,2.34737545252",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        876: {
            "attributes": {
                "Identifiant station": "16001",
                "Nom station": "Portugais - Kléber",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "12",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:25+01:00",
                "Coordonnées géographiques": "48.8712148,2.2937349",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        877: {
            "attributes": {
                "Identifiant station": "21204",
                "Nom station": "Estienne d'Orves - Henri Ginoux",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:15+01:00",
                "Coordonnées géographiques": "48.8153882204,2.32088679076",
                "Nom communes équipées": "Montrouge",
                "Code INSEE communes équipées": "",
            }
        },
        878: {
            "attributes": {
                "Identifiant station": "33015",
                "Nom station": "Charles Tillon - Cimetière Communal",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "11",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:45+01:00",
                "Coordonnées géographiques": "48.9184402364,2.39356309175",
                "Nom communes équipées": "Aubervilliers",
                "Code INSEE communes équipées": "",
            }
        },
        879: {
            "attributes": {
                "Identifiant station": "43001",
                "Nom station": "Paris - Laitières",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:06+01:00",
                "Coordonnées géographiques": "48.8463027415,2.42075532675",
                "Nom communes équipées": "Vincennes",
                "Code INSEE communes équipées": "",
            }
        },
        880: {
            "attributes": {
                "Identifiant station": "15311",
                "Nom station": "Quai du Président Roosevelt - Pont Aval",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:32+01:00",
                "Coordonnées géographiques": "48.8343883676,2.26568207145",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        881: {
            "attributes": {
                "Identifiant station": "21217",
                "Nom station": "Louis Rolland - République",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:19+01:00",
                "Coordonnées géographiques": "48.8171088974,2.3193106435",
                "Nom communes équipées": "Montrouge",
                "Code INSEE communes équipées": "",
            }
        },
        882: {
            "attributes": {
                "Identifiant station": "35006",
                "Nom station": "Delizy - Louis Nadot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:30+01:00",
                "Coordonnées géographiques": "48.8962128456,2.40945786238",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        883: {
            "attributes": {
                "Identifiant station": "9114",
                "Nom station": "Faubourg Poissonnière - Dunkerque",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:14+01:00",
                "Coordonnées géographiques": "48.8808834454,2.34993427992",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        884: {
            "attributes": {
                "Identifiant station": "15011",
                "Nom station": "Cambronne - Villa Croix Nivert",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "62",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "26",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.8458475614,2.30175744742",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        885: {
            "attributes": {
                "Identifiant station": "25001",
                "Nom station": "Colmar - Victor Hugo",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:29+01:00",
                "Coordonnées géographiques": "48.886522511,2.1739609167",
                "Nom communes équipées": "Rueil-Malmaison",
                "Code INSEE communes équipées": "",
            }
        },
        886: {
            "attributes": {
                "Identifiant station": "1021",
                "Nom station": "Halles - Bourdonnais",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:59+01:00",
                "Coordonnées géographiques": "48.8604489926,2.3460162431",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        887: {
            "attributes": {
                "Identifiant station": "35007",
                "Nom station": "Benjamin Delessert - Jean Lolive",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:17+01:00",
                "Coordonnées géographiques": "48.8939204362,2.41798192263",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        888: {
            "attributes": {
                "Identifiant station": "12018",
                "Nom station": "Saint-Mandé - Fabre d'Eglantine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:31+01:00",
                "Coordonnées géographiques": "48.8453579,2.3965022",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        889: {
            "attributes": {
                "Identifiant station": "44101",
                "Nom station": "Place Mongolfier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "35",
                "Vélos mécaniques disponibles": "31",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:55+01:00",
                "Coordonnées géographiques": "48.8151600301,2.45928794146",
                "Nom communes équipées": "Saint-Maurice",
                "Code INSEE communes équipées": "",
            }
        },
        890: {
            "attributes": {
                "Identifiant station": "17111",
                "Nom station": "Lemercier - Cardinet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:50+01:00",
                "Coordonnées géographiques": "48.8900971689,2.3173776269",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        891: {
            "attributes": {
                "Identifiant station": "31007",
                "Nom station": "Paris - Désiré Préaux",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "49",
                "Nombre bornettes libres": "44",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:07+01:00",
                "Coordonnées géographiques": "48.8574287669,2.43230491877",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        892: {
            "attributes": {
                "Identifiant station": "17119",
                "Nom station": "Toqueville - Cardinet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:03+01:00",
                "Coordonnées géographiques": "48.8849018711,2.31093931943",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        893: {
            "attributes": {
                "Identifiant station": "13032",
                "Nom station": "Italie - Jardin Joan Miro",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:23+01:00",
                "Coordonnées géographiques": "48.8199091405,2.3592948541",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        894: {
            "attributes": {
                "Identifiant station": "18041",
                "Nom station": "Rochechouart - Martyrs",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:08+01:00",
                "Coordonnées géographiques": "48.8821937796,2.34054956585",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        895: {
            "attributes": {
                "Identifiant station": "8037",
                "Nom station": "Monceau - Malesherbes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:34+01:00",
                "Coordonnées géographiques": "48.8795079,2.3145456",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        896: {
            "attributes": {
                "Identifiant station": "7017",
                "Nom station": "La Motte-Picquet - La Tour Maubourg",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "35",
                "Vélos mécaniques disponibles": "32",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:41+01:00",
                "Coordonnées géographiques": "48.8572316,2.3094136",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        897: {
            "attributes": {
                "Identifiant station": "22303",
                "Nom station": "Lucien Lanternier - Le Luth",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:44+01:00",
                "Coordonnées géographiques": "48.9315943056,2.28897839785",
                "Nom communes équipées": "Gennevilliers",
                "Code INSEE communes équipées": "",
            }
        },
        898: {
            "attributes": {
                "Identifiant station": "15071",
                "Nom station": "Suffren - Champ de Mars",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "19",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:15+01:00",
                "Coordonnées géographiques": "48.8547194,2.2952168",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        899: {
            "attributes": {
                "Identifiant station": "92001",
                "Nom station": "Mairie de quartier du Plateau - Mont-Valérien",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:27+01:00",
                "Coordonnées géographiques": "48.8799933,2.2072437",
                "Nom communes équipées": "Nanterre",
                "Code INSEE communes équipées": "",
            }
        },
        900: {
            "attributes": {
                "Identifiant station": "9004",
                "Nom station": "Gérando - Rochechouart",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:41+01:00",
                "Coordonnées géographiques": "48.8829434084,2.34648462385",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        901: {
            "attributes": {
                "Identifiant station": "23004",
                "Nom station": "Paul Vaillant-Couturier - Victor Hugo",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:19+01:00",
                "Coordonnées géographiques": "48.8981970226,2.29384801797",
                "Nom communes équipées": "Levallois-Perret",
                "Code INSEE communes équipées": "",
            }
        },
        902: {
            "attributes": {
                "Identifiant station": "4021",
                "Nom station": "Centre Georges Pompidou",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:55+01:00",
                "Coordonnées géographiques": "48.861419,2.352581",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        903: {
            "attributes": {
                "Identifiant station": "24003",
                "Nom station": "Place Charles de Gaulle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-09-17T01:16:54+02:00",
                "Coordonnées géographiques": "48.8966431015,2.25257009268",
                "Nom communes équipées": "Courbevoie",
                "Code INSEE communes équipées": "",
            }
        },
        904: {
            "attributes": {
                "Identifiant station": "15102",
                "Nom station": "Square Bela Bartok",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:27+01:00",
                "Coordonnées géographiques": "48.8512015683,2.28465367109",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        905: {
            "attributes": {
                "Identifiant station": "16002",
                "Nom station": "Pergolèse - Malakoff",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:24+01:00",
                "Coordonnées géographiques": "48.8752108878,2.28427816182",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        906: {
            "attributes": {
                "Identifiant station": "12026",
                "Nom station": "Place Ginette Hamelin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "51",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "42",
                "Vélos mécaniques disponibles": "37",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:04+01:00",
                "Coordonnées géographiques": "48.837853,2.382064",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        907: {
            "attributes": {
                "Identifiant station": "23010",
                "Nom station": "Voltaire - Anatole France",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8919993603,2.28426273912",
                "Nom communes équipées": "Levallois-Perret",
                "Code INSEE communes équipées": "",
            }
        },
        908: {
            "attributes": {
                "Identifiant station": "13009",
                "Nom station": "Auguste Blanqui - Corvisart",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:38+01:00",
                "Coordonnées géographiques": "48.8300989,2.3502726",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        909: {
            "attributes": {
                "Identifiant station": "18007",
                "Nom station": "Goutte d'or - Barbès",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:15+01:00",
                "Coordonnées géographiques": "48.8851243236,2.34969623387",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        910: {
            "attributes": {
                "Identifiant station": "21501",
                "Nom station": "Pont de Suresnes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "52",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:35+01:00",
                "Coordonnées géographiques": "48.8674150923,2.22589466796",
                "Nom communes équipées": "Suresnes",
                "Code INSEE communes équipées": "",
            }
        },
        911: {
            "attributes": {
                "Identifiant station": "15005",
                "Nom station": "Place Jacques et Thérèse Tréfouël",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "38",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:12+01:00",
                "Coordonnées géographiques": "48.8427264983,2.31256574392",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        912: {
            "attributes": {
                "Identifiant station": "9102",
                "Nom station": "Londres - Place d'Estienne d'Orves",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "24",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:39+01:00",
                "Coordonnées géographiques": "48.8766938058,2.33053620905",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        913: {
            "attributes": {
                "Identifiant station": "19128",
                "Nom station": "Quai de l'Oise - Aisne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:15+01:00",
                "Coordonnées géographiques": "48.8906231,2.3833701",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        914: {
            "attributes": {
                "Identifiant station": "12038",
                "Nom station": "Picpus - Louis Braille",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "44",
                "Nombre bornettes libres": "37",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:23+01:00",
                "Coordonnées géographiques": "48.8399887889,2.40052566855",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        915: {
            "attributes": {
                "Identifiant station": "22408",
                "Nom station": "Place du 14 Juillet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:25+01:00",
                "Coordonnées géographiques": "48.8192669595,2.30119988322",
                "Nom communes équipées": "Malakoff",
                "Code INSEE communes équipées": "",
            }
        },
        916: {
            "attributes": {
                "Identifiant station": "18030",
                "Nom station": "Joseph Dijon - Ornano",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:42+01:00",
                "Coordonnées géographiques": "48.8937445335,2.3476678133",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        917: {
            "attributes": {
                "Identifiant station": "11017",
                "Nom station": "Charonne - Bureau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "47",
                "Nombre bornettes libres": "42",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:03+01:00",
                "Coordonnées géographiques": "48.854176,2.396093",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        918: {
            "attributes": {
                "Identifiant station": "5024",
                "Nom station": "Place Monge",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:24+01:00",
                "Coordonnées géographiques": "48.8427618031,2.35240794718",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        919: {
            "attributes": {
                "Identifiant station": "18006",
                "Nom station": "Square Louise Michel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:17+01:00",
                "Coordonnées géographiques": "48.8846873545,2.34413869679",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        920: {
            "attributes": {
                "Identifiant station": "14137",
                "Nom station": "Maurice d'Ocagne - General Maudhuy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:55+01:00",
                "Coordonnées géographiques": "48.8241428252,2.31587894261",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        921: {
            "attributes": {
                "Identifiant station": "6027",
                "Nom station": "Blaise-Desgoffe - Vaugirard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:47+01:00",
                "Coordonnées géographiques": "48.8461288777,2.32426721603",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        922: {
            "attributes": {
                "Identifiant station": "12155",
                "Nom station": "Rottembourg - Général Michel Bizot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:44+01:00",
                "Coordonnées géographiques": "48.8378782599,2.40440525115",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        923: {
            "attributes": {
                "Identifiant station": "21401",
                "Nom station": "Mairie de Clamart",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:36+01:00",
                "Coordonnées géographiques": "48.800612,2.262151",
                "Nom communes équipées": "Clamart",
                "Code INSEE communes équipées": "",
            }
        },
        924: {
            "attributes": {
                "Identifiant station": "12028",
                "Nom station": "Vivaldi - Antoine-Julien Hénard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:39+01:00",
                "Coordonnées géographiques": "48.8419236633,2.38972897947",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        925: {
            "attributes": {
                "Identifiant station": "13044",
                "Nom station": "Place Jeanne d'Arc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:32+01:00",
                "Coordonnées géographiques": "48.829575,2.369126",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        926: {
            "attributes": {
                "Identifiant station": "9034",
                "Nom station": "Godot de Mauroy - Madeleine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:50+01:00",
                "Coordonnées géographiques": "48.869789,2.326347",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        927: {
            "attributes": {
                "Identifiant station": "9027",
                "Nom station": "Douai - Pierre Fontaine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:20+01:00",
                "Coordonnées géographiques": "48.8822910115,2.33324121684",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        928: {
            "attributes": {
                "Identifiant station": "12035",
                "Nom station": "Madagascar - Meuniers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:48+01:00",
                "Coordonnées géographiques": "48.8346046447,2.3970650509",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        929: {
            "attributes": {
                "Identifiant station": "41205",
                "Nom station": "Maréchal Joffre - Verdun",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:41:58+01:00",
                "Coordonnées géographiques": "48.85385,2.470307",
                "Nom communes équipées": "Fontenay-sous-Bois",
                "Code INSEE communes équipées": "",
            }
        },
        930: {
            "attributes": {
                "Identifiant station": "1003",
                "Nom station": "Saint-Denis - Rivoli",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "27",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:19+01:00",
                "Coordonnées géographiques": "48.8590593408,2.34769430012",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        931: {
            "attributes": {
                "Identifiant station": "22015",
                "Nom station": "Île de la Jatte",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:08+01:00",
                "Coordonnées géographiques": "48.8940361613,2.26663392037",
                "Nom communes équipées": "Neuilly-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        932: {
            "attributes": {
                "Identifiant station": "41211",
                "Nom station": "Louis Xavier de Ricard - Ancienne Mairie",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "33",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:14+01:00",
                "Coordonnées géographiques": "48.8462989907,2.47242037207",
                "Nom communes équipées": "Fontenay-sous-Bois",
                "Code INSEE communes équipées": "",
            }
        },
        933: {
            "attributes": {
                "Identifiant station": "44016",
                "Nom station": "Balzac - Olympes de Gouges",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:33+01:00",
                "Coordonnées géographiques": "48.7785033249,2.400678657",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        934: {
            "attributes": {
                "Identifiant station": "44011",
                "Nom station": "Lebrun - Colonel Fabien",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:01+01:00",
                "Coordonnées géographiques": "48.7826687073,2.38131780177",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        935: {
            "attributes": {
                "Identifiant station": "44005",
                "Nom station": "Place Saint-Just",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:56+01:00",
                "Coordonnées géographiques": "48.7903795187,2.38952267915",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        936: {
            "attributes": {
                "Identifiant station": "4019",
                "Nom station": "Cloître Saint-Merri - Renard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "20",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:13+01:00",
                "Coordonnées géographiques": "48.8588354489,2.35182791948",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        937: {
            "attributes": {
                "Identifiant station": "51002",
                "Nom station": "Placette Grégoire Collas",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.94342014,2.24874760956",
                "Nom communes équipées": "Argenteuil",
                "Code INSEE communes équipées": "",
            }
        },
        938: {
            "attributes": {
                "Identifiant station": "8116",
                "Nom station": "Champs-Elysees - Bassano",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-02-21T20:09:18+01:00",
                "Coordonnées géographiques": "48.8718118078,2.30023898184",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        939: {
            "attributes": {
                "Identifiant station": "32601",
                "Nom station": "Porte des Lilas - Faidherbe",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "35",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:45+01:00",
                "Coordonnées géographiques": "48.8786185567,2.41203278303",
                "Nom communes équipées": "Les Lilas",
                "Code INSEE communes équipées": "",
            }
        },
        940: {
            "attributes": {
                "Identifiant station": "20015",
                "Nom station": "Bagnolet - Orteaux",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "33",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:07+01:00",
                "Coordonnées géographiques": "48.8570864103,2.39841252565",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        941: {
            "attributes": {
                "Identifiant station": "33016",
                "Nom station": "Fort d'Aubervilliers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:32+01:00",
                "Coordonnées géographiques": "48.9150238237,2.4035281688",
                "Nom communes équipées": "Aubervilliers",
                "Code INSEE communes équipées": "",
            }
        },
        942: {
            "attributes": {
                "Identifiant station": "51001",
                "Nom station": "Gare d'Argenteuil",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:38+01:00",
                "Coordonnées géographiques": "48.9470224368,2.2568981722",
                "Nom communes équipées": "Argenteuil",
                "Code INSEE communes équipées": "",
            }
        },
        943: {
            "attributes": {
                "Identifiant station": "6012",
                "Nom station": "Saint-Germain - Ciseaux",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:58+01:00",
                "Coordonnées géographiques": "48.8533052776,2.33446546188",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        944: {
            "attributes": {
                "Identifiant station": "8027",
                "Nom station": "Mairie du 8ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "14",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:06+01:00",
                "Coordonnées géographiques": "48.877932,2.318264",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        945: {
            "attributes": {
                "Identifiant station": "12005",
                "Nom station": "Crozatier - Faubourg Saint-Antoine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "24",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:16+01:00",
                "Coordonnées géographiques": "48.8504400134,2.37890136621",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        946: {
            "attributes": {
                "Identifiant station": "2112",
                "Nom station": "Bibliothèque Nationale",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:24+01:00",
                "Coordonnées géographiques": "48.8682889935,2.33808930963",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        947: {
            "attributes": {
                "Identifiant station": "27005",
                "Nom station": "Gare du Stade",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:56+01:00",
                "Coordonnées géographiques": "48.9312676,2.2603778",
                "Nom communes équipées": "Colombes",
                "Code INSEE communes équipées": "",
            }
        },
        948: {
            "attributes": {
                "Identifiant station": "41403",
                "Nom station": "Gare RER - Malleret-Joinville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:42+01:00",
                "Coordonnées géographiques": "48.802936,2.42531",
                "Nom communes équipées": "Alfortville",
                "Code INSEE communes équipées": "",
            }
        },
        949: {
            "attributes": {
                "Identifiant station": "17123",
                "Nom station": "Mstislav Rostropovitch - Berthier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:10+01:00",
                "Coordonnées géographiques": "48.8921208212,2.30923410505",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        950: {
            "attributes": {
                "Identifiant station": "21303",
                "Nom station": "Jean Monnet - Iles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "23",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:51+01:00",
                "Coordonnées géographiques": "48.8233939841,2.24970731949",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        951: {
            "attributes": {
                "Identifiant station": "21319",
                "Nom station": "Esplanade du Foncet - Camille Desmoulins",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:45+01:00",
                "Coordonnées géographiques": "48.8270902335,2.2606857866",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        952: {
            "attributes": {
                "Identifiant station": "33014",
                "Nom station": "Parc Eli Lotar",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:53+01:00",
                "Coordonnées géographiques": "48.9184499305,2.37270858139",
                "Nom communes équipées": "Aubervilliers",
                "Code INSEE communes équipées": "",
            }
        },
        953: {
            "attributes": {
                "Identifiant station": "18044",
                "Nom station": "Clichy - Douai",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:44+01:00",
                "Coordonnées géographiques": "48.8841183189,2.32872873545",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        954: {
            "attributes": {
                "Identifiant station": "13052",
                "Nom station": "Leredde - Tolbiac",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "44",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "37",
                "Vélos mécaniques disponibles": "30",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.8288762966,2.37443827093",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        955: {
            "attributes": {
                "Identifiant station": "9006",
                "Nom station": "Dunkerque - Trudaine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:28+01:00",
                "Coordonnées géographiques": "48.8821516677,2.34632536769",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        956: {
            "attributes": {
                "Identifiant station": "15127",
                "Nom station": "Place Charles Michels",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "32",
                "Vélos mécaniques disponibles": "30",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:05+01:00",
                "Coordonnées géographiques": "48.846401,2.28502",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        957: {
            "attributes": {
                "Identifiant station": "21314",
                "Nom station": "Chemin de Bretagne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "27",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:47+01:00",
                "Coordonnées géographiques": "48.825147,2.25936",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        958: {
            "attributes": {
                "Identifiant station": "11011",
                "Nom station": "Richard Lenoir - Voltaire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:08+01:00",
                "Coordonnées géographiques": "48.8616427388,2.3727491498",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        959: {
            "attributes": {
                "Identifiant station": "92006",
                "Nom station": "Place Nelson Mandela",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:08+01:00",
                "Coordonnées géographiques": "48.8978743182,2.21828207374",
                "Nom communes équipées": "Nanterre",
                "Code INSEE communes équipées": "",
            }
        },
        960: {
            "attributes": {
                "Identifiant station": "15036",
                "Nom station": "Croix Nivert - Square Saint-Lambert",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "47",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "32",
                "Vélos mécaniques disponibles": "31",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:24+01:00",
                "Coordonnées géographiques": "48.8430515216,2.29520045221",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        961: {
            "attributes": {
                "Identifiant station": "31005",
                "Nom station": "Square Denise Buisson",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "41",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:48+01:00",
                "Coordonnées géographiques": "48.8562322731,2.42643624544",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        962: {
            "attributes": {
                "Identifiant station": "20105",
                "Nom station": "Benoît Frachon - République",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:55+01:00",
                "Coordonnées géographiques": "48.8523072901,2.41560548544",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        963: {
            "attributes": {
                "Identifiant station": "13516",
                "Nom station": "Glacière - Soeur Catherine Marie",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "38",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:55+01:00",
                "Coordonnées géographiques": "48.8295229894,2.34357040375",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        964: {
            "attributes": {
                "Identifiant station": "15018",
                "Nom station": "Dutot - Place Alleray",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "56",
                "Nombre bornettes libres": "52",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:40+01:00",
                "Coordonnées géographiques": "48.8368260918,2.30717010956",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        965: {
            "attributes": {
                "Identifiant station": "2108",
                "Nom station": "Vivienne - Montmartre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "36",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "21",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:07+01:00",
                "Coordonnées géographiques": "48.8713475974,2.34145883471",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        966: {
            "attributes": {
                "Identifiant station": "6002",
                "Nom station": "Saint-Benoît - Jacob",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:02+01:00",
                "Coordonnées géographiques": "48.8554513601,2.33347222209",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        967: {
            "attributes": {
                "Identifiant station": "9011",
                "Nom station": "Geoffroy - Marie",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:24+01:00",
                "Coordonnées géographiques": "48.8737819406,2.34451387634",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        968: {
            "attributes": {
                "Identifiant station": "18039",
                "Nom station": "Jardins d'Eole",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:20+01:00",
                "Coordonnées géographiques": "48.8869703,2.3668488",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        969: {
            "attributes": {
                "Identifiant station": "7109",
                "Nom station": "Place Joffre - Frédéric le Play",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:19+01:00",
                "Coordonnées géographiques": "48.8537544869,2.30442423373",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        970: {
            "attributes": {
                "Identifiant station": "14031",
                "Nom station": "Didot - Place de Moro-Giafferi",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:33+01:00",
                "Coordonnées géographiques": "48.8337,2.3215",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        971: {
            "attributes": {
                "Identifiant station": "13025",
                "Nom station": "Place de Rungis",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.8224559476,2.34738272948",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        972: {
            "attributes": {
                "Identifiant station": "20044",
                "Nom station": "Davout - Volga",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:55+01:00",
                "Coordonnées géographiques": "48.85291968,2.41052290978",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        973: {
            "attributes": {
                "Identifiant station": "5115",
                "Nom station": "Buffon - Jardin des Plantes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:35+01:00",
                "Coordonnées géographiques": "48.8426684661,2.36185871065",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        974: {
            "attributes": {
                "Identifiant station": "16019",
                "Nom station": "Porte de la Muette",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:27+01:00",
                "Coordonnées géographiques": "48.8630231784,2.26871881702",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        975: {
            "attributes": {
                "Identifiant station": "10021",
                "Nom station": "Marché Saint-Quentin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "15",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:15+01:00",
                "Coordonnées géographiques": "48.8773560577,2.35451144564",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        976: {
            "attributes": {
                "Identifiant station": "33012",
                "Nom station": "Landy - Heurtault",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:08+01:00",
                "Coordonnées géographiques": "48.9150965345,2.37595137209",
                "Nom communes équipées": "Aubervilliers",
                "Code INSEE communes équipées": "",
            }
        },
        977: {
            "attributes": {
                "Identifiant station": "4010",
                "Nom station": "Saint-Antoine Sévigné",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8550222,2.3612322",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        978: {
            "attributes": {
                "Identifiant station": "10019",
                "Nom station": "Paradis - Hauteville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:56+01:00",
                "Coordonnées géographiques": "48.8749999547,2.35227574019",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        979: {
            "attributes": {
                "Identifiant station": "21707",
                "Nom station": "Square Etienne Jarousse",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:54+01:00",
                "Coordonnées géographiques": "48.824851826,2.29259334505",
                "Nom communes équipées": "Vanves",
                "Code INSEE communes équipées": "",
            }
        },
        980: {
            "attributes": {
                "Identifiant station": "13018",
                "Nom station": "Bruant - Vincent Auriol",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8348484529,2.36682649963",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        981: {
            "attributes": {
                "Identifiant station": "15129",
                "Nom station": "Hôpital Necker",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "29",
                "Vélos mécaniques disponibles": "23",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:29+01:00",
                "Coordonnées géographiques": "48.8460434901,2.31324501336",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        982: {
            "attributes": {
                "Identifiant station": "14032",
                "Nom station": "Mairie du 14ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:14+01:00",
                "Coordonnées géographiques": "48.8325457812,2.32549231499",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        983: {
            "attributes": {
                "Identifiant station": "3102",
                "Nom station": "Bourg l'Abbé - Saint-Martin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "26",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:07+01:00",
                "Coordonnées géographiques": "48.8639333195,2.3524062708",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        984: {
            "attributes": {
                "Identifiant station": "14011",
                "Nom station": "Place des Droits de l'Enfant",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:20+01:00",
                "Coordonnées géographiques": "48.8274804,2.3319217",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        985: {
            "attributes": {
                "Identifiant station": "15039",
                "Nom station": "Abbé Groult - Convention",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "12",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:48+01:00",
                "Coordonnées géographiques": "48.8359692787,2.30206403951",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        986: {
            "attributes": {
                "Identifiant station": "11033",
                "Nom station": "Bréguet - Sabin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:09+01:00",
                "Coordonnées géographiques": "48.8569316783,2.37067494621",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        987: {
            "attributes": {
                "Identifiant station": "21315",
                "Nom station": "Jules Guesdes - Général de Gaulle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:26+01:00",
                "Coordonnées géographiques": "48.8193058129,2.27893419564",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        988: {
            "attributes": {
                "Identifiant station": "16005",
                "Nom station": "Raymond Poincaré - Place Victor Hugo",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:40+01:00",
                "Coordonnées géographiques": "48.8703061244,2.28507454142",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        989: {
            "attributes": {
                "Identifiant station": "6022",
                "Nom station": "Marché Saint-Germain",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.8516562586,2.33550030738",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        990: {
            "attributes": {
                "Identifiant station": "15008",
                "Nom station": "Sèvres - Lecourbe",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "62",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "23",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:52+01:00",
                "Coordonnées géographiques": "48.8447677,2.3110086",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        991: {
            "attributes": {
                "Identifiant station": "15006",
                "Nom station": "Cherche-Midi - Hôpital Necker",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:27+01:00",
                "Coordonnées géographiques": "48.8446106331,2.3179140687",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        992: {
            "attributes": {
                "Identifiant station": "5007",
                "Nom station": "Ecoles - Carmes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "23",
                "Vélos électriques disponibles": "11",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:50+01:00",
                "Coordonnées géographiques": "48.848904448,2.34734863043",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        993: {
            "attributes": {
                "Identifiant station": "12101",
                "Nom station": "Charenton - Prague",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:46+01:00",
                "Coordonnées géographiques": "48.84817513,2.37635844943",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        994: {
            "attributes": {
                "Identifiant station": "4006",
                "Nom station": "Bourdon - Pont Morland",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-08-29T09:39:32+02:00",
                "Coordonnées géographiques": "48.8478103,2.3659356",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        995: {
            "attributes": {
                "Identifiant station": "21402",
                "Nom station": "Place Aimé Césaire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:52+01:00",
                "Coordonnées géographiques": "48.7831947131,2.24291279912",
                "Nom communes équipées": "Clamart",
                "Code INSEE communes équipées": "",
            }
        },
        996: {
            "attributes": {
                "Identifiant station": "12015",
                "Nom station": "Place de la Nation - Picpus",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:40+01:00",
                "Coordonnées géographiques": "48.8468216815,2.40011304617",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        997: {
            "attributes": {
                "Identifiant station": "9007",
                "Nom station": "Condorcet - Turgot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:11+01:00",
                "Coordonnées géographiques": "48.8798515579,2.34523236752",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        998: {
            "attributes": {
                "Identifiant station": "1020",
                "Nom station": "Cambon - Rivoli",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "33",
                "Vélos mécaniques disponibles": "22",
                "Vélos électriques disponibles": "11",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:31+01:00",
                "Coordonnées géographiques": "48.866143391,2.32518006471",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        999: {
            "attributes": {
                "Identifiant station": "23102",
                "Nom station": "Hôtel de Ville de Sèvres",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:26+01:00",
                "Coordonnées géographiques": "48.8243635613,2.21310473979",
                "Nom communes équipées": "Sèvres",
                "Code INSEE communes équipées": "",
            }
        },
        1000: {
            "attributes": {
                "Identifiant station": "15001",
                "Nom station": "Gare Montparnasse - Arrivée",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.8438279943,2.32258111238",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1001: {
            "attributes": {
                "Identifiant station": "11001",
                "Nom station": "Richard Lenoir - Place de la Bastille",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "53",
                "Nombre bornettes libres": "33",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:55+01:00",
                "Coordonnées géographiques": "48.8538327,2.3697222",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1002: {
            "attributes": {
                "Identifiant station": "6010",
                "Nom station": "Chevreuse - Montparnasse",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "15",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.8417391612,2.33155221674",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1003: {
            "attributes": {
                "Identifiant station": "17024",
                "Nom station": "Villiers - Batignolles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:50+01:00",
                "Coordonnées géographiques": "48.8812525385,2.31653843075",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1004: {
            "attributes": {
                "Identifiant station": "10032",
                "Nom station": "Claude Vellefaux - Hôpital Saint-Louis",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:36+01:00",
                "Coordonnées géographiques": "48.872859139,2.37007170699",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1005: {
            "attributes": {
                "Identifiant station": "33103",
                "Nom station": "Jean Jaurès - Jules Jacquemin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:54+01:00",
                "Coordonnées géographiques": "48.884330536,2.40829024552",
                "Nom communes équipées": "Le Pré-Saint-Gervais",
                "Code INSEE communes équipées": "",
            }
        },
        1006: {
            "attributes": {
                "Identifiant station": "45002",
                "Nom station": "Roger Salengro - Général de Gaulle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.8200290104,2.48467605561",
                "Nom communes équipées": "Champigny-sur-Marne",
                "Code INSEE communes équipées": "",
            }
        },
        1007: {
            "attributes": {
                "Identifiant station": "41401",
                "Nom station": "Marne - Charles de Gaulle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:06+01:00",
                "Coordonnées géographiques": "48.8150717195,2.41216689348",
                "Nom communes équipées": "Alfortville",
                "Code INSEE communes équipées": "",
            }
        },
        1008: {
            "attributes": {
                "Identifiant station": "16042",
                "Nom station": "Murat - Porte de Saint-Cloud",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:40+01:00",
                "Coordonnées géographiques": "48.8375086262,2.25776519626",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1009: {
            "attributes": {
                "Identifiant station": "41203",
                "Nom station": "Stalingrad - Rigollots",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:35+01:00",
                "Coordonnées géographiques": "48.8499740384,2.45557576418",
                "Nom communes équipées": "Fontenay-sous-Bois",
                "Code INSEE communes équipées": "",
            }
        },
        1010: {
            "attributes": {
                "Identifiant station": "17019",
                "Nom station": "Phalsbourg - Villiers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:41+01:00",
                "Coordonnées géographiques": "48.882423079,2.30904534459",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1011: {
            "attributes": {
                "Identifiant station": "17009",
                "Nom station": "Paul Bodin - Clichy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:23+01:00",
                "Coordonnées géographiques": "48.8923961462,2.31714259833",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1012: {
            "attributes": {
                "Identifiant station": "16037",
                "Nom station": "Molitor - Michel-Ange",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "52",
                "Nombre bornettes libres": "33",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8451820815,2.26211277357",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1013: {
            "attributes": {
                "Identifiant station": "20022",
                "Nom station": "Porte de Bagnolet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.8647071391,2.40834463735",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1014: {
            "attributes": {
                "Identifiant station": "47003",
                "Nom station": "Charentonneau - Maisons Alfort",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:21+01:00",
                "Coordonnées géographiques": "48.8099329805,2.4432884565",
                "Nom communes équipées": "Maisons-Alfort",
                "Code INSEE communes équipées": "",
            }
        },
        1015: {
            "attributes": {
                "Identifiant station": "18026",
                "Nom station": "Ruisseau - Ordener",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:44+01:00",
                "Coordonnées géographiques": "48.8929946257,2.34014488757",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1016: {
            "attributes": {
                "Identifiant station": "20034",
                "Nom station": "Sorbier - Ménilmontant",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:40+01:00",
                "Coordonnées géographiques": "48.8685423957,2.38977212459",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1017: {
            "attributes": {
                "Identifiant station": "14026",
                "Nom station": "Plantes - Moulin Vert",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "43",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:08+01:00",
                "Coordonnées géographiques": "48.8300575531,2.32323020697",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1018: {
            "attributes": {
                "Identifiant station": "7012",
                "Nom station": "Cité Vaneau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:10+01:00",
                "Coordonnées géographiques": "48.8542210666,2.31961492449",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1019: {
            "attributes": {
                "Identifiant station": "16105",
                "Nom station": "Crevaux - Bugeaud",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:53+01:00",
                "Coordonnées géographiques": "48.8708094287,2.28114900769",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1020: {
            "attributes": {
                "Identifiant station": "10038",
                "Nom station": "Place du Colonel Fabien",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:09+01:00",
                "Coordonnées géographiques": "48.8775683209,2.36987918615",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1021: {
            "attributes": {
                "Identifiant station": "16012",
                "Nom station": "Flandrin - Longchamp",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "31",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "20",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:52+01:00",
                "Coordonnées géographiques": "48.8687784918,2.27435453918",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1022: {
            "attributes": {
                "Identifiant station": "16102",
                "Nom station": "Pergolèse - Marbeau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:09+01:00",
                "Coordonnées géographiques": "48.8735687069,2.28151252253",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1023: {
            "attributes": {
                "Identifiant station": "16036",
                "Nom station": "Porte Molitor",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "60",
                "Nombre bornettes libres": "38",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "20",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:44+01:00",
                "Coordonnées géographiques": "48.8452476351,2.25690152496",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1024: {
            "attributes": {
                "Identifiant station": "14135",
                "Nom station": "Jourdan - Cité Universitaire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "39",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:07+01:00",
                "Coordonnées géographiques": "48.8198973432,2.33970079641",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1025: {
            "attributes": {
                "Identifiant station": "42505",
                "Nom station": "Charles Frérot - Albert Guilpin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:32+01:00",
                "Coordonnées géographiques": "48.8155839184,2.35050290823",
                "Nom communes équipées": "Gentilly",
                "Code INSEE communes équipées": "",
            }
        },
        1026: {
            "attributes": {
                "Identifiant station": "21309",
                "Nom station": "Mairie d'Issy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "41",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:47+01:00",
                "Coordonnées géographiques": "48.8237677925,2.27231115103",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        1027: {
            "attributes": {
                "Identifiant station": "8117",
                "Nom station": "Place de la Madeleine - Royale",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "18",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:34+01:00",
                "Coordonnées géographiques": "48.8692280526,2.32442110777",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1028: {
            "attributes": {
                "Identifiant station": "20024",
                "Nom station": "Gambetta - Père Lachaise",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:01+01:00",
                "Coordonnées géographiques": "48.8642956962,2.39822866728",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1029: {
            "attributes": {
                "Identifiant station": "12036",
                "Nom station": "Reuilly - Place Felix Eboue",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "40",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:31+01:00",
                "Coordonnées géographiques": "48.840302745,2.39463039872",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1030: {
            "attributes": {
                "Identifiant station": "12156",
                "Nom station": "Niger - Soult",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:54+01:00",
                "Coordonnées géographiques": "48.8436990637,2.41036123722",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1031: {
            "attributes": {
                "Identifiant station": "26004",
                "Nom station": "Gare RER les Grésillons",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:20+01:00",
                "Coordonnées géographiques": "48.9198986336,2.31458459136",
                "Nom communes équipées": "Asnières-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1032: {
            "attributes": {
                "Identifiant station": "4108",
                "Nom station": "Quai de Béthune - Pont de la Tournelle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:39+01:00",
                "Coordonnées géographiques": "48.8509743319,2.35630720854",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1033: {
            "attributes": {
                "Identifiant station": "18139",
                "Nom station": "Poissonniers - Championnet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:59+01:00",
                "Coordonnées géographiques": "48.8954329973,2.35301781446",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1034: {
            "attributes": {
                "Identifiant station": "19002",
                "Nom station": "Centquatre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "40",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:07+01:00",
                "Coordonnées géographiques": "48.8893335385,2.37078543752",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1035: {
            "attributes": {
                "Identifiant station": "1006",
                "Nom station": "Grande Truanderie - Saint-Denis",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "29",
                "Vélos mécaniques disponibles": "22",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:01+01:00",
                "Coordonnées géographiques": "48.8626302,2.3498075",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1036: {
            "attributes": {
                "Identifiant station": "9009",
                "Nom station": "Square Montholon",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:44+01:00",
                "Coordonnées géographiques": "48.8766203775,2.34576411545",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1037: {
            "attributes": {
                "Identifiant station": "11112",
                "Nom station": "Jules Vallès - Charonne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:38+01:00",
                "Coordonnées géographiques": "48.8542843799,2.38476745784",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1038: {
            "attributes": {
                "Identifiant station": "14103",
                "Nom station": "Maine - Liancourt",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:41+01:00",
                "Coordonnées géographiques": "48.835064364,2.32373144997",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1039: {
            "attributes": {
                "Identifiant station": "19110",
                "Nom station": "Square Claude Bernard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8987601622,2.37532574683",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1040: {
            "attributes": {
                "Identifiant station": "3010",
                "Nom station": "Beaubourg - Michel-le-Comte",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:02+01:00",
                "Coordonnées géographiques": "48.8625349418,2.35431030393",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1041: {
            "attributes": {
                "Identifiant station": "22201",
                "Nom station": "Henri Ravera - Gabriel Péri",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:44+01:00",
                "Coordonnées géographiques": "48.7972227383,2.3021158576",
                "Nom communes équipées": "Bagneux",
                "Code INSEE communes équipées": "",
            }
        },
        1042: {
            "attributes": {
                "Identifiant station": "9037",
                "Nom station": "Parme - Clichy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:57+01:00",
                "Coordonnées géographiques": "48.8812664242,2.32819709345",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1043: {
            "attributes": {
                "Identifiant station": "3008",
                "Nom station": "Perle - Vieille du Temple",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:49+01:00",
                "Coordonnées géographiques": "48.8601346,2.3611206",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1044: {
            "attributes": {
                "Identifiant station": "13005",
                "Nom station": "Cordelières - Arago",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:00+01:00",
                "Coordonnées géographiques": "48.835480719,2.3485724617",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1045: {
            "attributes": {
                "Identifiant station": "5015",
                "Nom station": "Epée de Bois - Mouffetard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "63",
                "Nombre bornettes libres": "52",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:44+01:00",
                "Coordonnées géographiques": "48.8415086904,2.35006671399",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1046: {
            "attributes": {
                "Identifiant station": "19010",
                "Nom station": "Emile Reynaud - Porte de la Villette",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:09+01:00",
                "Coordonnées géographiques": "48.9009895363,2.38870225847",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1047: {
            "attributes": {
                "Identifiant station": "48004",
                "Nom station": "Maxime Gorki - Louis Aragon",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:40+01:00",
                "Coordonnées géographiques": "48.7890981473,2.36838787794",
                "Nom communes équipées": "Villejuif",
                "Code INSEE communes équipées": "",
            }
        },
        1048: {
            "attributes": {
                "Identifiant station": "15132",
                "Nom station": "Bargue -  Dutot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:36+01:00",
                "Coordonnées géographiques": "48.837884,2.308876",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1049: {
            "attributes": {
                "Identifiant station": "15128",
                "Nom station": "Cadix - Hameau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:45+01:00",
                "Coordonnées géographiques": "48.8340107632,2.28951618075",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1050: {
            "attributes": {
                "Identifiant station": "41405",
                "Nom station": "Place de l'Europe - Alfortville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:13+01:00",
                "Coordonnées géographiques": "48.7891370241,2.42848008871",
                "Nom communes équipées": "Alfortville",
                "Code INSEE communes équipées": "",
            }
        },
        1051: {
            "attributes": {
                "Identifiant station": "44014",
                "Nom station": "Camille Risch - Paul Armangot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:08+01:00",
                "Coordonnées géographiques": "48.774703,2.373685",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1052: {
            "attributes": {
                "Identifiant station": "20006",
                "Nom station": "Pyrénées - Plaine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "12",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:54+01:00",
                "Coordonnées géographiques": "48.8502670246,2.4062821269",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1053: {
            "attributes": {
                "Identifiant station": "12037",
                "Nom station": "Reuilly - Daumesnil",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:37+01:00",
                "Coordonnées géographiques": "48.8395385,2.3972516",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1054: {
            "attributes": {
                "Identifiant station": "21026",
                "Nom station": "Sèvres - Général Leclerc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "27",
                "Vélos mécaniques disponibles": "26",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:10+01:00",
                "Coordonnées géographiques": "48.8301215591,2.23142623901",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        1055: {
            "attributes": {
                "Identifiant station": "18004",
                "Nom station": "Place des Abbesses",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "12",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:53+01:00",
                "Coordonnées géographiques": "48.8845427261,2.33868677169",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1056: {
            "attributes": {
                "Identifiant station": "32013",
                "Nom station": "Saint-Denis - Université",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:06+01:00",
                "Coordonnées géographiques": "48.9456343095,2.363829799",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        1057: {
            "attributes": {
                "Identifiant station": "16013",
                "Nom station": "Mairie du 16ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:31+01:00",
                "Coordonnées géographiques": "48.8639368485,2.27659530938",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1058: {
            "attributes": {
                "Identifiant station": "7021",
                "Nom station": "Bosquet - Saint-Dominique",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "19",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:41+01:00",
                "Coordonnées géographiques": "48.8586517095,2.30371658023",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1059: {
            "attributes": {
                "Identifiant station": "20008",
                "Nom station": "Pyrénées - Avron",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:31+01:00",
                "Coordonnées géographiques": "48.8532468982,2.40587104453",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1060: {
            "attributes": {
                "Identifiant station": "16103",
                "Nom station": "Traktir - Foch",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:17+01:00",
                "Coordonnées géographiques": "48.8727553346,2.29154978025",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1061: {
            "attributes": {
                "Identifiant station": "18034",
                "Nom station": "Leibniz - Jean Dollfus",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:53+01:00",
                "Coordonnées géographiques": "48.8963283485,2.33339042383",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1062: {
            "attributes": {
                "Identifiant station": "11037",
                "Nom station": "Faubourg Du Temple - Republique",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:09+01:00",
                "Coordonnées géographiques": "48.8678724847,2.36489821581",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1063: {
            "attributes": {
                "Identifiant station": "6104",
                "Nom station": "Herschel - Observatoire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:24+01:00",
                "Coordonnées géographiques": "48.8434237622,2.33766451478",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1064: {
            "attributes": {
                "Identifiant station": "31709",
                "Nom station": "Place de la Fraternité",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:18+01:00",
                "Coordonnées géographiques": "48.8571640584,2.42284744978",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        1065: {
            "attributes": {
                "Identifiant station": "10034",
                "Nom station": "Louis Blanc - La Chapelle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:55+01:00",
                "Coordonnées géographiques": "48.8840089645,2.35993355513",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1066: {
            "attributes": {
                "Identifiant station": "10006",
                "Nom station": "Petites Ecuries - Faubourg Saint-Denis",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:46+01:00",
                "Coordonnées géographiques": "48.8728712059,2.35421508551",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1067: {
            "attributes": {
                "Identifiant station": "12019",
                "Nom station": "Gare de Reuilly - Picpus",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:31+01:00",
                "Coordonnées géographiques": "48.8426735412,2.39729437977",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1068: {
            "attributes": {
                "Identifiant station": "8028",
                "Nom station": "Arsène Houssaye - Champs-Elysées",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:10+01:00",
                "Coordonnées géographiques": "48.8734092763,2.29779046029",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1069: {
            "attributes": {
                "Identifiant station": "20104",
                "Nom station": "Louis Lumière.",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:35+01:00",
                "Coordonnées géographiques": "48.8621619807,2.41112578186",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1070: {
            "attributes": {
                "Identifiant station": "10011",
                "Nom station": "Bourse du Travail",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:03+01:00",
                "Coordonnées géographiques": "48.8689766388,2.3623438552",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1071: {
            "attributes": {
                "Identifiant station": "20001",
                "Nom station": "Place de la Nation - Charonne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "33",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:38+01:00",
                "Coordonnées géographiques": "48.8483646678,2.39976238598",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1072: {
            "attributes": {
                "Identifiant station": "20029",
                "Nom station": "Frères Flavien - Porte des Lilas",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:33+01:00",
                "Coordonnées géographiques": "48.8780992764,2.41104629853",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1073: {
            "attributes": {
                "Identifiant station": "18140",
                "Nom station": "Porte de Clignancourt - Ney",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:09+01:00",
                "Coordonnées géographiques": "48.899641124,2.34432142228",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1074: {
            "attributes": {
                "Identifiant station": "2003",
                "Nom station": "Saint-Denis - Réaumur",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:48+01:00",
                "Coordonnées géographiques": "48.8661116316,2.35077906755",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1075: {
            "attributes": {
                "Identifiant station": "8110",
                "Nom station": "Gare Saint-Lazare - Stockholm",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:27+01:00",
                "Coordonnées géographiques": "48.8771273167,2.32241716236",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1076: {
            "attributes": {
                "Identifiant station": "1102",
                "Nom station": "Française - Etienne Marcel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "33",
                "Vélos mécaniques disponibles": "25",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:40+01:00",
                "Coordonnées géographiques": "48.8640533047,2.34758701175",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1077: {
            "attributes": {
                "Identifiant station": "1007",
                "Nom station": "Turbigo - Française",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:43+01:00",
                "Coordonnées géographiques": "48.8635885811,2.34777946025",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1078: {
            "attributes": {
                "Identifiant station": "5122",
                "Nom station": "Saint-Michel - Luxembourg",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:03+01:00",
                "Coordonnées géographiques": "48.8450150763,2.3394941166",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1079: {
            "attributes": {
                "Identifiant station": "13051",
                "Nom station": "Halle Freyssinet - Parvis Alan Turing",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "12",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:34+01:00",
                "Coordonnées géographiques": "48.8328468145,2.37114217132",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1080: {
            "attributes": {
                "Identifiant station": "13017",
                "Nom station": "Nationale - Vincent Auriol",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:33+01:00",
                "Coordonnées géographiques": "48.832527084,2.36235441039",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1081: {
            "attributes": {
                "Identifiant station": "19201",
                "Nom station": "Petit - Georges Thill",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:03+01:00",
                "Coordonnées géographiques": "48.88603132,2.3895753175",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1082: {
            "attributes": {
                "Identifiant station": "2002",
                "Nom station": "Etienne Marcel - Montorgueil",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "24",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:07+01:00",
                "Coordonnées géographiques": "48.8645056726,2.3461959511",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1083: {
            "attributes": {
                "Identifiant station": "22301",
                "Nom station": "Chausson - Gabriel Péri",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:50+01:00",
                "Coordonnées géographiques": "48.9218910035,2.29523062706",
                "Nom communes équipées": "Gennevilliers",
                "Code INSEE communes équipées": "",
            }
        },
        1084: {
            "attributes": {
                "Identifiant station": "15111",
                "Nom station": "Olivier de Serres - Leriche",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:41+01:00",
                "Coordonnées géographiques": "48.8346022172,2.2957482934",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1085: {
            "attributes": {
                "Identifiant station": "10009",
                "Nom station": "Mairie du 10ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:11+01:00",
                "Coordonnées géographiques": "48.8720894499,2.35758259892",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1086: {
            "attributes": {
                "Identifiant station": "3005",
                "Nom station": "Sainte-Elisabeth - Turbigo",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:43+01:00",
                "Coordonnées géographiques": "48.8661033743,2.35964924097",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1087: {
            "attributes": {
                "Identifiant station": "2015",
                "Nom station": "Louis le Grand - Italiens",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:19+01:00",
                "Coordonnées géographiques": "48.8705089372,2.33405446194",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1088: {
            "attributes": {
                "Identifiant station": "19129",
                "Nom station": "Solitaires - Place des Fêtes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:55+01:00",
                "Coordonnées géographiques": "48.876601,2.391769",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1089: {
            "attributes": {
                "Identifiant station": "12002",
                "Nom station": "Lacuée - Lyon",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:45+01:00",
                "Coordonnées géographiques": "48.849268482,2.3704585433",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1090: {
            "attributes": {
                "Identifiant station": "2021",
                "Nom station": "Cléry - Montmartre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "44",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "38",
                "Vélos mécaniques disponibles": "32",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:57+01:00",
                "Coordonnées géographiques": "48.867458,2.3445211",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1091: {
            "attributes": {
                "Identifiant station": "21953",
                "Nom station": "Gare de Bellevue",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:01+01:00",
                "Coordonnées géographiques": "48.8192404686,2.22953092307",
                "Nom communes équipées": "Meudon",
                "Code INSEE communes équipées": "",
            }
        },
        1092: {
            "attributes": {
                "Identifiant station": "24006",
                "Nom station": "Adolphe Lalyre - Armand Silvestre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:13+01:00",
                "Coordonnées géographiques": "48.9040626696,2.27341622114",
                "Nom communes équipées": "Courbevoie",
                "Code INSEE communes équipées": "",
            }
        },
        1093: {
            "attributes": {
                "Identifiant station": "3001",
                "Nom station": "Beaumarchais - Saint-Gilles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "51",
                "Nombre bornettes libres": "44",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:58+01:00",
                "Coordonnées géographiques": "48.8580378,2.3679352",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1094: {
            "attributes": {
                "Identifiant station": "26005",
                "Nom station": "Hôtel de Ville d'Asnières sur Seine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:32+01:00",
                "Coordonnées géographiques": "48.9099933003,2.28823810816",
                "Nom communes équipées": "Asnières-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1095: {
            "attributes": {
                "Identifiant station": "12112",
                "Nom station": "Charenton - Jardiniers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:13+01:00",
                "Coordonnées géographiques": "48.8338217729,2.39469266411",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1096: {
            "attributes": {
                "Identifiant station": "3012",
                "Nom station": "Square Emile - Chautemps",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:43+01:00",
                "Coordonnées géographiques": "48.8674337242,2.35363572836",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1097: {
            "attributes": {
                "Identifiant station": "3011",
                "Nom station": "Turbigo - Réaumur",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "22",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8655740475,2.35641986132",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1098: {
            "attributes": {
                "Identifiant station": "14022",
                "Nom station": "Didot - Mariniers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:04+01:00",
                "Coordonnées géographiques": "48.8265054361,2.31313619827",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1099: {
            "attributes": {
                "Identifiant station": "10027",
                "Nom station": "Dunkerque - Alsace",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:11+01:00",
                "Coordonnées géographiques": "48.879355,2.358379",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1100: {
            "attributes": {
                "Identifiant station": "19012",
                "Nom station": "Lally-Tollendal - Armand Carrel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:41+01:00",
                "Coordonnées géographiques": "48.8827006622,2.37471755594",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1101: {
            "attributes": {
                "Identifiant station": "21506",
                "Nom station": "Maurice Payret-Dortail",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:30+01:00",
                "Coordonnées géographiques": "48.8781195706,2.22336251289",
                "Nom communes équipées": "Suresnes",
                "Code INSEE communes équipées": "",
            }
        },
        1102: {
            "attributes": {
                "Identifiant station": "31001",
                "Nom station": "Lagny - Auguste Blanqui",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:07+01:00",
                "Coordonnées géographiques": "48.8492256805,2.42186039686",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        1103: {
            "attributes": {
                "Identifiant station": "4103",
                "Nom station": "Archives - Rivoli",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:33+01:00",
                "Coordonnées géographiques": "48.8573467074,2.35397670418",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1104: {
            "attributes": {
                "Identifiant station": "14023",
                "Nom station": "Adolphe Pinard - Victor Hugo",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "52",
                "Nombre bornettes libres": "49",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:29+01:00",
                "Coordonnées géographiques": "48.8236825875,2.30778437108",
                "Nom communes équipées": "Malakoff",
                "Code INSEE communes équipées": "",
            }
        },
        1105: {
            "attributes": {
                "Identifiant station": "22503",
                "Nom station": "Joffre - Aigle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:15+01:00",
                "Coordonnées géographiques": "48.904033,2.245096",
                "Nom communes équipées": "La Garenne-Colombes",
                "Code INSEE communes équipées": "",
            }
        },
        1106: {
            "attributes": {
                "Identifiant station": "22502",
                "Nom station": "Kléber - National",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:47+01:00",
                "Coordonnées géographiques": "48.9063338987,2.23939105868",
                "Nom communes équipées": "La Garenne-Colombes",
                "Code INSEE communes équipées": "",
            }
        },
        1107: {
            "attributes": {
                "Identifiant station": "12020",
                "Nom station": "Cours de Vincennes - Soult",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "41",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:58+01:00",
                "Coordonnées géographiques": "48.8469187616,2.4105220288",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1108: {
            "attributes": {
                "Identifiant station": "9111",
                "Nom station": "Victoire - Laffitte",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:12+01:00",
                "Coordonnées géographiques": "48.8751437349,2.33811850514",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1109: {
            "attributes": {
                "Identifiant station": "2005",
                "Nom station": "Montmartre - Etienne Marcel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:57+01:00",
                "Coordonnées géographiques": "48.8650219458,2.34506844014",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1110: {
            "attributes": {
                "Identifiant station": "48005",
                "Nom station": "Eugène Varlin - Place de la Paix",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:44+01:00",
                "Coordonnées géographiques": "48.792862,2.363093",
                "Nom communes équipées": "Villejuif",
                "Code INSEE communes équipées": "",
            }
        },
        1111: {
            "attributes": {
                "Identifiant station": "3009",
                "Nom station": "Temple - Haudriettes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:28+01:00",
                "Coordonnées géographiques": "48.8616368802,2.35669602065",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1112: {
            "attributes": {
                "Identifiant station": "41208",
                "Nom station": "Alouettes - Maréchal de Lattre de Tassigny",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:08+01:00",
                "Coordonnées géographiques": "48.8551100929,2.49307002872",
                "Nom communes équipées": "Fontenay-sous-Bois",
                "Code INSEE communes équipées": "",
            }
        },
        1113: {
            "attributes": {
                "Identifiant station": "14132",
                "Nom station": "Bézout - Général Leclerc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:50+01:00",
                "Coordonnées géographiques": "48.828848045,2.32859730721",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1114: {
            "attributes": {
                "Identifiant station": "3003",
                "Nom station": "Fille du Calvaire - Turenne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "24",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "11",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:06+01:00",
                "Coordonnées géographiques": "48.8621050541,2.36496001482",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1115: {
            "attributes": {
                "Identifiant station": "4014",
                "Nom station": "Blancs-Manteaux - Archives",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:33+01:00",
                "Coordonnées géographiques": "48.859418007,2.35594712198",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1116: {
            "attributes": {
                "Identifiant station": "4017",
                "Nom station": "Place de l'Hôtel de Ville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "14",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:05+01:00",
                "Coordonnées géographiques": "48.8573314,2.351458",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1117: {
            "attributes": {
                "Identifiant station": "23702",
                "Nom station": "Hôtel de Ville de Chaville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:08+01:00",
                "Coordonnées géographiques": "48.8088714909,2.18813404441",
                "Nom communes équipées": "Chaville",
                "Code INSEE communes équipées": "",
            }
        },
        1118: {
            "attributes": {
                "Identifiant station": "10041",
                "Nom station": "Gare du Nord - La Chapelle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:14+01:00",
                "Coordonnées géographiques": "48.8841782873,2.35610570759",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1119: {
            "attributes": {
                "Identifiant station": "22306",
                "Nom station": "Place du 17 Octobre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:15+01:00",
                "Coordonnées géographiques": "48.9196269003,2.29782861342",
                "Nom communes équipées": "Gennevilliers",
                "Code INSEE communes équipées": "",
            }
        },
        1120: {
            "attributes": {
                "Identifiant station": "4009",
                "Nom station": "Village Saint-Paul",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:26+01:00",
                "Coordonnées géographiques": "48.8526398666,2.3607301712",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1121: {
            "attributes": {
                "Identifiant station": "13125",
                "Nom station": "Gare d'Austerlitz - Pont Charles de Gaulle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "55",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "43",
                "Vélos mécaniques disponibles": "36",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:43+01:00",
                "Coordonnées géographiques": "48.8423198301,2.36703000963",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1122: {
            "attributes": {
                "Identifiant station": "4015",
                "Nom station": "Mairie du 4ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "44",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "23",
                "Vélos électriques disponibles": "11",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:16+01:00",
                "Coordonnées géographiques": "48.8559499105,2.35638231039",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1123: {
            "attributes": {
                "Identifiant station": "21507",
                "Nom station": "Gardenat-Lapostol - Velettes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:20+01:00",
                "Coordonnées géographiques": "48.87387,2.224185",
                "Nom communes équipées": "Suresnes",
                "Code INSEE communes équipées": "",
            }
        },
        1124: {
            "attributes": {
                "Identifiant station": "12163",
                "Nom station": "Hippodrome Paris-Vincennes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:16+01:00",
                "Coordonnées géographiques": "48.8204838776,2.44627118111",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1125: {
            "attributes": {
                "Identifiant station": "3006",
                "Nom station": "Mairie du 3ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:48+01:00",
                "Coordonnées géographiques": "48.8644617814,2.36176282167",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1126: {
            "attributes": {
                "Identifiant station": "3004",
                "Nom station": "Place de la République - Temple",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:22+01:00",
                "Coordonnées géographiques": "48.8672112019,2.36354643705",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1127: {
            "attributes": {
                "Identifiant station": "15062",
                "Nom station": "Convention - Lourmel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "27",
                "Vélos mécaniques disponibles": "23",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:10+01:00",
                "Coordonnées géographiques": "48.8420550417,2.28599108756",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1128: {
            "attributes": {
                "Identifiant station": "3002",
                "Nom station": "Saint-Gilles - Turenne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:58+01:00",
                "Coordonnées géographiques": "48.8581589568,2.3647149921",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1129: {
            "attributes": {
                "Identifiant station": "18136",
                "Nom station": "Poulet - Barbès",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:37+01:00",
                "Coordonnées géographiques": "48.887148,2.349064",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1130: {
            "attributes": {
                "Identifiant station": "4101",
                "Nom station": "Bastille - Beaumarchais",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:32+01:00",
                "Coordonnées géographiques": "48.8538334639,2.36857026815",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1131: {
            "attributes": {
                "Identifiant station": "22504",
                "Nom station": "Château - République",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.905928,2.253629",
                "Nom communes équipées": "La Garenne-Colombes",
                "Code INSEE communes équipées": "",
            }
        },
        1132: {
            "attributes": {
                "Identifiant station": "1012",
                "Nom station": "Jean-Jacques Rousseau - Deux Ecus",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:19+01:00",
                "Coordonnées géographiques": "48.8629780597,2.34146352857",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1133: {
            "attributes": {
                "Identifiant station": "1014",
                "Nom station": "Rivoli - Musée du Louvre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:13+01:00",
                "Coordonnées géographiques": "48.8636463685,2.33409583569",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1134: {
            "attributes": {
                "Identifiant station": "19130",
                "Nom station": "Place Paul Delouvrier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:51+01:00",
                "Coordonnées géographiques": "48.8919762143,2.38512888551",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1135: {
            "attributes": {
                "Identifiant station": "14133",
                "Nom station": "Froidevaux - Boulard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:51+01:00",
                "Coordonnées géographiques": "48.8351473196,2.32944119722",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1136: {
            "attributes": {
                "Identifiant station": "35012",
                "Nom station": "Candale - Mehul",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:34+01:00",
                "Coordonnées géographiques": "48.8893935006,2.41437971592",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        1137: {
            "attributes": {
                "Identifiant station": "42211",
                "Nom station": "Séminaire de Conflans - Liberté",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:44+01:00",
                "Coordonnées géographiques": "48.823194,2.404179",
                "Nom communes équipées": "Charenton-le-Pont",
                "Code INSEE communes équipées": "",
            }
        },
        1138: {
            "attributes": {
                "Identifiant station": "31104",
                "Nom station": "Mairie de Rosny-sous-Bois",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:26+01:00",
                "Coordonnées géographiques": "48.871256519,2.48658075929",
                "Nom communes équipées": "Rosny-sous-Bois",
                "Code INSEE communes équipées": "",
            }
        },
        1139: {
            "attributes": {
                "Identifiant station": "21215",
                "Nom station": "Verdier - Jean Jaurès",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:50+01:00",
                "Coordonnées géographiques": "48.8161404871,2.31235619634",
                "Nom communes équipées": "Montrouge",
                "Code INSEE communes équipées": "",
            }
        },
        1140: {
            "attributes": {
                "Identifiant station": "22506",
                "Nom station": "Gare des Vallées",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:04+01:00",
                "Coordonnées géographiques": "48.913179,2.257726",
                "Nom communes équipées": "La Garenne-Colombes",
                "Code INSEE communes équipées": "",
            }
        },
        1141: {
            "attributes": {
                "Identifiant station": "42704",
                "Nom station": "Mairie du Kremlin-Bicêtre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:44+01:00",
                "Coordonnées géographiques": "48.8119198154,2.35701899976",
                "Nom communes équipées": "Le Kremlin-Bicêtre",
                "Code INSEE communes équipées": "",
            }
        },
        1142: {
            "attributes": {
                "Identifiant station": "13038",
                "Nom station": "Choisy - Malmaisons",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:01+01:00",
                "Coordonnées géographiques": "48.821919703,2.36334688341",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1143: {
            "attributes": {
                "Identifiant station": "4012",
                "Nom station": "Tiron - Rivoli",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:56+01:00",
                "Coordonnées géographiques": "48.8556922532,2.35790580511",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1144: {
            "attributes": {
                "Identifiant station": "1025",
                "Nom station": "Oratoire - Rivoli",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:39+01:00",
                "Coordonnées géographiques": "48.8613427609,2.34000474215",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1145: {
            "attributes": {
                "Identifiant station": "26006",
                "Nom station": "Bas - Gabriel Péri",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:46+01:00",
                "Coordonnées géographiques": "48.9158558054,2.29480952024",
                "Nom communes équipées": "Asnières-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1146: {
            "attributes": {
                "Identifiant station": "4018",
                "Nom station": "Saint-Bon - Rivoli",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:22+01:00",
                "Coordonnées géographiques": "48.8579996433,2.35021728098",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1147: {
            "attributes": {
                "Identifiant station": "42026",
                "Nom station": "Barbès - Monique Maunoury",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:05+01:00",
                "Coordonnées géographiques": "48.8155271882,2.36871693349",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1148: {
            "attributes": {
                "Identifiant station": "32019",
                "Nom station": "Place du 8 mai 1945",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:26+01:00",
                "Coordonnées géographiques": "48.9388822721,2.35662437975",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        1149: {
            "attributes": {
                "Identifiant station": "2012",
                "Nom station": "Choiseul - Quatre Septembre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "44",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "40",
                "Vélos mécaniques disponibles": "23",
                "Vélos électriques disponibles": "17",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8699355332,2.33599752188",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1150: {
            "attributes": {
                "Identifiant station": "21954",
                "Nom station": "Val-Fleury",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:21+01:00",
                "Coordonnées géographiques": "48.807671408,2.24072176963",
                "Nom communes équipées": "Meudon",
                "Code INSEE communes équipées": "",
            }
        },
        1151: {
            "attributes": {
                "Identifiant station": "16044",
                "Nom station": "Porte de Saint Cloud - Parc des Princes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "24",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:55+01:00",
                "Coordonnées géographiques": "48.8386627571,2.2525164485",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1152: {
            "attributes": {
                "Identifiant station": "1018",
                "Nom station": "Alger - Rivoli",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:17+01:00",
                "Coordonnées géographiques": "48.8647735535,2.32944939361",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1153: {
            "attributes": {
                "Identifiant station": "44001",
                "Nom station": "Quai Jules Guesde - Saint-Simon",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "29",
                "Vélos mécaniques disponibles": "25",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:08+01:00",
                "Coordonnées géographiques": "48.8039148722,2.40896616203",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1154: {
            "attributes": {
                "Identifiant station": "15040",
                "Nom station": "Saint-Amand - Labrouste",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.8340972748,2.30876773596",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1155: {
            "attributes": {
                "Identifiant station": "19111",
                "Nom station": "Cesaria Evora - Rosa Parks",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "40",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-01-03T11:28:28+01:00",
                "Coordonnées géographiques": "48.8971767283,2.37451739609",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1156: {
            "attributes": {
                "Identifiant station": "13054",
                "Nom station": "Pau Casals - Neuve Tolbiac",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "53",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "49",
                "Vélos mécaniques disponibles": "33",
                "Vélos électriques disponibles": "16",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:24+01:00",
                "Coordonnées géographiques": "48.8311602,2.3774395",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1157: {
            "attributes": {
                "Identifiant station": "21509",
                "Nom station": "Gardenat Lapostol - Franklin Roosevelt",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-11-03T13:49:24+01:00",
                "Coordonnées géographiques": "48.871839,2.222505",
                "Nom communes équipées": "Suresnes",
                "Code INSEE communes équipées": "",
            }
        },
        1158: {
            "attributes": {
                "Identifiant station": "14134",
                "Nom station": "Château - Place de Catalogne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:24+01:00",
                "Coordonnées géographiques": "48.8368055423,2.31852930039",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1159: {
            "attributes": {
                "Identifiant station": "4007",
                "Nom station": "Place de la Bastille - Bourdon",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:40+01:00",
                "Coordonnées géographiques": "48.8523643553,2.3681089282",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1160: {
            "attributes": {
                "Identifiant station": "12165",
                "Nom station": "Champs de Manoeuvre - Cartoucherie",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2019-03-21T10:50:32+01:00",
                "Coordonnées géographiques": "48.835261,2.447974",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1161: {
            "attributes": {
                "Identifiant station": "2004",
                "Nom station": "Square Pierre Lazareff",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "15",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8671417207,2.34839335084",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1162: {
            "attributes": {
                "Identifiant station": "6026",
                "Nom station": "Saint-Placide - Cherche Midi",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "66",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "41",
                "Vélos mécaniques disponibles": "37",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:50+01:00",
                "Coordonnées géographiques": "48.8490535919,2.32531797141",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1163: {
            "attributes": {
                "Identifiant station": "18015",
                "Nom station": "Custine - Ramey",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "41",
                "Nombre bornettes libres": "39",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:01+01:00",
                "Coordonnées géographiques": "48.8887358969,2.34695266932",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1164: {
            "attributes": {
                "Identifiant station": "1023",
                "Nom station": "Saint-Honoré - Musée du Louvre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:47+01:00",
                "Coordonnées géographiques": "48.8634431328,2.33487818464",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1165: {
            "attributes": {
                "Identifiant station": "21505",
                "Nom station": "Édouard Vaillant - Place Jean Jaurès",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:15+01:00",
                "Coordonnées géographiques": "48.86265538,2.20890373191",
                "Nom communes équipées": "Suresnes",
                "Code INSEE communes équipées": "",
            }
        },
        1166: {
            "attributes": {
                "Identifiant station": "48008",
                "Nom station": "Guy Môquet - Parvis Georges Marchais",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:08+01:00",
                "Coordonnées géographiques": "48.795188401,2.35943466425",
                "Nom communes équipées": "Villejuif",
                "Code INSEE communes équipées": "",
            }
        },
        1167: {
            "attributes": {
                "Identifiant station": "2016",
                "Nom station": "Aboukir - Alexandrie",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:20+01:00",
                "Coordonnées géographiques": "48.8686834444,2.35020432695",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1168: {
            "attributes": {
                "Identifiant station": "5003",
                "Nom station": "Le Goff - Gay-Lussac",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:44+01:00",
                "Coordonnées géographiques": "48.846102401,2.34122078866",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1169: {
            "attributes": {
                "Identifiant station": "9039",
                "Nom station": "Chaussée d'Antin - Haussmann",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-11-03T13:51:01+01:00",
                "Coordonnées géographiques": "48.87243,2.33359",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1170: {
            "attributes": {
                "Identifiant station": "6007",
                "Nom station": "Bréa - Raspail",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:00+01:00",
                "Coordonnées géographiques": "48.8427384783,2.32979422176",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1171: {
            "attributes": {
                "Identifiant station": "33020",
                "Nom station": "Waldeck Rochet - Germaine Tillon",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:49+01:00",
                "Coordonnées géographiques": "48.911059602,2.36763015389",
                "Nom communes équipées": "Aubervilliers",
                "Code INSEE communes équipées": "",
            }
        },
        1172: {
            "attributes": {
                "Identifiant station": "3007",
                "Nom station": "Archives - Pastourelle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "26",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:29+01:00",
                "Coordonnées géographiques": "48.8623615,2.3595041",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1173: {
            "attributes": {
                "Identifiant station": "2008",
                "Nom station": "Mairie du 2ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:28+01:00",
                "Coordonnées géographiques": "48.8672190355,2.34046254377",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1174: {
            "attributes": {
                "Identifiant station": "31020",
                "Nom station": "Résistance - Parmentier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:50+01:00",
                "Coordonnées géographiques": "48.8596513813,2.43670508265",
                "Nom communes équipées": "Montreuil",
                "Code INSEE communes équipées": "",
            }
        },
        1175: {
            "attributes": {
                "Identifiant station": "1026",
                "Nom station": "Place du Lieutenant Henri Karcher",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "20",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:48+01:00",
                "Coordonnées géographiques": "48.8634908717,2.34006710351",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1176: {
            "attributes": {
                "Identifiant station": "16139",
                "Nom station": "Suresnes - Muette à Neuilly",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:32+01:00",
                "Coordonnées géographiques": "48.8685704043,2.26600091904",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1177: {
            "attributes": {
                "Identifiant station": "23101",
                "Nom station": "Grande Rue - Europe",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:00+01:00",
                "Coordonnées géographiques": "48.8218104688,2.20587987453",
                "Nom communes équipées": "Sèvres",
                "Code INSEE communes équipées": "",
            }
        },
        1178: {
            "attributes": {
                "Identifiant station": "27006",
                "Nom station": "Victor Hugo - Gare",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:17+01:00",
                "Coordonnées géographiques": "48.9231339527,2.26009704173",
                "Nom communes équipées": "Colombes",
                "Code INSEE communes équipées": "",
            }
        },
        1179: {
            "attributes": {
                "Identifiant station": "2009",
                "Nom station": "Filles Saint-Thomas - Place de la Bourse",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "58",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "48",
                "Vélos mécaniques disponibles": "36",
                "Vélos électriques disponibles": "12",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:13+01:00",
                "Coordonnées géographiques": "48.8688998909,2.33975438452",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1180: {
            "attributes": {
                "Identifiant station": "42028",
                "Nom station": "Verdun - Carnot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.8084393761,2.37379521132",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1181: {
            "attributes": {
                "Identifiant station": "1116",
                "Nom station": "Ventadour - Opéra",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "18",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.8668106533,2.33438844825",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1182: {
            "attributes": {
                "Identifiant station": "12166",
                "Nom station": "Parc Floral du Bel Air",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:44+01:00",
                "Coordonnées géographiques": "48.8392857119,2.43856016546",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1183: {
            "attributes": {
                "Identifiant station": "27001",
                "Nom station": "Square Victor Basch",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:41+01:00",
                "Coordonnées géographiques": "48.9143048623,2.22891569138",
                "Nom communes équipées": "Colombes",
                "Code INSEE communes équipées": "",
            }
        },
        1184: {
            "attributes": {
                "Identifiant station": "1013",
                "Nom station": "Place du Palais Royal",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "67",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "62",
                "Vélos mécaniques disponibles": "48",
                "Vélos électriques disponibles": "14",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:39+01:00",
                "Coordonnées géographiques": "48.8623829706,2.33848225325",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1185: {
            "attributes": {
                "Identifiant station": "12157",
                "Nom station": "Fabre d'Eglantine - Place de la Nation",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "68",
                "Nombre bornettes libres": "59",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:47+01:00",
                "Coordonnées géographiques": "48.8471116,2.3956204",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1186: {
            "attributes": {
                "Identifiant station": "22603",
                "Nom station": "Place de la Division Leclerc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "35",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:47+01:00",
                "Coordonnées géographiques": "48.7938249886,2.27311404306",
                "Nom communes équipées": "Fontenay-aux-Roses",
                "Code INSEE communes équipées": "",
            }
        },
        1187: {
            "attributes": {
                "Identifiant station": "12164",
                "Nom station": "Stade Pershing",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "20",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:04+01:00",
                "Coordonnées géographiques": "48.8291078269,2.45835352689",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1188: {
            "attributes": {
                "Identifiant station": "2102",
                "Nom station": "Saint-Marc - Feydeau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "36",
                "Vélos mécaniques disponibles": "20",
                "Vélos électriques disponibles": "16",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:07+01:00",
                "Coordonnées géographiques": "48.8703106614,2.34197918326",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1189: {
            "attributes": {
                "Identifiant station": "1017",
                "Nom station": "Saint-Honoré - 29 juillet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8655711428,2.33056741758",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1190: {
            "attributes": {
                "Identifiant station": "2006",
                "Nom station": "Place des Victoires",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.8660354,2.3420333",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1191: {
            "attributes": {
                "Identifiant station": "17106",
                "Nom station": "Berthier - Porte de Clichy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:10+01:00",
                "Coordonnées géographiques": "48.8942226878,2.31258720068",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1192: {
            "attributes": {
                "Identifiant station": "21119",
                "Nom station": "Belfort - Victor Hugo",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:08+01:00",
                "Coordonnées géographiques": "48.9018492,2.3172724",
                "Nom communes équipées": "Clichy",
                "Code INSEE communes équipées": "",
            }
        },
        1193: {
            "attributes": {
                "Identifiant station": "17021",
                "Nom station": "Jouffroy d'Abbans - Villiers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:35+01:00",
                "Coordonnées géographiques": "48.884215547,2.30565335602",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1194: {
            "attributes": {
                "Identifiant station": "9019",
                "Nom station": "Victor Massé - Jean-Baptiste Pigalle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:21+01:00",
                "Coordonnées géographiques": "48.8811043724,2.33669556677",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1195: {
            "attributes": {
                "Identifiant station": "22601",
                "Nom station": "Marx Dormoy - Château Sainte-Barbe",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:26+01:00",
                "Coordonnées géographiques": "48.7917823411,2.29785684496",
                "Nom communes équipées": "Fontenay-aux-Roses",
                "Code INSEE communes équipées": "",
            }
        },
        1196: {
            "attributes": {
                "Identifiant station": "23042",
                "Nom station": "André Theuriet - Blagis",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:35+01:00",
                "Coordonnées géographiques": "48.7797732575,2.31305256486",
                "Nom communes équipées": "Bourg-la-Reine",
                "Code INSEE communes équipées": "",
            }
        },
        1197: {
            "attributes": {
                "Identifiant station": "42018",
                "Nom station": "Amédée Huon - Fort d'Ivry",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:44+01:00",
                "Coordonnées géographiques": "48.8051313769,2.3884896934",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1198: {
            "attributes": {
                "Identifiant station": "2109",
                "Nom station": "Réaumur - Montmartre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:50+01:00",
                "Coordonnées géographiques": "48.8682309998,2.34382603389",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1199: {
            "attributes": {
                "Identifiant station": "28005",
                "Nom station": "Jean Jaurès - Paul Lafargue",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:44+01:00",
                "Coordonnées géographiques": "48.8849331777,2.24844887853",
                "Nom communes équipées": "Puteaux",
                "Code INSEE communes équipées": "",
            }
        },
        1200: {
            "attributes": {
                "Identifiant station": "44004",
                "Nom station": "Place de l'Eglise - Vitry-sur-Seine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:45+01:00",
                "Coordonnées géographiques": "48.7919228199,2.39505775273",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1201: {
            "attributes": {
                "Identifiant station": "35005",
                "Nom station": "Quai de l'Aisne - Général Leclerc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:22+01:00",
                "Coordonnées géographiques": "48.895713,2.400571",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        1202: {
            "attributes": {
                "Identifiant station": "19118",
                "Nom station": "Simon Bolivar - Henri Turot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:05+01:00",
                "Coordonnées géographiques": "48.876887,2.374454",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1203: {
            "attributes": {
                "Identifiant station": "18112",
                "Nom station": "Arthur Ranc - Hôpital Bichat",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:09+01:00",
                "Coordonnées géographiques": "48.8978712501,2.33352184296",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1204: {
            "attributes": {
                "Identifiant station": "10166",
                "Nom station": "Strasbourg - Saint-Denis",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:47+01:00",
                "Coordonnées géographiques": "48.8696016424,2.35429085791",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1205: {
            "attributes": {
                "Identifiant station": "10201",
                "Nom station": "Magenta - Chabrol",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8764953509,2.35609900206",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1206: {
            "attributes": {
                "Identifiant station": "44008",
                "Nom station": "Youri Gagarine - Commune de Paris",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "29",
                "Vélos mécaniques disponibles": "26",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:39+01:00",
                "Coordonnées géographiques": "48.782427463,2.39397346973",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1207: {
            "attributes": {
                "Identifiant station": "48007",
                "Nom station": "Parc Départemental des Hautes-Bruyères",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:58+01:00",
                "Coordonnées géographiques": "48.7934579014,2.35090121627",
                "Nom communes équipées": "Villejuif",
                "Code INSEE communes équipées": "",
            }
        },
        1208: {
            "attributes": {
                "Identifiant station": "20140",
                "Nom station": "Paganini - Davout",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:50+01:00",
                "Coordonnées géographiques": "48.85135,2.412021",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1209: {
            "attributes": {
                "Identifiant station": "12113",
                "Nom station": "Cardinal Lavigerie",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "33",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:15+01:00",
                "Coordonnées géographiques": "48.8329341937,2.4026367494",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1210: {
            "attributes": {
                "Identifiant station": "45004",
                "Nom station": "République - Louis Loucheur",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "24",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "11",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:23+01:00",
                "Coordonnées géographiques": "48.8235492612,2.51018654555",
                "Nom communes équipées": "Champigny-sur-Marne",
                "Code INSEE communes équipées": "",
            }
        },
        1211: {
            "attributes": {
                "Identifiant station": "1015",
                "Nom station": "Place André Malraux",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:47+01:00",
                "Coordonnées géographiques": "48.8639251587,2.33562268317",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1212: {
            "attributes": {
                "Identifiant station": "23202",
                "Nom station": "Henri Barbusse - Gabriel Péri",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-07-27T14:53:38+02:00",
                "Coordonnées géographiques": "48.8017737958,2.28741663622",
                "Nom communes équipées": "Châtillon",
                "Code INSEE communes équipées": "",
            }
        },
        1213: {
            "attributes": {
                "Identifiant station": "19046",
                "Nom station": "Porte de Pantin.",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:08+01:00",
                "Coordonnées géographiques": "48.8891997256,2.39501819015",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1214: {
            "attributes": {
                "Identifiant station": "2022",
                "Nom station": "Saint-Fiacre - Poissonière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "20",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:14+01:00",
                "Coordonnées géographiques": "48.8708289116,2.34592135996",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1215: {
            "attributes": {
                "Identifiant station": "16010",
                "Nom station": "Belles Feuilles - Place de Mexico",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:16+01:00",
                "Coordonnées géographiques": "48.8658047465,2.28292062879",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1216: {
            "attributes": {
                "Identifiant station": "17020",
                "Nom station": "Jouffroy d'Abbans - Malesherbes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:07+01:00",
                "Coordonnées géographiques": "48.8850134,2.3070717",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1217: {
            "attributes": {
                "Identifiant station": "42019",
                "Nom station": "Lampes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "43",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "33",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:00+01:00",
                "Coordonnées géographiques": "48.8115248199,2.40179311484",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1218: {
            "attributes": {
                "Identifiant station": "23206",
                "Nom station": "Béranger - Félix Faure",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-10-15T10:01:49+02:00",
                "Coordonnées géographiques": "48.8040482862,2.28482633737",
                "Nom communes équipées": "Châtillon",
                "Code INSEE communes équipées": "",
            }
        },
        1219: {
            "attributes": {
                "Identifiant station": "1008",
                "Nom station": "Les Halles - Saint-Eustache",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "29",
                "Vélos mécaniques disponibles": "26",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:41+01:00",
                "Coordonnées géographiques": "48.863626959,2.34262391925",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1220: {
            "attributes": {
                "Identifiant station": "10202",
                "Nom station": "Bichat - Grange aux Belles",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-08-16T15:38:00+02:00",
                "Coordonnées géographiques": "48.873688,2.365015",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1221: {
            "attributes": {
                "Identifiant station": "1024",
                "Nom station": "Coquillière - Louvre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:41+01:00",
                "Coordonnées géographiques": "48.8637959097,2.34257362783",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1222: {
            "attributes": {
                "Identifiant station": "25007",
                "Nom station": "Bons Raisins - Voltaire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:22+01:00",
                "Coordonnées géographiques": "48.8728245146,2.19471555709",
                "Nom communes équipées": "Rueil-Malmaison",
                "Code INSEE communes équipées": "",
            }
        },
        1223: {
            "attributes": {
                "Identifiant station": "32020",
                "Nom station": "Jules Guesdes - Carnot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:18+01:00",
                "Coordonnées géographiques": "48.9382816993,2.35089920461",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        1224: {
            "attributes": {
                "Identifiant station": "42024",
                "Nom station": "Danielle Casanova - Ledru-Rollin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:41+01:00",
                "Coordonnées géographiques": "48.8157834979,2.3832597211",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1225: {
            "attributes": {
                "Identifiant station": "2020",
                "Nom station": "Danielle Casanova - Opéra",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:42+01:00",
                "Coordonnées géographiques": "48.8676432423,2.33338940889",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1226: {
            "attributes": {
                "Identifiant station": "18135",
                "Nom station": "Ney - Porte de Montmartre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:46+01:00",
                "Coordonnées géographiques": "48.897918,2.3364872",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1227: {
            "attributes": {
                "Identifiant station": "20010",
                "Nom station": "Sorbier - Gasnier-Guy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:28+01:00",
                "Coordonnées géographiques": "48.865217631,2.39365965128",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1228: {
            "attributes": {
                "Identifiant station": "8104",
                "Nom station": "Chateaubriand - Friedland",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-09-24T20:54:27+02:00",
                "Coordonnées géographiques": "48.8740144756,2.29991625787",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1229: {
            "attributes": {
                "Identifiant station": "21011",
                "Nom station": "Silly - Louis Blériot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:36+01:00",
                "Coordonnées géographiques": "48.8321904537,2.23401993513",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        1230: {
            "attributes": {
                "Identifiant station": "21002",
                "Nom station": "Place Denfert-Rochereau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:15+01:00",
                "Coordonnées géographiques": "48.8432525363,2.24598795176",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        1231: {
            "attributes": {
                "Identifiant station": "12016",
                "Nom station": "Saint-Mande - Picpus",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:37+01:00",
                "Coordonnées géographiques": "48.8451277283,2.40156862965",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1232: {
            "attributes": {
                "Identifiant station": "8018",
                "Nom station": "Saint-Augustin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "31",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:34+01:00",
                "Coordonnées géographiques": "48.8762009747,2.31978625059",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1233: {
            "attributes": {
                "Identifiant station": "23301",
                "Nom station": "Les Blagis",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:07+01:00",
                "Coordonnées géographiques": "48.785205,2.302526",
                "Nom communes équipées": "Sceaux",
                "Code INSEE communes équipées": "",
            }
        },
        1234: {
            "attributes": {
                "Identifiant station": "13015",
                "Nom station": "Chevaleret - Louise Weiss",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "31",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:34+01:00",
                "Coordonnées géographiques": "48.8325865134,2.37121138812",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1235: {
            "attributes": {
                "Identifiant station": "34013",
                "Nom station": "Blanqui - Docteur Bauer",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-08-13T11:36:30+02:00",
                "Coordonnées géographiques": "48.9077354891,2.34071928146",
                "Nom communes équipées": "Saint-Ouen-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1236: {
            "attributes": {
                "Identifiant station": "13116",
                "Nom station": "Place du Docteur Yersin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:08+01:00",
                "Coordonnées géographiques": "48.8201049499,2.37255837768",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1237: {
            "attributes": {
                "Identifiant station": "23103",
                "Nom station": "Manufacture Nationale",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-02-28T15:26:50+01:00",
                "Coordonnées géographiques": "48.828785141,2.22372226417",
                "Nom communes équipées": "Sèvres",
                "Code INSEE communes équipées": "",
            }
        },
        1238: {
            "attributes": {
                "Identifiant station": "11111",
                "Nom station": "Léon Frot - Charonne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8549638003,2.38721708537",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1239: {
            "attributes": {
                "Identifiant station": "42210",
                "Nom station": "Victor Hugo - Paul Eluard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "26",
                "Vélos mécaniques disponibles": "22",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:16+01:00",
                "Coordonnées géographiques": "48.8196025549,2.41143196731",
                "Nom communes équipées": "Charenton-le-Pont",
                "Code INSEE communes équipées": "",
            }
        },
        1240: {
            "attributes": {
                "Identifiant station": "13006",
                "Nom station": "Duméril - Jeanne d'Arc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "41",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:01+01:00",
                "Coordonnées géographiques": "48.8381516081,2.35710844956",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1241: {
            "attributes": {
                "Identifiant station": "48012",
                "Nom station": "Stalingrad - Henri Luisette",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:29+01:00",
                "Coordonnées géographiques": "48.7836309246,2.36693594676",
                "Nom communes équipées": "Villejuif",
                "Code INSEE communes équipées": "",
            }
        },
        1242: {
            "attributes": {
                "Identifiant station": "12119",
                "Nom station": "Dom-Pérignon - Gravelle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "41",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8254815744,2.4098803103",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1243: {
            "attributes": {
                "Identifiant station": "20048",
                "Nom station": "Bagnolet - Place Saint-Blaise",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:37+01:00",
                "Coordonnées géographiques": "48.85973,2.40334",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1244: {
            "attributes": {
                "Identifiant station": "48006",
                "Nom station": "Maxime Gorki - Paul Vaillant-Couturier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:13+01:00",
                "Coordonnées géographiques": "48.796735088,2.36833423376",
                "Nom communes équipées": "Villejuif",
                "Code INSEE communes équipées": "",
            }
        },
        1245: {
            "attributes": {
                "Identifiant station": "23303",
                "Nom station": "Gare de Robinson",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:29+01:00",
                "Coordonnées géographiques": "48.7799259212,2.28182863444",
                "Nom communes équipées": "Sceaux",
                "Code INSEE communes équipées": "",
            }
        },
        1246: {
            "attributes": {
                "Identifiant station": "18016",
                "Nom station": "Francoeur - Caulaincourt",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "33",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.889994441,2.34252970666",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1247: {
            "attributes": {
                "Identifiant station": "42022",
                "Nom station": "Gare RER - Maurice Grandcoing",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "29",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:52+01:00",
                "Coordonnées géographiques": "48.813881,2.392429",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1248: {
            "attributes": {
                "Identifiant station": "18010",
                "Nom station": "Riquet - Marx Dormoy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:25+01:00",
                "Coordonnées géographiques": "48.890210037,2.36017562449",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1249: {
            "attributes": {
                "Identifiant station": "6015",
                "Nom station": "André Mazet - Saint-André des Arts",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "55",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "30",
                "Vélos mécaniques disponibles": "24",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:08+01:00",
                "Coordonnées géographiques": "48.8537558106,2.33909580857",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1250: {
            "attributes": {
                "Identifiant station": "34010",
                "Nom station": "Hôtel de Ville de Saint-Ouen",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "48",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "24",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:55+01:00",
                "Coordonnées géographiques": "48.9113801858,2.33400768076",
                "Nom communes équipées": "Saint-Ouen-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1251: {
            "attributes": {
                "Identifiant station": "23703",
                "Nom station": "Gare de Chaville Rive Gauche",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:22+01:00",
                "Coordonnées géographiques": "48.8051603043,2.18827720731",
                "Nom communes équipées": "Chaville",
                "Code INSEE communes équipées": "",
            }
        },
        1252: {
            "attributes": {
                "Identifiant station": "10003",
                "Nom station": "Hauteville - Bonne Nouvelle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:08+01:00",
                "Coordonnées géographiques": "48.8707743602,2.34964963683",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1253: {
            "attributes": {
                "Identifiant station": "5027",
                "Nom station": "Gobelins - Port-Royal",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "26",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:09+01:00",
                "Coordonnées géographiques": "48.8371359003,2.351520136",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1254: {
            "attributes": {
                "Identifiant station": "15024",
                "Nom station": "Suffren - La Motte-Piquet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:18+01:00",
                "Coordonnées géographiques": "48.8508812309,2.30121999979",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1255: {
            "attributes": {
                "Identifiant station": "20142",
                "Nom station": "Mûriers - Père Lachaise",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:55+01:00",
                "Coordonnées géographiques": "48.8641305097,2.39164982067",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1256: {
            "attributes": {
                "Identifiant station": "13024",
                "Nom station": "Bobillot - Tolbiac",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:28+01:00",
                "Coordonnées géographiques": "48.8255961335,2.3502041772",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1257: {
            "attributes": {
                "Identifiant station": "21704",
                "Nom station": "Place du Maréchal de Lattre de Tassigny",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:58+01:00",
                "Coordonnées géographiques": "48.8217077153,2.28534314854",
                "Nom communes équipées": "Vanves",
                "Code INSEE communes équipées": "",
            }
        },
        1258: {
            "attributes": {
                "Identifiant station": "9025",
                "Nom station": "Taitbout - Châteaudun",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "19",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:34+01:00",
                "Coordonnées géographiques": "48.8763839694,2.33535667793",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1259: {
            "attributes": {
                "Identifiant station": "19121",
                "Nom station": "Belleville -  Pré Saint-Gervais",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:25+01:00",
                "Coordonnées géographiques": "48.8755659101,2.39304609597",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1260: {
            "attributes": {
                "Identifiant station": "13028",
                "Nom station": "Gouthière - Poterne des Peupliers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:48+01:00",
                "Coordonnées géographiques": "48.820507,2.351342",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1261: {
            "attributes": {
                "Identifiant station": "34011",
                "Nom station": "Clef des Champs - Albert Dhalenne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:41+01:00",
                "Coordonnées géographiques": "48.914938,2.330665",
                "Nom communes équipées": "Saint-Ouen-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1262: {
            "attributes": {
                "Identifiant station": "11014",
                "Nom station": "Place de la Nation - Taillebourg",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "47",
                "Nombre bornettes libres": "43",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:10+01:00",
                "Coordonnées géographiques": "48.8488408,2.3973043",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1263: {
            "attributes": {
                "Identifiant station": "24101",
                "Nom station": "Place Charles de Gaulle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:13+01:00",
                "Coordonnées géographiques": "48.82654572,2.18925887361",
                "Nom communes équipées": "Ville-d'Avray",
                "Code INSEE communes équipées": "",
            }
        },
        1264: {
            "attributes": {
                "Identifiant station": "14027",
                "Nom station": "Gergovie - Alésia",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:06+01:00",
                "Coordonnées géographiques": "48.8308871746,2.31888210121",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1265: {
            "attributes": {
                "Identifiant station": "19114",
                "Nom station": "Manin - Secrétan",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:15+01:00",
                "Coordonnées géographiques": "48.8791665,2.3787959",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1266: {
            "attributes": {
                "Identifiant station": "42303",
                "Nom station": "Place Eustache Deschamps",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:01+01:00",
                "Coordonnées géographiques": "48.793855,2.336637",
                "Nom communes équipées": "Cachan",
                "Code INSEE communes équipées": "",
            }
        },
        1267: {
            "attributes": {
                "Identifiant station": "12123",
                "Nom station": "Château de Vincennes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "52",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "18",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:05+01:00",
                "Coordonnées géographiques": "48.8442319326,2.43920953771",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1268: {
            "attributes": {
                "Identifiant station": "41605",
                "Nom station": "Général De Gaulle - Vivien",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:53+01:00",
                "Coordonnées géographiques": "48.8360742009,2.41891667247",
                "Nom communes équipées": "Saint-Mandé",
                "Code INSEE communes équipées": "",
            }
        },
        1269: {
            "attributes": {
                "Identifiant station": "24004",
                "Nom station": "Paix - Verdun",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:36+01:00",
                "Coordonnées géographiques": "48.900504,2.261173",
                "Nom communes équipées": "Courbevoie",
                "Code INSEE communes équipées": "",
            }
        },
        1270: {
            "attributes": {
                "Identifiant station": "12102",
                "Nom station": "Bercy - Traversiére",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:38+01:00",
                "Coordonnées géographiques": "48.8461926426,2.37038008869",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1271: {
            "attributes": {
                "Identifiant station": "19006",
                "Nom station": "Mathis - Flandre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "55",
                "Nombre bornettes libres": "48",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:48+01:00",
                "Coordonnées géographiques": "48.8906429899,2.37625546753",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1272: {
            "attributes": {
                "Identifiant station": "19119",
                "Nom station": "Edouard Pailleron - Bouret",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:04+01:00",
                "Coordonnées géographiques": "48.8805976941,2.37694948912",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1273: {
            "attributes": {
                "Identifiant station": "20003",
                "Nom station": "Pyrénées - Cours de Vincennes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:50+01:00",
                "Coordonnées géographiques": "48.8477967799,2.40617215633",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1274: {
            "attributes": {
                "Identifiant station": "35013",
                "Nom station": "Guillaume Tell - Parc Henri Barbusse",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:10+01:00",
                "Coordonnées géographiques": "48.8895892589,2.41988629103",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        1275: {
            "attributes": {
                "Identifiant station": "8052",
                "Nom station": "Balzac - Champs Elysées",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "13",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:20+01:00",
                "Coordonnées géographiques": "48.8726996396,2.30019539595",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1276: {
            "attributes": {
                "Identifiant station": "24005",
                "Nom station": "Edith Cavell - Volta",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:54+01:00",
                "Coordonnées géographiques": "48.9016165643,2.26935435086",
                "Nom communes équipées": "Courbevoie",
                "Code INSEE communes équipées": "",
            }
        },
        1277: {
            "attributes": {
                "Identifiant station": "23041",
                "Nom station": "Général Leclerc - Bièvre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:22+01:00",
                "Coordonnées géographiques": "48.7824144287,2.31666952372",
                "Nom communes équipées": "Bourg-la-Reine",
                "Code INSEE communes équipées": "",
            }
        },
        1278: {
            "attributes": {
                "Identifiant station": "43007",
                "Nom station": "Hôtel de Ville de Vincennes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:30+01:00",
                "Coordonnées géographiques": "48.8482760971,2.43989557028",
                "Nom communes équipées": "Vincennes",
                "Code INSEE communes équipées": "",
            }
        },
        1279: {
            "attributes": {
                "Identifiant station": "15069",
                "Nom station": "Félix Faure - Convention",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "44",
                "Nombre bornettes libres": "9",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "33",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:37+01:00",
                "Coordonnées géographiques": "48.841249336,2.28838660085",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1280: {
            "attributes": {
                "Identifiant station": "14004",
                "Nom station": "Port Royal - Hôpital Cochin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:59+01:00",
                "Coordonnées géographiques": "48.8383732338,2.34071787447",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1281: {
            "attributes": {
                "Identifiant station": "15012",
                "Nom station": "Cambronne - Mademoiselle",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "67",
                "Nombre bornettes libres": "56",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:50+01:00",
                "Coordonnées géographiques": "48.84327733,2.30230766886",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1282: {
            "attributes": {
                "Identifiant station": "14035",
                "Nom station": "Maine - Jean Zay",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "50",
                "Nombre bornettes libres": "47",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:04+01:00",
                "Coordonnées géographiques": "48.8379252639,2.32254289091",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1283: {
            "attributes": {
                "Identifiant station": "19117",
                "Nom station": "Euryale Dehaynin - Loire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:48+01:00",
                "Coordonnées géographiques": "48.8862489173,2.37745910883",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1284: {
            "attributes": {
                "Identifiant station": "20032",
                "Nom station": "Square des Amandiers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "12",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:49+01:00",
                "Coordonnées géographiques": "48.8661563067,2.38913241774",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1285: {
            "attributes": {
                "Identifiant station": "15053",
                "Nom station": "Lecourbe - Convention",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:46+01:00",
                "Coordonnées géographiques": "48.8392331924,2.29151979089",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1286: {
            "attributes": {
                "Identifiant station": "6025",
                "Nom station": "Assas - Rennes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:10+01:00",
                "Coordonnées géographiques": "48.8492084999,2.32851454623",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1287: {
            "attributes": {
                "Identifiant station": "8040",
                "Nom station": "Pierre Charron - Champs-Elysées",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "27",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "10",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.8701033599,2.30435013771",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1288: {
            "attributes": {
                "Identifiant station": "14009",
                "Nom station": "Tombe Issoire - René Coty",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:01+01:00",
                "Coordonnées géographiques": "48.8305128767,2.33436137438",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1289: {
            "attributes": {
                "Identifiant station": "21022",
                "Nom station": "Dôme - République",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "20",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:18+01:00",
                "Coordonnées géographiques": "48.832681,2.250257",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        1290: {
            "attributes": {
                "Identifiant station": "22404",
                "Nom station": "Pierre Brossolette - 12 Février 1934",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-11-03T13:49:00+01:00",
                "Coordonnées géographiques": "48.8176962417,2.30761237442",
                "Nom communes équipées": "Malakoff",
                "Code INSEE communes équipées": "",
            }
        },
        1291: {
            "attributes": {
                "Identifiant station": "15041",
                "Nom station": "Franquet - Labrouste",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "36",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:14+01:00",
                "Coordonnées géographiques": "48.8328770501,2.30676278472",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1292: {
            "attributes": {
                "Identifiant station": "16017",
                "Nom station": "Siam - La Pompe",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:40+01:00",
                "Coordonnées géographiques": "48.861720329,2.27534656856",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1293: {
            "attributes": {
                "Identifiant station": "11022",
                "Nom station": "Roquette - Auguste Laurent",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:24+01:00",
                "Coordonnées géographiques": "48.858001906,2.38179770265",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1294: {
            "attributes": {
                "Identifiant station": "17006",
                "Nom station": "Guy Môquet - Compoint",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:07+01:00",
                "Coordonnées géographiques": "48.8921355904,2.32321947813",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1295: {
            "attributes": {
                "Identifiant station": "41103",
                "Nom station": "Place de la vache noire",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:34+01:00",
                "Coordonnées géographiques": "48.8104051706,2.32706040144",
                "Nom communes équipées": "Arcueil",
                "Code INSEE communes équipées": "",
            }
        },
        1296: {
            "attributes": {
                "Identifiant station": "12022",
                "Nom station": "Sahel - Général Michel Bizot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:42+01:00",
                "Coordonnées géographiques": "48.8408729648,2.40444447845",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1297: {
            "attributes": {
                "Identifiant station": "41602",
                "Nom station": "Place Charles Digeon",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:32+01:00",
                "Coordonnées géographiques": "48.8435420313,2.41842716932",
                "Nom communes équipées": "Saint-Mandé",
                "Code INSEE communes équipées": "",
            }
        },
        1298: {
            "attributes": {
                "Identifiant station": "15027",
                "Nom station": "Duhamel - André Gide",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:21+01:00",
                "Coordonnées géographiques": "48.8365662244,2.3127071211",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1299: {
            "attributes": {
                "Identifiant station": "20110",
                "Nom station": "Gambetta - Tourelles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8753379023,2.40599211305",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1300: {
            "attributes": {
                "Identifiant station": "21021",
                "Nom station": "Enfants du Paradis - Peupliers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "35",
                "Vélos mécaniques disponibles": "23",
                "Vélos électriques disponibles": "12",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:55+01:00",
                "Coordonnées géographiques": "48.8331226865,2.25714594126",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        1301: {
            "attributes": {
                "Identifiant station": "48002",
                "Nom station": "LCL Campus - Place Oscar Niemeyer",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:22+01:00",
                "Coordonnées géographiques": "48.8053424806,2.36398369074",
                "Nom communes équipées": "Villejuif",
                "Code INSEE communes équipées": "",
            }
        },
        1302: {
            "attributes": {
                "Identifiant station": "31105",
                "Nom station": "Lavoisier - Missak Manouchian",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:50+01:00",
                "Coordonnées géographiques": "48.862864689,2.49567646533",
                "Nom communes équipées": "Rosny-sous-Bois",
                "Code INSEE communes équipées": "",
            }
        },
        1303: {
            "attributes": {
                "Identifiant station": "20012",
                "Nom station": "Buzenval - Vignoles",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:33+01:00",
                "Coordonnées géographiques": "48.8538581717,2.40000408143",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1304: {
            "attributes": {
                "Identifiant station": "17117",
                "Nom station": "Pereire - Saussure",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:57+01:00",
                "Coordonnées géographiques": "48.8882290755,2.31012962759",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1305: {
            "attributes": {
                "Identifiant station": "13120",
                "Nom station": "Pierre Joseph Desault",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "12",
                "Vélos mécaniques disponibles": "9",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:19+01:00",
                "Coordonnées géographiques": "48.821102308,2.3787754029",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1306: {
            "attributes": {
                "Identifiant station": "42205",
                "Nom station": "Place Aristide Briand",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:23+01:00",
                "Coordonnées géographiques": "48.8218923658,2.41300374269",
                "Nom communes équipées": "Charenton-le-Pont",
                "Code INSEE communes équipées": "",
            }
        },
        1307: {
            "attributes": {
                "Identifiant station": "20020",
                "Nom station": "Pyrénées - Charles Renouvier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:27+01:00",
                "Coordonnées géographiques": "48.8609593323,2.40042185204",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1308: {
            "attributes": {
                "Identifiant station": "7023",
                "Nom station": "Bourdonnais - Tour Eiffel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "60",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "41",
                "Vélos mécaniques disponibles": "37",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:42+01:00",
                "Coordonnées géographiques": "48.8608977979,2.29559959845",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1309: {
            "attributes": {
                "Identifiant station": "12008",
                "Nom station": "Hector Malot - Daumesnil",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "54",
                "Nombre bornettes libres": "51",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:11+01:00",
                "Coordonnées géographiques": "48.8469355299,2.37708762288",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1310: {
            "attributes": {
                "Identifiant station": "8007",
                "Nom station": "Rome - Provence",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "22",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:18+01:00",
                "Coordonnées géographiques": "48.8742300556,2.32557011172",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1311: {
            "attributes": {
                "Identifiant station": "13036",
                "Nom station": "Olympiades",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:35+01:00",
                "Coordonnées géographiques": "48.8267646652,2.36508607864",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1312: {
            "attributes": {
                "Identifiant station": "11029",
                "Nom station": "Ménilmontant - Oberkampf",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "35",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:29+01:00",
                "Coordonnées géographiques": "48.8666176587,2.38301344042",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1313: {
            "attributes": {
                "Identifiant station": "15050",
                "Nom station": "Dominique Pado - Croix Nivert",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "7",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8364879816,2.28972505778",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1314: {
            "attributes": {
                "Identifiant station": "20005",
                "Nom station": "Reynaldo Hahn - Lagny",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:07+01:00",
                "Coordonnées géographiques": "48.8492660436,2.41233032884",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1315: {
            "attributes": {
                "Identifiant station": "16025",
                "Nom station": "Jean de la Fontaine - Boulainvilliers",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "68",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "38",
                "Vélos mécaniques disponibles": "31",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:50+01:00",
                "Coordonnées géographiques": "48.8526479146,2.27586641908",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1316: {
            "attributes": {
                "Identifiant station": "15120",
                "Nom station": "Colonel Pierre Avia",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:17+01:00",
                "Coordonnées géographiques": "48.8296843301,2.27534258685",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1317: {
            "attributes": {
                "Identifiant station": "18021",
                "Nom station": "Marcadet - Ramey",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:06+01:00",
                "Coordonnées géographiques": "48.890875,2.3449019",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1318: {
            "attributes": {
                "Identifiant station": "13055",
                "Nom station": "M.- A. Lagroua Weill-Hallé -  F. Dolto",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "60",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "60",
                "Vélos mécaniques disponibles": "35",
                "Vélos électriques disponibles": "25",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:48+01:00",
                "Coordonnées géographiques": "48.8285952839,2.38022060627",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1319: {
            "attributes": {
                "Identifiant station": "16116",
                "Nom station": "George Sand - Jean de La Fontaine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:28+01:00",
                "Coordonnées géographiques": "48.8498691876,2.26825738753",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1320: {
            "attributes": {
                "Identifiant station": "21205",
                "Nom station": "Arthur Auger - Jean Jaurès",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:03+01:00",
                "Coordonnées géographiques": "48.8137364456,2.30692774057",
                "Nom communes équipées": "Montrouge",
                "Code INSEE communes équipées": "",
            }
        },
        1321: {
            "attributes": {
                "Identifiant station": "20013",
                "Nom station": "Alexandre Dumas - Place de la Réunion",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "44",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:38+01:00",
                "Coordonnées géographiques": "48.8553371074,2.39977036876",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1322: {
            "attributes": {
                "Identifiant station": "11028",
                "Nom station": "Guillaume Bertrand - République",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "37",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:14+01:00",
                "Coordonnées géographiques": "48.8638251743,2.38090637715",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1323: {
            "attributes": {
                "Identifiant station": "15045",
                "Nom station": "Porte de la Plaine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:24+01:00",
                "Coordonnées géographiques": "48.8280516961,2.29242771864",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1324: {
            "attributes": {
                "Identifiant station": "19014",
                "Nom station": "Lorraine - Jean Jaurès",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:32+01:00",
                "Coordonnées géographiques": "48.8863865,2.3825772",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1325: {
            "attributes": {
                "Identifiant station": "25004",
                "Nom station": "Marcel Pourtout - Édouard Manet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "9",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:06+01:00",
                "Coordonnées géographiques": "48.8755562076,2.16559678316",
                "Nom communes équipées": "Rueil-Malmaison",
                "Code INSEE communes équipées": "",
            }
        },
        1326: {
            "attributes": {
                "Identifiant station": "16023",
                "Nom station": "Passy - Place du Costa Rica",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "7",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:16+01:00",
                "Coordonnées géographiques": "48.8584893,2.2839655",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1327: {
            "attributes": {
                "Identifiant station": "14005",
                "Nom station": "René Coty - Place Denfert-Rochereau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:02+01:00",
                "Coordonnées géographiques": "48.8330564773,2.3329411447",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1328: {
            "attributes": {
                "Identifiant station": "22014",
                "Nom station": "Victor Hugo - Bineau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "10",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:34+01:00",
                "Coordonnées géographiques": "48.8875822589,2.27838903666",
                "Nom communes équipées": "Neuilly-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1329: {
            "attributes": {
                "Identifiant station": "21023",
                "Nom station": "Ancienne Mairie - André Morizet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "24",
                "Vélos mécaniques disponibles": "20",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:51+01:00",
                "Coordonnées géographiques": "48.838311,2.234077",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        1330: {
            "attributes": {
                "Identifiant station": "11016",
                "Nom station": "Philippe-Auguste",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "48",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:26+01:00",
                "Coordonnées géographiques": "48.8518534883,2.39329453558",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1331: {
            "attributes": {
                "Identifiant station": "43004",
                "Nom station": "Montreuil - République",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:53+01:00",
                "Coordonnées géographiques": "48.8502634946,2.43452042341",
                "Nom communes équipées": "Vincennes",
                "Code INSEE communes équipées": "",
            }
        },
        1332: {
            "attributes": {
                "Identifiant station": "10031",
                "Nom station": "Louis Blanc - Aqueduc",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "33",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:52+01:00",
                "Coordonnées géographiques": "48.8820301826,2.36356157809",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1333: {
            "attributes": {
                "Identifiant station": "8046",
                "Nom station": "Marceau - Président Wilson",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "21",
                "Vélos mécaniques disponibles": "19",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:13+01:00",
                "Coordonnées géographiques": "48.8652359,2.3002513",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1334: {
            "attributes": {
                "Identifiant station": "21027",
                "Nom station": "Fief - République",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:56+01:00",
                "Coordonnées géographiques": "48.8317088817,2.25577197969",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        1335: {
            "attributes": {
                "Identifiant station": "20014",
                "Nom station": "Charonne - Alexandre Dumas",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "33",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:49+01:00",
                "Coordonnées géographiques": "48.8560774151,2.39486932755",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1336: {
            "attributes": {
                "Identifiant station": "15044",
                "Nom station": "Dantzig - Lefebvre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:43:58+01:00",
                "Coordonnées géographiques": "48.8301392159,2.29593403637",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1337: {
            "attributes": {
                "Identifiant station": "17110",
                "Nom station": "Legendre - Nollet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:53+01:00",
                "Coordonnées géographiques": "48.8877421868,2.32029307988",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1338: {
            "attributes": {
                "Identifiant station": "12040",
                "Nom station": "Porte de Charenton",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "67",
                "Nombre bornettes libres": "59",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:42+01:00",
                "Coordonnées géographiques": "48.8312401059,2.39891074598",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1339: {
            "attributes": {
                "Identifiant station": "7010",
                "Nom station": "Constantine - Université",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "65",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "60",
                "Vélos mécaniques disponibles": "43",
                "Vélos électriques disponibles": "17",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:00+01:00",
                "Coordonnées géographiques": "48.8606804737,2.31484819558",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1340: {
            "attributes": {
                "Identifiant station": "18018",
                "Nom station": "Carpeaux - Place Jacques Froment",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:13+01:00",
                "Coordonnées géographiques": "48.8906542,2.3309428",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1341: {
            "attributes": {
                "Identifiant station": "31701",
                "Nom station": "Mairie de Bagnolet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:11+01:00",
                "Coordonnées géographiques": "48.8684213186,2.41795878857",
                "Nom communes équipées": "Bagnolet",
                "Code INSEE communes équipées": "",
            }
        },
        1342: {
            "attributes": {
                "Identifiant station": "19001",
                "Nom station": "Crimée - Ourcq",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "56",
                "Nombre bornettes libres": "51",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:55+01:00",
                "Coordonnées géographiques": "48.8941041403,2.37294608847",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1343: {
            "attributes": {
                "Identifiant station": "19102",
                "Nom station": "Rampal - Belleville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:26+01:00",
                "Coordonnées géographiques": "48.8730216016,2.37991098315",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1344: {
            "attributes": {
                "Identifiant station": "16022",
                "Nom station": "Jean Bologne - Passy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "19",
                "Nombre bornettes libres": "12",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:56+01:00",
                "Coordonnées géographiques": "48.8575668898,2.28002739531",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1345: {
            "attributes": {
                "Identifiant station": "5033",
                "Nom station": "Saint-Séverin - Saint-Michel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:26+01:00",
                "Coordonnées géographiques": "48.8527313055,2.34409276396",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1346: {
            "attributes": {
                "Identifiant station": "20011",
                "Nom station": "Pyrénées - Dagorno",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:23+01:00",
                "Coordonnées géographiques": "48.8554257707,2.4051643163",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1347: {
            "attributes": {
                "Identifiant station": "18101",
                "Nom station": "Championnet.",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:05+01:00",
                "Coordonnées géographiques": "48.8953643,2.3498166",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1348: {
            "attributes": {
                "Identifiant station": "13013",
                "Nom station": "Saint-Marcel - Hôpital",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "15",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:37+01:00",
                "Coordonnées géographiques": "48.8395044246,2.36098941702",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1349: {
            "attributes": {
                "Identifiant station": "21207",
                "Nom station": "Verdier - République",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "27",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:19+01:00",
                "Coordonnées géographiques": "48.8154037663,2.31739372015",
                "Nom communes équipées": "Montrouge",
                "Code INSEE communes équipées": "",
            }
        },
        1350: {
            "attributes": {
                "Identifiant station": "13023",
                "Nom station": "Bobillot - Moulin des Prés",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.8286573462,2.35302064632",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1351: {
            "attributes": {
                "Identifiant station": "12122",
                "Nom station": "Bois de Vincennes - Polygone",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "51",
                "Nombre bornettes libres": "46",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:56+01:00",
                "Coordonnées géographiques": "48.8354449162,2.43142108144",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1352: {
            "attributes": {
                "Identifiant station": "21007",
                "Nom station": "Rond-Point Rhin et Danube",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "53",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "42",
                "Vélos mécaniques disponibles": "39",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:36+01:00",
                "Coordonnées géographiques": "48.840452821,2.22802251577",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        1353: {
            "attributes": {
                "Identifiant station": "24007",
                "Nom station": "Jean-Baptiste Charcot - Sainte-Geneviève",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "18",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:08+01:00",
                "Coordonnées géographiques": "48.9031088372,2.27889295667",
                "Nom communes équipées": "Courbevoie",
                "Code INSEE communes équipées": "",
            }
        },
        1354: {
            "attributes": {
                "Identifiant station": "5105",
                "Nom station": "Geoffroy-Saint-Hilaire - Saint-Marcel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:21+01:00",
                "Coordonnées géographiques": "48.8390782642,2.35687734067",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1355: {
            "attributes": {
                "Identifiant station": "16033",
                "Nom station": "Marché d'Auteuil",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "11",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:22+01:00",
                "Coordonnées géographiques": "48.8484276137,2.26525382116",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1356: {
            "attributes": {
                "Identifiant station": "12039",
                "Nom station": "Général Michel Bizot - Claude Decaen",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:39+01:00",
                "Coordonnées géographiques": "48.8348126298,2.40092778813",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1357: {
            "attributes": {
                "Identifiant station": "6028",
                "Nom station": "Quatre Vents - Carrefour de l'Odéon",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "22",
                "Vélos mécaniques disponibles": "18",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:43+01:00",
                "Coordonnées géographiques": "48.8517513745,2.33816169513",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1358: {
            "attributes": {
                "Identifiant station": "12017",
                "Nom station": "Saint-Mandé - Docteur Arnold Netter",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "4",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:56+01:00",
                "Coordonnées géographiques": "48.8446258577,2.40494638681",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1359: {
            "attributes": {
                "Identifiant station": "20106",
                "Nom station": "Mairie du 20ème",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "27",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:11+01:00",
                "Coordonnées géographiques": "48.8653023243,2.39906698465",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1360: {
            "attributes": {
                "Identifiant station": "8039",
                "Nom station": "Colisée - Champs-Elysées",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "24",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "23",
                "Vélos mécaniques disponibles": "18",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:44+01:00",
                "Coordonnées géographiques": "48.8704753994,2.30761907995",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1361: {
            "attributes": {
                "Identifiant station": "15014",
                "Nom station": "Volontaires - Vaugirard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "11",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:27+01:00",
                "Coordonnées géographiques": "48.8413205018,2.30808901725",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1362: {
            "attributes": {
                "Identifiant station": "21033",
                "Nom station": "Sèvres - Maréchal Juin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "18",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "14",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:34+01:00",
                "Coordonnées géographiques": "48.8324305,2.2292812",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        1363: {
            "attributes": {
                "Identifiant station": "15114",
                "Nom station": "Pasteur - Cotentin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:31+01:00",
                "Coordonnées géographiques": "48.8389284,2.316356",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1364: {
            "attributes": {
                "Identifiant station": "17036",
                "Nom station": "Place Tristan Bernard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:41+01:00",
                "Coordonnées géographiques": "48.8791964,2.2915403",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1365: {
            "attributes": {
                "Identifiant station": "6004",
                "Nom station": "Regard - Rennes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "46",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:40+01:00",
                "Coordonnées géographiques": "48.847385405,2.32685554773",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1366: {
            "attributes": {
                "Identifiant station": "17050",
                "Nom station": "Batignolles - Abel Truchet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:58+01:00",
                "Coordonnées géographiques": "48.8831039046,2.32383474909",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1367: {
            "attributes": {
                "Identifiant station": "25002",
                "Nom station": "Hôtel de Ville de Rueil-Malmaison",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "12",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:45:52+01:00",
                "Coordonnées géographiques": "48.878194,2.181724",
                "Nom communes équipées": "Rueil-Malmaison",
                "Code INSEE communes équipées": "",
            }
        },
        1368: {
            "attributes": {
                "Identifiant station": "19018",
                "Nom station": "Cité de la Musique",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "60",
                "Nombre bornettes libres": "57",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:42+01:00",
                "Coordonnées géographiques": "48.8887965565,2.39258119319",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1369: {
            "attributes": {
                "Identifiant station": "15113",
                "Nom station": "Arsonval - Falguière",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:51+01:00",
                "Coordonnées géographiques": "48.8404004037,2.3131178673",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1370: {
            "attributes": {
                "Identifiant station": "10001",
                "Nom station": "René Boulanger - Lancry",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "62",
                "Nombre bornettes libres": "46",
                "Nombre total vélos disponibles": "11",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:27+01:00",
                "Coordonnées géographiques": "48.8685433,2.3600032",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1371: {
            "attributes": {
                "Identifiant station": "20033",
                "Nom station": "Etienne Dolet - Place Maurice Chevalier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "32",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:19+01:00",
                "Coordonnées géographiques": "48.8679830336,2.38514053111",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1372: {
            "attributes": {
                "Identifiant station": "44009",
                "Nom station": "Gare RER les Ardoines",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "49",
                "Nombre bornettes libres": "49",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2019-05-17T09:28:29+02:00",
                "Coordonnées géographiques": "48.7838563635,2.40790937175",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1373: {
            "attributes": {
                "Identifiant station": "6005",
                "Nom station": "Montparnasse -  Square Ozanam",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "49",
                "Nombre bornettes libres": "43",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:39+01:00",
                "Coordonnées géographiques": "48.8433105678,2.32652429491",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1374: {
            "attributes": {
                "Identifiant station": "10151",
                "Nom station": "Gare du Nord - Faubourg Saint-Denis",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "28",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "3",
                "Vélos mécaniques disponibles": "2",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:04+01:00",
                "Coordonnées géographiques": "48.8796127655,2.35684029758",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1375: {
            "attributes": {
                "Identifiant station": "14025",
                "Nom station": "Jacquier - Didot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:54+01:00",
                "Coordonnées géographiques": "48.8295333629,2.31817927212",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1376: {
            "attributes": {
                "Identifiant station": "21508",
                "Nom station": "Georges Pompidou - Verdun",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2019-09-05T14:10:50+02:00",
                "Coordonnées géographiques": "48.875481,2.232443",
                "Nom communes équipées": "Suresnes",
                "Code INSEE communes équipées": "",
            }
        },
        1377: {
            "attributes": {
                "Identifiant station": "11018",
                "Nom station": "Saint-Maur - République",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "38",
                "Nombre bornettes libres": "37",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:23+01:00",
                "Coordonnées géographiques": "48.8641858615,2.37831741571",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1378: {
            "attributes": {
                "Identifiant station": "4005",
                "Nom station": "Quai des Célestins - Henri IV",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "14",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "8",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:24+01:00",
                "Coordonnées géographiques": "48.8512971,2.3624535",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1379: {
            "attributes": {
                "Identifiant station": "19007",
                "Nom station": "Flandre - Ourcq",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "29",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:30+01:00",
                "Coordonnées géographiques": "48.8926015919,2.37909156829",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1380: {
            "attributes": {
                "Identifiant station": "8003",
                "Nom station": "Galilée - Vernet",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "17",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-11-03T13:51:01+01:00",
                "Coordonnées géographiques": "48.8717158787,2.2985444963",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1381: {
            "attributes": {
                "Identifiant station": "21016",
                "Nom station": "Point du Jour - Jean Jaurès",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "6",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:30+01:00",
                "Coordonnées géographiques": "48.8287531414,2.24591146701",
                "Nom communes équipées": "Boulogne-Billancourt",
                "Code INSEE communes équipées": "",
            }
        },
        1382: {
            "attributes": {
                "Identifiant station": "19045",
                "Nom station": "Macdonald - Césaria Evora",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "67",
                "Nombre bornettes libres": "67",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2020-06-29T14:52:20+02:00",
                "Coordonnées géographiques": "48.89883,2.37933",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1383: {
            "attributes": {
                "Identifiant station": "6020",
                "Nom station": "Place Saint-André des Arts",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "37",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "34",
                "Vélos mécaniques disponibles": "28",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:13+01:00",
                "Coordonnées géographiques": "48.8528861739,2.34265442938",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1384: {
            "attributes": {
                "Identifiant station": "12105",
                "Nom station": "Bercy - Villot",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "45",
                "Vélos mécaniques disponibles": "32",
                "Vélos électriques disponibles": "13",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:55+01:00",
                "Coordonnées géographiques": "48.841795009,2.37678501848",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1385: {
            "attributes": {
                "Identifiant station": "13008",
                "Nom station": "Place d'Italie - Soeur Rosalie",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "52",
                "Nombre bornettes libres": "47",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:01+01:00",
                "Coordonnées géographiques": "48.8319085204,2.35469434481",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1386: {
            "attributes": {
                "Identifiant station": "8004",
                "Nom station": "Malesherbes - Place de la Madeleine",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "67",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "78",
                "Vélos mécaniques disponibles": "57",
                "Vélos électriques disponibles": "21",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:39+01:00",
                "Coordonnées géographiques": "48.8704060285,2.32324350981",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1387: {
            "attributes": {
                "Identifiant station": "19123",
                "Nom station": "Metz - Marne",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "31",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:33+01:00",
                "Coordonnées géographiques": "48.891163,2.3865705",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1388: {
            "attributes": {
                "Identifiant station": "41212",
                "Nom station": "Nungesser - Santé",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "21",
                "Nombre total vélos disponibles": "1",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:29+01:00",
                "Coordonnées géographiques": "48.852335,2.448967",
                "Nom communes équipées": "Fontenay-sous-Bois",
                "Code INSEE communes équipées": "",
            }
        },
        1389: {
            "attributes": {
                "Identifiant station": "21107",
                "Nom station": "Bertrand Sincholle - Henri Barbusse",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "20",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:44+01:00",
                "Coordonnées géographiques": "48.8992598239,2.3042294383",
                "Nom communes équipées": "Clichy",
                "Code INSEE communes équipées": "",
            }
        },
        1390: {
            "attributes": {
                "Identifiant station": "13053",
                "Nom station": "Chevaleret - Tolbiac",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "21",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "18",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:46:03+01:00",
                "Coordonnées géographiques": "48.82941,2.375934",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1391: {
            "attributes": {
                "Identifiant station": "16104",
                "Nom station": "Paul Valery - Victor Hugo",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "25",
                "Nombre total vélos disponibles": "10",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:44+01:00",
                "Coordonnées géographiques": "48.8713740608,2.28871285915",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1392: {
            "attributes": {
                "Identifiant station": "18047",
                "Nom station": "St-Ouen - Lamarck",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "42",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:48+01:00",
                "Coordonnées géographiques": "48.8910830403,2.32673569484",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1393: {
            "attributes": {
                "Identifiant station": "5031",
                "Nom station": "Lacépède - Linné",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "54",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "19",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "4",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:16+01:00",
                "Coordonnées géographiques": "48.8437035477,2.35465127975",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1394: {
            "attributes": {
                "Identifiant station": "8002",
                "Nom station": "Gare Saint-Lazare - Cour du Havre",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "8",
                "Nombre total vélos disponibles": "32",
                "Vélos mécaniques disponibles": "29",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:56+01:00",
                "Coordonnées géographiques": "48.8756744009,2.32655983418",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1395: {
            "attributes": {
                "Identifiant station": "22007",
                "Nom station": "Commandant Charcot - Bretteville",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:24+01:00",
                "Coordonnées géographiques": "48.8753057077,2.25584607571",
                "Nom communes équipées": "Neuilly-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1396: {
            "attributes": {
                "Identifiant station": "13123",
                "Nom station": "Bibliothèque Nationale de France",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "42",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "75",
                "Vélos mécaniques disponibles": "56",
                "Vélos électriques disponibles": "19",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:27+01:00",
                "Coordonnées géographiques": "48.8350268238,2.3760157451",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1397: {
            "attributes": {
                "Identifiant station": "13037",
                "Nom station": "Ivry - Baudricourt",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "20",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "4",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:14+01:00",
                "Coordonnées géographiques": "48.8246959954,2.36310607358",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1398: {
            "attributes": {
                "Identifiant station": "34009",
                "Nom station": "Dora Maar - Dulcie September",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-05-28T18:31:59+02:00",
                "Coordonnées géographiques": "48.906438,2.32159",
                "Nom communes équipées": "Saint-Ouen-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1399: {
            "attributes": {
                "Identifiant station": "6009",
                "Nom station": "Guynemer - Jardin du Luxembourg",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:08+01:00",
                "Coordonnées géographiques": "48.8466127,2.3325478",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1400: {
            "attributes": {
                "Identifiant station": "9029",
                "Nom station": "Saint-Lazare - Place d'Estienne d'Orves",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "39",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "17",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:46+01:00",
                "Coordonnées géographiques": "48.8768345,2.3329067",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1401: {
            "attributes": {
                "Identifiant station": "42027",
                "Nom station": "Verdun - Pierre et Marie Curie",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:42:58+01:00",
                "Coordonnées géographiques": "48.8128223983,2.37071067095",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1402: {
            "attributes": {
                "Identifiant station": "4104",
                "Nom station": "Sebastopol - Rambuteau",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "19",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8618180927,2.3501381278",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1403: {
            "attributes": {
                "Identifiant station": "8044",
                "Nom station": "Alfred de Vigny - Place du Général Brocard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "16",
                "Nombre total vélos disponibles": "17",
                "Vélos mécaniques disponibles": "14",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:12+01:00",
                "Coordonnées géographiques": "48.8783613836,2.30525444822",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1404: {
            "attributes": {
                "Identifiant station": "42004",
                "Nom station": "Westermeyer - Paul Vaillant-Couturier",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "25",
                "Vélos mécaniques disponibles": "16",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:40+01:00",
                "Coordonnées géographiques": "48.8191161816,2.39666372538",
                "Nom communes équipées": "Ivry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1405: {
            "attributes": {
                "Identifiant station": "8031",
                "Nom station": "Jean Mermoz - Champs-Elysées",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "33",
                "Nombre bornettes libres": "2",
                "Nombre total vélos disponibles": "30",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "9",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:10+01:00",
                "Coordonnées géographiques": "48.8696898569,2.31073245406",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1406: {
            "attributes": {
                "Identifiant station": "22305",
                "Nom station": "Jean Jaurès - Place du Dr Roux",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "35",
                "Nombre bornettes libres": "6",
                "Nombre total vélos disponibles": "28",
                "Vélos mécaniques disponibles": "23",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:03+01:00",
                "Coordonnées géographiques": "48.9275670328,2.29473370174",
                "Nom communes équipées": "Gennevilliers",
                "Code INSEE communes équipées": "",
            }
        },
        1407: {
            "attributes": {
                "Identifiant station": "16140",
                "Nom station": "Longchamp - Suresnes",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-05-26T11:51:36+02:00",
                "Coordonnées géographiques": "48.8641322653,2.23990537226",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1408: {
            "attributes": {
                "Identifiant station": "34008",
                "Nom station": "Garibaldi - Gabriel Péri",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "19",
                "Nombre total vélos disponibles": "8",
                "Vélos mécaniques disponibles": "6",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:08+01:00",
                "Coordonnées géographiques": "48.9057419114,2.33160442081",
                "Nom communes équipées": "Saint-Ouen-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1409: {
            "attributes": {
                "Identifiant station": "10028",
                "Nom station": "Gare du Nord - Dunkerque",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "29",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:49+01:00",
                "Coordonnées géographiques": "48.8804235084,2.35276367515",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1410: {
            "attributes": {
                "Identifiant station": "22507",
                "Nom station": "Châteaudun - Place de la Liberté",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-10-20T09:43:31+02:00",
                "Coordonnées géographiques": "48.909708303,2.24544864806",
                "Nom communes équipées": "La Garenne-Colombes",
                "Code INSEE communes équipées": "",
            }
        },
        1411: {
            "attributes": {
                "Identifiant station": "44010",
                "Nom station": "8 Mai 1945 - 10 Juillet 1940",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "30",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2019-05-17T12:08:17+02:00",
                "Coordonnées géographiques": "48.7845670297,2.39790357649",
                "Nom communes équipées": "Vitry-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1412: {
            "attributes": {
                "Identifiant station": "12106",
                "Nom station": "Gare de Lyon - Roland Barthes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "68",
                "Nombre bornettes libres": "13",
                "Nombre total vélos disponibles": "54",
                "Vélos mécaniques disponibles": "46",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:51+01:00",
                "Coordonnées géographiques": "48.8439572949,2.37908821553",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1413: {
            "attributes": {
                "Identifiant station": "7007",
                "Nom station": "Musée d'Orsay",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "65",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "60",
                "Vélos mécaniques disponibles": "55",
                "Vélos électriques disponibles": "5",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:09+01:00",
                "Coordonnées géographiques": "48.8597261579,2.32586044818",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1414: {
            "attributes": {
                "Identifiant station": "22005",
                "Nom station": "Charles de Gaulle - Madrid",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "7",
                "Nombre total vélos disponibles": "13",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:40+01:00",
                "Coordonnées géographiques": "48.8844322704,2.26080447435",
                "Nom communes équipées": "Neuilly-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1415: {
            "attributes": {
                "Identifiant station": "35021",
                "Nom station": "Cartier Bresson - Denis Papin",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "24",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:48:06+01:00",
                "Coordonnées géographiques": "48.902616,2.397814",
                "Nom communes équipées": "Pantin",
                "Code INSEE communes équipées": "",
            }
        },
        1416: {
            "attributes": {
                "Identifiant station": "23203",
                "Nom station": "Division Leclerc - Kléber",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "22",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:49+01:00",
                "Coordonnées géographiques": "48.7981166342,2.2826877223",
                "Nom communes équipées": "Châtillon",
                "Code INSEE communes équipées": "",
            }
        },
        1417: {
            "attributes": {
                "Identifiant station": "18045",
                "Nom station": "Pierre Ginier - Clichy",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "16",
                "Nombre bornettes libres": "14",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:54+01:00",
                "Coordonnées géographiques": "48.8867667,2.326228",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1418: {
            "attributes": {
                "Identifiant station": "9035",
                "Nom station": "Gare Saint-Lazare - Place de Budapest",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "44",
                "Nombre bornettes libres": "4",
                "Nombre total vélos disponibles": "39",
                "Vélos mécaniques disponibles": "31",
                "Vélos électriques disponibles": "8",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:48+01:00",
                "Coordonnées géographiques": "48.877753,2.327339",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1419: {
            "attributes": {
                "Identifiant station": "21302",
                "Nom station": "Aristide Briand - Place de la Résistance",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "25",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "29",
                "Vélos mécaniques disponibles": "22",
                "Vélos électriques disponibles": "7",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:45+01:00",
                "Coordonnées géographiques": "48.821242484,2.25110024214",
                "Nom communes équipées": "Issy-les-Moulineaux",
                "Code INSEE communes équipées": "",
            }
        },
        1420: {
            "attributes": {
                "Identifiant station": "9104",
                "Nom station": "Caumartin - Provence",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "27",
                "Vélos mécaniques disponibles": "13",
                "Vélos électriques disponibles": "14",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:50+01:00",
                "Coordonnées géographiques": "48.8744227734,2.32846856117",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1421: {
            "attributes": {
                "Identifiant station": "32023",
                "Nom station": "Guy Môquet - Jean Moulin",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-07-28T11:33:19+02:00",
                "Coordonnées géographiques": "48.940931521,2.36394899993",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        1422: {
            "attributes": {
                "Identifiant station": "15056",
                "Nom station": "Place Balard",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "22",
                "Nombre bornettes libres": "1",
                "Nombre total vélos disponibles": "36",
                "Vélos mécaniques disponibles": "33",
                "Vélos électriques disponibles": "3",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:42+01:00",
                "Coordonnées géographiques": "48.8363957364,2.27841921151",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1423: {
            "attributes": {
                "Identifiant station": "22602",
                "Nom station": "Place Henri Dunant - Gare RER",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-07-28T09:28:26+02:00",
                "Coordonnées géographiques": "48.787543,2.2907628",
                "Nom communes équipées": "Fontenay-aux-Roses",
                "Code INSEE communes équipées": "",
            }
        },
        1424: {
            "attributes": {
                "Identifiant station": "21514",
                "Nom station": "Place de la Paix - Abbé Saint-Pierre",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-10-22T10:34:12+02:00",
                "Coordonnées géographiques": "48.8646272446,2.20475703385",
                "Nom communes équipées": "Suresnes",
                "Code INSEE communes équipées": "",
            }
        },
        1425: {
            "attributes": {
                "Identifiant station": "23201",
                "Nom station": "Avenir",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "45",
                "Nombre bornettes libres": "39",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "5",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:45+01:00",
                "Coordonnées géographiques": "48.8103438,2.3003776",
                "Nom communes équipées": "Châtillon",
                "Code INSEE communes équipées": "",
            }
        },
        1426: {
            "attributes": {
                "Identifiant station": "23204",
                "Nom station": "Roland Garros - Jean Bouin",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-10-22T10:43:20+02:00",
                "Coordonnées géographiques": "48.8094608572,2.28531986383",
                "Nom communes équipées": "Châtillon",
                "Code INSEE communes équipées": "",
            }
        },
        1427: {
            "attributes": {
                "Identifiant station": "23205",
                "Nom station": "Parc des Pierrettes",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "23",
                "Nombre bornettes libres": "23",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:32+01:00",
                "Coordonnées géographiques": "48.7973062,2.2909969",
                "Nom communes équipées": "Châtillon",
                "Code INSEE communes équipées": "",
            }
        },
        1428: {
            "attributes": {
                "Identifiant station": "26008",
                "Nom station": "Henri Barbusse - Bourguignons",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "34",
                "Nombre bornettes libres": "17",
                "Nombre total vélos disponibles": "16",
                "Vélos mécaniques disponibles": "10",
                "Vélos électriques disponibles": "6",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:47:45+01:00",
                "Coordonnées géographiques": "48.913017228,2.27342122708",
                "Nom communes équipées": "Asnières-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1429: {
            "attributes": {
                "Identifiant station": "17127",
                "Nom station": "Porte de Clichy - Place des Nations Unies",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "40",
                "Nombre bornettes libres": "34",
                "Nombre total vélos disponibles": "5",
                "Vélos mécaniques disponibles": "3",
                "Vélos électriques disponibles": "2",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:44:52+01:00",
                "Coordonnées géographiques": "48.8967372633,2.3105429992",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1430: {
            "attributes": {
                "Identifiant station": "17126",
                "Nom station": "Hélène et François Missoffe - Emile Borel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "32",
                "Nombre bornettes libres": "3",
                "Nombre total vélos disponibles": "26",
                "Vélos mécaniques disponibles": "15",
                "Vélos électriques disponibles": "11",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:51:02+01:00",
                "Coordonnées géographiques": "48.8999065384,2.32154399167",
                "Nom communes équipées": "Paris",
                "Code INSEE communes équipées": "",
            }
        },
        1431: {
            "attributes": {
                "Identifiant station": "32012",
                "Nom station": "Carrefour Pleyel",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "36",
                "Nombre bornettes libres": "5",
                "Nombre total vélos disponibles": "32",
                "Vélos mécaniques disponibles": "21",
                "Vélos électriques disponibles": "11",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:49:52+01:00",
                "Coordonnées géographiques": "48.9203415696,2.34385438263",
                "Nom communes équipées": "Saint-Denis",
                "Code INSEE communes équipées": "",
            }
        },
        1432: {
            "attributes": {
                "Identifiant station": "34005",
                "Nom station": "Adrien Lesesne - Docteur Bauer",
                "Station en fonctionnement": "OUI",
                "Capacité de la station": "30",
                "Nombre bornettes libres": "28",
                "Nombre total vélos disponibles": "2",
                "Vélos mécaniques disponibles": "1",
                "Vélos électriques disponibles": "1",
                "Borne de paiement disponible": "OUI",
                "Retour vélib possible": "OUI",
                "Actualisation de la donnée": "2021-11-03T13:50:36+01:00",
                "Coordonnées géographiques": "48.902638673,2.34824806343",
                "Nom communes équipées": "Saint-Ouen-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1433: {
            "attributes": {
                "Identifiant station": "26009",
                "Nom station": "Marcel Delorme - Pierre Brossolette",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-07-05T12:21:40+02:00",
                "Coordonnées géographiques": "48.9120606286,2.29589742357",
                "Nom communes équipées": "Asnières-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1434: {
            "attributes": {
                "Identifiant station": "34012",
                "Nom station": "Bateliers - Quai de Seine",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "26",
                "Nombre bornettes libres": "26",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2018-10-11T05:06:49+02:00",
                "Coordonnées géographiques": "48.9158175855,2.32395332986",
                "Nom communes équipées": "Saint-Ouen-sur-Seine",
                "Code INSEE communes équipées": "",
            }
        },
        1435: {
            "attributes": {
                "Identifiant station": "31301",
                "Nom station": "Gare RER",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-10-22T10:52:10+02:00",
                "Coordonnées géographiques": "48.8962414123,2.4595758301",
                "Nom communes équipées": "Noisy-le-Sec",
                "Code INSEE communes équipées": "",
            }
        },
        1436: {
            "attributes": {
                "Identifiant station": "31303",
                "Nom station": "Simone Signoret - Paris",
                "Station en fonctionnement": "NON",
                "Capacité de la station": "0",
                "Nombre bornettes libres": "0",
                "Nombre total vélos disponibles": "0",
                "Vélos mécaniques disponibles": "0",
                "Vélos électriques disponibles": "0",
                "Borne de paiement disponible": "NON",
                "Retour vélib possible": "NON",
                "Actualisation de la donnée": "2021-10-22T10:59:55+02:00",
                "Coordonnées géographiques": "48.9036006247,2.46540659627",
                "Nom communes équipées": "Noisy-le-Sec",
                "Code INSEE communes équipées": "",
            }
        },
    },
}
